Package: adegenet
Title: Exploratory Analysis of Genetic and Genomic Data
Version: 2.1.11
Authors@R: 
    c(person(given = "Thibaut",
             family = "Jombart",
             role = "aut",
             comment = c(ORCID = "0000-0003-2226-8692")),
      person(given = "Zhian N.",
             family = "Kamvar",
             role = c("aut", "cre"),
             email = "zkamvar@gmail.com",
             comment = c(ORCID = "0000-0003-1458-7108")),
      person(given = "Caitlin",
             family = "Collins",
             role = "ctb"),
      person(given = "Roman",
             family = "Lustrik",
             role = "ctb"),
      person(given = "Marie-Pauline",
             family = "Beugin",
             role = "ctb"),
      person(given = "Brian J.",
             family = "Knaus",
             role = "ctb"),
      person(given = "Peter",
             family = "Solymos",
             role = "ctb"),
      person(given = "Vladimir",
             family = "Mikryukov",
             role = "ctb"),
      person(given = "Klaus",
             family = "Schliep",
             role = "ctb"),
      person(given = "Tiago",
             family = "Maié",
             role = "ctb"),
      person(given = "Libor",
             family = "Morkovsky",
             role = "ctb"),
      person(given = "Ismail",
             family = "Ahmed",
             role = "ctb"),
      person(given = "Anne",
             family = "Cori",
             role = "ctb"),
      person(given = "Federico",
             family = "Calboli",
             role = "ctb"),
      person(given = "RJ",
             family = "Ewing",
             role = "ctb"),
      person(given = "Frédéric",
             family = "Michaud",
             role = "ctb"),
      person(given = "Rebecca",
             family = "DeCamp",
             role = "ctb"),
      person(given = "Alexandre",
             family = "Courtiol",
             role = "ctb",
             comment = c(ORCID = "0000-0003-0637-2959")),
      person(given = "Lindsay V.",
             family = "Clark",
             role = "ctb",
             comment = c(ORCID = "0000-0002-3881-9252")),
       person(given = "Pavel",
              family = "Dimens",
              role = "ctb",
              comment = c(ORCID = "0000-0003-3823-0373")),
       person(given = "Max",
              family = "Coulter",
              role = "ctb"),
       person(given = "Ivan",
              family = "Krylov",
              role = "ctb")
       )
Description: Toolset for the exploration of genetic and genomic
    data. Adegenet provides formal (S4) classes for storing and handling
    various genetic data, including genetic markers with varying ploidy
    and hierarchical population structure ('genind' class), alleles counts
    by populations ('genpop'), and genome-wide SNP data ('genlight'). It
    also implements original multivariate methods (DAPC, sPCA), graphics,
    statistical tests, simulation tools, distance and similarity measures,
    and several spatial methods. A range of both empirical and simulated
    datasets is also provided to illustrate various methods.
License: GPL (>= 2)
URL: https://github.com/thibautjombart/adegenet
Depends: R (>= 2.14), methods, ade4
Imports: utils, stats, grDevices, MASS, igraph, ape, shiny, ggplot2,
        seqinr, parallel, boot, reshape2, dplyr (>= 0.4.1), vegan
Suggests: adespatial, pegas, hierfstat, maps, spdep, interp, splancs,
        poppr, testthat
Encoding: UTF-8
LazyLoad: yes
RoxygenNote: 7.3.2
Collate: 'adegenet.package.R' 'datasets.R' 'dist.genlight.R'
        'orthobasis.R' 'classes.R' 'constructors.R' 'accessors.R'
        'basicMethods.R' 'handling.R' 'auxil.R' 'minorAllele.R'
        'setAs.R' 'SNPbin.R' 'strataMethods.R' 'hierarchyMethods.R'
        'glHandle.R' 'glFunctions.R' 'glSim.R' 'find.clust.R'
        'hybridize.R' 'scale.R' 'fstat.R' 'import.R' 'seqTrack.R'
        'chooseCN.R' 'genind2genpop.R' 'loadingplot.R' 'sequences.R'
        'gstat.randtest.R' 'makefreq.R' 'colorplot.R' 'monmonier.R'
        'spca.R' 'coords.monmonier.R' 'haploGen.R' 'old2new.R'
        'global_local_tests.R' 'dapc.R' 'compoplot.R' 'xvalDapc.R'
        'haploPop.R' 'PCtest.R' 'dist.genpop.R' 'Hs.R' 'propShared.R'
        'export.R' 'HWE.R' 'propTyped.R' 'inbreeding.R' 'glPlot.R'
        'gengraph.R' 'simOutbreak.R' 'mutations.R' 'snpposi.R'
        'snpzip.R' 'pairDist.R' 'snapclust.R' 'AIC.snapclust.R'
        'AICc.snapclust.R' 'BIC.snapclust.R' 'KIC.snapclust.R'
        'snapclust.choose.k.R' 'servers.R' 'showmekittens.R'
        'spca_randtest.R' 'export_to_mvmapper.R' 'doc_C_routines.R'
        'zzz.R'
NeedsCompilation: yes
Packaged: 2025-02-06 21:23:20 UTC; zkamvar
Author: Thibaut Jombart [aut] (<https://orcid.org/0000-0003-2226-8692>),
  Zhian N. Kamvar [aut, cre] (<https://orcid.org/0000-0003-1458-7108>),
  Caitlin Collins [ctb],
  Roman Lustrik [ctb],
  Marie-Pauline Beugin [ctb],
  Brian J. Knaus [ctb],
  Peter Solymos [ctb],
  Vladimir Mikryukov [ctb],
  Klaus Schliep [ctb],
  Tiago Maié [ctb],
  Libor Morkovsky [ctb],
  Ismail Ahmed [ctb],
  Anne Cori [ctb],
  Federico Calboli [ctb],
  RJ Ewing [ctb],
  Frédéric Michaud [ctb],
  Rebecca DeCamp [ctb],
  Alexandre Courtiol [ctb] (<https://orcid.org/0000-0003-0637-2959>),
  Lindsay V. Clark [ctb] (<https://orcid.org/0000-0002-3881-9252>),
  Pavel Dimens [ctb] (<https://orcid.org/0000-0003-3823-0373>),
  Max Coulter [ctb],
  Ivan Krylov [ctb]
Maintainer: Zhian N. Kamvar <zkamvar@gmail.com>
Repository: CRAN
Date/Publication: 2025-02-06 22:40:02 UTC
Built: R 4.3.3; x86_64-apple-darwin20; 2025-02-15 13:47:50 UTC; unix
Archs: adegenet.so.dSYM

Package: adegenet
Title: Exploratory Analysis of Genetic and Genomic Data
Version: 2.1.11
Authors@R: 
    c(person(given = "Thibaut",
             family = "Jombart",
             role = "aut",
             comment = c(ORCID = "0000-0003-2226-8692")),
      person(given = "Zhian N.",
             family = "Kamvar",
             role = c("aut", "cre"),
             email = "zkamvar@gmail.com",
             comment = c(ORCID = "0000-0003-1458-7108")),
      person(given = "Caitlin",
             family = "Collins",
             role = "ctb"),
      person(given = "Roman",
             family = "Lustrik",
             role = "ctb"),
      person(given = "Marie-Pauline",
             family = "Beugin",
             role = "ctb"),
      person(given = "Brian J.",
             family = "Knaus",
             role = "ctb"),
      person(given = "Peter",
             family = "Solymos",
             role = "ctb"),
      person(given = "Vladimir",
             family = "Mikryukov",
             role = "ctb"),
      person(given = "Klaus",
             family = "Schliep",
             role = "ctb"),
      person(given = "Tiago",
             family = "Maié",
             role = "ctb"),
      person(given = "Libor",
             family = "Morkovsky",
             role = "ctb"),
      person(given = "Ismail",
             family = "Ahmed",
             role = "ctb"),
      person(given = "Anne",
             family = "Cori",
             role = "ctb"),
      person(given = "Federico",
             family = "Calboli",
             role = "ctb"),
      person(given = "RJ",
             family = "Ewing",
             role = "ctb"),
      person(given = "Frédéric",
             family = "Michaud",
             role = "ctb"),
      person(given = "Rebecca",
             family = "DeCamp",
             role = "ctb"),
      person(given = "Alexandre",
             family = "Courtiol",
             role = "ctb",
             comment = c(ORCID = "0000-0003-0637-2959")),
      person(given = "Lindsay V.",
             family = "Clark",
             role = "ctb",
             comment = c(ORCID = "0000-0002-3881-9252")),
       person(given = "Pavel",
              family = "Dimens",
              role = "ctb",
              comment = c(ORCID = "0000-0003-3823-0373")),
       person(given = "Max",
              family = "Coulter",
              role = "ctb"),
       person(given = "Ivan",
              family = "Krylov",
              role = "ctb")
       )
Description: Toolset for the exploration of genetic and genomic
    data. Adegenet provides formal (S4) classes for storing and handling
    various genetic data, including genetic markers with varying ploidy
    and hierarchical population structure ('genind' class), alleles counts
    by populations ('genpop'), and genome-wide SNP data ('genlight'). It
    also implements original multivariate methods (DAPC, sPCA), graphics,
    statistical tests, simulation tools, distance and similarity measures,
    and several spatial methods. A range of both empirical and simulated
    datasets is also provided to illustrate various methods.
License: GPL (>= 2)
URL: https://github.com/thibautjombart/adegenet
Depends: R (>= 2.14), methods, ade4
Imports: utils, stats, grDevices, MASS, igraph, ape, shiny, ggplot2,
        seqinr, parallel, boot, reshape2, dplyr (>= 0.4.1), vegan
Suggests: adespatial, pegas, hierfstat, maps, spdep, interp, splancs,
        poppr, testthat
Encoding: UTF-8
LazyLoad: yes
RoxygenNote: 7.3.2
Collate: 'adegenet.package.R' 'datasets.R' 'dist.genlight.R'
        'orthobasis.R' 'classes.R' 'constructors.R' 'accessors.R'
        'basicMethods.R' 'handling.R' 'auxil.R' 'minorAllele.R'
        'setAs.R' 'SNPbin.R' 'strataMethods.R' 'hierarchyMethods.R'
        'glHandle.R' 'glFunctions.R' 'glSim.R' 'find.clust.R'
        'hybridize.R' 'scale.R' 'fstat.R' 'import.R' 'seqTrack.R'
        'chooseCN.R' 'genind2genpop.R' 'loadingplot.R' 'sequences.R'
        'gstat.randtest.R' 'makefreq.R' 'colorplot.R' 'monmonier.R'
        'spca.R' 'coords.monmonier.R' 'haploGen.R' 'old2new.R'
        'global_local_tests.R' 'dapc.R' 'compoplot.R' 'xvalDapc.R'
        'haploPop.R' 'PCtest.R' 'dist.genpop.R' 'Hs.R' 'propShared.R'
        'export.R' 'HWE.R' 'propTyped.R' 'inbreeding.R' 'glPlot.R'
        'gengraph.R' 'simOutbreak.R' 'mutations.R' 'snpposi.R'
        'snpzip.R' 'pairDist.R' 'snapclust.R' 'AIC.snapclust.R'
        'AICc.snapclust.R' 'BIC.snapclust.R' 'KIC.snapclust.R'
        'snapclust.choose.k.R' 'servers.R' 'showmekittens.R'
        'spca_randtest.R' 'export_to_mvmapper.R' 'doc_C_routines.R'
        'zzz.R'
NeedsCompilation: yes
Packaged: 2025-02-06 21:23:20 UTC; zkamvar
Author: Thibaut Jombart [aut] (<https://orcid.org/0000-0003-2226-8692>),
  Zhian N. Kamvar [aut, cre] (<https://orcid.org/0000-0003-1458-7108>),
  Caitlin Collins [ctb],
  Roman Lustrik [ctb],
  Marie-Pauline Beugin [ctb],
  Brian J. Knaus [ctb],
  Peter Solymos [ctb],
  Vladimir Mikryukov [ctb],
  Klaus Schliep [ctb],
  Tiago Maié [ctb],
  Libor Morkovsky [ctb],
  Ismail Ahmed [ctb],
  Anne Cori [ctb],
  Federico Calboli [ctb],
  RJ Ewing [ctb],
  Frédéric Michaud [ctb],
  Rebecca DeCamp [ctb],
  Alexandre Courtiol [ctb] (<https://orcid.org/0000-0003-0637-2959>),
  Lindsay V. Clark [ctb] (<https://orcid.org/0000-0002-3881-9252>),
  Pavel Dimens [ctb] (<https://orcid.org/0000-0003-3823-0373>),
  Max Coulter [ctb],
  Ivan Krylov [ctb]
Maintainer: Zhian N. Kamvar <zkamvar@gmail.com>
Repository: CRAN
Date/Publication: 2025-02-06 22:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 11:16:02 UTC; unix
Archs: adegenet.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method("[",haploGen)
S3method(AIC,snapclust)
S3method(AICc,snapclust)
S3method(BIC,snapclust)
S3method(KIC,snapclust)
S3method(as.POSIXct,haploGen)
S3method(as.data.frame,genind)
S3method(as.data.frame,genlight)
S3method(as.data.frame,genpop)
S3method(as.genpop,genind)
S3method(as.igraph,haploGen)
S3method(as.integer,SNPbin)
S3method(as.lda,dapc)
S3method(as.list,genlight)
S3method(as.matrix,genind)
S3method(as.matrix,genlight)
S3method(as.matrix,genpop)
S3method(c,SNPbin)
S3method(cbind,SNPbin)
S3method(cbind,genlight)
S3method(colorplot,default)
S3method(colorplot,spca)
S3method(compoplot,dapc)
S3method(compoplot,matrix)
S3method(compoplot,snapclust)
S3method(dapc,data.frame)
S3method(dapc,dudi)
S3method(dapc,genind)
S3method(dapc,genlight)
S3method(dapc,matrix)
S3method(export_to_mvmapper,dapc)
S3method(export_to_mvmapper,default)
S3method(export_to_mvmapper,dudi)
S3method(export_to_mvmapper,spca)
S3method(find.clusters,data.frame)
S3method(find.clusters,genind)
S3method(find.clusters,genlight)
S3method(find.clusters,matrix)
S3method(findMutations,DNAbin)
S3method(gengraph,DNAbin)
S3method(gengraph,default)
S3method(gengraph,dist)
S3method(gengraph,genind)
S3method(gengraph,genpop)
S3method(gengraph,matrix)
S3method(get.likelihood,seqTrack)
S3method(graphMutations,DNAbin)
S3method(labels,haploGen)
S3method(loadingplot,default)
S3method(loadingplot,glPca)
S3method(pairDistPlot,DNAbin)
S3method(pairDistPlot,default)
S3method(pairDistPlot,dist)
S3method(pairDistPlot,genind)
S3method(pairDistPlot,matrix)
S3method(plot,haploGen)
S3method(plot,monmonier)
S3method(plot,seqTrack)
S3method(plot,spca)
S3method(predict,dapc)
S3method(print,dapc)
S3method(print,genindSummary)
S3method(print,genpopSummary)
S3method(print,glPca)
S3method(print,haploGen)
S3method(print,monmonier)
S3method(print,spca)
S3method(rbind,genlight)
S3method(scatter,dapc)
S3method(scatter,glPca)
S3method(screeplot,spca)
S3method(seqTrack,default)
S3method(seqTrack,haploGen)
S3method(seqTrack,matrix)
S3method(snpposi.plot,DNAbin)
S3method(snpposi.plot,integer)
S3method(snpposi.plot,numeric)
S3method(snpposi.test,DNAbin)
S3method(snpposi.test,integer)
S3method(snpposi.test,numeric)
S3method(spca,data.frame)
S3method(spca,default)
S3method(spca,genind)
S3method(spca,genpop)
S3method(spca,matrix)
S3method(summary,dapc)
S3method(summary,spca)
S3method(xvalDapc,data.frame)
S3method(xvalDapc,default)
S3method(xvalDapc,genind)
S3method(xvalDapc,genlight)
S3method(xvalDapc,matrix)
export("addStrata<-")
export("hier<-")
export("nameStrata<-")
export("setPop<-")
export("splitStrata<-")
export("strata<-")
export(.genlab)
export(.internal_C_routines)
export(.readExt)
export(.render.server.info)
export(.rmspaces)
export(AICc)
export(Hs)
export(Hs.test)
export(KIC)
export(addStrata)
export(adegenetIssues)
export(adegenetTutorial)
export(adegenetWeb)
export(as.genind)
export(as.genpop)
export(chooseCN)
export(compoplot)
export(df2genind)
export(export_to_mvmapper)
export(extract.PLINKmap)
export(gengraph)
export(genind)
export(genind2df)
export(genind2genpop)
export(genpop)
export(hier)
export(hybridize)
export(is.genind)
export(is.genpop)
export(loadingplot)
export(makefreq)
export(minorAllele)
export(nameStrata)
export(old2new_genind)
export(old2new_genlight)
export(old2new_genpop)
export(read.PLINK)
export(read.fstat)
export(read.genepop)
export(read.genetix)
export(read.snp)
export(read.structure)
export(repool)
export(sample.haploGen)
export(scaleGen)
export(seqTrack)
export(setPop)
export(showmekittens)
export(snapclust)
export(snapclust.choose.k)
export(snpposi.plot)
export(snpposi.test)
export(spca_randtest)
export(splitStrata)
export(strata)
export(tab)
export(xvalDapc)
exportMethods(initialize)
exportMethods(makefreq)
exportMethods(scaleGen)
exportPattern("^[^\\.]")
import(ade4)
import(grDevices)
import(graphics)
import(methods)
import(parallel)
import(stats)
import(utils)
importFrom(MASS,"lda")
importFrom(ape,"[.DNAbin")
importFrom(ape,"as.DNAbin")
importFrom(ape,"as.DNAbin.alignment")
importFrom(ape,"as.DNAbin.character")
importFrom(ape,"as.DNAbin.list")
importFrom(ape,"as.character.DNAbin")
importFrom(ape,"as.list.DNAbin")
importFrom(ape,"as.matrix.DNAbin")
importFrom(ape,"c.DNAbin")
importFrom(ape,"cbind.DNAbin")
importFrom(ape,"dist.dna")
importFrom(ape,"labels.DNAbin")
importFrom(ape,"print.DNAbin")
importFrom(ape,"rbind.DNAbin")
importFrom(ape,"seg.sites")
importFrom(dplyr,bind_rows)
importFrom(ggplot2,"aes")
importFrom(ggplot2,"coord_flip")
importFrom(ggplot2,"geom_boxplot")
importFrom(ggplot2,"geom_density")
importFrom(ggplot2,"geom_jitter")
importFrom(ggplot2,"geom_rug")
importFrom(ggplot2,"geom_violin")
importFrom(ggplot2,"ggplot")
importFrom(ggplot2,"guide_legend")
importFrom(ggplot2,"guides")
importFrom(ggplot2,"labs")
importFrom(ggplot2,"xlim")
importFrom(igraph,"E")
importFrom(igraph,"E<-")
importFrom(igraph,"V")
importFrom(igraph,"V<-")
importFrom(igraph,"as.igraph")
importFrom(igraph,"clusters")
importFrom(igraph,"graph.adjacency")
importFrom(igraph,"graph.data.frame")
importFrom(igraph,"layout.fruchterman.reingold")
importFrom(igraph,"plot.igraph")
importFrom(igraph,"print.igraph")
importFrom(reshape2,colsplit)
importFrom(seqinr,s2c)
importFrom(shiny,"renderPrint")
importFrom(shiny,"runApp")
importFrom(vegan,orditorp)
useDynLib(adegenet, .registration = TRUE)

<!-- badges: start -->
[![R-CMD-check](https://github.com/thibautjombart/adegenet/workflows/R-CMD-check/badge.svg)](https://github.com/thibautjombart/adegenet/actions)

[![CRAN Status Badge](https://www.r-pkg.org/badges/version/adegenet)](https://cran.r-project.org/package=adegenet)

[![CRAN Downloads](https://cranlogs.r-pkg.org/badges/adegenet)](https://cran.r-project.org/package=adegenet)

[![Downloads from Rstudio mirror](https://cranlogs.r-pkg.org/badges/grand-total/adegenet)](https://cran.r-project.org/package=adegenet)

  <!-- badges: end -->

adegenet
========
*adegenet*: a R Package for the Multivariate Analysis of Genetic Markers

This wiki is dedicated to the development of *adegenet*.
It is relevant for developers of the package, developers of other packages depending on *adegenet*, and for users who want to be using the latest features as well.

The *adegenet website* is available at http://adegenet.r-forge.r-project.org/.

The following sections are available:
- [Installing adegenet devel](https://github.com/thibautjombart/adegenet/wiki/Installing-adegenet-devel)
- [Tutorials](https://github.com/thibautjombart/adegenet/wiki/Tutorials)
- [Ask a question](https://github.com/thibautjombart/adegenet/wiki/Questions)
- [Features request / bug report](https://github.com/thibautjombart/adegenet/issues)
- [Branches](https://github.com/thibautjombart/adegenet/wiki/Branches)
- [Contributors](https://github.com/thibautjombart/adegenet/wiki/Contributors)

library(shiny)
library(adegenet)


## DEFINE THE SERVER SIDE OF THE APPLICATION
shinyServer(function(input, output) {

    ## GET DYNAMIC ANNOTATION
    graphTitle <- reactive({
        paste(input$dataset, ": DAPC scatterplot, axes ", input$xax,"-", input$yax, sep="")
    })

    ## DEFINE CAPTION
    output$caption <- renderText({
        graphTitle()
    })


    ## GET DATA ##
    getData <- reactive({
        out <- NULL

        if(input$datatype=="expl"){
            if(input$dataset=="microbov") data("microbov", package="adegenet", envir=environment())
            if(input$dataset=="sim2pop") data("sim2pop", package="adegenet", envir=environment())
            if(input$dataset=="nancycats") data("nancycats", package="adegenet", envir=environment())
            out <- get(input$dataset)
        }

        if(input$datatype=="file" && !is.null(input$datafile)){
            ## need to rename input file
            oldName <- input$datafile$datapath
            extension <- .readExt(input$datafile$name)
            newName <- paste(input$datafile$datapath, extension, sep=".")
            file.rename(oldName, newName)

            ## treat different types of input
            if(extension %in% c("gtx","gen","dat","GTX","GEN","DAT")){
                out <- import2genind(newName)
            }

            if(extension %in% c("RData","Rdata","Rda","rda")){
                out <- get(load(newName))
            }

            if(extension %in% c("fasta","fa","fas","aln","FASTA","FA","FAS","ALN")){
                out <- DNAbin2genind(fasta2DNAbin(newName))
            }
        }
        return(out)
    })




    ## DYNAMIC UI COMPONENTS ##
    ## SELECTION OF PCA AXES
    output$npca <- renderUI({
        if(!is.null(x <- getData())) {
          nmax <- min(dim(x@tab))
          def <- min(10, nmax)

          if(input$useoptimnpca){
            xval1 <- xvaldapc()
            npca <- as.integer(xval1[[6]])
            def <- npca}

        } else {
            nmax <- 1000
            def <- 1
        }
        sliderInput("npca", "Number of PCA axes retained:", min=1, max=nmax, value=def,step=1)
    })

    ## SELECTION OF DA AXES
    output$nda <- renderUI({
        if(!is.null(x <- getData())) {
            nmax <- max(length(levels(pop(x)))-1,2)
            def <- length(levels(pop(x)))-1
        } else {
            nmax <- 100
            def <- 1
        }
        sliderInput("nda", "Number of DA axes retained:", min=1, max=nmax, value=def,step=1)
    })

    ## SELECTION OF PLOTTED AXES
    output$xax <- renderUI({
        if(!is.null(x <- getData())) {
            nmax <- min(dim(x@tab))
        } else {
            nmax <- 1000
        }
        numericInput("xax", "Indicate the x axis", value=1, min=1, max=nmax)
    })

   output$yax <- renderUI({
       def <- 1
       nda <- 1
       if(!is.null(input$nda)) nda <- input$nda
       if(!is.null(x <- getData())) {
            nmax <- min(dim(x@tab))
            if(nda>1 && length(levels(pop(x)))>1) def <- 2
        } else {
            nmax <- 1000
        }

        numericInput("yax", "Indicate the y axis", value=def, min=1, max=nmax)
    })

    ## CROSS-VALIDATION
    ## DYNAMIC TICKBOX (TICKED IF OPTIMNPCA CHOSEN)
    output$doxval <- renderUI({
        checkboxInput("doxval", "Perform cross validation (computer intensive)?", input$doxval)
    })

   ## DYNAMIC SLIDER FOR MAX NPCA SELECTION
   output$npcaMax <- renderUI({
     if(!is.null(x <- getData())) {
       nmax <- min(dim(x@tab))
       def <- nmax
     } else {
       nmax <- 1000
       def <- 1
     }
     sliderInput("npcaMax", "Maximum number of PCs:", min=1, max=nmax, value=def,step=1)
   })


    ## CROSS-VALIDATION FUNCTION
    xvaldapc <- reactive({
        doxval <- FALSE
        if(!is.null(input$doxval)) doxval <- input$doxval
        if(input$useoptimnpca || doxval){
            x <- getData()
            mat <- tab(x, NA.method="mean")
            grp <- pop(x)
            result <- input$result
            n.rep <- input$nrep
            nda <- 1
            if(!is.null(input$nda)) nda <- input$nda
            training.set <- input$trainingset
            npcaMax <- 1
            if(!is.null(input$npcaMax)) npcaMax <- input$npcaMax
            out <- xvalDapc(mat, grp, n.pca.max=npcaMax,
                            result=result, n.rep=n.rep, n.da=nda, training.set=training.set,
                            xval.plot=FALSE)
        }
        else{
            out <- NULL
        }
        return(out)
    })


    ## XVALPLOT
    output$xvalPlot <- renderPlot({
        xval1 <- xvaldapc()
        if(!is.null(xval1)){
            x <- getData()
            mat <- tab(x, NA.method="mean")
            grp <- pop(x)
            xval2 <- xval1[[1]]
            successV <-as.vector(xval2$success)
            random <- replicate(300, mean(tapply(sample(grp)==grp, grp, mean)))
            q.GRP <- quantile(random, c(0.025,0.5,0.975))
            smoothScatter(xval2$n.pca, successV, nrpoints=Inf, pch=20, col=transp("black"),
                          ylim=c(0,1), xlab="Number of PCA axes retained",
                          ylab="Proportion of successful outcome prediction",
                          main="DAPC Cross-Validation")
            print(abline(h=q.GRP, lty=c(2,1,2)))
        }
    })

    ## XVAL OUTPUT
    output$xvalResults1 <-renderPrint({
        xval1 <- xvaldapc()
        if(!is.null(xval1)){
            print(xval1[[1]])
        }

    })
    output$xvalResults2 <-renderPrint({
        xval1 <- xvaldapc()
        if(!is.null(xval1)){
            print(xval1[[2]])
        }

    })
   output$xvalResults3 <-renderPrint({
        xval1 <- xvaldapc()
        if(!is.null(xval1)){
            print(xval1[[3]])
        }
    })
    output$xvalResults4 <-renderPrint({
        xval1 <- xvaldapc()
        if(!is.null(xval1)){
            print(xval1[[4]])
        }

    })
    output$xvalResults5 <-renderPrint({
        xval1 <- xvaldapc()
        if(!is.null(xval1)){
            print(xval1[[5]])
        }

    })
    output$xvalResults6 <-renderPrint({
        xval1 <- xvaldapc()
        if(!is.null(xval1)){
            print(xval1[[6]])
        }

    })


    ## PERFORM THE DAPC ##
    getDapc <- reactive({
        out <- NULL
        x <- getData()
        npca <- nda <- 1

        ## n.pca determined by xval or slider?
        if(input$useoptimnpca){
            xval1 <- xvaldapc()
            npca <- as.integer(xval1[[6]])
        } else {
            if(!is.null(input$npca)) npca <- input$npca
        }
        if(!is.null(input$nda)) nda <- input$nda

        if(!is.null(x)) out <- dapc(x, n.pca=npca, n.da=nda, parallel=FALSE)
        return(out)
    })


    ## GET PLOT PARAM ##
    getPlotParam <- reactive({
        col.pal <- get(input$col.pal)
        return(list(col.pal=col.pal))
    })


    ## MAKE OUTPUT PLOT ##
    output$scatterplot <- renderPlot({
        dapc1 <- getDapc()
        if(!is.null(dapc1)){

            ## get colors
            K <- length(levels(dapc1$grp))
            myCol <- get(input$col.pal)(K)

            ## get screeplot info
            scree.pca <- ifelse(input$screepca=="none", FALSE, TRUE)
            scree.da <- ifelse(input$screeda=="none", FALSE, TRUE)
            cellipse <- ifelse(input$ellipses, 1.5, 0)
            cstar <- ifelse(input$stars, 1, 0)
            scatter(dapc1, xax=input$xax, yax=input$yax, col=myCol,
                    scree.pca=scree.pca, scree.da=scree.da,
                    posi.pca=input$screepca, posi.da=input$screeda,
                    cellipse=cellipse, cstar=cstar, mstree=input$mstree,
                    cex=input$pointsize, clabel=input$labelsize, solid=1-input$alpha)
        } else {
            NULL
        }
    })


    ## MAKE SUMMARY PLOT ##
    output$summary <- renderPrint({
        dapc1 <- getDapc()
        if(!is.null(dapc1)){
            summary(dapc1)
        }
    })

    ## MAKE COMPOPLOT ##
    output$compoplot <- renderPlot({
        dapc1 <- getDapc()
        if(!is.null(dapc1)){
            ## get colors
            K <- length(levels(dapc1$grp))
            myCol <- get(input$col.pal)(K)
            ##myCol <- transp(myCol, 1-input$alpha)
            compoplot(dapc1, col=myCol, lab=input$compo.lab, legend=input$compo.legend)

        }
    })


   ## DYNAMIC SELECTION OF DISCRIMINANT AXIS FOR LOADING PLOT
   output$LPax <- renderUI({
     def <- 1
     nda <- 1
     nmax <- 2
          if(!is.null(x <- getData())) {
            if(!is.null(input$nda)) nda <- input$nda
            nmax <- nda
            if(!is.null(input$LPax)) def <- input$LPax
          }

     numericInput("LPax", "Select discriminant axis", value=def, min=1, max=nmax)
   })



   # REACTIVE THRESHOLD/SNP SELECTION FUNCTION if using snpzip-like method
   selector <- reactive({

     dimension <- 1

     dapc1 <- getDapc()
     if(!is.null(dapc1)){
       if(!is.null(input$thresholdMethod)) method <- input$thresholdMethod
       if(!is.null(input$LPaxis)) dimension <- input$LPaxis
       x <- getData()
       mat <- tab(x, NA.method="mean")
     }

     if(method=="quartile"){
       x <- dapc1$var.contr[,dimension]
       thresh <- quantile(x,0.75)
       maximus <- which(x > thresh)
       n.snp.selected <- length(maximus)
       sel.snps <- mat[,maximus]
     }
     else{
     z <- dapc1$var.contr[,dimension]
     xTotal <- dapc1$var.contr[,dimension]
     toto <- which(xTotal%in%tail(sort(xTotal), 2000))
     z <- sapply(toto, function(e) xTotal[e])

     D <- dist(z)
     clust <- hclust(D,method)
     pop <- factor(cutree(clust,k=2,h=NULL))
     m <- which.max(tapply(z,pop,mean))
     maximus <- which(pop==m)
     maximus <- as.vector(unlist(sapply(maximus, function(e) toto[e])))
     popvect <- as.vector(unclass(pop))
     n.snp.selected <- sum(popvect==m)
     sel.snps <- mat[,maximus]
     }

     selection <- c((ncol(mat)-ncol(mat[,-maximus])), ncol(mat[,-maximus]))
     resultat <- list(selection, maximus, dimnames(sel.snps)[[2]], dapc1$var.contr[maximus, dimension])
     return(resultat)
   })



   ## MAKE LOADINGPLOT ##
   output$loadingplot <- renderPlot({
     dapc1 <- getDapc()
     LPaxis <- 1
     if(!is.null(dapc1)){
       ## get loadings for LP axis
       LPaxis <- 1
       if(!is.null(input$LPax)) LPaxis <- input$LPax
       if(input$threshold){
         # if threshold is by quantile
         if(input$thresholdMethod=="quartile"){
           x <- dapc1$var.contr[,LPaxis]
           def <- quantile(x,0.75)
         }else{
           # if threshold is by clustering
           select <- selector()
           thresh <- select[[2]]
           def <- abs(dapc1$var.contr[thresh][(which.min(dapc1$var.contr[thresh]))])-0.000001}
       }
       else{
         def <- NULL}
       loadingplot(dapc1$var.contr[,LPaxis], threshold=def)
     }
   })


   ## FEATURE SELECTION OUTPUT
   output$FS1 <-renderPrint({
     if(input$FS){
     fs1 <- selector()
     if(!is.null(fs1)){
       print(fs1[[1]])
     }
     }
   })

   output$FS2 <-renderPrint({
     if(input$FS){
     fs1 <- selector()
     if(!is.null(fs1)){
       print(fs1[[2]])
     }
     }
   })

   output$FS3 <-renderPrint({
     if(input$FS){
     fs1 <- selector()
     if(!is.null(fs1)){
       print(fs1[[3]])
     }
     }
   })

   output$FS4 <-renderPrint({
     if(input$FS){
     fs1 <- selector()
     if(!is.null(fs1)){
       print(fs1[[4]])
     }
     }
   })

    ## RENDER SYSTEM INFO ##
    output$systeminfo <- renderPrint({
      cat("\n== R version ==\n")
      print(R.version)

      cat("\n== Date ==\n")
      print(date())

      cat("\n== adegenet version ==\n")
      print(packageDescription("adegenet", fields=c("Package", "Version", "Date", "Built")))

      cat("\n== shiny version ==\n")
      print(packageDescription("adegenet", fields=c("Package", "Version", "Date", "Built")))

      cat("\n== attached packages ==\n")
      print(search())
    }) # end renderPrint

#      .render.server.info()

}) # end shinyServer

library(shiny)
##library(adegenet)

## DEFINE UI ##
shinyUI(
        pageWithSidebar(
                        ##  TITLE
                        headerPanel("DAPC web server"),

                        ## SIDE PANEL CONTENT
                        sidebarPanel(
                                     ## define the type of input
                                     conditionalPanel(
                                                      ## condition
                                                      condition = "$('li.active a').first().html()!='Help'",
                                                      radioButtons("datatype", "What data source to use?",
                                                                   list("Example from adegenet"="expl","Input file"="file"))),

                                     ## choice of dataset if source is an example
                                     conditionalPanel(condition = "input.datatype=='expl'&& $('li.active a').first().html()!= 'Help'",
                                                      selectInput("dataset", "Select an example dataset:",
                                                                  choices=c("microbov","sim2pop","nancycats"))
                                                      ),

                                     ## choice of dataset if source is a file
                                     conditionalPanel(condition = "input.datatype=='file'&& $('li.active a').first().html()!= 'Help'",
                                                      fileInput('datafile', 'Choose input file',
                                                                accept=c('gtx/gen/dat/GTX/GEN/DAT/RData/Rdata/Rda/rda', 'GENETIX/genepop/Fstat/R data')),
                                                      tags$hr()
                                                      ),



                                     ## CROSS-VALIDATION
                                        # n.pca.max slider
                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()=='Cross-Validation'",
                                                      h3("Cross-validation"),
                                                      uiOutput("doxval"),
                                                      uiOutput("npcaMax")
                                                      ),


                                     ## Select Output variable:
                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()!='Cross-Validation'&& $('li.active a').first().html()!= 'Help'",
                                                      checkboxInput("useoptimnpca", "Use suggested number of PCA components?", FALSE)
                                                      ),


                                     ##sliderInput("npca", "Number of PCA axes retained:", min=1, max=1000, value=10),
                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()!='Cross-Validation'&& $('li.active a').first().html()!= 'Help'",
                                                      uiOutput("npca")
                                                      ),

                                     ## select number of DA axes
                                     ##sliderInput("nda", "Number of discriminant functions retained:", min=1, max=100, value=1),
                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()!= 'Help'",
                                                      uiOutput("nda")),

                                     ## nrep slider
                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()=='Cross-Validation'",
                                                      sliderInput("nrep",
                                                                  "Number of replicates:",
                                                                  min = 1,
                                                                  max = 100,
                                                                  value = 3)
                                                      ),

                                     ## trainingset slider
                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()=='Cross-Validation'",
                                                      sliderInput("trainingset",
                                                                  "Training set size:",
                                                                  min = 0.1,
                                                                  max = 0.95,
                                                                  value = 0.9,
                                                                  step = 0.01)
                                                      ),


                                     ## result type
                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()=='Cross-Validation'",
                                                      radioButtons("result", "Assess by:",
                                                                   list("Group" = "groupMean",
                                                                        "Overall" = "overall"))
                                                      ),



                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()=='Scatterplot'",
                                                      h3("Graphical parameters")
                                                      ),



                                     ## inputs specific of scatterplot tab
                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()=='Scatterplot'",

                                                      ## select first axis to plot
                                                      ##numericInput("xax", "Indicate the x axis", value=1, min=1),
                                                      uiOutput("xax"),

                                                      ## select second axis to plot
                                                      ##numericInput("yax", "Indicate the y axis", value=1, min=1),
                                                      uiOutput("yax")
                                                      ),



                                     conditionalPanel(
                                                      ## condition
                                                      condition = "$('li.active a').first().html()=='Scatterplot'",
                                                      h3("Aesthetics")
                                                      ),

                                     conditionalPanel(
                                                      ## condition
                                                      condition = "$('li.active a').first().html()=='Compoplot'",
                                                      h3("Aesthetics")
                                                      ),



                                     ## select color palette
                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()=='Scatterplot'",
                                                      selectInput("col.pal", "Indicate a color palette to be used",
                                                                  choices=c("funky","spectral","seasun","azur","wasp"))
                                                      ),

                                     ## select color palette
                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()=='Compoplot'",
                                                      selectInput("col.pal", "Indicate a color palette to be used",
                                                                  choices=c("funky","spectral","seasun","azur","wasp"))
                                                      ),



                                     ## select transparency
                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()=='Scatterplot'",
                                                      sliderInput("alpha", "Choose transparency", min=0, max=1, step=0.05, value=0.5)
                                                      ),



                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()==='Scatterplot'",
                                                      ## symbol size
                                                      sliderInput("pointsize", "Size of the points", value=1, min=0, max=10, step=0.2),

                                                      ## label size
                                                      sliderInput("labelsize", "Size of the labels", value=1, min=0, max=10, step=0.2),

                                                      ## add screeplot of PCA?
                                                      selectInput("screepca", "Position of the PCA screeplot:",
                                                                  choices=c("None" = "none",
                                                                  "Bottom right" = "bottomright",
                                                                  "Bottom left" = "bottomleft",
                                                                  "Top right" = "topright",
                                                                  "Top left" = "topleft")),

                                                      ## add screeplot of DA?
                                                      selectInput("screeda", "Position of the DA screeplot:",
                                                                  choices=c("None" = "none",
                                                                  "Bottom right" = "bottomright",
                                                                  "Bottom left" = "bottomleft",
                                                                  "Top right" = "topright",
                                                                  "Top left" = "topleft")),

                                                      ## plot ellipses?
                                                      checkboxInput("ellipses", "Show inertia ellipses?", value=TRUE),

                                                      ## plot stars?
                                                      checkboxInput("stars", "Link points to their centre?", value=TRUE),

                                                      ## plot minimum spanning tree?
                                                      checkboxInput("mstree", "Show minimum spanning tree?", value=FALSE)

                                                      ),

                                     ## input specific of compoplot tab
                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()==='Compoplot'",

                                                      ## add legend?
                                                      checkboxInput("compo.legend", "Add a legend?", TRUE),

                                                      ## add labels?
                                                      checkboxInput("compo.lab", "Display labels?", FALSE)
                                                      ),

                                     ## inputs specific of loadingplot tab
                                     conditionalPanel(
                                                      ## condition
                                                      "$('li.active a').first().html()=='Loading Plot'",

                                                      h3("Graphical parameters"),

                                                      ## select axis to plot
                                                      uiOutput("LPax"),

                                                      checkboxInput("threshold", "Display threshold?", TRUE),

                                                      selectInput("thresholdMethod", "Method for selecting threshold:",
                                                                  choices=c("Third quartile" = "quartile",
                                                                  "Complete linkage clustering" = "complete",
                                                                  "Single linkage clustering" = "single",
                                                                  "Average linkage clustering" = "average",
                                                                  "Centroid clustering" = "centroid",
                                                                  "McQuitty's similarity analysis" = "mcquitty",
                                                                  "Median clustering" = "median",
                                                                  "Ward's minimum variance method" = "ward")),

                                                      checkboxInput("FS", "Select and describe features above threshold", FALSE))


                                     ), # end sidebarPanel

                        ## MAIN PANEL
                        mainPanel(
                                  tabsetPanel(

                                              tabPanel("Scatterplot",plotOutput("scatterplot")),

                                              tabPanel("Summary", verbatimTextOutput("summary")),

                                              tabPanel("Compoplot", plotOutput("compoplot")),

                                              tabPanel("Loading Plot", plotOutput("loadingplot"),
                                                       conditionalPanel(condition = "input.FS==1",
                                                                        h3("Number of selected vs. unselected alleles"),
                                                                        verbatimTextOutput("FS1"),
                                                                        h3("List of selected alleles"),
                                                                        verbatimTextOutput("FS2"),
                                                                        h3("Names of selected alleles"),
                                                                        verbatimTextOutput("FS3"),
                                                                        h3("Contributions of selected alleles to discriminant axis"),
                                                                        verbatimTextOutput("FS4"))),

                                              tabPanel("Cross-Validation", plotOutput("xvalPlot"),
                                                       h3("Mean success by number of PCs"),
                                                       verbatimTextOutput("xvalResults3"),
                                                       h3("Number of PCs with highest mean"),
                                                       verbatimTextOutput("xvalResults4"),
                                                       h3("RMSE by number of PCs"),
                                                       verbatimTextOutput("xvalResults5"),
                                                       h3("Number of PCs with lowest RMSE"),
                                                       verbatimTextOutput("xvalResults6"),
                                                       h3("Cross-validation results"),
                                                       verbatimTextOutput("xvalResults1"),
                                                       h3("Median and CI for random chance"),
                                                       verbatimTextOutput("xvalResults2")),





######################
#### HELP SECTION ####
######################

                                              tabPanel("Help",

                                                       ## OVERVIEW ##

                                                       h3("Overview"),
                                                       p("Welcome to the DAPC Server Help Section. Under each section heading below you can find a brief description and useful information
                  about the content of each tab on the Server. If you want to know more about a specific item or are seeking the definition of a term,
                  you may be more interested in the Glossary on the adjacent tab."),

                                                       p("The DAPC Server aims to provide a user-friendly interactive application of some of the functions contained in the R package adegenet.
                  More information about adegenet can be found on the adegenet website:"),

                                                       a("http://adegenet.r-forge.r-project.org/", href="http://adegenet.r-forge.r-project.org/", target="_blank"),

                                                       p(br(),"On the DAPC Server, users can explore the Discriminant Analysis of Principal Components (DAPC) method.
                  DAPC is a multivariate method that uses genetic data to describe the differences between pre-defined biological populations.
                  DAPC is a dimension reduction approach that generates synthetic variables composed of weighted combinations of the original variables
                  in a dataset to optimally capture between-group variation. DAPC uses Principal Components Analysis (PCA) as a prior step
                  to Discriminant Analysis (DA) to identify up to (K - 1) linearly uncorrelated discriminant axes that can optimally discriminate between
                  K groups of individuals. Unlike DA alone, DAPC is able to perform this procedure when the number of variables (alleles) greatly
                  exceeds the number of individuals, and also in the presence of correlations between variables."),

                                                       p("DAPC is a supervised method, which means that the clusters of individuals to be analysed must be pre-defined by the user.
                  In cases where individuals are not classified into groups, procedures (like that of the find.clusters function in R) can be
                  used to identify clusters so that a DAPC analysis can be carried out. These procedures are not included in the present version of the
                  DAPC Server, however, which requires all data to be imported with inherently defined populations."),

                                                       p("For any dataset containing a set of genetic variables for any number of individuals and a population grouping factor,
                  DAPC can be used to explore between-population differentiation,
                  estimate the probabilities of individual assignment to all possible groups,
                  and to examine the contribution of individual alleles to population structuring."),

                                                       p("Acceptable file types for input datasets include 'gtx/gen/dat/GTX/GEN/DAT/RData/Rdata/Rda/rda' and 'GENETIX/genepop/Fstat/R data'."),


                                                       ## EMAIL ##
                                                       h3(br(),"Ask your questions on the adegenet forum"),
                                                       p("Use the adegenet forum to ask all non-confidential questions: ", a("send an email", href="mailto:adegenet-forum@lists.r-forge.r-project.org", target="_top")),
                                                       p("Make sure to describe your problem clearly and to provide, whenever possible, a reproducible example for any reported error. Please also give your session info (copy and paste the content of the serverInfo tab at the end of your email)"),

                                                       p("Note that this mailing list is moderated, and if not subscribed your post may be differed by a day or two. To subscribe to the mailing list, go to", a("this page", href="http://lists.r-forge.r-project.org/cgi-bin/mailman/listinfo/adegenet-forum", target="_blank")),


                                                       ## CITATION ##

                                                       h3(br(),"Citing the DAPC Server"),
                                                       h5("Citation for adegenet:"),
                                                       p("Jombart T.(2008) adegenet: a R package for the multivariate analysis of genetic markers. Bioinformatics 24: 1403-1405. doi:10.1093/bioinformatics/btn129", a("[link to paper]", href="http://bioinformatics.oxfordjournals.org/cgi/reprint/btn129?ijkey=6sqx5BTXCdYtBZz&keytype=ref", target="_blank")),
                                                       p("Jombart T. and Ahmed I. (2011) adegenet 1.3-1: new tools for the analysis of genome-wide SNP data. Bioinformatics. doi: 10.1093/bioinformatics/btr521"),

                                                       h5("Citation for the DAPC:"),
                                                       p("Jombart T, Devillard S and Balloux F (2010) Discriminant analysis of principal components: a new method for the analysis of genetically structured populations. BMC Genetics 11:94. doi:10.1186/1471-2156-11-94", a("[link to paper]", href="http://www.biomedcentral.com/1471-2156/11/94", target="_blank")),


                                                       ## SCATTER PLOT ##

                                                       h3(br(),"Scatter Plot"),
                                                       p("The Scatter Plot page provides a visual assessment of between-population differentiation.
                  Generated by applying the R function scatterplot to a dapc object, the output generated will appear in one of two forms.
                  If only one DA is retained (always the case if there are only 2 groups),
                  or both the x-axis and y-axis of the scatterplot are set to the same value, the output will display the
                  densities of individuals on the given discriminant function.
                  If more than one DA is retained and selected, the output will display individuals as dots and
                  groups as inertia ellipses, and will represent the relative position of each along the two
                  selected axes."),

                                                       p("The number of axes retained in both the PCA and DA steps of DAPC will have an impact on the
                  analysis and affect the scatter plot. By default, the number of DA axes retained is set at
                  the maximum of (K - 1) axes, where K is the number of groups. The default value of the number of
                  PCA axes is more arbitrarily defined, however, the 'Use suggested number of PCA components?'
                  tickbox provides the user with the option to use cross-validation to identify and select an optimal number
                  of PCs, where one exists. For more on this, see the section on cross-validation."),

                                                       p("There are a wide variety of graphical parameters for the DAPC scatterplot that can be customised
                  by the user. Those parameters that lack intuitive definition are described further in the Glossary."),


                                                       ## SUMMARY ##

                                                       h3(br(),"Summary"),
                                                       p("This page provides a summary of the dapc object."),
                                                       p("$n.dim' indicates the number of retained DAPC axes, which is affected by both the number of PCA
                  axes and DA axes retained."),
                                                       p("'$n.pop' indicates the number of groups or populations, which is defined by the dataset."),
                                                       p("'$assign.prop' indicates the proportion of overall correct assignment"),
                                                       p("'$assign.per.pop' indicates the proportions of successful reassignment (based on the discriminant
                  functions) of individuals to their original clusters. Large values indicate clear-cut clusters, while low
                  values suggest admixed groups."),
                                                       p("'$prior.grp.size' indicates prior group sizes."),
                                                       p("'$post.grp.size' indicates posterior group sizes."),




                                                       ## COMPOPLOT ##

                                                       h3(br(),"Compoplot"),
                                                       p("This page displays a compoplot, which is a bar plot showing the probabilities of assignment of
                  individuals to the different clusters. Individuals are plotted along the x-axis and membership probabilities are
                  plotted along the y-axis.From the compoplot, one can draw inferences about potential
                  admixture, and about the way in which the selection of PCA axes affects the stability of membership probabilities."),


                                                       ## LOADING PLOT ##

                                                       h3(br(), "Loading Plot"),
                                                       p("The Loading Plot page allows the user to examine how the original variables contribute to the
                  discriminant functions created by DAPC. Variables are plotted along the x-axis, and the contribution
                  of those variables to the DAPC is plotted in the y-axis."),
                                                       p("The side panel on the Loading Plot page provides the option of selecting a threshold above which variables are identified.
                  This can be useful simply for clarifying the image; hence, by default, only variables above the third quartile threshold are labelled.
                  A drop-down menu contains a variety of clustering methods that can also be used to set this threshold.
                  If desired, the user can choose to 'Select and describe features above the threshold'"),


                                                       ## CROSS-VALIDATION ##

                                                       h3(br(),"Cross-validation"),
                                                       p("When the 'Perform cross-validation?' box is ticked, this optimisation procedure will be carried
                  out on the Cross-validation page."),
                                                       p("Cross-validation is an optimisation procedure that is used in the context of DAPC to identify the number of principal components
                  that gives rise to the model with the highest predictive capacity. In cross-validation for DAPC, the data is divided into a training
                  set and a validation set (by default, comprising 90% and 10% of the data, respectively). The analysis is run on the training set with
                  variable numbers of PCs retained, and the degree to which the analysis is able to accurately predict the group membership of
                  excluded individuals (those in the validation set) is used to select the optimal number of PCs to retain. This procedure is replicated
                  with different random sub-samples a number of times specified by a slider on the side panel. In  the interest of computational time,
                  only 3 replicates are performed by default, though more replicates are recommended to achieve greater optimisation.
                  Success is calculated either by group (the default) or measured as overall success."),

                                                       p("A scatterplot of the results is displayed, showing the number of PCs retained on the x-axis and
                  success on the y-axis. Individual replicates appear as dots, and the density of points is displayed
                  in blue."),
                                                       p("Ideally, the data should fall in an arc, indicating an optimal point at its maximum where
                  the number of PCs retained leads to better predictive success than numbers of PCs either
                  above or below."),
                                                       p("Below the plot, a variety of summary statistics are provided.
                  Ultimately, it is the number of PCs associated with the lowest RMSE (root mean squared error,
                  see Glossary) which is selected if 'Use suggested number of PCA components?' is ticked.",br(),br(),br())



                                                       ), # end Help section




                                              ## GLOSSARY ##

                                              tabPanel("Glossary",
                                                       h3("Compoplot"),
                                                       p("A compoplot is a bar plot showing the probabilities of assignment of individuals to the different clusters."),
                                                       h3("Cross-validation"),
                                                       p("Cross-validation is an optimisation procedure that is used in the context of DAPC to identify the number of principal components
                  that gives rise to the model with the highest predictive capacity. In cross-validation for DAPC, the data is divided into a training
                  set and a validation set (by default, comprising 90% and 10% of the data, respectively). The analysis is run on the training set with
                  variable numbers of PCs retained, and the degree to which the analysis is able to accurately predict the group membership of
                  excluded individuals (those in the validation set) is used to select the optimal number of PCs to retain."),
                                                       p("Note: Performing cross-validation can substantially improve the results of DAPC; however, the amount of computational time
                  required increases with the size of the dataset in question and the number of replicates carried out."),
                                                       h3("DA"),
                                                       p("Discriminant Analysis (DA) is a procedure for optimally describing the differences between groups of individuals.
                  DAPC uses DA subsequent to Principal Component Analysis to maximise discrimination between groups in conditions
                  where DA alone would be inappropriate."),
                                                       h3("DA axis"),
                                                       p("DAPC uses Discriminant Analysis (DA) to describe the differences between K groups of individuals along
                  a maximum of (K - 1) Discriminant Analysis axes (DA axes)."),
                                                       h3("DAPC"),
                                                       p("Discriminant Analysis of Principal Components (DAPC) is a multivariate method that uses genetic data to describe the
                  differences between pre-defined biological populations. DAPC uses Principal Component Analysis as a prior step to
                  Discriminant Analysis to identify weighted linear combinations of the original variables which give rise to optimal
                  between-group discrimination."),
                                                       p("For more, see: Jombart T, Devillard S and Balloux F (2010) Discriminant analysis of principal components: a new method for the
                  analysis of genetically structured populations. BMC Genetics11:94. doi:10.1186/1471-2156-11-94."),
                                                       h3("Inertia ellipse"),
                                                       p("The inertia ellipses displayed optionally on the DAPC scatter plot provide graphical summaries of a cloud of points.
                  They are meant to give shape to groups of individuals, and do not necessarily represent a 95% confidence
                  interval on the position of the centroid of a cluster."),
                                                       h3("Loading"),
                                                       p("Loadings provide a measure of the contribution of each original variable to the discrimination between groups along a given
                  discriminant axis. A loading plot is used to visualise these loadings so that one can, for example, assess the weight of each
                  variable and identify those variables whose contributions exceed a threshold of interest."),
                                                       h3("Minimum spanning tree"),
                                                       p("A minimum spanning tree is a graph theoretical phenomenon in which all of the vertices (in our case, cluster centroids) of
                  a graph are connected into the tree that contains the shortest set of possible paths between vertices."),
                                                       h3("PCA"),
                                                       p("Principal Component Analysis (PCA) is a multivariate statistical method that generates linearly
                  uncorrelated principal components (PCs) composed of weighted linear combinations of the original variables
                  to represent overall variation in the data in a reduced space."),
                                                       h3("PC, PCA axis"),
                                                       p("Principal components (PCs, or PCA axes) are sets of linearly uncorrelated synthetic variables composed of
                  weighted linear combinations of original variables. PCs are used to describe multivariate phenomenon in a smaller (or equal)
                  number of dimensions. The 'optimal' number of PCs depends on the data. In DAPC, retaining too few PCs will cause useful
                  information to be excluded from the analysis (and hence discriminative power to be lost), while retaining too many PCs
                  may lead to problems of overfitting and uninterpretability."),
                                                       h3("RMSE"),
                                                       p("The root mean squared error (RMSE) is a measure of the error of an estimator. On the DAPC Server, RMSE is used in
                  cross-validation to assess the ability of the model, with variable numbers of principal components retained, to achieve
                  perfect prediction of individuals into the correct group."),
                                                       p("RMSE = sqrt((1/n)*sum(i=1 to n)(Yhat.i - Yi)^2), where Yhat is a vector of n predictions,
                  and Y is the vector of the true values."),
                                                       h3("Screeplot"),
                                                       p("A graphical representation of the variance, or cumulative variance, contained in the set of principal components
                  or discriminant functions. Components shaded in black represent those that have been retained in the analysis."),
                                                       h3("Training set"),
                                                       p("The training set, in cross-validation, is the set of individuals retained in the analysis.
                  The complement of the training set is the 'validation set', which contains the remaining individuals excluded from the analysis
                  who are used to test the performance of the model which varies as a function of the number of PCs retained.",br(),br(),br())
                                                       ), # end Glossary


                                              ## SERVER INFO ##

                                              tabPanel("System info", verbatimTextOutput("systeminfo"))
                                              ) # end tabsetPanel
                                  ) # end mainPanel
                        ) # end pageWithSidebar
        ) # end shinyUI

#' snapclust's corrected Akaike Information Criterion (AICc)
#'
#' This function computes Akaike Information Criterion for small samples (AICc)
#' for \code{snapclust} results.
#'
#' @author Thibaut Jombart \email{thibautjombart@@gmail.com}
#'
#' @export
#'
#' @param object An object returned by the function \code{\link{snapclust}}.
#'
#' @param ... Further arguments for compatibility with the \code{AIC} generic
#'     (currently not used).
#'
#' @references Beugin M-P, Gayet T, Pontier D, Devillard S, Jombart T. A fast
#'     likelihood solution to the genetic clustering problem. Methods Ecol
#'     Evol. 2018;00:1–11. \url{https://doi.org/10.1111/2041-210X.12968}
#'
#' @seealso
#' \itemize{
#'  \item \code{\link{snapclust}}: to identify clusters
#'
#'  \item \code{\link{snapclust.choose.k}}: to find the number of clusters
#'
#'  \item \code{\link{AIC.snapclust}}: AIC computation
#'
#'  \item \code{\link{BIC.snapclust}}: BIC computation
#'
#'  \item \code{\link{KIC.snapclust}}: KIC computation
#' }
#'
#' @rdname AICc
#'
AICc <- function(object, ...) {
    UseMethod("AICc", object)
}





#' @export
#' @aliases AICc.snapclust
#' @rdname AICc
AICc.snapclust <- function(object, ...) {

    ## The number of parameters is defined as:
    ## (number of independent allele frequencies) x (nb clusters).
    k <- object$n.param
    n <- length(object$group)
    -2 * object$ll + (2 * k * n) / (n - k - 1)

}

#'
#' Expected heterozygosity (Hs)
#'
#' This function computes the expected heterozygosity (Hs) within
#' populations of a \linkS4class{genpop} object. This function is
#' available for codominant markers (\code{@@type="codom"}) only. Hs is
#' commonly used for measuring within population genetic diversity (and
#' as such, it still has sense when computed from haploid data).
#'
#' @aliases Hs
#'
#' @rdname Hs
#'
#' @export
#'
#' @details
#' Let \emph{m(k)} be the number of alleles of locus \emph{k}, with a
#' total of \emph{K} loci. We note \eqn{f_i} the allele frequency of
#' allele \emph{i} in a given population. Then, \eqn{Hs} is given for a
#' given population by:\cr
#'
#' \eqn{\frac{1}{K} \sum_{k=1}^K (1 - \sum_{i=1}^{m(k)} f_i^2)}
#'
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#'
#' @param x a \linkS4class{genind} or \linkS4class{genpop} object.
#' @param pop only used if x is a \linkS4class{genind}; an optional factor to be used as population; if not provided, pop(x) is used.
#'
#' @seealso \code{\link{Hs.test}} to test differences in Hs between two groups
#' @return a vector of Hs values (one value per population)
#' @examples
#' \dontrun{
#' data(nancycats)
#' Hs(genind2genpop(nancycats))
#' }
#'
Hs <- function(x, pop=NULL) {

    ## CHECKS
    if(is.genind(x)){
        if(!is.null(pop)) pop(x) <- pop
        x <- genind2genpop(x, quiet=TRUE)
    }
    if(!is.genpop(x)) stop("x is not a valid genpop object")
    if(x@type=="PA") stop("not implemented for presence/absence markers")


    ## MAIN COMPUTATIONS
    ## get number of typed loci per pop
    if(any(is.na(tab(x)))){
        temp <- apply(tab(x),1,tapply, locFac(x), function(e) !any(is.na(e)))
        nLoc.typed <- apply(temp,2,sum)
    } else {
        nLoc.typed <- nLoc(x)
    }

    F <- tab(x, freq=TRUE, NA.method="asis")
    res <- 1-(apply(F^2,1,sum, na.rm=TRUE))/nLoc.typed

    return(res)
} # end Hs


#'
#' Test differences in expected heterozygosity (Hs)
#'
#' This procedure permits to test if two groups have
#' significant differences in expected heterozygosity (Hs).
#' The test statistic used is simply the difference in Hs
#' between the two groups 'x' and 'y':
#'
#' \eqn{Hs(x) - Hs(y)}
#'
#' Individuals are randomly permuted between groups to obtain
#' a reference distribution of the test statistics.
#'
#' @rdname Hs.test
#' @aliases Hs.test
#'
#' @export
#'
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#'
#' @param x a \linkS4class{genind} object.
#' @param y a \linkS4class{genind} object.
#' @param n.sim the number of permutations to be used to generate the reference distribution.
#' @param alter a character string indicating the alternative hypothesis
#'
#' @seealso \code{\link{Hs}} to compute Hs for different populations;
#' \code{\link[ade4]{as.randtest}} for the class of Monte Carlo tests.
#'
#' @return an object of the class randtest
#'
#' @examples
#' \dontrun{
#' data(microbov)
#' Hs(microbov)
#' test <- Hs.test(microbov[pop="Borgou"],
#'                 microbov[pop="Lagunaire"],
#'                 n.sim=499)
#' test
#' plot(test)
#' }
#'
Hs.test <- function(x, y, n.sim=999, alter=c("two-sided", "greater", "less")){
    ## CHECKS ##
    if(!is.genind(x)) stop("x is not a valid genind object")
    if(!is.genind(y)) stop("y is not a valid genind object")
    if(x@type=="PA" || y@type=="PA") stop("not implemented for presence/absence markers")
    alter <- match.arg(alter)

    ## POOL DATA ##
    xy <- repool(x,y)
    pop(xy) <- rep(c("x","y"), c(nInd(x), nInd(y)))

    ## AUXIL FUNCTION ##
    f1 <- function(x, pop) -diff(Hs(x, pop))

    ## COMPUTE STATS
    ## observed value
    obs <- f1(xy, pop(xy))

    ## permuted
    sim <- replicate(n.sim, f1(xy, sample(pop(xy))))

    ## randtest
    res <- as.randtest(sim=sim, obs=obs, alter=alter)
    res$call <- match.call()

    return(res)

} # end Hs.test

coords.monmonier <- function(x){

if (!inherits(x, "monmonier")) stop("Use only with 'monmonier' objects")

xy.full <- x$xy
n.points <- nrow(xy.full)
k <- 1

#cn list to cn matrix
cn.matr <- matrix(data = 0, nrow = n.points, ncol = n.points)

for(i in 1:n.points){
	eval <- is.element(c(1:n.points), x$cn[[i]])
	cn.matr[i,which(eval == TRUE)] <- 1
	}

#halfway <- matrix(data = NA, nrow = (n.points^2-n.points)/2, ncol = 4)
halfway <- matrix(data = NA, nrow = sum(cn.matr)/2, ncol = 4)
colnames(halfway) <- c("x.hw","y.hw","first","second")

for(first in 1:(n.points-1)){
for(second in (first+1):n.points){
	if(cn.matr[first,second] == 1){
	halfway[k,] <- c(
		(xy.full[first,1]+xy.full[second,1])/2,
		(xy.full[first,2]+xy.full[second,2])/2,
		first, second)
	k <- k+1}
	}}

output=list()
for(run in 1:x$nrun){
	runname <- paste('run',run,sep='')
	output[[runname]] <- list(dir1=list(),dir2=list())

dir1.in <- x[[runname]]$dir1$path
output[[runname]]$dir1 <- matrix(data = NA, nrow = nrow(dir1.in), ncol = 4)
colnames(output[[runname]]$dir1) <- c("x.hw","y.hw","first","second")
rownames(output[[runname]]$dir1) <- rownames(x[[runname]]$dir1$path)
for(i in 1:nrow(dir1.in)){
	eval.x <- is.element(halfway[,1], dir1.in[i,1])
	eval.y <- is.element(halfway[,2], dir1.in[i,2])
	output[[runname]]$dir1[i,] <- halfway[which(eval.x == TRUE & eval.y == TRUE),]
	}

dir2.in <- x[[runname]]$dir2$path
output[[runname]]$dir2 <- matrix(data = NA, nrow = nrow(dir2.in), ncol = 4)
colnames(output[[runname]]$dir2) <- c("x.hw","y.hw","first","second")
rownames(output[[runname]]$dir2) <- rownames(x[[runname]]$dir2$path)
for(i in 1:nrow(dir2.in)){
	eval.x <- is.element(halfway[,1], dir2.in[i,1])
	eval.y <- is.element(halfway[,2], dir2.in[i,2])
	output[[runname]]$dir2[i,] <- halfway[which(eval.x == TRUE & eval.y == TRUE),]
	}
}

return(output)
}

##########
## glSim
##########
glSim <- function(n.ind, n.snp.nonstruc, n.snp.struc = 0, grp.size = c(0.5, 0.5), k = NULL,
                    pop.freq = NULL, ploidy = 1, alpha = 0, parallel = FALSE,
                    LD = TRUE, block.minsize = 10, block.maxsize = 1000, theta = NULL,
                    sort.pop = FALSE, ...){
  
  
  
  ## BASIC CHECKS ##
  if( any(c(n.ind, n.snp.nonstruc+n.snp.struc) <1)) stop("null numbers of individuals and/or SNPs requested")
  ## alpha parameter
  if(alpha>0.5){
    alpha <- 0.5
    warning("alpha cannot exceed 0.5 - changing the value to 0.5 (total forced asymmetry)")
  }
  if(alpha<0){
    alpha <- 0
    warning("alpha cannot be lower than 0 - changing the value to 0 (no forced asymmetry)")
  }
  
  ## handle group sizes  
  if(length(grp.size)!=2) stop("grp.size should be a vector of length 2")
  grp.size <- grp.size/sum(grp.size)
  grpA.size <- grp.size[[1]]*n.ind
  if(grpA.size >= n.ind) stop("grpA.size is >= n.ind")
  grpB.size <- n.ind - grpA.size
  
  
  # handle pop.freq for k populations
  if(is.null(k) & is.null(pop.freq)){
    pop.freq <- 1
  }
  if(!is.null(k) & !is.null(pop.freq)){
    if(k != length(pop.freq)){
      warning("k != length(pop.freq), length(pop.freq) will be taken as k")
    }
  }
  if(!is.null(k) & is.null(pop.freq)){
    pops <-c(1:k) 
    popBaseline <- rep(pops, c(10))
    popBaseline <- factor(sample(popBaseline, length(popBaseline)))
    popExtra <- factor(sample(pops, (n.ind - length(popBaseline)), replace=TRUE))
    pop <- c(popBaseline, popExtra)
    pop <- factor(pop)
    pop.freq <- as.vector(unlist(sapply(pops, function(e) sum(pop==e)))) 
  }
  
  
  n.all <- n.snp.nonstruc
  
  
  
  
  
  simNeutralSNPs <- function(n.ind, n.all, pop.freq=1, LD=TRUE, ploidy=1,
                             block.minsize=10, block.maxsize=1000, theta=NULL,
                             sort.pop=FALSE, parallel=parallel){
    ## CHECKS ##
    ## force population frequencies to 1
    pop.freq <- pop.freq/sum(pop.freq)
    K <- length(pop.freq)
    if(!is.null(theta) && theta<1e-14) theta <- NULL
    
    
    ## GET POPULATION FACTOR ##
    pop <- paste("pop", sample(1:K, n.ind, replace=TRUE, prob=pop.freq),sep=".")
    if(sort.pop) pop <- sort(pop)
    
    
    ## AUXILIARY FUNCTIONS ##
    
    ## this function simulates a block of allele frequencies for K pop
    ## without linkage
    ## nbAll is a number of alleles to simulate
    ## 'pop' is a factor indicating populations
    simBlock.NOLD <- function(nbAll, pop){
      ## function to simulate one allele ##
      simOneAll <- function(){
        f <- runif(K)
        names(f) <- unique(pop)
        return(rbinom(n=n.ind, prob=f[pop], size=ploidy))
      }
      
      ## simulate all allele frequencies ##
      out <- replicate(nbAll, simOneAll())
      return(new("genlight", out, ploidy=ploidy, parallel=parallel))
    } # end simBlock.NOLD
    
    
    ## this function simulates a block of allele frequencies for K pop
    ## with linkage
    ## nbAll is a number of alleles to simulate
    ## 'pop' is a factor indicating populations
    
    if(is.null(theta)){
      ## FUNCTION WITHOUT THETA ##
      simBlock.LD <- function(nbAll, pop){
        ## get master allele ##
        f.ori <- runif(K)
        names(f.ori) <- unique(pop)
        prob.ori <- f.ori[pop]
        
        ## function to simulate one allele ##
        simOneAll <- function(){
          return(rbinom(n=n.ind, prob=prob.ori, size=ploidy))
        }
        ## simulate all allele frequencies ##
        out <- replicate(nbAll, simOneAll())
        out <- matrix(out, ncol=nbAll)
        return(out)
      } # end simBlock.LD
    } else {
      ## FUNCTION WITH THETA ##
      ## function to alter frequencies ##
      tweak.freq <- function(f){
        f[f<.5] <- suppressWarnings(f[f<.5] + runif(length(f[f<.5]), 0, theta))
        f[f>.5] <- suppressWarnings(f[f>.5] - runif(length(f[f>.5]), 0, theta))
        f[f<0] <- 0
        f[f>1] <- 1
        return(f)
      }
      
      simBlock.LD <- function(nbAll, pop){
        ## get master allele ##
        f.ori <- runif(K)
        names(f.ori) <- unique(pop)
        prob.ori <- f.ori[pop]
        
        ## function to simulate one allele ##
        simOneAll <- function(){
          return(rbinom(n=n.ind, prob=tweak.freq(prob.ori), size=ploidy))
        }
        
        ## simulate all allele frequencies ##
        out <- replicate(nbAll, simOneAll())
        out <- matrix(out, ncol=nbAll)
        return(out)
      }
    } # end function with theta
    
    
    ## SIMULATE ALL DATA ##
    if(!LD){ # no LD
      out <- simBlock.NOLD(n.all, pop)
    } else { # with LD IFF n.snp.nonstruc > 0 ! 
      if(n.snp.nonstruc > 0){
      ## determine blocks ##
      block.sizes <- round(runif(1,min=block.minsize,max=block.maxsize))
      while(sum(block.sizes)<n.all){
        block.sizes <- c(block.sizes, round(runif(1,min=block.minsize,max=block.maxsize)))
      }
      block.sizes <- block.sizes[-length(block.sizes)]
      if(sum(block.sizes)<n.all) block.sizes <- c(block.sizes, n.all-sum(block.sizes))
      
      ## simulate all blocks ##
      temp <- lapply(block.sizes, simBlock.LD, pop)
      
      ## put blocks together ##
      out <- temp[[1]]
      if(length(temp)>1){
        for(i in 2:length(temp)){
          out <- cbind(out, temp[[i]])
        }
      }
      out <- new("genlight", out, ploidy=ploidy, parallel=parallel)
    }else{
      out <- simBlock.NOLD(0, pop)
    }}
    out@other <- list(factor(pop))
    names(out@other) <- "ancestral.pops"
    outpop <- list(out, pop)
    return(outpop)
  } # end simNeutralSNPs
  
  
  ## carry out fn to get non-structural SNPs:
  res.ns <- simNeutralSNPs(n.ind, n.snp.nonstruc, pop.freq=pop.freq, ploidy=ploidy, 
                           LD=LD, block.minsize=block.minsize, block.maxsize=block.maxsize, theta=theta,
                           sort.pop=sort.pop, parallel=FALSE)
  
  
  
  ## GET STRUCTURAL SNPS:
  
  ## GET PHEN FACTOR ##
  phen <- paste("phen", sample(rep(c("A", "B"), c(grpA.size, grpB.size)), 
                               n.ind, replace=TRUE), sep=".")
  if(!sort.pop) phen <- sort(phen)           
  
  ## function simulating structural SNPs for phenotypic groups A and B 
  f2 <- function(){
    probA <- runif(1, min=0, max=0.5-alpha)  # generates probA for SNP p
    probB <- 1 - probA                       # generates probB for SNP p
    # get vector of probabilities by phenotype at SNP p               
    phenProbs <- c(probA, probB)
    names(phenProbs) <- unique(phen)
    # draw SNP p for all individuals according to the phenProbs for that SNP
    rbinom(n=rep(1,n.ind), prob=phenProbs[phen], size=ploidy)
  }
  
  if(n.snp.struc > 0){
    struct <- replicate(n.snp.struc, f2())
    struct <- matrix(struct, ncol=n.snp.struc)
    phen <- factor(phen)  
    struct <- new("genlight", struct, ploidy=ploidy, parallel=FALSE)
    struct@pop <- phen
  }  # end snp.struc
  
  
  pop <- res.ns[[2]]
  res.ns <- res.ns[[1]]
  if(n.snp.struc>0){
    res <- cbind(res.ns, struct, parallel = parallel)
    res@pop <- phen
  }
  else{
    res <- res.ns
  }
  res@other <- list(factor(pop))
  names(res@other) <- "ancestral.pops"
  return(res)
}   
# 

context("Genlight construction")


test_that("glSim does not call parallel by default", {
  skip_on_cran()
  if ("parallel" %in% sessionInfo()$basePkgs){
    skip("parallel is already loaded")
  }
  no_parallel <- sessionInfo()$basePkgs
  expect_false("parallel" %in% no_parallel)
  x <- glSim(2, n.snp.nonstruc = 10, n.snp.struc = 10, parallel = FALSE)
  check_parallel <- sessionInfo()$basePkgs
  expect_false("parallel" %in% check_parallel)
  expect_is(x, "genlight")
})

test_that("Genlight objects can be created predictably", {
  skip_on_cran()
  expect_warning(a <- new("genlight", list(c(1,0,1), c(0,0,1,0)), parallel = FALSE ))
  expect_warning(b <- new("genlight", list(c(1,0,1,1,1,1), c(1,0)), parallel = FALSE ))
  locNames(a) <- letters[1:4]
  locNames(b) <- 1:6
  c <- cbind(a, b, parallel = FALSE)
  cbound <- cbind(as.matrix(a), as.matrix(b))
  rbound <- rbind(as.matrix(a),as.matrix(a))
  expect_identical(as.matrix(c), cbound) 
  expect_identical(as.matrix(rbind(a, a, parallel = FALSE)), rbound)
})


x <- new("genlight", list(a=1,b=0,c=1), other=list(1:3, letters, data.frame(2:4)), parallel = FALSE)
pop(x) <- c("pop1","pop1", "pop2")

test_that("subsetting with/without @other works", {
  skip_on_cran()  
  expect_that(x[1:2, ]@other[[1]], equals(1:2))
  expect_that(x[1:2, ]@other[[2]], equals(letters))
  expect_that(x[1:2, ]@other[[3]], equals(x@other[[3]][1:2, , drop = FALSE]))
})

test_that("population accessors work", {
  skip_on_cran()
  expect_that(nPop(x), equals(2))
  expect_that(pop(x), is_equivalent_to(factor(c("pop1","pop1", "pop2"))))
  expect_that(popNames(x), equals(levels(pop(x))))
  popNames(x)[1] <- "replacement"
  expect_that(popNames(x), equals(c("replacement", "pop2")))
  expect_error(popNames(x) <- NULL)
  expect_error(popNames(x)[2] <- NA)
})

x <- "
X13049 X13050 X13051 X13052 X13053
AA36881      2     NA      2      2      2
AA36883      2      2      2      2      2
AA36884      2      2      2      2      2
AA36802     NA      2      2      2      2
AA36803      2      2      2      2      2
AA36804      2     NA      2      2      2
AA36181      2     NA      2      2      2
AA36183      2      2      2      2      2"

xxdf <- read.table(text = x)
xx   <- new("genlight", xxdf, parallel = FALSE)
pop(xx) <- rep(LETTERS[1:2], each = 4)

test_that("missing data is properly subset with logical subscripts", {
  skip_on_cran()
  Apop <- pop(xx) == "A"
  Bpop <- pop(xx) == "B"
  expect_identical(NA.posi(xx), NA.posi(xx[]))
  expect_identical(xxdf[Apop, ], as.data.frame(xx[Apop, ]))
  expect_identical(xxdf[Bpop, ], as.data.frame(xx[Bpop, ]))
  keepers <- c(FALSE, rep(TRUE, 4))
  expect_identical(xxdf[keepers], as.data.frame(xx[, keepers]))
})

test_that("missing data is properly subset with positive subscripts", {
  skip_on_cran()
  rl <- sample(5)
  # Can subset single locus
  expect_identical(xxdf[, 1, drop = FALSE], as.data.frame(xx[, 1]))
  # Can subset range of loci
  expect_identical(xxdf[, 1:3, drop = FALSE], as.data.frame(xx[, 1:3]))
  # Can subset by position
  expect_identical(xxdf[, rl, drop = FALSE], as.data.frame(xx[, rl]))
})

test_that("missing data is properly subset with negative subscripts", {
  skip_on_cran()
  expect_identical(xxdf[, -1], as.data.frame(xx[, -1]))
  expect_identical(xxdf[, -c(1, 3)], as.data.frame(xx[, -c(1, 3)]))
})

test_that("missing data is properly subset with a character vector", {
  skip_on_cran()
  lnames <- locNames(xx)
  rl     <- sample(lnames)
  expect_identical(xxdf[, rl], as.data.frame(xx[, rl]))
  expect_identical(xxdf[, lnames[1:2]], as.data.frame(xx[, lnames[1:2]]))
})

test_that("genlight objects do not take a mixture of positive and negative subscripts", {
  skip_on_cran()
  expect_error(xx[, c(2, -1)], ".* may be mixed with negative subscript")
})

#################
## findMutations
#################

## GENERIC
findMutations <- function(...){
    UseMethod("findMutations")
}



## METHOD FOR DNABIN
findMutations.DNAbin <- function(x, from=NULL, to=NULL, allcomb=TRUE, ...){
    ## CHECKS ##
    if(!inherits(x,"DNAbin")) stop("x is not a DNAbin object")
    x <- as.matrix(x)

    ## function to pull out mutations from sequence a to b ##
    NUCL <- c('a','t','g','c')
    f1 <- function(a,b){
        seqa <- as.character(x[a,])
        seqb <- as.character(x[b,])
        temp <- which(seqa != seqb)
        ori <- seqa[temp]
        mut <- seqb[temp]
        names(ori) <- names(mut) <- temp
        toRemove <- !ori %in% NUCL | !mut %in% NUCL
        ori <- ori[!toRemove]
        mut <- mut[!toRemove]
        if(all(toRemove)) return(NULL)
        res <- data.frame(ori,mut)
        names(res) <- rownames(x)[c(a,b)]
        res$short <- paste(row.names(res),":",res[,1],"->",res[,2],sep="")
        return(res)
    }

    ## GET LIST OF PAIRS TO COMPARE ##
    ## handle from/to as character
    if(is.character(from)) from <- match(from, rownames(x))
    if(is.character(to)) to <- match(to, rownames(x))

    ## handle NULL
    if(is.null(from)) from <- 1:nrow(x)
    if(is.null(to)) to <- 1:nrow(x)

    ## get pairs
    if(allcomb){
        pairs <- expand.grid(to, from)[,2:1,drop=FALSE]
    } else {
        N <- max(length(from),length(to))
        from <- rep(from, length=N)
        to <- rep(to, length=N)
        pairs <- cbind(from, to)
    }

    ## remove unwanted comparisons
    pairs <- pairs[pairs[,1]!=pairs[,2],,drop=FALSE]

    ## GET NUMBER OF MUTATIONS ##
    out <- lapply(1:nrow(pairs), function(i) f1(pairs[i,1], pairs[i,2]))
    names(out) <- paste(rownames(x)[pairs[,1]], rownames(x)[pairs[,2]],sep="->")

    return(out)

} # end findMutations







##################
## graphMutations
##################

## GENERIC
graphMutations <- function(...){
    UseMethod("graphMutations")
}



## METHOD FOR DNABIN
graphMutations.DNAbin <- function(x, from=NULL, to=NULL, allcomb=TRUE, plot=TRUE, curved.edges=TRUE, ...){

    ## GET MUTATIONS ##
    x <- findMutations(x, from=from, to=to, allcomb=allcomb)

    ## GET GRAPH ##
    from <- gsub("->.*","",names(x))
    to <- gsub(".*->","",names(x))
    vnames <- sort(unique(c(from,to)))
    dat <- data.frame(from,to,stringsAsFactors=FALSE)
    out <- graph.data.frame(dat, directed=TRUE, vertices=data.frame(vnames, label=vnames))

    ## SET ANNOTATIONS FOR THE BRANCHES ##
    annot <- unlist(lapply(x, function(e) paste(e$short, collapse="\n")))
    E(out)$label <- annot
    E(out)$curved <- curved.edges

    ## PLOT / RETURN ##
    if(plot) plot(out, ...)

    return(out)
} # end graphMutations

## ##########################
## ## Function gstat.randtest
## ##########################
## gstat.randtest <- function(x,pop=NULL, method=c("global","within","between"),
##                            sup.pop=NULL, sub.pop=NULL, nsim=499){
##     message("Sorry, this function depends on hierfstat, and has been moved to the package hierfstat.")
##     return()

##     ## ##   cat("\nSorry, hierfstat package has been disabled - this function will be restored in a future release.\n")
##     ## ## return(invisible())

##     ## if(!is.genind(x)) stop("x is not a valid genind object")
##     ## if(x@ploidy != as.integer(2)) stop("not implemented for non-diploid genotypes")
##     ## checkType(x)
##     ## ## if(!require(hierfstat)) stop("hierfstat package is required. Please install it.")

##     ## if(is.null(pop)) pop <- x@pop
##     ## if(is.null(pop)) pop <- as.factor(rep("P1",nrow(x@tab)))
##     ## if(length(pop)!=nrow(x@tab)) stop("pop has a wrong length.")

##     ## met <- tolower(method[1])
##     ## if(met=="within" && is.null(sup.pop)) stop("Method 'within' chosen but 'sup.pop' is not provided.")
##     ## if(met=="between" && is.null(sub.pop)) stop("Method 'between' chosen but 'sub.pop' is not provided.")

##     ## ## make data for hierfstat
##     ## X <- genind2hierfstat(x=x,pop=pop)

##     ## ## compute obs gstat
##     ## obs <- g.stats.glob(X)$g.stats

##     ## pop <- X[,1]
##     ## X <- X[,-1]

##     ## ## simulations according one of the 3 different schemes
##     ## ## note: for, lapply and sapply are all equivalent
##     ## ## recursive functions would require options("expression") to be modified...
##     ## sim <- vector(mode="numeric",length=nsim)

##     ## if(met=="global"){

##     ##     sim <- sapply(1:nsim, function(i) g.stats.glob(cbind(sample(pop),X))$g.stats)

##     ## } else if(met=="within"){

##     ##     if(length(sup.pop) != length(pop)) stop("pop and sup.pop do not have the same length.")
##     ##     sim <- sapply(1:nsim, function(i) g.stats.glob(cbind(pop,X[samp.within(sup.pop),]))$g.stats)

##     ## } else if(met=="between"){

##     ##     if(length(sub.pop) != length(pop)) stop("pop and sub.pop do not have the same length.")
##     ##     sim <- sapply(1:nsim, function(i) g.stats.glob(cbind(pop,X[samp.between(sub.pop),]))$g.stats)

##     ## } else {
##     ##     stop("Unknown method requested.")
##     ## }

##     ## prevcall <- match.call()

##     ## res <- as.randtest(sim=sim, obs=obs, call=prevcall)

##     ## return(res)

## }

###########################
#
# Auxiliary functions for
# adegenet objects
#
# T. Jombart
###########################


#######################
# Function rmspaces
#######################
# removes spaces and tab at the begining and the end of each element of charvec
.rmspaces <- function(charvec){
    charvec <- gsub("^([[:blank:]]*)([[:space:]]*)","",charvec)
    charvec <- gsub("([[:blank:]]*)([[:space:]]*)$","",charvec)
    return(charvec)
}





###################
# Function readExt
###################
.readExt <- function(char){
    temp <- as.character(char)
    temp <- unlist(strsplit(char,"[.]"))
    res <- temp[length(temp)]
    return(res)
}





###################
# Function .genlab
###################
# recursive function to have labels of constant length
# base = a character string
# n = number of labels
.genlab <- function(base, n) {
  f1 <- function(cha,n){
    if(nchar(cha)<n){
      cha <- paste("0",cha,sep="")
      return(f1(cha,n))
    } else {return(cha)}
  }
  w <- as.character(1:n)
  max0 <- max(nchar(w))
  w <- sapply(w, function(cha) f1(cha,max0))
  return(paste(base,w,sep=""))
}





#'
#' Functions to access online resources for adegenet
#'
#' These functions simply open websites or documents
#' available online providing resources for adegenet.
#'
#' \itemize{
#' \item adegenetWeb opens adegenet's website
#' \item adegenetTutorial opens adegenet tutorials
#' \item adegenetIssues opens the issue page on github;
#' this is used to report a bug or post a feature request.
#' }
#'
#' @param which a character string indicating which tutorial to open (see details)
#'
#' @details
#'
#' Available tutorials are:
#' \itemize{
#' \item 'basics': general introduction to adegenet; covers basic data structures, import/export, handling, and a number of population genetics methods
#' \item 'spca': spatial genetic structures using the spatial Principal Component Analysis
#' \item 'dapc': population structure using the Discriminant Analysis of Principal Components
#' \item 'genomics': handling large genome-wide SNP data using adegenet
#' \item 'strata': introduction to hierarchical population structure in adegenet
#' \item 'snapclust': introduction to fast maximum-likelihood genetic clustering using snapclust
#' }
#'
#' @export
#'
#' @rdname web
#'
adegenetWeb <- function(){
    cat("Opening url \"http://adegenet.r-forge.r-project.org/\" ...\n")
    browseURL("http://adegenet.r-forge.r-project.org/")
}




#' @rdname web
#' @export
adegenetTutorial <- function(which = c("basics","spca","dapc","genomics","strata","snapclust")){

    ## which <- match.arg(which)
    which <- which[1]
    choices <- c("basics","spca","dapc","genomics","strata","snapclust")
    if (!which %in% choices) {
        stop("Unknown tutorial")
    }
    if(which=="basics"){
        url <- "https://github.com/thibautjombart/adegenet/raw/master/tutorials/tutorial-basics.pdf"
        cat("\n")
        cat("  >> Opening the general introduction to adegenet.\n")
        cat("  >> Seeking url: ",url,"\n ", sep="")
        cat("\n")
    }
    if(which=="spca"){
        url <- "https://github.com/thibautjombart/adegenet/raw/master/tutorials/tutorial-spca.pdf"
        cat("\n")
        cat("  >> Opening the sPCA tutorial.\n")
        cat("  >> Seeking url: ",url,"\n ", sep="")
        cat("\n")
    }
    if(which=="dapc"){
        url <- "https://github.com/thibautjombart/adegenet/raw/master/tutorials/tutorial-dapc.pdf"
        cat("\n")
        cat("  >> Opening the DAPC tutorial.\n")
        cat("  >> Seeking url: ",url,"\n ", sep="")
        cat("\n")
    }
    if(which=="genomics"){
        url <- "https://github.com/thibautjombart/adegenet/raw/master/tutorials/tutorial-genomics.pdf"
        cat("\n")
        cat("  >> Opening the genomics tutorial.\n")
        cat("  >> Seeking url: ",url,"\n ", sep="")
        cat("\n")
    }
   if(which=="strata"){
        url <- "https://github.com/thibautjombart/adegenet/raw/master/tutorials/tutorial-strata.pdf"
        cat("\n")
        cat("  >> Opening the strata tutorial.\n")
        cat("  >> Seeking url: ",url,"\n ", sep="")
        cat("\n")
    }
    if(which=="snapclust"){
        url <- "https://github.com/thibautjombart/adegenet/raw/master/tutorials/tutorial-snapclust.pdf"
        cat("\n")
        cat("  >> Opening the snapclust tutorial.\n")
        cat("  >> Seeking url: ",url,"\n ", sep="")
        cat("\n")
    }

    browseURL(url)

    return(invisible(NULL))
}




#' @rdname web
#' @export
adegenetIssues <- function(){
    cat("Opening url \"https://github.com/thibautjombart/adegenet/issues\" ...\n")
    browseURL("https://github.com/thibautjombart/adegenet/issues")
}





############
# checkType
############
##
## WARNING: this does not work with S4 methods
##
checkType <- function(x){
    if(is.character(x)){
        markType <- x
    } else {
        markType <- x@type
    }

    if(markType=="codom") return() # always ok for codominant markers

    currCall <- as.character(sys.call(sys.parent()))[1]
    currFunction <- sub("[[:space:]]*[(].*","",currCall)
    if(currFunction==".local"){
        warning("Current call not found - stopping check (please report this warning).")
        return()
    }

    ## names of functions which are ok for dominant markers
    PAOk <- c("genind","genpop","genind2genpop","summary","df2genind", "genind2df",
                 "truenames","seppop","na.replace","nLoc","scaleGen","spca","selpop")

    PAWarn <- c("df2genind")

    ## function exists but is experimental
    if(currFunction %in% PAWarn){
        msg <- paste(currFunction,"is implemented but experimental presence/absence markers")
        warning(msg)
        return()
    }

    ## function not implemented
    if(! currFunction %in% PAOk){
        msgError <- paste(currFunction,"is not implemented for presence/absence markers")
        stop(msgError)
    } else return() # else, ok.
} # end checkType






##########
## transp
##########
## AUXIL FUNCTION TO USE TRANSPARENT COLORS
transp <- function(col, alpha=.5){
    res <- apply(col2rgb(col),2, function(c) rgb(c[1]/255, c[2]/255, c[3]/255, alpha))
    return(res)
}



##########
## corner
##########
## AUXIL FUNCTION TO ADD LETTER TO A PLOT
corner <- function(text, posi="topleft",  inset=0.1, ...){
    oxpd <- par("xpd")
    on.exit(par(xpd=oxpd))
    par(xpd=TRUE)
    myUsr <- par("usr")
    xrange <- myUsr[1:2]
    yrange <- myUsr[3:4]
    x.size <- abs(diff(xrange))
    y.size <- abs(diff(yrange))
    inset <- rep(inset, length=2)
    x.inset <- inset[1]
    y.inset <- inset[2]

    if(length(grep("top", posi))==1){
        y <- yrange[2] - y.size*y.inset
    } else {
        y <- yrange[1] + y.size*y.inset
    }

    if(length(grep("right", posi))==1){
        x <- xrange[2] - x.size*x.inset
    } else {
        x <- xrange[1] + x.size*x.inset
    }

    text(x, y, lab=text, ...)
}



.listenToTheBrood <- function(){
    char <- c("?","??","?!?!?")
    for(i in 1:3){
        cat("\nGrind",  char[i], " (y/N): ")
        x <- readLines(con = getOption('adegenet.testcon'), n=1)
        if(x!="y") {
            cat("\n =( \n")
            return(invisible())
        }
    }

    cat("\nGRIIIINNND !!!\n")
    cat("\n\\m/ ^o^ \\m/\n\n")
    url.vec <- c("http://thebrooduk.bandcamp.com/album/swallowed-by-the-earth-single",
                 "http://thebrooduk.bandcamp.com/album/the-brood")
    Sys.sleep(2)
    browseURL(sample(url.vec,1))
}



###########
## num2col
###########
## translate numeric values into colors of a palette
num2col <- function(x, col.pal=heat.colors, reverse=FALSE,
                    x.min=min(x,na.rm=TRUE), x.max=max(x,na.rm=TRUE),
                    na.col="transparent"){
    ## if(any(is.na(x))) warning("NAs detected in x")
    x[x < x.min] <- x.min
    x[x > x.max] <- x.max
    x <- x-x.min # min=0
    x.max <- x.max-x.min # update x.max
    x <- x/x.max # max=1
    x <- round(x*100)
    x[x<=0] <- 1
    if(!reverse) {
        pal <- col.pal(100)
    } else {
        pal <- rev(col.pal(100))
    }

    res <- pal[x]
    res[is.na(res)] <- na.col
    return(res)
}





###########
## fac2col
###########
## translate a factor into colors of a palette
## colors are randomized based on the provided seed
fac2col <- function(x, col.pal=funky, na.col="transparent", seed=NULL){
    ## get factors and levels
    x <- factor(x)
    lev <- levels(x)
    nlev <- length(lev)

    ## get colors corresponding to levels
    if(!is.null(seed)){
        set.seed(seed)
        newseed <- round(runif(1,1,1e9))
        on.exit(set.seed(newseed))
        col <- sample(col.pal(nlev))
    } else {
        col <- col.pal(nlev)
    }

    ## get output colors
    res <- rep(na.col, length(x))
    res[!is.na(x)] <- col[as.integer(x[!is.na(x)])]

    ## return
    return(res)
}


###########
## any2col
###########
any2col <- function(x, col.pal=seasun, na.col="transparent"){
    ## handle numeric data
    if(is.numeric(x)){
        col <- num2col(x, col.pal=col.pal, na.col=na.col)
        leg.col <- num2col(pretty(x), x.min=min(x, na.rm=TRUE),
                           x.max=max(x, na.rm=TRUE), col.pal=col.pal,
                           na.col=na.col)
        leg.txt <- pretty(x)
    } else{ ## handle factor
        x <- factor(x)
        col <- fac2col(x, col.pal=col.pal, na.col=na.col)
        leg.col <- col.pal(length(levels(x)))
        leg.txt <- levels(x)
    }

    return(list(col=col, leg.col=leg.col, leg.txt=leg.txt))
} # end any2col



## pre-defined palettes ##
## mono color
bluepal <- colorRampPalette(c("#F7FBFF","#DEEBF7","#C6DBEF",
                              "#9ECAE1","#6BAED6","#4292C6",
                              "#2171B5","#08519C","#08306B"))
redpal <- colorRampPalette(c("#FFF5F0","#FEE0D2","#FCBBA1",
                             "#FC9272","#FB6A4A","#EF3B2C",
                             "#CB181D","#A50F15","#67000D"))
greenpal <- colorRampPalette(c("#F7FCF5","#E5F5E0","#C7E9C0",
                               "#A1D99B","#74C476","#41AB5D",
                               "#238B45","#006D2C","#00441B"))
greypal <- colorRampPalette(c("#FFFFFF","#F0F0F0","#D9D9D9",
                              "#BDBDBD","#969696","#737373",
                              "#525252","#252525","#000000"))

## bi-color
flame <- colorRampPalette(c("gold","red3"))
azur <- colorRampPalette(c("gold","royalblue"))

## tri-color
seasun <- colorRampPalette(c("blue","gold","red"))
lightseasun <- colorRampPalette(c("deepskyblue2","gold","red1"))
deepseasun <- colorRampPalette(c("blue2","gold","red2"))
wasp <-  colorRampPalette(c("yellow2","brown", "black"))
spectral <- colorRampPalette(c("#D53E4F","#F46D43","#FDAE61",
                               "#FEE08B","#FFFFBF","#E6F598",
                               "#ABDDA4","#66C2A5","#3288BD"))

## psychedelic
funky <- colorRampPalette(c("#A6CEE3","#1F78B4","#B2DF8A",
                            "#33A02C","#FB9A99","#E31A1C",
                            "#FDBF6F","#FF7F00","#CAB2D6",
                            "#6A3D9A","#FFFF99","#B15928"))



## viridis
virid <- colorRampPalette(c("#440154FF", "#482173FF", "#433E85FF", "#38598CFF",
                            "#2D708EFF", "#25858EFF", "#1E9B8AFF", "#2BB07FFF",
                            "#51C56AFF", "#85D54AFF", "#C2DF23FF", "#FDE725FF"))


## reorder colors for hybrids
hybridpal <- function(col.pal = virid) {
    function(n) {
        if (n < 3) {
            return(col.pal(n))
        }
        col.pal(n)[c(1, n, 2:(n-1))]
    }
}

#' Internal C routines
#'
#' These functions are internal C routines used in adegenet. Do not use them
#' unless you know what you are doing.
#'
#' @aliases CheckAllSeg GLdotProd GLsumFreq GLsumInt binIntToBytes bytesToBinInt
#' bytesToInt nb_shared_all
#'
#' @author Thibaut Jombart
#'
#' @export
#'
#' @rdname doc_C_routines
#' 
".internal_C_routines" <- NULL

#####################
# Function chooseCN
#####################


#' Function to choose a connection network
#'
#' The function \code{chooseCN} is a simple interface to build a connection
#' network (CN) from xy coordinates. The user chooses from 6 types of graph and
#' one additional weighting scheme.  \code{chooseCN} calls functions from
#' appropriate packages, handles non-unique coordinates and returns a
#' connection network either with classe \code{nb} or \code{listw}. For graph
#' types 1-4, duplicated locations are not accepted and will issue an error.
#'
#' There are 7 kinds of graphs proposed: \cr Delaunay triangulation (type 1)\cr
#' Gabriel graph (type 2)\cr Relative neighbours (type 3)\cr Minimum spanning
#' tree (type 4)\cr Neighbourhood by distance (type 5)\cr K nearests neighbours
#' (type 6)\cr Inverse distances (type 7)\cr
#'
#' The last option (type=7) is not a true neighbouring graph: all sites are
#' neighbours, but the spatial weights are directly proportional to the
#' inversed spatial distances.\cr Also not that in this case, the output of the
#' function is always a \code{listw} object, even if \code{nb} was
#' requested.\cr
#'
#' The choice of the connection network has been discuted on the adegenet
#' forum. Please search the archives from adegenet website (section 'contact')
#' using 'graph' as keyword.
#'
#' @param xy an matrix or data.frame with two columns for x and y coordinates.
#' @param ask a logical stating whether graph should be chosen interactively
#' (TRUE,default) or not (FALSE). Set to FALSE if \code{type} is provided.
#' @param type an integer giving the type of graph (see details).
#' @param result.type a character giving the class of the returned object.
#' Either "nb" (default) or "listw", both from \code{spdep} package. See
#' details.
#' @param d1 the minimum distance between any two neighbours. Used if
#' \code{type=5.}
#' @param d2 the maximum distance between any two neighbours. Used if
#' \code{type=5}. Can also be a character: "dmin" for the minimum distance so
#' that each site has at least one connection, or "dmax" to have all sites
#' connected (despite the later has no sense).
#' @param k the number of neighbours per point. Used if \code{type=6}.
#' @param a the exponent of the inverse distance matrix. Used if \code{type=7}.
#' @param dmin the minimum distance between any two distinct points. Used to
#' avoid infinite spatial proximities (defined as the inversed spatial
#' distances). Used if \code{type=7}.
#' @param plot.nb a logical stating whether the resulting graph should be
#' plotted (TRUE, default) or not (FALSE).
#' @param edit.nb a logical stating whether the resulting graph should be
#' edited manually for corrections (TRUE) or not (FALSE, default).
#' @param check.duplicates a logical indicating if duplicate coordinates should be detected; this can be an issue for some graphs; TRUE by default.
#'
#' @return Returns a connection network having the class \code{nb} or
#' \code{listw}. The xy coordinates are passed as attribute to the created
#' object.
#'
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#'
#' @seealso \code{\link{spca}}
#' @keywords spatial utilities
#' @examples
#'
#' \dontrun{
#' data(nancycats)
#'
#' par(mfrow=c(2,2))
#' cn1 <- chooseCN(nancycats@@other$xy,ask=FALSE,type=1)
#' cn2 <- chooseCN(nancycats@@other$xy,ask=FALSE,type=2)
#' cn3 <- chooseCN(nancycats@@other$xy,ask=FALSE,type=3)
#' cn4 <- chooseCN(nancycats@@other$xy,ask=FALSE,type=4)
#' par(mfrow=c(1,1))
#' }
#'
#' @export chooseCN
#' @importFrom spdep "tri2nb" "gabrielneigh" "graph2nb" "relativeneigh" "dnearneigh" "knearneigh" "knn2nb" "nb2listw" "mat2listw" "listw2mat" "lag.listw" "card"
#' @import ade4
#'
chooseCN <- function(xy, ask = TRUE, type = NULL, result.type = "nb",
                     d1 = NULL, d2 = NULL, k = NULL, a = NULL,
                     dmin = NULL, plot.nb = TRUE, edit.nb = FALSE,
                     check.duplicates = TRUE){

  if(is.data.frame(xy)) xy <- as.matrix(xy)
  if(ncol(xy) != 2) stop("xy does not have two columns.")
  if(any(is.na(xy))) stop("NA entries in xy.")
  result.type <- tolower(result.type)
   if(is.null(type) & !ask) stop("Non-interactive mode but no graph chosen; please provide a value for 'type' argument.")

  ## if(!require(spdep, quietly=TRUE)) stop("spdep library is required.")

  res <- list()

  if(!is.null(d2)){
      if(d2=="dmin"){
          tempmat <- as.matrix(dist(xy))
          d2min <- max(apply(tempmat, 1, function(r) min(r[r>1e-12])))
          d2min <- d2min * 1.0001 # to avoid exact number problem
          d2 <- d2min
      } else if(d2=="dmax"){
          d2max <- max(dist(xy))
          d2max <- d2max * 1.0001 # to avoid exact number problem
          d2 <- d2max
      }
  } # end handle d2

  d1.first <- d1
  d2.first <- d2
  k.first <- k

  ## handle type argument
  if(!is.null(type)){
      type <- as.integer(type)
      if(type < 1 |type > 7) stop("type must be between 1 and 7")
      ask <- FALSE
  }

  ## check for uniqueness of coordinates
  if(check.duplicates && any(xyTable(xy)$number>1)){ # if duplicate coords
      DUPLICATE.XY <- TRUE
  } else {
      DUPLICATE.XY <- FALSE
  }


  ## if(is.null(type) & !ask) { type <- 1 }

  ### begin large while ###
  chooseAgain <- TRUE
  while(chooseAgain){
    # re-initialisation of some variables
    d1 <- d1.first
    d2 <- d2.first
    k <- k.first

  ## read type from console
    if(ask){
      temp <- TRUE
      while(temp){
        cat("\nChoose a connection network:\n")
        cat("\t Delaunay triangulation (type 1)\n")
        cat("\t Gabriel graph (type 2)\n")
        cat("\t Relative neighbours (type 3)\n")
        cat("\t Minimum spanning tree (type 4)\n")
        cat("\t Neighbourhood by distance (type 5)\n")
        cat("\t K nearest neighbours (type 6)\n")
        cat("\t Inverse distances (type 7)\n")
        cat("Answer: ")

        type <- as.integer(readLines(con = getOption('adegenet.testcon'), n = 1))
        temp <- type < 1 |type > 7
        if(temp) cat("\nWrong answer\n")

        if(type %in% 1:4 & DUPLICATE.XY){
            cat("\n\n== PROBLEM DETECTED ==")
            cat("\nDuplicate locations detected\nPlease choose another graph (5-7) or add random noise to locations (see ?jitter).\n")
            temp <- TRUE
        }

      } # end while
    }
    ##

    ## warning about duplicate xy coords
    if(type %in% 1:4 & DUPLICATE.XY){
        stop("Duplicate locations detected and incompatible with graph type 1-4.\nPlease choose another graph (5-7) or add random noise to locations (see ?jitter).")
    }

    ## graph types
    ## type 1: Delaunay
    if(type==1){
      ## if(!require(tripack, quietly=TRUE)) stop("tripack library is required.")
      cn <- tri2nb(xy)
    }

    # type 2: Gabriel
    if(type==2){
      cn <- gabrielneigh(xy)
      cn <- graph2nb(cn, sym=TRUE)
    }

    ## type 3: Relative neighbours
    if(type==3){
      cn <- relativeneigh(xy)
      cn <- graph2nb(cn, sym=TRUE)
    }

    ## type 4: Minimum spanning tree
    if(type==4){
      cn <- ade4::mstree(dist(xy)) # there is also a spdep::mstree
      cn <- neig2nb(cn)
    }

    ## type 5: Neighbourhood by distance
    if(type==5){
      if(is.null(d1) |is.null(d2)){
        tempmat <- as.matrix(dist(xy))
        d2min <- max(apply(tempmat, 1, function(r) min(r[r>1e-12])))
        d2min <- d2min * 1.0001 # to avoid exact number problem
        d2max <- max(dist(xy))
        d2max <- d2max * 1.0001 # to avoid exact number problem
        dig <- options("digits")
        options("digits=5")
        cat("\n Enter minimum distance: ")
        d1 <- as.numeric(readLines(con = getOption('adegenet.testcon'), n = 1))
        cat("\n Enter maximum distance \n(dmin=", d2min, ", dmax=", d2max, "): ")
        d2 <- readLines(con = getOption('adegenet.testcon'), n = 1)
        ## handle character
        if(d2=="dmin") {
            d2 <- d2min
        } else if(d2=="dmax") {
            d2 <- d2max
        } else {
            d2 <- as.numeric(d2)
        }
        ## restore initial digit option
        options(dig)
      }
    # avoid that a point is its neighbour
      dmin <- mean(dist(xy))/100000
      if(d1<dmin) d1 <- dmin
      if(d2<d1) stop("d2 < d1")
      cn <- dnearneigh(x=xy, d1=d1, d2=d2)
    }

    ## type 6: K nearests
    if(type==6){
      if(is.null(k)) {
        cat("\n Enter the number of neighbours: ")
        k <- as.numeric(readLines(con = getOption('adegenet.testcon'), n = 1))
      }
      cn <- knearneigh(x=xy, k=k)
      cn <- knn2nb(cn, sym=TRUE)
    }

    ## type 7: inverse distances
    if(type==7){
        if(is.null(a)) {
            cat("\n Enter the exponent: ")
            a <- as.numeric(readLines(con = getOption('adegenet.testcon'), n = 1))
        }
        cn <- as.matrix(dist(xy))
        if(is.null(dmin)) {
            cat("\n Enter the minimum distance \n(range = 0 -", max(cn),"): ")
            dmin <- as.numeric(readLines(con = getOption('adegenet.testcon'), n = 1))
        }
        if(a<1) { a <- 1 }
        thres <- mean(cn)/1e8
        if(dmin < thres) dmin <- thres
        cn[cn < dmin] <- dmin
        cn <- 1/(cn^a)
        diag(cn) <- 0
        cn <- prop.table(cn,1)
        plot.nb <- FALSE
        edit.nb <- FALSE
        result.type <- "listw"
    } # end type 7

    ## end graph types

    if(ask & plot.nb) {
      plot(cn,xy)
      cat("\nKeep this graph (y/n)? ")
    ans <- tolower(readLines(con = getOption('adegenet.testcon'), n=1))
      if(ans=="n") {chooseAgain <- TRUE} else {chooseAgain <- FALSE}
    }
    else if(plot.nb){
      plot(cn,xy)
      chooseAgain <- FALSE
    }
  else {chooseAgain <- FALSE}

  }
### end large while

  if(edit.nb) {cn <- edit(cn,xy)}

  if(result.type == "listw") {
      if(type!=7) {
          cn <- nb2listw(cn, style="W", zero.policy=TRUE)
      } else {
          cn <- mat2listw(cn)
          cn$style <- "W"
      }
  }

  res <- cn

  attr(res,"xy") <- xy

  return(res)

} # end chooseCN

#######
## dapc
########
dapc <- function (x, ...) UseMethod("dapc")

###################
## dapc.data.frame
###################
dapc.data.frame <- function(x, grp, n.pca=NULL, n.da=NULL,
                            center=TRUE, scale=FALSE, var.contrib=TRUE, var.loadings=FALSE,
                            pca.info=TRUE, pca.select=c("nbEig","percVar"), perc.pca=NULL, ..., dudi=NULL){

    ## FIRST CHECKS
    grp <- as.factor(grp)
    if(length(grp) != nrow(x)) stop("Inconsistent length for grp")
    pca.select <- match.arg(pca.select)
    if(!is.null(perc.pca) & is.null(n.pca)) pca.select <- "percVar"
    if(is.null(perc.pca) & !is.null(n.pca)) pca.select <- "nbEig"
    if(!is.null(dudi) && !inherits(dudi, "dudi")) stop("dudi provided, but not of class 'dudi'")


    ## SOME GENERAL VARIABLES
    N <- nrow(x)
    REDUCEDIM <- is.null(dudi)

    if(REDUCEDIM){ # if no dudi provided
        ## PERFORM PCA ##
        maxRank <- min(dim(x))
        pcaX <- dudi.pca(x, center = center, scale = scale, scannf = FALSE, nf=maxRank)
    } else { # else use the provided dudi
        pcaX <- dudi
    }
    cumVar <- 100 * cumsum(pcaX$eig)/sum(pcaX$eig)

    if(!REDUCEDIM){
        myCol <- rep(c("black", "lightgrey"), c(ncol(pcaX$li),length(pcaX$eig)))
    } else {
        myCol <- "black"
    }

    ## select the number of retained PC for PCA
    if(is.null(n.pca) & pca.select=="nbEig"){
        plot(cumVar, xlab="Number of retained PCs", ylab="Cumulative variance (%)", main="Variance explained by PCA", col=myCol)
        cat("Choose the number PCs to retain (>=1): ")
        n.pca <- as.integer(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    if(is.null(perc.pca) & pca.select=="percVar"){
        plot(cumVar, xlab="Number of retained PCs", ylab="Cumulative variance (%)", main="Variance explained by PCA", col=myCol)
        cat("Choose the percentage of variance to retain (0-100): ")
        nperc.pca <- as.numeric(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    ## get n.pca from the % of variance to conserve
    if(!is.null(perc.pca)){
        n.pca <- min(which(cumVar >= perc.pca))
        if(perc.pca > 99.999) n.pca <- length(pcaX$eig)
        if(n.pca<1) n.pca <- 1
    }


    ## keep relevant PCs - stored in XU
    X.rank <- sum(pcaX$eig > 1e-14)
    n.pca <- min(X.rank, n.pca)
    if(n.pca >= N) n.pca <- N-1
    n.pca <- round(n.pca)

    U <- pcaX$c1[, 1:n.pca, drop=FALSE] # principal axes
    rownames(U) <- colnames(x) # force to restore names
    XU <- pcaX$li[, 1:n.pca, drop=FALSE] # principal components
    XU.lambda <- sum(pcaX$eig[1:n.pca])/sum(pcaX$eig) # sum of retained eigenvalues
    names(U) <- paste("PCA-pa", 1:ncol(U), sep=".")
    names(XU) <- paste("PCA-pc", 1:ncol(XU), sep=".")


    ## PERFORM DA ##
    ldaX <- lda(XU, grp, tol=1e-30) # tol=1e-30 is a kludge, but a safe (?) one to avoid fancy rescaling by lda.default
    lda.dim <- sum(ldaX$svd^2 > 1e-10)
    ldaX$svd <- ldaX$svd[1:lda.dim]
    ldaX$scaling <- ldaX$scaling[,1:lda.dim,drop=FALSE]

    if(is.null(n.da)){
        barplot(ldaX$svd^2, xlab="Linear Discriminants", ylab="F-statistic", main="Discriminant analysis eigenvalues", col=heat.colors(length(levels(grp))) )
        cat("Choose the number discriminant functions to retain (>=1): ")
        n.da <- as.integer(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    ##n.da <- min(n.da, length(levels(grp))-1, n.pca) # can't be more than K-1 disc. func., or more than n.pca
    n.da <- round(min(n.da, lda.dim)) # can't be more than K-1 disc. func., or more than n.pca
    predX <- predict(ldaX, dimen=n.da)


    ## BUILD RESULT
    res <- list()
    res$n.pca <- n.pca
    res$n.da <- n.da
    res$tab <- XU
    res$grp <- grp
    res$var <- XU.lambda
    res$eig <- ldaX$svd^2
    res$loadings <- ldaX$scaling[, 1:n.da, drop=FALSE]
    res$means <- ldaX$means
    res$ind.coord <-predX$x
    res$grp.coord <- apply(res$ind.coord, 2, tapply, grp, mean)
    res$prior <- ldaX$prior
    res$posterior <- predX$posterior
    res$assign <- predX$class
    res$call <- match.call()


    ## optional: store loadings of variables
    if(pca.info){
        res$pca.loadings <- as.matrix(U)
        res$pca.cent <- pcaX$cent
        res$pca.norm <- pcaX$norm
        res$pca.eig <- pcaX$eig
    }

    ## optional: get loadings of variables
    if(var.contrib || var.loadings){
        var.load <- as.matrix(U) %*% as.matrix(ldaX$scaling[,1:n.da,drop=FALSE])

        if(var.contrib){
        f1 <- function(x){
            temp <- sum(x*x)
            if(temp < 1e-12) return(rep(0, length(x)))
            return(x*x / temp)
        }
        res$var.contr <- apply(var.load, 2, f1)
    }
        if(var.loadings) res$var.load <- var.load
    }

    class(res) <- "dapc"
    return(res)
} # end dapc.data.frame





#############
## dapc.matrix
#############
dapc.matrix <- function(x, ...){
    return(dapc(as.data.frame(x), ...))
}




#############
## dapc.genind
#############
dapc.genind <- function(x, pop=NULL, n.pca=NULL, n.da=NULL,
                        scale=FALSE, truenames=TRUE, var.contrib=TRUE, var.loadings=FALSE,
                        pca.info=TRUE, pca.select=c("nbEig","percVar"), perc.pca=NULL, ...){

    ## FIRST CHECKS
    if(!is.genind(x)) stop("x must be a genind object.")

    if(is.null(pop)) {
        pop.fac <- pop(x)
    } else {
        pop.fac <- pop
    }

    if(is.null(pop.fac)) stop("x does not include pre-defined populations, and `pop' is not provided")


    ## SOME GENERAL VARIABLES
    N <- nInd(x)

    ## PERFORM PCA ##
    maxRank <- min(tab(x))

    X <- scaleGen(x, center = TRUE, scale = scale,
                  NA.method = "mean")

    ## CALL DATA.FRAME METHOD ##
    res <- dapc(X, grp=pop.fac, n.pca=n.pca, n.da=n.da,
                center=FALSE, scale=FALSE, var.contrib=var.contrib,
                var.loadings=var.loadings, pca.select=pca.select, perc.pca=perc.pca)

    res$call <- match.call()

    ## restore centring/scaling
    res$pca.cent <- attr(X, "scaled:center")

    if(scale) {
        res$pca.norm <- attr(X, "scaled:scale")
    }

    return(res)
} # end dapc.genind






######################
## Function dapc.dudi
######################
dapc.dudi <- function(x, grp, ...){
    return(dapc.data.frame(x$li, grp, dudi=x, ...))
}





#################
## dapc.genlight
#################
dapc.genlight <- function(x, pop=NULL, n.pca=NULL, n.da=NULL,
                          scale=FALSE,  var.contrib=TRUE, var.loadings=FALSE, pca.info=TRUE,
                          pca.select=c("nbEig","percVar"), perc.pca=NULL, glPca=NULL, ...){
    ## FIRST CHECKS ##
    if(!inherits(x, "genlight")) stop("x must be a genlight object.")

    pca.select <- match.arg(pca.select)

    if(is.null(pop)) {
        pop.fac <- pop(x)
    } else {
        pop.fac <- pop
    }

    if(is.null(pop.fac)) stop("x does not include pre-defined populations, and `pop' is not provided")



    ## PERFORM PCA ##
    REDUCEDIM <- is.null(glPca)

    if(REDUCEDIM){ # if no glPca provided
        maxRank <- min(c(nInd(x), nLoc(x)))
        pcaX <- glPca(x, center = TRUE, scale = scale, nf=maxRank, loadings=FALSE, returnDotProd = TRUE, ...)
    }

    if(!REDUCEDIM){ # else use the provided glPca object
        if(is.null(glPca$loadings) & var.contrib) {
            warning("Contribution of variables requested but glPca object provided without loadings.")
            var.contrib <- FALSE
        }
        pcaX <- glPca
    }

    if(is.null(n.pca)){
        cumVar <- 100 * cumsum(pcaX$eig)/sum(pcaX$eig)
    }


    ## select the number of retained PC for PCA
    if(!REDUCEDIM){
        myCol <- rep(c("black", "lightgrey"), c(ncol(pcaX$scores),length(pcaX$eig)))
    } else {
        myCol <- "black"
    }

    if(is.null(n.pca) & pca.select=="nbEig"){
        plot(cumVar, xlab="Number of retained PCs", ylab="Cumulative variance (%)", main="Variance explained by PCA", col=myCol)
        cat("Choose the number PCs to retain (>=1): ")
        n.pca <- as.integer(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    if(is.null(perc.pca) & pca.select=="percVar"){
        plot(cumVar, xlab="Number of retained PCs", ylab="Cumulative variance (%)", main="Variance explained by PCA", col=myCol)
        cat("Choose the percentage of variance to retain (0-100): ")
        nperc.pca <- as.numeric(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    ## get n.pca from the % of variance to conserve
    if(!is.null(perc.pca)){
        n.pca <- min(which(cumVar >= perc.pca))
        if(perc.pca > 99.999) n.pca <- length(pcaX$eig)
        if(n.pca<1) n.pca <- 1
    }

    if(!REDUCEDIM){
        if(n.pca > ncol(pcaX$scores)) {
            n.pca <- ncol(pcaX$scores)
        }
    }


    ## recompute PCA with loadings if needed
    if(REDUCEDIM){
        pcaX <- glPca(x, center = TRUE, scale = scale, nf=n.pca, loadings=var.contrib, matDotProd = pcaX$dotProd)
    }


    ## keep relevant PCs - stored in XU
    N <- nInd(x)
    X.rank <- sum(pcaX$eig > 1e-14)
    n.pca <- min(X.rank, n.pca)
    if(n.pca >= N) n.pca <- N-1

    U <- pcaX$loadings[, 1:n.pca, drop=FALSE] # principal axes
    XU <- pcaX$scores[, 1:n.pca, drop=FALSE] # principal components
    XU.lambda <- sum(pcaX$eig[1:n.pca])/sum(pcaX$eig) # sum of retained eigenvalues
    names(U) <- paste("PCA-pa", 1:ncol(U), sep=".")
    names(XU) <- paste("PCA-pc", 1:ncol(XU), sep=".")


    ## PERFORM DA ##
    ldaX <- lda(XU, pop.fac, tol=1e-30) # tol=1e-30 is a kludge, but a safe (?) one to avoid fancy rescaling by lda.default
    lda.dim <- sum(ldaX$svd^2 > 1e-10)
    ldaX$svd <- ldaX$svd[1:lda.dim]
    ldaX$scaling <- ldaX$scaling[,1:lda.dim,drop=FALSE]

    if(is.null(n.da)){
        barplot(ldaX$svd^2, xlab="Linear Discriminants", ylab="F-statistic", main="Discriminant analysis eigenvalues", col=heat.colors(length(levels(pop.fac))) )
        cat("Choose the number discriminant functions to retain (>=1): ")
        n.da <- as.integer(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    n.da <- min(n.da, length(levels(pop.fac))-1, n.pca, sum(ldaX$svd>1e-10)) # can't be more than K-1 disc. func., or more than n.pca
    n.da <- round(n.da)
    predX <- predict(ldaX, dimen=n.da)


    ## BUILD RESULT
    res <- list()
    res$n.pca <- n.pca
    res$n.da <- n.da
    res$tab <- XU
    res$grp <- pop.fac
    res$var <- XU.lambda
    res$eig <- ldaX$svd^2
    res$loadings <- ldaX$scaling[, 1:n.da, drop=FALSE]
    res$means <- ldaX$means
    res$ind.coord <-predX$x
    res$grp.coord <- apply(res$ind.coord, 2, tapply, pop.fac, mean)
    res$prior <- ldaX$prior
    res$posterior <- predX$posterior
    res$assign <- predX$class
    res$call <- match.call()


    ## optional: store loadings of variables
    if(pca.info){
        res$pca.loadings <- as.matrix(U)
        res$pca.cent <- glMean(x,alleleAsUnit=FALSE)
        if(scale) {
            res$pca.norm <- sqrt(glVar(x,alleleAsUnit=FALSE))
        } else {
            res$pca.norm <- rep(1, nLoc(x))
        }
        res$pca.eig <- pcaX$eig
    }

    ## optional: get loadings of variables
    if(var.contrib || var.loadings){
        var.load <- as.matrix(U) %*% as.matrix(ldaX$scaling[,1:n.da,drop=FALSE])

        if(var.contrib){
        f1 <- function(x){
            temp <- sum(x*x)
            if(temp < 1e-12) return(rep(0, length(x)))
            return(x*x / temp)
        }
        res$var.contr <- apply(var.load, 2, f1)
    }
        if(var.loadings) res$var.load <- var.load
    }

    class(res) <- "dapc"
    return(res)
} # end dapc.genlight






######################
# Function print.dapc
######################
print.dapc <- function(x, ...){
    cat("\t#################################################\n")
    cat("\t# Discriminant Analysis of Principal Components #\n")
    cat("\t#################################################\n")
    cat("class: ")
    cat(class(x))
    cat("\n$call: ")
    print(x$call)
    cat("\n$n.pca:", x$n.pca, "first PCs of PCA used")
    cat("\n$n.da:", x$n.da, "discriminant functions saved")
    cat("\n$var (proportion of conserved variance):", round(x$var,3))
    cat("\n\n$eig (eigenvalues): ")
    l0 <- sum(x$eig >= 0)
    cat(signif(x$eig, 4)[1:(min(5, l0))])
    if (l0 > 5)
        cat(" ...\n\n")

    ## vectors
    TABDIM <- 4
    if(!is.null(x$pca.loadings)){
        TABDIM <- TABDIM + 3
    }
    sumry <- array("", c(TABDIM, 3), list(1:TABDIM, c("vector", "length", "content")))
    sumry[1, ] <- c('$eig', length(x$eig),  'eigenvalues')
    sumry[2, ] <- c('$grp', length(x$grp), 'prior group assignment')
    sumry[3, ] <- c('$prior', length(x$prior), 'prior group probabilities')
    sumry[4, ] <- c('$assign', length(x$assign), 'posterior group assignment')
    if(!is.null(x$pca.loadings)){
        sumry[5, ] <- c('$pca.cent', length(x$pca.cent), 'centring vector of PCA')
        sumry[6, ] <- c('$pca.norm', length(x$pca.norm), 'scaling vector of PCA')
        sumry[7, ] <- c('$pca.eig', length(x$pca.eig), 'eigenvalues of PCA')
    }
    class(sumry) <- "table"
    print(sumry)

    ## data.frames
    cat("\n")
    TABDIM <- 6
    if(!is.null(x$pca.loadings)){
        TABDIM <- TABDIM + 1
    }
    if(!is.null(x$var.contr)){
        TABDIM <- TABDIM + 1
    }
    if(!is.null(x$var.load)){
        TABDIM <- TABDIM + 1
    }

    sumry <- array("", c(TABDIM, 4), list(1:TABDIM, c("data.frame", "nrow", "ncol", "content")))

    sumry[1, ] <- c("$tab", nrow(x$tab), ncol(x$tab), "retained PCs of PCA")
    sumry[2, ] <- c("$means", nrow(x$means), ncol(x$means), "group means")
    sumry[3, ] <- c("$loadings", nrow(x$loadings), ncol(x$loadings), "loadings of variables")
    sumry[4, ] <- c("$ind.coord", nrow(x$ind.coord), ncol(x$ind.coord), "coordinates of individuals (principal components)")
    sumry[5, ] <- c("$grp.coord", nrow(x$grp.coord), ncol(x$grp.coord), "coordinates of groups")
    sumry[6, ] <- c("$posterior", nrow(x$posterior), ncol(x$posterior), "posterior membership probabilities")
    count <- 6
    if(!is.null(x$pca.loadings)){
        count <- count+1
        sumry[count, ] <- c("$pca.loadings", nrow(x$pca.loadings), ncol(x$pca.loadings), "PCA loadings of original variables")
    }
    if(!is.null(x$var.contr)){
        count <- count+1
        sumry[count, ] <- c("$var.contr", nrow(x$var.contr), ncol(x$var.contr), "contribution of original variables")
    }
    if(!is.null(x$var.load)){
        count <- count+1
        sumry[count, ] <- c("$var.load", nrow(x$var.load), ncol(x$var.load), "loadings of original variables")
    }
    class(sumry) <- "table"
    print(sumry)

    ## cat("\nother elements: ")
    ## if (length(names(x)) > 15)
    ##     cat(names(x)[15:(length(names(x)))], "\n")
    ## else cat("NULL\n")
    cat("\n")
} # end print.dapc






##############
## summary.dapc
##############
summary.dapc <- function(object, ...){
    x <- object
    res <- list()

    ## number of dimensions
    res$n.dim <- ncol(x$loadings)
    res$n.pop <- length(levels(x$grp))

    ## assignment success
    temp <- as.character(x$grp)==as.character(x$assign)
    res$assign.prop <- mean(temp)
    res$assign.per.pop <- tapply(temp, x$grp, mean)

    ## group sizes
    res$prior.grp.size <- table(x$grp)
    res$post.grp.size <- table(x$assign)

    return(res)
} # end summary.dapc






##############
## scatter.dapc
##############
#' @importFrom vegan orditorp
#' 
scatter.dapc <- function(x, xax = 1, yax = 2, grp = x$grp,
                         col = seasun(length(levels(grp))),
                         pch = 20, bg = "white", solid = .7,
                         scree.da = TRUE, scree.pca = FALSE,
                         posi.da = "bottomright",
                         posi.pca = "bottomleft",
                         bg.inset = "white",
                         ratio.da = .25, ratio.pca = .25,
                         inset.da = 0.02, inset.pca = 0.02, inset.solid = .5,
                         onedim.filled = TRUE, mstree = FALSE,
                         lwd = 1, lty = 1, segcol = "black",
                         legend = FALSE, posi.leg = "topright",
                         cleg = 1, txt.leg = levels(grp),
                         cstar = 1, cellipse = 1.5, axesell = FALSE,
                         label = levels(grp), clabel = 1, xlim = NULL, ylim = NULL,
                         grid = FALSE, addaxes = TRUE, origin = c(0,0),
                         include.origin = TRUE, sub = "", csub = 1, possub = "bottomleft",
                         cgrid = 1, pixmap = NULL, contour = NULL,
                         area = NULL, label.inds = NULL, ...){
    ONEDIM <- xax==yax | ncol(x$ind.coord)==1


    ## recycle color and pch
    col <- rep(col, length(levels(grp)))
    pch <- rep(pch, length(levels(grp)))
    col <- transp(col, solid)
    bg.inset <- transp(bg.inset, inset.solid)

    ## handle grp
    if(is.null(grp)){
        grp <- x$grp
    }

    ## handle xax or yax NULL
    if(is.null(xax)||is.null(yax)){
        xax <- 1L
        yax <- ifelse(ncol(x$ind.coord)==1L, 1L, 2L)
        ONEDIM <- TRUE
    }

    ## handle 1 dimensional plot
    if(!ONEDIM){
        ## set par
        opar <- par(mar = par("mar"))
        par(mar = c(0.1, 0.1, 0.1, 0.1), bg=bg)
        on.exit(par(opar))
        axes <- c(xax,yax)

        ## basic empty plot
        s.class(x$ind.coord[,axes], fac = grp, col = col, cpoint = 0,
                cstar = cstar, cellipse = cellipse, axesell = axesell,
                label = label, clabel = clabel, xlim = xlim, ylim = ylim,
                grid = grid, addaxes = addaxes, origin = origin,
                include.origin = include.origin,
                sub = sub, csub = csub, possub = possub,
                cgrid = cgrid, pixmap = pixmap,
                contour = contour, area = area)

        ## add points
        colfac <- pchfac <- grp
        levels(colfac) <- col
        levels(pchfac) <- pch
        colfac <- as.character(colfac)
        pchfac <- as.character(pchfac)
        if(is.numeric(col)) colfac <- as.numeric(colfac)
        if(is.numeric(pch)) pchfac <- as.numeric(pchfac)

        points(x$ind.coord[,xax],
               x$ind.coord[,yax],
               col = colfac, pch = pchfac, ...)
        s.class(x$ind.coord[,axes], fac = grp, col = col, cpoint = 0,
                add.plot=TRUE, cstar = cstar, cellipse = cellipse,
                axesell = axesell, label = label,clabel = clabel,
                xlim = xlim, ylim = ylim, grid = grid,
                addaxes = addaxes, origin = origin,
                include.origin = include.origin,
                sub = sub, csub = csub, possub = possub,
                cgrid = cgrid, pixmap = pixmap,
                contour = contour, area = area)

        ## Add labels of individuals if specified. Play around with "air" to get
        ## a satisfactory result.

        if (!is.null(label.inds) & is.list(label.inds)) {
            appendList <- function (x, val) {
                                        # recursevly "bind" a list into a longer list,
                                        # from http://stackoverflow.com/a/9519964/322912
                stopifnot(is.list(x), is.list(val))
                xnames <- names(x)
                for (v in names(val)) {
                    x[[v]] <- if (v %in% xnames && is.list(x[[v]]) && is.list(val[[v]]))
                        appendList(x[[v]], val[[v]])
                    else c(x[[v]], val[[v]])
                }
                x
            }

            do.call("orditorp",
                    c(appendList(list(x = x$ind.coord[, c(xax, yax)],
                                      display = "species"),
                                 label.inds)))
        }

        ## add minimum spanning tree if needed
        if(mstree){
            meanposi <- apply(x$tab,2, tapply, grp, mean)
            D <- dist(meanposi)^2
            tre <- ade4::mstree(D)
            x0 <- x$grp.coord[tre[,1], axes[1]]
            y0 <- x$grp.coord[tre[,1], axes[2]]
            x1 <- x$grp.coord[tre[,2], axes[1]]
            y1 <- x$grp.coord[tre[,2], axes[2]]
            segments(x0, y0, x1, y1, lwd = lwd, lty = lty, col = segcol)
        }

    } else {
        ## set screeplot of DA to FALSE (just 1 bar)
        scree.da <- FALSE

        ## get plotted axis
        if(ncol(x$ind.coord)==1) {
            pcLab <- 1
        } else{
            pcLab <- xax
        }
        ## get densities
        ldens <- tapply(x$ind.coord[,pcLab], grp, density)
        allx <- unlist(lapply(ldens, function(e) e$x))
        ally <- unlist(lapply(ldens, function(e) e$y))
        par(bg=bg)
        plot(allx, ally, type = "n",
             xlab = paste("Discriminant function", pcLab),
             ylab = "Density")
        for(i in 1:length(ldens)){
            if(!onedim.filled) {
                lines(ldens[[i]]$x, ldens[[i]]$y, col = col[i], lwd = 2) # add lines
            } else {
                polygon(c(ldens[[i]]$x, rev(ldens[[i]]$x)),
                        c(ldens[[i]]$y, rep(0,length(ldens[[i]]$x))),
                        col = col[i], lwd = 2, border = col[i]) # add lines
            }
            points(x = x$ind.coord[grp==levels(grp)[i], pcLab],
                   y = rep(0, sum(grp==levels(grp)[i])),
                   pch = "|", col = col[i]) # add points for indiv
        }
    }

    ## ADD INSETS ##
    ## group legend
    if(legend){
        ## add a legend
        temp <- list(...)$cex
        if(is.null(temp)) temp <- 1
        if(ONEDIM | temp<0.5 | all(pch == "")) {
            legend(posi.leg, fill = col, legend = txt.leg,
                   cex = cleg, bg = bg.inset)
        } else {
            legend(posi.leg, col = col, legend = txt.leg, cex = cleg,
                   bg = bg.inset, pch = pch, pt.cex = temp)
        }
    }

    ## eigenvalues discriminant analysis
    if(scree.da && ratio.da>.01) {
        inset <- function(){
            myCol <- rep("white", length(x$eig))
            myCol[1:x$n.da] <- "grey"
            myCol[c(xax, yax)] <- "black"
            myCol <- transp(myCol, inset.solid)
            barplot(x$eig, col=myCol, xaxt="n", yaxt="n", ylim=c(0, x$eig[1]*1.1))
            mtext(side=3, "DA eigenvalues", line=-1.2, adj=.8)
            box()
        }

        add.scatter(inset(), posi = posi.da, ratio = ratio.da,
                    bg.col = bg.inset, inset = inset.da)
    }

    ## eigenvalues PCA
    if(scree.pca && !is.null(x$pca.eig) && ratio.pca>.01) {
        inset <- function(){
            temp <- 100* cumsum(x$pca.eig) / sum(x$pca.eig)
            myCol <- rep(c("black","grey"), c(x$n.pca, length(x$pca.eig)))
            myCol <- transp(myCol, inset.solid)
            plot(temp, col=myCol, ylim=c(0,115),
                 type="h", xaxt="n", yaxt="n", xlab="", ylab="", lwd=2)
            mtext(side=3, "PCA eigenvalues", line=-1.2, adj=.1)
        }
        add.scatter(inset(), posi = posi.pca, ratio = ratio.pca,
                    bg.col = bg.inset, inset = inset.pca)
    }


    return(invisible(match.call()))
} # end scatter.dapc






############
## assignplot
############
assignplot <- function(x, only.grp=NULL, subset=NULL, new.pred=NULL, cex.lab=.75, pch=3){
    if(!inherits(x, "dapc")) stop("x is not a dapc object")

    ## handle data from predict.dapc ##
    if(!is.null(new.pred)){
        n.new <- length(new.pred$assign)
        x$grp <- c(as.character(x$grp), rep("unknown", n.new))
        x$assign <- c(as.character(x$assign), as.character(new.pred$assign))
        x$posterior <- rbind(x$posterior, new.pred$posterior)
    }


    ## treat other arguments ##
    if(!is.null(only.grp)){
        only.grp <- as.character(only.grp)
        ori.grp <- as.character(x$grp)
        x$grp <- x$grp[only.grp==ori.grp]
        x$assign <- x$assign[only.grp==ori.grp]
        x$posterior <- x$posterior[only.grp==ori.grp, , drop=FALSE]
    } else if(!is.null(subset)){
        x$grp <- x$grp[subset]
        x$assign <- x$assign[subset]
        x$posterior <- x$posterior[subset, , drop=FALSE]
    }


    ##table.paint(x$posterior, col.lab=ori.grp, ...)
    ## symbols(x$posterior)


    ## FIND PLOT PARAMETERS
    n.grp <- ncol(x$posterior)
    n.ind <- nrow(x$posterior)
    Z <- t(x$posterior)
    Z <- Z[,ncol(Z):1,drop=FALSE ]

    image(x=1:n.grp, y=seq(.5, by=1, le=n.ind), Z, col=rev(heat.colors(100)), yaxt="n", ylab="", xaxt="n", xlab="Clusters")
    axis(side=1, at=1:n.grp,tick=FALSE, labels=colnames(x$posterior))
    axis(side=2, at=seq(.5, by=1, le=n.ind), labels=rev(rownames(x$posterior)), las=1, cex.axis=cex.lab)
    abline(h=1:n.ind, col="lightgrey")
    abline(v=seq(0.5, by=1, le=n.grp))
    box()

    newGrp <- colnames(x$posterior)
    x.real.coord <- rev(match(x$grp, newGrp))
    y.real.coord <- seq(.5, by=1, le=n.ind)

    points(x.real.coord, y.real.coord, col="deepskyblue2", pch=pch)

    return(invisible(match.call()))
} # end assignplot






###############
## a.score
###############
a.score <- function(x, n.sim=10, ...){
    if(!inherits(x,"dapc")) stop("x is not a dapc object")

    ## perform DAPC based on permuted groups
    lsim <- lapply(1:n.sim, function(i) summary(dapc(x$tab, sample(x$grp), n.pca=x$n.pca, n.da=x$n.da))$assign.per.pop)
    sumry <- summary(x)

    ## get the a-scores
    f1 <- function(Pt, Pf){
        tol <- 1e-7
        ##res <- (Pt-Pf) / (1-Pf)
        ##res[Pf > (1-tol)] <- 0
        res <- Pt-Pf
        return(res)
    }

    lscores <- lapply(lsim, function(e) f1(sumry$assign.per.pop, e))

    ## make a table of a-scores
    tab <- data.frame(lscores)
    colnames(tab) <- paste("sim", 1:n.sim, sep=".")
    rownames(tab) <- names(sumry$assign.per.pop)
    tab <- t(as.matrix(tab))

    ## make result
    res <- list()
    res$tab <- tab
    res$pop.score <- apply(tab, 2, mean)
    res$mean <- mean(tab)

    return(res)

} # end a.score







##############
## optim.a.score
##############
optim.a.score <- function(x, n.pca=1:ncol(x$tab), smart=TRUE, n=10, plot=TRUE,
                         n.sim=10, n.da=length(levels(x$grp)), ...){
    ## A FEW CHECKS ##
    if(!inherits(x,"dapc")) stop("x is not a dapc object")
    if(max(n.pca)>ncol(x$tab)) {
        n.pca <- min(n.pca):ncol(x$tab)
    }
    if(n.da>length(levels(x$grp))){
        n.da <- min(n.da):length(levels(x$grp))
    }
    pred <- NULL
    if(length(n.pca)==1){
        n.pca <- 1:n.pca
    }
    if(length(n.da)==1){
        n.da <- 1:n.da
    }


    ## AUXILIARY FUNCTION ##
    f1 <- function(ndim){
        temp <- dapc(x$tab[,1:ndim,drop=FALSE], x$grp, n.pca=ndim, n.da=x$n.da)
        a.score(temp, n.sim=n.sim)$pop.score
    }


    ## SMART: COMPUTE A FEW VALUES, PREDICT THE BEST PICK ##
    if(smart){
        ## if(!require(stats)) stop("the package stats is required for 'smart' option")
        o.min <- min(n.pca)
        o.max <- max(n.pca)
        n.pca <- pretty(n.pca, n) # get evenly spaced nb of retained PCs
        n.pca <- n.pca[n.pca>0 & n.pca<=ncol(x$tab)]
        if(!any(o.min==n.pca)) n.pca <- c(o.min, n.pca) # make sure range is OK
        if(!any(o.max==n.pca)) n.pca <- c(o.max, n.pca) # make sure range is OK
        lres <- lapply(n.pca, f1)
        names(lres) <- n.pca
        means <- sapply(lres, mean)
        sp1 <- smooth.spline(n.pca, means) # spline smoothing
        pred <- predict(sp1, x=1:max(n.pca))
        best <- pred$x[which.max(pred$y)]
    } else { ## DO NOT TRY TO BE SMART ##
        lres <- lapply(n.pca, f1)
        names(lres) <- n.pca
        best <- which.max(sapply(lres, mean))
        means <- sapply(lres, mean)
    }


    ## MAKE FINAL OUTPUT ##
    res <- list()
    res$pop.score <- lres
    res$mean <- means
    if(!is.null(pred)) res$pred <- pred
    res$best <- best

    ## PLOTTING (OPTIONAL) ##
    if(plot){
        if(smart){
            boxplot(lres, at=n.pca, col="gold", xlab="Number of retained PCs", ylab="a-score", xlim=range(n.pca)+c(-1,1), ylim=c(-.1,1.1))
            lines(pred, lwd=3)
            points(pred$x[best], pred$y[best], col="red", lwd=3)
            title("a-score optimisation - spline interpolation")
            mtext(paste("Optimal number of PCs:", res$best), side=3)
        } else {
            myCol <- rep("gold", length(lres))
            myCol[best] <- "red"
            boxplot(lres, at=n.pca, col=myCol, xlab="Number of retained PCs", ylab="a-score", xlim=range(n.pca)+c(-1,1), ylim=c(-.1,1.1))
            lines(n.pca, sapply(lres, mean), lwd=3, type="b")
            myCol <- rep("black", length(lres))
            myCol[best] <- "red"
            points(n.pca, res$mean, lwd=3, col=myCol)
            title("a-score optimisation - basic search")
            mtext(paste("Optimal number of PCs:", res$best), side=3)
        }
    }

    return(res)
} # end optim.a.score






#############
## as.lda.dapc
#############
as.lda <- function(...){
    UseMethod("as.lda")
}

as.lda.dapc <- function(x, ...){
    if(!inherits(x,"dapc")) stop("x is not a dapc object")
    res <- list()

    res$N <- nrow(res$ind.coord)
    res$call <- match.call()
    res$counts <- as.integer(table(x$grp))
    res$lev <- names(res$counts) <- levels(x$grp)
    res$means <- x$means
    res$prior <- x$prior
    res$scaling <- x$loadings
    res$svd <- sqrt(x$eig)

    class(res) <- "lda"

    return(res)
} # end as.lda.dapc






##############
## predict.dapc
##############
predict.dapc <- function(object, newdata, prior = object$prior, dimen,
                         method = c("plug-in", "predictive", "debiased"), ...){

    if(!inherits(object,"dapc")) stop("x is not a dapc object")
    method <- match.arg(method)

    x <- as.lda(object)


    ## HANDLE NEW DATA ##
    if(!missing(newdata)){
        ## make a few checks
        if(is.null(object$pca.loadings)) stop("DAPC object does not contain loadings of original variables. \nPlease re-run DAPC using 'pca.loadings=TRUE'.")
        ## We need to convert the data as they were converted during the analysis. Behaviour is:
        ## - genind: allele frequencies, missing data = mean
        ## - genlight: allele frequencies, missing data = mean

        if (is.genind(newdata)) { # genind object
            newdata <- tab(newdata, freq = TRUE, NA.method = "mean")
        } else if (inherits(newdata, "genlight")) { # genlight object
               newdata <- as.matrix(newdata) / ploidy(newdata)
           } else { # any other type of object
                newdata <- as.matrix(newdata)
            }
        if(ncol(newdata) != nrow(object$pca.loadings)) stop("Number of variables in newdata does not match original data.")

        ## centre/scale data
        for(i in 1:nrow(newdata)){ # this is faster for large, flat matrices)
            newdata[i,] <- (newdata[i,] - object$pca.cent) / object$pca.norm
        }
        newdata[is.na(newdata)] <- 0

        ## project as supplementary individuals
        XU <- newdata %*% as.matrix(object$pca.loadings)
    } else {
        XU <- object$tab
    }

    ## FORCE IDENTICAL VARIABLE NAMES ##
    colnames(XU) <- colnames(object$tab)


    ## HANDLE DIMEN ##
    if(!missing(dimen)){
        if(dimen > object$n.da) stop(paste("Too many dimensions requested. \nOnly", object$n.da, "discriminant functions were saved in DAPC."))
    } else {
        dimen <- object$n.da
    }

    ## CALL PREDICT.LDA ##
    temp <- predict(x, XU, prior, dimen, method, ...)


    ## FORMAT OUTPUT ##
    res <- list()
    res$assign <- temp$class
    res$posterior <- temp$posterior
    res$ind.scores <- temp$x

    return(res)

} # end predict.dapc







## #############
## ## discriVal
## #############

## discriVal <- function (x, ...) UseMethod("discriVal")

## discriVal.data.frame <- function(x, grp, n.pca.max, n.da=NULL, center=TRUE, scale=FALSE, n.pca=NULL, ...){

##     ## CHECKS ##
##     grp <- factor(grp)
##     n.pca <- n.pca[n.pca>0]
##     if(is.null(n.da)) {
##         n.da <- length(levels(grp))-1
##     }

##     ## GET FULL PCA ##
##     if(missing(n.pca.max)) n.pca.max <- min(dim(x))
##     pcaX <- dudi.pca(x, nf=n.pca.max, scannf=FALSE, center=center, scale=scale)
##     n.pca.max <- min(n.pca.max,pcaX$rank)

##     ## DETERMINE N.PCA IF NEEDED ##
##     if(is.null(n.pca)){
##         n.pca <- round(pretty(1:n.pca.max,10))
##     }
##     n.pca <- n.pca[n.pca>0 & n.pca<n.pca.max]

##     ## FUNCTION GETTING THE TOTAL DISCRIMINATION (SUM OF EIGENVALUES) FOR ONE GIVEN NB OF PCA PCs ##
##     ## n.pca is a number of retained PCA PCs
##     get.totdiscr <- function(n.pca){
##             temp.dapc <- suppressWarnings(dapc(x, grp, n.pca=n.pca, n.da=n.da, dudi=pcaX))
##             return(sum(temp.dapc$eig))
##     }


##     ## GET %SUCCESSFUL OF ACCURATE PREDICTION FOR ALL VALUES ##
##     res.all <- sapply(n.pca, get.totdiscr)
##     res <- data.frame(n.pca=n.pca, success=res.all)
##     return(res)
## } # end discriVal.data.frame


## discriVal.matrix <- discriVal.data.frame




## There's a bunch of problems down there, commenting it for nowé
## xval.dapc <- function(object, n.pca, n.da, training.set = 90, ...){
##   training.set = training.set/100
##   kept.id <- unlist(tapply(1:nInd(object), pop(object), function(e) {pop.size = length(e); pop.size.train = round(pop.size * training.set); sample(e, pop.size.train, replace=FALSE)})) # this can't work: nInd/pop not defined for DAPC objects
##   training <- object[kept.id]
##   validating <- object[-kept.id]
##   post = vector(mode = 'list', length = n.pca)
##   asgn = vector(mode = 'list', length = n.pca)
##   ind = vector(mode = 'list', length = n.pca)
##   mtch = vector(mode = 'list', length = n.pca)
##   for(i in 1:n.pca){
##     dapc.base = dapc(training, n.pca = i, n.da = 15) # Why 15??
##     dapc.p = predict.dapc(dapc.base, newdata = validating)
##     match.prp = mean(as.character(dapc.p$assign)==as.character(pop(validating)))
##     post[[i]] = dapc.p$posterior
##     asgn[[i]] = dapc.p$assign
##     ind[[i]] = dapc.p$ind.score
##     mtch[[i]] = match.prp
##   }
##   res = list(assign = asgn, posterior = post, ind.score = ind, match.prp = mtch)
##   return(res)
## } # end of xval.dapc

## xval.genind  <- function(object, n.pca, n.da, training.set = 90, ...){
##   res = xval.dapc(object = object, n.pca = n.pca, n.da = n.da, training.set = training.set)
##   return(res)
## }
## ###############
## ## randtest.dapc
## ###############
## ##randtest.dapc <- function(x, nperm = 999, ...){

## ##} # end randtest.dapc




######## TESTS IN R #######

## TEST PREDICT.DAPC ##
## data(sim2pop)
## temp <- seppop(sim2pop)
## temp <- lapply(temp, function(e) hybridize(e,e,n=30)) # force equal pop sizes
## hyb <- hybridize(temp[[1]], temp[[2]], n=30)
## newdat <- repool(temp[[1]], temp[[2]], hyb)
## pop(newdat) <- rep(c("pop A", "popB", "hyb AB"), c(30,30,30))


## ##dapc1 <- dapc(newdat[1:61],n.pca=10,n.da=1)
## dapc1 <- dapc(newdat[1:60],n.pca=2,n.da=1)
## scatter(dapc1)
## hyb.pred <- predict(dapc1, newdat[61:90])

## scatter(dapc1)
## points(hyb.pred$ind.scores, rep(.1, 30))

## assignplot(dapc1, new.pred=hyb.pred)
## title("30 indiv popA, 30 indiv pop B, 30 hybrids")

#' Compute minor allele frequency
#'
#' This function computes the minor allele frequency for each locus in a \linkS4class{genind} object. To test if loci are polymorphic, see the function  \code{\link{isPoly}}.
#'
#' @param x  a \linkS4class{genind} object
#'
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#'
#' @export
#'
#' @examples
#' \dontrun{
#'
#' ## LOAD DATA
#' data(nancycats)
#'
#' ## COMPUTE ALLELE FREQUENCIES
#' x <- nancycats
#' apply(tab(x, freq=TRUE),2,mean, na.rm=TRUE)
#'
#' ## GET MINOR ALLELE FREQUENCY
#' m.freq <- minorAllele(x)
#' m.freq
#' }
#'
#' @seealso \code{\link{isPoly}}
#'
minorAllele <- function(x){
    ## CHECK INPUT
    if(!is.genind(x)) stop("x is not a valid genind object")

    ## AUXIL FUNCTION
    f1 <- function(vec){
        vec <- sort(vec, decreasing=TRUE)
        res <- vec[min(2,length(vec))]
        return(res)
    }

    ## GET ALLELE FREQUENCIES
    freq <- apply(tab(x, freq=TRUE),2,mean, na.rm=TRUE)

    ## GET MINOR ALLELE
    out <- tapply(freq, locFac(x), f1)

    ## RETURN OUTPUT
    return(out)
}

.onAttach <- function(libname, pkgname){
    # adegenet specific options -----------------------------------------------
    op <- options()
    op.adegenet <- list(
      adegenet.testcon = stdin() # for readLines, read from stdin. This allows it to be changed for tests.
    )
    toset <- !(names(op.adegenet) %in% names(op))
    if(any(toset)) options(op.adegenet[toset])

    # startup message ---------------------------------------------------------
    pkg.version <- packageDescription("adegenet", fields = "Version")
    startup.txt <- paste("\n   /// adegenet ", pkg.version, " is loaded ////////////",
                         "\n\n   > overview: '?adegenet'",
                         "\n   > tutorials/doc/questions: 'adegenetWeb()' ",
                         "\n   > bug reports/feature requests: adegenetIssues()\n\n", sep="")
    packageStartupMessage(startup.txt)
}

## ############
## ## haploPop
## ############
## ##
## ## Simulate only SNPs, allow reverse mutations.
## ##
## ## - haplo.length: length of simulated haplotypes
## ## - mu: substitution rate / nucleotide / year
## ## - n.steps: number of generations to simulate
## ##
## haploPop <- function(n.steps=20, ini.obj=NULL, ini.haplo=NULL, haplo.length=1e6, mu=1e-5, n.snp.ini=1,
##                      birth.func=function(){ sample(0:3, 1, prob=c(.05, .45, .35, .15))},
##                      max.pop.size=function(){1e4}, max.nb.pop=30, ini.pop.size=10, regen=FALSE,
##                      p.new.pop=function(){1e-4}, death.func=function(age){age>1},
##                      quiet=FALSE, allow.reverse=TRUE) {


##     ## SOME CHECKS
##     ## if(is.numeric(ini.pop.size)){
##     ##     ini.pop.size.val <- ini.pop.size
##     ##     ini.pop.size <- function(){ini.pop.size.val}
##     ## }

##     if(is.numeric(max.pop.size)){
##         max.pop.size.val <- max.pop.size
##         max.pop.size <- function(){max.pop.size.val}
##     }

##      if(is.numeric(p.new.pop)){
##         p.new.pop.val <- p.new.pop
##         p.new.pop <- function(){p.new.pop.val}
##     }

##     if(is.numeric(birth.func)){
##         birth.func.val <- birth.func[1]
##         birth.func <- function(){birth.func.val}
##     }

##     if(is.numeric(death.func)){
##         death.func.val <- death.func[1]
##         death.func <- function(age){age>death.func.val}
##     }


##     ## GLOBAL VARIABLES ##
##     SNP.POOL <- 1:haplo.length
##     vecS <- 1 # will be redefined later, but needed for evolveOnePop definition

##     ## AUXILIARY FUNCTIONS ##
##     if(allow.reverse){
##         createMutations <- function(N){ # L:genome length; N: pop size
##             nb.mutations <- sum(rbinom(N, size=haplo.length, prob=mu))
##             return( sample(SNP.POOL, size=nb.mutations, replace=TRUE) )
##         }
##     } else {
##         createMutations <- function(N){ # L:genome length; N: pop size
##             nb.mutations <- sum(rbinom(N, size=haplo.length, prob=mu))
##             res <- sample(SNP.POOL, size=nb.mutations, replace=TRUE)
##             SNP.POOL <<- setdiff(SNP.POOL, res)# update pool of SNPs
##             return(res)
##         }
##     }

##     ## clean reverse mutations
##     cleanRes <- function(vec){
##         temp <- table(vec)
##         return( as.integer(names(temp)[temp %% 2 != 0]) )
##     }


##     ## assign mutation to haplotypes
##     assignMutations <- function(myPop, mutations){ # mypop: list of `haplotypes'; mutations: vector of SNPs
##         if(length(mutations)==0 | length(myPop)==0) return(myPop)
##         id <- sample(1:length(myPop), size=length(mutations), replace=TRUE)
##         mutations <- split(mutations, id)

##         ## function to merge new mutations - handle reverse case
##         f1 <- function(a,b){
##             revMut <- intersect(a,b)
##             if(length(revMut)==0) return(c(a,b))
##             return(setdiff(c(a ,b), revMut))
##         }

##         ##myPop[as.integer(names(mutations))] <- mapply(c, myPop[as.integer(names(mutations))], mutations, SIMPLIFY=FALSE)
##          myPop[as.integer(names(mutations))] <- mapply(f1, myPop[as.integer(names(mutations))], mutations, SIMPLIFY=FALSE)

##         return(myPop)
##     } # end assignMutations


##     if(!regen){
##         ## VERSION FOR NO REGENERATION OF SUSCEPTIBLES
##         evolveOnePop <- function(myPop, myS, myAge){ # myPop: pop to evolve; myS: nb of susceptible in the pop; myAge: vector of ages
##             ## strains get older
##             myAge <- myAge + 1
##             ## toKill <- death.func(myAge)
##             ## myPop[toKill] <- NULL
##             ## myAge <- myAge[!toKill]

##             ## generate new strains for new generation
##             sampSize <- round(min( length(myPop)*birth.func(), myS)) # number of strains for next step
##             if(sampSize<1){ # if no new strains
##                 ## old strains die
##                 toKill <- death.func(myAge)
##                 myPop[toKill] <- NULL
##                 myAge <- myAge[!toKill]
##                 return(list(pop=myPop, S=myS, age=myAge))
##             } # if there are new strains, do...
##             newGen <- myPop[sample(1:length(myPop), sampSize, replace=TRUE)] # sample strains for new generations
##             newGen <- assignMutations(newGen, createMutations(sampSize)) # mutate strains
##             newAge <- rep(0, sampSize) # new ages for newborns

##             ## old strains die
##             toKill <- death.func(myAge)
##             myPop[toKill] <- NULL
##             myAge <- myAge[!toKill]

##             ## merge old and new generation
##             myPop <- c(myPop,newGen)
##             myAge <- c(myAge, newAge)

##             ## possibly create one or more new pop
##             if((length(listPop) < max.nb.pop) & (p.new.pop()>0)) { # total number of pop. limitation
##                 nbNewPop <- rbinom(1, length(myPop), prob=p.new.pop())
##             } else {
##                 nbNewPop <- 0
##             }
##             if(nbNewPop>0){
##                 ## newPop <- sample(listPop, size=nbNewPop, replace=TRUE) # wrong
##                 newPop <- lapply(sample(myPop, size=nbNewPop, replace=TRUE), as.list)
##                 listPop <<- c(listPop, newPop)
##                 vecS <<- c(vecS, replicate(nbNewPop, max.pop.size()) )
##                 listAges <<- c(listAges, replicate(nbNewPop, 0, simplify=FALSE) )
##             } # end new pop
##             return(list(pop=myPop, S=myS-sampSize, age=myAge))
##         } # end no regen version
##     } else { ## REGEN VERSION
##         evolveOnePop <- function(myPop, myS, myAge){ # myPop: pop to evolve; myS: nb of susceptible in the pop; myAge: vector of ages
##             ## strains get older
##             myAge <- myAge + 1
##             ## toKill <- death.func(myAge)
##             ## myPop[toKill] <- NULL
##             ## myAge <- myAge[!toKill]
##             myS <- max.pop.size() ## DIFFERENCE between the two versions of the function

##             ## generate new strains for new generation
##             sampSize <- round(min( length(myPop)*birth.func(), myS)) # number of strains for next step
##             if(sampSize<1){ # if no sample
##                 ## old strains die
##                 toKill <- death.func(myAge)
##                 myPop[toKill] <- NULL
##                 myAge <- myAge[!toKill]
##                 return(list(pop=myPop, S=myS, age=myAge))
##             }
##             newGen <- myPop[sample(1:length(myPop), sampSize, replace=TRUE)] # sample strains for new generations
##             newGen <- assignMutations(newGen, createMutations(sampSize)) # mutate strains
##             newAge <- rep(0, sampSize) # new ages for newborns

##             ## old strains die
##             toKill <- death.func(myAge)
##             myPop[toKill] <- NULL
##             myAge <- myAge[!toKill]

##             ## merge old and new generation
##             myPop <- c(myPop,newGen)
##             myAge <- c(myAge, newAge)

##             ## possibly create one or more new pop
##             if((length(listPop) < max.nb.pop) & (p.new.pop()>0)) { # total number of pop. limitation
##                 nbNewPop <- rbinom(1, length(myPop), prob=p.new.pop())
##             } else {
##                 nbNewPop <- 0
##             }
##             if(nbNewPop>0){
##                 ## newPop <- sample(listPop, size=nbNewPop, replace=TRUE) # wrong
##                 newPop <- lapply(sample(myPop, size=nbNewPop, replace=TRUE), as.list)
##                 listPop <<- c(listPop, newPop)
##                 vecS <<- c(vecS, replicate(nbNewPop, max.pop.size()) )
##                 listAges <<- c(listAges, replicate(nbNewPop, 0, simplify=FALSE) )
##             } # end new pop
##             return(list(pop=myPop, S=myS, age=myAge)) ## DIFFERENCE between the two versions of the function
##         } # end no regen version
##     } ## end evolveOnePop (both versions)



##     ## INITIATE SIMULATIONS ##
##     ## INITIALIZE FROM SCRATCH
##     vecS <- max.pop.size() # susceptibles

##     if(is.null(ini.obj)){
##         ##vecS <- max.pop.size() -  n.snp.ini # susceptibles
##         if(is.null(ini.haplo)) {
##             haplo.ini <- sample(SNP.POOL, n.snp.ini, replace=TRUE)
##         } else {
##             haplo.ini <- ini.haplo
##         }

##         ANCES <- haplo.ini
##         listPop <- list()
##         listPop[[1]] <- lapply(1:ini.pop.size, function(i) haplo.ini) # contains only one population of identical clones to start with
##         listAges <- list() # will contain vectors of ages of haplotypes (a time of appearance, age=0)
##         listAges[[1]] <- rep(0, ini.pop.size)
##     } else { ## INITIALIZE WITH PROVIDED OBJECT
##         if(!inherits(ini.obj, "haploPop")) stop("x is not a haploPop object")
##         ##vecS <- ini.obj$S
##         ANCES <- attr(ini.obj, "ances")
##         listPop <- ini.obj$pop
##         listAges <- ini.obj$ages
##     }

##     ## MAKE SIMULATIONS ##

##     ## evolve all populations
##     i <- 1L
##     if(!quiet){
##         cat("\nSimulating populations of haplotypes through time: \n")
##     }
##     ##while((sum(vecS)>0) & (i<(n.steps+1))){ # evolve all generations
##     while(i<(n.steps+1)){ # evolve all generations
##         i <- i + 1L # update iterator
##         if(!quiet){
##             catStep <- max(round(n.steps/100), 10)
##             cat(ifelse((i %% catStep)==0, paste(" ...", i), ""))
##         }


##         ## make populations evolve of one generation
##         ##idx <- which(vecS>0) # make sure that new pop won't evolve this time ! leads to not dying
##         idx <- 1:length(listPop)  # make sure that new pop won't evolve this time
##         if(length(idx)>0){
##             for(j in idx){
##                 temp <- evolveOnePop(listPop[[j]], vecS[j], listAges[[j]])
##                 listPop[[j]] <- temp$pop
##                 vecS[j] <- temp$S
##                 listAges[[j]] <- temp$age
##             }
##         }

##         ## ## purge non-susceptible pop
##         ## listPop <- listPop[vecS>0]
##         ## vecS <- vecS[vecS>0]

##         ## purge empty populations
##         toKeep <- sapply(listPop, length)>0
##         listPop <- listPop[toKeep]
##         vecS <- vecS[toKeep]
##         listAges <- listAges[toKeep]

##         ## stop if all pop go extinct
##         if(length(listPop)==0L){
##             if(!quiet) cat("\n All populations went extinct at time",i,"\n")
##             return(invisible(NULL))
##         }

##         ## FOR DEBUGGING
##         ## cat("\n=== ",i," ===")
##         ## cat("\nlistPop")
##         ## print(listPop)
##         ## cat("\nvecS")
##         ## print(vecS)
##         ## cat("\nlistAges")
##         ## print(listAges)
##         ## END DEBUGGING
##     } # end while

##     if(!quiet){
##         cat("\n... done! \n")
##     }

##     ## END OF SIMULATIONS ##


##     ## CLEAN RESULTS ##
##     ## handle reverse mutations
##     ## if(clean.haplo){
##     ##     if(!quiet){
##     ##         cat("\n... Cleaning haplotypes (handling reverse mutations)\n")
##     ##     }

##     ##     cleanRes <- function(vec){
##     ##         temp <- table(vec)
##     ##         return(sort(as.integer(names(temp)[temp %% 2 != 0])))
##     ##     }

##     ##     for(i in 1:length(listPop)){
##     ##         listPop[[i]] <- lapply(listPop[[i]], cleanRes)
##     ##     }

##     ##     if(!quiet){
##     ##         cat("\n... done! \n")
##     ##     }
##     ## }

##     ## RETURN RESULTS ##
##     res <- list(pop=listPop, ages=listAges, S=vecS)
##     class(res) <- "haploPop"
##     res$call <- match.call()
##     attr(res,"ances") <- ANCES # ancestral genotype
##     return(res)

## } # end haploPop






## ##################
## ## print.haploPop
## ##################
## print.haploPop <- function(x, ...){
##     myCall <- x$call

##     cat("\n== haploPop object ==\n")
##     cat("\nNumber of populations :", length(x$pop))

##     N <- sum(sapply(x$pop,length))
##     cat("\nNumber of haplotypes :", N)

##     N.mut <- length(unique(unlist(x$pop)))
##     cat("\nNumber of mutations :", N.mut)

##     N.empty <- sum(sapply(x$pop, function(e) length(e)==0))
##     cat("\nNumber of unmutated genotypes :", N.empty)

##     if( (length(x$pop) == length(x$ages)) & (length(x$pop) == length(x$S)) ){
##         cat("\nSlot lengths consistency:  OK\n")
##     } else {
##         cat("\nSlot lengths consistency: !! NOT OK !!\n")
##     }
## } # end print.haploPop






## ##################
## ## summary.haploPop
## ##################
## summary.haploPop <- function(object, ...){
##     x <- object$pop
##     myCall <- x$call
##     x$call <- NULL
##     res <- list()

##     ## cat("\t\n=======================================")
##     ## cat("\t\n= simulated populations of haplotypes =")
##     ## cat("\t\n=          (haploPop object)          =")
##     ## cat("\t\n=======================================\n")

##     cat("\nNumber of populations :", length(x))

##     cat("\nPopulation sizes :\n")
##     temp <- sapply(x,length)
##     names(temp) <- 1:length(temp)
##     print(temp)
##     res$pop.size <- temp

##     cat("\nNumber of SNPs per population :\n")
##     temp <- sapply(x,function(e) length(unique(unlist(e))))
##     names(temp) <- 1:length(temp)
##     print(temp)
##     res$n.snp <- temp

##     return(invisible(res))
## } # end print.haploPop






## ##################
## ## sample.haploPop
## ##################
## sample.haploPop <- function(x, n, n.pop=NULL, keep.pop=TRUE){
##     if(!inherits(x, "haploPop")) stop("x is not a haploPop object")
##     x$call <- NULL

##     if(!is.null(n.pop)){ # pre-treatment: reduce to n.pop populations with same size
##         ## kEEP ONLY SOME POP
##         popToKeep <- sample(which(sapply(x$pop, length) > n), n.pop, replace=FALSE) # keep n.pop large enough populations
##         if(length(popToKeep)==0L) stop("No population is big enough for this sampling.")
##         x$pop <- x$pop[popToKeep]
##         x$ages <- x$ages[popToKeep]
##         x$S <- x$S[popToKeep]

##         ## MAKE THEM THE SAME SIZE
##         popSizes <- sapply(x$pop, length)
##         for(i in 1:n.pop){
##             idx <- sample(1:popSizes[i], n, replace=FALSE)
##             x$pop[[i]] <- x$pop[[i]][idx]
##             x$ages[[i]] <- x$ages[[i]][idx]
##         }

##     } # end pop pre-treatment

##     if(keep.pop){
##         popSizes <- sapply(x$pop, length)
##         pop.id <- rep(1:length(x$pop), popSizes)
##     }

##     x$pop <- unlist(x$pop, recursive=FALSE)
##     x$ages <- unlist(x$ages, recursive=FALSE)

##     idx <- sample(1:length(x$pop), n, replace=FALSE)
##     res <- list(pop=list(), ages=list() )

##     if(keep.pop){
##         res$pop <- split(x$pop[idx], pop.id[idx])
##         res$ages <- split(x$ages[idx], pop.id[idx])
##     } else {
##         res$pop[[1]] <- x$pop[idx]
##         res$ages[[1]] <- x$ages[idx]
##     }

##     res$S <- rep(n, length(res$pop))

##     class(res) <- "haploPop"
##     attr(res, "ances") <- attr(x, "ances")
##     return(res)
## } # end sample.haploPop






## ###############
## ## dist.haploPop
## ###############
## dist.haploPop <- function(x, add.root=TRUE, res.type=c("dist","matrix")){
##     if(!inherits(x, "haploPop")) stop("x is not a haploPop object")
##     res.type <- match.arg(res.type)
##     ANCES <- attr(x,"ances")

##     x <- unlist(x$pop, recursive=FALSE)

##     ## handle root
##     if(add.root){ # add the root
##        x <- c(ANCES, x)
##     }

##     n <- length(x)

##     f1 <- function(a,b){
##         return(sum(!union(unlist(a),unlist(b)) %in% intersect(unlist(a),unlist(b))))
##     }

##     ## res <- outer(x, x, FUN=f1)
##     res <- matrix(0, ncol=n, nrow=n)
##     for(i in 1:(n-1)){
##         for(j in (i+1):n){
##             res[i,j] <- f1(x[[i]], x[[j]])
##         }
##     }

##     res <- res+t(res)

##     if(res.type=="dist"){
##         res <- as.dist(res)
##     }
##     return(res)
## } # end dist.haploPop






## ###############
## ## plot.haploPop
## ###############
## plot.haploPop <- function(x, y=NULL, type="unrooted", size.limit=300, show.pop=TRUE, col=NULL,
##                           transp=TRUE, tip.cex=2, method=c("nj", "bionj", "fastme.bal", "fastme.ols"), ...){
##     ## CHECKS ##
##     if(!require(ape)) stop("ape package is required")
##     if(!inherits(x, "haploPop")) stop("x is not a haploPop object")
##     method <- match.arg(method)

##     N <- sum(sapply(x$pop,length))

##     if(N > size.limit) {
##         stop("tree exceeds size limit")
##     }


##     ## PLOT TREE ##
##     f1 <- get(method)
##     if(method %in% c("nj","bionj")){
##         tre <- root(f1(dist.haploPop(x)),"1")
##     } else {
##         tre <- f1(dist.haploPop(x))
##      }
##     plot(tre, type=type, ...)
##     xy <- get("last_plot.phylo", envir = .PlotPhyloEnv)


##     ## SHOW POPULATIONS ##
##     if(!is.null(col)){
##         if(is.integer(col) | is.numeric(col)) {
##             col <- palette()[col]
##         }
##         if(transp){
##             transp <- function(col, alpha=.5){
##                 res <- apply(col2rgb(col),2, function(c) rgb(c[1]/255, c[2]/255, c[3]/255, alpha))
##                 return(res)
##             }

##             col <- transp(col)
##         }
##           points(xy$xx[2:(N+1)], xy$yy[2:(N+1)], pch=20, col=col, cex=tip.cex)

##     } else if(show.pop){
##         nPop <- length(x$pop)
##         popSizes <- sapply(x$pop, length)
##         pop.id <- rep(1:length(x$pop), popSizes)
##         opal <- palette()
##         on.exit(palette(opal))
##         if(nPop>1){
##             pop.col <- rainbow(nPop)
##         } else {
##             pop.col <- c("red","red")
##         }

##         if(transp){
##             transp <- function(col, alpha=.5){
##                 res <- apply(col2rgb(col),2, function(c) rgb(c[1]/255, c[2]/255, c[3]/255, alpha))
##                 return(res)
##             }

##             pop.col <- transp(pop.col)
##         }

##         palette(pop.col)
##         points(xy$xx[2:(N+1)], xy$yy[2:(N+1)], pch=20, col=pop.id, cex=tip.cex)
##     }


##     ## SHOW ROOT ##
##     points(xy$xx[1], xy$yy[1], pch=20, cex=3)

##     return(invisible(tre))
## } # end plot.haploPop
























## ##########################################################################
## ##########################################################################
## ##########################################################################
## ##########################################################################
## ##########################################################################
## ##########################################################################
## ##########################################################################























## ############
## ## haploPopDiv
## ############
## haploPopDiv <- function(n.steps=20, ini.obj=NULL, ini.haplo=NULL, haplo.length=1e6, mu=1e-5, n.snp.ini=1,
##                         birth.func=function(){ sample(0:3, 1, prob=c(.05, .45, .35, .15))},
##                         max.pop.size=function(){1e4}, max.nb.pop=30, ini.pop.size=10, regen=FALSE,
##                         p.new.pop=function(){1e-4}, death.func=function(age){age>1},
##                         quiet=FALSE, allow.reverse=TRUE,
##                         track=c("div", "distRoot", "freq","nbMut"), root.haplo=NULL, samp.size=50) {


##     ## SOME CHECKS
##     ## if(is.numeric(ini.pop.size)){
##     ##     ini.pop.size.val <- ini.pop.size
##     ##     ini.pop.size <- function(){ini.pop.size.val}
##     ## }

##     track <- match.arg(track)

##     if(is.numeric(max.pop.size)){
##         max.pop.size.val <- max.pop.size
##         max.pop.size <- function(){max.pop.size.val}
##     }

##      if(is.numeric(p.new.pop)){
##         p.new.pop.val <- p.new.pop
##         p.new.pop <- function(){p.new.pop.val}
##     }

##     if(is.numeric(birth.func)){
##         birth.func.val <- birth.func[1]
##         birth.func <- function(){birth.func.val}
##     }

##     if(is.numeric(death.func)){
##         death.func.val <- death.func[1]
##         death.func <- function(age){age>death.func.val}
##     }


##     ## GLOBAL VARIABLES ##
##     SNP.POOL <- 1:haplo.length
##     vecS <- 1 # will be redefined later, but needed for evolveOnePop definition

##     ## AUXILIARY FUNCTIONS ##
##     if(allow.reverse){
##         createMutations <- function(N){ # L:genome length; N: pop size
##             nb.mutations <- sum(rbinom(N, size=haplo.length, prob=mu))
##             return( sample(SNP.POOL, size=nb.mutations, replace=TRUE) )
##         }
##     } else {
##         createMutations <- function(N){ # L:genome length; N: pop size
##             nb.mutations <- sum(rbinom(N, size=haplo.length, prob=mu))
##             res <- sample(SNP.POOL, size=nb.mutations, replace=TRUE)
##             SNP.POOL <<- setdiff(SNP.POOL, res)# update pool of SNPs
##             return(res)
##         }
##     }


##     ## assign mutation to haplotypes
##     assignMutations <- function(myPop, mutations){ # mypop: list of `haplotypes'; mutations: vector of SNPs
##         if(length(mutations)==0 | length(myPop)==0) return(myPop)
##         id <- sample(1:length(myPop), size=length(mutations), replace=TRUE)
##         mutations <- split(mutations, id)

##         ## function to merge new mutations - handle reverse case
##         f1 <- function(a,b){
##             revMut <- intersect(a,b)
##             if(length(revMut)==0) return(c(a,b))
##             return(setdiff(c(a ,b), revMut))
##         }

##         ##myPop[as.integer(names(mutations))] <- mapply(c, myPop[as.integer(names(mutations))], mutations, SIMPLIFY=FALSE)
##          myPop[as.integer(names(mutations))] <- mapply(f1, myPop[as.integer(names(mutations))], mutations, SIMPLIFY=FALSE)

##         return(myPop)
##     } # end assignMutations


##     if(!regen){
##         ## VERSION FOR NO REGENERATION OF SUSCEPTIBLES
##         evolveOnePop <- function(myPop, myS, myAge){ # myPop: pop to evolve; myS: nb of susceptible in the pop; myAge: vector of ages
##             ## strains get older
##             myAge <- myAge + 1
##             ## toKill <- death.func(myAge)
##             ## myPop[toKill] <- NULL
##             ## myAge <- myAge[!toKill]

##             ## generate new strains for new generation
##             sampSize <- round(min( length(myPop)*birth.func(), myS)) # number of strains for next step
##             if(sampSize<1){ # if no sample
##                 ## old strains die
##                 toKill <- death.func(myAge)
##                 myPop[toKill] <- NULL
##                 myAge <- myAge[!toKill]
##                 return(list(pop=myPop, S=myS, age=myAge))
##             }
##             newGen <- myPop[sample(1:length(myPop), sampSize, replace=TRUE)] # sample strains for new generations
##             newGen <- assignMutations(newGen, createMutations(sampSize)) # mutate strains
##             newAge <- rep(0, sampSize) # new ages for newborns

##             ## old strains die
##             toKill <- death.func(myAge)
##             myPop[toKill] <- NULL
##             myAge <- myAge[!toKill]

##             ## merge old and new generation
##             myPop <- c(myPop,newGen)
##             myAge <- c(myAge, newAge)

##             ## possibly create one or more new pop
##             if((length(listPop) < max.nb.pop) & (p.new.pop()>0)) { # total number of pop. limitation
##                 nbNewPop <- rbinom(1, length(myPop), prob=p.new.pop())
##             } else {
##                 nbNewPop <- 0
##             }
##             if(nbNewPop>0){
##                 ## newPop <- sample(listPop, size=nbNewPop, replace=TRUE) # wrong
##                 newPop <- lapply(sample(myPop, size=nbNewPop, replace=TRUE), as.list)
##                 listPop <<- c(listPop, newPop)
##                 vecS <<- c(vecS, replicate(nbNewPop, max.pop.size()) )
##                 listAges <<- c(listAges, replicate(nbNewPop, 0, simplify=FALSE) )
##             } # end new pop
##             return(list(pop=myPop, S=myS-sampSize, age=myAge))
##         } # end no regen version
##     } else { ## REGEN VERSION
##         evolveOnePop <- function(myPop, myS, myAge){ # myPop: pop to evolve; myS: nb of susceptible in the pop; myAge: vector of ages
##             ## strains get older
##             myAge <- myAge + 1
##             ## toKill <- death.func(myAge)
##             ## myPop[toKill] <- NULL
##             ## myAge <- myAge[!toKill]
##             myS <- max.pop.size() ## DIFFERENCE between the two versions of the function

##             ## generate new strains for new generation
##             sampSize <- round(min( length(myPop)*birth.func(), myS)) # number of strains for next step
##             if(sampSize<1){ # if no sample
##                 ## old strains die
##                 toKill <- death.func(myAge)
##                 myPop[toKill] <- NULL
##                 myAge <- myAge[!toKill]
##                 return(list(pop=myPop, S=myS, age=myAge))
##             }
##             newGen <- myPop[sample(1:length(myPop), sampSize, replace=TRUE)] # sample strains for new generations
##             newGen <- assignMutations(newGen, createMutations(sampSize)) # mutate strains
##             newAge <- rep(0, sampSize) # new ages for newborns

##             ## old strains die
##             toKill <- death.func(myAge)
##             myPop[toKill] <- NULL
##             myAge <- myAge[!toKill]

##             ## merge old and new generation
##             myPop <- c(myPop,newGen)
##             myAge <- c(myAge, newAge)

##             ## possibly create one or more new pop
##             if((length(listPop) < max.nb.pop) & (p.new.pop()>0)) { # total number of pop. limitation
##                 nbNewPop <- rbinom(1, length(myPop), prob=p.new.pop())
##             } else {
##                 nbNewPop <- 0
##             }
##             if(nbNewPop>0){
##                 ## newPop <- sample(listPop, size=nbNewPop, replace=TRUE) # wrong
##                 newPop <- lapply(sample(myPop, size=nbNewPop, replace=TRUE), as.list)
##                 listPop <<- c(listPop, newPop)
##                 vecS <<- c(vecS, replicate(nbNewPop, max.pop.size()) )
##                 listAges <<- c(listAges, replicate(nbNewPop, 0, simplify=FALSE) )
##             } # end new pop
##             return(list(pop=myPop, S=myS, age=myAge)) ## DIFFERENCE between the two versions of the function
##         } # end no regen version
##     } ## end evolveOnePop (both versions)


##     ## INITIATE SIMULATIONS ##
##     ## INITIALIZE FROM SCRATCH
##     vecS <- max.pop.size() # susceptibles

##     if(is.null(ini.obj)){
##         ## vecS <- max.pop.size() -  n.snp.ini # susceptibles

##         if(is.null(ini.haplo)) {
##             haplo.ini <- sample(SNP.POOL, n.snp.ini, replace=TRUE)
##         } else {
##             haplo.ini <- ini.haplo
##         }
##         ANCES <- haplo.ini
##         listPop <- list()
##         listPop[[1]] <- lapply(1:ini.pop.size, function(i) haplo.ini) # contains only one population of identical clones to start with
##         listAges <- list() # will contain vectors of ages of haplotypes (a time of appearance, age=0)
##         listAges[[1]] <- rep(0, ini.pop.size)
##     } else { ## INITIALIZE WITH PROVIDED OBJECT
##         if(!inherits(ini.obj, "haploPop")) stop("x is not a haploPopDiv object")
##         ## vecS <- ini.obj$S
##         ANCES <- attr(ini.obj, "ances")
##         listPop <- ini.obj$pop
##         listAges <- ini.obj$ages
##     }

##     ## function getting pairwise distances
##     if(track=="div"){
##         fRes <- function(list.pop){
##             list.pop <- list(pop=list.pop) # kludge needed for dist.haploPop
##             class(list.pop) <- "haploPop" # kludge needed for dist.haploPop
##             N <- sum(sapply(list.pop$pop, length))
##             if(N<2) return(0)
##             if(N > samp.size){
##                 return(dist.haploPop(sample.haploPop(list.pop, samp.size, keep.pop=FALSE), add.root=FALSE)) # do not include the root in distances.
##             } else {
##                 return(dist.haploPop(list.pop, add.root=FALSE))
##             }
##         } # end fRes
##     }

##     ## function getting distances to the root
##     if(track=="distRoot"){
##         if(is.null(root.haplo)) {
##             root.haplo <- ANCES
##         }
##         fRes <- function(list.pop){
##             list.pop <- list(pop=list.pop) # kludge needed for sample.haploPop
##             class(list.pop) <- "haploPop" # kludge needed for sample.haploPop
##             N <- sum(sapply(list.pop$pop, length))
##             if(N<1) return(0)
##             if(N > samp.size){
##                 list.pop <- sample.haploPop(list.pop, samp.size, keep.pop=FALSE)
##             }

##             res <- sapply(unlist(list.pop$pop, recursive=FALSE), function(e) sum(!e %in% root.haplo))
##             return(res)
##         } # end fRes
##     }

##     ## function getting allele absolute frequencies
##     if(track=="freq"){
##         fRes <- function(list.pop){
##             res <- table(unlist(list.pop))
##             return(res)
##         } # end fRes
##     }

##     ## function getting allele absolute frequencies
##     if(track=="nbMut"){
##         fRes <- function(list.pop){
##             list.pop <- list(pop=list.pop) # kludge needed for sample.haploPop
##             class(list.pop) <- "haploPop" # kludge needed for sample.haploPop
##             N <- sum(sapply(list.pop$pop, length))
##             if(N<1) return(0)
##             if(N > samp.size){
##                 list.pop <- sample.haploPop(list.pop, samp.size, keep.pop=FALSE)
##             }

##             return( length(unique(unlist(list.pop))) )
##         } # end fRes

##     }



##     res <- list(div=list(), popSize=integer())
##     res$div[[1]] <- fRes(listPop)
##     res$popSize[1] <- sum(sapply(listPop, length))


##     ## MAKE SIMULATIONS ##

##     ## evolve all populations
##     i <- 1L
##     if(!quiet){
##         cat("\nSimulating populations of haplotypes through time: \n")
##     }
##     ##while((sum(vecS)>0) & (i<(n.steps+1))){ # evolve all generations
##     while(i<(n.steps+1)){ # evolve all generations
##         i <- i + 1L # update iterator
##         if(!quiet){
##             catStep <- max(round(n.steps/100), 10)
##             cat(ifelse((i %% catStep)==0, paste(" ...", i), ""))
##         }


##         ## make populations evolve of one generation
##         ##idx <- which(vecS>0) # make sure that new pop won't evolve this time ! leads to not dying
##         idx <- 1:length(listPop)  # make sure that new pop won't evolve this time
##         if(length(idx)>0){
##             for(j in idx){
##                 temp <- evolveOnePop(listPop[[j]], vecS[j], listAges[[j]])
##                 listPop[[j]] <- temp$pop
##                 vecS[j] <- temp$S
##                 listAges[[j]] <- temp$age
##             }

##         }

##         ## ## purge non-susceptible pop
##         ## listPop <- listPop[vecS>0]
##         ## vecS <- vecS[vecS>0]

##         ## purge empty populations
##         toKeep <- sapply(listPop, length)>0
##         listPop <- listPop[toKeep]
##         vecS <- vecS[toKeep]
##         listAges <- listAges[toKeep]

##         ## stop if all pop go extinct
##         if(length(listPop)==0L){
##             if(!quiet) cat("\n All populations went extinct at time",i,"\n")
##             return(res)
##         }

##         res$div[[i]] <- fRes(listPop)
##         res$popSize[i] <- sum(sapply(listPop, length))

##         ## FOR DEBUGGING
##         ## cat("\n=== ",i," ===")
##         ## cat("\nlistPop")
##         ## print(listPop)
##         ## cat("\nvecS")
##         ## print(vecS)
##         ## cat("\nlistAges")
##         ## print(listAges)
##         ## END DEBUGGING
##     } # end while

##     if(!quiet){
##         cat("\n... done! \n")
##     }

##     ## END OF SIMULATIONS ##

##     ## STORE HAPLOPOP OBJECT
##      obj <- list(pop=listPop, ages=listAges, S=vecS)
##     class(obj) <- "haploPop"
##     obj$call <- match.call()
##     attr(obj,"ances") <- ANCES # ancestral genotype

##     if(!quiet) cat("\nStored haploPop object in 'last.haploPop'\n")
##     assign("last.haploPop", obj, envir= .GlobalEnv)


##     ## RETURN RES
##     return(res)

## } # end haploPopDiv

#' Monte Carlo test for sPCA
#'
#' The function \code{spca_randtest} implements Monte-Carlo tests for the
#' presence of significant spatial structures in a sPCA object. Two tests are
#' run, for global (positive autocorrelation) and local (negative
#' autocorrelation) structures, respectively. The test statistics used are the
#' sum of the absolute values of the corresponding eigenvalues.
#'
#' @export
#'
#' @author Original code by Valeria Montano adapted by Thibaut Jombart.
#'
#' @param x A \code{\link{spca}} object.
#'
#' @param nperm The number of permutations to be used for the test.
#'
#' @return
#'
#' A list with two objects of the class 'randtest' (see
#' \code{\link[ade4]{as.randtest}}), the first one for 'global' structures
#' (positivie autocorrelation) and the second for 'local' structures (negative
#' autocorrelation).
#'
#' @examples
#'
#' \dontrun{
#' ## Load data
#' data(sim2pop)
#'
#' ## Make spca
#' spca1 <- spca(sim2pop, type = 1, scannf = FALSE, plot.nb = FALSE)
#'
#' spca1
#' plot(spca1)
#'
#' ## run tests (use more permutations in practice, e.g. 999)
#' tests <- spca_randtest(spca1, nperm = 49)
#'
#' ## check results
#' tests
#' plot(tests[[1]]) # global structures
#  plot(tests[[2]]) # local structures
#'
#' }
#'
spca_randtest <-function(x, nperm = 499){

  if(!inherits(x, "spca")){
    stop("x must be an spca object")
  }


  ## This function compute the test statistics for a given data object. Two test
  ## statistics are computed, from the eigenvalues of the sPCA, called 'lambda':

  ## sum(lambda >= 0)
  ## sum(lambda < 0)

  get_stats <- function(obj){
    obj_pca <- ade4::dudi.pca(obj, center = FALSE, scale = FALSE,
                              scannf = FALSE)
    obj_spca <- ade4::multispati(dudi = obj_pca,
                                 listw = x$lw, scannf = FALSE,
                                 nfposi = 1, nfnega = 1)
    lambda <- obj_spca$eig
    lambda_pos <- lambda[lambda >= 0]
    lambda_neg <- lambda[lambda < 0]
    stats <- c(pos = sum(lambda_pos),
               neg = sum(abs(lambda_neg)))

    return(stats)
  }


  ## This function permutes individuals (rows) in the dataset.

  perm_data <- function(obj = x$tab){
    obj[sample(1:nrow(obj)), , drop = FALSE]
  }

  sims <- vapply(seq_len(nperm),
                 function(i) get_stats(perm_data()),
                 double(2))

  obs <- get_stats(x$tab)

  pos_test <- as.randtest(sim = sims[1,], obs = obs[1], alter = "greater")
  neg_test <- as.randtest(sim = sims[2,], obs = obs[2], alter = "greater")

  list(global = pos_test, local = neg_test)
}

############
# propTyped
############
setGeneric("propTyped", function(x,...){
    standardGeneric("propTyped")
})



setMethod("propTyped","genind", function(x, by=c("ind","loc","both")){

    ## checkType(x)
    by <- match.arg(by)

    ## PA case ##
    if(x@type=="PA"){
        temp <- as.matrix(x)

        if(by=="ind"){
            res <- apply(temp,1,function(r) mean(is.na(r)))
            return(1-res)
        }

        if(by=="loc"){
            res <- apply(temp,2,function(c) mean(is.na(c)))
            return(1-res)
        }

        if(by=="both"){
            res <- !is.na(temp)
            return(res*1)
        }
    } # end PA case


    ## codom case ##

    ## auxil function f1
    f1 <- function(vec){
        if(any(is.na(vec))) return(0)
        else return(1)
    }

    ## temp is a list (one component / marker)
    ## with n values (0: not typed, 1: typed)
    kX <- seploc(x,res.type="matrix")
    temp <- lapply(kX, function(X) apply(X, 1, f1))

    ## by individual
    if(by=="ind"){
        temp <- as.data.frame(temp)
        res <- apply(temp,1,mean)
    }

    ## by locus
    if(by=="loc"){
        res <- unlist(lapply(temp,mean))
    }

    ## by individual and locus
    if(by=="both"){
        res <- as.matrix(as.data.frame(temp))
    }

    return(res)
})





setMethod("propTyped","genpop", function(x, by=c("pop","loc","both")){

    ## checkType(x)
    by <- match.arg(by)


    ## PA case ##
    if(x@type=="PA"){
        temp <- as.matrix(x)

        if(by=="pop"){
            res <- apply(temp,1,function(r) mean(is.na(r)))
            return(1-res)
        }

        if(by=="loc"){
            res <- apply(temp,2,function(c) mean(is.na(c)))
            return(1-res)
        }

        if(by=="both"){
            res <- !is.na(temp)
            return(res*1)
        }
    } # end PA case


    ## codom case ##


    ## auxil function f1
    f1 <- function(vec){
        if(any(is.na(vec))) return(0)
        else return(1)
    }

    ## weighted mean
    mean.w <- function(x,w=rep(1/length(x),length(x))){
        x <- x[!is.na(x)]
        w <- w[!is.na(x)]
        w <- w/sum(w)
        return(sum(x*w))
    }

    ## temp is a list (one component / marker)
    ## with n values (0: not typed, 1: typed)
    kX <- seploc(x,res.type="matrix")
    temp <- lapply(kX, function(X) apply(X, 1, f1))

    ## by individual
    if(by=="pop"){
        temp <- as.data.frame(temp)
        w <- unlist(lapply(kX, sum,na.rm=TRUE))
        res <- apply(temp,1,mean.w,w=w)
    }

    ## by locus
    if(by=="loc"){
        w <- apply(x@tab,1,sum,na.rm=TRUE)
        res <- unlist(lapply(temp,mean.w,w=w))
    }

    ## by individual and locus
    if(by=="both"){
        res <- as.matrix(as.data.frame(temp))
    }

    return(res)
})

############################################
#
# Functions to transform a genind object
# into other R classes
#
# Thibaut Jombart
# t.jombart@imperial.ac.uk
#
############################################


## ############################
## # Function genind2hierfstat
## ############################
## genind2hierfstat <- function(x,pop=NULL){
##     ##  if(!inherits(x,"genind")) stop("x must be a genind object (see ?genind)")
##     ##   invisible(validObject(x))
##     if(!is.genind(x)) stop("x is not a valid genind object")
##     if(any(ploidy(x) != 2L)) stop("not implemented for non-diploid genotypes")
##     checkType(x)

##     if(is.null(pop)) pop <- pop(x)
##     if(is.null(pop)) pop <- as.factor(rep("P1",nrow(x@tab)))

##     ## ## NOTES ON THE CODING IN HIERFSTAT ##
##     ## - interpreting function is genot2al
##     ## - same coding has to be used for all loci
##     ## (i.e., all based on the maximum number of digits to be used)
##     ## - alleles have to be coded as integers
##     ## - alleles have to be sorted by increasing order when coding a genotype
##     ## - for instance, 121 is 1/21, 101 is 1/1, 11 is 1/1

##     ## find max number of alleles ##
##     max.nall <- max(nAll(x))
##     x@all.names <- lapply(alleles(x), function(e) .genlab("",max.nall)[1:length(e)])


##     ## VERSION USING GENIND2DF ##
##     gen <- genind2df(x, sep="", usepop=FALSE)
##     gen <- as.matrix(data.frame(lapply(gen, as.numeric)))
##     res <- cbind(as.numeric(pop),as.data.frame(gen))
##     colnames(res) <- c("pop", locNames(x))
##     if(!any(table(indNames(x))>1)){
##         rownames(res) <- indNames(x)
##     } else {
##         warning("non-unique labels for individuals; using generic labels")
##         rownames(res) <- 1:nrow(res)
##     }

##     return(res)
## } # end genind2hierfstat





#####################
# Function genind2df
#####################
#' Convert a genind object to a data.frame.
#'
#' The function \code{genind2df} converts a \linkS4class{genind} back to a
#' data.frame of raw allelic data.
#'
#' @aliases genind2df
#'
#' @param x a \linkS4class{genind} object
#' @param pop an optional factor giving the population of each individual.
#' @param sep a character string separating alleles. See details.
#' @param usepop a logical stating whether the population (argument \code{pop}
#' or \code{x@@pop} should be used (TRUE, default) or not (FALSE)).
#' @param oneColPerAll a logical stating whether or not alleles should be split
#' into columns (defaults to \code{FALSE}). This will only work with data with
#' consistent ploidies.
#'
#' @return a data.frame of raw allelic data, with individuals in rows and loci in column
#'
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#'
#' @seealso \code{\link{df2genind}}, \code{\link{import2genind}}, \code{\link{read.genetix}},
#' \code{\link{read.fstat}}, \code{\link{read.structure}}
#' @keywords manip
#' @examples
#'
#' ## simple example
#' df <- data.frame(locusA=c("11","11","12","32"),
#' locusB=c(NA,"34","55","15"),locusC=c("22","22","21","22"))
#' row.names(df) <- .genlab("genotype",4)
#' df
#'
#' obj <- df2genind(df, ploidy=2, ncode=1)
#' obj
#' obj@@tab
#'
#'
#' ## converting a genind as data.frame
#' genind2df(obj)
#' genind2df(obj, sep="/")
#'
#' @export
#'
genind2df <- function(x, pop=NULL, sep="", usepop=TRUE, oneColPerAll = FALSE){

  if(!is.genind(x)) stop("x is not a valid genind object")
  ## checkType(x)

  if(is.null(pop)) {
      pop <- x@pop
  }

  ## PA case ##
  if(x@type=="PA"){
      res <- tab(x)
      if(usepop && !is.null(pop)) res <- cbind.data.frame(pop=pop(x),res)
      return(res) # exit here
  }

  ## codom case ##
  # make one table by locus from x@tab
  kX <- seploc(x,res.type="matrix")

  if (oneColPerAll & all(x@ploidy == x@ploidy[1])){
    sep <- "/"
  }
  ## function to recode a genotype in form "A1[sep]...[sep]Ak" from frequencies
  recod <- function(vec,lab){
      if(any(is.na(vec))) return(NA)
      res <- paste(rep(lab,vec), collapse=sep)
      return(res)
  }


  # kGen is a list of nloc vectors of genotypes
  kGen <- lapply(1:length(kX), function(i) apply(kX[[i]],1,recod,x@all.names[[i]]))
  names(kGen) <- locNames(x)

  ## if use one column per allele
  if(oneColPerAll){
    if (all(x@ploidy == x@ploidy[1])){
      f1 <- function(vec){ # to repeat NA with seperators
          vec[is.na(vec)] <- paste(rep("NA", x@ploidy[1]), collapse=sep)
          return(vec)
      }
      temp <- lapply(kGen, f1)
      temp <- lapply(temp, strsplit,sep)

      res <- lapply(temp, function(e) matrix(unlist(e), ncol=x@ploidy[1], byrow=TRUE))
      res <- data.frame(res,stringsAsFactors=FALSE)
      names(res) <- paste(rep(locNames(x),each=x@ploidy[1]), 1:x@ploidy[1], sep=".")

      ## handle pop here
      if(!is.null(pop) & usepop) res <- cbind.data.frame(pop,res,stringsAsFactors=FALSE)
      rownames(res) <- indNames(x)
      return(res) # exit here
    } else {
      warning("All ploidies must be equal in order to separate the alleles.\nReturning one column per locus")
    }
  } # end if oneColPerAll

  ## build the final data.frame
  ## res <- as.data.frame(cbind(kGen,stringsAsFactors=FALSE))
  ## faster option
  res <- as.data.frame(do.call(cbind, kGen), stringsAsFactors=FALSE)
  rownames(res) <- indNames(x)
  colnames(res) <- locNames(x)

  ## handle pop here
  if(!is.null(pop) & usepop) res <- cbind.data.frame(pop,res,stringsAsFactors=FALSE)

  return(res)
}

#############
## GENERIC ##
#############
gengraph <-  function (x, ...) UseMethod("gengraph")





#############
## DEFAULT ##
#############
gengraph.default <- function(x, cutoff=NULL, ngrp=NULL, computeAll=FALSE, plot=TRUE,
                             show.graph=TRUE, col.pal=funky, truenames=TRUE, nbreaks=10, ...){
    stop(paste("No method for objects of class",class(x)))
} # end gengraph.default





############
## MATRIX ##
############
##
## this is the basic method
##
gengraph.matrix <- function(x, cutoff=NULL, ngrp=NULL, computeAll=FALSE, plot=TRUE, show.graph=TRUE, col.pal=funky,
                            truenames=TRUE, nbreaks=10, ...){
    ## CHECKS ##

    ## IF COMPUTEALL IS TRUE ##
    if(computeAll){
        cutoffvec <- 1:max(x)
        res <- lapply(cutoffvec, function(i) gengraph.matrix(x, cutoff=i, computeAll=FALSE))
        temp <- sapply(res, function(e) e$clust$no)
        if(plot){
            plot(cutoffvec, temp, xlab="Cut-off Hamming distance chosen", ylab="Number of groups")
        }
        return(res)
    }


    ## INTERACTIVE MODE IF BOTH CUTOFF AND NGRP MISSING ##
    if(is.null(cutoff) & is.null(ngrp)){
        chooseAgain <- TRUE
        while (chooseAgain) {
            if(plot){
                hist(x, nclass=50, col="deepskyblue1",xlab="Hamming distance",ylab="Frequency",main="Distribution of frequences")
            }
            cat("\nPlease choose a cutoff distance:  ")
            ans <- NA
            while(is.null(ans) || is.na(ans)) suppressWarnings(ans <- as.numeric(readLines(con = getOption('adegenet.testcon'), n = 1)))
            if(plot){
                abline(v=ans,col="red",lty=2, lwd=2)
            }
            res <- gengraph.matrix(x, cutoff=ans, truenames=truenames)
            if(truenames){
                V(res$graph)$label <- rownames(x)
            }

            cat(paste("\nNumber of clusters found:  ", res$clust$no, sep=""))
            if(plot && show.graph) plot(res$graph)
            ans <- ""
            while(!ans %in% c("y","n")){
                cat("\nAre you satisfied with this solution? (yes:y / no:n): ")
                ans <- tolower(readLines(con = getOption('adegenet.testcon'), n = 1))
            }
            if(ans=="y") chooseAgain <- FALSE
        }
        return(res)
    }



    ## MAIN CASE: IF CUT-OFF POINT IS GIVEN ##
    if(!is.null(cutoff)){
        x[x>=cutoff] <- 0
        g <- graph.adjacency(x, mode="undirected", weighted=TRUE, diag=FALSE)
        clust <- clusters(g)
        V(g)$color <- col.pal(clust$no)[clust$membership]
        col <- col.pal(clust$no)[1:clust$no]
        names(col) <- 1:clust$no

        ## assign labels to vertices
        if(truenames){
            V(g)$label <- rownames(x)
        } else {
            V(g)$label <- 1:nrow(x)
        }

        ## assign labels to edges
        if(length(E(g))>0) {
            E(g)$label <- E(g)$weight
        }

        ## graph plotting options ##
        V(g)$label.dist <- 0.75
        V(g)$size <- 10
        V(g)$label.family <- "sans"
        V(g)$label.color <- "black"

        ## make result
        res <- list(graph=g, clust=clusters(g), cutoff=cutoff, col=col)

    } else { ## IF CUT-OFF POINT NEEDS TO BE FOUND ##
        if(ngrp>=nrow(x)) stop("ngrp is greater than or equal to the number of individuals")

        ## FIRST HAVE A LOOK AT A RANGE OF VALUES ##
        cutToTry <- pretty(x,nbreaks)
        cutToTry <- cutToTry[cutToTry>0 & cutToTry<max(x)]
        if(length(cutToTry)==0) cutToTry <- 1
        tempRes <- lapply(cutToTry, function(i) gengraph.matrix(x,cutoff=i))
        temp <- sapply(tempRes,function(e) e$clust$no)
        if(!min(abs(temp-ngrp))<1) warning(paste("The exact number of groups was not found. Tried increasing nbreaks"))
        cutoff <- cutToTry[which.min(abs(temp-ngrp))]
        ## if(!any(temp<ngrp)) {
        ##     cutoff <- 1
        ## } else {
        ##     cutoff <- cutToTry[max(which(temp>ngrp))]
        ## }

        ## FIND THE LOWEST CUTOFF GIVING NGRP ##
        res <- gengraph.matrix(x,cutoff=cutoff)
        while(res$clust$no>ngrp){
            cutoff <- cutoff+1
            res <- gengraph.matrix(x,cutoff=cutoff)
        }

        if(res$clust$no != ngrp) cat("\nNote: the exact number of clusters could not be found.\n")
    }


    ## RETURN ##
    return(res)

} # end gengraph.matrix







############
## GENIND ##
############
gengraph.dist <- function(x, cutoff=NULL, ngrp=NULL, computeAll=FALSE, plot=TRUE,
                          show.graph=TRUE, col.pal=funky, truenames=TRUE, nbreaks=10, ...){
    ## CHECKS ##

    ## USE MATRIX METHOD ##
    res <- gengraph(as.matrix(x), cutoff=cutoff, ngrp=ngrp, computeAll=computeAll, plot=plot, show.graph=show.graph, col.pal=col.pal,
                    truenames=truenames, nbreaks=nbreaks, ...)
    return(res)
} # end gengraph.dist







############
## GENIND ##
############
gengraph.genind <- function(x, cutoff=NULL, ngrp=NULL, computeAll=FALSE, plot=TRUE,
                            show.graph=TRUE, col.pal=funky, truenames=TRUE, nbreaks=10, ...){
    ## CHECKS ##

    ## COMPUTE DISTANCES ##
    x$tab[is.na(x$tab)] <- 0
    D <- (1-propShared(x))*nLoc(x)*ploidy(x)

    ## USE MATRIX METHOD ##
    res <- gengraph(D, cutoff=cutoff, ngrp=ngrp, computeAll=computeAll, plot=plot, show.graph=show.graph, col.pal=col.pal,
                    truenames=truenames, nbreaks=nbreaks, ...)
    if(truenames){
        V(res$graph)$label <- indNames(x)
    }

    return(res)
} # end gengraph.genind








############
## GENPOP ##
############
gengraph.genpop <- function(x, cutoff=NULL, ngrp=NULL, computeAll=FALSE, plot=TRUE, show.graph=TRUE,
                            col.pal=funky, method=1, truenames=TRUE, nbreaks=10, ...){ ## CHECKS ##

    ## COMPUTE DISTANCES ##
    x$tab[is.na(x$tab)] <- 0
    D <- as.matrix(dist.genpop(x, method=method))

    ## USE MATRIX METHOD ##
    res <- gengraph(D, cutoff=cutoff, ngrp=ngrp, computeAll=computeAll, plot=plot, show.graph=show.graph, col.pal=col.pal,
                    truenames=truenames, nbreaks=nbreaks, ...)
    if(truenames){
        V(res$graph)$label <- popNames(x)
    }

    return(res)
} # end gengraph.genpop







############
## DNABIN ##
############
gengraph.DNAbin <- function(x, cutoff=NULL, ngrp=NULL, computeAll=FALSE, plot=TRUE, show.graph=TRUE, col.pal=funky,
                            truenames=TRUE, nbreaks=10, ...){
    ## CHECKS #
    ## COMPUTE DISTANCES ##
    D <- as.matrix(round(dist.dna(x,model="raw", pairwise.deletion = TRUE)*ncol(x)))

    ## USE MATRIX METHOD ##
    res <- gengraph(D, cutoff=cutoff, ngrp=ngrp, computeAll=computeAll, plot=plot, show.graph=show.graph, col.pal=col.pal,
                    truenames=truenames, nbreaks=nbreaks, ...)
    return(res)
} # end gengraph.DNAbin

###################################################################
## Fonctions designed to import files from other softwares
## into genind objects
##
## currently supported formats are :
## .gtx (GENETIX)
## .dat (Fstat)
## .gen (Genepop)
## .stru (STRUCTURE)
##
## Thibaut Jombart, avril 2006
## Revised March 2015
## t.jombart@imperial.ac.uk
##
##################################################################


######################
## Function df2genind
######################
#' Convert a data.frame of allele data to a genind object.
#'
#' The function \code{df2genind} converts a data.frame (or a matrix) into a
#' \linkS4class{genind} object. The data.frame must meet the following
#' requirements:
#' \itemize{
#' \item genotypes are in row (one row per genotype)
#' \item markers/loci are in columns
#' \item each element is a string of characters coding alleles, ideally
#' separated by a character string (argument \code{sep}); if no separator is
#' used, the number of characters coding alleles must be indicated (argument
#' \code{ncode}).}
#'
#' See \code{\link{genind2df}} to convert \linkS4class{genind} objects back to
#' such a data.frame.
#'
#' === Details for the \code{sep} argument ===\cr this character is directly
#' used in reguar expressions like \code{gsub}, and thus require some characters
#' to be preceeded by double backslashes. For instance, "/" works but "|" must
#' be coded as "\\|".
#'
#' @aliases df2genind
#' @param X a matrix or a data.frame containing allelle data only (see
#'   decription)
#' @param sep a character string separating alleles. See details.
#' @param ncode an optional integer giving the number of characters used for
#'   coding one genotype at one locus. If not provided, this is determined from
#'   data.
#' @param ind.names optinal, a vector giving the individuals names; if NULL,
#'   taken from rownames of X. If factor or numeric, vector is converted to
#'   character.
#' @param loc.names an optional character vector giving the markers names; if
#'   NULL, taken from colnames of X.
#' @param pop an optional factor giving the population of each individual.
#' @param NA.char a character string corresponding to missing allele (to be
#'   treated as NA)
#' @param ploidy an integer indicating the degree of ploidy of the genotypes.
#' @param type a character string indicating the type of marker: 'codom' stands
#'   for 'codominant' (e.g. microstallites, allozymes); 'PA' stands for
#'   'presence/absence' markers (e.g. AFLP, RAPD).
#' @param strata an optional data frame that defines population stratifications
#'   for your samples. This is especially useful if you have a hierarchical or
#'   factorial sampling design.
#' @param hierarchy a hierarchical formula that explicitely defines hierarchical
#'   levels in your strata. see \code{\link{hierarchy}} for details.
#'
#' @return an object of the class \linkS4class{genind} for \code{df2genind}; a
#'   matrix of biallelic genotypes for \code{genind2df}
#'
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}, Zhian N. Kamvar
#'   \email{kamvarz@@science.oregonstate.edu}
#'
#' @seealso \code{\link{genind2df}}, \code{\link{import2genind}},
#'   \code{\link{read.genetix}}, \code{\link{read.fstat}},
#'   \code{\link{read.structure}}
#'
#' @keywords manip
#' @examples
#'
#' ## simple example
#' df <- data.frame(locusA=c("11","11","12","32"),
#' locusB=c(NA,"34","55","15"),locusC=c("22","22","21","22"))
#' row.names(df) <- .genlab("genotype",4)
#' df
#'
#' obj <- df2genind(df, ploidy=2, ncode=1)
#' obj
#' tab(obj)
#'
#'
#' ## converting a genind as data.frame
#' genind2df(obj)
#' genind2df(obj, sep="/")
#'
#' @export
#'
df2genind <- function(X, sep=NULL, ncode=NULL, ind.names=NULL, loc.names=NULL,
                      pop=NULL, NA.char="", ploidy=2, type=c("codom","PA"),
                      strata = NULL, hierarchy = NULL){

    ## CHECKS ##
    if(is.data.frame(X)) X <- as.matrix(X)
    if (!inherits(X, "matrix")) stop ("X is not a matrix")
    res <- list()
    type <- match.arg(type)
    if (is.null(sep) && is.null(ncode) && any(ploidy > 1)){
        stop("Not enough information to convert data: please indicate the separator (sep=...) or the number of characters coding an allele (ncode=...)")
    }
    if(length(NA.char)>1) {
        warning("NA.char has several values; only the first one will be considered")
        NA.char <- NA.char[1]
    }

    # If by any chance provided ind.names are of class int/factor, they are (silently?) converted to characters.
    if (!is.character(ind.names) & !is.null(ind.names)) {
      ind.names <- as.character(ind.names)
    }


    ## TYPE-INDEPENDENT STUFF ##
    ## misc variables
    n <- nrow(X)
    nloc <- ncol(X)
    if (length(ploidy) < n){
      if (length(ploidy) == 1){
        ploidy <- rep(as.integer(ploidy), length=n)
      } else {
        undefined <- length(ploidy)/n
        msg <- paste0("\nPloidy is undefined for ",
                      undefined*100,
                      "% of data.\n",
                      "This must be a single integer indicating the ploidy of",
                      "the entire data set or vector of integers the same",
                      "length as the number of samples.")
        stop(msg)
      }

    }
    if(any(ploidy < 1L)) stop("ploidy cannot be less than 1")

    ## check individual labels
    if(is.null(ind.names)) ind.names <- rownames(X)
    if(is.null(ind.names)) ind.names <- .genlab("",n)
    if(any(duplicated(ind.names))){
        warning("duplicate labels detected for some individuals; using generic labels")
        ind.names <- .genlab("",n)
    }
    rownames(X) <- ind.names

    ## check locus labels
    if(is.null(loc.names)) loc.names <- colnames(X)
    if(is.null(loc.names)) loc.names <- .genlab("loc",nloc)
    if(any(duplicated(loc.names))){
        warning("duplicate labels detected for some loci; using generic labels")
        loc.names <- .genlab("loc",nloc)
    }
    if(length(grep("[.]", loc.names))>0L){
        warning("character '.' detected in names of loci; replacing with '_'")
        gsub("[.]","_", loc.names)
    }
    colnames(X) <- loc.names


    ## check alleles for periods
    if (length(grep("[.]", X)) > 0L){
      if (is.null(sep) || sep != "_"){
        warning("character '.' detected in names of loci; replacing with '_'")
        replacement <- "_"
      } else {
        warning("character '.' detected in names of loci; replacing with 'p'")
        replacement <- "p"
      }
      X <- apply(X, 2, function(i) gsub("[.]", replacement, i))
    }

    ## PRESENCE/ABSENCE MARKERS ##
    if(toupper(type)=="PA"){
        ## preliminary stuff
        rownames(X) <- ind.names
        colnames(X) <- loc.names

        ## Erase entierely non-typed loci
        temp <- colSums(is.na(X))==nrow(X)
        if(any(temp)){
            X <- X[,!temp]
            warning("entirely non-type marker(s) deleted")
        }

        ## Erase entierely non-type individuals
        temp <- rowSums(is.na(X))==ncol(X)
        if(any(temp)){
            X <- X[!temp,,drop=FALSE]
            if(!is.null(pop)) pop <- pop[!temp]
            ploidy <- ploidy[!temp]
            ind.names <- ind.names[!temp]
            warning("entirely non-type individual(s) deleted")
        }

        ## erase non-polymorphic loci
        temp <- apply(X, 2, function(loc) length(unique(loc[!is.na(loc)]))==1)
        if(any(temp)){
            X <- X[,!temp,drop=FALSE]
            loc.names <- loc.names[!temp]
            nloc <- ncol(X)
            warning("non-polymorphic marker(s) deleted")
        }

        prevcall <- match.call()

        res <- genind(tab=X, pop=pop, prevcall=prevcall, ploidy=ploidy,
                      type = "PA", strata = strata, hierarchy = hierarchy)

        return(res)
    } # end type PA


    ## CODOMINANT MARKERS ##
    ## make sure X is in character mode
    mode(X) <- "character"


    ## HANDLE MISSING SEPARATORS
    if(is.null(sep) && any(ploidy>1)){
        ## check that ncode is provided
        if(is.null(ncode)) stop("please indicate either the separator (sep) or the number of characters coding an allele (ncode).")

        ## add "/" as separator
        X <- gsub(paste("([[:alnum:]]{",ncode,"})",sep=""), "\\1/", X)
        X <- sub("/$","",X)
        sep <- "/"
    }

    ## HANDLE NAs
    ## find all strings which are in fact NAs
    NA.list <- unlist(lapply(unique(ploidy), function(nrep) paste(rep(NA.char, nrep), collapse=sep)))
    NA.list <- unique(c(NA.list, NA.char))

    ## replace NAs
    X[X %in% NA.list] <- NA

    ## erase entirely non-type loci
    toRemove <- which(colSums(is.na(X))==nrow(X))
    if(length(toRemove) > 0){
        X <- X[,-toRemove, drop = FALSE]
        loc.names <- loc.names[-toRemove]
        warning("entirely non-type marker(s) deleted")
    }


    ## erase entierely non-type individuals
    toRemove <- which(rowSums(is.na(X))==ncol(X))
    if(length(toRemove) > 0){
        X <- X[-toRemove, , drop = FALSE]
        ind.names <- rownames(X)
        ploidy <- ploidy[-toRemove]
        if(!is.null(pop)) pop <- pop[-toRemove]
        warning("entirely non-type individual(s) deleted")
    }


    ## TRANSLATE DATA INTO ALLELE COUNTS ##
    ## get dimensions of X
    nloc <- ncol(X)
    nind <- nrow(X)

    ## unfold data for each cell of the table
    if (any(ploidy > 1)){
        allele.data <- strsplit(X, sep)
        n.items <- sapply(allele.data, length)
        locus.data <- rep(rep(loc.names, each = nind), n.items)
        ind.data <- rep(rep(ind.names,ncol(X)), n.items)
        allele.data <- unlist(allele.data)
    } else {
        n.items     <- rep(1, length(X))
        locus.data  <- rep(rep(loc.names, each=nind), n.items)
        ind.data    <- rep(rep(ind.names, ncol(X)), n.items)
        allele.data <- unlist(X)
    }


    ## identify NAs
    NA.posi <- which(is.na(allele.data))
    NA.ind <- ind.data[NA.posi]
    NA.locus <- locus.data[NA.posi]

    ## remove NAs
    if(length(NA.posi)>0){
        allele.data <- allele.data[-NA.posi]
        locus.data <- locus.data[-NA.posi]
        ind.data <- ind.data[-NA.posi]
    }

    ## get matrix of allele counts
    allele.data <- paste(locus.data, allele.data, sep=".")
    allele.data <- factor(allele.data, levels=unique(allele.data))
    out         <- table(ind.data, allele.data)
    out         <- out[ind.names, , drop = FALSE] # table sorts alphabetically. This resets.

    ## force type 'matrix'
    class(out) <- NULL
    dimnames(out) <- list(rownames(out), colnames(out))

    ## restore NAs
    ##
    ## Thanks to Klaus Schliep for the proposed speedup:
    ##
    # if (length(NA.posi) > 0) {
    #     out.colnames <- colnames(out)
    #     NA.row <- match(NA.ind, rownames(out))
    #     loc <- paste0(NA.locus, "\\.")
    #     uloc <- unique(loc)
    #     loc.list <- lapply(uloc, grep, out.colnames)
    #     NA.col <- match(loc, uloc)
    #     out[cbind(rep(NA.row, unlist(lapply(loc.list, length))[NA.col]), unlist(loc.list[NA.col]))] <- NA
    #  }
    ## This one is modified from above to make everything more explicit.
    if (length(NA.posi) > 0) {
      out.colnames <- colnames(out)
      NA.row <- match(NA.ind, rownames(out))
      loc <- paste0(NA.locus, "\\.")
      uloc <- unique(loc)
      loc.list <- lapply(uloc, FUN = function(x, y) {
        grep(pattern = paste("^", x, sep = ""), x = out.colnames, perl = TRUE)
      }, y = out.colnames)
      NA.col <- match(loc, uloc)

      # Coordinates for missing rows
      missing.ind <- vapply(loc.list, length, integer(1))[NA.col]
      missing.ind <- rep(NA.row, missing.ind)
      # Coordinates for missing columns
      missing.loc <- unlist(loc.list[NA.col], use.names = FALSE)

      missing_coordinates <- matrix(0L, nrow = length(missing.ind), ncol = 2L)
      missing_coordinates[, 1] <- missing.ind
      missing_coordinates[, 2] <- missing.loc
      #      [,1] [,2]
      # [1,]    2    1
      # [2,]    3    1
      # [3,]    4   13
      # [4,]    4   14

      out[missing_coordinates] <- NA

      #          X1401_25.33
      # A_KH1584           1
      # C_KH1059           1
      # M_KH1834           1
      # M_KH1837           1
    }

    ploidmat <- vapply(loc.names, function(i){
      rowSums(out[, grepl(paste0("^", i, "\\."), colnames(out)), drop = FALSE], na.rm = TRUE)
      }, FUN.VALUE = double(nrow(out)))
    if (max(ploidmat, na.rm = TRUE) > max(ploidy, na.rm = TRUE)) {
      oran <- paste(range(ploidmat, na.rm = TRUE), collapse = "-")
      eran <- paste(range(ploidy, na.rm = TRUE), collapse = "-")
      msg <- paste0("The observed allele dosage (", oran, ") ", 
                    "does not match the defined ploidy ", "(", eran, ").\n",
                    "Please check that your input parameters (ncode, sep) ",
                    "are correct.")
      warning(msg, immediate. = TRUE)
    }
    ## call upon genind constructor
    prevcall <- match.call()
    out <- genind(tab=out, pop=pop, prevcall=prevcall, ploidy=ploidy, type=type,
                  strata = strata, hierarchy = hierarchy)

    return(out)
} # end df2genind







########################################
## Function read.genetix
## code based on previous ade4 functions
########################################
#'
#' Reading data from GENETIX
#'
#' The function \code{read.genetix} reads GENETIX data files (.gtx) and convert
#' them into a \linkS4class{genind} object.
#'
#' Note: \code{read.genetix} is meant for DIPLOID DATA ONLY. Haploid data with
#' the GENETIX format can be read into R using \code{read.table} or
#' \code{read.csv} after removing headers and 'POP' lines, and then converted
#' using \code{\link{df2genind}}.
#'
#' @param file a character string giving the path to the file to convert, with
#' the appropriate extension.
#' @param quiet logical stating whether a conversion message must be printed
#' (TRUE,default) or not (FALSE).
#' @return an object of the class \code{genind}
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#' @seealso \code{\link{import2genind}}, \code{\link{df2genind}},
#' \code{\link{read.fstat}}, \code{\link{read.structure}},
#' \code{\link{read.genepop}}
#' @references Belkhir K., Borsa P., Chikhi L., Raufaste N. & Bonhomme F.
#' (1996-2004) GENETIX 4.05, logiciel sous Windows TM pour la genetique des
#' populations. Laboratoire Genome, Populations, Interactions, CNRS UMR 5000,
#' Universite de Montpellier II, Montpellier (France). \cr
#' @keywords manip
#' @examples
#'
#' obj <- read.genetix(system.file("files/nancycats.gtx",package="adegenet"))
#' obj
#'
#' @export read.genetix
read.genetix <- function(file=NULL,quiet=FALSE) {
    if(!quiet) cat("\n Converting data from GENETIX to a genind object... \n")


    ## read from file
    ## if(!file.exists(file)) stop("Specified file does not exist.") <- not needed

    if(toupper(.readExt(file)) != "GTX") stop("File extension .gtx expected")
    ## retrieve first infos
    nloc <- as.integer(scan(file,nlines=1,what="character",quiet=TRUE)[1])
    npop <- as.integer(scan(file,nlines=1,skip=1,what="character",quiet=TRUE)[1])
    txt <- scan(file,skip=2,what="character",sep="\n",quiet=TRUE)
    txt <- gsub("\t"," ",txt)
    ## check that nloc is consistent with actual nloc (bug-report 1.2-2.02)
    temp <- temp <- trimws(txt[length(txt)])
    nlocbis <- length(unlist(strsplit(temp, "[[:space:]]+")))-1
    if(nloc != nlocbis) {
        warning(paste("\n== Genetix file error == \n",
                      "Indicated number of locus (", nloc, ")\n",
                      "does not match actual number (", nlocbis, ").\n",
                      "Using ", nlocbis, " as number of locus.\n",
                      "Please check your file.", sep=""))
        nloc <- nlocbis
    }
    loc.names <- txt[seq(1,by=2,length=nloc)]
    txt <- txt[-(1:(nloc*2))]

    ## retrieve populations infos
    pop.names <- vector(mode="character",length=npop)
    pop.nind <- vector(mode="integer",length=npop)
    index <- 1
    temp <- vector(mode="integer",length=npop)
    for(i in 1:npop){
        pop.names[i] <- txt[index]
        pop.nind[i] <- as.numeric(txt[index+1])
        temp[i] <- index
        index <- index + pop.nind[i] + 2
    }
    pop.names <- trimws(pop.names)

    ## retrieve genotypes infos
    txt <- txt[-c(temp,temp+1)]
    txt <- trimws(txt)
    txt <- sapply(1:length(txt),function(i) unlist(strsplit(txt[i],"([[:space:]]+)|([[:blank:]]+)")) )
    X <- t(txt)
    if(ncol(X) == (nloc+1)){
        rownames(X) <- X[,1]
        X <- X[,-1]
    } else{
        rownames(X) <- 1:nrow(X)
    }

    colnames(X) <- loc.names

    ## pop is kept as character; treatment and conversion to a factor belongs to the constructor
    ## (otherwise there is potential for inconsistencies across different import functions
    pop <- as.character(rep(pop.names,pop.nind))

    ## pass X to df2genind
    res <- df2genind(X=X, ncode=3, pop=pop, ploidy=2, NA.char="000")
    res@call <- match.call()

    if(!quiet) cat("\n...done.\n\n")

    return(res)
} # end read.genetix





######################
## Function read.fstat
######################
#' Reading data from Fstat
#'
#' The function \code{read.fstat} reads Fstat data files (.dat) and convert
#' them into a \linkS4class{genind} object.
#'
#' Note: \code{read.fstat} is meant for DIPLOID DATA ONLY. Haploid data with
#' the Hierfstat format can be read into R using \code{read.table} or
#' \code{read.csv} after removing headers and 'POP' lines, and then converted
#' using \code{\link{df2genind}}.
#'
#' @param file a character string giving the path to the file to convert, with
#' the appropriate extension.
#' @param quiet logical stating whether a conversion message must be printed
#' (TRUE,default) or not (FALSE).
#' @return an object of the class \code{genind}
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#' @seealso \code{\link{import2genind}}, \code{\link{df2genind}},
#' \code{\link{read.genetix}}, \code{\link{read.structure}},
#' \code{\link{read.genepop}}
#' @references Fstat (version 2.9.3). Software by Jerome Goudet.
#' http://www2.unil.ch/popgen/softwares/fstat.htm\cr
#' @keywords manip
#' @examples
#'
#' obj <- read.fstat(system.file("files/nancycats.dat",package="adegenet"))
#' obj
#'
#' @export read.fstat
read.fstat <- function(file, quiet=FALSE){
    ##if(!file.exists(file)) stop("Specified file does not exist.") <- not needed
    if(toupper(.readExt(file)) != "DAT") stop("File extension .dat expected")

    if(!quiet) cat("\n Converting data from a FSTAT .dat file to a genind object... \n\n")

    call <- match.call()
    txt <- scan(file,what="character",sep="\n",quiet=TRUE)
    txt <- gsub("\t"," ",txt)

    ## read length of allele
    ncode <- as.integer(unlist(strsplit(txt[1], " "))[4])
    NA.char <- paste(rep("0",ncode),collapse="")

    ## read first infos
    info <- unlist(strsplit(txt[1],"([[:space:]]+)"))
    ## npop <- as.numeric(info[1]) ## no longer used
    nloc <- as.numeric(info[2])

    loc.names <- txt[2:(nloc+1)]

    ## build genotype matrix
    txt <- txt[-(1:(nloc+1))]
    txt <- trimws(txt)
    txt <- sapply(1:length(txt),function(i) unlist(strsplit(txt[i],"([[:space:]]+)|([[:blank:]]+)")) )
    X <- t(txt)
    pop <- as.character(X[,1])
    if(length(unique(pop)) == 1 ) pop <- NULL
    X <- X[,-1]

    colnames(X) <- loc.names
    rownames(X) <- 1:nrow(X)

    ## replace all possible missing data coding by NA.char
    allNAs <- sapply(1:8, function(i) paste(rep("0",i),collapse=""))
    X[X %in% allNAs] <- NA.char

    ## call df2genind
    res <- df2genind(X=X, pop=pop, ploidy=2, ncode=ncode, NA.char=NA.char)
    res@call <- call

    if(!quiet) cat("\n...done.\n\n")

    return(res)

} # end read.fstat





##########################
## Function read.genepop
##########################
#' Reading data from Genepop
#'
#' The function \code{read.genepop} reads Genepop data files (.gen) and convert
#' them into a \linkS4class{genind} object.
#'
#' Note: \code{read.genepop} is meant for DIPLOID DATA ONLY. Haploid data with
#' the Genepop format can be read into R using \code{read.table} or
#' \code{read.csv} after removing headers and 'POP' lines, and then converted
#' using \code{\link{df2genind}}.
#'
#' @param file a character string giving the path to the file to convert, with
#' the appropriate extension.
#' @param ncode an integer indicating the number of characters used to code an allele.
#' @param quiet logical stating whether a conversion message must be printed
#' (TRUE,default) or not (FALSE).
#' @return an object of the class \code{genind}
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#' @seealso \code{\link{import2genind}}, \code{\link{df2genind}},
#' \code{\link{read.fstat}}, \code{\link{read.structure}},
#' \code{\link{read.genetix}}
#' @references Raymond M. & Rousset F, (1995). GENEPOP (version 1.2):
#' population genetics software for exact tests and ecumenicism. \emph{J.
#' Heredity}, \bold{86}:248-249 \cr
#' @keywords manip
#' @examples
#'
#' obj <- read.genepop(system.file("files/nancycats.gen",package="adegenet"))
#' obj
#'
#' @export read.genepop
read.genepop <- function(file, ncode=2L, quiet=FALSE){
    ## if(!file.exists(file)) stop("Specified file does not exist.") <- not needed
    if(toupper(.readExt(file)) != "GEN") stop("File extension .gen expected")

    if(!quiet) cat("\n Converting data from a Genepop .gen file to a genind object... \n\n")

    prevcall <- match.call()

    txt <- scan(file,sep="\n",what="character",quiet=TRUE)
    if(!quiet) cat("\nFile description: ",txt[1], "\n")
    txt <- txt[-1]
    txt <- gsub("\t", " ", txt)
    NA.char <- paste(rep("0",ncode), collapse="")

    ## two cases for locus names:
    ## 1) all on the same row, separated by ","
    ## 2) one per row
    ## ! spaces and tab allowed
    ## a bug was reported by S. Devillard, occuring
    ## when the two cases occur together,
    ## that is:
    ## loc1,
    ## loc2,
    ## ...


    ## new strategy (shorter): isolate the 'locus names' part and then parse it.
    locinfo.idx <- 1:(min(grep("POP",toupper(txt)))-1)
    locinfo <- txt[locinfo.idx]
    locinfo <- paste(locinfo,collapse=",")
    loc.names <- unlist(strsplit(locinfo,"([,]|[\n])+"))
    loc.names <- trimws(loc.names)
    nloc <- length(loc.names)
    txt <- txt[-locinfo.idx]

    ## locus names have been retreived

    ## build the pop factor
    ## and correct the genotypes splited on more than 1 line
    pop.idx <- grep("^([[:space:]]*)POP([[:space:]]*)$",toupper(txt))
    npop <- length(pop.idx)
    ## correction for splited genotype
    ## isolated by the absence of comma on a line not containing "pop"
    nocomma <- which(! (1:length(txt)) %in% grep(",",txt))
    splited <- nocomma[which(! nocomma %in% pop.idx)]
    if(length(splited)>0){
        for(i in sort(splited,decreasing=TRUE)){
            txt[i-1] <- paste(txt[i-1],txt[i],sep=" ")
        }
        txt <- txt[-splited]
    }
    ## end correction

    ## reevaluate pop index
    pop.idx <- grep("^([[:space:]]*)POP([[:space:]]*)$",toupper(txt))

    txt[length(txt)+1] <- "POP"
    nind.bypop <- diff(grep("^([[:space:]]*)POP([[:space:]]*)$",toupper(txt)))-1
    pop <- factor(rep(1:npop,nind.bypop))

    txt <- txt[-c(pop.idx,length(txt))]

    temp <- sapply(1:length(txt),function(i) strsplit(txt[i],","))
    ## temp is a list with nind elements, first being ind. name and 2nd, genotype

    ind.names <- vapply(temp, function(e) e[1], character(1))
    ind.names <- trimws(ind.names)
    ## individuals' name are now clean

    vec.genot <- vapply(temp, function(e) e[2], character(1))
    vec.genot <- trimws(vec.genot)

    ## X is a individual x locus genotypes matrix
    X <- matrix(unlist(strsplit(vec.genot,"[[:space:]]+")),ncol=nloc,byrow=TRUE)

    # If there are any duplicate names, make them unique and issue a warning. Else
    # use existing individual names.
    if (any(duplicated(ind.names))) {
      rownames(X) <- .genlab("", nrow(X))
      warning("Duplicate individual names detected. Coercing them to be unique.")
    } else {
      rownames(X) <- ind.names
    }
    colnames(X) <- loc.names

    ## give right pop names
    ## beware: genepop takes the name of the last individual of a sample as this sample's name
    pop.names.idx <- cumsum(table(pop))
    pop.names <- ind.names[pop.names.idx]
    levels(pop) <- pop.names

    ## check that data are consistent with NCODE and ploidy=2
    if(!all(unique(nchar(X))==(ncode*2))) stop(paste("some alleles are not encoded with", ncode,
                                                     "characters\nCheck 'ncode' argument"))

    res <- df2genind(X=X, pop=as.character(pop), ploidy=2, ncode=ncode, NA.char=NA.char)
    res@call <- prevcall

    if(!quiet) cat("\n...done.\n\n")

    return(res)

} # end read.genepop





############################
## Function read.structure
############################
#' Reading data from STRUCTURE
#'
#' The function \code{read.structure} reads STRUCTURE data files (.str ou
#' .stru) and convert them into a \linkS4class{genind} object. By default, this
#' function is interactive and asks a few questions about data content. This
#' can be disabled (for optional questions) by turning the 'ask' argument to
#' FALSE. However, one has to know the number of genotypes, of markers and if
#' genotypes are coded on a single or on two rows before importing data.
#'
#' Note: \code{read.structure} is meant for DIPLOID DATA ONLY. Haploid data
#' with the STRUCTURE format can easily be read into R using \code{read.table}
#' or \code{read.csv} and then converted using \code{\link{df2genind}}.
#'
#' @param file a character string giving the path to the file to convert, with
#' the appropriate extension.
#' @param n.ind an integer giving the number of genotypes (or 'individuals') in
#' the dataset
#' @param n.loc an integer giving the number of markers in the dataset
#' @param onerowperind a STRUCTURE coding option: are genotypes coded on a
#' single row (TRUE), or on two rows (FALSE, default)
#' @param col.lab an integer giving the index of the column containing labels
#' of genotypes. '0' if absent.
#' @param col.pop an integer giving the index of the column containing
#' population to which genotypes belong. '0' if absent.
#' @param col.others an vector of integers giving the indexes of the columns
#' containing other informations to be read. Will be available in @@other of the
#' created object.
#' @param row.marknames an integer giving the index of the row containing the
#' names of the markers. '0' if absent.
#' @param NA.char the character string coding missing data. "-9" by default.
#' Note that in any case, series of zero (like "000") are interpreted as NA
#' too.
#' @param pop an optional factor giving the population of each individual.
#' @param sep a character string used as separator between alleles.
#' @param ask a logical specifying if the function should ask for optional
#' informations about the dataset (TRUE, default), or try to be as quiet as
#' possible (FALSE).
#' @param quiet logical stating whether a conversion message must be printed
#' (TRUE,default) or not (FALSE).
#' @return an object of the class \code{genind}
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#' @seealso \code{\link{import2genind}}, \code{\link{df2genind}},
#' \code{\link{read.fstat}}, \code{\link{read.genetix}},
#' \code{\link{read.genepop}}
#' @references Pritchard, J.; Stephens, M. & Donnelly, P. (2000) Inference of
#' population structure using multilocus genotype data. \emph{Genetics},
#' \bold{155}: 945-959
#' @keywords manip
#' @examples
#'
#' obj <- read.structure(system.file("files/nancycats.str",package="adegenet"),
#'   onerowperind=FALSE, n.ind=237, n.loc=9, col.lab=1, col.pop=2, ask=FALSE)
#'
#' obj
#'
#' @export read.structure
read.structure <- function(file, n.ind=NULL, n.loc=NULL,  onerowperind=NULL,
                           col.lab=NULL, col.pop=NULL, col.others=NULL,
                           row.marknames=NULL, NA.char="-9", pop=NULL,
                           sep=NULL, ask=TRUE, quiet=FALSE){

    ## if(!file.exists(file)) stop("Specified file does not exist.") <- not needed
    if(!toupper(.readExt(file)) %in% c("STR","STRU")) stop("File extension .stru expected")

    ## set defaults for non optional arguments without default values
    if(!ask){
        if(is.null(col.lab)) col.lab <- as.integer(0)
        if(is.null(col.pop)) col.pop <- as.integer(0)
        if(is.null(row.marknames)) row.marknames <- as.integer(0)
    }

    ## required questions
    if(is.null(n.ind)){
        cat("\n How many genotypes are there? ")
        n.ind <- as.integer(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    if(is.null(n.loc)){
        cat("\n How many markers are there? ")
        n.loc <- as.integer(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    if(is.null(col.lab)){
        cat("\n Which column contains labels for genotypes ('0' if absent)? ")
        col.lab <- as.integer(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    if(is.null(col.pop)){
        cat("\n Which column contains the population factor ('0' if absent)? ")
        col.pop <- as.integer(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    if(is.null(col.others) & ask){
        cat("\n Which other optional columns should be read (press 'return' when done)? ")
        col.others <- scan(quiet=TRUE)
        if(length(col.others) == 0)  col.others <- NULL
    }

    if(is.null(row.marknames)){
        cat("\n Which row contains the marker names ('0' if absent)? ")
        row.marknames <- as.integer(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    if(is.null(onerowperind)){
        cat("\n Are genotypes coded by a single row (y/n)? ")
        onerowperind <- toupper(readLines(con = getOption('adegenet.testcon'), n = 1))
        if(onerowperind == "Y") {
            onerowperind <- TRUE
        } else {
            onerowperind <- FALSE
        }
    }

    if(is.null(NA.char)){
        cat("\n What is the code for missing data (default is '-9')? ")
        NA.char <- as.character(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    ## message to console
    if(!quiet) cat("\n Converting data from a STRUCTURE .stru file to a genind object... \n\n")

    ## read the file
    txt <- scan(file,sep="\n",what="character",quiet=TRUE)

    ## remove empty lines and spaces/tabs at the end of a line
    temp <- grep("^[[:space:]]*$",txt)
    if(length(temp) > 0) {
        txt <- txt[-temp]
    }

    txt <- gsub("([[:blank:]]+)$","",txt)

    ## isolate each useful component of the file
    ## matrix of data
    if(onerowperind) {
        n <- n.ind
        p <- 2*n.loc
    } else{
        n <- 2*n.ind
        p <- n.loc
    }

    lastline <- length(txt)
    mat <- txt[(lastline-n+1):lastline]
    mat <- t(as.data.frame(strsplit(mat,"[[:blank:]]+")))
    rownames(mat) <- 1:n
    gen <- mat[, (ncol(mat)-p+1):ncol(mat)]


    ## markers names
    if(row.marknames != 0) {
        loc.names <- trimws(txt[row.marknames])
        loc.names <- unlist(strsplit(loc.names,"[[:blank:]]+"))
    } else {
        loc.names <- .genlab("L",n.loc)
    }

    ## genotypes labels
    if(col.lab !=0) {
        ind.names <- mat[, col.lab]
    } else {
        ind.names <- .genlab("",n.ind)
    }

    ## population factor
    if(col.pop !=0) {
        pop <- as.character(mat[, col.pop])
    } else {
        pop <- NULL
    }

    ## other variables
    if(!is.null(col.others)){
        X.other <- mat[,col.others]
    }

    ## transformations if onerowperind is FALSE
    if(!onerowperind) {
        temp <- seq(1,n,by=2)
        ind.names <- ind.names[temp]
        if(length(ind.names) < n.ind) warning("Duplicated identifier for genotypes")
        pop <- pop[temp]
        if(exists("X.other")) X.other <- X.other[temp]

        ## make sur that all strings in gen have the same number of characters
        ncode <- max(nchar(gen))
        keepCheck <- any(nchar(gen) < ncode)

        while(keepCheck){
            mat0 <- matrix("", ncol=ncol(gen), nrow=nrow(gen))
            mat0[nchar(gen) < ncode] <- "0"
            gen <-  matrix(paste(mat0, gen, sep=""), nrow=nrow(mat0))
            keepCheck <- any(nchar(gen) < ncode)
        }

        ## reorder matrix of genotypes
        X <- t(sapply(temp, function(i) paste(gen[i,],gen[i+1,],sep="") ))

    } else { # if onerowperind
        temp <- seq(1,p-1,by=2)
        X <- paste(gen[,temp] , gen[,temp+1], sep="/")
        X <- matrix(X, nrow=n.ind)
        sep <- "/"
    }

    ## replace missing values by NAs
    X <- gsub(NA.char,NA,X)
    rownames(X) <- ind.names
    colnames(X) <- loc.names

    res <- df2genind(X=X, pop=pop, ploidy=2, sep=sep, ncode=ncode)

    res@call <- match.call()

    if(exists("X.other")) {res@other <- list(X=X.other)}

    return(res)

}




#########################
## Function import2genind
#########################
#'
#' Importing data from several softwares to a genind object
#'
#' Their are several ways to import genotype data to a \linkS4class{genind}
#' object: i) from a data.frame with a given format (see
#' \code{\link{df2genind}}), ii) from a file with a recognized extension, or
#' iii) from an alignement of sequences (see \code{\link{DNAbin2genind}}).\cr
#'
#' The function \code{import2genind} detects the extension of the file given in
#' argument and seeks for an appropriate import function to create a
#' \code{genind} object.\cr Current recognized formats are :\cr - GENETIX files
#' (.gtx) \cr - Genepop files (.gen) \cr - Fstat files (.dat) \cr - STRUCTURE
#' files (.str or .stru) \cr
#'
#' Beware: same data in different formats are not expected to produce exactly
#' the same \code{genind} objects.\cr For instance, conversions made by GENETIX
#' to Fstat may change the the sorting of the genotypes; GENETIX stores
#' individual names whereas Fstat does not; Genepop chooses a sample's name
#' from the name of its last genotype; etc.
#'
#' @aliases import2genind
#' @param file a character string giving the path to the file to convert, with
#' the appropriate extension.
#' @param quiet logical stating whether a conversion message must be printed
#' (TRUE,default) or not (FALSE).
#' @param \dots other arguments passed to the appropriate 'read' function
#' (currently passed to \code{read.structure})
#' @return an object of the class \code{genind}
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#' @seealso \code{\link{import2genind}}, \code{\link{read.genetix}},
#' \code{\link{read.fstat}}, \code{\link{read.structure}},
#' \code{\link{read.genepop}}
#' @references Belkhir K., Borsa P., Chikhi L., Raufaste N. & Bonhomme F.
#' (1996-2004) GENETIX 4.05, logiciel sous Windows TM pour la genetique des
#' populations. Laboratoire Genome, Populations, Interactions, CNRS UMR 5000,
#' Universite de Montpellier II, Montpellier (France). \cr
#'
#' Pritchard, J.; Stephens, M. & Donnelly, P. (2000) Inference of population
#' structure using multilocus genotype data. \emph{Genetics}, \bold{155}:
#' 945-959
#'
#' Raymond M. & Rousset F, (1995). GENEPOP (version 1.2): population genetics
#' software for exact tests and ecumenicism. \emph{J. Heredity},
#' \bold{86}:248-249 \cr
#'
#' Fstat (version 2.9.3). Software by Jerome Goudet.
#' http://www2.unil.ch/popgen/softwares/fstat.htm\cr
#'
#' Excoffier L. & Heckel G.(2006) Computer programs for population genetics
#' data analysis: a survival guide \emph{Nature}, \bold{7}: 745-758
#' @keywords manip
#' @examples
#'
#' import2genind(system.file("files/nancycats.gtx",
#' package="adegenet"))
#'
#' import2genind(system.file("files/nancycats.dat",
#' package="adegenet"))
#'
#' import2genind(system.file("files/nancycats.gen",
#' package="adegenet"))
#'
#' import2genind(system.file("files/nancycats.str",
#' package="adegenet"), onerowperind=FALSE, n.ind=237, n.loc=9, col.lab=1, col.pop=2, ask=FALSE)
#'
import2genind <- function(file, quiet=FALSE, ...){
    ## if(!file.exists(file)) stop("Specified file does not exist.") <- not needed
    ext <- .readExt(file)
    ext <- toupper(ext)

    if(ext == "GTX")
        return(read.genetix(file,quiet=quiet))

    if(ext == "DAT")
        return(read.fstat(file, quiet=quiet))

    if(ext == "GEN")
        return(read.genepop(file, quiet=quiet, ...))

    if(ext %in% c("STR","STRU"))
        return(read.structure(file, quiet=quiet, ...))

    ## evaluated only if extension is not supported
    cat("\n File format (",ext,") not supported.\n")
    cat("\nSupported formats are:\nGENETIX (.gtx) \nFSTAT (.dat) \nGenepop (.gen)\n \nSTRUCTURE (.str)\n")

    return(invisible())
}







#######################
## Function read.snp
#######################
#' Reading Single Nucleotide Polymorphism data
#'
#' The function \code{read.snp} reads a SNP data file with extension '.snp' and
#' converts it into a \linkS4class{genlight} object. This format is devoted to
#' handle biallelic SNP only, but can accommodate massive datasets such as
#' complete genomes with considerably less memory than other formats.
#'
#' The function reads data by chunks of a few genomes (minimum 1, no maximum)
#' at a time, which allows one to read massive datasets with negligible RAM
#' requirements (albeit at a cost of computational time). The argument
#' \code{chunkSize} indicates the number of genomes read at a time. Increasing
#' this value decreases the computational time required to read data in, while
#' increasing memory requirements.
#'
#' A description of the .snp format is provided in an example file distributed
#' with adegenet (see example below).
#'
#' === The .snp format ===
#'
#' Details of the .snp format can be found in the example file distributed with
#' adegenet (see below), or on the adegenet website (type \code{adegenetWeb()}
#' in R).
#'
#' @param file a character string giving the path to the file to convert, with
#' the extension ".snp".
#' @param quiet logical stating whether a conversion messages should be printed
#' (TRUE,default) or not (FALSE).
#' @param chunkSize an integer indicating the number of genomes to be read at a
#' time; larger values require more RAM but decrease the time needed to read
#' the data.
#' @param parallel a logical indicating whether multiple cores -if available-
#' should be used for the computations (TRUE, default), or not (FALSE);
#' requires the package \code{parallel} to be installed (see details).
#' @param n.cores if \code{parallel} is TRUE, the number of cores to be used in
#' the computations; if NULL, then the maximum number of cores available on the
#' computer is used.
#' @param \dots other arguments to be passed to other functions - currently not
#' used.
#' @return an object of the class \code{"\linkS4class{genlight}"}
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#' @seealso - \code{?genlight} for a description of the class
#' \code{"\linkS4class{genlight}"}.
#'
#' - \code{\link{read.PLINK}}: read SNPs in PLINK's '.raw' format.
#'
#' - \code{\link{fasta2genlight}}: extract SNPs from alignments with fasta
#' format.
#'
#' - \code{\link{df2genind}}: convert any multiallelic markers into adegenet
#' \code{"\linkS4class{genlight}"}.
#'
#' - \code{\link{import2genind}}: read multiallelic markers from various
#' software into adegenet.\cr
#' @keywords manip
#' @examples
#'
#' \dontrun{
#' ## show the example file ##
#' ## this is the path to the file:
#' system.file("files/exampleSnpDat.snp",package="adegenet")
#'
#' ## show its content:
#' file.show(system.file("files/exampleSnpDat.snp",package="adegenet"))
#'
#'
#' ## read the file
#' obj <-
#' read.snp(system.file("files/exampleSnpDat.snp",package="adegenet"), chunk=2)
#' obj
#' as.matrix(obj)
#' ploidy(obj)
#' alleles(obj)
#' locNames(obj)
#' }
#'
#' @export read.snp
#'
read.snp <- function(file, quiet=FALSE, chunkSize=1000,
                     parallel=FALSE, n.cores=NULL, ...){
    ext <- .readExt(file)
    ext <- toupper(ext)
    if(ext != "SNP") warning("wrong file extension - '.snp' expected")
    if(!quiet) cat("\n Reading biallelic SNP data file into a genlight object... \n\n")
    ## if(parallel && !require(parallel)) stop("parallel package requested but not installed")
    if(parallel && is.null(n.cores)){
        n.cores <- parallel::detectCores()
    }

    call <- match.call()

    ## HANDLE THE COMMENTS ##
    if(!quiet) cat("\n Reading comments... \n")

    count <- 0L
    i <- 0
    while(count < 2L){
        txt <- scan(file,what="character",sep="\n",quiet=TRUE, skip=i, nmax=1, blank.lines.skip=FALSE)
        if(length(grep(">>>>", txt)>0)){
            count <- count + 1L
        }
        i <- i+1
        if(count==0L && i>10){
            warning("No comment section at the beginning of the file. Format may be wrong.")
            i <- 0
            break
        }
    }

    lines.to.skip <- i


    ## READ GENERAL DATA (>>) ##
    if(!quiet) cat("\n Reading general information... \n")

    misc.info <- list()

    txt <- scan(file,what="character",sep="\n",quiet=TRUE, skip=lines.to.skip, nmax=1)

    while(length(grep(">>", txt))>0){
        itemName <- gsub(">>","", txt)
        itemName <- gsub("(^[[:space:]]+)|([[:space:]]+$)", "", itemName)
        misc.info[itemName] <- scan(file,what="character",sep="\n",quiet=TRUE, skip=lines.to.skip + 1, nmax=1)
        lines.to.skip <-lines.to.skip + 2
        txt <- scan(file,what="character",sep="\n",quiet=TRUE, skip=lines.to.skip, nmax=1)
    }

    ## transform each character string into a vector
    misc.info <- sapply(misc.info, function(e) unlist(strsplit(e,"[[:space:]]+")))

    ## READ GENOTYPE DATA ##
    ## one genotype is read/converted at a time to spare RAM
    if(!quiet) cat("\n Reading",ifelse(is.null(misc.info$population),"",length(misc.info$population)), "genotypes... \n")


    res <- list() # this will be a list of SNPbin objects

    txt <- scan(file,what="character",sep="\n",quiet=TRUE, skip=lines.to.skip, nmax=chunkSize*2)

    ID.INDIV <- grep(">", txt)

    COUNT <- 0 # used to count the nb reads

    while(length(ID.INDIV)>0){
        COUNT <- COUNT + 1
        if(!quiet) {
            if(COUNT %% 5 == 0){
                cat(length(res)+length(ID.INDIV))
            } else {
                cat(".")
            }
        }

        ind.lab <- gsub(">","", txt[ID.INDIV])
        ind.lab <- gsub("(^[[:space:]]+)|([[:space:]]+$)", "", ind.lab)
        temp <- strsplit(txt[ID.INDIV+1], "")
        temp <- lapply(temp, function(e) suppressWarnings(as.integer(e)))
        if(parallel){
            res <- c(res, parallel::mclapply(temp, function(e) new("SNPbin", e),
                                   mc.cores=n.cores, mc.silent=TRUE, mc.cleanup=TRUE, mc.preschedule=FALSE) )
        } else {
            res <- c(res, lapply(temp, function(e) new("SNPbin", e)) )
        }
        names(res)[(length(res)-length(ID.INDIV)+1):length(res)] <- ind.lab
        lines.to.skip <-lines.to.skip + length(txt)
        txt <- scan(file,what="character",sep="\n",quiet=TRUE, skip=lines.to.skip, nmax=chunkSize*2)
        ID.INDIV <- grep(">", txt)
    }


    ## CHECK CONSISTENCY ##
    if(!quiet) cat("\n Checking consistency... \n")

    n.loc <- unique(sapply(res, nLoc))
    n.ind <- length(res)
    other <- list()

    if(length(n.loc)>1) {
        print(n.loc)
        warning("!!! Differing numbers of loci detected between individuals !!!")
    }

    if(!is.null(misc.info$position) && length(misc.info$position)!=n.loc) {
        other$position <- misc.info$position
        misc.info$position <- NULL
        warning("vector of positions of the SNPs does not match the number of SNPs - storing this information in @other")
    }
    if(!is.null(misc.info$allele) && length(misc.info$allele)!=n.loc) {
        other$allele <- misc.info$allele
        misc.info$allele <- NULL
        warning("vector of alleles of the SNPs does not match the number of SNPs - storing this information in @other")
    }
    if(!is.null(misc.info$chromosome) && length(misc.info$chromosome)!=n.loc) {
        other$chromosome <- misc.info$chromosome
        misc.info$chromosome <- NULL
        warning("vector of chromosomes of the SNPs does not match the number of SNPs - storing this information in @other")
    }
    if(!is.null(misc.info$population) && length(misc.info$population)!=n.ind) {
        other$population <- misc.info$population
        misc.info$population <- NULL
        warning("vector of population of the individuals does not match the number of individuals - storing this information in @other")
    }
    if(!is.null(misc.info$ploidy) && length(misc.info$ploidy)>1 && length(misc.info$ploidy)!=n.ind) {
        other$ploidy <- misc.info$ploidy
        misc.info$ploidy <- NULL
        warning("vector of ploidy of the individuals has more than one value but does not match the number of individuals - storing this information in @other")
    }


    ## BUILD OUTPUT ##
    if(!quiet) cat("\n Building final object... \n")

    ind.names <- names(res)
    if(!is.null(misc.info$chromosome)){
        other <- list(chromosome = misc.info$chromosome)
    }

    res <- new("genlight", gen=res, ind.names=ind.names, position=misc.info$position, loc.all=misc.info$allele, ploidy=misc.info$ploidy, pop=misc.info$population, other=other, parallel=parallel)

    if(!quiet) cat("\n...done.\n\n")

    return(res)

} # end read.snp







####################
## extract.PLINKmap
####################
#' @export
#' @rdname read.PLINK
#' @aliases extract.PLINKmap
#'
#' @param x an optional object of the class \code{"\linkS4class{genlight}"}, in which
#' the information read is stored; if provided, information is matched against
#' the names of the loci in \code{x}, as returned by \code{locNames(x)}; if not
#' provided, a list of two components is returned, containing chromosome and
#' position information.
#'
extract.PLINKmap <- function(file, x = NULL){
    ## CHECK EXTENSION ##
    ext <- .readExt(file)
    ext <- toupper(ext)
    if(ext != "MAP") warning("wrong map.file extension - '.map' expected")


    ## READ FILE ##
    ## find nb of columns
    txt <- scan(file, what = "character", sep = "\n", quiet = TRUE,  nlines = 1)
    nb.col <- length( unlist(strsplit(txt,"[[:blank:]]+")))

    ## read file
    txt <- scan(file, what = "character", sep= "\n", quiet = TRUE)
    txt <- unlist(strsplit(as.vector(txt), split = "[[:blank:]]"))
    txt <- matrix(txt, ncol=4, byrow=TRUE)


    ## EXTRACT INFO AND RETURN OBJECT ##
    ## return a genlight
    if (!is.null(x)) {
        if(!inherits(x, "genlight")) stop("x is not a genlight object")

        ## match data: we need remove the potential alleles added to locus names
        marker_id <- sub("_.*$", "", locNames(x))
        ord <- match(marker_id, txt[,2])

        chromosome(x) <- factor(txt[ord, 1])
        position(x) <- as.integer(txt[ord, 4])

        return(x)
    }

    ## return a list
    res <- list(chromosome = factor(txt[ord, 1]),
                position = as.integer(txt[ord, 4]))

    return(res)
} # end extract.PLINKmap







########################
## Function read.PLINK
########################
#' Reading PLINK Single Nucleotide Polymorphism data
#'
#' The function \code{read.PLINK} reads a data file exported by the PLINK
#' software with extension '.raw' and converts it into a \code{"\linkS4class{genlight}"}
#' object. Optionally, information about SNPs can be read from a ".map" file,
#' either by specifying the argument \code{map.file} in \code{read.PLINK}, or
#' using \code{extract.PLINKmap} to add information to an existing
#' \code{"\linkS4class{genlight}"} object.
#'
#' The function reads data by chunks of several genomes (minimum 1, no maximum)
#' at a time, which allows one to read massive datasets with negligible RAM
#' requirements (albeit at a cost of computational time). The argument
#' \code{chunkSize} indicates the number of genomes read at a time. Increasing
#' this value decreases the computational time required to read data in, while
#' increasing memory requirements.
#'
#' See details for the documentation about how to export data using PLINK to
#' the '.raw' format.
#'
#' === Exporting data from PLINK ===
#'
#' Data need to be exported from PLINK using the option "--recodeA" (and NOT
#' "--recodeAD"). The PLINK command should therefore look like: \code{plink
#' --file data --recodeA}. For more information on this topic, please look at
#' this webpage: \url{http://zzz.bwh.harvard.edu/plink/}
#'
#' @aliases read.PLINK read.plink
#' @param file for \code{read.PLINK} a character string giving the path to the
#' file to convert, with the extension ".raw"; for \code{extract.PLINKmap}, a
#' character string giving the path to a file with extension ".map".
#' @param map.file an optional character string indicating the path to a ".map"
#' file, which contains information about the SNPs (chromosome, position). If
#' provided, this information is processed by \code{extract.PLINKmap} and
#' stored in the \code{@@other} slot.
#' @param quiet logical stating whether a conversion messages should be printed
#' (TRUE,default) or not (FALSE).
#' @param chunkSize an integer indicating the number of genomes to be read at a
#' time; larger values require more RAM but decrease the time needed to read
#' the data.
#' @param parallel a logical indicating whether multiple cores -if available-
#' should be used for the computations (TRUE, default), or not (FALSE);
#' requires the package \code{parallel} to be installed (see details).
#' @param n.cores if \code{parallel} is TRUE, the number of cores to be used in
#' the computations; if NULL, then the maximum number of cores available on the
#' computer is used.
#' @param \dots other arguments to be passed to other functions - currently not
#' used.
#'
#' @return - read.PLINK: an object of the class \code{"\linkS4class{genlight}"}
#'
#' - extract.PLINKmap: if a \code{"\linkS4class{genlight}"} is provided as argument
#' \code{x}, this object incorporating the new information about SNPs in the
#' \code{@@other} slot (with new components 'chromosome' and 'position');
#' otherwise, a list with two components containing chromosome and position
#' information.
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#' @seealso - \code{?genlight} for a description of the class
#' \code{"\linkS4class{genlight}"}.
#'
#' - \code{\link{read.snp}}: read SNPs in adegenet's '.snp' format.
#'
#' - \code{\link{fasta2genlight}}: extract SNPs from alignments with fasta
#' format.
#'
#' - other import function in adegenet: \code{\link{import2genind}},
#' \code{\link{df2genind}}, \code{\link{read.genetix}}
#' \code{\link{read.fstat}}, \code{\link{read.structure}},
#' \code{\link{read.genepop}}.
#'
#' - another function \code{read.plink} is available in the package
#' \code{snpMatrix}.
#' @keywords manip
#' @export
#' @rdname read.PLINK
read.PLINK <- function(file, map.file=NULL, quiet=FALSE, chunkSize=1000,
                       parallel=require("parallel"), n.cores=NULL, ...){
    ## HANDLE ARGUMENTS ##
    ext <- .readExt(file)
    ext <- toupper(ext)
    if(ext != "RAW") warning("wrong file extension - '.raw' expected")
    if(!quiet) cat("\n Reading PLINK raw format into a genlight object... \n\n")
    ## if(parallel && !require(parallel)) stop("parallel package requested but not installed")
    if(parallel && is.null(n.cores)){
        n.cores <- parallel::detectCores()
    }


    ## READ NAMES OF LOCI ##
    if(!quiet) cat("\n Reading loci information... \n")

    loc.names <- scan(file,what="character",sep=" ",quiet=TRUE,  nlines=1, blank.lines.skip=FALSE)
    n.loc <- length(loc.names) - 6
    misc.info <- lapply(1:6,function(i) NULL)
    names(misc.info) <- loc.names[1:6]
    loc.names <- loc.names[7:length(loc.names)]
    loc.names <- gsub("_[1-9]$","",loc.names)

    ## READ GENOTYPES ##
    if(!quiet) cat("\n Reading and converting genotypes... \n")

    res <- list() # this will be a list of SNPbin objects

    ## initialize reading
    lines.to.skip <- 1
    txt <- scan(file,what="character",sep="\n",quiet=TRUE, skip=lines.to.skip, nmax=chunkSize)
    txt <- lapply(txt, function(e) unlist(strsplit(e,"[[:blank:]]+") ))

    COUNT <- 0 # used to count the nb reads

    while(length(txt)>0){
        COUNT <- COUNT + 1
        if(!quiet) {
            if(COUNT %% 5 == 0){
                cat(length(res)+length(txt))
            } else {
                cat(".")
            }
        }


        ## handle misc info
        temp <- lapply(txt, function(e) e[1:6])
        for(i in 1:6){
            misc.info[[i]] <- c(misc.info[[i]], unlist(lapply(temp, function(e) e[[i]])) )
        }


        ## build SNPbin objects
        txt <- lapply(txt, function(e) suppressWarnings(as.integer(e[-(1:6)])))

        if(parallel){
            res <- c(res, parallel::mclapply(txt, function(e) new("SNPbin", snp=e, ploidy=2L),
                                   mc.cores=n.cores, mc.silent=TRUE, mc.cleanup=TRUE, mc.preschedule=FALSE) )
        } else {
            res <- c(res, lapply(txt, function(e) new("SNPbin", snp=e, ploidy=2L)) )
        }

        lines.to.skip <-lines.to.skip + length(txt)

        ## read lines
        txt <- scan(file,what="character",sep="\n",quiet=TRUE, skip=lines.to.skip, nmax=chunkSize)
        txt <- lapply(txt, function(e) unlist(strsplit(e,"[[:blank:]]+") ))
    }


    ## MAKE A FEW CHECKS ##
    if(!all(sapply(res, nLoc)==n.loc)) stop(paste("some individuals do not have",n.loc,"SNPs."))


    ## BUILD FINAL OBJECT ##
    if(!quiet) cat("\n Building final object... \n")

    res <- new("genlight",res, ploidy=2L, parallel=parallel)
    indNames(res) <- misc.info$IID
    pop(res) <- misc.info$FID
    locNames(res) <- loc.names
    misc.info <- misc.info[c("SEX", "PHENOTYPE", "PAT","MAT")]
    names(misc.info) <- tolower(names(misc.info))
    misc.info$sex[misc.info$sex==1] <- "m"
    misc.info$sex[misc.info$sex==2] <- "f"
    misc.info$sex <- factor(misc.info$sex)
    misc.info$phenotype[misc.info$phenotype==1] <- "control"
    misc.info$phenotype[misc.info$phenotype==2] <- "case"
    misc.info$phenotype <- factor(misc.info$phenotype)
    other(res) <- misc.info


    ## HANDLE MAP FILE INFO ##
    if(!is.null(map.file)){
        res <- extract.PLINKmap(map.file, res)
    }


    ## RETURN OUTPUT ##
    if(!quiet) cat("\n...done.\n\n")

    return(res)
} # end read.PLINK







###########################
## Function fasta2genlight
###########################
fasta2genlight <- function(file, quiet=FALSE, chunkSize=1000, saveNbAlleles=FALSE,
                       parallel=require("parallel"), n.cores=NULL, ...){
    ## HANDLE ARGUMENTS ##
    ext <- .readExt(file)
    ext <- toupper(ext)
    if(!ext %in% c("FASTA", "FA", "FAS")) warning("wrong file extension - '.fasta', '.fa' or '.fas' expected")
    if(!quiet) cat("\n Converting FASTA alignment into a genlight object... \n\n")
    ## if(parallel && !require(parallel)) stop("parallel package requested but not installed")
    if(parallel && is.null(n.cores)){
        n.cores <- parallel::detectCores()
    }


    ## PRIOR CHECKS ##
    ## find nb of lines per genome
    lines.to.skip <- 0
    txt <- scan(file,what="character",sep="\n",quiet=TRUE, nmax=1)

    while(length(grep("^>.+", txt))<2){
        lines.to.skip <- lines.to.skip + 1
        txt <- scan(file,what="character",sep="\n",quiet=TRUE, nmax=lines.to.skip)
    }

    LINES.PER.IND <- lines.to.skip-1


    ## find length of a genome
    NLOC <- sum(nchar(txt[2:LINES.PER.IND]))


    ## SCAN ALL POSITIONS AND IDENTIFY SNPs ##
    if(!quiet) cat("\n Looking for polymorphic positions... \n")

    ## read all genomes by chunks
    ## initialize
    lines.to.skip <- 0
    IND.LAB <- NULL
    POOL <- as.list(rep("-", NLOC))
    COUNT <- 0 # used to count the nb reads

    txt <- scan(file,what="character",sep="\n",quiet=TRUE, skip=lines.to.skip, nmax=LINES.PER.IND*chunkSize)

    ## read and process chunks
    while(length(txt)>0){
        COUNT <- COUNT + 1
        if(!quiet) {
            for(i in 1:(COUNT*chunkSize)) cat(".")
        }


        nb.ind <- length(grep("^>", txt))
        IND.LAB <- c(IND.LAB, sub(">","",txt[grep("^>", txt)])) # find individuals' labels
        txt <- split(txt, rep(1:nb.ind, each=LINES.PER.IND)) # split per individuals
        if(parallel){
            txt <- parallel::mclapply(txt, function(e) strsplit(paste(e[-1], collapse=""), split=""),
                            mc.cores=n.cores, mc.silent=TRUE, mc.cleanup=TRUE, mc.preschedule=FALSE) # each genome -> one vector
        } else {
            txt <- lapply(txt, function(e) strsplit(paste(e[-1], collapse=""), split="")) # each genome -> one vector
        }

        ## POOL contains all alleles of each position
        temp <- as.list(apply(matrix(unlist(txt), byrow=TRUE, nrow=length(txt)),2,unique)) # alleles current genomes
        POOL <- mapply(function(x,y) unique(c(x,y)), POOL, temp, SIMPLIFY=FALSE) # update global pool

        lines.to.skip <- lines.to.skip + nb.ind*LINES.PER.IND
        txt <- scan(file,what="character",sep="\n",quiet=TRUE, skip=lines.to.skip, nmax=LINES.PER.IND*chunkSize)
    }


    ## analyse pool of alleles
    letterOK <- c("a","g","c","t","A","G","C","T")
    POOL <- lapply(POOL, function(e) e[e %in% letterOK]) # keep only proper letters
    ## POOL <- lapply(POOL, setdiff, "-")
    nb.alleles <- sapply(POOL, length)
    snp.posi <- nb.alleles==2
    if(all(!snp.posi)){
        warning("No polymorphism in the alignment - returning empty object")
        return(new("genlight"))
    }
    sec.all <- unlist(lapply(POOL[snp.posi], function(e) e[2]))



    ## RE-READ DATA, CONVERT SNPs TO GENLIGHT ##
    if(!quiet) cat("\n Extracting SNPs from the alignment... \n")

    ## initialize
    lines.to.skip <- 0
    COUNT <- 0 # used to count the nb reads
    res <- list()

    txt <- scan(file,what="character",sep="\n",quiet=TRUE, skip=lines.to.skip, nmax=LINES.PER.IND*chunkSize)

    ## returns a vector of nb of second alleles or NAs
    f1 <- function(vec){
        out <- as.integer(vec==sec.all)
        out[!vec %in% letterOK] <- NA
        return(out)
    }

    ## read and process chunks
    while(length(txt)>0){
        COUNT <- COUNT + 1
        if(!quiet) {
            for(i in 1:(COUNT*chunkSize)) cat(".")
        }


        ## read SNPs
        nb.ind <- length(grep("^>", txt))
        txt <- split(txt, rep(1:nb.ind, each=LINES.PER.IND)) # split per individuals
        if(parallel){
            txt <- parallel::mclapply(txt, function(e) strsplit(paste(e[-1], collapse=""), split="")[[1]][snp.posi],
                                        mc.cores=n.cores, mc.silent=TRUE, mc.cleanup=TRUE, mc.preschedule=FALSE) # each genome -> one SNP vector
        } else {
            txt <- lapply(txt, function(e) strsplit(paste(e[-1], collapse=""), split="")[[1]][snp.posi]) # each genome -> one SNP vector
        }

        ## convert to genlight
        ##res <- c(res, lapply(txt, function(e) new("SNPbin", as.integer(e==sec.all))))
        res <- c(res, lapply(txt, function(e) new("SNPbin", f1(e))))

        lines.to.skip <- lines.to.skip + nb.ind*LINES.PER.IND
        txt <- scan(file,what="character",sep="\n",quiet=TRUE, skip=lines.to.skip, nmax=LINES.PER.IND*chunkSize)
    }



    ## BUILD FINAL OBJECT ##
    if(!quiet) cat("\n Building final object... \n")

    res <- new("genlight",res, ploidy=1, parallel=parallel)
    indNames(res) <- IND.LAB
    alleles(res) <- sapply(POOL[snp.posi], paste, collapse="/")
    position(res) <- which(snp.posi)
    if(saveNbAlleles) other(res) <- list(nb.all.per.loc=nb.alleles)


    ## RETURN OUTPUT ##
    if(!quiet) cat("\n...done.\n\n")

    return(res)
} # end fasta2genlight








###########################
## Function fasta2DNAbin
###########################
fasta2DNAbin <- function(file, quiet=FALSE, chunkSize=10, snpOnly=FALSE){

    ## HANDLE ARGUMENTS ##
    if (!is(file, "connection")) {
        ext <- .readExt(file)
        ext <- toupper(ext)
        if(!ext %in% c("FASTA", "FA", "FAS")) warning("wrong file extension - '.fasta', '.fa' or '.fas' expected")
    }
    if(!quiet) cat("\n Converting FASTA alignment into a DNAbin object... \n\n")


    ## PRIOR CHECKS ##
    ## find nb of lines per genome ##

    ## find length of a single line of sequence
    if(!quiet) cat("\n Finding the size of a single genome... \n\n")
    lines.to.skip <- 0
    txt <- scan(file,what="character",sep="\n",quiet=TRUE, nmax=1)

    while(length(grep("^>.+", txt))==1){
        lines.to.skip <- lines.to.skip + 1
        txt <- scan(file,what="character",sep="\n",quiet=TRUE, skip=lines.to.skip, n=1)
    }

    char.per.line <- nchar(txt)
    nbOfLinesToRead <- max(round(1e6/char.per.line),1)#

    ## read file to find genome size, 1e6 characters at a time
    nblocks <- 1
    txt <- scan(file,what="character",sep="\n",quiet=TRUE, n=nbOfLinesToRead*nblocks)

    while(length(grep("^>.+", txt))<2){
        nblocks <- nblocks+1
        txt <- scan(file,what="character",sep="\n",quiet=TRUE, n=nbOfLinesToRead*nblocks)
    }

    ## this is the nb of lines for one genome
    ## including the first line of annotation
    LINES.PER.IND <- diff(grep("^>.+", txt))[1]

    ## this is the length of a genome single
    GENOMESIZE <- sum(nchar(txt[2:LINES.PER.IND]))
    if(!quiet) cat("\n genome size is:", format(GENOMESIZE, big.mark=","), "nucleotides \n")
    if(!quiet) cat("\n(",format(LINES.PER.IND, big.mark=","), " lines per genome )\n")


    ## START READING / CONVERTING GENOMES ##
    if(!quiet) cat("\n Importing sequences... \n")

    ## read all genomes by chunks
    ## initialize
    lines.to.skip <- 0
    IND.LAB <- NULL
    COUNT <- 0 # used to count the nb reads

    txt <- scan(file,what="character",sep="\n",quiet=TRUE, skip=lines.to.skip, n=LINES.PER.IND*chunkSize)
    res <- raw()

    ## read and process chunks
    while(length(txt)>0){
        ## clean memory ##
        invisible(gc())

        ## progression... ##
        COUNT <- COUNT + 1
        if(!quiet) {
            for(i in 1:(COUNT*chunkSize)) cat(".")
        }

        ## process txt ##
        nb.ind <- length(grep("^>", txt))
        IND.LAB <- c(IND.LAB, sub(">","",txt[grep("^>", txt)])) # find individuals' labels
        txt <- split(txt, rep(1:nb.ind, each=LINES.PER.IND)) # split per individuals
        txt <- lapply(txt, function(e) unlist(strsplit(tolower(paste(e[-1], collapse="")), split=""))) # each genome -> one vector

        ## convert character vectors to DNAbin output
        res <- c(res, unlist(lapply(txt, as.DNAbin)))

        ## ## POOL contains all alleles of each position
        ## temp <- as.list(apply(matrix(unlist(txt), byrow=TRUE, nrow=length(txt)),2,unique)) # alleles current genomes
        ## POOL <- mapply(function(x,y) unique(c(x,y)), POOL, temp, SIMPLIFY=FALSE) # update global pool

        ## scan file further ##
        lines.to.skip <- lines.to.skip + nb.ind*LINES.PER.IND
        txt <- scan(file,what="character",sep="\n",quiet=TRUE, skip=lines.to.skip, n=LINES.PER.IND*chunkSize)
    }

    ## GET FINAL OBJECT ##
    if(!quiet) cat("\n Forming final object... \n")

    ## form matrix ##
    res <- matrix(res, nrow=length(IND.LAB), byrow=TRUE)
    class(res) <- "DNAbin"
    rownames(res) <- IND.LAB

    ## extract snps if needed ##
    if(snpOnly){
        if(!quiet) cat("\n Extracting SNPs... \n")
        snp.posi <- seg.sites(res)
        if(length(snp.posi)==0) warning("no polymorphic site in the sequences")
        res <- res[,seg.sites(res),drop=FALSE]
        colnames(res) <- snp.posi
    }

    ## RETURN OUTPUT ##
    if(!quiet) cat("\n...done.\n\n")

    return(res)
} # end fasta2DNAbin

context("Import Tests")

test_that("df2genind works with haploids", {
  skip_on_cran()
  x <- matrix(as.character(sample(20)), nrow = 10, ncol = 2)
  res <- df2genind(x, ploidy = 1)
  expect_equal(sum(res@loc.n.all), 20)
  expect_equal(nInd(res), 10)
  expect_equal(nLoc(res), 2)
  resdf <- genind2df(res)
  expect_equal(unlist(resdf, use.names=FALSE), as.vector(x))
})

test_that("df2genind makes sense for given example", {
  skip_on_cran()
  df <- data.frame(locusA=c("11","11","12","32"),
                   locusB=c(NA,"34","55","15"),
                   locusC=c("22","22","21","22"))
  row.names(df) <- .genlab("genotype",4)
  obj <- df2genind(df, ploidy=2, ncode=1)
  expect_equal(nInd(obj), 4)
  expect_equal(nLoc(obj), 3)
  expect_equal(locNames(obj), colnames(df))
  expect_equal(indNames(obj), rownames(df))
  expect_equivalent(obj@loc.n.all, c(3, 4, 2))
  objdf <- genind2df(obj)
  expect_equivalent(df, df)
})

test_that("df2genind handles NAs for 'numerically named' samples correctly", {
  skip_on_cran()

  df <- read.table(text = "
AnimalID,Samp,INRA21,AHT137,REN169D01,AHTk253
730,AX.0CCE,092 098,132 132,NA,284 286
498,AP.07P4,092 092,124 142,204 208,280 280
677,AP.088P,092 096,140 146,204 204,280 280
678,AP.088T,096 098,124 148,198 204,280 280
544,AP.07XM,096 098,134 146,198 198,280 286
533,AP.07UM,092 098,134 148,198 204,280 286",
                   header = TRUE, sep = ",", colClasses = rep("factor", 6))

  obj <- df2genind(X = df[, !grepl("AnimalID|Samp", colnames(df))], ind.names = df$AnimalID,
                   sep = " ", ncode = 6)
  g <- tab(obj)
  expect_equivalent(g["730", grepl("REN169D01", colnames(g))],
                    c(REN169D01.204 = NA_integer_,
                      REN169D01.208 = NA_integer_,
                      REN169D01.198 = NA_integer_))
  })

test_that("df2genind will handle duplicate samples and loci", {
  skip_on_cran()
  x <-
    "A B
  1/2 3/4
  5/6 4/5
  2/6 3/9"
  xdf <- read.table(text = x, header = TRUE, stringsAsFactors = FALSE)
  inds <- c("one", "one", "two")
  loci <- rep("double", 2)
  expect_warning(xgid <- df2genind(xdf, sep = "/", ind.names = inds, loc.names = loci))
  expect_equivalent(unique(rowSums(tab(xgid))), 4)
  expect_equivalent(genind2df(xgid, sep = "/"), xdf)
})


test_that("read.X functions work as expected", {
  skip_on_cran()
  suppressWarnings({
    gpop <- read.genepop(system.file("files/nancycats.gen",package="adegenet"), quiet = TRUE)
  })
  fsta <- read.fstat(system.file("files/nancycats.dat",package="adegenet"), quiet = TRUE)
  gntx <- read.genetix(system.file("files/nancycats.gtx",package="adegenet"), quiet = TRUE)
  stru <- read.structure(system.file("files/nancycats.str",package="adegenet"),
                         onerowperind=FALSE, n.ind=237, n.loc=9, col.lab=1,
                         col.pop=2, ask=FALSE, quiet = TRUE)
  data("nancycats", package = "adegenet")
  # Making sure that the populations are all named the same. The order of the
  # isolates are mixed up within these data.
  levels(pop(gpop)) <- levels(pop(nancycats))
  levels(pop(fsta)) <- levels(pop(nancycats))
  levels(pop(gntx)) <- levels(pop(nancycats))
  levels(pop(stru)) <- levels(pop(nancycats))

  # Ensuring that the locus and population summaries are equivalent
  summary_stats <- summary(nancycats)
  expect_equivalent(summary(gpop), summary_stats)
  expect_equivalent(summary(fsta), summary_stats)
  expect_equivalent(summary(gntx), summary_stats)
  expect_equivalent(summary(stru), summary_stats)
})

test_that("read.genpop can import duplicate names", {
  skip_on_cran()
  x <- "
  Microsat on Chiracus radioactivus, a pest species
     Loc1, Loc2, Loc3, Y-linked, Loc4
POP
AA8, 0405 0711 0304 0000      0505
AA9, 0405 0609 0208 0000      0505
A10, 0205 0609 0101 0000      0305
A11, 0405 0606 0102 0000      0504
A12, 0202 0609 0105 0000      0507
A13, 0505 0909 0107 0000      0505
A14, 0202 0609 0207 0000      0503
A15, 0405 0609 0101 0000      0505
Pop
AF, 0000 0000 0000 0000      0505
AF, 0205 0307 0102 0000      0505
AF, 0202 0609 0202 0000      0505
AF, 0205 0909 0000 0000      0505
AF, 0205 0307 0202 0000      0505
AF, 0505 0303 0102 0000      0505
AF, 0205 0700 0000 0000      0505
AF, 0505 0900 0000 0000      0405
AF, 0205 0600 0000 0000      0505
AF, 0505 0606 0202 0000      0505
pop
C45, 0505 0606 0202 0000      0505
C45, 0505 0909 0202 0000      0505
C45, 0505 0306 0202 0000      0505
C45, 0505 0909 0102 0000      0405
C45, 0205 0303 0202 0000      0505
C45, 0205 0909 0202 0000      0405
  "
  tmp <- tempfile(fileext = ".gen")
  cat(x, file = tmp)
  expect_warning(gp <- read.genepop(tmp, quiet = TRUE))
  expect_identical(indNames(gp), .genlab("", nInd(gp)))

})



test_that("df2genind can handle periods in input", {
  skip_on_cran()
  dat <-
    data.frame(
      pop = c(1, 1, 2, 2),
      loc1 = c("1/1", "1/2", "1.1/2", "2/2"),
      loc2 = c("1/1", "1/2", "1/2", "2/2")
    )
  expect_warning(datgi <- df2genind(dat[, -1], sep = "/", pop = dat[, 1]))
  expect_equivalent(alleles(datgi)$loc1, c("1", "2", "1_1"))
})

test_that("df2genind can handle periods in input with underscore separator", {
  skip_on_cran()
  dat <-
    data.frame(
      pop = c(1, 1, 2, 2),
      loc1 = c("1/1", "1/2", "1.1/2", "2/2"),
      loc2 = c("1/1", "1/2", "1/2", "2/2")
    )
  dat <- apply(dat, 2, function(i) gsub("/", "_", i))
  expect_warning(datgi <- df2genind(dat[, -1], sep = "_", pop = dat[, 1]))
  expect_equivalent(alleles(datgi)$loc1, c("1", "2", "1p1"))
})


test_that("different imports sort populations in the same way", {
    skip_on_cran()

    ## read nancycats data from different formats
  expect_output({
    x.str <- read.structure(
      system.file("files/nancycats.str", package = "adegenet"),
      onerowperind = FALSE,
      n.ind = 237,
      n.loc = 9,
      col.lab = 1,
      col.pop = 2,
      ask = FALSE
    )
  }, "Converting data")
  expect_output({
    expect_warning({
      x.gen <- read.genepop(system.file("files/nancycats.gen", package = "adegenet"))
    }, "Duplicate individual names detected.")
  }, "Converting data")
  
  expect_output({
    x.dat <- read.fstat(system.file("files/nancycats.dat", package = "adegenet"))
  }, "Converting data")
  
  expect_output({
    x.gtx <- read.genetix(system.file("files/nancycats.gtx", package = "adegenet"))
  }, "Converting data")
  
  
    ## check that the pop are identical:

    ## we use 'table(pop(...))' because individuals may be sorted differently in the files, so
    ## 'pop(...)' may be different

    identical(table(pop(x.gen)), table(pop(x.str)))
    identical(table(pop(x.gen)), table(pop(x.dat)))
    identical(table(pop(x.gen)), table(pop(x.gtx)))
})


test_that("ensure importing structure files with numbers for locus names imports correectly", {
  skip_on_cran()
  
  # Column names should have no extra characters in front of them. Your IDE
  # may be adding them, so watch out!
  tmp <- tempfile(fileext = ".stru")
  cat(
"		1_25	8_54	1358_15	1363_12	1368_57	1369_41	1372_14	1373_9	1377_42	1378_53	1379_10	1382_37	1386_27	1398_46	1400_9	1401_25	1403_13	1404_17	1409_42	1416_48	1419_11	1421_14	1423_5	1424_74	1426_55	1429_46	1432_23	1435_30	1436_7	1438_9	1443_37
A_KH1584	A	1	4	4	1	1	3	2	4	4	2	3	3	2	4	1	3	1	1	2	3	1	4	4	3	2	2	3	4	4	4	2
A_KH1584	A	1	4	4	1	1	3	2	4	4	4	3	3	4	4	1	3	1	3	2	3	3	4	4	3	4	2	3	4	4	4	2
C_KH1059	C	0	4	4	1	1	3	2	4	4	2	1	3	2	4	1	3	1	3	2	3	3	2	4	3	2	2	3	2	4	4	2
C_KH1059	C	0	4	4	1	1	3	2	4	4	4	3	3	2	4	1	3	1	3	2	3	3	4	4	3	2	2	3	4	4	4	2
M_KH1834	M	0	2	2	1	1	3	2	4	4	2	3	3	2	4	1	3	1	1	2	3	3	4	4	3	2	2	3	2	4	4	2
M_KH1834	M	0	4	4	1	3	3	2	4	4	2	3	3	2	4	1	3	1	3	2	3	3	4	4	3	2	4	3	4	4	4	2
M_KH1837	M	1	4	4	1	1	3	2	4	4	0	3	3	2	2	1	3	1	1	2	3	3	4	4	3	4	2	3	4	4	4	2
M_KH1837	M	1	4	4	1	3	3	2	4	4	0	3	3	4	4	1	3	1	3	2	3	3	4	4	3	4	2	3	4	4	4	2", 
      file = tmp)
  
  xy1 <- read.structure(
    tmp,
    NA.char = "0",
    n.ind = 4,
    n.loc = 31,
    onerowperind = FALSE,
    col.lab = 1,
    col.pop = 2,
    row.marknames = 1,
    sep = "\t",
    col.others = 0,
    quiet = TRUE
  )
  
  x1 <- tab(xy1)
  # should return all 1, incorrect is NA
  expect_true(all(x1[, grepl("1401_25", colnames(x1)), drop = FALSE] == 1))

  tmp2 <- tempfile(fileext = ".stru")
  # Column names should have no extra characters in front of them. Your IDE
  # may be adding them, so watch out!
  cat(
"		X1_25	X8_54	X1358_15	X1363_12	X1368_57	X1369_41	X1372_14	X1373_9	X1377_42	X1378_53	X1379_10	X1382_37	X1386_27	X1398_46	X1400_9	X1401_25	X1403_13	X1404_17	X1409_42	X1416_48	X1419_11	X1421_14	X1423_5	X1424_74	X1426_55	X1429_46	X1432_23	X1435_30	X1436_7	X1438_9	X1443_37
A_KH1584	A	1	4	4	1	1	3	2	4	4	2	3	3	2	4	1	3	1	1	2	3	1	4	4	3	2	2	3	4	4	4	2
A_KH1584	A	1	4	4	1	1	3	2	4	4	4	3	3	4	4	1	3	1	3	2	3	3	4	4	3	4	2	3	4	4	4	2
C_KH1059	C	0	4	4	1	1	3	2	4	4	2	1	3	2	4	1	3	1	3	2	3	3	2	4	3	2	2	3	2	4	4	2
C_KH1059	C	0	4	4	1	1	3	2	4	4	4	3	3	2	4	1	3	1	3	2	3	3	4	4	3	2	2	3	4	4	4	2
M_KH1834	M	0	2	2	1	1	3	2	4	4	2	3	3	2	4	1	3	1	1	2	3	3	4	4	3	2	2	3	2	4	4	2
M_KH1834	M	0	4	4	1	3	3	2	4	4	2	3	3	2	4	1	3	1	3	2	3	3	4	4	3	2	4	3	4	4	4	2
M_KH1837	M	1	4	4	1	1	3	2	4	4	0	3	3	2	2	1	3	1	1	2	3	3	4	4	3	4	2	3	4	4	4	2
M_KH1837	M	1	4	4	1	3	3	2	4	4	0	3	3	4	4	1	3	1	3	2	3	3	4	4	3	4	2	3	4	4	4	2", 
      file = tmp2)
  
  xy2 <- read.structure(
    tmp2,
    NA.char = "0",
    n.ind = 4,
    n.loc = 31,
    onerowperind = FALSE,
    col.lab = 1,
    col.pop = 2,
    row.marknames = 1,
    sep = "\t",
    col.others = 0,
    quiet = TRUE
  )
  
  x2 <- tab(xy2)
  
  # should return all 1
  expect_true(all(x2[, grepl("1401_25", colnames(x2)), drop = FALSE] == 1))
  
})


test_that("df2genind throws a warning when the user borks the ncode", {
  dat <- data.frame(stringsAsFactors = FALSE,
                    A = c("A5A5", "A5A4", "A5A5", "A5A5", "A5A4", "A5A4", "A5A5",
                          "A5A4", "A5A4", "A5A5"),
                    B = c("B1B1", "B1B1", "B1B1", "B1B1", "B1B1", "B1B1", "B1B1",
                          "B1B1", "B1B2", "B1B1"),
                    C = c("C2C2", "C2C2", "C2C2", "C2C2", "C2C2", "C2C2", "C2C2",
                          "C2C2", "C2C2", "C2C2"),
                    D = c("D2D5", "D5D3", "D5D5", "D2D5", "D2D1", "D5D3", "D1D1",
                          "D2D2", "D2D5", "D2D4")
  )
  dat
  
  expect_warning(df2genind(dat, ploidy = 2, ncode = 1), "observed allele dosage \\(4-4\\)")
  
})

context("Accessor tests")

data("microbov")
strata(microbov) <- data.frame(other(microbov))

test_that("individual accessors work as expected", {
  skip_on_cran()
  expect_equal(nInd(microbov), 704)
  indNames(microbov)[1] <- "replacement"
  expect_equal(indNames(microbov)[1], "replacement")
})

test_that("population accessors work for genind objects", {
  skip_on_cran()
  expect_equal(nPop(microbov), 15)
  expect_equivalent(popNames(microbov), levels(pop(microbov)))
  popNames(microbov)[1] <- "replacement"
  expect_equal(popNames(microbov)[1], "replacement")
  expect_equivalent(unique(head(pop(microbov))), factor("replacement"))
})

test_that("population accessors work for genpop objects", {
  skip_on_cran()
  micpop <- genind2genpop(microbov, quiet = TRUE)
  expect_equal(nPop(micpop), 15)
  expect_equivalent(popNames(micpop), rownames(micpop@tab))
  popNames(micpop)[1] <- "replacement"
  expect_equal(popNames(micpop)[1], "replacement")
  expect_equal(rownames(micpop@tab)[1], "replacement")
})

test_that("locus accessors work for genind objects", {
  skip_on_cran()
  expect_equal(nLoc(microbov), 30)
  locNames(microbov)[1] <- "replacement"
  expect_equal(locNames(microbov)[1], "replacement")
})

test_that("locus accessors work for genpop objects", {
  skip_on_cran()
  micpop <- genind2genpop(microbov, quiet = TRUE)
  expect_equal(nLoc(micpop), 30)
  locNames(micpop)[1] <- "replacement"
  expect_equal(locNames(micpop)[1], "replacement")
})

test_that("'[' method works for genind objects", {
  skip_on_cran()
  two_random_loci    <- sample(locNames(microbov), 2)
  ten_random_samples <- sample(nInd(microbov), 10)
#   cat("\nLoci:", dput(two_random_loci), "\nSamples:",
#       dput(ten_random_samples), "\n")
  pops <- levels(pop(microbov))
  pops <- pops[pops %in% pop(microbov)[ten_random_samples]]
  loci <- microbov@loc.fac[microbov@loc.fac %in% two_random_loci]
  loci <- factor(loci)

  mic10     <- microbov[ten_random_samples]
  mic2Loc   <- microbov[loc = two_random_loci]
  mic2Loc10 <- microbov[ten_random_samples, loc = two_random_loci]

  names(two_random_loci) <- two_random_loci
  two_random_loci <- two_random_loci[levels(loci)]

  expect_equal(nInd(mic10), 10)
  expect_equal(nInd(mic2Loc), nInd(microbov))
  expect_equal(nInd(mic2Loc10), 10)

  expect_equal(nLoc(mic10), nLoc(microbov))
  expect_equal(nLoc(mic2Loc), 2)
  expect_equal(nLoc(mic2Loc10), 2)

  expect_equivalent(popNames(mic10), pops)
  expect_equivalent(popNames(mic2Loc10), pops)
  expect_equal(nPop(mic10), length(pops))
  expect_equal(nPop(mic2Loc10), length(pops))

  expect_equal(length(mic10@loc.fac), ncol(tab(microbov)))
  expect_equal(mic2Loc@loc.fac, loci)
  expect_equal(mic2Loc10@loc.fac, loci)

  expect_equal(mic10@loc.n.all, microbov@loc.n.all)
  expect_equal(mic2Loc@loc.n.all, microbov@loc.n.all[two_random_loci])
  expect_equal(mic2Loc10@loc.n.all, microbov@loc.n.all[two_random_loci])
})

test_that("'[' method works for genind objects with drop = TRUE", {
  skip_on_cran()
  two_random_loci    <- sample(locNames(microbov), 2)
  ten_random_samples <- sample(nInd(microbov), 10)
#   cat("\nLoci:", dput(two_random_loci),
#       "\nSamples:", dput(ten_random_samples), "\n")
  pops <- levels(pop(microbov))
  pops <- pops[pops %in% pop(microbov)[ten_random_samples]]
  j    <- microbov@loc.fac %in% two_random_loci
  loci <- microbov@loc.fac[j]
  loci <- factor(loci)
  loci <- loci[colSums(tab(microbov)[, j], na.rm = TRUE) > 0]
  ten_ind_loci <- colSums(tab(microbov[ten_random_samples, ]), na.rm = TRUE) > 0


  mic10     <- microbov[ten_random_samples, , drop = TRUE]
  mic2Loc   <- microbov[loc = two_random_loci]
  mic2Loc10 <- microbov[ten_random_samples, loc = two_random_loci, drop = TRUE]

  names(two_random_loci) <- two_random_loci
  two_random_loci <- two_random_loci[levels(loci)]

  expect_equal(nInd(mic10), 10)
  expect_equal(nInd(mic2Loc), nInd(microbov))
  expect_equal(nInd(mic2Loc10), 10)

  expect_equal(nLoc(mic10), nLoc(microbov))
  expect_equal(nLoc(mic2Loc), 2)
  expect_equal(nLoc(mic2Loc10), 2)

  expect_equivalent(popNames(mic10), pops)
  expect_equivalent(popNames(mic2Loc10), pops)
  expect_equal(nPop(mic10), length(pops))
  expect_equal(nPop(mic2Loc10), length(pops))

  ten_ind_loci2 <- factor(mic10@loc.fac[mic10@loc.fac %in% levels(loci)])
  expect_equal(length(mic10@loc.fac), ncol(tab(microbov)[, ten_ind_loci]))
  expect_equal(mic2Loc@loc.fac, loci)
  expect_equal(mic2Loc10@loc.fac, ten_ind_loci2)

  expect_true(all(mic10@loc.n.all <= microbov@loc.n.all))
  expect_equal(mic2Loc@loc.n.all, microbov@loc.n.all[two_random_loci])
  expect_equal(names(mic2Loc10@loc.n.all), names(microbov@loc.n.all[two_random_loci]))
  expect_true(all(mic2Loc10@loc.n.all <= microbov@loc.n.all[two_random_loci]))
})

test_that("tab will retain dimensions", {
  skip_on_cran()
  micpop <- genind2genpop(microbov[pop(microbov) %in% popNames(microbov)[1]], quiet = TRUE)
  tabdim <- dim(micpop@tab)
  expect_equal(tabdim, dim(tab(micpop)))
  expect_equal(tabdim, dim(tab(micpop, freq = TRUE)))
})

test_that("tab will return frequencies for PA data", {
  skip_on_cran()
  x <- read.table(system.file("files/AFLP.txt", package = "adegenet"))
  aflp <- df2genind(x, type = "PA", ploidy = 1, pop = c(rep(1, 4), rep(2, 3)))
  apop <- genind2genpop(aflp, quiet = TRUE)
  atab <- tab(apop, freq = TRUE)
  res  <- tab(apop)/rowSums(tab(apop))
  expect_equivalent(atab, res)
})

test_that("subsettors give one warning for individuals", {
  skip_on_cran()
  expect_warning(microbov[c("bippity", "hop", "bop")], "the following specified individuals do not exist: bippity, hop, bop")
  expect_warning(microbov[c("bippity", "hop", "bop")], "no individual selected - ignoring")
})


test_that("subsettors give one warning for loci", {
  skip_on_cran()
  expect_warning(microbov[loc = c("bippity", "hop", "bop")], "the following specified loci do not exist: bippity, hop, bop")
  expect_warning(microbov[loc = c("bippity", "hop", "bop")], "no loci selected - ignoring")
})
  
test_that("subsettors give one warning for populations", {
  skip_on_cran()
  expect_warning(microbov[pop = c("bippity", "hop", "bop")], "the following specified populations do not exist: bippity, hop, bop")
  expect_warning(microbov[pop = c("bippity", "hop", "bop")], "no populations selected - ignoring")
})

test_that("subsettors give one warning for genpop objects", {
  skip_on_cran()
  micropop <- genind2genpop(microbov, quiet = TRUE)
  expect_warning(micropop[c("bippity", "hop", "bop")], "the following specified populations do not exist: bippity, hop, bop")
  expect_warning(micropop[c("bippity", "hop", "bop")], "no population selected - ignoring")
})

## #################
## # fstat function
## #################
## #
## # Wrapper for fst estimator from hierfstat package
## #

## fstat <- function(x, pop=NULL, fstonly=FALSE){
##     message("Sorry, this function depends on hierfstat, and has been moved to the package hierfstat.")
##     return()
##     ## ## cat("\nSorry, hierfstat package has been disabled - this function will be restored in a future release.\n")
##     ## ## return(invisible())
##     ## ## misc checks
##     ## if(!is.genind(x)) stop("x is not a valid genind object")
##     ## ## if(!require(hierfstat)) stop("hierfstat package is required. Please install it.")
##     ## if(x@ploidy != as.integer(2)) stop("not implemented for non-diploid genotypes")
##     ## checkType(x)

##     ## if(is.null(pop)) pop <- x@pop
##     ## if(is.null(pop)) stop("no pop factor provided")
##     ## if(length(pop)!=nrow(x@tab)) stop("pop has a wrong length.")

##     ## ## computations
##     ## dat <- genind2hierfstat(x)[,-1]
##     ## res <- varcomp.glob(levels=data.frame(pop), loci=dat)$F

##     ## if(fstonly) {res <- res[1,1]}
##     ## return(res)
## }



## ###############
## ## pairwise.fst
## ###############
## ##
## ## pairwise fst sensu Nei (Ht - mean(Hs))/Ht
## ##
## pairwise.fst <- function(x, pop=NULL, res.type=c("dist","matrix"), truenames=TRUE){
##     message("This function has been moved to the package hierfstat. You can still use it in adegenet, but this version is deprecated.")
##     ## MISC CHECKS ##
##     if(!is.genind(x)) stop("x is not a valid genind object")
##     if(!is.null(pop)){
##         pop(x) <- pop
##     }
##     temp <- pop(x)
##     if(is.null(temp)) stop("no grouping factor (pop) provided")
##     if(length(levels(temp)) < 2){
##         warning("There is only one pop - returning NULL")
##         return(NULL)
##     }

##     res.type <- match.arg(res.type)


##     ## COMPUTATIONS ##

##     ## function to compute one Fst ##
##     f1 <- function(pop1, pop2){ # pop1 and pop2 are genind obj. with a single pop each
##         n1 <- nrow(pop1@tab)
##         n2 <- nrow(pop2@tab)
##         temp <- repool(pop1,pop2)
##         b <- weighted.mean(Hs(temp), c(n1,n2)) # mean Hs is weighted for pop sizes
##         pop(temp) <- NULL
##         a <- Hs(temp)
##         return((a-b)/a)
##     }


##     ## compute pairwise Fst for all pairs
##     lx <- seppop(x,treatOther=FALSE)
##     temp <- pop(x)
##     levPop <- levels(temp)
##     allPairs <- combn(1:length(levPop), 2)
##     if(!is.matrix(allPairs)){
##         allPairs <- matrix(allPairs,nrow=2)
##     }

##     vecRes <- numeric()
##     for(i in 1:ncol(allPairs)){
##         vecRes[i] <- f1(lx[[allPairs[1,i]]], lx[[allPairs[2,i]]])
##     }


##     squelres <- dist(1:length(levPop))
##     res <- vecRes
##     attributes(res) <- attributes(squelres)

##     if(res.type=="matrix"){
##         res <- as.matrix(res)
##         lab <- popNames(x)

##         colnames(res) <- rownames(res) <- lab
##     }

##     return(res)
## } # end of pairwise.fst

###########
## glPlot
############
glPlot <- function(x, col=NULL, legend=TRUE, posi="bottomleft", bg=rgb(1,1,1,.5),...) {

    ## get plotted elements ##
    X <- t(as.matrix(x))
    X <- X[,ncol(X):1]
    ylabpos <- pretty(1:nInd(x),5)
    if(is.null(col)) {
        myCol <- colorRampPalette(c("royalblue3", "firebrick1"))(max(X,na.rm=TRUE)+1)
    } else {
        myCol <- col
    }

    ## draw the plot ##
    ## main plot
    image(x=1:nLoc(x), y=1:nInd(x), z=X, xlab="SNP index", ylab="Individual index", yaxt="n", col=myCol, ...)

    ## add y axis
    axis(side=2, at=nInd(x)-ylabpos+1, labels=ylabpos)

    ## add legend
    if(legend){
        legend(posi, fill=myCol, legend=0:max(X,na.rm=TRUE), horiz=TRUE, bg=bg, title="Number of 2nd allele")
    }

    return(invisible())
} # end plot for glPlot



## hack to remove the NOTE in R CMD check about:
## "plot,genlight: no visible binding for global variable ‘y’"
if(getRversion() >= "2.15.1")  utils::globalVariables("y")

## plot method
setMethod("plot", signature(x="genlight", y="ANY"), function(x, y=NULL, col=NULL, legend=TRUE,
                                          posi="bottomleft", bg=rgb(1,1,1,.5),...) {
    glPlot(x, col=col, legend=legend, posi=posi, bg=bg, ...)
})

## AVOID:
## airDistPlot.dist: no visible binding for global variable ‘groups’
## pairDistPlot.dist: no visible binding for global variable ‘distance’
if(getRversion() >= "2.15.1")  utils::globalVariables(c("groups","distance"))



##############
## GENERICS ##
##############
pairDistPlot <-  function (x, ...) UseMethod("pairDistPlot")
pairDist <-  function (x, ...) UseMethod("pairDistPlot")





#############
## DEFAULT ##
#############
pairDistPlot.default <- function(x, ...){
    stop(paste("No method for objects of class",class(x)))
} # end pairDistPlot.default


pairDist.default <- function(x, grp, within=FALSE, sep="-", ...){
    temp <- pairDistPlot(x=x, grp=grp, within=within, sep=sep,
                         data=TRUE, violin=FALSE, boxplot=FALSE, jitter=FALSE)
    return(temp$data)
}


##########
## DIST ##
##########
##
## this is the basic method
##
pairDistPlot.dist <- function(x, grp, within=FALSE, sep="-", data=TRUE, violin=TRUE, boxplot=TRUE,
                              jitter=TRUE, ...){
    ## CHECKS ##
    if(attr(x, "Size")!=length(grp)) stop("inconsistent length for grp")
    grp <- factor(grp)
    K <- length(levels(grp))
    N <- length(grp)


    ## GET DATA FOR OUTPUT AND PLOTTING ##
    ## get groups of pairwise comparisons ##
    allCombs <- combn(N, 2)
    d <- as.vector(x)

    ## remove within if needed ##
    if(!within){
        toKeep <- grp[allCombs[1,]] != grp[allCombs[2,]]
        allCombs <- allCombs[,toKeep,drop=FALSE]
        d <- d[toKeep]
    }

    ## get group-group ##
    d.grp <- paste(grp[allCombs[1,]], grp[allCombs[2,]], sep=sep)


    ## BUILD OUTPUT ##
    out <- list()

    ## data ##
    fig.dat <- data.frame(distance=d, groups=d.grp)
    if(data){
        out$data <- fig.dat
    }

    ## plots ##
    base <- ggplot(data=fig.dat)

    ## violinplot
    if(violin){
        out$violin <- base + geom_violin(aes(x=groups, y=distance, fill=groups), alpha=.5) +
            coord_flip() + guides(fill=FALSE) + labs(x="",y="Pairwise distances")
    }

    ## boxplot
    if(boxplot){
        out$boxplot <- base + geom_boxplot(aes(x=groups, y=distance, fill=groups), alpha=.5) +
            coord_flip() + guides(fill=FALSE) + labs(x="",y="Pairwise distances")
    }

    ## jitter
    if(jitter){
        out$jitter <- base + geom_jitter(aes(x=groups, y=distance, colour=groups), alpha=.2) +
            coord_flip() + guides(colour=FALSE) + labs(x="",y="Pairwise distances")
    }


    return(out)

} # end pairDistPlot.dist







############
## MATRIX ##
############
pairDistPlot.matrix <- function(x, grp, within=FALSE, sep="-", data=TRUE, violin=TRUE, boxplot=TRUE,
                                jitter=TRUE, ...){
    ## CHECKS ##
    if(nrow(x) != ncol(x)) stop("x is not a square matrix")

    ## RETURN ##
    out <- pairDistPlot(as.dist(x), grp=grp, within=within, sep=sep,
                        data=data, violin=violin, boxplot=boxplot, jitter=jitter, ...)

    return(out)
} # end pairDistPlot.matrix






############
## GENIND ##
############
pairDistPlot.genind <- function(x, grp, within=FALSE, sep="-", data=TRUE, violin=TRUE, boxplot=TRUE,
                                jitter=TRUE, ...){
    ## CHECKS ##
    if(missing(grp)){
        if(!is.null(pop(x))) {
            grp <- pop(x)
        } else {
            stop("grp is missing with no population defined in x")
        }
    }


    ## RETURN ##
    D <- dist(x@tab)^2
    out <- pairDistPlot(D, grp=grp, within=within, sep=sep,
                        data=data, violin=violin, boxplot=boxplot, jitter=jitter, ...)

    return(out)
} # end pairDistPlot.matrix






############
## DNAbin ##
############
pairDistPlot.DNAbin <- function(x, grp, within=FALSE, sep="-", data=TRUE, violin=TRUE, boxplot=TRUE,
                                jitter=TRUE, ...){

    ## RETURN ##
    D <- dist.dna(x, ...)
    out <- pairDistPlot(D, grp=grp, within=within, sep=sep,
                        data=data, violin=violin, boxplot=boxplot, jitter=jitter, ...)

    return(out)
} # end pairDistPlot.matrix

################
## snpposi.test
################

## GENERIC
snpposi.test <- function(...){
    UseMethod("snpposi.test")
}





## METHOD FOR INTEGER - BASIC METHOD
snpposi.test.integer <- function(x, genome.size, n.sim=999, stat=median, ...){
    ## AUXILIARY FUNCTION ##
    ## computes the test statistics for one vector of SNP positions
    f1 <- function(snpposi, stat=median){
        temp <- as.matrix(dist(snpposi))
        diag(temp) <- 1e15
        out <- stat(apply(temp, 1, min))
        return(out)
    }

    ## GET OBSERVATION ##
    obs <- f1(x, stat=stat)

    ## GET SIMULATIONS ##
    n.snps <- length(x)
    sim <- sapply(1:n.sim, function(e) f1(sample(1:genome.size, n.snps, replace=FALSE), stat=stat))

    ## MAKE RANDTEST OUTPUT ##
    out <- as.randtest(obs=obs, sim=sim, alter="less")
    return(out)
} # end snpposi.test.integer





## METHOD FOR NUMERIC
snpposi.test.numeric <- function(x, ...){
    out <- snpposi.test(as.integer(x), ...)
    return(out)
}





## METHOD FOR DNABIN
snpposi.test.DNAbin <- function(x, ...){
    out <- snpposi.test(x=as.integer(seg.sites(x)),
                        genome.size=ncol(x), ...)
    return(out)
} # end snpposi.test.DNAbin











################
## snpposi.plot
################

## GENERIC
snpposi.plot <- function(...){
    UseMethod("snpposi.plot")
}




## METHOD FOR INTEGER - BASIC METHOD
snpposi.plot.integer <- function(x, genome.size, smooth=0.1, col="royalblue", alpha=.2,
                                 codon=TRUE, start.at=1, ...){
    ## IF WE REPRESENT DENSITY PER CODON POSITION ##
    if(codon){
        ## define base positions (1/2/3) ##
        codon.posi <- ((2 + x) %% 3) + 1
        fac <- factor(codon.posi, levels=1:3)

        ## make ggplot output ##
        out <- ggplot(data.frame(x=x, codon=fac), aes(x=x)) + xlim(0, genome.size)
        out <- out + geom_density(adjust=smooth, aes(fill=codon, colour=codon),alpha=I(alpha)) + geom_rug(aes(colour=codon),alpha=.7)
        out <- out + labs(x="Nucleotide position", title="Distribution of SNPs in the genome")
        out <- out + guides(fill=guide_legend(title="Codon position"), colour=guide_legend(title="Codon position"))
    } else {
        ## OTHERWISE, JUST ONE DENSITY ##
        ## make ggplot output ##
        out <- ggplot(data.frame(x=x), aes(x=x)) + xlim(0, genome.size)
        out <- out + geom_density(adjust=smooth, fill=transp(col,alpha=alpha), colour=col) + geom_rug(colour=col,alpha=.7)
        out <- out + labs(x="Nucleotide position", title="Distribution of SNPs in the genome")
    }


    ## return ##
    return(out)
} # end snpposi.plot.integer





## METHOD FOR NUMERIC
snpposi.plot.numeric <- function(x, ...){
    out <- snpposi.plot(as.integer(x), ...)
    return(out)
}




## METHOD FOR DNABIN
snpposi.plot.DNAbin <- function(x, ...){
    out <- snpposi.plot(x=as.integer(seg.sites(x)),
                        genome.size=ncol(x), ...)
    return(out)
} # end snpposi.plot.DNAbin

## propShared computes the proportion of shared alleles
## in a genind object


######################
# Function propShared
######################
propShared <- function(obj){
    ## CHECK THAT THIS IS A VALID GENIND ##
    if(!inherits(obj,"genind")) stop("obj must be a genind object.")
    invisible(validObject(obj))


    ## GET MATRIX OF NB OF ALLELES ##
    x <- tab(obj)
    x[is.na(x)] <- 0L

    ## COMPUTE NB OF SHARED ALLELES ##
    n <- nInd(obj)
    resVec <- integer(n*(n-1)/2)

    res <- .C("nb_shared_all", as.integer(x), as.integer(resVec), as.integer(n), as.integer(ncol(obj$tab)), PACKAGE="adegenet")[[2]]
    attr(res,"Size") <- n
    attr(res,"Diag") <- FALSE
    attr(res,"Upper") <- FALSE
    class(res) <- "dist"
    res <- as.matrix(res)


    ## COMPUTE NB OF ALLELES TYPED IN COMMON ##
    tabNA <- propTyped(obj, by="both")
    tabTypCom <- tabNA %*% t(tabNA) * ploidy(obj)


    ## GET PROPORTIONS OF SHARED ALLELES ##
    res <- res/tabTypCom
    diag(res) <- 1L
    colnames(res) <-rownames(res) <- indNames(obj)
    return(res)
}

context("Test PropShared")

data("microbov", package = "adegenet")
obj <- microbov[1:5, loc = locNames(microbov)[1:2]]
psh <- propShared(obj)
expected_values <- structure(c(1, 0.5, 0.5, 0.75, 0.5, 0.5, 1, 0.75, 0.75, 0.75,
              0.5, 0.75, 1, 0.75, 1, 0.75, 0.75, 0.75, 1, 0.75, 0.5, 0.75, 1,
              0.75, 1), .Dim = c(5L, 5L), .Dimnames = list(c("AFBIBOR9503",
              "AFBIBOR9504", "AFBIBOR9505", "AFBIBOR9506", "AFBIBOR9507"),
              c("AFBIBOR9503", "AFBIBOR9504", "AFBIBOR9505", "AFBIBOR9506",
              "AFBIBOR9507")))
test_that("propShared produces expected results", {
  skip_on_cran()
  expect_that(psh, equals(expected_values))
  expect_true(all(psh <= 1))
  expect_that(ncol(psh), equals(nrow(psh)))
  expect_that(ncol(psh), equals(nInd(obj)))
})

#' When you need a break...
#'
#' Genetic data analysis can be a harsh, tiring, daunting task.
#' Sometimes, a mere break will not cut it.
#' Sometimes, you need a kitten.
#'
#' @author Thibaut Jombart \email{thibautjombart@@gmail.com}
#'
#' @export
#'
#' @details
#'
#' Please send us more! Either pull request or submit an issue with a URL (use
#' \code{adegenetIssues()}).
#'
#'
#' @param x the name or index of the video to display; if NULL, a random video is chosen
#'
#' @param list a logical indicating if the list of available videos should be displayed
#'
showmekittens <- function(x = NULL, list = FALSE){
    ## 'pool' is a named character vector of video URLs
    pool <- c(capucine = "http://www.youtube.com/watch?v=KIePsbJSS04",
              vacuum = "https://www.youtube.com/watch?v=9J4UKYm9320",
              eclipse = "https://www.youtube.com/watch?v=RsUH2MuY33o",
              nom     = "https://www.youtube.com/watch?v=Cl5Pfc5TyO0")

    ## either we return the list of videos, or we show one
    if (list) {
        return(pool)
    }

    if (is.null(x)) {
        x <- sample(seq_along(pool), 1L)
    }

    ## check that x is okay
    if (is.numeric(x) && (x < 1 || x > length(pool))) {
        stop(sprintf("Video index (%d) is wrong; there are currently %d videos in the list", x, length(pool)))
    }

    if (is.character(x) && !x %in% names(pool)) {
        stop(sprintf("Video name (%s) is not in the list; use the option 'list=TRUE' to see available videos.", x))
    }

    browseURL(pool[x])
}

#' genind constructor
#'
#' The function \code{new} has a method for building \linkS4class{genind} objects.
#' See the class description of \linkS4class{genind} for more information on this data structure.
#' The functions \code{genind} and \code{as.genind} are aliases for \code{new("genind", ...)}.
#'
#' Most users do not need using the constructor, but merely to convert raw allele data using \code{\link{df2genind}} and related functions.
#'
#' @export
#' @docType methods
#'
#' @aliases initialize,genind-methods
#' @aliases genind
#' @aliases as.genind
#'
#' @rdname new.genind
#'
#' @param .Object prototyped object (generated automatically when calling 'new')
#' @param tab A matrix of integers corresponding to the @@tab slot of a genind
#'   object, with individuals in rows and alleles in columns, and containing
#'   either allele counts (if type="codom") or allele presence/absence (if
#'   type="PA")
#' @param pop an optional factor with one value per row in \code{tab} indicating
#'   the population of each individual
#' @param prevcall an optional call to be stored in the object
#' @param ploidy an integer vector indicating the ploidy of the individual; each
#'   individual can have a different value; if only one value is provided, it is
#'   recycled to generate a vector of the right length.
#' @param type a character string indicating the type of marker: codominant
#'   ("codom") or presence/absence ("PA")
#' @param strata a data frame containing population hierarchies or
#'   stratifications in columns. This must be the same length as the number of
#'   individuals in the data set.
#' @param hierarchy a hierarchical formula defining the columns of the strata
#'   slot that are hierarchical. Defaults to NULL.
#' @param ... further arguments passed to other methods (currently not used)
#'
#' @return a \linkS4class{genind} object
#'
#' @seealso the description of the \linkS4class{genind} class; \code{\link{df2genind}}
#'
setMethod("initialize", "genind", function(.Object, tab, pop=NULL, prevcall=NULL, ploidy=2L, type=c("codom","PA"), strata = NULL, hierarchy = NULL, ...){
   ## HANDLE ARGUMENTS ##
    out <- .Object
    if (missing(tab)){
        .Object@tab       <- matrix(0L, ncol = 0, nrow = 0)
        .Object@pop       <- NULL
        .Object@strata    <- NULL
        .Object@hierarchy <- NULL
        .Object@call      <- NULL
        .Object@loc.n.all  <- integer(0)
        .Object@loc.fac   <- NULL
        .Object@ploidy    <- integer(0)
        .Object@other     <- NULL
        .Object@type      <- "codom"
        return(.Object)
    }
    if(is.null(colnames(tab))) {colnames(tab) <- seq(ncol(tab))}
    if(is.null(rownames(tab))) {rownames(tab) <- seq(nrow(tab))}

    ## force matrix & integer
    if(!is.matrix(tab)) tab <- as.matrix(tab)
    old.rownames <- rownames(tab)
    old.colnames <- colnames(tab)
    old.dim <- dim(tab)
    if(typeof(tab)!="integer"){
        tab <- as.integer(tab)
        dim(tab) <- old.dim
        rownames(tab) <- old.rownames
        colnames(tab) <- old.colnames
    }
    type <- match.arg(type)
    nind <- nrow(tab)
    ploidy <- as.integer(ploidy)
    ploidy <- rep(ploidy, length=nind)

    ## HANDLE LABELS ##
    ## loc names is not type-dependent
    if(length(grep("([.][^.]*){2,}", old.colnames))>0L){
        stop("more than one '.' in column names; please name column as [LOCUS].[ALLELE]")
    }
    temp <- sub("[.].*$", "", old.colnames)
    temp <- .rmspaces(temp)
    loc.names <- unique(temp)
    nloc <- length(loc.names)

    ## ind names is not type-dependent either
    ## only use generic label if no name or duplicates
    if(is.null(rownames(tab))) {
        rownames(tab) <- .genlab("", nind)
    }
    ind.names <- rownames(tab)
    if(length(unique(ind.names))!=length(ind.names)) {
        warning("duplicate labels detected for some individuals; using generic labels")
        rownames(tab) <- ind.names <- .genlab("", nind)
    }

    if (!is.null(strata)){
      # Make sure that the hierarchies are factors.
      strata <- data.frame(lapply(strata, function(f) factor(f, unique(f))))
      rownames(strata) <- rownames(tab)
    }

    if (!is.null(strata) && !is.null(hierarchy)){
      if (is.language(hierarchy)){
        the_names <- all.vars(hierarchy)
        if (all(the_names %in% names(strata))){
          ## TODO: CHECK HIERARCHY HERE
        } else {
          warning("hierarchy names do not match names of strata. Setting slot to NULL")
          hierarchy <- NULL
        }
      } else {
        warning("hierarchy must be a formula. Setting slot to NULL.")
        hierarchy <- NULL
      }
    }


    if(type=="codom"){
        ## loc.n.all
        loc.n.all <-  table(temp)[match(loc.names,names(table(temp)))]
        loc.n.all <- as.integer(loc.n.all)
        names(loc.n.all) <- loc.names

        ## loc.fac
        loc.fac <- factor(rep(loc.names,loc.n.all),levels=loc.names)

        ## alleles name
        temp <- colnames(tab)
        temp <- sub("^[^.]*[.]","",temp)
        temp <- .rmspaces(temp)
        all.names <- split(temp,loc.fac)
        all.names <- all.names[loc.names]

    } else { # end if type=="codom" <=> if type=="PA"
        loc.fac   <- NULL
        all.names <- NULL
        loc.n.all  <- NULL
    }

    ## Ideally I should use an 'initialize' method here
    out@tab       <- tab
    out@loc.n.all  <- loc.n.all
    out@loc.fac   <- loc.fac
    out@all.names <- all.names
    out@strata    <- strata
    out@hierarchy <- hierarchy
    ## populations name (optional)
    ## beware, keep levels of pop sorted in
    ## there order of appearance
    if(!is.null(pop)) {
        ## convert pop to a factor if it is not; for consistentcy with the accessor 'pop', we need
        ## to make sure that levels will be sorted by their order of appearance
        if(!is.factor(pop)) {
            pop <- as.character(pop)
            pop <- factor(pop, levels=unique(pop))
        }
        out@pop <- pop
    }

    ## ploidy
    plo <- as.integer(ploidy)
    if(any(plo < 1L)) stop("ploidy inferior to 1")
    out@ploidy <- plo

    ## type of marker
    out@type <- as.character(type)

    if(is.null(prevcall)) {prevcall <- match.call()}
    out@call <- prevcall

    return(out)
})


#' @export
#' @rdname new.genind
genind <- function(...){
    out <- new("genind", ...)
    return(out)
} # end genind

#' @export
#' @rdname new.genind
as.genind <- function(...){
    out <- new("genind", ...)
    return(out)
} # end genind






#' genpop constructor
#'
#' The function \code{new} has a method for building \linkS4class{genpop} objects.
#' See the class description of \linkS4class{genpop} for more information on this data structure.
#' The functions \code{genpop} and \code{as.genpop} are aliases for \code{new("genpop", ...)}.
#'
#' Most users do not need using the constructor, but merely to convert raw allele data using \code{\link{genind2genpop}}.
#'
#' @export
#' @docType methods
#'
#' @aliases initialize,genpop-methods
#' @aliases genpop
#' @aliases as.genpop
#'
#' @rdname new.genpop
#'
#' @param .Object prototyped object (generated automatically when calling 'new')
#' @param tab A matrix of integers corresponding to the @@tab slot of a genpop object, with individuals in rows and alleles in columns, and containing either allele counts
#' @param prevcall an optional call to be stored in the object
#' @param ploidy an integer vector indicating the ploidy of the individual; each individual can have a different value; if only one value is provided, it is recycled to generate a vector of the right length.
#' @param type a character string indicating the type of marker: codominant ("codom") or presence/absence ("PA")
#' @param ... further arguments passed to other methods (currently not used)
#'
#' @return a \linkS4class{genpop} object
#'
#' @seealso the description of the \linkS4class{genpop} class; \code{\link{df2genind}} and related functions for reading raw allele data
#'
##################
# Function genpop
##################
setMethod("initialize", "genpop", function(.Object, tab, prevcall=NULL, ploidy=2L, type=c("codom","PA"), ...){
    ## HANDLE ARGS ##
    out <- .Object
    if (missing(tab)){
        .Object@tab       <- matrix(0L, ncol = 0, nrow = 0)
        .Object@call      <- NULL
        .Object@loc.n.all  <- integer(0)
        .Object@loc.fac   <- NULL
        .Object@ploidy    <- integer(0)
        .Object@other     <- NULL
        .Object@type      <- "codom"
        return(.Object)
    }
    if(is.null(colnames(tab))) stop("tab columns have no name.")
    if(is.null(rownames(tab))) {rownames(tab) <- 1:nrow(tab)}

    ## force matrix & integer
    if(!is.matrix(tab)) tab <- as.matrix(tab)
    old.rownames <- rownames(tab)
    old.colnames <- colnames(tab)
    old.dim <- dim(tab)
    if(typeof(tab)!="integer"){
        tab <- as.integer(tab)
        dim(tab) <- old.dim
        rownames(tab) <- old.rownames
        colnames(tab) <- old.colnames
    }
    type <- match.arg(type)
    ploidy <- as.integer(ploidy)
    npop <- nrow(tab)


    ## HANDLE LABELS ##

    ## loc names is not type-dependent
    if(length(grep("([.][^.]*){2,}", old.colnames))>0L){
        stop("more than one '.' in column names; please name column as [LOCUS].[ALLELE]")
    }
    temp <-  sub("[.].*$", "", old.colnames)
    temp <- .rmspaces(temp)
    loc.names <- unique(temp)
    nloc <- length(loc.names)

    ## pop names is not type-dependent either
    ## only use generic label if no name or duplicates
    if(is.null(rownames(tab))) {
        rownames(tab) <- .genlab("", npop)
    }
    pop.names <- rownames(tab)
    if(length(unique(pop.names))!=length(pop.names)) {
        warning("duplicate labels detected for some populations; using generic labels")
        rownames(tab) <- pop.names <- .genlab("", npop)
    }

    if(type=="codom"){
        ## loc.n.all
        loc.n.all <-  table(temp)[match(loc.names,names(table(temp)))]
        loc.n.all <- as.integer(loc.n.all)
        names(loc.n.all) <- loc.names

        ## loc.fac
        loc.fac <- factor(rep(loc.names,loc.n.all),levels=loc.names)

        ## alleles name
        temp <- colnames(tab)
        temp <- sub("^[^.]*[.]","",temp)
        temp <- .rmspaces(temp)
        all.names <- split(temp,loc.fac)
        all.names <- all.names[loc.names]
        loc.fac <- as.factor(loc.fac)

    } else { # end if type=="codom" <=> if type=="PA"
        loc.fac <- NULL
        all.names <- NULL
        loc.n.all <- NULL
    }

    ## build final output
    out@tab <- tab
    out@loc.n.all <- loc.n.all
    out@loc.fac <- loc.fac
    out@all.names <- all.names
    out@ploidy <- ploidy
    out@type <- as.character(type)

    if(is.null(prevcall)) {prevcall <- match.call()}
    out@call <- prevcall

    return(out)
})



#' @export
#' @rdname new.genpop
genpop <- function(...){
    out <- new("genpop", ...)
    return(out)
} # end genpop

#' @export
#' @rdname new.genpop
as.genpop <- function(...){
    out <- new("genpop", ...)
    return(out)
} # end genpop

context("constructor tests")

test_that("genind objects can be constructed with nothing", {
  skip_on_cran()
  gen <- genind()
  expect_that(gen, is_a("genind"))
  expect_true(is.genind(gen))
})

test_that("genpop objects can be constructed with nothing", {
  skip_on_cran()
  gen <- genpop()
  expect_that(gen, is_a("genpop"))
  expect_true(is.genpop(gen))
})

#######
# nLoc
#######
setGeneric("nLoc", function(x,...){
    standardGeneric("nLoc")
})



# setMethod("nLoc","genind", function(x,...){
#   if (x@type == "PA"){
#     return(ncol(x@tab))
#   } else {
#     return(length(levels(x@loc.fac)))
#   }
# })
# 
# 
# 
# setMethod("nLoc","genpop", function(x,...){
#     if (x@type == "PA"){
#       return(ncol(x@tab))
#     } else {
#       return(length(levels(x@loc.fac)))
#     }
# 
# })

setMethod("nLoc","gen", function(x,...){
    if (x@type == "PA"){
      return(ncol(x@tab))
    } else {
      return(length(levels(x@loc.fac)))
    }
})

setMethod("nLoc", "genind", function(x, ...){
  callNextMethod()
})

setMethod("nLoc", "genpop", function(x, ...){
  callNextMethod()
})

#########
# locFac
#########
setGeneric("locFac", function(x, ...){
    standardGeneric("locFac")
})

setMethod("locFac","gen", function(x,...){
    return(x@loc.fac)
})

setMethod("locFac","genind", function(x,...){
  callNextMethod()
})

setMethod("locFac","genpop", function(x,...){
    callNextMethod()
})


#######
# nAll
#######
setGeneric("nAll", function(x,...){
    standardGeneric("nAll")
})


setMethod("nAll","gen", function(x,...){
  if (x@type == "PA"){
    return(ncol(x@tab))
  } else {
    return(x@loc.n.all)
  }
})

setMethod("nAll","genind", function(x,...){
  callNextMethod()
})

setMethod("nAll","genpop", function(x,...){
    callNextMethod()
})


#######
# nPop (no gen method)
#######
setGeneric("nPop", function(x,...){
    standardGeneric("nPop")
})

setMethod("nPop","genind", function(x,...){
    return(length(levels(x@pop)))
})

setMethod("nPop","genpop", function(x,...){
     return(nrow(x@tab))
})


#######
# nInd (no gen method)
#######
setGeneric("nInd", function(x,...){
    standardGeneric("nInd")
})



setMethod("nInd","genind", function(x,...){
    return(nrow(x@tab))
})





######
# pop (no gen method)
######
setGeneric("pop", function(x) {
  standardGeneric("pop")
})



setGeneric("pop<-",
           function(x, value) {
               standardGeneric("pop<-")
           })



setMethod("pop","genind", function(x){
    return(x@pop)
})



setReplaceMethod("pop", "genind", function(x, value) {
    if(is.null(value)){
        x@pop <- NULL
        return(x)
    }

    if(length(value) != nrow(x$tab)) stop("wrong length for population factor")

    ## coerce to factor (put levels in their order of appearance)
    newPop <- as.character(value)
    newPop <- factor(newPop, levels=unique(newPop))

    ## construct output
    x$pop <- newPop

    return(x)
})





###########
# locNames
###########
setGeneric("locNames", function(x,...){
    standardGeneric("locNames")
})

setGeneric("locNames<-", function(x, value) {
    standardGeneric("locNames<-")
})


setMethod("locNames","gen", function(x, withAlleles=FALSE, ...){
    if (withAlleles){
        res <- colnames(x@tab)
    } else {
        res <- unique(sub("[.][^.]*$","",colnames(x@tab)))
    }
    return(res)
})


setReplaceMethod("locNames","gen",function(x,value) {
    ## check input
    value <- as.character(value)
    if(length(value) != nLoc(x)) stop("Vector length does no match number of loci")

    ## make changes in the object
    names(x@all.names) <- value
    levels(x@loc.fac) <- value
    names(x@loc.n.all) <- value
    newColNames <- paste(rep(value, x@loc.n.all), unlist(x@all.names), sep=".")
    colnames(x@tab) <- newColNames

    ## return
    return(x)
})


setMethod("locNames","genpop", function(x, withAlleles=FALSE, ...){
    callNextMethod()
})
setReplaceMethod("locNames", "genpop", function(x, value) {
    callNextMethod()
})


setMethod("locNames","genind", function(x, withAlleles=FALSE, ...){
  callNextMethod()
})
setReplaceMethod("locNames", "genind", function(x, value) {
  callNextMethod()
})


###########
# indNames (no gen method)
###########
setGeneric("indNames", function(x,...){
    standardGeneric("indNames")
})

setGeneric("indNames<-", function(x, value){
    standardGeneric("indNames<-")
})

setMethod("indNames","genind", function(x, ...){
    return(rownames(x@tab))
})


setReplaceMethod("indNames","genind",function(x,value) {
    if(length(value) != nrow(x@tab)) stop("Vector length does not match number of individuals")
    rownames(x@tab) <- as.character(value)
    return(x)
})

###########
# popNames (no gen method)
###########
setGeneric("popNames", function(x,...){
  standardGeneric("popNames")
})

setGeneric("popNames<-", function(x, value){
  standardGeneric("popNames<-")
})

setMethod("popNames","genind", function(x, ...){
  return(levels(pop(x)))
})


setReplaceMethod("popNames","genind",function(x, value) {
  value <- as.character(value)
  if(length(value) != length(levels(pop(x)))){
    stop("Vector length does not match number of populations")
  }
  levels(pop(x)) <- value
  return(x)
})

setMethod("popNames","genpop", function(x, ...){
  return(rownames(x@tab))
})


setReplaceMethod("popNames","genpop",function(x, value) {
  value <- as.character(value)
  if (length(value) != nrow(x@tab)){
    stop("Vector length does not match number of populations")
  }
  rownames(x@tab) <- value
  return(x)
})


##########
# alleles
##########
setGeneric("alleles", function(x,...){
    standardGeneric("alleles")
})

setGeneric("alleles<-", function(x, value){
    standardGeneric("alleles<-")
})

setMethod("alleles","gen", function(x, ...){
    return(x@all.names)
})

setReplaceMethod("alleles","gen", function(x, value){
    if(!is.list(value)) stop("replacement value must be a list")
    if(length(value)!=nLoc(x)) stop("replacement list must be of length nLoc(x)")
    if(any(sapply(value, length) != x$loc.n.all)) stop("number of replacement alleles do not match that of the object")
    x@all.names <- value
    names(x@all.names) <- locNames(x)
    return(x)
})

setMethod("alleles","genind", function(x, ...){
  callNextMethod()
})
setReplaceMethod("alleles","genind", function(x, value){
  callNextMethod()
})

setMethod("alleles","genpop", function(x, ...){
    callNextMethod()
})
setReplaceMethod("alleles","genpop", function(x, value){
    callNextMethod()
})



##########
## ploidy (no gen method)
##########
setGeneric("ploidy", function(x,...){
    standardGeneric("ploidy")
})

setGeneric("ploidy<-", function(x, value){
    standardGeneric("ploidy<-")
})

setMethod("ploidy","genind", function(x,...){
    return(x@ploidy)
})


setReplaceMethod("ploidy","genind",function(x,value) {
    value <- as.integer(value)
    if(any(value)<1) stop("Negative or null values provided")
    if(any(is.na(value))) stop("NA values provided")
    if(length(value)!=nInd(x)) value <- rep(value, length=nInd(x))
    slot(x,"ploidy",check=TRUE) <- value
    return(x)
})


setMethod("ploidy","genpop", function(x,...){
    return(x@ploidy)
})


setReplaceMethod("ploidy","genpop",function(x,value) {
    value <- as.integer(value)
    if(any(value)<1) stop("Negative or null values provided")
    if(any(is.na(value))) stop("NA values provided")
    if(length(value)>1) warning("Several ploidy numbers provided; using only the first integer")
    slot(x,"ploidy",check=TRUE) <- value[1]
    return(x)
})






##########
## other
#########
setGeneric("other", function(x,...){
    standardGeneric("other")
})

setGeneric("other<-", function(x, value){
    standardGeneric("other<-")
})

setMethod("other","gen", function(x,...){
    if(length(x@other)==0) return(NULL)
    return(x@other)
})


setReplaceMethod("other","gen",function(x,value) {
    if( !is.null(value) && (!is.list(value) | is.data.frame(value)) ) {
        value <- list(value)
    }
    slot(x,"other",check=TRUE) <- value
    return(x)
})


setMethod("other","genind", function(x,...){
  callNextMethod()
})
setReplaceMethod("other","genind",function(x,value) {
  callNextMethod()
})

setMethod("other","genpop", function(x,...){
    callNextMethod()
})
setReplaceMethod("other","genpop",function(x,value) {
    callNextMethod()
})

#' Simulated data illustrating the DAPC
#'
#' Datasets illustrating the Discriminant Analysis of Principal Components
#' (DAPC, Jombart et al. submitted).\cr
#'
#' These data were simulated using various models using Easypop (2.0.1).  The
#' \code{dapcIllus} is a list containing the following \linkS4class{genind}
#' objects:\cr - "a": island model with 6 populations \cr - "b": hierarchical
#' island model with 6 populations (3,2,1) \cr - "c": one-dimensional stepping
#' stone with 2x6 populations, and a boundary between the two sets of 6
#' populations\cr - "d": one-dimensional stepping stone with 24 populations\cr
#'
#' See "source" for a reference providing simulation details.
#'
#'
#' @name dapcIllus
#' @docType data
#' @format \code{dapcIllus} is list of 4 components being all genind objects.
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#' @seealso - \code{\link{dapc}}: implements the DAPC.
#'
#' - \code{\link{eHGDP}}: dataset illustrating the DAPC and
#' \code{find.clusters}.
#'
#' - \code{\link{H3N2}}: dataset illustrating the DAPC.
#'
#' - \code{\link{find.clusters}}: to identify clusters without prior.
#' @references Jombart, T., Devillard, S. and Balloux, F.  Discriminant
#' analysis of principal components: a new method for the analysis of
#' genetically structured populations. Submitted to \emph{Genetics}.
#' @source Jombart, T., Devillard, S. and Balloux, F.  Discriminant analysis of
#' principal components: a new method for the analysis of genetically
#' structured populations. Submitted to \emph{BMC genetics}.
#' @keywords datasets
#' @examples
#'
#' \dontrun{
#'
#' data(dapcIllus)
#' attach(dapcIllus)
#' a # this is a genind object, like b, c, and d.
#'
#'
#' ## FINS CLUSTERS EX NIHILO
#' clust.a <- find.clusters(a, n.pca=100, n.clust=6)
#' clust.b <- find.clusters(b, n.pca=100, n.clust=6)
#' clust.c <- find.clusters(c, n.pca=100, n.clust=12)
#' clust.d <- find.clusters(d, n.pca=100, n.clust=24)
#'
#' ## examin outputs
#' names(clust.a)
#' lapply(clust.a, head)
#'
#'
#' ## PERFORM DAPCs
#' dapc.a <- dapc(a, pop=clust.a$grp, n.pca=100, n.da=5)
#' dapc.b <- dapc(b, pop=clust.b$grp, n.pca=100, n.da=5)
#' dapc.c <- dapc(c, pop=clust.c$grp, n.pca=100, n.da=11)
#' dapc.d <- dapc(d, pop=clust.d$grp, n.pca=100, n.da=23)
#'
#'
#' ## LOOK AT ONE RESULT
#' dapc.a
#' summary(dapc.a)
#'
#' ## FORM A LIST OF RESULTS FOR THE 4 DATASETS
#' lres <- list(dapc.a, dapc.b, dapc.c, dapc.d)
#'
#'
#' ## DRAW 4 SCATTERPLOTS
#' par(mfrow=c(2,2))
#' lapply(lres, scatter)
#'
#'
#' # detach data
#' detach(dapcIllus)
#' }
#'
NULL







#' Extended HGDP-CEPH dataset
#'
#' This dataset consists of 1350 individuals from native Human populations
#' distributed worldwide typed at 678 microsatellite loci. The original
#' HGDP-CEPH panel [1-3] has been extended by several native American
#' populations [4]. This dataset was used to illustrate the Discriminant
#' Analysis of Principal Components (DAPC, [5]).
#'
#'
#' @name eHGDP
#' @docType data
#' @format \code{eHGDP} is a genind object with a data frame named
#' \code{popInfo} as supplementary component (\code{eHGDP@@other$popInfo}),
#' which contains the following variables: \describe{ \item{Population: }{a
#' character vector indicating populations.} \item{Region: }{a character vector
#' indicating the geographic region of each population.} \item{Label: }{a
#' character vector indicating the correspondence with population labels used
#' in the genind object (i.e., as output by \code{pop(eHGDP)}).}
#' \item{Latitude,Longitude: }{geographic coordinates of the populations,
#' indicated as north and east degrees.} }
#' @references [1] Rosenberg NA, Pritchard JK, Weber JL, Cann HM, Kidd KK, et
#' al. (2002) Genetic structure of human populations. \emph{Science} 298:
#' 2381-2385.
#'
#' [2] Ramachandran S, Deshpande O, Roseman CC, Rosenberg NA, Feldman MW, et
#' al. (2005) Support from the relationship of genetic and geographic distance
#' in human populations for a serial founder effect originating in Africa.
#' \emph{Proc Natl Acad Sci U S A} 102: 15942-15947.
#'
#' [3] Cann HM, de Toma C, Cazes L, Legrand MF, Morel V, et al. (2002) A human
#' genome diversity cell line panel.  \emph{Science} 296: 261-262.
#'
#' [4] Wang S, Lewis CM, Jakobsson M, Ramachandran S, Ray N, et al. (2007)
#' Genetic Variation and Population Structure in Native Americans. \emph{PLoS
#' Genetics} 3: e185.
#'
#' [5] Jombart, T., Devillard, S. and Balloux, F.  Discriminant analysis of
#' principal components: a new method for the analysis of genetically
#' structured populations. Submitted to \emph{BMC genetics}.
#' @source Original panel by Human Genome Diversity Project (HGDP) and Centre
#' d'Etude du Polymorphisme Humain (CEPH). See reference [4] for Native
#' American populations.
#'
#' This copy of the dataset was prepared by Francois Balloux.
#' @keywords datasets
#' @examples
#'
#' \dontrun{
#' ## LOAD DATA
#' data(eHGDP)
#' eHGDP
#'
#'
#' ## PERFORM DAPC - USE POPULATIONS AS CLUSTERS
#' ## to reproduce exactly analyses from the paper, use "n.pca=1000"
#' dapc1 <- dapc(eHGDP, all.contrib=TRUE, scale=FALSE,
#' n.pca=200, n.da=80) # takes 2 minutes
#' dapc1
#'
#' ## (see ?dapc for details about the output)
#'
#'
#'
#' ## SCREEPLOT OF EIGENVALUES
#' barplot(dapc1$eig, main="eHGDP - DAPC eigenvalues",
#' col=c("red","green","blue", rep("grey", 1000)))
#'
#'
#'
#' ## SCATTERPLOTS
#' ## (!) Note: colors may be inverted with respect to [5]
#' ## as signs of principal components are arbitrary
#' ## and change from one computer to another
#' ##
#' ## axes 1-2
#' s.label(dapc1$grp.coord[,1:2], clab=0, sub="Axes 1-2")
#' par(xpd=T)
#' colorplot(dapc1$grp.coord[,1:2], dapc1$grp.coord, cex=3, add=TRUE)
#' add.scatter.eig(dapc1$eig,10,1,2, posi="bottomright", ratio=.3, csub=1.25)
#'
#' ## axes 2-3
#' s.label(dapc1$grp.coord[,2:3], clab=0, sub="Axes 2-3")
#' par(xpd=T)
#' colorplot(dapc1$grp.coord[,2:3], dapc1$grp.coord, cex=3, add=TRUE)
#' add.scatter.eig(dapc1$eig,10,1,2, posi="bottomright", ratio=.3, csub=1.25)
#'
#'
#'
#' ## MAP DAPC1 RESULTS
#' if(require(maps)){
#'
#' xy <- cbind(eHGDP$other$popInfo$Longitude, eHGDP$other$popInfo$Latitude)
#'
#' par(mar=rep(.1,4))
#' map(fill=TRUE, col="lightgrey")
#' colorplot(xy, -dapc1$grp.coord, cex=3, add=TRUE, trans=FALSE)
#' }
#'
#'
#'
#' ## LOOK FOR OTHER CLUSTERS
#' ## to reproduce results of the reference paper, use :
#' ## grp <- find.clusters(eHGDP, max.n=50, n.pca=200, scale=FALSE)
#' ## and then
#' ## plot(grp$Kstat, type="b", col="blue")
#'
#' grp <- find.clusters(eHGDP, max.n=30, n.pca=200,
#' scale=FALSE, n.clust=4) # takes about 2 minutes
#' names(grp)
#'
#' ## (see ?find.clusters for details about the output)
#'
#'
#'
#' ## PERFORM DAPC - USE POPULATIONS AS CLUSTERS
#' ## to reproduce exactly analyses from the paper, use "n.pca=1000"
#' dapc2 <- dapc(eHGDP, pop=grp$grp, all.contrib=TRUE,
#' scale=FALSE, n.pca=200, n.da=80) # takes around a 1 minute
#' dapc2
#'
#'
#' ## PRODUCE SCATTERPLOT
#' scatter(dapc2) # axes 1-2
#' scatter(dapc2,2,3) # axes 2-3
#'
#'
#' ## MAP DAPC2 RESULTS
#' if(require(maps)){
#' xy <- cbind(eHGDP$other$popInfo$Longitude,
#' eHGDP$other$popInfo$Latitude)
#'
#' myCoords <- apply(dapc2$ind.coord, 2, tapply, pop(eHGDP), mean)
#'
#' par(mar=rep(.1,4))
#' map(fill=TRUE, col="lightgrey")
#' colorplot(xy, myCoords, cex=3, add=TRUE, trans=FALSE)
#' }
#'
#' }
#'
NULL






#' Seasonal influenza (H3N2) HA segment data
#'
#' The dataset \code{H3N2} consists of 1903 strains of seasonal influenza
#' (H3N2) distributed worldwide, and typed at 125 SNPs located in the
#' hemagglutinin (HA) segment. It is stored as an R object with class
#' \linkS4class{genind} and can be accessed as usual using \code{data(H3N2)}
#' (see example). These data were gathered from DNA sequences available from
#' Genbank (http://www.ncbi.nlm.nih.gov/Genbank/).
#'
#' The data file \code{usflu.fasta} is a toy dataset also gathered from
#' Genbank, consisting of the aligned sequences of 80 seasonal influenza
#' isolates (HA segment) sampled in the US, in \code{fasta} format. This file
#' is installed alongside the package; the path to this file is automatically
#' determined by R using \code{system.file} (see example in this manpage and in
#' ?fasta2genlight) as well.
#'
#'
#' @name H3N2
#' @aliases H3N2 usflu usflu.fasta USflu USflu.fasta
#' @docType data
#' @format \code{H3N2} is a genind object with several data frame as
#' supplementary components (\code{H3N2@@other) slort}, which contains the
#' following items: \describe{ \item{x}{a \code{data.frame} containing
#' miscellaneous annotations of the sequences.} \item{xy}{a matrix with two
#' columns indicating the geographic coordinates of the strains, as longitudes
#' and latitudes.} \item{epid}{a character vector indicating the epidemic of
#' the strains.} }
#' @references Jombart, T., Devillard, S. and Balloux, F. Discriminant analysis
#' of principal components: a new method for the analysis of genetically
#' structured populations. Submitted to \emph{BMC genetics}.
#' @source This dataset was prepared by Thibaut Jombart
#' (t.jombart@@imperia.ac.uk), from annotated sequences available on Genbank
#' (http://www.ncbi.nlm.nih.gov/Genbank/).
#' @keywords datasets
#' @examples
#'
#' \dontrun{
#' #### H3N2 ####
#' ## LOAD DATA
#' data(H3N2)
#' H3N2
#'
#' ## set population to yearly epidemics
#' pop(H3N2) <- factor(H3N2$other$epid)
#'
#'
#'
#' ## PERFORM DAPC - USE POPULATIONS AS CLUSTERS
#' ## to reproduce exactly analyses from the paper, use "n.pca=1000"
#' dapc1 <- dapc(H3N2, all.contrib=TRUE, scale=FALSE, n.pca=150, n.da=5)
#' dapc1
#'
#' ## (see ?dapc for details about the output)
#'
#'
#' ## SCREEPLOT OF EIGENVALUES
#' barplot(dapc1$eig, main="H3N2 - DAPC eigenvalues")
#'
#'
#' ## SCATTERPLOT (axes 1-2)
#' scatter(dapc1, posi.da="topleft", cstar=FALSE, cex=2, pch=17:22,
#' solid=.5, bg="white")
#'
#'
#'
#'
#' #### usflu.fasta ####
#' myPath <- system.file("files/usflu.fasta",package="adegenet")
#' myPath
#'
#' ## extract SNPs from alignments using fasta2genlight
#' ## see ?fasta2genlight for more details
#' obj <- fasta2genlight(myPath, chunk=10) # process 10 sequences at a time
#' obj
#' }
#'
NULL







#' Microsatellites genotypes of 15 cattle breeds
#'
#' This data set gives the genotypes of 704 cattle individuals for 30
#' microsatellites recommended by the FAO. The individuals are divided into two
#' countries (Afric, France), two species (Bos taurus, Bos indicus) and 15
#' breeds. Individuals were chosen in order to avoid pseudoreplication
#' according to their exact genealogy.
#'
#'
#' @name microbov
#' @docType data
#' @format \code{microbov} is a genind object with 3 supplementary components:
#' \describe{ \item{coun}{a factor giving the country of each individual (AF:
#' Afric; FR: France).} \item{breed}{a factor giving the breed of each
#' individual.} \item{spe}{is a factor giving the species of each individual
#' (BT: Bos taurus; BI: Bos indicus).} }
#' @references Lalo\"e D., Jombart T., Dufour A.-B. and Moazami-Goudarzi K.
#' (2007) Consensus genetic structuring and typological value of markers using
#' Multiple Co-Inertia Analysis. \emph{Genetics Selection Evolution}.
#' \bold{39}: 545--567.
#' @source Data prepared by Katayoun Moazami-Goudarzi and Denis Lalo\"e (INRA,
#' Jouy-en-Josas, France)
#' @keywords datasets
#' @examples
#'
#' \dontrun{
#' data(microbov)
#' microbov
#' summary(microbov)
#'
#' # make Y, a genpop object
#' Y <- genind2genpop(microbov)
#'
#' # make allelic frequency table
#' temp <- makefreq(Y,missing="mean")
#' X <- temp$tab
#' nsamp <- temp$nobs
#'
#' # perform 1 PCA per marker
#'
#' kX <- ktab.data.frame(data.frame(X),Y@@loc.n.all)
#'
#' kpca <- list()
#' for(i in 1:30) {kpca[[i]] <- dudi.pca(kX[[i]],scannf=FALSE,nf=2,center=TRUE,scale=FALSE)}
#'
#'
#' sel <- sample(1:30,4)
#' col = rep('red',15)
#' col[c(2,10)] = 'darkred'
#' col[c(4,12,14)] = 'deepskyblue4'
#' col[c(8,15)] = 'darkblue'
#'
#' # display %PCA
#' par(mfrow=c(2,2))
#' for(i in sel) {
#' s.multinom(kpca[[i]]$c1,kX[[i]],n.sample=nsamp[,i],coulrow=col,sub=locNames(Y)[i])
#' add.scatter.eig(kpca[[i]]$eig,3,xax=1,yax=2,posi="top")
#' }
#'
#' # perform a Multiple Coinertia Analysis
#' kXcent <- kX
#' for(i in 1:30) kXcent[[i]] <- as.data.frame(scalewt(kX[[i]],center=TRUE,scale=FALSE))
#' mcoa1 <- mcoa(kXcent,scannf=FALSE,nf=3, option="uniform")
#'
#' # coordinated %PCA
#' mcoa.axes <- split(mcoa1$axis, Y@@loc.fac)
#' mcoa.coord <- split(mcoa1$Tli,mcoa1$TL[,1])
#' var.coord <- lapply(mcoa.coord,function(e) apply(e,2,var))
#'
#' par(mfrow=c(2,2))
#' for(i in sel) {
#' s.multinom(mcoa.axes[[i]][,1:2],kX[[i]],n.sample=nsamp[,i],coulrow=col,sub=locNames(Y)[i])
#' add.scatter.eig(var.coord[[i]],2,xax=1,yax=2,posi="top")
#' }
#'
#' # reference typology
#' par(mfrow=c(1,1))
#' s.label(mcoa1$SynVar,lab=popNames(microbov),sub="Reference typology",csub=1.5)
#' add.scatter.eig(mcoa1$pseudoeig,nf=3,xax=1,yax=2,posi="top")
#'
#' # typologial values
#' tv <- mcoa1$cov2
#' tv <- apply(tv,2,function(c) c/sum(c))*100
#' rownames(tv) <- locNames(Y)
#' tv <- tv[order(locNames(Y)),]
#'
#' par(mfrow=c(3,1),mar=c(5,3,3,4),las=3)
#' for(i in 1:3){
#' barplot(round(tv[,i],3),ylim=c(0,12),yaxt="n",main=paste("Typological value -
#' structure",i))
#' axis(side=2,at=seq(0,12,by=2),labels=paste(seq(0,12,by=2),"%"),cex=3)
#' abline(h=seq(0,12,by=2),col="grey",lty=2)
#' }
#' }
#'
NULL





#' Microsatellites genotypes of 237 cats from 17 colonies of Nancy (France)
#'
#' This data set gives the genotypes of 237 cats (\emph{Felis catus} L.) for 9
#' microsatellites markers. The individuals are divided into 17 colonies whose
#' spatial coordinates are also provided.
#'
#'
#' @name nancycats
#' @docType data
#' @format \code{nancycats} is a genind object with spatial coordinates of the
#' colonies as a supplementary components (@@xy).
#' @references Devillard, S.; Jombart, T. & Pontier, D. Disentangling spatial
#' and genetic structure of stray cat (\emph{Felis catus} L.) colonies in urban
#' habitat using: not all colonies are equal. submitted to \emph{Molecular
#' Ecology}
#' @source Dominique Pontier (UMR CNRS 5558, University Lyon1, France)
#' @keywords datasets
#' @examples
#'
#' \dontrun{
#' data(nancycats)
#' nancycats
#'
#' # summary's results are stored in x
#' x <- summary(nancycats)
#'
#' # some useful graphics
#' barplot(x$loc.n.all,ylab="Alleles numbers",main="Alleles numbers
#' per locus")
#'
#' plot(x$pop.eff,x$pop.nall,type="n",xlab="Sample size",ylab="Number of alleles")
#' text(x$pop.eff,y=x$pop.nall,lab=names(x$pop.nall))
#'
#' par(las=3)
#' barplot(table(nancycats@@pop),ylab="Number of genotypes",main="Number of genotypes per colony")
#'
#' # are cats structured among colonies ?
#' if(require(hierfstat)){
#'
#' gtest <- gstat.randtest(nancycats,nsim=99)
#' gtest
#' plot(gtest)
#'
#'
#' dat <- genind2hierfstat(nancycats)
#'
#' Fstat <- varcomp.glob(dat$pop,dat[,-1])
#' Fstat
#' }
#' }
#'
NULL







#' Microsatellites genotypes of 335 chamois (Rupicapra rupicapra) from the
#' Bauges mountains (France)
#'
#' This data set contains the genotypes of 335 chamois (\emph{Rupicapra
#' rupicapra}) from the Bauges mountains, in France. No prior clustering about
#' individuals is known. Each genotype is georeferenced. These data also
#' contain a raster map of elevation of the sampling area.
#'
#'
#' @name rupica
#' @docType data
#' @encoding utf-8
#' @format \code{rupica} is a genind object with 3 supplementary components
#' inside the \code{@@other} slot: \describe{ \item{xy}{a matrix containing the
#' spatial coordinates of the genotypes.} \item{mnt}{a raster map of elevation,
#' with the \code{asc} format from the \code{adehabitat} package.}
#' \item{showBauges}{a function to display the map of elevation with an
#' appropriate legend (use \code{showBauges()}).} }
#' @references Cassar S (2008) Organisation spatiale de la variabilité
#' génétique et phénotypique a l'échelle du paysage: le cas du chamois et du
#' chevreuil, en milieu de montagne. PhD Thesis. University Claude Bernard -
#' Lyon 1, France. \cr
#'
#' Cassar S, Jombart T, Loison A, Pontier D, Dufour A-B, Jullien J-M, Chevrier
#' T, Maillard D. Spatial genetic structure of Alpine chamois (\emph{Rupicapra
#' rupicapra}): a consequence of landscape features and social factors?
#' submitted to \emph{Molecular Ecology}.
#' @source Daniel Maillard, 'Office National de la Chasse et de la Faune
#' Sauvage' (ONCFS), France.
#' @keywords datasets
#' @examples
#'
#'
#' data(rupica)
#' rupica
#'
#'
#' \dontrun{
#' if(require(adehabitat)){
#'
#' ## see the sampling area
#' showBauges <- rupica$other$showBauges
#' showBauges()
#' points(rupica$other$xy,col="red")
#'
#' ## perform a sPCA
#' spca1 <- spca(rupica,type=5,d1=0,d2=2300,plot=FALSE,scannf=FALSE,nfposi=2,nfnega=0)
#' barplot(spca1$eig,col=rep(c("black","grey"),c(2,100)),main="sPCA eigenvalues")
#' screeplot(spca1,main="sPCA eigenvalues: decomposition")
#'
#' ## data visualization
#' showBauges(,labcex=1)
#' s.value(spca1$xy,spca1$ls[,1],add.p=TRUE,csize=.5)
#' add.scatter.eig(spca1$eig,1,1,1,posi="topleft",sub="Eigenvalues")
#'
#' showBauges(,labcex=1)
#' s.value(spca1$xy,spca1$ls[,2],add.p=TRUE,csize=.5)
#' add.scatter.eig(spca1$eig,2,2,2,posi="topleft",sub="Eigenvalues")
#'
#' rupica$other$showBauges()
#' colorplot(spca1$xy,spca1$li,cex=1.5,add.plot=TRUE)
#'
#' ## global and local tests
#' Gtest <- global.rtest(rupica@@tab,spca1$lw,nperm=999)
#' Gtest
#' plot(Gtest)
#' Ltest <- local.rtest(rupica@@tab,spca1$lw,nperm=999)
#' Ltest
#' plot(Ltest)
#' }
#' }
#'
NULL








#' Simulated genotypes of two georeferenced populations
#'
#' This simple data set was obtained by sampling two populations evolving in a
#' island model, simulated using Easypop (2.0.1). See \code{source} for
#' simulation details. Sample sizes were respectively 100 and 30 genotypes. The
#' genotypes were given spatial coordinates so that both populations were
#' spatially differentiated.
#'
#'
#' @name sim2pop
#' @docType data
#' @format \code{sim2pop} is a genind object with a matrix of xy coordinates as
#' supplementary component.
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#' @references Balloux F (2001) Easypop (version 1.7): a computer program for
#' oppulation genetics simulations \emph{Journal of Heredity}, \bold{92}:
#' 301-302
#' @source Easypop version 2.0.1 was run with the following parameters:\cr -
#' two diploid populations, one sex, random mating\cr - 1000 individuals per
#' population\cr - proportion of migration: 0.002\cr - 20 loci\cr - mutation
#' rate: 0.0001 (KAM model)\cr - maximum of 50 allelic states\cr - 1000
#' generations (last one taken)\cr
#' @keywords datasets
#' @examples
#'
#' \dontrun{
#' data(sim2pop)
#'
#' if(require(hierfstat)){
#' ## try and find the Fst
#' temp <- genind2hierfstat(sim2pop)
#' varcomp.glob(temp[,1],temp[,-1])
#' # Fst = 0.038
#' }
#'
#' ## run monmonier algorithm
#'
#' # build connection network
#' gab <- chooseCN(sim2pop@@other$xy,ask=FALSE,type=2)
#'
#' # filter random noise
#' pca1 <- dudi.pca(sim2pop@@tab,scale=FALSE, scannf=FALSE, nf=1)
#'
#' # run the algorithm
#' mon1 <- monmonier(sim2pop@@other$xy,dist(pca1$l1[,1]),gab, scanthres=FALSE)
#'
#' # graphical display
#' temp <- sim2pop@@pop
#' levels(temp) <- c(17,19)
#' temp <- as.numeric(as.character(temp))
#' plot(mon1)
#' points(sim2pop@@other$xy,pch=temp,cex=2)
#' legend("topright",leg=c("Pop A", "Pop B"),pch=c(17,19))
#' }
#'
NULL






#' Simulated data illustrating the sPCA
#'
#' Datasets illustrating the spatial Principal Component Analysis (Jombart et
#' al. 2009).  These data were simulated using various models using Easypop
#' (2.0.1).  Spatial coordinates were defined so that different spatial
#' patterns existed in the data. The \code{spca-illus} is a list containing the
#' following \linkS4class{genind} or \linkS4class{genpop} objects:\cr - dat2A:
#' 2 patches \cr - dat2B: cline between two pop \cr - dat2C: repulsion among
#' individuals from the same gene pool \cr - dat3: cline and repulsion \cr -
#' dat4: patches and local alternance \cr
#'
#' See "source" for a reference providing simulation details.
#'
#'
#' @name spcaIllus
#' @docType data
#' @format \code{spcaIllus} is list of 5 components being either genind or
#' genpop objects.
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#' @seealso \code{\link{spca}}
#' @references Jombart, T., Devillard, S., Dufour, A.-B. and Pontier, D.
#' Revealing cryptic spatial patterns in genetic variability by a new
#' multivariate method. \emph{Heredity}, \bold{101}, 92--103.
#'
#' Balloux F (2001) Easypop (version 1.7): a computer program for oppulation
#' genetics simulations \emph{Journal of Heredity}, \bold{92}: 301-302
#' @source Jombart, T., Devillard, S., Dufour, A.-B. and Pontier, D. Revealing
#' cryptic spatial patterns in genetic variability by a new multivariate
#' method. \emph{Heredity}, \bold{101}, 92--103.
#' @keywords datasets spatial
#' @examples
#'
#' data(spcaIllus)
#' attach(spcaIllus)
#' opar <- par(no.readonly=TRUE)
#' ## comparison PCA vs sPCA
#'
#' # PCA
#' pca2A <- dudi.pca(dat2A$tab,center=TRUE,scale=FALSE,scannf=FALSE)
#' pca2B <- dudi.pca(dat2B$tab,center=TRUE,scale=FALSE,scannf=FALSE)
#' pca2C <- dudi.pca(dat2C$tab,center=TRUE,scale=FALSE,scannf=FALSE)
#' pca3 <- dudi.pca(dat3$tab,center=TRUE,scale=FALSE,scannf=FALSE,nf=2)
#' pca4 <- dudi.pca(dat4$tab,center=TRUE,scale=FALSE,scannf=FALSE,nf=2)
#'
#' # sPCA
#' spca2A <-spca(dat2A,xy=dat2A$other$xy,ask=FALSE,type=1,
#' plot=FALSE,scannf=FALSE,nfposi=1,nfnega=0)
#'
#' spca2B <- spca(dat2B,xy=dat2B$other$xy,ask=FALSE,type=1,
#' plot=FALSE,scannf=FALSE,nfposi=1,nfnega=0)
#'
#' spca2C <- spca(dat2C,xy=dat2C$other$xy,ask=FALSE,
#' type=1,plot=FALSE,scannf=FALSE,nfposi=0,nfnega=1)
#'
#' spca3 <- spca(dat3,xy=dat3$other$xy,ask=FALSE,
#' type=1,plot=FALSE,scannf=FALSE,nfposi=1,nfnega=1)
#'
#' spca4 <- spca(dat4,xy=dat4$other$xy,ask=FALSE,
#' type=1,plot=FALSE,scannf=FALSE,nfposi=1,nfnega=1)
#'
#' # an auxiliary function for graphics
#' plotaux <- function(x,analysis,axis=1,lab=NULL,...){
#' neig <- NULL
#' if(inherits(analysis,"spca")) neig <- nb2neig(analysis$lw$neighbours)
#' xrange <- range(x$other$xy[,1])
#' xlim <- xrange + c(-diff(xrange)*.1 , diff(xrange)*.45)
#' yrange <- range(x$other$xy[,2])
#' ylim <- yrange + c(-diff(yrange)*.45 , diff(yrange)*.1)
#'
#' s.value(x$other$xy,analysis$li[,axis],include.ori=FALSE,addaxes=FALSE,
#' cgrid=0,grid=FALSE,neig=neig,cleg=0,xlim=xlim,ylim=ylim,...)
#'
#' par(mar=rep(.1,4))
#' if(is.null(lab)) lab = gsub("[P]","",x$pop)
#' text(x$other$xy, lab=lab, col="blue", cex=1.2, font=2)
#' add.scatter({barplot(analysis$eig,col="grey");box();
#' title("Eigenvalues",line=-1)},posi="bottomright",ratio=.3)
#' }
#'
#' # plots
#' plotaux(dat2A,pca2A,sub="dat2A - PCA",pos="bottomleft",csub=2)
#' plotaux(dat2A,spca2A,sub="dat2A - sPCA glob1",pos="bottomleft",csub=2)
#'
#' plotaux(dat2B,pca2B,sub="dat2B - PCA",pos="bottomleft",csub=2)
#' plotaux(dat2B,spca2B,sub="dat2B - sPCA glob1",pos="bottomleft",csub=2)
#'
#' plotaux(dat2C,pca2C,sub="dat2C - PCA",pos="bottomleft",csub=2)
#' plotaux(dat2C,spca2C,sub="dat2C - sPCA loc1",pos="bottomleft",csub=2,axis=2)
#'
#' par(mfrow=c(2,2))
#' plotaux(dat3,pca3,sub="dat3 - PCA axis1",pos="bottomleft",csub=2)
#' plotaux(dat3,spca3,sub="dat3 - sPCA glob1",pos="bottomleft",csub=2)
#' plotaux(dat3,pca3,sub="dat3 - PCA axis2",pos="bottomleft",csub=2,axis=2)
#' plotaux(dat3,spca3,sub="dat3 - sPCA loc1",pos="bottomleft",csub=2,axis=2)
#'
#' plotaux(dat4,pca4,lab=dat4$other$sup.pop,sub="dat4 - PCA axis1",
#' pos="bottomleft",csub=2)
#' plotaux(dat4,spca4,lab=dat4$other$sup.pop,sub="dat4 - sPCA glob1",
#' pos="bottomleft",csub=2)
#' plotaux(dat4,pca4,lab=dat4$other$sup.pop,sub="dat4 - PCA axis2",
#' pos="bottomleft",csub=2,axis=2)
#' plotaux(dat4,spca4,lab=dat4$other$sup.pop,sub="dat4 - sPCA loc1",
#' pos="bottomleft",csub=2,axis=2)
#'
#' # color plot
#' par(opar)
#' colorplot(spca3, cex=4, main="colorplot sPCA dat3")
#' text(spca3$xy[,1], spca3$xy[,2], dat3$pop)
#'
#' colorplot(spca4, cex=4, main="colorplot sPCA dat4")
#' text(spca4$xy[,1], spca4$xy[,2], dat4$other$sup.pop)
#'
#' # detach data
#' detach(spcaIllus)
#'
#'
NULL



#' Toy hybrid dataset
#' @name hybridtoy
#' @aliases hybridtoy
#' @docType data
#' @format a \linkS4class{genind} object
#' @author Data simulated by  Marie-Pauline Beugin. Example by Thibaut Jombart.
#'
#' @examples
#' data(hybridtoy)
#' x <- hybridtoy
#' pca1 <- dudi.pca(tab(x), scannf=FALSE, scale=FALSE)
#' s.class(pca1$li, pop(x))
#'
#' if(require(ggplot2)) {
#' p <- ggplot(pca1$li, aes(x=Axis1)) +
#'     geom_density(aes(fill=pop(x)), alpha=.4, adjust=1) +
#'     geom_point(aes(y=0, color=pop(x)), pch="|", size=10, alpha=.5)
#' p
#' }
#'
#' ## kmeans
#' km <- find.clusters(x, n.pca=10, n.clust=2)
#' table(pop(x), km$grp)
#'
#' ## dapc
#' dapc1 <- dapc(x, pop=km$grp, n.pca=10, n.da=1)
#' scatter(dapc1)
#' scatter(dapc1, grp=pop(x))
#' compoplot(dapc1, col.pal=spectral, n.col=2)
#'
#' ## ML-EM with hybrids
#' res <- snapclust(x, k=2, hybrids=TRUE, detailed=TRUE)
#' compoplot(res, n.col=3)
#' table(res$group, pop(x))
#'
NULL

#' @title Distance matrices from genlight objects
#' 
#' @name dist.genlight
#' @rdname dist.genlight
#' 
#' @description 
#' Create distance matrices from genlight objects
#' 
#' @details 
#' 
#' The creation of distance matrices, matrices of numbers that describe how different each of the samples are, is a fundamental task in the statistical analysis of individuals or populations (i.e., groups of individuals).
#' However, there isn't actually a function that creates distance matrices from genlight objects in adegenet.
#' Instead, the authors of adegenet created an `as.matrix()` function that converts a genlight object to a matrix.
#' This is clever because the function `dist()` in the package `stats` tries to convert whatever object it is given to a matrix.
#' The result is that when you call `dist()` on a genlight object it uses the `dist()` function to create a distance matrix.
#' The reason this is clever is because it uses pre-existing code.
#' The downside is that because there is no function to specifically create distance matrices from genlight objects in adegenet, there is no documentation in genlight for how this is done.
#' And because the author of `dist()` never anticipated it could be used on genlight objects, there is no documentation for it there either.
#' And we can find documentation for this function with `?dist`.
#' To summarize, we can create a distance matrix from a genlight object using `dist()`.
#' 
#'  
#' There are also functions to create distance matrices from genlight objects that exist in other packages.
#' The function `bitwise.dist()` in the package [poppr](https://CRAN.R-project.org/package=poppr) is an example.
#' We can find documentation for this function with `?poppr::bitwise.dist`.
#' Again, a downside of this is that you need to know where to look for this information or you may not find it.
#' 
#' 
#' Lastly, because you can use `as.matrix()` on your genlight object, and most distance algorithms can use this matrix as input, you can use this as an intermediate step to create a matrix from your genlight object and pass it to your distance algorithm of choice.
#' Options include [ade4](https://CRAN.R-project.org/package=ade4), `vegdist()` in [vegan](https://CRAN.R-project.org/package=vegan), or `daisy()` in [cluster](https://CRAN.R-project.org/package=cluster).
#' Note that it is up to you to determine which distance metric is best for your analysis.
#' A number of options therefore exist for creating distance matrices from genlight objects.

########################################################################
# strata methods definitions. 
#
# Zhian Kamvar, March 2015
# kamvarz@science.oregonstate.edu
########################################################################
#==============================================================================#
#==============================================================================#
# Workhorse functions for strata methods. These avoids needless copying and
# pasting of code to dispatch the methods to two unrelated classes 
# (genind and genlight).
#==============================================================================#
#==============================================================================#
.getStrata <- function(x, formula = NULL, combine = TRUE, call = match.call()){
  if (is.null(x@strata)) return(NULL)
  if (is.null(formula)) return(x@strata)
  vars <- all.vars(formula)
  if (any(!vars %in% names(x@strata))){
    stop(.strata_incompatible_warning(vars, x@strata), call. = FALSE)
  }
  if (combine){
    strata <- .make_strata(formula, x@strata)
  } else {
    strata <- x@strata[all.vars(formula)]
  }
  invisible(return(strata))
}

.setStrata <- function(x, value, call = match.call()){
  if (is.null(value)){
    x@strata <- value
    return(x)
  }
  if (!inherits(value, "data.frame")){
    callval <- as.character(call["value"])
    stop(paste(callval, "is not a data frame"), call. = FALSE)
  }
  if (nrow(value) != nInd(x)){
    stop("Number of rows in data frame not equal to number of individuals in object.", call. = FALSE)
  }
  value <- data.frame(lapply(value, function(f) factor(f, unique(f))))
  x@strata <- value
  return(x)
}

.nameStrata <- function(x, value, call = match.call()){
  if (missing(value)){
    return(names(x@strata))
  }
  if (is.null(x@strata)){
    warning("Cannot name an empty strata", call. = FALSE)
    return(x)
  }
  if (is.language(value)){
    value <- all.vars(value)
  }
  if (!is.vector(value) | length(value) != length(x@strata)){
    stop(paste("nameStrata needs a vector argument of length", length(x@strata)), call. = FALSE)
  }
  names(x@strata) <- value
  return(x)
}

.splitStrata <- function(x, value, sep = "_", call = match.call()){
  if (is.null(x@strata)){
    warning("Cannot split empty strata", call. = FALSE)
    return(x)
  }
  if (is.language(value)){
    # valterms <- attr(terms(value), "term.labels")
    # valterms <- valterms[length(valterms)]
    # valterms <- gsub(":", sep, valterms)
    value    <- all.vars(value)
  } else {
    stop("Value must be a formula.", call. = FALSE)
  }
  if (length(value) < 1){
    stop("Value must have more than one level.", call. = FALSE)
  }
  strata  <- x@strata
  if (length(strata) > 1){
    warning("Strata must be length 1. Taking the first column.", call. = FALSE)
    strata <- strata[1]
  }
  seps     <- gregexpr(sep, strata[[1]])
  sepmatch <- vapply(seps, function(val) all(as.integer(val) > 0), logical(1))
  seps     <- vapply(seps, length, numeric(1))
  all_seps_match <- all(sepmatch)
  given_seps     <- length(value) - 1 
  if (!all_seps_match | all(seps != given_seps)){
    seps <- ifelse(all_seps_match, seps[1], 0) + 1
    msg1 <- paste("\n  Data has", seps, ifelse(seps == 1, "level", "levels"),
                  "of strata with the separator", sep, ".")
    msg2 <- paste("Here is the fist column of the data:", strata[1, ])
    stop(paste(msg1, "\n ", msg2), call. = FALSE)
  }
  x@strata <- colsplit(as.character(strata[[1]]), pattern = sep, value)
  x@strata <- data.frame(lapply(x@strata, function(f) factor(f, levels = unique(f))))
  # names(strata) <- value
  # x@strata      <- strata
  return(x) 
}

.addStrata <- function(x, value, name = "NEW", call = match.call()){
  if (is.null(x@strata)){
    strata <- data.frame(vector(mode = "character", length = nInd(x)))
    wasNULL <- TRUE
  } else {
    strata  <- x@strata  
    wasNULL <- FALSE    
  }

  if ((is.vector(value) | is.factor(value)) & length(value) == nrow(strata)){
    value <- factor(value, levels = unique(value))
    NEW <- data.frame(value)
    names(NEW) <- name
    strata <- cbind(strata, NEW)
  } else if (is.data.frame(value) && nrow(value) == nrow(strata)){
    value <- data.frame(lapply(value, function(f) factor(f, unique(f))))
    strata <- cbind(strata, value)
  } else if (is.data.frame(value)){
    callval <- as.character(call["value"])
    msg     <- .not_enough_rows_warning(callval, nrow(value), nrow(strata))
    stop(msg, call. = FALSE)
  } else {
    stop("value must be a vector or data frame.", call. = FALSE)
  }
  if (wasNULL){
    strata <- strata[-1, , drop = FALSE]
  }
  x@strata <- strata
  return(x) 
}

.setPop <- function(x, formula = NULL, call = match.call()){
  if (is.null(x@strata)){
    warning("Cannot set the population from an empty strata", call. = FALSE)
    return(x)
  }
  if (is.null(formula) | !is.language(formula)){
    callform <- as.character(call["formula"])
    stop(paste(callform, "must be a valid formula object."), call. = FALSE)
  }
  vars <- all.vars(formula)
  if (!all(vars %in% names(x@strata))){
    stop(.strata_incompatible_warning(vars, x@strata), call. = FALSE)
  }
  pop(x) <- .make_strata(formula, x@strata)[[length(vars)]]
  return(x)
}

#==============================================================================#
#==============================================================================#
# Formal methods definitions and documentation.
#==============================================================================#
#==============================================================================#

#==============================================================================#
#' Access and manipulate the population strata for genind or genlight objects.
#' 
#' The following methods allow the user to quickly change the strata of a genind
#' or genlight object.
#' 
#' @export 
#' @rdname strata-methods
#' @aliases strata,genind-method strata,genlight-method
#' @param x a genind or genlight object
#' @param formula a nested formula indicating the order of the population
#' strata.
#' @param combine if \code{TRUE} (default), the levels will be combined according to the
#' formula argument. If it is \code{FALSE}, the levels will not be combined.
#' @param value a data frame OR vector OR formula (see details).
#' @docType methods
#'   
#' @details \subsection{Function Specifics}{ \itemize{ \item \strong{strata()} -
#'   Use this function to view or define population stratification of a 
#'   \linkS4class{genind} or \linkS4class{genlight} object. \item 
#'   \strong{nameStrata()} - View or rename the different levels of strata. 
#'   \item \strong{splitStrata()} - Split strata that are combined with a common
#'   separator. This function should only be used once during a workflow. 
#'   \itemize{ \item \emph{Rationale:} It is often difficult to import files 
#'   with several levels of strata as most data formats do not allow unlimited 
#'   population levels. This is circumvented by collapsing all population strata
#'   into a single population factor with a common separator for each 
#'   observation. } \item \strong{addStrata()} - Add levels to your population 
#'   strata. This is ideal for adding groups defined by 
#'   \code{\link{find.clusters}}. You can input a data frame or a vector, but if
#'   you put in a vector, you have the option to name it. }}
#'   
#'   \subsection{Argument Specifics}{
#'   
#'   These functions allow the user to seamlessly carry all possible population 
#'   stratification with their \linkS4class{genind} or \linkS4class{genlight} 
#'   object. Note that there are two ways of performing all methods: \itemize{ 
#'   \item modifying: \code{strata(myData) <- myStrata} \item preserving: 
#'   \code{myNewData <- strata(myData, value = myStrata)} } They essentially do 
#'   the same thing except that the modifying assignment method (the one with 
#'   the "\code{<-}") will modify the object in place whereas the non-assignment
#'   method will preserve the original object (unless you overwrite it). Due to 
#'   convention, everything right of the assignment is termed \code{value}. To 
#'   avoid confusion, here is a guide to the argument \strong{\code{value}} for 
#'   each function: \itemize{ \item \strong{strata()} \code{value = }a 
#'   \code{\link{data.frame}} that defines the strata for each individual in the
#'   rows. \item \strong{nameStrata()} \code{value = }a \code{\link{vector}} or 
#'   a \code{\link{formula}} that will define the names. \item 
#'   \strong{splitStrata()} \code{value = }a \code{\link{formula}} argument with
#'   the same number of levels as the strata you wish to split. \item 
#'   \strong{addStrata()} \code{value = }a \code{\link{vector}} or 
#'   \code{\link{data.frame}} with the same length as the number of individuals 
#'   in your data. }}
#'   
#'   \subsection{Details on Formulas}{
#'   
#'   The preferred use of these functions is with a \code{\link{formula}} 
#'   object. Specifically, a hierarchical formula argument is used to assign the
#'   levels of the strata. An example of a hierarchical formula would
#'   be:\tabular{r}{ \code{~Country/City/Neighborhood}} This convention was
#'   chosen as it becomes easier to type and makes intuitive sense when defining
#'   a \code{\link{hierarchy}}. Note: it is important to use hiearchical
#'   formulas when specifying hierarchies as other types of formulas (eg. 
#'   \code{~Country*City*Neighborhood}) will give incorrect results.}
#'   
#' @seealso \code{\link{setPop}} \code{\link{genind}}
#'   \code{\link{as.genind}}
#'   
#' @author Zhian N. Kamvar
#' @examples
#' # let's look at the microbov data set:
#' data(microbov)
#' microbov
#' 
#' # We see that we have three vectors of different names in the 'other' slot. 
#' # ?microbov
#' # These are Country, Breed, and Species
#' names(other(microbov))
#' 
#' # Let's set the strata
#' strata(microbov) <- data.frame(other(microbov))
#' microbov
#' 
#' # And change the names so we know what they are
#' nameStrata(microbov) <- ~Country/Breed/Species
#' 
#' \dontrun{
#' # let's see what the strata looks like by Species and Breed:
#' head(strata(microbov, ~Breed/Species))
#' 
#' # If we didn't want the last column combined with the first, we can set
#' # combine = FALSE
#' head(strata(microbov, ~Breed/Species, combine = FALSE))
#' 
#' #### USING splitStrata ####
#' 
#' # For the sake of example, we'll imagine that we have imported our data set
#' # with all of the stratifications combined. 
#' setPop(microbov) <- ~Country/Breed/Species
#' strata(microbov) <- NULL
#' 
#' # This is what our data would look like after import.
#' microbov
#' 
#' # To set our strata here, we need to use the functions strata and splitStrata
#' strata(microbov) <- data.frame(x = pop(microbov))
#' microbov # shows us that we have "one" level of stratification
#' head(strata(microbov)) # all strata are separated by "_"
#' 
#' splitStrata(microbov) <- ~Country/Breed/Species
#' microbov # Now we have all of our strata named and split
#' head(strata(microbov)) # all strata are appropriately named and split.
#' }
#==============================================================================#
strata <- function(x, formula = NULL, combine = TRUE, value){
  standardGeneric("strata")
} 

#' @export
setGeneric("strata")

setMethod(
  f = "strata",
  signature(x = "genind"),
  definition = function(x, formula = NULL, combine = TRUE, value){
    theCall <- match.call()
    if (missing(value)){
      .getStrata(x, formula = formula, combine = combine, theCall)  
    } else {
      .setStrata(x, value, theCall)
    }
  })

setMethod(
  f = "strata",
  signature(x = "genlight"),
  definition = function(x, formula = NULL, combine = TRUE, value){
    theCall <- match.call()
    if (missing(value)){
      .getStrata(x, formula = formula, combine = combine, theCall)  
    } else {
      .setStrata(x, value, theCall)
    }
    
  })


#==============================================================================#
#' @export 
#' @rdname strata-methods
#' @aliases strata<-,genind-method strata<-,genlight-method
#' @docType methods
#==============================================================================#
"strata<-" <- function(x, value){
  standardGeneric("strata<-")
}  

#' @export
setGeneric("strata<-")

setMethod(
  f = "strata<-",
  signature(x = "genind"),
  definition = function(x, value){
    theCall <- match.call()
    return(.setStrata(x, value, theCall))
  })

setMethod(
  f = "strata<-",
  signature(x = "genlight"),
  definition = function(x, value){
    theCall <- match.call()
    return(.setStrata(x, value, theCall))
  })

#==============================================================================#
#' @export 
#' @rdname strata-methods
#' @aliases nameStrata,genind-method nameStrata,genlight-method
#' @docType methods
#==============================================================================#
nameStrata <- function(x, value){
  standardGeneric("nameStrata")
}  

#' @export
setGeneric("nameStrata")

setMethod(
  f = "nameStrata",
  signature(x = "genind"),
  definition = function(x, value){
    .nameStrata(x, value)
  })

setMethod(
  f = "nameStrata",
  signature(x = "genlight"),
  definition = function(x, value){
    .nameStrata(x, value)
  })

#==============================================================================#
#' @export 
#' @rdname strata-methods
#' @aliases nameStrata<-,genind-method nameStrata<-,genlight-method
#' @docType methods
#==============================================================================#
"nameStrata<-" <- function(x, value){
  standardGeneric("nameStrata<-")
}  

#' @export
setGeneric("nameStrata<-")

setMethod(
  f = "nameStrata<-",
  signature(x = "genind"),
  definition = function(x, value){
    return(nameStrata(x, value))
  })

setMethod(
  f = "nameStrata<-",
  signature(x = "genlight"),
  definition = function(x, value){
    return(nameStrata(x, value))
  })
#==============================================================================#
#' @export 
#' @rdname strata-methods
#' @aliases splitStrata,genind-method splitStrata,genlight-method
#' @docType methods
#' @param sep a \code{character} indicating the character used to separate
#' hierarchical levels. This defaults to "_".
#' @importFrom reshape2 colsplit
#==============================================================================#
splitStrata <- function(x, value, sep = "_"){
  standardGeneric("splitStrata")
}  

#' @export
setGeneric("splitStrata")

setMethod(
  f = "splitStrata",
  signature(x = "genind"),
  definition = function(x, value, sep = "_"){
    .splitStrata(x, value, sep = sep) 
  })

setMethod(
  f = "splitStrata",
  signature(x = "genlight"),
  definition = function(x, value, sep = "_"){
    .splitStrata(x, value, sep = sep) 
  })

#==============================================================================#
#' @export 
#' @rdname strata-methods
#' @aliases splitStrata<-,genind-method splitStrata<-,genlight-method
#' @docType methods
#==============================================================================#
"splitStrata<-" <- function(x, sep = "_", value){
  standardGeneric("splitStrata<-")
}  

#' @export
setGeneric("splitStrata<-")

setMethod(
  f = "splitStrata<-",
  signature(x = "genind"),
  definition = function(x, sep = "_", value){
    return(splitStrata(x, value, sep))
  })

setMethod(
  f = "splitStrata<-",
  signature(x = "genlight"),
  definition = function(x, sep = "_", value){
    return(splitStrata(x, value, sep))
  })

#==============================================================================#
#' @export 
#' @rdname strata-methods
#' @aliases addStrata,genind-method addStrata,genlight-method
#' @param name an optional name argument for use with addStrata if supplying
#'   a vector. Defaults to "NEW".
#' @docType methods
#==============================================================================#
addStrata <- function(x, value, name = "NEW"){
  standardGeneric("addStrata")
}  

#' @export
setGeneric("addStrata")

setMethod(
  f = "addStrata",
  signature(x = "genind"),
  definition = function(x, value, name = "NEW"){
    theCall <- match.call()
    .addStrata(x, value, name = name, theCall)
  })

setMethod(
  f = "addStrata",
  signature(x = "genlight"),
  definition = function(x, value, name = "NEW"){
    theCall <- match.call()
    .addStrata(x, value, name = name, theCall)
  })

#==============================================================================#
#' @export 
#' @rdname strata-methods
#' @aliases addStrata<-,genind-method addStrata<-,genlight-method
#' @docType methods
#==============================================================================#
"addStrata<-" <- function(x, name = "NEW", value){
  standardGeneric("addStrata<-")
}  

#' @export
setGeneric("addStrata<-")

setMethod(
  f = "addStrata<-",
  signature(x = "genind"),
  definition = function(x, name = "NEW", value){
    theCall <- match.call()
    return(.addStrata(x, value, name = name, theCall))
  })

setMethod(
  f = "addStrata<-",
  signature(x = "genlight"),
  definition = function(x, name = "NEW", value){
    theCall <- match.call()
    return(.addStrata(x, value, name = name, theCall))
  })

#==============================================================================#
#' Manipulate the population factor of genind objects.
#' 
#' The following methods allow the user to quickly change the population of a 
#' genind object. 
#' 
#' @export 
#' @rdname population-methods
#' @param x a genind or genlight object
#' @param formula a nested formula indicating the order of the population
#' strata.
#' @param value same as formula
#' @aliases setPop,genind-method setPop,genlight-method
#' @docType methods 
#' @author Zhian N. Kamvar
#' @examples
#' 
#' data(microbov)
#' 
#' strata(microbov) <- data.frame(other(microbov))
#' 
#' # Currently set on just 
#' head(pop(microbov)) 
#' 
#' # setting the strata to both Pop and Subpop
#' setPop(microbov) <- ~coun/breed 
#' head(pop(microbov))
#' 
#' \dontrun{
#' 
#' # Can be used to create objects as well.
#' microbov.old <- setPop(microbov, ~spe) 
#' head(pop(microbov.old))
#' }
#==============================================================================#
setPop <- function(x, formula = NULL) standardGeneric("setPop")

#' @export
setGeneric("setPop")

setMethod(
  f = "setPop",
  signature(x = "genind"),
  definition = function(x, formula = NULL){
    .setPop(x, formula = formula)
  })

setMethod(
  f = "setPop",
  signature(x = "genlight"),
  definition = function(x, formula = NULL){
    .setPop(x, formula = formula)
  })
#==============================================================================#
#' @export
#' @rdname population-methods
#' @aliases setPop<-,genind-method setPop<-,genlight-method
#' @docType methods
#==============================================================================#
"setPop<-" <- function(x, value) standardGeneric("setPop<-")

#' @export
setGeneric("setPop<-")

setMethod(
  f = "setPop<-",
  signature(x = "genind"),
  definition = function(x, value){
    return(setPop(x, value))
  })

setMethod(
  f = "setPop<-",
  signature(x = "genlight"),
  definition = function(x, value){
    return(setPop(x, value))
  })

#==============================================================================#
#==============================================================================#
# internal functions utilized
#==============================================================================#
#==============================================================================#

#==============================================================================#
# A function for creating a population strata using a formula and data frame
# 
# hier = a nested formula such as ~ A/B/C where C is nested within B, which is
# nested within A.
#
# df = a data frame containing columns corresponding to the variables in hier.
#
# example:
# df <- data.frame(list(a = letters, b = LETTERS, c = 1:26))
# newdf <- .make_strata(~ a/b/c, df)
# df[names(newdf)] <- newdf # Add new columns.
#
# Public functions utilizing this function:
#
# # setPop, strata
#
# Internal functions utilizing this function:
# # none
#==============================================================================#
.make_strata <- function(strata, df, expand_label = FALSE){
  newlevs <- attr(terms(strata), "term.labels")
  levs <- all.vars(strata)
  if (length(levs) > 1){
    newlevs <- gsub(":", "_", newlevs)
  }
  if (!all(levs %in% names(df))){
    stop(.strata_incompatible_warning(levs, df), call. = FALSE)
  }
  newdf <- df[levs[1]]
  if (!expand_label){
    newlevs <- levs
  }
  lapply(1:length(levs), function(x) newdf[[newlevs[x]]] <<- as.factor(.pop_combiner(df, levs[1:x])))
  return(newdf)
}

#==============================================================================#
# This will be used to join heirarchical population vectors for the purposes of
# maintaining strata. 
# Public functions utilizing this function:
# # 
#
# Internal functions utilizing this function:
# # .make_strata
#==============================================================================#

.pop_combiner <- function(df, strata=c(1), sep="_"){
  if(!is.list(df)){
    warning("df must be a data frame or a list", call. = FALSE)
    return(df)
  }
  else{
    if(length(strata)==1){
      return(df[[strata]])
    }
    else{
      comb <- vector(length=length(df[[strata[1]]]))
      comb <- df[[strata[1]]]
      lapply(strata[-1], function(x) comb <<- paste(comb, df[[x]], sep=sep))
      return(comb)
    }
  }
}

#==============================================================================#
# A function that will quit the function if a level in the strata is not
# present in the given data frame.
#
# Public functions utilizing this function:
# # setPop strata poppr.amova
#
# Internal functions utilizing this function:
# # .make_strata make_ade_df
#==============================================================================#
.strata_incompatible_warning <- function(levs, df){
  msg <- paste("One or more levels in the given strata is not present", 
               "in the data frame.\n", paste(rep("-", 78), collapse = ""),
               "\nstrata:\t", paste(levs, collapse = ", "), "\nData:\t", 
               paste(names(df), collapse = ", "))
  return(msg)
}

#==============================================================================#
# A function that will send a message if the number of rows in a data frame does
# not match what is expected.
#
# Public functions utilizing this function:
# # addStrata
#
# Internal functions utilizing this function:
# # .addStrata
#==============================================================================#
.not_enough_rows_warning <- function(dfname, dfrow, nind){
  msg <- paste("The data frame or vector does not have enough rows\n", 
               paste(rep("-", 78), collapse = ""),
               "\n", dfname, ":\t", dfrow, "\nData :\t", nind)
  return(msg)
}

context("Test seppop")

data(microbov)

test_that("seppop will use the internal population factor by default", {
  skip_on_cran()
  blist <- seppop(microbov)
  expect_is(blist, "list")
  expect_equal(length(blist), nPop(microbov))
  expect_equivalent(names(blist), popNames(microbov))
})

test_that("seppop will use the external population factor", {
  skip_on_cran()
  coun <- other(microbov)$coun
  clist <- seppop(microbov, pop = coun)
  expect_is(clist, "list")
  expect_equal(length(clist), nlevels(coun))
  expect_equivalent(names(clist), levels(coun))
})

test_that("seppop will use formula input", {
  skip_on_cran()
  strata(microbov) <- data.frame(other(microbov))
  slist <- seppop(microbov, pop = ~spe)
  setPop(microbov) <- ~spe
  expect_is(slist, "list")
  expect_equal(length(slist), nPop(microbov))
  expect_equivalent(names(slist), popNames(microbov))
})



x <- new("genlight", list(a=rep(1,1e3),b=rep(0,1e3),c=rep(1, 1e3)), parallel = FALSE)
pop(x) <- c("pop1","pop2", "pop1")

test_that("seppop will work for genlight objects", {
  skip_on_cran()
  plist <- seppop(x)
  expect_is(plist, "list")
  expect_equal(length(plist), nPop(x))
  expect_equivalent(names(plist), popNames(x))
})

test_that("seppop will work for genlight objects with external factor", {
  skip_on_cran()
  uniqpop <- rev(LETTERS)[1:3]
  ulist <- seppop(x, pop = uniqpop)
  expect_is(ulist, "list")
  expect_equal(length(ulist), length(uniqpop))
  expect_equivalent(names(ulist), sort(uniqpop))
})

test_that("seppop will work for genlight objects with formula", {
  skip_on_cran()
  uniqpop <- rev(LETTERS)[1:3]
  strata(x) <- data.frame(pop = pop(x), uni = uniqpop, all = rep("A", 3))
  setPop(x) <- ~all
  alist <- seppop(x, pop = ~all)
  expect_is(alist, "list")
  expect_equal(length(alist), nPop(x))
  expect_equivalent(names(alist), popNames(x))
})

context("Repool tests")

data("microbov")
strata(microbov) <- data.frame(other(microbov))

test_that("slots are equivalent", {
  skip_on_cran()
  temp   <- seppop(microbov)
  retemp <- repool(temp)
  expect_null(other(retemp))
  expect_failure(expect_identical(microbov@call, retemp@call))
  expect_equal(names(microbov), names(retemp))

  # Alleles in repooled samples are out of order.
  # This makes sure they are ordered.
  retempallnames <- lapply(retemp@all.names, sort)
  retemptab      <- retemp@tab[, colnames(microbov@tab)]

  expect_equivalent(slot(microbov, 'tab'), retemptab)
  expect_equivalent(slot(microbov, 'all.names'), retempallnames)
  expect_equivalent(slot(microbov, 'strata'), slot(retemp, 'strata'))
  expect_equivalent(slot(microbov, 'hierarchy'), slot(retemp, 'hierarchy'))
  expect_equivalent(slot(microbov, 'loc.fac'), slot(retemp, 'loc.fac'))
  expect_equivalent(slot(microbov, 'loc.n.all'), slot(retemp, 'loc.n.all'))
  expect_equivalent(slot(microbov, 'pop'), slot(retemp, 'pop'))
  expect_equivalent(slot(microbov, 'ploidy'), slot(retemp, 'ploidy'))
  expect_equivalent(slot(microbov, 'type'), slot(retemp, 'type'))
  
})

context("Strata methods")

test_that("strata methods work for genind objects.", {
  skip_on_cran()

  data(microbov, package = "adegenet")
  expect_null(strata(microbov))
  strata(microbov) <- data.frame(other(microbov))
  breeds <- c("Borgou", "Zebu", "Lagunaire", "NDama", "Somba", "Aubrac", "Bazadais",
              "BlondeAquitaine", "BretPieNoire", "Charolais", "Gascon", "Limousin",
              "MaineAnjou", "Montbeliard", "Salers")

  expect_equal(length(strata(microbov)), 3)
  expect_equal(popNames(microbov), breeds)
  expect_warning(
    expect_error({
      microbovsplit <- splitStrata(microbov, ~Pop/Subpop)
      })
    )
  nameStrata(microbov) <- ~Country/Breed/Species
  expect_equal(names(strata(microbov)), c("Country", "Breed", "Species"))
  setPop(microbov) <- ~Country/Species
  expect_equal(popNames(microbov), c("AF_BI", "AF_BT", "FR_BT"))
})

test_that("strata produce proper errors", {
  skip_on_cran()
  expect_warning(setPop(microbov, ~bippity/boppity/boo))
  strata(microbov) <- data.frame(other(microbov))
  expect_error({strata(microbov) <- data.frame(a = 1)})
  expect_error({addStrata(microbov) <- data.frame(a = 1:10)})
  expect_error(setPop(microbov, ~bippity/boppity/boo))
  expect_error({strata(microbov) <- "a stratum"})
  expect_error({setPop(microbov) <- "thepop"})
})

test_that("strata methods work for genlight objects", {
  skip_on_cran()

  michier <- data.frame(other(microbov))
  make_gl <- function(n = 10, hier = michier){
    objs <- lapply(seq(n), function(x) sample(c(0, 1, NA), 10, replace = TRUE, prob = c(0.49, 0.49, 0.01)))
    return(new("genlight", objs, strata = hier[sample(704, 10), sample(3, 2)], parallel = FALSE))
  }
  set.seed(9999)
  glTest <- lapply(1:10, function(x, y, z) make_gl(y, z), 10, michier)
  res <- do.call("rbind.genlight", c(glTest, parallel = FALSE))
  expect_is(res, "genlight")
  expect_equal(nInd(res), 100)
  expect_equal(nLoc(res), 10)
  expect_equal(length(strata(res)), 3)
  nameStrata(res) <- ~Hickory/Dickory/Doc
  expect_equal(names(strata(res)), c("Hickory", "Dickory", "Doc"))
})

#############
# S4 methods
#############
setAs("genind", "data.frame", function(from, to) {
    return(as.data.frame(from@tab))
})



setAs("genpop", "data.frame", function(from, to) {
    return(as.data.frame(from@tab))
})



setAs("genind", "matrix", function(from, to) {
    return(from@tab)
})



setAs("genpop", "matrix", function(from, to) {
    return(from@tab)
})



setAs("genind", "genpop", function(from, to) {
    if(!is.genind(from)) stop("object is not a valid genind")

    x <- genind2genpop(from, quiet=TRUE)
    warning("You had better use genind2genpop to specify treatment of NAs")

    return(x@tab)
})




setOldClass("ktab")
setAs("genind", "ktab", function(from, to) {
    checkType(from)
    res <- ktab.data.frame(df=as.data.frame(from), blocks=from@loc.n.all, rownames=indNames(from),
                           colnames=unlist(alleles(from)), tabnames=locNames(from))
    return(res)
})




setAs("genpop", "ktab", function(from, to) {
    checkType(from)
    res <- ktab.data.frame(df=as.data.frame(from), blocks=from@loc.n.all, rownames=popNames(from),
                           colnames=unlist(alleles(from)), tabnames=locNames(from))
    return(res)
})




##############
# S3 versions
##############
as.data.frame.genind <- function(x,...){
    return(as(x,"data.frame"))
}



as.data.frame.genpop <- function(x,...){
    return(as(x,"data.frame"))
}



as.matrix.genind <- function(x,...){
    return(as(x,"matrix"))
}



as.matrix.genpop <- function(x,...){
    return(as(x,"matrix"))
}



as.genpop.genind <- function(x,...){
    return(as(x,"genpop"))
}




as.ktab.genind <- function(x,...){
    return(as(x,"ktab"))
}




as.ktab.genpop <- function(x,...){
    return(as(x,"ktab"))
}

######################################
##
## The code below implements import
## from alignement data.
##
######################################



################
# DNAbin2genind
################
DNAbin2genind <- function(x, pop=NULL, exp.char=c("a","t","g","c"), polyThres=1/100){

    ## MISC CHECKS ##
    if(!inherits(x,"DNAbin")) stop("x is not a DNAbin object")
    if(is.list(x)) {
        x <- as.matrix(x)
    }

    if(is.null(colnames(x))) {
        colnames(x) <- 1:ncol(x)
    }


    ## FUNCTION TO PROCESS ONE LOCUS ##
    ## INPUTS:
    ## locus is a column of a DNAbin matrix
    ## posi is the index of this column
    ## OUTPUTS:
    ## returns NULL if no polymorphism
    ## returns a disjonctive table with named columns otherwise
    ## column names are given as [position.allele]
    processLocus <- function(locus, posi){
        vec <- as.character(locus)
        vec[!vec %in% exp.char] <- NA
        N <- sum(!is.na(vec)) # N: number of sequences
        if(N==0 || sum(table(vec)/N >= polyThres )<2) return(NULL) # escape if untyped locus or no SNPs
        alleles <- unique(na.omit(vec))
        ## vec[!vec %in% alleles] <- NA # this would replace rare alleles by NAs
        out <- sapply(alleles, function(e) 1*(vec==e))
        colnames(out) <- paste(posi, alleles, sep=".")
        return(out)
    }


    ## PROCESS ALL LOCI ##
    ## get disjonctive matrix ##
    ## system.time(res@tab <- Reduce(cbind, lapply(1:ncol(x), function(i) processLocus(x[,i], i)))) # works, but Reduce is real slow
    temp <- lapply(1:ncol(x), function(i) processLocus(x[,i], i)) # process all loci, return a list
    col.names <- unlist(sapply(temp, colnames))
    temp <-  as.matrix(data.frame(temp[!sapply(temp, is.null)])) # remove NULL slots, list -> matrix
    if(is.null(temp) || ncol(temp)==0){
        cat("\nNo polymorphism detected - returning NULL.\n")
        return(NULL)
    }

    ## sort out col/row names ##
    ## check if col.names is in matrix format and change to vector format
    if(class(col.names)=="matrix") col.names <- as.vector(col.names)
    colnames(temp) <- col.names # restore correct names
    rownames(temp) <- rownames(x)

    ## create genind output ##
    res <- genind(temp, ploidy=1, pop=pop, type="codom")
    rm(temp) # remove temp
    gc() # collect garbage
    res$call <- match.call()

    return(res)
} # end DNAbin2genind







####################
## alignment2genind
####################
alignment2genind <- function(x, pop=NULL, exp.char=c("a","t","g","c"), na.char="-", polyThres=1/100){

    ## misc checks
    ## if(!require(seqinr)) stop("The package seqinr is required.")
    if(!inherits(x,"alignment")) stop("x is not a alignment object")
    N <- length(x$seq)
    if(!is.null(x$nam) && length(x$nam)!=N) stop("Inconsistent names in x (length of x$nam and x$seq do not match). ")


    ## check that na.char does not overide specified exp.char
    if(!is.null(na.char) && na.char %in% exp.char){
        na.char <- na.char[!na.char %in% exp.char]
        if(length(na.char)==0) na.char <- NULL
    }


    ## convert alignment to matrix of characters
    mat <- sapply(x$seq, s2c, USE.NAMES=FALSE)
    if(nrow(mat)!=x$nb){
        mat <- t(mat)
    }

    rownames(mat) <- x$nam

    if(is.null(colnames(x))) {
        colnames(mat) <- 1:ncol(mat)
    }

    ## replace NAs
    if(is.null(na.char)){
        if(is.null(exp.char)) stop("both exp.char and na.char are NULL")
        temp <- paste(exp.char, collapse="", sep="")
        if(any(exp.char=="-")) {
            temp <- paste("-",temp, sep="") # string '-' must begin the regexp
        }
        temp <- paste("[^", temp, "]", sep="") # anything but the expected is NA
        mat <- gsub(temp,NA,mat)
    } else {
        temp <- paste(na.char, collapse="", sep="")
        if(any(na.char=="-")) {
            temp <- paste("-",temp, sep="") # string '-' must start the regexp
        }
        temp <- paste("[", temp, "]", sep="")
        mat <- gsub(temp,NA,mat)
    }

    ## keep only columns with polymorphism (i.e., SNPs)
    isPoly <- function(vec){
        N <- sum(!is.na(vec)) # N: number of sequences
        temp <- table(vec)/N
        if(sum(temp > polyThres) >= 2) return(TRUE)
        return(FALSE)
    }

    toKeep <- apply(mat, 2, isPoly)
    if(sum(toKeep)==0) stop("No polymorphic site detected")

    mat <- mat[,toKeep, drop=FALSE]

    ## build output
    res <- df2genind(mat, pop=pop, ploidy=1, ncode=1, type="codom")
    res$call <- match.call()

    if(!is.null(x$com)){
        res@other$com <- x$com
    }

    return(res)
} # end alignment2genind







## #################
## ## findMutations
## #################

## ## GENERIC
## findMutations <- function(...){
##     UseMethod("findMutations")
## }

## ## METHOD FOR DNABIN
## findMutations.DNAbin <- function(x, from=NULL, to=NULL, ...){
##     ## CHECKS ##
##     if(!require(ape)) stop("the ape package is needed")
##     if(!inherits(x,"DNAbin")) stop("x is not a DNAbin object")
##     x <- as.matrix(x)

##     ## function to pull out mutations from sequence a to b ##
##     NUCL <- c('a','t','g','c')
##     f1 <- function(a,b){
##         seqa <- as.character(x[a,])
##         seqb <- as.character(x[b,])
##         temp <- which(seqa != seqb)
##         ori <- seqa[temp]
##         mut <- seqb[temp]
##         names(ori) <- names(mut) <- temp
##         toRemove <- !ori %in% NUCL | !mut %in% NUCL
##         ori <- ori[!toRemove]
##         mut <- mut[!toRemove]
##         if(all(toRemove)) return(NULL)
##         res <- data.frame(ori,mut)
##         names(res) <- rownames(x)[c(a,b)]
##         res$short <- paste(row.names(res),":",res[,1],"->",res[,2],sep="")
##         return(res)
##     }

##     ## GET LIST OF PAIRS TO COMPARE ##
##     ## handle NULL
##     if(is.null(from)) from <- 1:nrow(x)
##     if(is.null(to)) to <- 1:nrow(x)

##     ## get pairs
##     pairs <- expand.grid(from, to)

##     ## remove unwanted comparisons
##     pairs <- pairs[pairs[,1]!=pairs[,2],,drop=FALSE]

##     ## GET NUMBER OF MUTATIONS ##
##     out <- lapply(1:nrow(pairs), function(i) f1(pairs[i,1], pairs[i,2]))
##     names(out) <- paste(rownames(x)[pairs[,1]], rownames(x)[pairs[,2]],sep="->")

##     return(out)

## } # end findMutations







## ##################
## ## graphMutations
## ##################

## ## GENERIC
## graphMutations <- function(...){
##     UseMethod("graphMutations")
## }

## ## METHOD FOR DNABIN
## graphMutations.DNAbin <- function(x, from=NULL, to=NULL, plot=TRUE, edge.curved=TRUE, ...){
##     if(!require(igraph)) stop("igraph is required")

##     ## GET MUTATIONS ##
##     x <- findMutations(x, from=from, to=to)

##     ## GET GRAPH ##
##     from <- gsub("->.*","",names(x))
##     to <- gsub(".*->","",names(x))
##     vnames <- sort(unique(c(from,to)))
##     dat <- data.frame(from,to,stringsAsFactors=FALSE)
##     out <- graph.data.frame(dat, directed=TRUE, vertices=data.frame(vnames, label=vnames))

##     ## SET ANNOTATIONS FOR THE BRANCHES ##
##     annot <- unlist(lapply(x, function(e) paste(e$short, collapse="\n")))
##     E(out)$label <- annot
##     E(out)$curved <- edge.curved

##     ## PLOT / RETURN ##
##     if(plot) plot(out, ...)

##     return(out)
## } # end graphMutations














## ###############
## ## transiProb
## ###############
## ##
## ## proba/distance based on transition prob from one sequence to another
## ## time is taken into account
## ## output: matrix with term proba(rowIdx to colIdx)
## ##
## transiProb <- function(x, mu, dates, result=c("prob","dist")){
##     ## MISC CHECKS ##
##     if(!inherits(x,"DNAbin")) stop("x is not a DNAbin object")
##     if(!require(ape)) stop("The package ape is required.")
##     result <- match.arg(result)

##     ## COMPUTATIONS ##

##     ## get numbers of differing nucleotides between sequences
##     seq.length <- ncol(as.matrix(x))
##     D <- as.matrix(dist.dna(x, model="raw")) * seq.length
##     ## if(sum(D-round(D)) > 1e-10){ # make sure we've got integers there
##     ##         warning("Number of nucleotides are not all integers")
##     ##     }
##     D <- round(D)

##     ## compute matrix T (time between sequences)
##     if(inherits(dates,"POSIXct")){ # dates in POSIXct format
##         temp <- outer(dates, dates, difftime, unit="days")
##         T <- -matrix(as.numeric(temp),ncol=length(dates))
##     } else { # dates are numeric
##         T <- -outer(dates, dates, "-")
##     }

##     ## spot negative times
##     toSetToNull <- T < 1e-15

##     ## compute proba(no change @ a site) term
##     mu <- mu/365 # express mu per day
##     p1 <- exp(-T*mu) + (1-exp(-T*mu))/4
##     p1[toSetToNull] <- 0
##     res <- dbinom(D, size=seq.length, prob=(1-p1))

##     ## PROCESS/RETURN RESULT
##     if(result=="prob"){ # return probabilities
##         res[toSetToNull] <- 0
##         diag(res) <- 1
##     } else { # return d = -log(proba)
##         res <- -log(res)
##         res[toSetToNull] <- 1e15
##         diag(res) <- 0
##     }

##     return(res)
## } # end transiProb

###########################
#
# Auxiliary functions for
# adegenet objects
#
# T. Jombart
###########################

##############################
# Method truenames for genind
##############################
setGeneric("truenames", function(x) standardGeneric("truenames"))

setMethod("truenames", signature(x="genind"), function(x){
    message("This accessor is now deprecated. Please use 'tab' instead.")
    return(x@tab)
})



##############################
# Method truenames for genpop
##############################
setMethod("truenames",signature(x="genpop"), function(x){
    message("This accessor is now deprecated. Please use 'tab' instead.")
    return(x@tab)
})




###########################
## Generic / methods 'tab'
###########################
#'
#' Access allele counts or frequencies
#'
#' This accessor is used to retrieve a matrix of allele data.
#' By default, a matrix of integers representing allele counts is returned.
#' If \code{freq} is TRUE, then data are standardised as frequencies, so that for any individual and any locus the data sum to 1.
#' The argument \code{NA.method} allows to replace missing data (NAs).
#' This accessor replaces the previous function \code{truenames} as well as the function \code{makefreq}.
#'
#' @export
#'
#' @aliases tab
#'
#' @rdname tab
#'
#' @docType methods
#'
#' @param x a \linkS4class{genind} or \linkS4class{genpop} object.
#' @param freq a logical indicating if data should be transformed into relative frequencies (TRUE); defaults to FALSE.
#' @param NA.method a method to replace NA; asis: leave NAs as is; mean: replace by the mean allele frequencies; zero: replace by zero
#' @param ... further arguments passed to other methods.
#' @return a matrix of integers or numeric
#'
#' @examples
#'
#' data(microbov)
#' head(tab(microbov))
#' head(tab(microbov,freq=TRUE))
#'
#'
setGeneric("tab", function(x, ...) standardGeneric("tab"))

.tabGetter <- function(x, freq = FALSE, NA.method = c("asis","mean","zero"), ...){
    ## handle arguments
    NA.method <- match.arg(NA.method)
    # outdim <- dim(x@tab)
    ## get matrix of data
    if (!freq){
        out <- x@tab
    } else {
        out <- x@tab/x@ploidy
    }

    ## replace NAs if needed
    if (NA.method == "mean"){
        f1 <- function(vec){
            m <- mean(vec, na.rm = TRUE)
            vec[is.na(vec)] <- m
            return(vec)
        }

        out <- apply(out, 2, f1)

    }
    if (NA.method == "zero"){
        out[is.na(out)] <- ifelse(freq, 0, 0L)
    }
    # dim(out) <- outdim
    ## return output
    return(out)
}

#' @rdname tab
#' @aliases tab,genind-methods
#' @aliases tab.genind
setMethod("tab", signature(x = "genind"),
          function (x, freq = FALSE, NA.method = c("asis","mean","zero"), ...){
            .tabGetter(x, freq = freq, NA.method = NA.method, ...)
          })



#' @rdname tab
#' @aliases tab,genpop-methods
#' @aliases tab.genpop

setMethod("tab", signature(x="genpop"), function(x, freq=FALSE, NA.method=c("asis","mean","zero"), ...){
 ## handle arguments
    NA.method <- match.arg(NA.method)
    # outdim <- dim(x@tab)
    ## get matrix of data
    if(!freq) {
        out <- x@tab
    } else {
        out <- x@tab
        f1 <- function(vec) return(vec/sum(vec,na.rm=TRUE))
        ## compute frequencies
        fac <- x@loc.fac
        if (is.null(fac)) fac <- rep(1, nLoc(x))
        out <- apply(x@tab, 1, tapply, fac, f1, simplify = FALSE)
        if (ncol(x@tab) > 1){
          ## reshape into matrix
          col.names <- do.call(c,lapply(out[[1]],names))
          row.names <- names(out)
          out <- matrix(unlist(out), byrow=TRUE, nrow=nrow(x@tab),
                        dimnames=list(row.names, col.names))
          ## reorder columns
          out <- out[, colnames(x@tab), drop = FALSE]
        } else {
          out <- matrix(out, nrow = length(out), ncol = 1,
                        dimnames = list(rownames(x@tab), colnames(x@tab)))
        }

    }

    ## replace NAs if needed
    if(NA.method=="mean"){
        f1 <- function(vec){
            m <- mean(vec, na.rm=TRUE)
            vec[is.na(vec)] <- m
            return(vec)
        }

        out <- apply(out, 2, f1)
    }
    if(NA.method=="zero"){
        out[is.na(out)] <- ifelse(freq, 0, 0L)
    }
    # dim(out) <- outdim
    ## return output
    return(out)
})




###########################
# Method seploc for genind
###########################
setGeneric("seploc", function(x, ...) standardGeneric("seploc"))

setMethod("seploc", signature(x="genind"), function(x,truenames=TRUE,res.type=c("genind","matrix")){
    truenames <- TRUE # this argument will be deprecated
    if(x@type=="PA"){
        msg <- paste("seploc is not implemented for presence/absence markers")
        cat("\n",msg,"\n")
        return(invisible())
    }

    if(!is.genind(x)) stop("x is not a valid genind object")
    res.type <- match.arg(res.type)

    ## make separate tables
    kX <- list()
    locfac.char <- as.character(x@loc.fac)
    for(i in locNames(x)){
        kX[[i]] <- x@tab[, i==locfac.char,drop=FALSE]
    }

    prevcall <- match.call()
    if(res.type=="genind"){
        ## ploidy bug fixed by Zhian N. Kamvar
        ##kX <- lapply(kX, genind, pop=x@pop, prevcall=prevcall)
        kX <- lapply(kX, genind, pop=x@pop, prevcall=prevcall, ploidy=x@ploidy, type=x@type)
        for(i in 1:length(kX)){
            kX[[i]]@other <- x@other
        }
    }

    return(kX)
})



###########################
# Method seploc for genpop
###########################
setMethod("seploc", signature(x="genpop"), function(x,truenames=TRUE,res.type=c("genpop","matrix")){
    truenames <- TRUE # this argument will be deprecated
    if(x@type=="PA"){
         msg <- paste("seploc is not implemented for presence/absence markers")
         cat("\n",msg,"\n")
         return(invisible())
    }


    if(!is.genpop(x)) stop("x is not a valid genpop object")
    res.type <- match.arg(res.type)
    if(res.type=="genpop") { truenames <- TRUE }

    temp <- x@loc.fac
    nloc <- length(levels(temp))
    levels(temp) <- 1:nloc

    ## make separate tables
    kX <- list()
    locfac.char <- as.character(x@loc.fac)
    for(i in locNames(x)){
        kX[[i]] <- x@tab[,i==locfac.char,drop=FALSE]
    }

    names(kX) <- locNames(x)

    prevcall <- match.call()
    if(res.type=="genpop"){
        kX <- lapply(kX, genpop, prevcall=prevcall, ploidy=x@ploidy, type=x@type)
        for(i in 1:length(kX)){
            kX[[i]]@other <- x@other
        }
    }

    return(kX)
})





###############
# '$' operator
###############
setMethod("$","genind",function(x,name) {
    return(slot(x,name))
})


setMethod("$<-","genind",function(x,name,value) {
   slot(x,name,check=TRUE) <- value
  return(x)
})







##################
# Function seppop
##################
setGeneric("seppop", function(x, ...) standardGeneric("seppop"))

## genind
setMethod("seppop", signature(x="genind"), function(x,pop=NULL,truenames=TRUE,res.type=c("genind","matrix"),
                              drop=FALSE, treatOther=TRUE, quiet=TRUE){
    ## checkType(x)
    truenames <- TRUE # this argument will be deprecated

    ## misc checks
    if(!is.genind(x)) stop("x is not a valid genind object")
    if(is.null(pop)) { # pop taken from @pop
        if(is.null(x@pop)) stop("pop not provided and x@pop is empty")
        pop <- pop(x)
    } else if (is.language(pop)){
      setPop(x) <- pop
      pop <- pop(x)
    } else {
        pop <- factor(pop)
    }


    res.type <- match.arg(res.type)

    ## pop <- x@pop # comment to take pop arg into account

    ## make a list of genind objects
    kObj <- lapply(levels(pop), function(lev) x[pop==lev, , drop=drop, treatOther=treatOther, quiet=quiet])
    names(kObj) <- levels(pop)

    ## res is a list of genind
    if(res.type=="genind"){ return(kObj) }

    ## res is list of matrices
    res <- lapply(kObj, function(obj) tab(obj))

    return(res)
}) # end seppop







## #####################
## # Methods na.replace
## #####################
## setGeneric("na.replace", function(x, ...) standardGeneric("na.replace"))

## ## genind method
## setMethod("na.replace", signature(x="genind"), function(x, method, quiet=FALSE){
##     ## checkType(x)

##     ## preliminary stuff
##     validObject(x)
##     if(!any(is.na(x@tab))) {
##         if(!quiet) cat("\n Replaced 0 missing values \n")
##         return(x)
##     }
##     method <- tolower(method)
##     method <- match.arg(method, c("0","mean"))

##     res <- x

##     if(method=="0"){
##         res@tab[is.na(x@tab)] <- 0
##     }

##     if(method=="mean"){
##         f1 <- function(vec){
##             m <- mean(vec,na.rm=TRUE)
##             vec[is.na(vec)] <- m
##             return(vec)
##         }

##         res@tab <- apply(x@tab, 2, f1)
##     }

##     if(!quiet){
##         Nna <- sum(is.na(x@tab))
##         cat("\n Replaced",Nna,"missing values \n")
##     }

##     return(res)

## })




## ## genpop method
## setMethod("na.replace", signature(x="genpop"), function(x,method, quiet=FALSE){
##     ## checkType(x)

##     ## preliminary stuff
##     validObject(x)
##     if(!any(is.na(x@tab))) {
##         if(!quiet) cat("\n Replaced 0 missing values \n")
##         return(x)
##     }

##     method <- tolower(method)
##     method <- match.arg(method, c("0","chi2"))

##     res <- x

##     if(method=="0"){
##         res@tab[is.na(x@tab)] <- 0
##     }

##     if(method=="chi2"){
##         ## compute theoretical counts
##         ## (same as in a Chi-squared)
##         X <- x@tab
##         sumPop <- apply(X,1,sum,na.rm=TRUE)
##         sumLoc <- apply(X,2,sum,na.rm=TRUE)
##         X.theo <- sumPop %o% sumLoc / sum(X,na.rm=TRUE)

##         X[is.na(X)] <- X.theo[is.na(X)]
##         res@tab <- X
##     }

##     if(!quiet){
##         Nna <- sum(is.na(x@tab))
##         cat("\n Replaced",Nna,"missing values \n")
##     }

##     return(res)
## })

# Function to bind strata from a list of genind objects and return a single
# genind object.
.rbind_strata <- function(myList, res){
    strata_list <- lapply(myList, slot, "strata")
    null_strata <- vapply(strata_list, is.null, TRUE)
    if (!all(null_strata)){
        # NULL strata must be converted to data frames.
        # Solution: take the first non-empty strata, and create a new one
        # with one variable.
        if (any(null_strata)){

            # Extract the name of the first column of the first full strata
            fullname <- names(strata_list[[which(!null_strata)[1]]])[1]

            # loop over all the empty strata and replace them with a data
            # frame that has the same number of elements as the samples in that
            # genlight object.
            for (i in which(null_strata)){
                replace_strata        <- data.frame(rep(NA, nInd(myList[[i]])))
                names(replace_strata) <- fullname
                strata_list[[i]]      <- replace_strata
            }
        }
        strata(res) <- as.data.frame(suppressWarnings(dplyr::bind_rows(strata_list)))
    } else {
        res@strata <- NULL
    }
    return(res)
}




#'
#' Pool several genotypes into a single dataset
#'
#'  The function \code{repool} allows to merge genotypes from different
#'  \linkS4class{genind} objects into a single 'pool' (i.e. a new \linkS4class{genind}).
#'  The markers have to be the same for all objects to be merged, but
#'  there is no constraint on alleles.\cr
#'
#'  This function can be useful, for instance, when hybrids are created
#'  using \code{\link{hybridize}}, to merge hybrids with their parent
#'  population for further analyses. Note that \code{repool} can also
#'  reverse the action of \code{\link{seppop}}.
#'
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#'
#' @seealso \code{\link{seploc}}, \code{\link{seppop}}
#'
#' @examples
#' \dontrun{
#' ## use the cattle breeds dataset
#' data(microbov)
#' temp <- seppop(microbov)
#' names(temp)

#' ## hybridize salers and zebu -- nasty cattle
#' zebler <- hybridize(temp$Salers, temp$Zebu, n=40)
#' zebler

#' ## now merge zebler with other cattle breeds
#' nastyCattle <- repool(microbov, zebler)
#' nastyCattle
#' }
#'
#' @export
#'
#' @param ... a list of \linkS4class{genind} objects, or a series of \linkS4class{genind} objects separated by commas
#' @param list a logical indicating whether a list of objects with matched alleles shall be returned (TRUE), or a single \linkS4class{genind} object (FALSE, default).
#'
#'
#'
repool <- function(..., list=FALSE){

    ## PRELIMINARY STUFF
    x <- list(...)
    old.names <- names(x)
    if(is.list(x[[1]])) x <- x[[1]] ## if ... is a list, keep this list for x
    if(!inherits(x,"list")) stop("x must be a list")
    if(!all(sapply(x,is.genind))) stop("x is does not contain only valid genind objects")
    temp <- sapply(x,function(e) locNames(e))
    if(!all(table(temp)==length(x))) stop("markers are not the same for all objects")
    ## temp <- sapply(x,function(e) e$ploidy)
    ## if(length(unique(temp)) != as.integer(1)) stop("objects have different levels of ploidy")


    ## MAKE A LIST OF OBJECTS
    listTab <- lapply(x,genind2df,usepop=FALSE,sep="/")
    newPloidy <- unlist(lapply(x,ploidy))


    ## SET POPS IF MISSING
    ## STORE OLD POP
    old.pop <- lapply(x, pop)

    for(i in 1:length(x)){
        if(is.null(pop(x[[i]]))){
            pop(x[[i]]) <- rep(paste("unknown",i,sep="_"), nInd(x[[i]]))
        }
    }

    new.pop <- lapply(x, pop)


    ## MERGE RAW DATASETS
    ## reorder columns like in first dataset
    markNames <- colnames(listTab[[1]])
    listTab <- lapply(listTab, function(tab) tab[,markNames,drop=FALSE]) # resorting of the tabs

    ## bind all tabs by rows
    tab <- listTab[[1]]
    for(i in 2:length(x)){
        tab <- rbind(tab,listTab[[i]])
    }

    ## GET SINGLE GENIND
    res <- df2genind(tab, ploidy=newPloidy, type=x[[1]]@type, sep="/")
    pop(res) <- unlist(new.pop)
    res <- .rbind_strata(x, res)
    res@hierarchy <- NULL
    res$call <- match.call()

    ## IF A LIST OF GENIND IS TO BE RETURNED
    if(list){
        ## SEPARATE DATASETS
        old.n <- sapply(x, nInd)
        new.pop <- rep(1:length(x), old.n)
        pop(res) <- new.pop
        res <- seppop(res)

        ## RESTORE OLD OTHER AND POP
        old.other <- lapply(x, other)
        for(i in 1:length(res)){
            other(res[[i]]) <- old.other[[i]]
            pop(res[[i]]) <- old.pop[[i]]
        }

        ## SET OBJECT NAMES
        names(res) <- old.names
    }

    ## RETURN
    return(res)
} # end repool





#############
# selpopsize
#############
setGeneric("selPopSize", function(x, ...) standardGeneric("selPopSize"))

## genind method ##
setMethod("selPopSize", signature(x="genind"), function(x,pop=NULL,nMin=10){

    ## misc checks
    ## checkType(x)
    if(!is.genind(x)) stop("x is not a valid genind object")
    if(is.null(pop)) { # pop taken from @pop
        if(is.null(x@pop)) stop("pop not provided and x@pop is empty")
        pop <- pop(x)
    } else{
        pop <- factor(pop)
    }

    ## select retained individuals
    effPop <- table(pop)
    popOk <- names(effPop)[effPop >= nMin]
    toKeep <- pop %in% popOk

    ## build result
    res <- x[toKeep]
    pop(res) <- pop[toKeep]

    return(res)
}) # end selPopSize





#########
# isPoly
#########
setGeneric("isPoly", function(x, ...) standardGeneric("isPoly"))

## genind method ##
setMethod("isPoly", signature(x="genind"), function(x, by=c("locus","allele"), thres=1/100){

    ## misc checks
    ## checkType(x)
    if(!is.genind(x)) stop("x is not a valid genind object")
    by <- match.arg(by)


    ## main computations ##

    ## PA case ##
    if(x@type=="PA") {
        allNb <- apply(x@tab, 2, mean, na.rm=TRUE) # allele frequencies
        toKeep <- (allNb >= thres) & (allNb <= (1-thres))
        return(toKeep)
    }


    ## codom case ##
    allNb <- apply(x@tab, 2, sum, na.rm=TRUE) # allele absolute frequencies

    if(by=="locus"){
        f1 <- function(vec){
            if(sum(vec) < 1e-10) return(FALSE)
            vec <- vec/sum(vec, na.rm=TRUE)
            if(sum(vec >= thres) >= 2) return(TRUE)
            return(FALSE)
        }

        toKeep <- tapply(allNb, x@loc.fac, f1)
    } else { # i.e. if mode==allele
        toKeep <- (allNb >= thres)
    }

    return(toKeep)
}) # end isPoly





## ## genpop method ##
## setMethod("isPoly", signature(x="genpop"), function(x, by=c("locus","allele"), thres=1/100){

##     ## misc checks
##     checkType(x)
##     if(!is.genpop(x)) stop("x is not a valid genind object")
##     by <- match.arg(by)


##     ## main computations ##
##     ## ## PA case ##
## ##     if(x@type=="PA") {
## ##         allNb <- apply(x@tab, 2, mean, na.rm=TRUE) # allele frequencies
## ##         toKeep <- (allNb >= thres) & (allNb <= (1-thres))
## ##         return(toKeep)
## ##     }


##     ## codom case ##
##     allNb <- apply(x@tab, 2, sum, na.rm=TRUE) # alleles absolute frequencies

##     if(by=="locus"){
##         f1 <- function(vec){
##             if(sum(vec) < 1e-10) return(FALSE)
##             vec <- vec/sum(vec, na.rm=TRUE)
##             if(sum(vec >= thres) >= 2) return(TRUE)
##             return(FALSE)
##         }

##         toKeep <- tapply(allNb, x@loc.fac, f1)
##     } else { # i.e. if mode==allele
##         toKeep <- allNb >= thres
##     }

##     return(toKeep)
## }) # end isPoly

context("Test snapclust")


test_that("snapclust gives decent results for F1 Zebu-Salers", {
    skip_on_cran()


    set.seed(1)

    ## Simulate hybrids F1 Zebu/Salers
    data(microbov)
    zebu <- microbov[pop="Zebu"]
    salers <- microbov[pop="Salers"]
    hyb <- hybridize(zebu, salers, n=30)
    x <- repool(zebu, salers, hyb)

    ## run analysis
    res.hyb <- snapclust(x, k=2, hybrids=TRUE)

    ## check results
    expect_true(res.hyb$converged)
    expect_equal(unname(apply(table(pop(x), res.hyb$group),1,max)),
                 c(50,50,30)) # indiv from original pop all in one group

})




test_that("snapclust gives decent results for F1 & back-cross Zebu-Salers", {
    skip_on_cran()

    set.seed(1)

    ## Simulate hybrids F1 Zebu/Salers
    data(microbov)
    zebu <- microbov[pop="Zebu"]
    salers <- microbov[pop="Salers"]
    hyb <- hybridize(zebu, salers, n=30)
    x <- repool(zebu, salers, hyb)


    ## Simulate hybrids backcross (F1 / parental)
    f1.zebu <- hybridize(hyb, zebu, 20, pop = "f1.zebu")
    f1.salers <- hybridize(hyb, salers, 25, pop = "f1.salers")
    y <- repool(x, f1.zebu, f1.salers)


    ## method with back-cross
    res.back <- snapclust(y, k=2, hybrids = TRUE, hybrid.coef = c(.25,.5))
    tab <- table(pop(y), res.back$group)

    ## check results
    expect_true(res.back$converged)
    expect_true(tab[1,1] > 47)
    expect_true(tab[2,2] > 47)
    expect_true(tab[3,4] > 25)
    expect_true(tab[4,3] > 10)
    expect_true(tab[5,5] > 10)

})


test_that("snapclust.choose.k will recognize objects that inherit genind objects", {
  skip_on_cran()
  skip_if_not_installed("poppr")
  requireNamespace("poppr", quietly = TRUE)
  data(microbov)
  zebu <- poppr::as.genclone(microbov[pop = "Zebu"])
  expect_is(zebu, "genclone")
  res <- snapclust.choose.k(2, zebu)
  expect_length(res, 2)
  expect_is(res, "numeric")
  unloadNamespace("poppr")
})

test_that("snapclust.choose.k will ignore any extra genind objects supplied", {
  skip_on_cran()
  data(microbov)
  expect_warning({
    res <- snapclust.choose.k(2, microbov[pop = "Zebu"], microbov[pop = "Salers"])  
  }, "Too many genind objects provided")
  expect_is(res, "numeric")
  expect_length(res, 2)
})

test_that("snapclust.choose.k will throw an error if there are no genind objects", {
  skip_on_cran()
  expect_error(snapclust.choose.k(2, 1), "No genind provided")
})

##################
## adegenetServer
##################
## function starting a web-server
## for adegenet tools
adegenetServer <- function(what=c("DAPC")){
    what <- match.arg(what)
    if(what=="DAPC"){
        .dapcServer()
    }

    return(invisible())
}


###############
## .dapcServer
###############
## hidden function - DAPC server
.dapcServer <- function(){
    runApp(system.file("dapcServer",package="adegenet"))
}



#######################
## .render.server.info
#######################
## INVISIBLE FUNCTION RENDERING SERVER INFO ##
.render.server.info <- function(){
    renderPrint(
            {
                cat("\n== R version ==\n")
                print(R.version)

                cat("\n== Date ==\n")
                print(date())

                cat("\n== adegenet version ==\n")
                print(packageDescription("adegenet", fields=c("Package", "Version", "Date", "Built")))

                cat("\n== shiny version ==\n")
                print(packageDescription("adegenet", fields=c("Package", "Version", "Date", "Built")))

                cat("\n== attached packages ==\n")
                print(search())
            }
                ) # end renderPrint
} # end .render.server.info

############
## generics
############
seqTrack <- function(...){
    UseMethod("seqTrack")
}

seqTrack.default <- function(x, ...){
    cat("\nseqTrack not implemented for object of the class", class(x),"\n")
    return(invisible(NULL))
}

## seqTrackG <- function(...){
##     UseMethod("seqTrackG")
## }


## optimize.seqTrack <- function(...){
##     UseMethod("optimize.seqTrack")
## }


get.likelihood <- function(...){
    UseMethod("get.likelihood")
}





########################
## seqTrack - basic version
########################
##
## - x is a matrix giving weights x[i,j] such that:
## 'i is ancestor j'
## - prox.mat is a directed proximity measure, so that prox.mat[i,j] is
## the 'proximity when going from i to j'
##
seqTrack.matrix <- function(x, x.names, x.dates, best=c("min","max"),
                     prox.mat=NULL, mu=NULL, haplo.length=NULL, ...){

    ## CHECKS ##
    best <- match.arg(best)
    if(best=="min"){
        best <- min
        which.best <- which.min
    } else {
        best <- max
        which.best <- which.max
    }

    if(length(x.names) != length(x.dates)){
        stop("inconsistent length for x.dates")
    }

    if(is.character(x.dates)){
        msg <- paste("x.dates is a character vector; " ,
                     "please convert it as dates using 'as.POSIXct'" ,
                     "\n(making sure dates are given as 'YYYY/MM/DD' or 'YYYY-MM-DD').", sep="")
        stop(msg)
    }

    x <- as.matrix(x)

    if(!is.null(prox.mat) && !identical(dim(prox.mat),dim(x))) {
        stop("prox.mat is provided but its dimensions are inconsistent with that of x")
    }

    N <- length(x.names)
    id <- 1:N

    x.dates <- as.POSIXct(round.POSIXt(x.dates,units="days")) # round dates to the day

    temp <- as.vector(unique(x))
    D.ARE.MUT <- all(temp-round(temp,10)<1e-14)


    ## rename dimensions using id
    colnames(x) <- rownames(x) <- id
    if(!is.null(prox.mat)){
        colnames(prox.mat) <- rownames(prox.mat) <- id
    }

    if(length(x.names) != nrow(x)){
        stop("inconsistent dimension for x")
    }


    ## AUXILIARY FUNCTIONS ##
    ## test equality in floats
    test.equal <- function(val,vec){
        return(abs(val-vec) < 1e-12)
    }


    ## return the names of optimal value(s) in a named vector
    which.is.best <- function(vec){
        res <- names(vec)[test.equal(best(vec), vec)]
        return(res)
    }


    ## select among different possible ancestors
    selAmongAncestors <- function(idx,ances){
        ## Choose the most connected ancestor, given prox.mat
        if(!is.null(prox.mat)){ # if we've got no other info
            toKeep <- test.equal(max(prox.mat[ances,idx]), prox.mat[ances,idx])
            ances <- ances[toKeep]
        }

        ## If several ancestors remain, take the one closest to the average generation time.
        if(length(ances)>1){
            if(!D.ARE.MUT | is.null(mu) | is.null(haplo.length)) { # if we don't have mutation rates / haplo length, or if dist. are not nb of mutations
                ances <- ances[which.min(x.dates[ances])] # take the oldest ancestor
            } else { # if distances are mutations and we've got mu and L
                timeDiff <- as.numeric(difftime(x.dates[idx], x.dates[ances], units="day")) # days between candidates and target
                ##nbGen <- round(timeDiff / gen.time) # number of generations
                nbMut <- x[ances, idx]
                prob <- dbinom(nbMut, timeDiff*haplo.length, mu)
                ances <- ances[which.max(prob)] # take the most likely ancestor
            }
        }

        return(ances)
    }


    ## findAncestor
    findAncestor <- function(idx){ # returns the index of one seq's ancestor
        candid <- which(x.dates < x.dates[idx])
        if(length(candid)==0) return(list(ances=NA, weight=NA))
        if(length(candid)==1) return(list(ances=candid, weight=x[candid, idx]))
        ancesId <- as.numeric(which.is.best(x[candid, idx]))
        if(length(ancesId)>1) {
            ancesId <- selAmongAncestors(idx,ancesId) # handle several 'best' ancestors
        }
        return(list(ances=ancesId, weight=x[ancesId, idx])) # Id of the ancestor
    }


    ## BUILD THE OUTPUT ##
    res <- sapply(id, findAncestor)
    res <- data.frame(ances=unlist(res[1,]), weight=unlist(res[2,]))
    ances.date <- x.dates[res[,1]]
    res <- cbind.data.frame(id,res, date=x.dates, ances.date)
    rownames(res) <- x.names

    class(res) <- c("seqTrack","data.frame")

    return(res)
} # end seqTrack.matrix





################
## plotSeqTrack
################
plotSeqTrack <- function(x, xy, use.arrows=TRUE, annot=TRUE, labels=NULL,
                         col=NULL, bg="grey", add=FALSE, quiet=FALSE, date.range=NULL,
                         jitter.arrows=0, plot=TRUE,...){

    ## CHECKS ##
    if(!inherits(x,"seqTrack")) stop("x is not a seqTrack object")
    if(ncol(xy) != 2) stop("xy does not have two columns")
    if(nrow(xy) != nrow(x)) stop("x and xy have inconsistent dimensions")


    ## RECYCLE COL
    if(!is.null(col)){
        col <- rep(col,length=nrow(x))
    } else {
        col <- rep("black", nrow(x))
    }


    ## DEFAULT LABELS
    if(is.null(labels)){
        if(!is.null(rownames(x))){
            labels <- rownames(x)
        } else {
            labels <- 1:nrow(x)
        }
    }


    ## SUBSET DATA (REMOVE NAs) ##
    isNA <- is.na(x[,2])
    x <- x[!isNA,,drop=FALSE]
    xy.all <- xy # used to retrieve all coordinates
    xy <- xy[!isNA,,drop=FALSE]
    if(!is.null(labels)){ # subset labels
        labels <- labels[!isNA]
    }
    if(!is.null(col)){ # subset colors
        col <- col[!isNA]
    }


    ## FIND SEGMENTS COORDS ##
    from <- unlist(x[,2])
    to <- unlist(x[,1])

    x.from <- xy.all[from,1]
    y.from <- xy.all[from,2]
    x.to <- xy.all[to,1]
    y.to <- xy.all[to,2]


    ## HANDLE RANGE OF DATES ##
    if(!is.null(date.range)){

        if(is.character(date.range)){
            msg <- paste("date.range is a character vector; " ,
                         "please convert it as dates using 'as.POSIXct'" ,
                         "\n(making sure dates are given as 'YYYY/MM/DD' or 'YYYY-MM-DD').", sep="")
            stop(msg)
        }

        if(any(is.na(date.range))){
            stop("NA in date.range")
        }

        dates <- x$date
        toKeep <- (dates > min(date.range)) & (dates < max(date.range))
        if(sum(toKeep)==0) {
            if(!quiet) cat("\nNo item in the specified date range.\n")
            return(NULL)
        }


        ## SUBSETTING
        x.from <- x.from[toKeep]
        y.from <- y.from[toKeep]
        x.to <- x.to[toKeep]
        y.to <- y.to[toKeep]
        col <- col[toKeep]
        xy <- xy[toKeep,,drop=FALSE]
        x <- x[toKeep,,drop=FALSE]
        labels <- labels[toKeep]

    }


    ## DO THE PLOTTING ##
    if(plot){
        obg <- par("bg")
        on.exit(par(bg=obg))
        if(!add){
            par(bg=bg)
            plot(xy, type="n", ...)
        }
    }


    ## PLOTTING ##
    if(plot){
        ## ARROWS
        if(use.arrows){
            ## handle segments/arrows with length 0 ##
            nullLength <- (abs(x.from-x.to)<1e-10) & (abs(y.from-y.to)<1e-10)

            ## handle random noise around coordinates
            if(jitter.arrows>0){
                x.from[!nullLength] <- jitter(x.from[!nullLength], jitter.arrows)
                x.to[!nullLength] <- jitter(x.to[!nullLength], jitter.arrows)
                y.from[!nullLength] <- jitter(y.from[!nullLength], jitter.arrows)
                y.to[!nullLength] <- jitter(y.to[!nullLength], jitter.arrows)
            }

            arrows(x.from[!nullLength], y.from[!nullLength],
                   x.to[!nullLength], y.to[!nullLength],
                   col=col[!nullLength], angle=15, ...)
        } else{
        ## SEGMENTS
            ## handle random noise around coordinates
            if(jitter.arrows>0){
                x.from[!nullLength] <- jitter(x.from[!nullLength], jitter.arrows)
                x.to[!nullLength] <- jitter(x.to[!nullLength], jitter.arrows)
                y.from[!nullLength] <- jitter(y.from[!nullLength], jitter.arrows)
                y.to[!nullLength] <- jitter(y.to[!nullLength], jitter.arrows)
            }

            segments(x.from, y.from, x.to, y.to, col=col,...)
        }

        ## ANNOTATIONS
        if(annot) {
            text(xy,lab=labels, font=2)
        }

        ## SUNFLOWERS / POINTS
        if(any(nullLength)) {
            sunflowerplot(x.from[nullLength], y.from[nullLength], seg.lwd=2, size=1/6,
                          col=col[nullLength], seg.col=col[nullLength], add=TRUE, ...)
            points(x.from[nullLength], y.from[nullLength], col=col[nullLength], cex=2, pch=20, ...)
        }

    }


    ## RESULT ##
    res <- data.frame(x.from, y.from, x.to, y.to, col=col)

    return(invisible(res))
} # end plotSeqTrack






###########################
## get.likelihood.seqTrack
###########################
get.likelihood.seqTrack <- function(x, mu, haplo.length,...){
    if(any(na.omit(x$weight - round(x$weight)) > 1e-10)){
        warning("Non-integer weights: number of mutations expected in x$weight.")
    }

    dates <- as.POSIXct(x$date)
    anc.dates <- as.POSIXct(x$ances.date)
    nb.days <- abs(as.integer(anc.dates-dates))
    nb.mut <- x$weight

    ##    res <- dbinom(nb.mut, size=haplo.length*nb.days, prob=mu)
    res <- dpois(x=nb.mut, lambda=mu*haplo.length*nb.days)

    return(res)
} # end get.likelihood.seqTrack






######################
## as.igraph.seqTrack
######################
as.igraph.seqTrack <- function(x, col.pal=redpal, ...){

    ## GET DAG ##
    from.old <- x$ances
    to.old <- x$id
    isNotNA <- !is.na(from.old) & !is.na(to.old)
    vnames <- sort(unique(c(from.old,to.old)))
    from <- match(from.old,vnames)
    to <- match(to.old,vnames)
    dat <- data.frame(from,to,stringsAsFactors=FALSE)[isNotNA,,drop=FALSE]

    out <- graph.data.frame(dat, directed=TRUE, vertices=data.frame(names=vnames))

    ## SET VARIOUS INFO ##
    ## WEIGHTS FOR EDGES
    E(out)$weight <- x$weight[isNotNA]

    ## DATES FOR VERTICES (IN NB OF DAYS FROM EARLIEST DATE)
    V(out)$dates <- difftime(x$date, min(x$date), units="days")

    ## SET EDGE LABELS ##
    E(out)$label <- E(out)$weight

    ## SET EDGE COLORS
    E(out)$color <- num2col(E(out)$weight, col.pal=col.pal, reverse=TRUE)

    ## SET LAYOUT ##
    ypos <- V(out)$dates
    ypos <- abs(ypos-max(ypos))
    attr(out, "layout") <- layout.fruchterman.reingold(out, params=list(miny=ypos, maxy=ypos))

    ## RETURN OBJECT ##
    return(out)

} # end as.igraph.seqTrack






#################
## plot.seqTrack
#################
plot.seqTrack <- function(x, y=NULL, col.pal=redpal, ...){

    ## get graph ##
    g <- as.igraph(x, col.pal=col.pal)

    ## make plot ##
    plot(g, layout=attr(g,"layout"), ...)

    ## return graph invisibly ##
    return(invisible(g))

} # end plot.seqTrack




























################################################
################################################
######### OLD STUFF - NOT USED FOR NOW ######
################################################
################################################

##########################
## as("seqTrack", "graphNEL")
##########################
## if(require(graph)){
## setOldClass("seqTrack")
## setAs("seqTrack", "graphNEL", def=function(from){
##     ##    if(!require(ape)) stop("package ape is required")
##     if(!require(graph)) stop("package graph is required")

##     ori.labels <- rownames(from)
##     from <- from[!is.na(from$ances),,drop=FALSE]


##      ## CONVERT TO GRAPH
##     res <- ftM2graphNEL(ft=cbind(ori.labels[from$ances], ori.labels[from$id]), W=from$weight, edgemode = "directed", V=ori.labels)
##     return(res)
## })
## }




## #############
## ## .dTimeSeq
## #############
## ##
## ## mu0 and L are vectors, having one value per segment/chromosome
## ## mu0 is per nucleotide and per day
## .dTimeSeq <- function(mu, L, maxNbDays=100){
##     ##mu <- mu/365 # mutation rates / site / day
##     t <- 0:maxNbDays # in days added / substracted
##     temp <- sapply((1-mu)^L, function(x) x^t  )
##     Pt <- apply(temp,1,prod)
##     t <- c(-rev(t[-1]), t)
##     Pt <- c(rev(Pt[-1]), Pt)
##     return(list(t, Pt))
## }


## #############
## ## .rTimeSeq
## #############
## ##
## ## mu and L are vectors, having one value per segment/chromosome
## ##
## ## this returns nb days
## .rTimeSeq <- function(n, mu, L, maxNbDays=100){
##     temp <- .dTimeSeq(mu, L, maxNbDays)
##     res <- sample(temp[[1]], size=n, replace=TRUE, prob= temp[[2]]/sum(temp[[2]]))
##     return(res)
## }



## #################
## ## .rUnifDate
## #################
## ##
## ## this returns random uniform dates in a given range
## ##
## .rUnifDate <- function(n, dateMin, dateMax, ...){
##     rangeSize <-  as.integer(difftime(dateMax,dateMin, units="days"))
##     nbDays <- round(runif(n, min=0, max=rangeSize))
##     res <- dateMin + nbDays*3600*24
##     res <- as.POSIXct(round.POSIXt(res, units="days"))
##     return(res)
## }



## #################
## ## .rNormTimeSeq
## #################
## ##
## ## this returns nb of days
## .rNormTimeSeq <- function(n, mean, sd, ...){
##     res <- round(rnorm(n, mean=mean, sd=sd))
##     return(res)
## }



## #################
## ## .rSampTimeSeq
## #################
## ##
## ## this returns nb of days
## .rSampTime <- function(n,...){
##     res <- round(rnorm(n*2, -2))
##     res <- res[res < 0 & res > -7][1:n]
##     return(res)
## }


## ##################
## ## .pAbeforeBfast
## ##################
## ##
## ## faster version, same mu and length for both sequences
## ## already vectorised for dateA and dateB
## .pAbeforeBfast <- function(dateA, dateB, mu, L, maxNbDays=100){
##     ## proba dist for both haplo
##     temp <- .dTimeSeq(mu, L, maxNbDays)
##     days <- temp[[1]]
##     p <- temp[[2]]/sum(temp[[2]]) # scale to proba mass function

##     ## days for A and B
##     nbDays <- as.integer(round(difftime(dateB,dateA,units="days"))) # dateA - dateB, in days

##     ## function for one comparison
##     f1 <- function(Dt,max){ # does not work for Dt < 0 (have to reverse proba after)
##         if(is.na(Dt)) return(NA)
##         if(Dt>max) return(1)
##         if(round(Dt)==0){
##             temp <- sapply(1:(max-1), function(i) p[i]*sum(p[(i+1):max]))
##             return(sum(temp))
##         }
##         term1 <- sum(p[1:Dt])
##         idx <- seq(2,by=1,length=(max-Dt))
##         temp <- sapply(idx, function(i) sum(p[i:max]))
##         term2 <- sum( p[(Dt+1):max] * temp)

##         return(term1+term2)
##     }

##     ## computations
##     distribSize <- length(days)
##     res <- sapply(nbDays, f1, max=distribSize)
##     res[nbDays<0] <- 1-res[nbDays<0] # reverse proba for negative time diff

##     return(res)
## } # end .pAbeforeBfast




## ##############
## ## .pAbeforeB
## ##############
## ##
## ## allows for different distributions for both haplo
## .pAbeforeB <- function(dateA, dateB, muA, muB, LA, LB, maxNbDays=100){
##     ## proba dist for A
##     tempA <- .dTimeSeq(muA, LA, maxNbDays)
##     days <- tempA[[1]]
##     pA <- tempA[[2]]/sum(tempA[[2]]) # scale to proba mass function

##     ## proba dist for B
##     tempB <- .dTimeSeq(muB, LB, maxNbDays)
##     pB <- tempB[[2]]/sum(tempB[[2]]) # scale to proba mass function

##     ## days for A and B
##     nbDaysDiff <- as.integer(round(difftime(dateA,dateB,units="days"))) # dateA - dateB, in days
##     daysA <- days
##     daysB <- days - nbDaysDiff

##     f1 <- function(i){ # proba A before B for one day
##         idx <- daysB > daysA[i]
##         return(pA[i] * sum(pB[idx]))
##     }

##     res <- sapply(1:length(days), f1) # proba for all days
##     res <- sum(res) # sum
##     return(res)
## }

## .pAbeforeB <- Vectorize(.pAbeforeB,
##                         vectorize.args=c("dateA","dateB", "muA", "muB", "LA", "LB")) ## end .pAbeforeB




## ##############
## ## seqTrackG - graph version of SeqTrack
## ##############
## ##
## ## - x is a matrix giving weights x[i,j] such that:
## ## 'i is ancestor j'; the algo looks for maximal weight branching
## ##
## ## - prox.mat is a directed proximity measure, so that prox.mat[i,j] is
## ## the 'proximity when going from i to j'
## ##
## seqTrackG.default <- function(x, x.names, x.dates, best=c("min","max"), force.temporal.order=TRUE,
##                               res.type=c("seqTrack", "graphNEL"), ...){

##     ## CHECKS ##
##     if(!require("graph")) stop("the graph package is not installed")
##     if(!require("RBGL")) stop("the RBGL package is not installed")
##     if(!exists("edmondsOptimumBranching")) {
##         stop("edmondsOptimumBranching does not exist; \nmake sure to use the latest Bioconductor (not CRAN) version of RBGL")
##         cat("\nWould you like to try and install latest version of RBGL (needs internet connection)\n y/n: ")
##         ans <- tolower(as.character(readLines(con = getOption('adegenet.testcon'), n = 1)))
##         if(ans=="y"){
##             source("http://bioconductor.org/biocLite.R")
##             biocLite("RBGL")
##         }
##     }

##     best <- match.arg(best)
##     res.type  <- match.arg(res.type)

##     if(length(x.names) != length(x.dates)){
##         stop("inconsistent length for x.dates")
##     }

##     if(is.character(x.dates)){
##         msg <- paste("x.dates is a character vector; " ,
##                      "please convert it as dates using 'as.POSIXct'" ,
##                      "\n(making sure dates are given as 'YYYY/MM/DD' or 'YYYY-MM-DD').", sep="")
##         stop(msg)
##     }

##     x <- as.matrix(x)

##     x.dates <- as.POSIXct(round.POSIXt(x.dates,units="days")) # round dates to the day

##     if(length(x.names) != nrow(x)){
##         stop("inconsistent dimension for x")
##     }


##     ## HANDLE BEST==MIN ##
##     ## reverse x by a translation
##     x.old <- x
##     areZero <- x<1e-14
##     x <- (max(x)+1) - x
##     x[areZero] <- 0
##     diag(x) <- 0


##     ## BUILD THE GRAPH ##
##     ## make prox matrix with temporal consistency
##     if(force.temporal.order){
##         x[outer(myDates, myDates, ">=")] <- 0
##     }

##     ## tweak to get around a bug in as(...,"graphNEL") - looses edge weights
##     ## to replace with commented line once fixed in CRAN release
##     ## myGraph <- as(x, "graphNEL")
##     myGraph <- as(new("graphAM", x, edgemode="directed", values=list(weight=1)), "graphNEL")


##     ## CALL EDMONDSOPTIMUMBRANCHING  ##
##     temp <- edmondsOptimumBranching(myGraph)


##     ## SHAPE OUTPUT ##
##     if(res.type=="seqTrack"){
##         ## reorder output from edmondsOptimumBranching
##         N <- length(x.names)
##         myLev <- x.names
##         ances <- as.integer(factor(temp$edgeList["from",], levels=myLev))
##         desc <- as.integer(factor(temp$edgeList["to",], levels=myLev))
##         newOrd <- order(desc)
##         desc <- 1:N
##         ances <- ances[newOrd]
##         weights <- as.numeric(temp$weights)[newOrd]

##         ## create the data.frame
##         res <- data.frame(id=1:N)
##         hasNoAnces <- difftime(myDates, min(myDates), units="secs") < 1 # 1 sec resolution for dates
##         res$weight <- res$ances <- 1:N
##         res$date <- x.dates

##         ## fill in the d.f. with correct values
##         res$ances[!hasNoAnces] <- ances
##         res$ances[hasNoAnces] <- NA

##         res$weight[!hasNoAnces] <- weights
##         res$weight[hasNoAnces] <- NA

##         res$ances.date <- x.dates[res$ances]

##         res[is.na(res)] <- NA # have clean NAs
##         row.names(res) <- x.names
##         class(res) <- c("seqTrack","data.frame")

##         ## handle best==min
##         if(best=="min"){
##             res$weight <- max(x.old)+1 - res$weight
##         }


##     }

##     if(res.type=="graphNEL"){
##         ## handle optim==min
##         if(best=="min"){
##             temp$weights <- max(x.old)+1 - temp$weights
##         }
##         res <- ftM2graphNEL(t(temp$edgeList), W=temp$weights, edgemode="directed")
##     }


##     ## RETURN RESULT ##
##     return(res)

## } # end seqTrackG




#####################
## optimize.seqTrack
#####################
##
## TODO:
## 1) Change the output to retain xxx simulations | ok. -- done.
## 2) VECTORIZE mu and seq.length, recycle if needed with a warning
## 3) uncomment, adapt, and test code for missing data
##
## optimize.seqTrack.default <- function(x, x.names, x.dates, typed.chr=NULL, mu=NULL, seq.length=NULL,
##                                       thres=0.2, best=c("min","max"), prox.mat=NULL, nstep=10, step.size=1e3,
##                                       rDate=.rTimeSeq, arg.rDate=NULL, rMissDate=.rUnifDate, ...){


##     ## CHECKS ##
##     best <- match.arg(best)
##     if(best=="min"){
##         which.best <- which.min
##     } else {
##         which.best <- which.max
##     }

##     if(length(x.names) != length(x.dates)){
##         stop("inconsistent length for x.dates")
##     }

##     if(is.character(x.dates)){
##         msg <- paste("x.dates is a character vector; " ,
##                      "please convert it as dates using 'as.POSIXct'" ,
##                      "\n(making sure dates are given as 'YYYY/MM/DD' or 'YYYY-MM-DD').", sep="")
##         stop(msg)
##     }

##     isMissDate <- is.na(x.dates)

##     if(!identical(rDate, .rTimeSeq)){
##         if(is.null(arg.rDate)){
##             warning("Specific time distribution specified without arguments.")
##             arg.rDate <- list(n=step.size)
##         } else {
##             if(!is.list(arg.rDate)) stop("If provided, arg.rDate must be a list.")
##             if(!is.null(arg.rDate$n)) {
##                 warning("arg.rDate$n is provided, but will be replaced by step.size.")
##             }
##             arg.rDate$n <- step.size
##         }
##     }


##     N <- length(x.names)
##     id <- 1:N
##     ## if(length(mu) < N) { # recycle mu
##     ##         mu <- rep(mu, length=N)
##     ##     }
##     ##     if(length(seq.length) < N) {# recycle seq.length
##     ##         seq.length <- rep(seq.length, length=N)
##     ##     }


##     ## handle typed.chr, mu, seq.length
##     if(identical(rDate, .rTimeSeq)){
##         if(is.null(typed.chr)|is.null(mu)|is.null(seq.length)){
##             stop("typed.chr, mu, and seq.length must be provided if rDate is .rTimeSeq")
##         }

##         if(!is.list(typed.chr)) {
##             stop("typed.chr must be a list")
##         }
##         if(length(typed.chr)!=N) {
##             stop("typed.chr has an inconsistent length")
##         }

##         if(is.null(names(mu))) stop("mu has no names")
##         if(is.null(names(seq.length))) stop("seq.length has no names")
##         if(any(mu > 1)) stop("mu has values > 1")
##         if(any(mu < 0)) stop("mu has negative values")

##         if(!identical(names(mu) , names(seq.length))) stop("Names of mu and seq.length differ.")
##         if(any(!unique(unlist(typed.chr)) %in% names(mu))) {
##             stop("Some chromosomes indicated in typed.chr are not in mu.")
##         }

##         list.mu <- lapply(typed.chr, function(e) mu[e])
##         list.seq.length <- lapply(typed.chr, function(e) seq.length[e])
##     }

##     x.dates <- as.POSIXct(round.POSIXt(x.dates,units="days")) # round dates to the day

##     x <- as.matrix(x)

##     if(!is.null(prox.mat) && !identical(dim(prox.mat),dim(x))) {
##         stop("prox.mat is provided but its dimensions are inconsistent with that of x")
##     }


##     ## rename dimensions using id
##     colnames(x) <- rownames(x) <- id

##     if(length(x.names) != nrow(x)){
##         stop("inconsistent dimension for x")
##     }


##     ## SET THRESHOLD IF NEEDED ## ## NO LONGER USED
##     ##  if(is.null(thres)){
##     ##         thres <- sum(seqTrack(x.names=x.names, x.dates=x.dates, W=W,
##     ##                                     best=best, prox.mat=prox.mat, ...)$weight, na.rm=TRUE)
##     ##     }


##     ## AUXILIARY FUNCTIONS ##

##     ## to compare results -> returns a list of length two: logical, and the value of the res
##     val.res <- function(res){
##         return(sum(res$weight, na.rm=TRUE))
##     }


##     ## DO THE OPTIMISATION ##
##     RANGE.DATES <- as.integer(round(diff(range(x.dates, na.rm=TRUE)))) # time window of the sample, in days
##     NB.DATES.TO.SIM <- sum(!isMissDate)


##     ## for loop is not to slow for < 1e6 rep
##     ## and allows not to handle huge objects
##     ## (which would grow exponentially)

##     ##    res.best <- res.ini # initialization


##     ## DEFINE OUTPUTS ##
##     ances <- integer(0)
##     date <- character(0)
##     ances.date <- character(0)
##     valRes <- numeric(0)


##     ## DEFAULT CASE: NO MISSING DATES
##     if(!any(isMissDate)){
##         ## dates initialisation, taken from initial prior
##         ## If dates distrib is .rTimeSeq
##         if(identical(rDate, .rTimeSeq)){
##             newDates <- sapply(1:N, function(i)
##                                rDate(n=step.size, mu=list.mu[[i]], L=list.seq.length[[i]],
##                                      maxNbDays=RANGE.DATES))
##         } else { ## Else, any other distrib with free arguements
##             newDates <- sapply(1:N, function(i) do.call(rDate, arg.rDate))
##         }

##         newDates <- t(newDates)*24*3600 + x.dates

##         ## >> one step of 'step.size' simulations, all with same prior << ##
##         for(i in 1:nstep){
##             ## >> each step contains 'step.size' iterations << ##
##             for(j in 1:step.size){
##                 myDates <- as.POSIXct(newDates[,j])

##                 res.new <- seqTrack(x, x.names=x.names, x.dates=myDates,
##                                     best=best, prox.mat=prox.mat, ...)

##                 ##ances <- cbind(ances, res.new$ances) # not needed now
##                 date <- cbind(date, as.character(res.new$date))
##                 ##ances.date <- cbind(ances.date, as.character(res.new$ances.date)) # not needed now
##                 valRes <- c(valRes, val.res(res.new))
##                 ##}
##             } # end for j

##             ## retain a given % (thres) of the dates ##
##             toKeep <- valRes <= quantile(valRes, thres) ## NOT WORKING FOR optim==max !!!
##             valRes <- valRes[toKeep]

##             date <- date[,toKeep,drop=FALSE] # retained posterior

##             ## DEBUGING ##
##             ## cat("\ntoKeep:\n")
##             ##             print(toKeep)
##             ##             cat("\nhead date (posterior):\n")
##             ##             print(head(date))
##             ## END DEBUGING ##

##             newDates <- apply(date, 1, function(vec)
##                               sample(vec, size=step.size, replace=TRUE)) # new prior
##             newDates <- t(newDates)

##             ## stop if all dates are fixed
##             if(all(apply(newDates, 1, function(r) length(unique(r))==1))){
##                 cat("\nConvergence reached at step",i,"\n")
##                 break # stop the algorithm
##             }

##             ## re-initialize posterior distributions
##             if(i<nstep){
##                 ## ances <- integer(0) # not needed now
##                 date <- character(0)
##                 ## ances.date <- character(0) # not needed now
##                 valRes <- numeric(0)
##             } # end if

##         } # end for i

##         ##  ## dates: new prior taken from obtained posterior
##         ##             if(length(valRes)==0) { # if no simul are retained
##         ##                 warning(paste("No simulation was retained at the given threshold at step",i))
##         ##             } else {
##         ##  if(optim=="min"){ # define weights for further samplings
##         ##                     w <- max(valRes,na.rm=TRUE) - valRes
##         ##                     w <- w/sum(w)
##         ##                 } else {
##         ##                     w <- valRes
##         ##                     w <- w/sum(w)
##         ##                 }

##         ## newDates <- apply(date, 1, function(vec) #  used a weighted sampling
##         ##                                  sample(vec, size=step.size, replace=TRUE, prob=w))


##     } # end if(!any(isMissDate))


##     ##  ## OTHER CASE: HANDLE MISSING DATES
##     ##     if(any(isMissDate)){

##     ##         ## Handle distribution and its parameters ##
##     ##         argList <- list(...)

##     ##         if(is.null(argList$dateMin) & identical(rMissDate, .rUnifDate)){ # earliest date
##     ##             argList$dateMin <- min(x.dates,na.rm=TRUE)
##     ##         } else {
##     ##             argList$dateMin[is.na(argList$dateMin)] <- min(x.dates,na.rm=TRUE)
##     ##         }
##     ##         if(is.null(argList$dateMax) & identical(rMissDate, .rUnifDate)){ # latest date
##     ##             argList$dateMax <- max(x.dates,na.rm=TRUE)
##     ##         } else {
##     ##             argList$dateMax[is.na(argList$dateMax)] <- max(x.dates,na.rm=TRUE)
##     ##         }

##     ##         argList$n <- sum(isMissDate)

##     ##         ## Make simulations ##
##     ##         for(i in 1:nstep){
##     ##             myDates <- x.dates
##     ##             ## distribution for available dates
##     ##             myDates[!isMissDate] <- myDates[!isMissDate] +
##     ##                 rDate(n=NB.DATES.TO.SIM, mu=mu, L=seq.length, maxNbDays=2*RANGE.DATES)
##     ##             ## distribution for missing dates
##     ##             myDates[isMissDate] <- do.call(rMissDate, argList)

##     ##             res.new <- seqTrack(x.names=x.names, x.dates=myDates, W=W, optim=optim, prox.mat=prox.mat, ...)

##     ##             valRes[i] <- sum(res.new$weight,na.rm=TRUE)
##     ##             if(use.new.res(res.best, res.new)){
##     ##                 res.best <- res.new
##     ##             }
##     ##         }
##     ##     }


##     ## RESULT ##

##     ## reconstruct the result with new dates
##     res <- lapply(1:ncol(date), function(i)
##                    seqTrack(x=x, x.names=x.names, x.dates=as.POSIXct(date[,i]),
##                                     best=best, prox.mat=prox.mat, ...))
##     ances <- data.frame(lapply(res, function(e) e$ances))
##     ances <- matrix(as.integer(unlist(ances)), nrow=nrow(ances))

##     ances.date <- data.frame(lapply(res, function(e) as.character(e$ances.date)))
##     ances.date <- matrix(as.character(unlist(ances.date)), nrow=nrow(ances.date))

##     res <- list(ances=ances, date=date, ances.date=ances.date, valsim=valRes)
##     return(res)

## } # optimize.seqTrack






## #################
## ## get.result.by
## #################
## .get.result.by <- function(x, ...){
##     dat <- list(...)
##     if(length(dat)==0) return(x)


##     ## DEFINE NEW VALUES ##

##     convertElem <- function(e){
##         if(class(e)=="DNAbin") {
##             e <- as.matrix(e)
##         }
##         ori.dim <- dim(e)
##         e <- as.character(e)
##         dim(e) <- ori.dim
##         return(e)
##     }


##     dat <- lapply(dat,convertElem)
##     dat <- as.matrix(data.frame(dat))

##     newval <- apply(dat, 1, function(vec) paste(vec, collapse=""))
##     newval <- unclass(factor(newval))
##     newlev <- levels(newval)


##     ## if x is a single output of seqTrack
##     if(is.vector(x$ances)){
##         newId <- newval # new values
##         newAnces <- newval[x$ances] # new values
##         ## make output
##         res <- x
##         res$id <- newId
##         res$ances <- newAnces
##         attr(res$ances, "levels") <- newlev
##     }


##     ## if x is an optimize.seqTrack output
##     if(is.matrix(x$ances)){
##         res <- x
##         ori.ncol <- ncol(res$ances)
##         res$ances <- matrix(newval[res$ances], ncol=ori.ncol)
##         attr(res$ances, "levels") <- newlev
##     }

##     ## method for haploGen
##     if(inherits(x,"haploGen")){
##         res <- x
##         ances.id <- match(x$ances, labels(x))
##     }

##     return(res)

## } # end get.result.by






## #################
## ## get.consensus
## #################
## .get.consensus <- function(orires, listres, mode=c("majority","best")){
##     ## handle mode
##     mode <- match.arg(mode)

##     res <- orires

##     if(mode=="majority"){
##         nbDraws <- 0

##         ## tables of occurences of ancestors
##         temp <- apply(listres$ances, 1, table)

##         ## compute compromise
##         if(!is.list(temp)){
##             newances <- temp
##             ances.support <- rep(1,length(temp))
##         } else {
##             f1 <- function(tab){
##                 if(length(tab)==0) return(NA)

##                 res <- names(tab)[tab==max(tab)]
##                 ## if(length(res)==1) return(res)
##                 ##             return(NA)
##                 if(length(res)>1) {
##                     nbDraws <- nbDraws+1
##                 }
##                 return(res[1])
##             }

##             newances <- sapply(temp, f1)
##             ances.support <- sapply(temp, function(e) max(e, na.rm=TRUE)/sum(e, na.rm=TRUE))
##             ances.support[is.na(newances)] <- NA
##         }

##         ## form the output
##         olev <- levels(orires$ances)
##         res$ances <- newances
##         levels(res$ances) <- olev
##         res$support <- ances.support
##         res$weight <- rep(1, length(res$date))

##         if(is.numeric(listres$ances)){
##             res$ances <- as.numeric(res$ances)
##         }
##         cat("\nThere were\n",nbDraws, "draws.\n")
##     } # end majority


##     if(mode=="best"){
##         toKeep <- which.min(listres$valsim)
##         nbDraws <- sum(listres$valsim < (min(listres$valsim) + 1e-10 )) -1
##         cat("\nThere were\n",nbDraws, "draws.\n")

##         res$ances <- listres$ances[,toKeep]
##         res$inf.date <- listres$date[,toKeep]
##         res$ances.date <- listres$ances.date[,toKeep]
##         res$weight <- rep(-1, length(res$date))
##     }

##     return(res)
## } # end get.consensus

## pctest <- function(x, nperm=99, center=TRUE, scale=TRUE, method=c("sigma", "binom"), quiet=FALSE, plot=TRUE){
##     ## check x
##     if(!is.genind(x) & !is.genpop(x)){
##         stop("x is not a genind or a genpop object")
##     }

##     ## a few general variables
##     N <- nrow(x@tab)
##     P <- ncol(x@tab)

##     ## make tables of allele frequencies
##     X <- scaleGen(x, center=center, scale=scale, method=method, missing="mean")
##     fac.loc <- factor(sub("[.][^.]*$","",colnames(X)))
##     lX <- lapply(levels(fac.loc), function(id) X[,fac.loc==id,drop=FALSE])

##     ## auxil function to compute the first eigenvalue
##     if(N > P){ # N > P
##         f1 <- function(A){
##             Z <- t(A) %*% A / N
##             return(eigen(Z, symmetric=TRUE, only.values=TRUE)$values[1])
##         }
##     } else { #p <= n
##         f1 <- function(A){
##             Z <- A %*% t(A) / N
##             return(eigen(Z, symmetric=TRUE, only.values=TRUE)$values[1])
##         }
##     }


##     ## Monte Carlo procedure
##     makeOnePerm <- function(listX){
##         return(as.matrix(data.frame(lapply(listX, function(e) e[sample(N),,drop=FALSE]))))
##     }

##     if(quiet){
##         sim <- sapply(1:nperm, function(i) f1(makeOnePerm(lX)))
##     } else {
##         cat("\n Computing", nperm, "simulated eigenvalues ")
##         sim <- sapply(1:nperm, function(i) {cat(ifelse(i%%10==0, i, "."));return(f1(makeOnePerm(lX)))} )
##         cat(" done.\n")
##     }
##     ini <- f1(X)

##     ## return res
##     myCall <- match.call()
##     res <- as.randtest(sim=sim, obs=ini, alter="greater", call=myCall)
##     if(plot) {
##         plot(res, nclass=NULL, main="1st eigenvalue vs simulated eigenvalues (histogram)")
##     }
##     return(res)
## }

#' Export analysis for mvmapper visualisation
#'
#' \code{mvmapper} is an interactive tool for visualising outputs of a
#' multivariate analysis on a map from a web browser. The function
#' \code{export_to_mvmapper} is a generic with methods for several standard
#' classes of analyses in \code{adegenet} and \code{ade4}. Information on
#' individual locations, as well as any other relevant data, is passed through
#' the second argument \code{info}. By default, the function returns a formatted
#' \code{data.frame} and writes the output to a .csv file.\cr
#'
#' \code{mvmapper} can be found at:
#' \url{https://popphylotools.github.io/mvMapper/}
#'
#' @aliases swallowtails
#' 
#' @author Thibaut Jombart \email{thibautjombart@@gmail.com}
#'
#'
#' @param x The analysis to be exported. Can be a \code{dapc}, \code{spca}, or a
#'   \code{dudi} object.
#'
#' @param info A \code{data.frame} with additional information containing at
#' least the following columns: \code{key} (unique individual identifier),
#' \code{lat} (latitude), and \code{lon} (longitude). Other columns will be
#' exported as well, but are optional.
#'
#'
#' @param write_file A \code{logical} indicating if the output should be written
#'   out to a .csv file. Defaults to \code{TRUE}.
#'
#' @param out_file A character string indicating the file to which the output
#'   should be written. If NULL, the file used will be named
#'   \code{'mvmapper_data_[date and time].csv'}
#'
#' @param ... Further arguments to pass to other methods.
#'
#' @return A \code{data.frame} which can serve as input to \code{mvmapper},
#' containing at least the following columns:
#'
#' \itemize{
#'
#' \item \code{key}: unique individual identifiers
#'
#' \item \code{PC1}: first principal component; further principal components are
#' optional, but if provided will be numbered and follow \code{PC1}.
#'
#' \item \code{lat}: latitude for each individual
#'
#' \item \code{lon}: longitude for each individual
#'
#' }
#'
#' In addition, specific information is added for some analyses:
#'
#' \itemize{
#'
#' \item \code{spca}: \code{Lag_PC} columns contain the lag-vectors of the
#' principal components; the lag operator computes, for each individual, the
#' average score of neighbouring individuals; it is useful for clarifying
#' patches and clines.
#'
#' \item \code{dapc}: \code{grp} is the group used in the analysis;
#' \code{assigned_grp} is the group assignment based on the discriminant
#' functions; \code{support} is the statistical support (i.e. assignment
#' probability) for \code{assigned_grp}.
#'
#' }
#'
#'
#'
#' @export
#'
#' @rdname export_to_mvmapper
#'
#' @seealso
#'
#' \code{mvmapper} is available at:
#' \url{https://popphylotools.github.io/mvMapper/}
#'


export_to_mvmapper <- function(x, ...) {
  UseMethod("export_to_mvmapper")
}





#' @export
#' @rdname export_to_mvmapper

export_to_mvmapper.default <- function(x, ...) {
  msg <- sprintf("No method available for the class %s",
                 paste(class(x), collapse = ", "))
  stop(msg)
}






## All method will consist in merging output from the analysis with extra info
## containing latitude and longitude, stored in 'info'.

#' @export
#' @rdname export_to_mvmapper
#' @examples
#'
#' # An example using the microsatellite dataset of Dupuis et al. 2016 (781
#' # individuals, 10 loci, doi: 10.1111/jeb.12931)
#'
#' # Reading input file from adegenet
#'
#' input_data <- system.file("data/swallowtails.rda", package="adegenet")
#' data(swallowtails)
#'
#'
#' # conducting a DAPC (n.pca determined using xvalDapc, see ??xvalDapc)
#'
#' dapc1 <- dapc(swallowtails, n.pca=40, n.da=200)
#'
#'
#' # read in swallowtails_loc.csv, which contains "key", "lat", and "lon"
#' # columns with column headers (this example contains additional columns
#' # containing species identifications, locality descriptions, and COI
#' # haplotype clades)
#'
#' input_locs <- system.file("files/swallowtails_loc.csv", package = "adegenet")
#' loc <- read.csv(input_locs, header = TRUE)
#'
#'
#' # generate mvmapper input file, automatically write the output to a csv, and
#' # name the output csv "mvMapper_Data.csv"
#'
#' out <- export_to_mvmapper(dapc1, loc, write_file = TRUE,
#'                           out_file = "mvMapper_Data.csv")
#'

export_to_mvmapper.dapc <- function(x, info, write_file = TRUE, out_file = NULL, ...) {

  ## Extract principal components, groups, assigned groups and the corresponding
  ## probability.

  pcs <- x$ind.coord
  colnames(pcs) <- paste0("PC", 1:ncol(pcs))
  key <- rownames(pcs)
  grp <- x$grp
  assigned_grp <- x$assign
  support <- apply(x$posterior, 1, max)


  analysis <- cbind.data.frame(key, pcs,
                               grp,
                               assigned_grp,
                               support)

  ## process 'info' (checks that required columns are there)
  info <- .check_info(info, key)

  out <- merge(analysis, info, by = "key")
  .write_mvmapper_output(out, write_file, out_file)
  return(out)
}






#' @export
#' @rdname export_to_mvmapper

export_to_mvmapper.dudi <- function(x, info, write_file = TRUE, out_file = NULL, ...) {

  ## Extract principal components, groups, assigned groups and the corresponding
  ## probability.

  pcs <- x$li
  colnames(pcs) <- paste0("PC", 1:ncol(pcs))
  key <- rownames(pcs)

  analysis <- cbind.data.frame(key, pcs)

  ## process 'info' (checks that required columns are there)
  info <- .check_info(info, key)

  out <- merge(analysis, info, by = "key")
  .write_mvmapper_output(out, write_file, out_file)
  return(out)
}






#' @export
#' @rdname export_to_mvmapper


export_to_mvmapper.spca <- function(x, info, write_file = TRUE, out_file = NULL, ...) {

  ## Extract principal components, groups, assigned groups and the corresponding
  ## probability.

  pcs <- x$li
  colnames(pcs) <- paste0("PC", 1:ncol(pcs))
  lag_pcs <- x$ls
  colnames(lag_pcs) <- paste0("Lag_PC", 1:ncol(pcs))
  key <- rownames(pcs)

  analysis <- cbind.data.frame(key, pcs, lag_pcs)

  ## process 'info' (checks that required columns are there)
  info <- .check_info(info, key)

  out <- merge(analysis, info, by = "key")
  .write_mvmapper_output(out, write_file, out_file)
  return(out)
}






## This internal function will merely check the content of the extra 'info'
## being provided, making sure key, latitude and longitude are provided.

.check_info <- function(info, ref_keys,
                       look_for = c("key", "lat", "lon")) {

  info <- as.data.frame(info)

  if (length(look_for) > 0L) {
    for (e in look_for) {
      if (!e %in% names(info)) {
        msg <- sprintf("'info' is missing a '%s' column", e)
        stop(msg)
      }
    }
  }

  nb_missing <- sum(!ref_keys %in% info$key)
  if (nb_missing > 0L) {
    msg <- sprintf("%d individuals are not documented in 'info'",
                   nb_missing)
    warning(msg)
  }

  return(info)
}






## This internal function writes results to a csv file if needed, and does
## nothing otherwise.
##
## 'x' is the data.frame output from the export function
## other arguments as documented
##
.write_mvmapper_output <- function(x, write_file = TRUE, out_file = NULL) {
  if (write_file) {
    if (is.null(out_file)) {
      out_file <- paste0("mvmapper_data_",
                         gsub(" ", "_", Sys.time()),
                         ".csv")
    }
    message("Writing output to the file: ", out_file)
    write.csv(x, out_file, row.names = FALSE)
  }
}

#' The adegenet package
#'
#'
#' This package is devoted to the multivariate analysis of genetic markers
#' data. These data can be codominant markers (e.g. microsatellites) or
#' presence/absence data (e.g. AFLP), and have any level of ploidy.  'adegenet'
#' defines three formal (S4) classes:\cr - \linkS4class{genind}: a class for
#' data of individuals ("genind" stands for genotypes-individuals).\cr -
#' \linkS4class{genpop}: a class for data of groups of individuals ("genpop"
#' stands for genotypes-populations)\cr - \linkS4class{genlight}: a class for
#' genome-wide SNP data\cr
#'
#' For more information about these classes, type "class ? genind", "class ?
#' genpop", or "?genlight".\cr
#'
#' Essential functionalities of the package are presented througout 4
#' tutorials, accessible using \code{adegenetTutorial(which="name-below")}:\cr
#' - \code{basics}: introduction to the package.\cr - \code{spca}: multivariate
#' analysis of spatial genetic patterns.\cr - \code{dapc}: population structure
#' and group assignment using DAPC.\cr - \code{genomics}: introduction to the
#' class \linkS4class{genlight} for the handling and analysis of genome-wide
#' SNP data.\cr
#'
#' Note: In older versions of adegenet, these tutorials were avilable as
#' vignettes, accessible through the function \code{vignette("name-below",
#' package="adegenet")}:\cr - \code{adegenet-basics}.\cr -
#' \code{adegenet-spca}.\cr - \code{adegenet-dapc}.\cr -
#' \code{adegenet-genomics}.\cr
#'
#' Important functions are also summarized below.\cr
#'
#' === IMPORTING DATA ===\cr = TO GENIND OBJECTS = \cr \code{adegenet} imports
#' data to \linkS4class{genind} object from the following softwares:\cr -
#' STRUCTURE: see \code{\link{read.structure}}\cr - GENETIX: see
#' \code{\link{read.genetix}}\cr - FSTAT: see \code{\link{read.fstat}}\cr -
#' Genepop: see \code{\link{read.genepop}}\cr To import data from any of these
#' formats, you can also use the general function
#' \code{\link{import2genind}}.\cr
#'
#' In addition, it can extract polymorphic sites from nucleotide and amino-acid
#' alignments:\cr - DNA files: use \code{\link[ape]{read.dna}} from the ape
#' package, and then extract SNPs from DNA alignments using
#' \code{\link{DNAbin2genind}}. \cr
#'
#' - protein sequences alignments: polymorphic sites can be extracted from
#' protein sequences alignments in \code{alignment} format (package
#' \code{seqinr}, see \code{\link[seqinr]{as.alignment}}) using the function
#' \code{\link{alignment2genind}}. \cr
#'
#' The function \code{\link{fasta2DNAbin}} allows for reading fasta files into
#' DNAbin object with minimum RAM requirements.\cr
#'
#' It is also possible to read genotypes coded by character strings from a
#' data.frame in which genotypes are in rows, markers in columns. For this, use
#' \code{\link{df2genind}}. Note that \code{\link{df2genind}} can be used for
#' any level of ploidy.\cr
#'
#' = TO GENLIGHT OBJECTS = \cr SNP data can be read from the following
#' formats:\cr - PLINK: see function \code{\link{read.PLINK}}\cr - .snp
#' (adegenet's own format): see function \code{\link{read.snp}}\cr
#'
#' SNP can also be extracted from aligned DNA sequences with the fasta format,
#' using \code{\link{fasta2genlight}}\cr
#'
#' === EXPORTING DATA ===\cr \code{adegenet} exports data from
#'
#' Genotypes can also be recoded from a \linkS4class{genind} object into a
#' data.frame of character strings, using any separator between alleles. This
#' covers formats from many softwares like GENETIX or STRUCTURE. For this, see
#' \code{\link{genind2df}}.\cr
#'
#' Also note that the \code{pegas} package imports \linkS4class{genind} objects
#' using the function \code{as.loci}.
#'
#' === MANIPULATING DATA ===\cr Several functions allow one to manipulate
#' \linkS4class{genind} or \linkS4class{genpop} objects\cr -
#' \code{\link{genind2genpop}}: convert a \linkS4class{genind} object to a
#' \linkS4class{genpop} \cr - \code{\link{seploc}}: creates one object per
#' marker; for \linkS4class{genlight} objects, creates blocks of SNPs.\cr -
#' \code{\link{seppop}}: creates one object per population \cr -
#' - \code{\link{tab}}: access the allele data (counts or frequencies) of an object
#' (\linkS4class{genind} and \linkS4class{genpop}) \cr -
#' x[i,j]: create a new object keeping only genotypes (or populations) indexed
#' by 'i' and the alleles indexed by 'j'.\cr - \code{\link{makefreq}}: returns
#' a table of allelic frequencies from a \linkS4class{genpop} object.\cr -
#' \code{\link{repool}} merges genoptypes from different gene pools into one
#' single \linkS4class{genind} object.\cr - \code{\link{propTyped}} returns the
#' proportion of available (typed) data, by individual, population, and/or
#' locus.\cr - \code{\link{selPopSize}} subsets data, retaining only genotypes
#' from a population whose sample size is above a given level.\cr -
#' \code{\link{pop}} sets the population of a set of genotypes.\cr
#'
#' === ANALYZING DATA ===\cr Several functions allow to use usual, and less
#' usual analyses:\cr - \code{\link{HWE.test.genind}}: performs HWE test for all
#' populations and loci combinations \cr - \code{\link{dist.genpop}}: computes 5
#' genetic distances among populations.  \cr - \code{\link{monmonier}}:
#' implementation of the Monmonier algorithm, used to seek genetic boundaries
#' among individuals or populations. Optimized boundaries can be obtained using
#' \code{\link{optimize.monmonier}}. Object of the class \code{monmonier} can be
#' plotted and printed using the corresponding methods. \cr -
#' \code{\link{spca}}: implements Jombart et al.  (2008) spatial Principal
#' Component Analysis \cr - \code{\link{global.rtest}}: implements Jombart et
#' al. (2008) test for global spatial structures \cr -
#' \code{\link{local.rtest}}: implements Jombart et al. (2008) test for local
#' spatial structures \cr - \code{\link{propShared}}: computes the proportion of
#' shared alleles in a set of genotypes (i.e. from a genind object)\cr -
#' \code{\link{propTyped}}: function to investigate missing data in several ways
#' \cr - \code{\link{scaleGen}}: generic method to scale \linkS4class{genind} or
#' \linkS4class{genpop} before a principal component analysis \cr -
#' \code{\link{Hs}}: computes the average expected heterozygosity by population
#' in a \linkS4class{genpop}. Classically Used as a measure of genetic
#' diversity.\cr - \code{\link{find.clusters}} and \code{\link{dapc}}: implement
#' the Discriminant Analysis of Principal Component (DAPC, Jombart et al.,
#' 2010).\cr - \code{\link{seqTrack}}: implements the SeqTrack algorithm for
#' recontructing transmission trees of pathogens (Jombart et al., 2010) .\cr
#' \code{\link{glPca}}: implements PCA for \linkS4class{genlight} objects.\cr -
#' \code{\link{gengraph}}: implements some simple graph-based clustering using
#' genetic data.  - \code{\link{snpposi.plot}} and \code{\link{snpposi.test}}:
#' visualize the distribution of SNPs on a genetic sequence and test their
#' randomness.  - \code{\link{adegenetServer}}: opens up a web interface for
#' some functionalities of the package (DAPC with cross validation and feature
#' selection).\cr
#'
#' === GRAPHICS ===\cr - \code{\link{colorplot}}: plots points with associated
#' values for up to three variables represented by colors using the RGB system;
#' useful for spatial mapping of principal components.\cr -
#' \code{\link{loadingplot}}: plots loadings of variables. Useful for
#' representing the contribution of alleles to a given principal component in a
#' multivariate method. \cr - \code{\link{scatter.dapc}}: scatterplots for DAPC
#' results.\cr - \code{\link{compoplot}}: plots membership probabilities from a
#' DAPC object. \cr
#'
#' === SIMULATING DATA ===\cr - \code{\link{hybridize}}: implements
#' hybridization between two populations. \cr - \code{\link{haploGen}}:
#' simulates genealogies of haplotypes, storing full genomes. \cr
#' \code{\link{glSim}}: simulates simple \linkS4class{genlight} objects.\cr
#'
#' === DATASETS ===\cr - \code{\link{H3N2}}: Seasonal influenza (H3N2) HA
#' segment data. \cr - \code{\link{dapcIllus}}: Simulated data illustrating the
#' DAPC. \cr - \code{\link{eHGDP}}: Extended HGDP-CEPH dataset. \cr -
#' \code{\link{microbov}}: Microsatellites genotypes of 15 cattle breeds. \cr -
#' \code{\link{nancycats}}: Microsatellites genotypes of 237 cats from 17
#' colonies of Nancy (France). \cr - \code{\link{rupica}}: Microsatellites
#' genotypes of 335 chamois (Rupicapra rupicapra) from the Bauges mountains
#' (France).\cr - \code{\link{sim2pop}}: Simulated genotypes of two
#' georeferenced populations.\cr - \code{\link{spcaIllus}}: Simulated data
#' illustrating the sPCA. \cr
#'
#' For more information, visit the adegenet website using the function
#' \code{\link{adegenetWeb}}.\cr
#'
#' Tutorials are available via the command \code{adegenetTutorial}.\cr
#'
#' To cite adegenet, please use the reference given by
#' \code{citation("adegenet")} (or see references below).
#'
#' @name adegenet.package
#' @encoding utf-8
#' @aliases adegenet.package adegenet
#' @docType package
#' @author Thibaut Jombart <t.jombart@@imperial.ac.uk>\cr
#' Developers: Zhian N. Kamvar <zkamvar@@gmail.com>,
#' Caitlin Collins <caitiecollins17@@gmail.com>,
#' Ismail Ahmed <ismail.ahmed@@inserm.fr>,
#' Federico Calboli, Tobias Erik Reiners, Peter
#' Solymos, Anne Cori, \cr Contributed datasets from: Katayoun
#' Moazami-Goudarzi, Denis Laloë, Dominique Pontier, Daniel Maillard, Francois
#' Balloux.
#' @seealso adegenet is related to several packages, in particular:\cr -
#' \code{ade4} for multivariate analysis\cr - \code{pegas} for population
#' genetics tools\cr - \code{ape} for phylogenetics and DNA data handling\cr -
#' \code{seqinr} for handling nucleic and proteic sequences\cr - \code{shiny}
#' for R-based web interfaces\cr
#' @references Jombart T. (2008) adegenet: a R package for the multivariate
#' analysis of genetic markers \emph{Bioinformatics} 24: 1403-1405. doi:
#' 10.1093/bioinformatics/btn129\cr
#'
#' Jombart T. and Ahmed I. (2011) adegenet 1.3-1: new tools for the analysis of
#' genome-wide SNP data.  \emph{Bioinformatics}. doi:
#' 10.1093/bioinformatics/btr521
#'
#' Jombart T, Devillard S and Balloux F (2010) Discriminant analysis of
#' principal components: a new method for the analysis of genetically
#' structured populations. BMC Genetics 11:94.  doi:10.1186/1471-2156-11-94\cr
#'
#' Jombart T, Eggo R, Dodd P, Balloux F (2010) Reconstructing disease outbreaks
#' from genetic data: a graph approach. \emph{Heredity}. doi:
#' 10.1038/hdy.2010.78.\cr
#'
#' Jombart, T., Devillard, S., Dufour, A.-B. and Pontier, D. (2008) Revealing
#' cryptic spatial patterns in genetic variability by a new multivariate
#' method. \emph{Heredity}, \bold{101}, 92--103.\cr
#'
#' See adegenet website: \url{http://adegenet.r-forge.r-project.org/}\cr
#'
#' Please post your questions on 'the adegenet forum':
#' adegenet-forum@@lists.r-forge.r-project.org
#' @keywords manip multivariate
#'
#' @exportPattern "^[^\\.]"
#'
#' @export .rmspaces .readExt .genlab .render.server.info
#'
#' @S3method as.POSIXct haploGen
#' @S3method as.data.frame genind
#' @S3method as.data.frame genlight
#' @S3method as.data.frame genpop
#' @S3method as.integer SNPbin
#' @S3method as.lda dapc
#' @S3method as.list genlight
#' @S3method as.matrix genind
#' @S3method as.matrix genlight
#' @S3method as.matrix genpop
#' @S3method c SNPbin
#' @S3method cbind SNPbin
#' @S3method "[" haploGen
#' @S3method cbind genlight
#' @S3method rbind genlight
#' @S3method xvalDapc data.frame
#' @S3method xvalDapc default
#' @S3method xvalDapc genind
#' @S3method xvalDapc genlight
#' @S3method xvalDapc matrix
#' @S3method colorplot default
#' @S3method colorplot spca
#' @S3method dapc data.frame
#' @S3method dapc dudi
#' @S3method dapc genind
#' @S3method dapc genlight
#' @S3method dapc matrix
#' @S3method find.clusters data.frame
#' @S3method find.clusters genind
#' @S3method find.clusters genlight
#' @S3method find.clusters matrix
#' @S3method findMutations DNAbin
#' @S3method gengraph DNAbin
#' @S3method gengraph default
#' @S3method gengraph dist
#' @S3method gengraph genind
#' @S3method gengraph genpop
#' @S3method gengraph matrix
#' @S3method get.likelihood seqTrack
#' @S3method graphMutations DNAbin
#' @S3method labels haploGen
#' @S3method loadingplot default
#' @S3method loadingplot glPca
#' @S3method pairDistPlot DNAbin
#' @S3method pairDistPlot default
#' @S3method pairDistPlot dist
#' @S3method pairDistPlot genind
#' @S3method pairDistPlot matrix
#' @S3method plot haploGen
#' @S3method plot monmonier
#' @S3method plot seqTrack
#' @S3method plot spca
#' @S3method predict dapc
#' @S3method print dapc
#' @S3method print glPca
#' @S3method print haploGen
#' @S3method print monmonier
#' @S3method print spca
#' @S3method scatter dapc
#' @S3method scatter glPca
#' @S3method screeplot spca
#' @S3method seqTrack default
#' @S3method seqTrack haploGen
#' @S3method seqTrack matrix
#' @S3method snpposi.plot DNAbin
#' @S3method snpposi.plot integer
#' @S3method snpposi.plot numeric
#' @S3method snpposi.test DNAbin
#' @S3method snpposi.test integer
#' @S3method snpposi.test numeric
#' @S3method spca default
#' @S3method spca matrix
#' @S3method spca data.frame
#' @S3method spca genind
#' @S3method spca genpop
#' @S3method summary dapc
#' @S3method summary spca
#' @S3method print genindSummary
#' @S3method print genpopSummary
#'
#'
#' @import methods
#'
#' @import parallel
#'
#' @import utils
#'
#' @import stats
#'
#' @import graphics
#'
#' @import grDevices
#'
#' @import ade4
#'
#' @importFrom seqinr s2c
#'
#' @importFrom MASS "lda"
#'
#' @importFrom ape "as.character.DNAbin" "as.DNAbin" "as.DNAbin.alignment"
#' "as.DNAbin.character" "as.DNAbin.list" "as.list.DNAbin" "as.matrix.DNAbin"
#' "cbind.DNAbin" "c.DNAbin" "[.DNAbin" "labels.DNAbin" "print.DNAbin"
#' "rbind.DNAbin" "dist.dna" "seg.sites"
#'
#' @importFrom igraph "graph.data.frame" "V" "V<-" "E" "E<-"
#' "layout.fruchterman.reingold" "as.igraph" "plot.igraph" "print.igraph"
#' "graph.adjacency" "clusters"
#'
#' @importFrom shiny "runApp" "renderPrint"
#'
#' @importFrom ggplot2 "ggplot" "geom_density" "geom_rug" "labs" "aes" "xlim"
#' "guides" "guide_legend" "geom_boxplot" "geom_violin" "geom_jitter"
#' "coord_flip"
#'
#' @useDynLib adegenet, .registration = TRUE
#'


NULL

#############
## find.clusters
#############
find.clusters <- function (x, ...) UseMethod("find.clusters")

############################
## find.clusters.data.frame
############################
find.clusters.data.frame <- function(x, clust = NULL, n.pca = NULL, n.clust = NULL,
                                     method = c("kmeans", "ward"),
                                     stat = c("BIC", "AIC", "WSS"), choose.n.clust = TRUE,
                                     criterion = c("diffNgroup", "min", "goesup",
                                                   "smoothNgoesup", "goodfit"),
                                     max.n.clust = round(nrow(x)/10), n.iter = 1e5,
                                     n.start = 10, center = TRUE, scale = TRUE,
                                     pca.select = c("nbEig","percVar"),
                                     perc.pca = NULL, ..., dudi = NULL){

    ## CHECKS ##
    stat <- match.arg(stat)
    pca.select <- match.arg(pca.select)
    criterion <- match.arg(criterion)
    min.n.clust <- 2
    max.n.clust <- max(max.n.clust, 2)
    method <- match.arg(method)
    
    ## KEEP TRACK OF SOME ORIGINAL PARAMETERS
    ## n.pca.ori <- n.pca
    ##n.clust.ori <- n.clust


    ## ESCAPE IF SUB-CLUST ARE SEEKED ##
    if(!is.null(clust)){
        res <- .find.sub.clusters(x = x, clust = clust, n.pca = n.pca,
                                  n.clust = n.clust, stat = stat,
                                  max.n.clust = max.n.clust,
                                  n.iter = n.iter, n.start = n.start,
                                  choose.n.clust = choose.n.clust,
                                  criterion = criterion,
                                  method = method,
                                  center = center, scale = scale)
        return(res)
    }
    ## END SUB-CLUST


    ## PERFORM PCA ##
    N <- nrow(x)
    REDUCEDIM <- is.null(dudi)

    if(REDUCEDIM){ # if no dudi provided
        ## PERFORM PCA ##
        maxRank <- min(dim(x))
        pcaX <- dudi.pca(x, center = center, scale = scale, scannf = FALSE, nf=maxRank)
    } else { # else use the provided dudi
        pcaX <- dudi
    }
    cumVar <- 100 * cumsum(pcaX$eig)/sum(pcaX$eig)

    if(!REDUCEDIM){
        myCol <- rep(c("black", "lightgrey"), c(ncol(pcaX$li),length(pcaX$eig)))
    } else {
        myCol <- "black"
    }

    ## select the number of retained PC for PCA
    if(is.null(n.pca) & pca.select == "nbEig"){
        plot(cumVar, xlab = "Number of retained PCs",
             ylab = "Cumulative variance (%)",
             main = "Variance explained by PCA",
             col = myCol)
        cat("Choose the number PCs to retain (>= 1): ")
        n.pca <- as.integer(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    if(is.null(perc.pca) & pca.select == "percVar"){
        plot(cumVar, xlab = "Number of retained PCs",
             ylab = "Cumulative variance (%)",
             main = "Variance explained by PCA",
             col = myCol)
        cat("Choose the percentage of variance to retain (0-100): ")
        nperc.pca <- as.numeric(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    ## get n.pca from the % of variance to conserve
    if(!is.null(perc.pca)){
        n.pca <- min(which(cumVar >= perc.pca))
        if(perc.pca > 99.999) n.pca <- length(pcaX$eig)
        if(n.pca<1) n.pca <- 1
    }


     ## keep relevant PCs - stored in XU
    X.rank <- length(pcaX$eig)
    n.pca <- min(X.rank, n.pca)
    if(n.pca >= N) warning("number of retained PCs of PCA is greater than N")
    ##if(n.pca > N/3) warning("number of retained PCs of PCA may be too large (> N /3)")

    XU <- pcaX$li[, 1:n.pca, drop=FALSE] # principal components

    ## PERFORM K-MEANS
    if(is.null(n.clust)){
        nbClust <- min.n.clust:max.n.clust
        WSS <- numeric(0)

        for(i in 1:length(nbClust)){
            if (method == "kmeans") {
                ## kmeans clustering (original method)
                temp <- kmeans(XU, centers = nbClust[i], iter.max = n.iter, nstart = n.start)
                ##WSS[i] <- sum(temp$withinss)
            } else {
                ## ward clustering
                temp <- list()
                temp$cluster <- cutree(hclust(dist(XU)^2, method = "ward.D2"), k = nbClust[i])
            }
                WSS[i] <- .compute.wss(XU, temp$cluster)
            
        }


        ## DETERMINE THE NUMBER OF GROUPS
        ##TSS <- sum(pcaX$eig) * N
        ##betweenVar <- (1 - ((stat/(N-nbClust-1))/(TSS/(N-1)) )) *100
        ##WSS.ori <- sum(apply(XU, 2, function(v) sum((v-mean(v))^2) ))
        ##reducWSS <- -diff(c(WSS.ori, stat))
        ##reducWSS <- reducWSS/max(reducWSS)

        if(stat=="AIC"){
            WSS.ori <- sum(apply(XU, 2, function(v) sum((v-mean(v))^2) ))
            k <- nbClust
            myStat <- N*log(c(WSS.ori,WSS)/N) + 2*c(1,nbClust)
            myLab <- "AIC"
            myTitle <- "Value of AIC \nversus number of clusters"

        }
        if(stat=="BIC"){
            WSS.ori <- sum(apply(XU, 2, function(v) sum((v-mean(v))^2) ))
            k <- nbClust
            myStat <- N*log(c(WSS.ori,WSS)/N) + log(N) *c(1,nbClust)
            myLab <- "BIC"
            myTitle <- "Value of BIC \nversus number of clusters"
        }
        if(stat=="WSS"){
            WSS.ori <- sum(apply(XU, 2, function(v) sum((v-mean(v))^2) ))
            myStat <- c(WSS.ori, WSS)
            ##            reducWSS <- -diff(c(WSS.ori, stat))
            ##            myStat <- reducWSS/max(reducWSS)
            myLab <- "Within sum of squares"
            myTitle <- "Value of within SS\nversus number of clusters"
        }

        if(choose.n.clust){
            plot(c(1,nbClust), myStat, xlab = "Number of clusters",
                 ylab = myLab, main = myTitle, type = "o", col = "blue")
            abline(h=0, lty=2, col="red")
            cat("Choose the number of clusters (>=2: ")
            n.clust <- NA
            while(is.na(n.clust)){
                n.clust <- max(1, as.integer(readLines(con = getOption('adegenet.testcon'), n = 1)))
            }
        } else {
            if(criterion=="min") {
                n.clust <- which.min(myStat)
            }
            if(criterion=="goesup") {
                ## temp <- diff(myStat)
                ## n.clust <- which.max( which( (temp-min(temp))<max(temp)/1e4))
                n.clust <- min(which(diff(myStat)>0))
            }
            if(criterion=="goodfit") {
                temp <- min(myStat) + 0.1*(max(myStat) - min(myStat))
                n.clust <- min( which(myStat < temp))-1
            }
            if(criterion=="diffNgroup") {
                temp <- cutree(hclust(dist(diff(myStat)), method="ward.D"), k=2)
                goodgrp <- which.min(tapply(diff(myStat), temp, mean))
                n.clust <- max(which(temp==goodgrp))+1
            }
            if(criterion=="smoothNgoesup") {
                temp <- myStat
                temp[2:(length(myStat)-1)] <- sapply(1:(length(myStat)-2),
                                                     function(i) mean(myStat[c(i,i+1,i+2)]))
                n.clust <- min(which(diff(temp)>0))
            }

        }
    } else { # if n.clust provided
        myStat <- NULL
    }

    ## get final groups
    if(n.clust >1){
        if (method == "kmeans") {
            best <-  kmeans(XU, centers = n.clust, iter.max = n.iter,
                            nstart = n.start)
        } else {
            best <- list()
            best$cluster <- cutree(hclust(dist(XU)^2, method = "ward.D2"),
                               k = n.clust)
            best$size <- table(best$cluster)
        }
         
    } else {
        best <- list(cluster=factor(rep(1,N)), size=N)
    }


    ## MAKE RESULT ##
    if(!is.null(myStat)){
        names(myStat) <- paste("K",c(1,nbClust), sep="=")
    }

    res <- list(Kstat=myStat, stat=myStat[n.clust], grp=factor(best$cluster), size=best$size)

    return(res)
} # end find.clusters.data.frame






########################
## find.clusters.genind
########################
find.clusters.genind <- function(x, clust = NULL, n.pca = NULL, n.clust = NULL, 
                                 method = c("kmeans", "ward"),
                                 stat = c("BIC", "AIC", "WSS"),
                                 choose.n.clust=TRUE, 
                                 criterion = c("diffNgroup", "min","goesup", "smoothNgoesup", "goodfit"),
                                 max.n.clust = round(nrow(x@tab)/10), n.iter = 1e5, n.start = 10,
                                 scale = FALSE, truenames = TRUE, ...){

    ## CHECKS ##
    if(!is.genind(x)) stop("x must be a genind object.")
    stat <- match.arg(stat)


    ## SOME GENERAL VARIABLES ##
    N <- nrow(x@tab)
    min.n.clust <- 2

    ## PERFORM PCA ##
    maxRank <- min(dim(x@tab))

    X <- scaleGen(x, center = TRUE, scale = scale,
                  NA.method = "mean")

    ## CALL DATA.FRAME METHOD
    res <- find.clusters(X, clust=clust, n.pca=n.pca, n.clust=n.clust, stat=stat,
                         max.n.clust=max.n.clust, n.iter=n.iter, n.start=n.start,
                         choose.n.clust=choose.n.clust, method = method,
                         criterion=criterion, center=FALSE, scale=FALSE,...)
    return(res)
} # end find.clusters.genind





###################
## find.clusters.matrix
###################
find.clusters.matrix <- function(x, ...){
    return(find.clusters(as.data.frame(x), ...))
}








##########################
## find.clusters.genlight
##########################
find.clusters.genlight <- function(x, clust = NULL, n.pca = NULL, n.clust = NULL,
                                   method = c("kmeans", "ward"),
                                   stat = c("BIC", "AIC", "WSS"),
                                   choose.n.clust = TRUE, 
                                   criterion = c("diffNgroup", "min","goesup", "smoothNgoesup", "goodfit"),
                                   max.n.clust = round(nInd(x)/10), n.iter = 1e5, n.start = 10,
                                   scale = FALSE, pca.select = c("nbEig","percVar"),
                                   perc.pca = NULL, glPca = NULL, ...){

    ## CHECKS ##
    if(!inherits(x, "genlight")) stop("x is not a genlight object.")
    stat <- match.arg(stat)
    pca.select <- match.arg(pca.select)


    ## SOME GENERAL VARIABLES ##
    N <- nInd(x)
    min.n.clust <- 2


    ## PERFORM PCA ##
    REDUCEDIM <- is.null(glPca)

    if(REDUCEDIM){ # if no glPca provided
        maxRank <- min(c(nInd(x), nLoc(x)))
        pcaX <- glPca(x, center = TRUE, scale = scale, nf=maxRank, loadings=FALSE, returnDotProd = FALSE, ...)
    } else {
        pcaX <- glPca
    }

    if(is.null(n.pca)){
        cumVar <- 100 * cumsum(pcaX$eig)/sum(pcaX$eig)
    }


    ## select the number of retained PC for PCA
    if(!REDUCEDIM){
        myCol <- rep(c("black", "lightgrey"), c(ncol(pcaX$scores),length(pcaX$eig)))
    } else {
        myCol <- "black"
    }

    if(is.null(n.pca) & pca.select=="nbEig"){
        plot(cumVar, xlab="Number of retained PCs", ylab="Cumulative variance (%)", main="Variance explained by PCA", col=myCol)
        cat("Choose the number PCs to retain (>=1): ")
        n.pca <- as.integer(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    if(is.null(perc.pca) & pca.select=="percVar"){
        plot(cumVar, xlab="Number of retained PCs", ylab="Cumulative variance (%)", main="Variance explained by PCA", col=myCol)
        cat("Choose the percentage of variance to retain (0-100): ")
        nperc.pca <- as.numeric(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    ## get n.pca from the % of variance to conserve
    if(!is.null(perc.pca)){
        n.pca <- min(which(cumVar >= perc.pca))
        if(perc.pca > 99.999) n.pca <- length(pcaX$eig)
        if(n.pca<1) n.pca <- 1
    }

    if(!REDUCEDIM){
        if(n.pca > ncol(pcaX$scores)) {
            n.pca <- ncol(pcaX$scores)
        }
    }


    ## convert PCA
    pcaX <- .glPca2dudi(pcaX)


    ## CALL DATA.FRAME METHOD
    res <- find.clusters(pcaX$li, clust=clust, n.pca=n.pca, n.clust=n.clust,
                         stat=stat, max.n.clust=max.n.clust, n.iter=n.iter, n.start=n.start,
                         choose.n.clust=choose.n.clust, method = method,
                         criterion=criterion, center=FALSE, scale=FALSE, dudi=pcaX)
    return(res)
} # end find.clusters.genlight










###################
## .find.sub.clusters
###################
.find.sub.clusters <- function(x, ...){

    ## GET ... ##
    myArgs <- list(...)
    if(!is.null(myArgs$quiet)){
        quiet <- myArgs$quiet
        myArgs$quiet <- NULL
    } else {
        quiet <- FALSE
    }

    clust <- myArgs$clust
    myArgs$clust <- NULL

    if(is.null(clust)) stop("clust is not provided")
    clust <- as.factor(clust)

    ## temp will store temporary resuts
    newFac <- character(length(clust))

    ## find sub clusters
    for(i in levels(clust)){
        if(!quiet) cat("\nLooking for sub-clusters in cluster",i,"\n")
        myArgs$x <- x[clust==i, , drop = FALSE]
        myArgs$max.n.clust <- nrow(x[clust==i, , drop = FALSE]) - 1
        temp <- do.call(find.clusters, myArgs)$grp
        levels(temp) <- paste(i, levels(temp), sep=".")
        newFac[clust==i] <- as.character(temp)
    }

    res <- list(stat=NA, grp=factor(newFac), size=as.integer(table(newFac)))

    return(res)
}





## Compute within sum of squares from a matrix 'x' and a factor 'f'
.compute.wss <- function(x, f) {
    x.group.mean <- apply(x, 2, tapply, f, mean)
    sum((x - x.group.mean[as.character(f),])^2)
}

context("find.clusters tests")

test_that("find.clusters works with pre-defined clusters", {
  skip_on_cran()
  data(nancycats)
  
  # set connection
  f <- file()
  options(adegenet.testcon = f)
  
  # add data to connection
  twos <- paste(rep(2, nPop(nancycats)), collapse = "\n")
  write(twos, f)
  
  # test function
  expect_warning(capture.output(res <- find.clusters(nancycats, clust = pop(nancycats), n.pca = 100)))
  # We expect each group to receive two clusters
  expect_equal(length(levels(res$grp)), nPop(nancycats) * 2)
  # We expect all individuals accounted for
  expect_equal(length(res$grp), nInd(nancycats))
  
  # reset variable
  options(adegenet.testcon = stdin())
  # close connection
  close(f)
})

#' Compute scaled allele frequencies
#'
#' The generic function \code{scaleGen} is an analogue to the \code{scale}
#' function, but is designed with further arguments giving scaling options.\cr
#'
#' Methods are defined for \linkS4class{genind} and \linkS4class{genpop}
#' objects.  Both return data.frames of scaled allele frequencies.
#'
#'
#' @rdname scaleGen
#' @aliases scaleGen scaleGen-methods scaleGen,genind-method
#' scaleGen,genpop-method
#' @docType methods
#' @export
#' @param x a \linkS4class{genind} and \linkS4class{genpop} object
#' @param center a logical stating whether alleles frequencies should be
#' centred to mean zero (default to TRUE). Alternatively, a vector of numeric
#' values, one per allele, can be supplied: these values will be substracted
#' from the allele frequencies.
#' @param scale a logical stating whether alleles frequencies should be scaled
#' (default to TRUE). Alternatively, a vector of numeric values, one per
#' allele, can be supplied: these values will be substracted from the allele
#' frequencies.
#' @param truenames no longer used; kept for backward compatibility
#' @param NA.method a method to replace NA; asis: leave NAs as is; mean: replace by the mean allele frequencies; zero: replace by zero
#' @param ... further arguments passed to other methods.
#' @return A matrix of scaled allele frequencies with genotypes
#' (\linkS4class{genind}) or populations in (\linkS4class{genpop}) in rows and
#' alleles in columns.
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#' @keywords methods manip
#' @examples
#'
#' \dontrun{
#' ## load data
#' data(microbov)
#' obj <- genind2genpop(microbov)
#'
#' ## compare different scaling
#' X1 <- scaleGen(obj)
#' X2 <- scaleGen(obj,met="bin")
#'
#' ## compute PCAs
#' pcaObj <- dudi.pca(obj,scale=FALSE,scannf=FALSE) # pca with no scaling
#' pcaX1 <- dudi.pca(X1,scale=FALSE,scannf=FALSE,nf=100) # pca with usual scaling
#' pcaX2 <- dudi.pca(X2,scale=FALSE,scannf=FALSE,nf=100) # pca with scaling for binomial variance
#'
#' ## get the loadings of alleles for the two scalings
#' U1 <- pcaX1$c1
#' U2 <- pcaX2$c1
#'
#'
#' ## find an optimal plane to compare loadings
#' ## use a procustean rotation of loadings tables
#' pro1 <- procuste(U1,U2,nf=2)
#'
#' ## graphics
#' par(mfrow=c(2,2))
#' # eigenvalues
#' barplot(pcaObj$eig,main="Eigenvalues\n no scaling")
#' barplot(pcaX1$eig,main="Eigenvalues\n usual scaling")
#' barplot(pcaX2$eig,main="Eigenvalues\n 'binomial' scaling")
#' # differences between loadings of alleles
#' s.match(pro1$scor1,pro1$scor2,clab=0,sub="usual -> binom (procustean rotation)")
#'
#' }
#'
setGeneric("scaleGen", function(x,...){standardGeneric("scaleGen")})

#' @rdname scaleGen
#' @export
setMethod("scaleGen", "genind", function(x, center=TRUE, scale=TRUE,
                                         NA.method=c("asis","mean","zero"), truenames=TRUE){

    THRES <- 1e-10

    ## get table of frequencies
    out <- tab(x, NA.method=NA.method, freq=TRUE, quiet=TRUE)

    ## scale output
    out <- scale(out, center=center, scale=scale)

    ## issue a warning if some variances are null
    temp <- attr(out,"scaled:scale") < THRES
    if(any(temp)) {
        warning("Some scaling values are null.\n Corresponding alleles are removed.")
        out <- out[, !temp]
        attr(out,"scaled:center") <- attr(out,"scaled:center")[!temp]
        attr(out,"scaled:scale") <- attr(out,"scaled:scale")[!temp]
    }

    return(out)
})





#' @rdname scaleGen
#' @export
setMethod("scaleGen", "genpop", function(x, center=TRUE, scale=TRUE,
                                         NA.method=c("asis","mean","zero"), truenames=TRUE){

    THRES <- 1e-10

    ## get table of frequencies
    out <- tab(x, NA.method=NA.method, freq=TRUE, quiet=TRUE)

    ## scale output
    out <- scale(out, center=center, scale=scale)
    ## issue a warning if some variances are null
    temp <- attr(out,"scaled:scale") < THRES
    if(any(temp)) {
        warning("Some scaling values are null.\n Corresponding alleles are removed.")
        out <- out[, !temp]
        attr(out,"scaled:center") <- attr(out,"scaled:center")[!temp]
        attr(out,"scaled:scale") <- attr(out,"scaled:scale")[!temp]
    }

    return(out)
})

####################
# Function makefreq
####################

#' Compute allelic frequencies
#'
#' The function \code{makefreq} is a generic to compute allele frequencies.
#' These can be derived for \linkS4class{genind} or \linkS4class{genpop} objects.
#' In the case of \linkS4class{genind} objects, data are kept at the individual level, but standardised so that allele frequencies sum up to 1.
#'
#' There are 3 treatments for missing values: \cr - NA: kept as NA.\cr - 0:
#' missing values are considered as zero. Recommended for a PCA on
#' compositionnal data.\cr - "mean": missing values are given the mean
#' frequency of the corresponding allele. Recommended for a centred PCA.
#'
#' Note that this function is now a simple wrapper for the accessor \code{\link{tab}}.
#'
#' @param x a \linkS4class{genind} or \linkS4class{genpop} object.
#' @param quiet logical stating whether a conversion message must be printed
#' (TRUE,default) or not (FALSE).
#' @param missing treatment for missing values. Can be NA, 0 or "mean" (see
#' details)
#' @param truenames deprecated; there for backward compatibility
#' @param ... further arguments (curently unused)
#'
#' @return Returns a list with the following components: \item{tab}{matrix of
#' allelic frequencies (rows: populations; columns: alleles).}
#' \item{nobs}{number of observations (i.e. alleles) for each population x
#' locus combinaison.} \item{call}{the matched call}
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#' @seealso \code{\link{genpop}}
#' @keywords manip multivariate
#'
#' @examples
#'
#' \dontrun{
#' data(microbov)
#' obj1 <- microbov
#' obj2 <- genind2genpop(obj1)
#'
#' # perform a correspondance analysis on counts data
#' Xcount <- tab(obj2, NA.method="zero")
#' ca1 <- dudi.coa(Xcount,scannf=FALSE)
#' s.label(ca1$li,sub="Correspondance Analysis",csub=1.2)
#' add.scatter.eig(ca1$eig,nf=2,xax=1,yax=2,posi="topleft")
#'
#' # perform a principal component analysis on frequency data
#' Xfreq <- makefreq(obj2, missing="mean")
#' Xfreq <- tab(obj2, NA.method="mean") # equivalent to line above
#' pca1 <- dudi.pca(Xfreq,scale=FALSE,scannf=FALSE)
#' s.label(pca1$li,sub="Principal Component Analysis",csub=1.2)
#' add.scatter.eig(pca1$eig,nf=2,xax=1,yax=2,posi="top")
#' }
#'
#' @export
#' @docType methods
#' @rdname makefreq
#'
setGeneric("makefreq", function(x, ...) standardGeneric("makefreq"))

#' @export
#' @rdname makefreq
#' @aliases makefreq,genind-methods
#' @aliases makefreq.genind
setMethod ("makefreq", signature(x="genind"), function(x, quiet=FALSE, missing=NA, truenames=TRUE, ...){
  if(!quiet) cat("\n Finding allelic frequencies from a genpop object... \n")

  ## treat 'missing' argument for compatibility with 'tab
  ## NA -> "asis"
  ## 0 -> "zero"
  ## "mean" -> "mean"
  if(is.na(missing)) {
      NA.method <- "asis"
  } else if(is.numeric(missing) && missing==0){
      NA.method <- "zero"
  } else {
      NA.method <- "mean"
  }

  out <- tab(x, freq=TRUE, NA.method=NA.method)

  if(!quiet) cat("\n...done.\n\n")

  return(out)
}) #end makefreq for genind




## genpop method
#' @export
#' @rdname makefreq
#' @aliases makefreq,genpop-methods
#' @aliases makefreq.genpop
setMethod ("makefreq", signature(x="genpop"), function(x, quiet=FALSE, missing=NA, truenames=TRUE, ...){

  if(!quiet) cat("\n Finding allelic frequencies from a genpop object... \n")

   ## treat 'missing' argument for compatibility with 'tab
  ## NA -> "asis"
  ## 0 -> "zero"
  ## "mean" -> "mean"
  if(is.na(missing)) {
      NA.method <- "asis"
  } else if(is.numeric(missing) && missing==0){
      NA.method <- "zero"
  } else {
      NA.method <- "mean"
  }

  out <- tab(x, freq=TRUE, NA.method=NA.method)

  if(!quiet) cat("\n...done.\n\n")

  return(out)
}) #end makefreq for genpop

setMethod("$","genpop",function(x,name) {
  return(slot(x,name))
})


setMethod("$<-","genpop",function(x,name,value) {
  slot(x,name,check=TRUE) <- value
  return(x)
})


.drop_alleles <- function(x, toKeep){
  all.vec <- unlist(alleles(x), use.names = FALSE)[toKeep]
  loc.fac <- factor(locFac(x)[toKeep])

  x@all.names <- split(all.vec, loc.fac)
  x@loc.n.all  <- setNames(tabulate(loc.fac), levels(loc.fac))
  x@loc.fac   <- loc.fac
  return(x)
}


###############
# '[' operator
###############
## genind
setMethod("[", signature(x="genind", i="ANY", j="ANY", drop="ANY"), function(x, i, j, ..., pop=NULL, loc=NULL, treatOther=TRUE, quiet=TRUE, drop=FALSE) {

  if (missing(i)) i <- TRUE
  if (missing(j)) j <- TRUE

  ## HANDLE 'i' as character
  if(is.factor(i)) i <- as.character(i)
  if(is.character(i)){
      old.i <- i
      i <- match(i, indNames(x))
      if(any(is.na(i))){
          warning(paste("the following specified individuals do not exist:", paste0(old.i[is.na(i)], collapse = ", ")), call. = FALSE)
          i <- i[!is.na(i)]
          if(length(i)==0) {
              warning("no individual selected - ignoring", call. = FALSE)
              i <- TRUE
          }
      }
  }

  ## HANDLE 'POP'
  if(!is.null(pop) && !is.null(pop(x))){
    if(is.factor(pop)) pop <- as.character(pop)
    if(!is.character(pop)) pop <- popNames(x)[pop]
    temp <- !pop %in% pop(x)
    if (any(temp)) { # if wrong population specified
      warning(paste("the following specified populations do not exist:", paste0(pop[temp], collapse = ", ")), call. = FALSE)
      if (all(temp)){
        warning("no populations selected - ignoring", call. = FALSE)
        pop <- pop(x)
      }
    }
    i <- pop(x) %in% pop
  }

  ## handle population factor
  if(!is.null(pop(x))) {
    pop <- factor(pop(x)[i])
  } else {
    pop <- NULL
  }

  tab       <- tab(x)
  old.other <- other(x)
  hier      <- x@strata
  prevcall  <- match.call()

  if (x@type == "codom"){
    ## handle loc argument
    if(!is.null(loc)){
      if(is.factor(loc)) loc <- as.character(loc)
      if(!is.character(loc)) loc <- locNames(x)[loc]
      temp <- !loc %in% locFac(x)
      if (any(temp)) { # if wrong loci specified
        warning(paste("the following specified loci do not exist:", paste0(loc[temp], collapse = ", ")), call. = FALSE)
        if (all(temp)){
          warning("no loci selected - ignoring", call. = FALSE)
          loc <- x@loc.fac
        }
      }
      j <- x$loc.fac %in% loc
    } # end loc argument
    if (drop){
      tab    <- tab[i, , ..., drop = FALSE]
      allNb  <- colSums(tab, na.rm=TRUE) # allele absolute frequencies
      toKeep <- (allNb > 1e-10)
      j      <- j & toKeep
      tab    <- tab[, j, ..., drop=FALSE]
    } else {
      tab <- tab[i, j, ..., drop=FALSE]
    }
  } else { # PA case
    tab <- tab[i, j, ..., drop = FALSE]
  }


  ## handle 'other' slot
  nOther <- length(other(x))
  namesOther <- names(other(x))
  counter <- 0
  if(treatOther){
    f1 <- function(obj,n=nrow(tab(x))){
      counter <<- counter+1
      if(!is.null(dim(obj)) && nrow(obj)==n) { # if the element is a matrix-like obj
        obj <- obj[i,,drop=FALSE]
      } else if(length(obj) == n) { # if the element is not a matrix but has a length == n
        obj <- obj[i]
        if(is.factor(obj)) {obj <- factor(obj)}
      } else {if(!quiet) warning(paste("cannot treat the object",namesOther[counter]))}

      return(obj)
    } # end f1

    x@other <- lapply(other(x), f1) # treat all elements

  } else {
    other(x) <- old.other
  } # end treatOther

  x@tab    <- tab
  x@pop    <- pop
  x@call   <- prevcall
  x@type   <- x@type

  # Treat sample and strata
  x@ploidy    <- ploidy(x)[i]
  x@hierarchy <- x@hierarchy
  x@strata    <- hier[i, , drop = FALSE]

  if (x@type == "codom"){
    # Treat locus items
    x <- .drop_alleles(x, j)
  }
  return(x)
})





## genpop
setMethod("[", "genpop", function(x, i, j, ..., loc=NULL, treatOther=TRUE, drop=FALSE) {

  if (missing(i)) i <- TRUE
  if (missing(j)) j <- TRUE

  tab <- tab(x)
  old.other <- other(x)


  ## HANDLE 'i' as character
  if(is.factor(i)) i <- as.character(i)
  if(is.character(i)){
      old.i <- i
      i <- match(i, popNames(x))
      if(any(is.na(i))){
          warning(paste("the following specified populations do not exist:", paste0(old.i[is.na(i)], collapse = ", ")), call. = FALSE)
          i <- i[!is.na(i)]
          if(length(i)==0) {
              warning("no population selected - ignoring", call. = FALSE)
              i <- TRUE
          }
      }
  }

  ## handle loc argument
  if(!is.null(loc)){
    if(is.factor(loc)) loc <- as.character(loc)
    if(!is.character(loc)) loc <- locNames(x)[loc]
    temp <- !loc %in% locFac(x)
    if(any(temp)) { # si mauvais loci
      warning(paste("the following specified loci do not exist:", paste0(loc[temp], collapse = ", ")), call. = FALSE)
    }
    if (all(temp)){
      warning("no loci selected - ignoring", call. = FALSE)
      loc <- x@loc.fac
    }
    j <- x$loc.fac %in% loc
  } # end loc argument

  prevcall <- match.call()
  # tab <- tab[i, j, ...,drop=FALSE]

  # if(drop){
  #     allNb <- apply(tab, 2, sum, na.rm=TRUE) # allele absolute frequencies
  #     toKeep <- (allNb > 1e-10)
  #     tab <- tab[,toKeep, drop=FALSE]
  # }
  if (drop){
    tab    <- tab[i, , ..., drop = FALSE]
    allNb  <- colSums(tab, na.rm=TRUE) # allele absolute frequencies
    toKeep <- (allNb > 1e-10)
    j      <- j & toKeep
    tab    <- tab[, j, ..., drop=FALSE]
  } else {
    tab <- tab[i, j, ..., drop=FALSE]
  }

  # res <- genpop(tab,prevcall=prevcall,ploidy=x@ploidy)

  ## handle 'other' slot
  nOther <- length(other(x))
  namesOther <- names(other(x))
  counter <- 0
  if(treatOther){
    f1 <- function(obj,n=nrow(tab(x))){
      counter <<- counter+1
      if(!is.null(dim(obj)) && nrow(obj)==n) { # if the element is a matrix-like obj
        obj <- obj[i,,drop=FALSE]
      } else if(length(obj) == n) { # if the element is not a matrix but has a length == n
        obj <- obj[i]
        if(is.factor(obj)) {obj <- factor(obj)}
      } else {warning(paste("cannot treat the object",namesOther[counter]), call. = FALSE)}

      return(obj)
    } # end f1

    x@other <- lapply(other(x), f1) # treat all elements

  } else {
    other(x) <- old.other
  } # end treatOther

  x@tab    <- tab
  x@call   <- prevcall
  x@type   <- x@type

  # Treat populations
  x@ploidy    <- ploidy(x)

  # Treat locus items
  x <- .drop_alleles(x, j)

  return(x)
})


##########################
## Method show for genind
##########################
setMethod ("show", "genind", function(object){
  ## GET USEFUL VARIABLES
  x <- object
  indTxt <- ifelse(nInd(x)>1, "individuals;", "individual;")
  locTxt <- ifelse(nLoc(x)>1, "loci;", "locus;")
  allTxt <- ifelse(ncol(tab((x)))>1, "alleles;", "allele;")

  ## HEADER
  cat("/// GENIND OBJECT /////////")

  cat("\n\n //", format(nInd(x), big.mark=","), indTxt,
      format(nLoc(x), big.mark=","), locTxt,
      format(ncol(tab(x)), big.mark=","), allTxt,
      "size:", format(object.size(x), units="auto"))

  ## BASIC CONTENT
  cat("\n\n // Basic content")
  p <- ncol(tab(x))
  len <- 7

  cat("\n   @tab: ", nrow(tab(x)), "x", ncol(tab(x)), "matrix of allele counts" )

  if(!is.null(nAll(x))){
    alleletxt <- paste("(range: ", paste(range(nAll(x)), collapse="-"), ")", sep="")
    cat("\n   @loc.n.all: number of alleles per locus", alleletxt)
  }

  if(!is.null(locFac(x))){
    cat("\n   @loc.fac: locus factor for the", ncol(tab(x)), "columns of @tab")
  }
  if(!is.null(alleles(x))){
    cat("\n   @all.names: list of allele names for each locus")
  }

  ploidytxt <- paste("(range: ", paste(range(ploidy(x)), collapse="-"), ")", sep="")
  cat("\n   @ploidy: ploidy of each individual ", ploidytxt)
  cat("\n   @type: ",x@type)
  cat("\n   @call: ")
  print(x@call)

  ## OPTIONAL CONTENT
  cat("\n // Optional content")
  optional <- FALSE
  if(!is.null(pop(x))){
    optional <- TRUE
    poptxt <- paste("(group size range: ", paste(range(table(pop(x))), collapse="-"), ")", sep="")
    cat("\n   @pop:", paste("population of each individual", poptxt))
  }

  if (!is.null(x@strata)){
    optional <- TRUE
    cat("\n   @strata: ")
    levs <- names(x@strata)
    if (length(levs) > 6){
      levs <- paste(paste(head(levs), collapse = ", "), "...", sep = ", ")
    } else {
      levs <- paste(levs, collapse = ", ")
    }
    cat("a data frame with", length(x@strata), "columns (", levs, ")")
  }

  if (!is.null(x@hierarchy)){
    optional <- TRUE
    cat("\n   @hierarchy:", paste(x@hierarchy, collapse = ""))
  }

  if(!is.null(other(x))){
    optional <- TRUE
    cat("\n   @other: ")
    cat("a list containing: ")
    cat(ifelse(is.null(names(other(x))), "elements without names", paste(names(other(x)), collapse= "  ")), "\n")
  }

  if(!optional) cat("\n   - empty -")

  cat("\n")
}
) # end show method for genind




##########################
## Method show for genpop
##########################
setMethod ("show", "genpop", function(object){
  ## GET USEFUL VARIABLES
  x <- object
  popTxt <- ifelse(nPop(x)>1, "populations;", "population;")
  locTxt <- ifelse(nLoc(x)>1, "loci;", "locus;")
  allTxt <- ifelse(ncol(tab((x)))>1, "alleles;", "allele;")

  ## HEADER
  cat("/// GENPOP OBJECT /////////")

  cat("\n\n //", format(nPop(x), big.mark=","), popTxt,
      format(nLoc(x), big.mark=","), locTxt,
      format(ncol(tab(x)), big.mark=","), allTxt,
      "size:", format(object.size(x), units="auto"))

  ## BASIC CONTENT
  cat("\n\n // Basic content")
  p <- ncol(tab(x))
  len <- 7

  cat("\n   @tab: ", nrow(tab(x)), "x", ncol(tab(x)), "matrix of allele counts" )

  if(!is.null(nAll(x))){
    alleletxt <- paste("(range: ", paste(range(nAll(x)), collapse="-"), ")", sep="")
    cat("\n   @loc.n.all: number of alleles per locus", alleletxt)
  }

  if(!is.null(locFac(x))){
    cat("\n   @loc.fac: locus factor for the", ncol(tab(x)), "columns of @tab")
  }
  if(!is.null(alleles(x))){
    cat("\n   @all.names: list of allele names for each locus")
  }

  ploidytxt <- paste("(range: ", paste(range(ploidy(x)), collapse="-"), ")", sep="")
  cat("\n   @ploidy: ploidy of each individual ", ploidytxt)
  cat("\n   @type: ",x@type)
  cat("\n   @call: ")
  print(x@call)

  ## OPTIONAL CONTENT
  cat("\n // Optional content")
  optional <- FALSE
  if(!is.null(other(x))){
    optional <- TRUE
    cat("\n   @other: ")
    cat("a list containing: ")
    cat(ifelse(is.null(names(other(x))), "elements without names", paste(names(other(x)), collapse= "  ")), "\n")
  }

  if(!optional) cat("\n   - empty -")

  cat("\n")

}
) # end show method for genpop





############################
# Method summary for genind
############################
if(!isGeneric("summary")){
  setGeneric("summary", function(object, ...) standardGeneric("summary"))
}
setMethod ("summary", signature(object="genind"), function(object, verbose = TRUE, ...){
  x <- object
  if(!is.genind(x)) stop("Provided object is not a valid genind.")


  if(is.null(pop(x))){
    pop(x) <- rep("P1", nInd(x))
  }

  ## BUILD THE OUTPUT ##
  ## type-independent stuff
  res <- list()

  res$n <- nrow(tab(x))

  res$n.by.pop <- as.numeric(table(pop(x)))
  names(res$n.by.pop) <- popNames(x)

  ## PA case ##
  if(x@type=="PA"){
    ## % of missing data
    res$NA.perc <- 100*sum(is.na(tab(x)))/prod(dim(tab(x)))

    return(invisible(res))
  }


  ## codom case ##
  res$loc.n.all <- nAll(x)

  temp <- tab(genind2genpop(x,quiet=TRUE))

  res$pop.n.all <- apply(temp,1,function(r) sum(r!=0,na.rm=TRUE))

  res$NA.perc <- 100*(1-mean(propTyped(x,by="both")))

  ## handle heterozygosity
  if(any(ploidy(x) > 1)){
    ## auxiliary function to compute observed heterozygosity
    temp <- lapply(seploc(x),tab, freq=TRUE)
    f1 <- function(tab){
      H <- apply(tab, 1, function(vec) any(vec > 0 & vec < 1))
      H <- mean(H,na.rm=TRUE)
      return(H)
    }

    res$Hobs <- unlist(lapply(temp,f1))

    ## auxiliary function to compute expected heterozygosity
    ## freq is a vector of frequencies
    f2 <- function(freq){
      H <- 1-sum(freq*freq,na.rm=TRUE)
      return(H)
    }

    temp <- genind2genpop(x,pop=rep(1,nInd(x)),quiet=TRUE)
    temp <- tab(temp, freq=TRUE, quiet=TRUE)
    res$Hexp <-tapply(temp^2, locFac(x), function(e) 1-sum(e, na.rm=TRUE))
  } else { # no possible heterozygosity for haploid genotypes
    res$Hobs <- 0
    res$Xexp <- 0
  }

  ## add class and return
  class(res) <- "genindSummary"
  return(res)
})  # end summary.genind





############################
# Method summary for genpop
############################
setMethod ("summary", signature(object="genpop"), function(object, verbose = TRUE, ...){
  x <- object
  if(!inherits(x,"genpop")) stop("To be used with a genpop object")

  ## BUILD THE OUTPUT ##
  ## type-independent stuff
  res <- list()

  res$n.pop <- nrow(tab(x))

  ## PA case ##
  if(x@type=="PA"){
    ## % of missing data
    res$NA.perc <- 100*sum(is.na(tab(x)))/prod(dim(tab(x)))

    ## add class and return
    class(res) <- "genpopSummary"
    return(res)
  }


  ## codom case ##
  res$loc.n.all <- nAll(x)

  res$pop.n.all <- apply(tab(x),1,function(r) sum(r>0,na.rm=TRUE))

  ##  res$NA.perc <- 100*sum(is.na(x@tab))/prod(dim(x@tab)) <- old version
  mean.w <- function(x,w=rep(1/length(x),length(x))){
    x <- x[!is.na(x)]
    w <- w[!is.na(x)]
    w <- w/sum(w)
    return(sum(x*w))
  }

  w <- apply(tab(x),1,sum,na.rm=TRUE) # weights for populations
  res$NA.perc <- 100*(1-mean.w(propTyped(x), w=w))

  ## add class and return
  class(res) <- "genpopSummary"
  return(res)
}
)# end summary.genpop




#######################
## print for summaries
#######################
print.genindSummary <- function(x, ...){
    if(!is.null(x$n)) cat("\n// Number of individuals:", x$n)
    if(!is.null(x$n.by.pop)) cat("\n// Group sizes:", x$n.by.pop)
    if(!is.null(x$loc.n.all)) cat("\n// Number of alleles per locus:", x$loc.n.all)
    if(!is.null(x$pop.n.all)) cat("\n// Number of alleles per group:", x$pop.n.all)
    if(!is.null(x$NA.perc)) cat("\n// Percentage of missing data:", round(x$NA.perc,2), "%")
    if(!is.null(x$Hobs)) cat("\n// Observed heterozygosity:", round(x$Hobs,2))
    if(!is.null(x$Hexp)) cat("\n// Expected heterozygosity:", round(x$Hexp,2))
    cat("\n")
} # end print.genindSummary


print.genpopSummary <- function(x, ...){
    if(!is.null(x$n.pop)) cat("\n// Number of populations:", x$n.pop)
    if(!is.null(x$loc.n.all)) cat("\n// Number of alleles per locus:", x$loc.n.all)
    if(!is.null(x$pop.n.all)) cat("\n// Number of alleles per group:", x$pop.n.all)
    if(!is.null(x$NA.perc)) cat("\n// Percentage of missing data:", round(x$NA.perc,2), "%")
    cat("\n")
} # end print.genpopSummary



###############
# Methods "is"
###############
is.genind <- function(x){
  res <- ( is(x, "genind") & validObject(x))
  return(res)
}

is.genpop <- function(x){
  res <- ( is(x, "genpop") & validObject(x))
  return(res)
}



.hasUniquePloidy <- function(x){
  return(length(unique(ploidy(x)))==1)
}

##############
### snpzip ###
##############


snpzip <- function(snps, y, plot=TRUE, xval.plot=FALSE, loading.plot=FALSE,
                   method=c("complete","single","average","centroid",
                            "mcquitty","median","ward"), ...) {
  
  ## dapc input prompts only SNP selection function
  if(class(y)=="dapc"){ 
    
    dapc1 <- y
    phen <- 0
    
    
    if(xval.plot==TRUE){
      warning("cross-validation not performed when x is a dapc object; xval.plot will not be shown")
      xval.plot=FALSE
    }
  }
  
  ## snps, phen input prompts cross-validation, DAPC, and SNP selection functions
  else{
    
    if(missing(y)){
      stop("phen argument needed")
    }
    phen <- y
    
  }
  
  
  if(class(y)!="dapc"){
    
    ################################################
    ######## Stratified Cross-Validation ###########
    ################################################
    
    xvalDapc <- function(x, grp, n.pca.max = 200, n.da = NULL, 
                         training.set = 0.9, result = "groupMean", 
                         center = TRUE, scale = FALSE, n.pca = NULL, n.rep = 30, ...){
      
      ## CHECKS ##
      grp <- factor(grp)
      n.pca <- n.pca[n.pca>0]
      n.da <- length(levels(grp))-1
      if(missing(training.set)){
        training.set <- 0.9}
      else{
        training.set <- training.set}
      if(missing(n.rep)){
        n.rep<-30}
      else{
        n.rep<-n.rep}
      
      
      ## GET TRAINING SET SIZE ##
      N <- nrow(x)
      groups <- levels(grp)
      if(all(lapply(groups, function(e) sum(as.vector(unclass(grp==e))))>=10)==TRUE){
        N.training <- round(N*training.set)}
      else{
        groups1 <- (levels(grp))[(as.vector(which.min((lapply(groups, 
                                                              function(e) sum(as.vector(unclass(grp==e))))))))]
        popmin <- length(which(grp%in%groups1))
        training.set2 <- ((popmin - 1)/popmin)
        N.training <- round(N*training.set2)}
      
      
      ## GET FULL PCA ##
      if(missing(n.pca.max)) n.pca.max <- min(dim(x))
      pcaX <- dudi.pca(x, nf=n.pca.max, scannf=FALSE, center=center, scale=scale)
      n.pca.max <- min(n.pca.max, pcaX$rank, N.training-1)
      
      ## DETERMINE N.PCA IF NEEDED ##
      if(n.pca.max < 10){
        runs <- n.pca.max}
      else{
        runs<- 10}
      
      if(is.null(n.pca)){
        n.pca <- round(pretty(1:n.pca.max, runs))
      }
      n.pca <- n.pca[n.pca>0 & n.pca<(N.training-1)]
      
      ## FUNCTION GETTING THE % OF ACCURATE PREDICTION FOR ONE NUMBER OF PCA PCs ##
      ## n.pca is a number of retained PCA PCs
      get.prop.pred <- function(n.pca){
        f1 <- function(){
          if(all(lapply(groups, function(e) sum(as.vector(unclass(grp==e))))>=10)==TRUE){
            toKeep <- unlist(lapply(groups, function(e) sample(which(grp==e), 
                                                               size=(round(training.set*sum(as.vector(unclass(grp==e))))))))}
          else{
            toKeep <- unlist(lapply(groups, function(e) sample(which(grp==e), 
                                                               size=(round(training.set2*sum(as.vector(unclass(grp==e))))))))}
          temp.pca <- pcaX
          temp.pca$li <- temp.pca$li[toKeep,,drop=FALSE]
          temp.dapc <- suppressWarnings(dapc(x[toKeep,,drop=FALSE], grp[toKeep], 
                                             n.pca=n.pca, n.da=n.da, dudi=temp.pca))
          temp.pred <- predict.dapc(temp.dapc, newdata=x[-toKeep,,drop=FALSE])
          if(result=="overall"){
            out <- mean(temp.pred$assign==grp[-toKeep])
          }
          if(result=="groupMean"){
            out <- mean(tapply(temp.pred$assign==grp[-toKeep], grp[-toKeep], mean), na.rm=TRUE)
          }
          return(out)
        }
        return(replicate(n.rep, f1()))
      }
      
      
      ## GET %SUCCESSFUL OF ACCURATE PREDICTION FOR ALL VALUES ##
      res.all <- unlist(lapply(n.pca, get.prop.pred))
      xval <- data.frame(n.pca=rep(n.pca, each=n.rep), success=res.all)    
      
      
      n.pcaF <- as.factor(xval$n.pca)
      successV <- as.vector(xval$success)
      pca.success <- tapply(successV, n.pcaF, mean)
      n.opt <- which.max(tapply(successV, n.pcaF, mean))
      
      
      ################################################
      #####  MSE Calculation and n.pca Selection #####
      ################################################
      temp <- seq(from=1, to=length(xval$n.pca), by=n.rep)
      orary <-c(temp+(n.rep-1))
      index <-c(1:length(temp))
      lins <-sapply(index, function(e) seq(from=temp[e], to=orary[e]))
      lin <-c(1:ncol(lins))
      col <-successV
      cait<-sapply(lin, function(e) ((col[lins[,e]])-1)^2)
      FTW <-sapply(lin, function(e) sum(cait[,e])/n.rep)
      RMSE <- sqrt(FTW)
      names(RMSE) <- xval$n.pca[temp]
      best.n.pca <- names(which.min(RMSE))
      
      
      
      ################################################
      #################  DAPC  #######################
      ################################################
      n.pcaF <- as.factor(xval$n.pca)
      successV <- as.vector(xval$success)
      pca.success <- tapply(successV, n.pcaF, mean)
      n.opt <- which.max(tapply(successV, n.pcaF, mean))
      n.pca <- as.integer(best.n.pca)
      n.da <- nlevels(grp)-1
      dapc1 <- dapc(x, grp, n.pca=n.pca, n.da=n.da)
      
      answerme <- list(n.da, n.pca, dapc1, xval, successV, RMSE, best.n.pca, pca.success, n.opt)
      return(answerme)
    }
    
    
    x <- snps
    grp <- phen
    
    XVAL <- xvalDapc(x, grp, n.da=NULL, training.set=0.9, result="groupMean",
                     center=TRUE, scale=FALSE, n.pca=NULL, ...)
    n.da <- XVAL[[1]]
    n.pca <- XVAL[[2]]
    dapc1 <- XVAL[[3]]
    xval <- XVAL[[4]]
    successV <- XVAL[[5]]
    RMSE <- XVAL[[6]]
    best.n.pca <- XVAL[[7]]
    pca.success <- XVAL[[8]]
    n.opt <- XVAL[[9]]
    
    
    ################################################
    ######## Show Cross-Validation Results #########
    ################################################
    snps <- x
    phen <- grp
    
    if(xval.plot==TRUE){
      par(ask=TRUE)
      random <- replicate(300, mean(tapply(sample(phen)==phen, phen, mean)))
      q.phen <- quantile(random, c(0.025,0.5,0.975))
      smoothScatter(xval$n.pca, successV, nrpoints=Inf, pch=20, col=transp("black"),
                    ylim=c(0,1), xlab="Number of PCA axes retained",
                    ylab="Proportion of successful outcome prediction", 
                    main="DAPC Cross-Validation")
      abline(h=q.phen,lty=c(2,1,2))
      
      xvalResults <- list(xval, q.phen, pca.success, (names(n.opt)), RMSE, best.n.pca)
      names(xvalResults)[[1]] <- "Cross-Validation Results"
      names(xvalResults)[[2]] <- "Median and Confidence Interval for Random Chance"
      names(xvalResults)[[3]] <- "Mean Successful Assignment by Number of PCs of PCA"
      names(xvalResults)[[4]] <- "Number of PCs Achieving Highest Mean Success"
      names(xvalResults)[[5]] <- "Root Mean Squared Error by Number of PCs of PCA"
      names(xvalResults)[[6]] <- "Number of PCs Achieving Lowest MSE"
      print(xvalResults)
    }
    
  } # end of snps, phen section
  
  
  ################################################
  #############  Plot DAPC Results   #############
  ################################################
  
  if(plot==TRUE){
    myCol <- colorRampPalette(c("blue", "gold", "red"))
    scatter(dapc1, bg="white", scree.da=FALSE, scree.pca=TRUE, posi.pca="topright",
            col=myCol((dapc1$n.da)+1), legend=TRUE, posi.leg="topleft")
    title("DAPC")}
  
  ################################################
  ###### Select Cluster of Structural SNPS #######
  ################################################
  
  if(missing(method)){
    method <- "ward"
  }
  else{ 
    method <- method}
  
  selector <- function(dapc1, dimension){
    
    z <- dapc1$var.contr[,dimension]
    xTotal <- dapc1$var.contr[,dimension]
    toto <- which(xTotal%in%tail(sort(xTotal), 2000))
    z <- sapply(toto, function(e) xTotal[e])
    
    D <- dist(z)
    clust <- hclust(D,method)
    pop <- factor(cutree(clust,k=2,h=NULL))
    m <- which.max(tapply(z,pop,mean))
    maximus <- which(pop==m)
    maximus <- as.vector(unlist(sapply(maximus, function(e) toto[e])))
    popvect <- as.vector(unclass(pop))
    n.snp.selected <- sum(popvect==m)
    sel.snps <- snps[,maximus]
    
    selection <- c((ncol(snps)-ncol(snps[,-maximus])), ncol(snps[,-maximus]))
    
    resultat <- list(selection, maximus, dimnames(sel.snps)[[2]], dapc1$var.contr[maximus, dimension])
    
    names(resultat)[[1]] <- "Number of selected vs. unselected alleles"
    names(resultat)[[2]] <- "List of selected alleles"
    names(resultat)[[3]] <- "Names of selected alleles"
    names(resultat)[[4]] <- "Contributions of selected alleles to discriminant axis"
    gc()
    return(resultat)
  }
  
  if(dapc1$n.da==1){
    features <- selector(dapc1, dimension=1)      
  }
  else{
    dimensions <- c(1:dapc1$n.da)
    features <- lapply(dimensions, function(e)
      selector(dapc1, dimension=e))     
  }
  
  ################################################
  ####  Calculate success in discrimination   ####
  ################################################
  
  # overall
  grp <- dapc1$grp
  ass <- dapc1$assign
  dapc.success.overall <- length(which(ass==grp)) / length(grp)
  
  # by group        
  
  # give grp and ass numbered factor levels
  GRP <- factor(grp, levels=levels(grp), labels=c(0:(nlevels(grp)-1)))
  ASS <- factor(ass, levels=levels(ass), labels=c(0:(nlevels(ass)-1)))
  
  # make each level of those factors into one element of a list
  index <- c(0:(nlevels(grp) - 1))
  ASSIGN <- sapply(index, function(e) which(ASS==e))
  GROUP <- sapply(index, function(e) which(GRP==e))
  
  index2 <- c(1:(nlevels(grp)))
  dapc.success.byGroup <- sum(sapply(index2, function(e) 
    (length(which(ASSIGN[[e]]%in%GROUP[[e]]))) / 
      length(GROUP[[e]]))) / length(index2)
  
  dapc.success <- c(dapc.success.overall, dapc.success.byGroup)
  
  ################################################
  #### Loading Plot Delineating SNP Clusters  ####
  ################################################
  
  if(loading.plot==TRUE){
    if(dapc1$n.da==1){
      par(ask=TRUE)
      maximus <- features[[2]]
      decimus <- abs(dapc1$var.contr[maximus][(which.min(dapc1$var.contr[maximus]))])-0.000001
      meridius <- loadingplot(dapc1$var.contr[,1], threshold=c(decimus))
    }
    else{
      par(ask=TRUE)
      # specify that you want to run the following lines for all DA (ie. from DA=1 to DA=(k-1))
      DA <- c(1:dapc1$n.da)
      # generate separate loading plots for each DA
      for(i in DA){
        title <- paste("Loading Plot for DA", i, sep=" ")
        maximus <- features[[i]][[2]] 
        decimus <- abs(dapc1$var.contr[maximus,i][(which.min(dapc1$var.contr[maximus,i]))])-0.000001
        meridius <- loadingplot(dapc1$var.contr[, i], threshold=decimus, main=title)
      }
    }
  }
  
  ################################################
  ########## Return snpzip Results  ##############
  ################################################
  
  if(class(y)=="dapc"){
    
    answer <- list(dapc1$n.pca, features)
    names(answer)[[1]] <- "Number of PCs of PCA retained"
    names(answer)[[2]] <- "FS"
    return(answer) 
  }
  
  else{ 
    answer <- list(best.n.pca, features, dapc.success, dapc1)
    names(answer)[[1]] <- "Number of PCs of PCA retained"
    names(answer)[[2]] <- "FS"
    names(answer)[[3]] <- "Discrimination success overall & by group"
    names(answer)[[4]] <- "DAPC"
    return(answer)
  }
  
  par(ask=FALSE)
} # end snpzip

context("xvalDapc test")

xvalnames <- c("Cross-Validation Results", 
  "Median and Confidence Interval for Random Chance", 
  "Mean Successful Assignment by Number of PCs of PCA", 
  "Number of PCs Achieving Highest Mean Success", 
  "Root Mean Squared Error by Number of PCs of PCA", 
  "Number of PCs Achieving Lowest MSE", 
  "DAPC")

data(sim2pop)


test_that("xvalDapc returns expected results", {
  skip_on_cran()
  xval <- xvalDapc(sim2pop@tab, pop(sim2pop), n.pca.max=100, n.rep=3, xval.plot = FALSE)
  cvr  <- xval[["Cross-Validation Results"]]
  msa  <- xval[["Mean Successful Assignment by Number of PCs of PCA"]]
  expect_that(xval, is_a("list"))
  expect_equivalent(names(xval), xvalnames)
  expect_that(xval$DAPC, is_a("dapc"))
  expect_that(nrow(cvr), equals(3 * length(msa)))
})

test_that("xvalDapc throws a warning with populations of 1 sample", {
  skip_on_cran()
  dat_pop <- as.character(pop(sim2pop))
  dat_pop[1] <- "Pop C"
  pop(sim2pop) <- dat_pop
  expect_warning(xvalDapc(sim2pop@tab, pop(sim2pop), n.pca.max=100, n.rep=3, xval.plot = FALSE))
})

xvalDapc <- function (x, ...) UseMethod("xvalDapc")

##############
## xvalDapc ##
##############

# Return randomly sampled indices from a group.
# @param e group name
# @param vector of group assignments per sample
# @param training.set fraction of samples to be kept for validation
.group_sampler <- function(e, grp, training.set){
  group_e   <- grp == e 
  N_group_e <- sum(group_e)
  if (N_group_e < 2){
    # If the size of the group is less than two, then leave the whole thing in 
    # the training set
    return(which(group_e))
  } else {
    samp_group <- which(group_e)
    samp_size <- round(training.set * N_group_e)
    return(sample(samp_group, size = samp_size))    
  }
}


# Function to subsample the data. This is to be used as the ran.gen function
# in the boot function. DATA is a data frame or matrix containing the samples,
# GRP is the group identities of the samples, PCA is the result of dudi.pca
# on the full data set, KEEP is the subset of samples based on the training
# set (mle). Note that this function only has two inputs, dat, and mle.
.boot_group_sampler <- function(dat = list(DATA = NULL, GRP = NULL, PCA = NULL, 
                                           KEEP = NULL), 
                                mle = NULL){
  to_keep    <- unlist(lapply(levels(dat$GRP), .group_sampler, dat$GRP, mle))
  dat$PCA$li <- dat$PCA$li[to_keep, , drop = FALSE]
  dat$KEEP   <- to_keep
  return(dat)
}


# Function to pass to the "statistic" parameter of boot. This will subset the
# data, calculate the DAPC, give the predictions and return the results.
.boot_dapc_pred <- function(x, n.pca = n.pca, n.da = n.da, result = "overall"){
  if (length(x$KEEP) == nrow(x$DATA)){
    out <- 1
  } else {
    new_dat   <- x$DATA[-x$KEEP, ,drop = FALSE]
    train_dat <- x$DATA[x$KEEP, ,drop = FALSE]
    
    new_grp   <- x$GRP[-x$KEEP]
    train_grp <- x$GRP[x$KEEP]      
    dapclist <- list(train_dat,
                     train_grp,
                     n.pca = n.pca,
                     n.da = n.da,
                     dudi = x$PCA)
    temp.dapc <- suppressWarnings(do.call("dapc", dapclist))
    temp.dapc <- suppressWarnings(dapc(train_dat, train_grp, dudi = x$PCA, 
                                       n.pca = n.pca, n.da = n.da))
    temp.pred <- predict.dapc(temp.dapc, newdata = new_dat)
    if (identical(result, "overall")){
      out <- mean(temp.pred$assign == new_grp)
    }
    if (identical(result, "groupMean")){
      out <- mean(tapply(temp.pred$assign == new_grp, new_grp, mean), na.rm = TRUE)
    }
  }
  return(out)
}


# Function that will actually do the bootstrapping. It will return a numeric
# vector with the successes ratios. Note that the ellipses are used to pass
# parameters to boot. When implemented in the xvalDapc function, this will allow
# the user to implement this in parallel. 
# @param n.pca number of pcs
# @param x data frame/matrix with samples in rows
# @param n.da number of das
# @param groups the names of each of the populations
# @param grp factor of group assignments per sample
# @param training.set fraction of samples used for training
# @param training.set2 NULL or largest possible fraction that can be obtained
# @param pcaX principal componenets
# @param result user's choice of result type
# @param reps the number of replicates per number of retained PCs
# @param ... methods to be passed on to boot such as parallel and ncores
####################
## .get.prop.pred ##
####################
.get.prop.pred <- function(n.pca, x, n.da, groups, grp, training.set, 
                           pcaX, result = "overall", reps = 100,
                           ...){
  
  bootlist      <- list(DATA = x, GRP = grp, PCA = pcaX, KEEP = 1:nrow(x))

  out <- boot::boot(bootlist, .boot_dapc_pred, sim = "parametric", R = reps,
                      ran.gen = .boot_group_sampler, mle = training.set, 
                      n.pca = n.pca, n.da = n.da, result = result, ...)$t      
  #   }
  return(as.vector(out))
} # end .get.prop.pred




##############
## xvalDapc ##
##############
xvalDapc.default <- function(x, grp, n.pca.max = 300, n.da = NULL, training.set = 0.9, 
                     result = c("groupMean", "overall"), center = TRUE, scale = FALSE, 
                     n.pca = NULL, n.rep = 30, xval.plot = TRUE, ...){
  
  ## CHECKS ##
  grp <- factor(grp)
  n.pca <- n.pca[n.pca > 0]
  if(!is.null(n.da)){
    n.da <- n.da
  }else{
    n.da <- length(levels(grp))-1}
  #   if(missing(n.da)){
  #   n.da <- length(levels(grp))-1}
  #   if(is.null(n.da)){
  #     n.da <- length(levels(grp))-1} # want to fix this to make interactive n.da selection an option! 
  #   else{
  #     n.da <- n.da}
  if(missing(training.set)){
    training.set <- 0.9
  }else{
    training.set <- training.set}
  if(missing(n.rep)){
    n.rep <- 30
  }else{
    n.rep <- n.rep}
  
  if(missing(result)){
    result <- "groupMean"
  }else{ 
    if(length(result) > 1) result <- result[1]
    result <- result}
  
  
  ## GET TRAINING SET SIZE ##
  N <- nrow(x)
  groups <- levels(grp)
  ## identify the sizes of groups
  group.n <- as.vector(table(grp))
  ## identify the smallest group size
  popmin <- min(group.n)
  
  ## check if any groups are of length 1:
  if(popmin == 1){
    singles <- which(group.n == 1)
    counter <- length(singles)
    
    ## get msg to print
    if(counter == 1){
      msg <- "1 group has only 1 member so it cannot be represented in both training and validation sets."
    }else{
      msg <- paste(counter, "groups have only 1 member: these groups cannot be represented in both training and validation sets.") 
    }
    warning(msg)
    
    ## exclude groups of length 1
    popmin <- min(group.n[-which(group.n==1)])        
  } # end if popmin ==1
  
  ## get training.set2 
  ## (ie. the max proportion we can use as training.set | smallest group)
  ## to be used as argument to .get.prop.pred
  training.set2 <- (popmin - 1)/popmin
  ## update training.set if needs reduction to accommodate small groups
  if(training.set2 < training.set) training.set <- training.set2
  
  ## get N.training | training.set 
  N.training <- round(N*training.set)
   
  
  
  ## GET FULL PCA ##
  if(missing(n.pca.max)) n.pca.max <- min(dim(x))

  pcaX      <- dudi.pca(x, nf=n.pca.max, scannf=FALSE, center=center, scale=scale)
  n.pca.max <- min(n.pca.max, pcaX$rank, N.training-1) # re-defines n.pca.max (so user's input may not be the value used...)
    
  ## DETERMINE N.PCA IF NEEDED ##
  if(n.pca.max < 10){
    runs <- n.pca.max
  }else{
    runs <- 10}
  
  if(is.null(n.pca)){
    n.pca <- round(pretty(1:n.pca.max, runs))
  }

  n.pca <- n.pca[n.pca>0 & n.pca<(N.training-1) & n.pca<n.pca.max]
  
  
  ## GET %SUCCESSFUL OF ACCURATE PREDICTION FOR ALL VALUES ##
  res.all <- unlist(lapply(n.pca, .get.prop.pred, x, n.da, groups, grp,
                           training.set, pcaX, result, 
                           n.rep, ...))
  xval <- data.frame(n.pca=rep(n.pca, each=n.rep), success=res.all)    
  
  
  n.pcaF <- as.factor(xval$n.pca)
  successV <- as.vector(xval$success)
  pca.success <- tapply(successV, n.pcaF, mean)
  n.opt <- which.max(tapply(successV, n.pcaF, mean))
  
  
  ###### MSE-BASED OPTIMAL n.pca SELECTION:
  temp <- seq(from=1, to=length(xval$n.pca), by=n.rep)
  orary <-c(temp+(n.rep-1))
  index <-c(1:length(temp))
  lins <-sapply(index, function(e) seq(from=temp[e], to=orary[e]))
  lin <-c(1:ncol(lins))
  col <-successV
  cait<-sapply(lin, function(e) ((col[lins[,e]])-1)^2)
  FTW <-sapply(lin, function(e) sum(cait[,e])/n.rep)
  RMSE <- sqrt(FTW)
  names(RMSE) <- xval$n.pca[temp]
  ## if more than one n.pc give highest success, choose the largest one
  best.n.pca <- names(which(RMSE == min(RMSE)))
  if(length(best.n.pca) > 1) best.n.pca <- best.n.pca[length(best.n.pca)]
  
  # DAPC
  n.pca <- as.integer(best.n.pca)
  n.da <- nlevels(grp)-1
  dapc1 <- suppressWarnings(dapc(x, grp, n.pca=n.pca, n.da=n.da))
  
  # PLOT CROSS-VALIDATION RESULTS
  snps <- x
  phen <- grp
  random <- replicate(300, mean(tapply(sample(phen)==phen, phen, mean)))
  q.phen <- quantile(random, c(0.025,0.5,0.975))
  
  if(xval.plot==TRUE){
    smoothScatter(xval$n.pca, successV, nrpoints=Inf, pch=20, col=transp("black"),
                  ylim=c(0,1), xlab="Number of PCA axes retained",
                  ylab="Proportion of successful outcome prediction", 
                  main="DAPC Cross-Validation")
    abline(h=q.phen, lty=c(2,1,2))
  }
  
  
  # RESULTS
  xvalResults <- list(xval, q.phen, pca.success, (names(n.opt)), RMSE, best.n.pca, dapc1)
  names(xvalResults)[[1]] <- "Cross-Validation Results"
  names(xvalResults)[[2]] <- "Median and Confidence Interval for Random Chance"
  names(xvalResults)[[3]] <- "Mean Successful Assignment by Number of PCs of PCA"
  names(xvalResults)[[4]] <- "Number of PCs Achieving Highest Mean Success"
  names(xvalResults)[[5]] <- "Root Mean Squared Error by Number of PCs of PCA"
  names(xvalResults)[[6]] <- "Number of PCs Achieving Lowest MSE"
  names(xvalResults)[[7]] <- "DAPC"
  
  
  return(xvalResults)
  
} # end xvalDapc.data.frame


xvalDapc.data.frame <- xvalDapc.default
xvalDapc.matrix <- xvalDapc.data.frame
xvalDapc.genlight <- function(x, ...){
  xvalDapc.matrix(as.matrix(x), ...)
}
xvalDapc.genind <- function(x, ...){
  xvalDapc.matrix(tab(x), ...)
}

#' Genotype composition plot
#'
#' The compoplot uses a barplot to represent the group assignment probability of
#' individuals to several groups. It is a generic with methods for the following
#' objects:
#'
#' \itemize{
#' 
#' \item \code{matrix}: a matrix with individuals in row and genetic clusters in
#' column, each entry being an assignment probability of the corresponding
#' individual to the corresponding group
#' 
#' \item \code{dapc}: the output of the \code{dapc} function; in this case,
#' group assignments are based upon geometric criteria in the discriminant space
#'
#' \item \code{snapclust}: the output of the \code{snapclust} function; in
#' this case, group assignments are based upon the likelihood of genotypes
#' belonging to their groups
#' 
#' }
#'
#' @author Thibaut Jombart \email{thibautjombart@@gmail.com}
#'
#' @export
#'
#' @rdname compoplot
#' @aliases compoplot
#'
#' @param x an object to be used for plotting (see description)
#' 
#' @param ... further arguments to be passed to \code{barplot}
#' 
compoplot <- function(x, ...){
    UseMethod("compoplot", x)
}


#' Palette parser
#'
#' @param inPAL a palette function OR a character vector (named or unnamed)
#' @param npop number of populations/colors desired
#' @param pnames names of these populations if `inPal` is a function
#'
#' @md
#' @return a named character vector specifying the colors for the palette.
#' @keywords internal
#' 
#' @note This was originally from the poppr package [commit a0818eed6](https://github.com/grunwaldlab/poppr/commit/a0818eed6a72d9145e46da73715dc22be0640b0c)
#'
#' @examples
#' palette_parser(rainbow, 5, letters[1:5])
#' palette_parser(colors()[1:5], 5, letters[1:5])
.palette_parser <- function(inPAL, npop, pnames){
  PAL <- try(match.fun(inPAL, descend = FALSE), silent = TRUE)
  if ("try-error" %in% class(PAL)){
    if (all(pnames %in% names(inPAL))){
      color <- inPAL[pnames]
    } else if (npop == length(inPAL)){
      color <- stats::setNames(inPAL, pnames)
    } else if (npop < length(inPAL)){
      warning("Number of populations fewer than number of colors supplied. Discarding extra colors.")
      color <- stats::setNames(inPAL[1:npop], pnames)
    } else {
      warning("insufficient color palette supplied. Using funky().")
      color <- stats::setNames(funky(npop), pnames)
    }
  } else {
    color   <- stats::setNames(PAL(npop), pnames)
  }
  return(color)
}


#' @rdname compoplot
#'
#' @aliases compoplot.matrix
#' @export
#'
#' @param col.pal a color palette to be used for the groups; defaults to \code{funky}
#'
#' @param border a color for the border of the barplot; use \code{NA} to
#' indicate no border. 
#' 
#' @param show.lab a logical indicating if individual labels should be displayed
#' 
#' @param lab a vector of individual labels; if NULL, row.names of the matrix are used
#' 
#' @param legend a logical indicating whether a legend should be provided for the colors
#' 
#' @param txt.leg a character vector to be used for the legend
#' 
#' @param n.col the number of columns to be used for the legend
#' 
#' @param posi the position of the legend
#' 
#' @param cleg a size factor for the legend
#' 
#' @param bg the background to be used for the legend
#' 
#' @param subset a subset of individuals to retain
#'
compoplot.matrix <- function(x, col.pal = funky, border = NA,
                             subset = NULL, show.lab = FALSE,
                             lab = rownames(x), legend = TRUE,
                             txt.leg = colnames(x), n.col = 4,
                             posi = NULL, cleg = .8,
                             bg = transp("white"), ...) {

    ## individual labels
    if (!show.lab) {
        lab <- rep("", nrow(x))
    }

    ## handle subset
    if (!is.null(subset)) {
        names(lab) <- rownames(x)
        x <- x[subset, , drop=FALSE]
        lab <- lab[rownames(x)]
    }

    ## group labels
    if (is.null(txt.leg)) {
        txt.leg <- colnames(x)
    }
    if (is.null(txt.leg)) {
        txt.leg <- seq_len(ncol(x))
    }
    ## generate colors, process arguments
    col <- .palette_parser(col.pal, ncol(x), txt.leg)
    
    ## position of the legend
    if (is.null(posi)) {
        posi <- list(x=0, y=-.01)
    }

    ## make the plot: we need to suppress warnings because '...' could contain
    ## arguments from other methods not meant to be used by 'barplot'

    suppressWarnings(
        out <- barplot(t(x), col = col,
                       ylab = "membership probability",
                       names.arg = lab, las = 3,
                       border = border, ...) )

    if (legend) {
        oxpd <- par("xpd")
        par(xpd=TRUE)
        legend(posi, fill=col, legend = txt.leg,
               cex = cleg, ncol = n.col, bg = bg)
        on.exit(par(xpd=oxpd))
    }

    return(invisible(out))
}





#' @rdname compoplot
#' @aliases compoplot.dapc
#' @export
#' @param only.grp a subset of groups to retain

## The compoplot for DAPC is basically a compoplot.matrix on the predicted group membership
## probabilities. Only extra features related to keeping a subset of groups or individuals.

compoplot.dapc <- function(x, only.grp=NULL, border = NA, ...){
    ## get predictions and subset if needed
    pred <- predict(x)$posterior

    ## handle group subsetting
    if (!is.null(only.grp)) {
        if(is.numeric(only.grp) || is.logical(only.grp)) {
            only.grp <- levels(x$grp)[only.grp]
        }
        to.keep <- as.character(x$grp) %in% only.grp
        pred <- pred[to.keep, , drop=FALSE]
        lab <- lab[to.keep]
    }

    ## call matrix method
    compoplot(pred, border = border, ...)

} # end compoplot









#' @rdname compoplot
#' @export
compoplot.snapclust <- function(x, border = NA, ...) {
    compoplot(x$proba, border = border, ...)
}

context("Compoplot Tests")

mat <- t(apply(matrix(sample(10, 30, replace = TRUE), ncol = 3), 1, function(x) x/sum(x)))

test_that("compoplot.matrix works", {
  skip_on_cran()
  expect_silent(compoplot(mat))
})

test_that("compoplot works with a custom palette", {
  skip_on_cran()
  expect_silent(compoplot(mat, col.pal = c("cyan", "magenta", "yellow")))
})

test_that("compoplot works with a named custom palette", {
  skip_on_cran()
  expect_silent(compoplot(mat, col.pal = c(`1`="cyan", `2`="magenta", `3`="yellow")))
})

test_that("compoplot throws a warning if there are too many colors", {
  skip_on_cran()
  expect_warning(compoplot(mat, funky(4)), "populations fewer than")
})

test_that("compoplot throws a warning if there are not enough colors and does its own thing", {
  skip_on_cran()
  expect_warning(compoplot(mat, c("red", "blue")), "Using funky()")
})

#########################
# Function genind2genpop
#########################


#' Conversion from a genind to a genpop object
#' 
#' The function \code{genind2genpop} converts genotypes data (genind) into 
#' alleles counts per population (genpop).
#' 
#' === 'missing' argument ===\cr The values of the 'missing' argument in 
#' \code{genind2genpop} have the following effects:\cr - "NA": if all genotypes 
#' of a population for a given allele are missing, count value will be NA\cr
#' 
#' - "0": if all genotypes of a population for a given allele are missing, count
#' value will be 0\cr
#' 
#' - "chi2": if all genotypes of a population for a given allele are missing, 
#' count value will be that of a theoretical count in of a Chi-squared test. 
#' This is obtained by the product of the margins sums divided by the total 
#' number of alleles.\cr
#' 
#' === processing the \code{@@other} slot ===\cr Essentially, 
#' \code{genind2genpop} is about aggregating data per population. The function 
#' can do the same for all numeric items in the \code{@@other} slot provided 
#' they have the same length (for vectors) or the same number of rows 
#' (matrix-like objects) as the number of genotypes. When the case is 
#' encountered and if \code{process.other} is TRUE, then these objects are 
#' processed using the function defined in \code{other.action} per population. 
#' For instance, spatial coordinates of genotypes would be averaged to obtain 
#' population coordinates.
#' 
#' @param x an object of class \code{genind}.
#' @param pop a factor giving the population of each genotype in 'x' OR a
#'   formula specifying which strata are to be used when converting to a genpop
#'   object. If none provided, population factors are sought in x@@pop, but if
#'   given, the argument prevails on x@@pop.
#' @param quiet logical stating whether a conversion message must be printed 
#'   (TRUE,default) or not (FALSE).
#' @param process.other a logical indicating whether the \code{@@other} slot 
#'   should be processed (see details).
#' @param other.action a function to be used when processing the \code{@@other} 
#'   slot. By default, 'mean' is used.
#' @return A genpop object. The component @@other in 'x' is passed to the 
#'   created genpop object.
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#' @seealso \linkS4class{genind}, \linkS4class{genpop}
#' @keywords classes manip multivariate
#' @examples
#'
#' ## simple conversion
#' data(nancycats)
#' nancycats
#' catpop <- genind2genpop(nancycats)
#' catpop
#' summary(catpop)
#'
#' ## processing the @@other slot
#' data(sim2pop)
#' sim2pop$other$foo <- letters
#' sim2pop
#' dim(sim2pop$other$xy) # matches the number of genotypes
#' sim2pop$other$foo # does not match the number of genotypes
#'
#' obj <- genind2genpop(sim2pop, process.other=TRUE)
#' obj$other # the new xy is the populations' centre
#'
#' pch <- as.numeric(pop(sim2pop))
#' col <- pop(sim2pop)
#' levels(col) <- c("blue","red")
#' col <- as.character(col)
#' plot(sim2pop$other$xy, pch=pch, col=col)
#' text(obj$other$xy, lab=row.names(obj$other$xy), col=c("blue","red"), cex=2, font=2)
#' \dontrun{
#' data(microbov)
#' strata(microbov) <- data.frame(other(microbov))
#' summary(genind2genpop(microbov)) # Conversion based on population factor
#' summary(genind2genpop(microbov, ~coun)) # Conversion based on country
#' summary(genind2genpop(microbov, ~coun/spe)) # Conversion based on country and species
#' 
#' }
#'
#' @export genind2genpop
genind2genpop <- function(x, pop = NULL, quiet = FALSE, 
                          process.other = FALSE, other.action = mean){

    ## CHECKS ##
    if (!is.genind(x)) stop("x is not a valid genind object")
    checkType(x)
    if (!all(ploidy(x)[1]==ploidy(x))) stop("conversion to genpop not supported for varying ploidy")
    if (!is.null(pop)){
      if (is.language(pop)){
        setPop(x) <- pop
      } else {
        pop(x) <- pop
      }
    }

    if (is.null(pop(x))) {
        if(!quiet) warning("\npop is not provided either in x or in pop - assuming one single group")
        pop(x) <- factor(rep(1, nInd(x)))
    }

    if (!quiet) cat("\n Converting data from a genind to a genpop object... \n")

    ## tabcount is a matrix pop x alleles, counting alleles per pop
    tabcount <- apply(tab(x), 2, tapply, pop(x), sum, na.rm=TRUE)

    ## restitute matrix class when only one pop
    if(is.null(dim(tabcount))) {
        lab.col <- names(tabcount)
        tabcount <- matrix(tabcount,nrow=1)
        colnames(tabcount) <- lab.col
        rownames(tabcount) <- levels(pop(x))[1]
    }

    ## MAKE FINAL OBJECT ##
    prevcall <- match.call()
    res <- new("genpop", tab=tabcount, prevcall=prevcall, ploidy=x@ploidy[1], type=x@type)

    ## handle @other here
    res@other <- x@other
    if(process.other){
        ## auxiliary function doing the job
        fOther <- function(e){
            N <- nrow(x@tab)
            if(is.vector(e) && is.numeric(e) && length(e)==N){ # numeric vector
                res <- tapply(e, pop(x), other.action)
                return(res)
            } else if(is.matrix(e) && is.numeric(e) && nrow(e)==N){ # numeric matrix
                res <- apply(e, 2, function(vec) tapply(vec, pop(x), other.action))
                colnames(res) <- colnames(e)
                return(res)
            } else if(is.data.frame(e) && nrow(e)==N && all(sapply(e,is.numeric)) ){ # df of numeric vectors
                res <- lapply(e, function(vec) tapply(vec, pop(x), other.action))
                res <- data.frame(res)
                names(res) <- names(e)
                return(res)
            } else return(e)
        } # end fOther

        res@other <- lapply(res@other, fOther)
    } # end if(process.other)

    if(!quiet) cat("\n...done.\n\n")

    return(res)

} # end genind2genpop

context("subset tests")

data("nancycats", package = "adegenet")

test_that("subsetters work for genind objects", {
  skip_on_cran()
  expect_equivalent(nInd(nancycats[1:10]), 10)
  expect_equivalent(nLoc(nancycats[loc = locNames(nancycats)[1]]), 1)
  expect_equivalent(nLoc(nancycats[loc = 1]), 1)
  expect_equivalent(nLoc(nancycats[loc = -1]), 8)
})

test_that("subsetters work for genind objects", {
  skip_on_cran()
  nanpop <- genind2genpop(nancycats, quiet = TRUE)
  expect_equivalent(nPop(nanpop[1:10]), 10)
  expect_equivalent(nLoc(nanpop[loc = locNames(nanpop)[1]]), 1)
  expect_equivalent(nLoc(nanpop[loc = 1]), 1)
  expect_equivalent(nLoc(nanpop[loc = -1]), 8)
})

##############
# loadingplot
##############
loadingplot <- function (x, ...) UseMethod("loadingplot")


loadingplot.default <- function(x, at=NULL, threshold=quantile(x,0.75), axis=1, fac=NULL, byfac=FALSE,
                        lab=NULL, cex.lab=0.7, cex.fac=1, lab.jitter=0,
                        main="Loading plot", xlab="Variables", ylab="Loadings", srt=0, adj=NULL, ...){
    ## some checks
    if(is.data.frame(x) | is.matrix(x)){
        if(is.null(lab)) {lab <- rownames(x)}
        x <- x[,axis]
    } else {
        if(is.null(lab)) {lab <- names(x)}
    }

    names(x) <- lab <- rep(lab, length=length(x))

    if(!is.numeric(x)) stop("x is not numeric")
    if(any(is.na(x))) stop("NA entries in x")
    if(any(x<0)) {
        warning("Some values in x are less than 0\n Using abs(x) instead, but this might not be optimal.")
        x <- abs(x)
    }

    ## handle lab
    if(is.null(lab)) {lab <- 1:length(x)}

    ## handle fac
    if(!is.null(fac)){
        if(byfac){
            x <- tapply(x, fac, sum)
            if(length(lab) != length(x)) lab <- names(x)
        } else {
            fac <- factor(fac, levels=unique(fac))
            grp.idx <- cumsum(table(fac)) + 0.5
            grp.lab.idx <- tapply(1:length(x), fac, mean)
            grp.lab <- names(grp.idx)
            grp.idx <- grp.idx[-length(grp.idx)]
        }
    }

    ## preliminary computations
    y.min <- min(min(x),0)
    y.max <- max(max(x),0)
    y.offset <- (y.max-y.min)*0.02

    ## handle 'at'
    if(is.null(at)){
        at <- 1:length(x)
    } else {
        if(length(at) != length(x)) stop("x and at do not have the same length.")
    }


    ## start the plot
    dat <- cbind(at, x)
    plot(dat, type="h", xlab=xlab, ylab=ylab,
         main=main, xaxt="n", ylim=c(y.min,y.max*1.2), ...)

    ## add groups of variables (optional)
    if(!is.null(fac) & !byfac) {
        abline(v=grp.idx,lty=2) # split groups of variables
        text(x=grp.lab.idx,y=y.max*1.15, labels=grp.lab, cex=cex.fac) # annotate groups
    }

    ## annotate variables that are above the threshold
    if(sum(x > threshold)>0){
        x.ann <- at[x > threshold]
        x.ann <- jitter(x.ann,factor=lab.jitter)

        y.ann <- x[x > threshold] + y.offset
        y.ann <- jitter(y.ann,factor=lab.jitter)

        txt.ann <- lab[x > threshold]
        text(x=x.ann, y=y.ann, label=txt.ann, cex=cex.lab, srt=srt, adj=adj)

    ## indicate the threshold
    abline(h=threshold, col="grey")

    ## build the result

    res <- list(threshold=threshold,
                var.names=txt.ann,
                var.idx=which(x > threshold),
                var.values=x[x > threshold])
    return(invisible(res))
    }

    return(NULL) # if no point above threshold
} # end loadingplot

###############
## inbreeding
###############
inbreeding <- function(x, pop=NULL, truenames=TRUE, res.type=c("sample","function","estimate"), N=200, M=N*10){
    ## CHECKS ##
    if(!is.genind(x)) stop("x is not a valid genind object")
    checkType(x)
    res.type <- match.arg(res.type)[1]

    ## if(x$ploidy != 2) stop("this inbreeding coefficient is designed for diploid genotypes only")
    PLO <- ploidy(x)
    LOC <- locFac(x)

    if(!is.null(pop)) pop(x) <- pop
    if(is.null(pop(x)) && is.null(pop)) {
        pop(x) <- factor(rep(1, nInd(x)))
    }

    ## COMPUTATIONS ##

    ## get allele frequencies and \sum p_i^2 by pop and loc ##
    ## (generalized to any ploidy) ##
    ## For genpop objects, a constant ploidy is assumed/needed. This kludge will do for now.
    tabfreq2 <- tab(genind2genpop(x, quiet = TRUE), freq=TRUE) ^ PLO[1]
    sumpi2   <- t(apply(tabfreq2, 1, tapply, LOC, sum))

    ## function to check a 1-locus genotype for homozigosity
    ## returns 1 if homoz, 0 otherwise
    ## !!! NOTE : reverse the values returned by f1 to obtain a strange thing !!!
    f1 <- function(gen){
        if(any(is.na(gen))) return(NA)
        return(ifelse(sum(gen > 0) == 1, 1, 0))
        # if(any(round(gen, 10)==1)) return(1)
        # return(0)
    }

    ## get the table of binary hetero/homo data
    X       <- tab(x)
    homotab <- t(apply(X, 1, tapply, LOC, f1))
    
    ## get pi2 for the appropriate pop
    popx   <- as.character(pop(x))
    tabpi2 <- sumpi2[popx, , drop=FALSE]



    ## function returning a likelihood function - multi-locus ##
    ## x is a vector of 1/0 for each locus, with 1=homoz
    LIK <- function(x, sumpi2){
        ## return( sum(log(   x*(F+(1-F)*sumpi2) + (1-x)*(1-(F+(1-F)*sumpi2)) ) ,na.rm=TRUE) ) # returns the log-likelihood
        myEnv <- new.env()
        assign("x", x, envir=myEnv)
        assign("sumpi2", sumpi2, envir=myEnv)
        res <- function(Fest) { # F estimate
            ## cat("\nx used:\n") # debugging
            ## print(x)
            ## cat("\nsumpi2 used:\n") # debugging
            ## print(sumpi2)
            phom <- Fest + (1-Fest)*sumpi2
            return(exp(sum(log(x*phom + (1-x)*(1-phom)), na.rm=TRUE)))
        }
        environment(res) <- myEnv

        return(res)
    }


    ## get likelihood functions for all individuals
    res <- lapply(1:nrow(homotab), function(i) LIK(homotab[i,], tabpi2[i,]) )
    res <- setNames(lapply(res, Vectorize), indNames(x))


    ## IF WE RETURN FUNCTIONS ##
    if (res.type=="function"){
      return(res)
    }

    ## IF WE RETURN MLE ##
    if (res.type == "estimate"){
      opfun  <- function(x, ...) optimize(x, ...)[[1]]
      funval <- numeric(1)
      res <- vapply(res, FUN = opfun , FUN.VALUE = funval, interval = c(0, 1),
                    maximum = TRUE, tol = .Machine$double.eps^0.75)
      return(res)
    }


    ## IF WE RETURN SAMPLES ##
    ## function to get one sample
    getSample <- function(f){ # f is a vectorized density function
        x <- runif(M, 0, 1)
        fx <- f(x)
        fx <- fx - min(fx)
        if(sum(fx)<1e-14) {
            warning("Likelihood uniformly zero likely reflecting precision issue\nreturning uniformly distributed sample")
            return(runif(N))
        }
        fx <- fx/sum(fx)
        return(sample(x, size=N, prob=fx))
    }

    res <- lapply(res, getSample)
    return(res)
} # end inbreeding

## ###############
## ## simOutbreak
## ###############
## simOutbreak <- function(R0, infec.curve, n.hosts=200, duration=50,
##                         seq.length=1e4, mu.transi=1e-4, mu.transv=mu.transi/2,
##                         tree=TRUE){

##     ## CHECKS ##
##     if(!require(ape)) stop("The ape package is required.")


##     ## HANDLE ARGUMENTS ##
##     ## normalize gen.time
##     infec.curve <- infec.curve/sum(infec.curve)
##     infec.curve <- c(infec.curve, rep(0, duration)) # make sure dates go all the way
##     t.clear <- which(diff(infec.curve<1e-10)==1) # time at which infection is cleared

##     ## GENETIC FUNCTIONS ##
##     NUCL <- as.DNAbin(c("a","t","c","g"))
##     TRANSISET <- list('a'=as.DNAbin('g'), 'g'=as.DNAbin('a'), 'c'=as.DNAbin('t'), 't'=as.DNAbin('c'))
##     TRANSVSET <- list('a'=as.DNAbin(c('c','t')), 'g'=as.DNAbin(c('c','t')), 'c'=as.DNAbin(c('a','g')), 't'=as.DNAbin(c('a','g')))


##     ## AUXILIARY FUNCTIONS ##
##     ## generate sequence from scratch
##     seq.gen <- function(){
##         ##res <- list(sample(NUCL, size=seq.length, replace=TRUE)) # DNAbin are no longer lists by default
##         res <- sample(NUCL, size=seq.length, replace=TRUE)
##         class(res) <- "DNAbin"
##         return(res)
##     }

##     ## create substitutions for defined SNPs - no longer used
##     substi <- function(snp){
##         res <- sapply(1:length(snp), function(i) sample(setdiff(NUCL,snp[i]),1)) # ! sapply does not work on DNAbin vectors directly
##         class(res) <- "DNAbin"
##         return(res)
##     }

##     ## create transitions for defined SNPs
##     transi <- function(snp){
##         res <- unlist(TRANSISET[as.character(snp)])
##         class(res) <- "DNAbin"
##         return(res)
##     }

##     ## create transversions for defined SNPs
##     transv <- function(snp){
##         res <- sapply(TRANSVSET[as.character(snp)],sample,1)
##         class(res) <- "DNAbin"
##         return(res)
##     }

##     ## duplicate a sequence (including possible mutations)
##     seq.dupli <- function(seq, T){ # T is the number of time units between ancestor and decendent
##         ## transitions ##
##         n.transi <- rbinom(n=1, size=seq.length*T, prob=mu.transi) # total number of transitions
##         if(n.transi>0) {
##             idx <- sample(1:seq.length, size=n.transi, replace=FALSE)
##             seq[idx] <- transi(seq[idx])
##         }

##         ## transversions ##
##         n.transv <- rbinom(n=1, size=seq.length*T, prob=mu.transv) # total number of transitions
##         if(n.transv>0) {
##             idx <- sample(1:seq.length, size=n.transv, replace=FALSE)
##             seq[idx] <- transv(seq[idx])
##         }
##         return(seq)
##     }


##     ## MAIN FUNCTION ##
##     ## initialize results ##
##     dynam <- data.frame(nsus=integer(duration+1), ninf=integer(duration+1), nrec=integer(duration+1))
##     rownames(dynam) <- 0:duration
##     res <- list(n=1, dna=NULL, dates=NULL, id=NULL, ances=NULL, dynam=dynam)
##     res$dynam$nsus[1] <- n.hosts-1
##     res$dynam$ninf[1] <- 1
##     res$dates[1] <- 0
##     res$ances <- NA
##     res$dna <- matrix(seq.gen(),nrow=1)
##     class(res$dna) <- "DNAbin"

##     ## run outbreak ##
##     for(t in 1:duration){
##         ## individual force of infection
##         indivForce <- infec.curve[t-res$dates+1]

##         ## global force of infection (R0 \sum_j I_t^j / N)
##         globForce <- sum(indivForce)*R0/n.hosts

##         ## number of new infections
##         nbNewInf <- rbinom(1, size=res$dynam$nsus[t], prob=globForce)

##         ## dates of new infections
##         if(nbNewInf>0){
##             res$dates <- c(res$dates, rep(t,nbNewInf))

##             ## ancestries of the new infections
##             temp <- as.vector(rmultinom(1, size=nbNewInf, prob=indivForce))
##             newAnces <- rep(which(temp>0), temp[which(temp>0)])
##             res$ances <- c(res$ances,newAnces)

##             ## dna sequences of the new infections
##             newSeq <- t(sapply(newAnces, function(i) seq.dupli(res$dna[i,], t-res$dates[i])))
##             res$dna <- rbind(res$dna, newSeq)
##         }

##         ## update nb of infected, recovered, etc.
##         res$dynam$nrec[t+1] <- sum(res$dates>=t.clear)
##         res$dynam$ninf[t+1] <- sum(res$dates>=0 & res$dates < t.clear)
##         res$dynam$nsus[t+1] <- res$dynam$nsus[t] - nbNewInf
##     } # end for


##     ## SHAPE AND RETURN OUTPUT ##
##     res$n <- nrow(res$dna)
##     res$id <- 1:res$n
##     res$nmut <- sapply(1:res$n, function(i) dist.dna(res$dna[c(res$id[i],res$ances[i]),], model="raw"))*ncol(res$dna)
##     res$call <- match.call()
##     if(tree){
##         res$tree <- fastme.ols(dist.dna(res$dna, model="TN93"))
##         res$tree <- root(res$tree,"1")
##     }
##     class(res) <- "simOutbreak"
##     return(res)

## } # end simOutbreak








## ##################
## ## print.simOutbreak
## ##################
## print.simOutbreak <- function(x, ...){

##     cat("\t\n=========================")
##     cat("\t\n=   simulated outbreak  =")
##     cat("\t\n=  (simOutbreak object) =")
##     cat("\t\n=========================\n")

##     cat("\nSize :", x$n,"cases (out of", x$dynam$nsus[1],"susceptible hosts)")
##     cat("\nGenome length :", ncol(x$dna),"nucleotids")
##     cat("\nDate range :", min(x$dates),"-",max(x$dates))

##     cat("\nContent:\n")
##     print(names(x))

##     return(NULL)
## } # end print.simOutbreak







## ##############
## ## [.simOutbreak
## ##############
## "[.simOutbreak" <- function(x,i,j,drop=FALSE){
##     res <- x
##     res$dna <- res$dna[i,,drop=FALSE]
##     res$id <- res$id[i]
##     res$ances <- res$ances[i]
##     res$ances[!res$ances %in% res$id] <- NA
##     res$dates <- res$dates[i]
##     res$n <- nrow(res$dna)

##     return(res)
## }





## ##################
## ## labels.simOutbreak
## ##################
## labels.simOutbreak <- function(object, ...){
##     return(object$id)
## }






## #########################
## ## as.igraph.simOutbreak
## #########################
## as.igraph.simOutbreak <- function(x, ...){
##     if(!require(igraph)) stop("package igraph is required for this operation")
##     if(!require(ape)) stop("package ape is required for this operation")

##     ## GET DAG ##
##     from <- x$ances
##     to <- x$id
##     isNotNA <- !is.na(from) & !is.na(to)
##     dat <- data.frame(from,to,stringsAsFactors=FALSE)[isNotNA,,drop=FALSE]
##     vnames <- as.character(unique(unlist(dat)))
##     out <- graph.data.frame(dat, directed=TRUE, vertices=data.frame(names=vnames, dates=x$dates[vnames]))

##     ## SET WEIGHTS ##
##     D <- as.matrix(dist.dna(x$dna,model="raw")*ncol(x$dna))
##     temp <- mapply(function(i,j) return(D[i,j]), as.integer(from), as.integer(to))
##     E(out)$weight <- temp[isNotNA]

##     ## SET ARROW WIDTH ##
##     temp <- max(E(out)$weight) - E(out)$weight
##     temp <- temp/max(temp) * 4
##     E(out)$width <- round(temp)+1

##     return(out)
## }







## ####################
## ## plot.simOutbreak
## ####################
## plot.simOutbreak <- function(x, y=NULL, cex=1, col=num2col(x$dates), label=x$id,
##                              edge.col=num2col(x$nmut[-1], col.pal=seasun), lwd=1, ...){
##     if(!require(igraph)) stop("package igraph is required for this operation")
##     if(!require(ape)) stop("package ape is required for this operation")
##     plot(as.igraph(x), vertex.size=15*cex, vertex.color=col, vertex.label=label,
##          vertex.label.cex=cex, edge.color=edge.col, edge.width=lwd, ...)
## } # end plot.simOutbreak




















## ## #####################
## ## ## seqTrack.simOutbreak
## ## #####################
## ## seqTrack.simOutbreak <- function(x, best=c("min","max"), prox.mat=NULL, ...){
## ##     myX <- dist.dna(x$dna, model="raw")
## ##     x.names <- labels(x)
## ##     x.dates <- as.POSIXct(x)
## ##     seq.length <- ncol(x$dna)
## ##     myX <- myX * seq.length
## ##     myX <- as.matrix(myX)
## ##     prevCall <- as.list(x$call)
## ##     if(is.null(prevCall$mu)){
## ##         mu0 <- 0.0001
## ##     } else {
## ##         mu0 <- eval(prevCall$mu)
## ##     }
## ##     res <- seqTrack(myX, x.names=x.names, x.dates=x.dates, best=best, prox.mat=prox.mat,...)
## ##     return(res)
## ## }






## ## ########################
## ## ## as.seqTrack.simOutbreak
## ## ########################
## ## as.seqTrack.simOutbreak <- function(x){
## ##     ## x.ori <- x
## ##     ## x <- na.omit(x)
## ##     toSetToNA <- x$dates==min(x$dates)
## ##     res <- list()
## ##     res$id <- labels(x)
## ##     res <- as.data.frame(res)
## ##     res$ances <- x$ances
## ##     res$ances[toSetToNA] <- NA
## ##     res$weight <- 1 # ??? have to recompute that...
## ##     res$weight[toSetToNA] <- NA
## ##     res$date <- as.POSIXct(x)[labels(x)]
## ##     res$ances.date <- as.POSIXct(x)[x$ances]

## ##     ## set results as indices rather than labels
## ##     res$ances <- match(res$ances, res$id)
## ##     res$id <- 1:length(res$id)

## ##     ## SET CLASS
## ##     class(res) <- c("seqTrack", "data.frame")

## ##     return(res)
## ## }








## ## ###################
## ## ## sample.simOutbreak
## ## ###################
## ## sample.simOutbreak <- function(x, n){
## ## ##sample.simOutbreak <- function(x, n, rDate=.rTimeSeq, arg.rDate=NULL){
## ##     ## EXTRACT THE SAMPLE ##
## ##     res <- x[sample(1:nrow(x$dna), n, replace=FALSE)]


## ##     ## RETRIEVE SOME PARAMETERS FROM HAPLOSIM CALL
## ##     prevCall <- as.list(x$call)
## ##     if(!is.null(prevCall$mu)){
## ##         mu0 <- eval(prevCall$mu)
## ##     } else {
## ##         mu0 <- 1e-04
## ##     }

## ##     if(!is.null(prevCall$dna.length)){
## ##         L <- eval(prevCall$dna.length)
## ##     } else {
## ##         L <- 1000
## ##     }

## ##     ## truedates <- res$dates
## ##     ## daterange <- diff(range(res$dates,na.rm=TRUE))

## ##     ## if(identical(rDate,.rTimeSeq)){
## ##     ##     sampdates <- .rTimeSeq(n=length(truedates), mu=mu0, L=L, maxNbDays=daterange/2)
## ##     ## } else{
## ##     ##     arg.rDate$n <- n
## ##     ##     sampdates <- do.call(rDate, arg.rDate)
## ##     ## }
## ##     ## sampdates <- truedates + abs(sampdates)

## ##     return(res)
## ## } # end sample.simOutbreak

# Algorithm to detect boundaries, based on Monmonier's algorithm
# Extended to any connection network
# Thibaut Jombart 2006-2008 (t.jombart@imperial.ac.uk)



#####################
# function monmonier
#####################
monmonier <- function(xy, dist, cn, threshold=NULL, bd.length=NULL, nrun=1,
                      skip.local.diff=rep(0,nrun), scanthres=is.null(threshold), allowLoop=TRUE){
if(!inherits(cn,"nb")) stop('cn is not a nb object')
if(is.data.frame(xy)) xy <- as.matrix(xy)
if(!is.matrix(xy)) stop('xy must be a matrix')
if(!inherits(dist,"dist")) stop('Argument \'dist\' must be a distance matrix of class dist')
if(nrow(xy) != nrow(as.matrix(dist))) stop('Number of sites and number of observations differ')

## set to TRUE to debug
## DEBUG <- FALSE

## if(DEBUG) {
##     plot.nb(cn, xy, col="grey", points=FALSE)
##     if(exists("x1")) {
##         x1 <<- x1
##         s.value(xy,x1,add.p=TRUE)
##     }
##     text(xy,lab=1:nrow(xy),font=2,col="darkgreen")
## }

## PRECISION of the xy coordinates (in digits)
## used when coordinates are inputed in C code.
PRECISION=12

## conversion of the connection network
cn.nb <- cn
cn <- nb2neig(cn)
## binary matrix of neighbourhood
M <- neig2mat(cn)
## distance matrix
D <- as.matrix(dist)
## matrix of distances among neighbours
D <- M*D

## set/check the threshold value of the distances among neighbours
## default: the median of all distances among neighbours
if(is.null(threshold) || threshold<0) {Dlim <- summary(unique(D[D>0]))[5]} else {Dlim <- threshold}

## handle 'bd.length' / must prevail over threshold.
if(!is.null(bd.length)) {
    if(bd.length < 1) bd.length <- 1
    Dlim <- 0
    scanthres <- FALSE
} else {
    bd.length <- nrow(xy)^2
}


if(scanthres){
  plot(sort(unique(D)[unique(D) > 0],decreasing=TRUE),main="Local distances plot",type="l",xlab="rank",ylab="Sorted local distances")
  abline(h=Dlim,lty=2)
  mtext("Dashed line indicates present threshold")
  cat("Indicate the threshold (\'d\' for default): ")
  temp <- as.character(readLines(con = getOption('adegenet.testcon'), n = 1))
  if(toupper(temp)!="D") { Dlim <- as.numeric(temp) }
}

#### OLD VERSION ####
## ## build a data.frame of coords of neighbours, with columns x1 y1 x2 y2
## listCpl <- neig.util.GtoL(neig2mat(cn))
## allSeg <- cbind(xy[,1][listCpl[,1]] , xy[,2][listCpl[,1]] , xy[,1][listCpl[,2]] , xy[,2][listCpl[,2]])
## colnames(allSeg) <- c('xP','yP','xQ','yQ')
#### END OLD VERSION ####

## Compute once and for all a matrix matSegVal with columns A, B and val
## A: index of one vertex
## B: index if another vertex, connected (beighbour) to B
## val: distance between A and B
## This table is such that "val" is sorted in decreasing order.
matSegVal <- which(D>0,arr.ind=TRUE)
matSegVal.names <- apply(matSegVal,1,function(vec) paste(sort(vec),collapse="-"))
rownames(matSegVal) <- matSegVal.names # used to get unique segments (cause D is symmetric)
matSegVal <- matSegVal[unique(rownames(matSegVal)),]
temp <- apply(matSegVal,1,function(vec) D[vec[1],vec[2]])
matSegVal <- cbind(matSegVal,temp) # matSegVal has its 3 colums
idx <- order(matSegVal[,3], decreasing=TRUE)
matSegVal <- matSegVal[idx,]

## allSeg is a matrix giving the coordinates of all edges (in row), x1 y1 x2 y2.
allSeg <- cbind(xy[matSegVal[,1],] , xy[matSegVal[,2],])
rownames(allSeg) <- rownames(matSegVal)
colnames(allSeg) <- c("x1","y1","x2","y2")

## auxiliary function used to remove a segment from matSegVal (A,B: indices of vertices of the removed edge)
rmFromMatSegVal <- function(A,B){
    AB.name <- paste(sort(c(A,B)), collapse="-")
    toRemove <- match(AB.name, rownames(matSegVal))
    matSegVal <<- matSegVal[-toRemove,]
}

## Given a rank 'rang', getNext retrieves the two corresponding neighbours (A and B),
## their middle (M), and the corresponding value.
## A is the index of the vertex, xA and yA are its coordinates
getNext <- function(rang){
    ## ## ## OLD VERSION ## ## ##
    ## val <- unique(sort(D,decreasing=TRUE))[rang]
    ##  A <- which(round(D,10)==round(val,10),TRUE)[1,1] # beware: == used on doubles
    ##     B <- which(round(D,10)==round(val,10),TRUE)[1,2] # beware: == used on doubles
    ##     xA <- xy[A,1]
    ##     yA <- xy[A,2]
    ##     xB <- xy[B,1]
    ##     yB <- xy[B,2]
    ## 	xM <- (xA + xB)/2
    ##     yM <- (yA + yB)/2
    ## ## ## END OLD VERSION ## ## ##

    #### Must take into account that the some local distances occur for different segments
    #### Create a table with columns A, B and val

    temp <- matSegVal[rang,]
    A <- temp[1]
    B <- temp[2]
    val <- temp[3]
    xA <- xy[A,1]
    yA <- xy[A,2]
    xB <- xy[B,1]
    yB <- xy[B,2]
    xM <- mean(c(xA,xB))
    yM <- mean(c(yA,yB))

    return( list(A=c(A,xA,yA), B=c(B,xB,yB), M=c(xM,yM), val=val) )
}

## checkNext returns TRUE if it is ok to draw the segment MN, i.e. if MN does not cross another edge,
## and FALSE otherwise
## - M and N define the segment of interest
## M = c(xM,yM) ; N=c(xN,yN)
## -segMat is the matrix of all known edges
## A,B,C,D are not coordinates, but indices of vertices
## -AB is the edge whose middle is M ; given to avoid checking MN vs AB
## -CD is the edge whose middle is N ; given to avoid checking MN vs CD
## - toRemove: a vector of characters naming other edges in segMat to be removed before checking
checkNext <- function(M,N,A,B,C,D,segMat=allSeg,toRemove=NULL,curDir=NULL){
    ## orientation of the segment MN
    xmin <- min(M[1],N[1])
    xmax <- max(M[1],N[1])
    ymin <- min(M[2],N[2])
    ymax <- max(M[2],N[2])

### From here segment that would be difficult to check for crossing are removed
### If MN is the segment of interest, with M middle of AB and N middle of CD,

    subsetSeg <- segMat
    ## The segment that was just drawn before is removed from the checks for crossing (messy code 2)
    if(!is.null(curDir)){
        prevSeg <- grep(paste("dir",curDir,sep=""),rownames(subsetSeg))
        if(length(prevSeg)>0){ # this is not the 1st seg. in this dir
            prevSeg <- prevSeg[length(prevSeg)]
            subsetSeg <- segMat[-prevSeg,,drop=FALSE]
        } else { # this is the 1st seg. in this direction -> rm 1st seg of other dir
            otherDir <- ifelse(curDir==1L, 2, 1)
            prevSeg <- grep(paste("dir",otherDir,"-1-2",sep=""),rownames(subsetSeg))
            if(length(prevSeg)>0) { subsetSeg <- segMat[-prevSeg,,drop=FALSE] }
        }
    }

    ## AB and CD are now taken out (doubtful code 2 returned by the C procedure CheckAllSeg)
    AB.name <- paste(sort(c(A,B)),collapse="-")
    AB.idx <- match(AB.name, rownames(subsetSeg))
    CD.name <- paste(sort(c(C,D)),collapse="-")
    CD.idx <- match(CD.name, rownames(subsetSeg))
    subsetSeg <- subsetSeg[-c(AB.idx,CD.idx),,drop=FALSE]

    ## subsetSeg is a matrix, each of its rows being a segment: xP,yP,xQ,yQ
    ## edges out of the square of diagonal MN cannot be crossed, so they are removed from
    ## the checks for crossing
    subsetSeg <- subsetSeg[!(subsetSeg[,1] < xmin & subsetSeg[,3] < xmin),,drop=FALSE]
    subsetSeg <- subsetSeg[!(subsetSeg[,1] > xmax & subsetSeg[,3] > xmax),,drop=FALSE]
    subsetSeg <- subsetSeg[!(subsetSeg[,2] < ymin & subsetSeg[,4] < ymin),,drop=FALSE]
    subsetSeg <- subsetSeg[!(subsetSeg[,2] > ymax & subsetSeg[,4] > ymax),,drop=FALSE]

    ## handle toRemove here
    if(!is.null(toRemove)){
        idx <- match(toRemove, rownames(subsetSeg))
        idx <- idx[!is.na(idx)]
        if(length(idx)>0) { subsetSeg <- subsetSeg[-idx,,drop=FALSE] }
    }

    ## temp is used for the output of CheckAllSeg
    ## initialized at 10, which is never returned by CheckAllSeg
    temp <- as.integer(10)

    ## call to CheckAllSeg
    ## =======================
    ## output code for CheckAllSeg:
    ## - 0: no intersection
    ## - 1: all kind of intersection, including the codes from SegSeg function
    ## (a C function inside monmonier-utils.C):
    ## - 3 : The segments collinearly overlap, sharing at least a point.
    ## - 2 : An endpoint (vertex) of one segment is on the other segment,
    ## but segments aren't collinear.
    ## - 1 : The segments intersect properly (i.e. not case 2 or 3)
    ## =======================
    ##
    ## restore the matrix type if there is only one segment to check for crossing in subsetSeg

    ## round down coordinates in subsetSeg
    subsetSeg <- round(subsetSeg, digits=PRECISION)

    if(nrow(subsetSeg)>0) {
        temp <- .C("CheckAllSeg",as.integer(nrow(subsetSeg)),as.integer(ncol(subsetSeg)),
                   as.double(as.matrix(subsetSeg)), as.double(M), as.double(N), temp,PACKAGE="adegenet")[[6]]
    } else {temp <- 0}

    ## for debugging
    ##  if(DEBUG) {
    ##                 if(temp==1)  cat("\n can't go there (code",temp,")") else cat("\n new segment ok (code",temp,")")
    ##             }

    ## if a code 1 or 3 is returned, CheckAllSeg returns FALSE
    ## else it returns TRUE
    ## additional control used (code 10, CheckAllSeg failed)
    if(temp==10) stop("CheckAllSeg failure (returned value=10, i.e. unchanged, not computed). Please report the bug.")
    if(temp==1 | temp==2 | temp==3) return(FALSE) else return(TRUE)
} # end of checkNext

## result object is created
## this is a list with one component per run (a run = a boundary)
result <-list()
for(run in 1:nrun){
    result[[run]] <- list(dir1=list(),dir2=list())
}


#### MAIN FUNCTION HERE
#### a for loop is used to handle several runs
## each boundary seeks one starting point, and then expands both sides (dir1 / dir2)
for(run in 1:nrun){

    ## handle skip.local.diff here
    ## the corresponding values of distance are set definitely to -1
    ##   if(skip.local.diff[run] >0)
    ##         for(i in 1:skip.local.diff[run]){
    ##             temp <- getNext(1)
    ##             ## D[temp$A[1],temp$B[1] ] <- -1
    ##             ## D[temp$B[1],temp$A[1] ] <- -1
    ##             rmFromMatSegVal(temp$A[1],temp$B[1])
    ##         }

    ## starting point: find the highest distance among neighbours
    ## then expand
    currentDir1 <- getNext(1 + skip.local.diff[run])
    currentDir2 <- currentDir1
    current.bd.length <- 1
    if(currentDir1$val<=Dlim) stop(paste('Algorithm reached the threshold value at the first step of run',run))
    result[[run]]$dir1[[1]] <- currentDir1 # first point dir1
    result[[run]]$dir2[[1]] <- currentDir2 # first point dir2 (same as dir1)
    ## D[result[[run]]$dir1[[1]]$A[1],result[[run]]$dir1[[1]]$B[1]] <- -1 # update D matrix
    ## D[result[[run]]$dir1[[1]]$B[1],result[[run]]$dir1[[1]]$A[1]] <- -1 # update D matrix
    rmFromMatSegVal(result[[run]]$dir1[[1]]$A[1], result[[run]]$dir1[[1]]$B[1]) # update matSegVal

    ## dir1 => i1: result index; s1: index for the rank of the distance between neighbours (decreasing order)
    ## dir2 => i2: result index; s2: index for the rank of the distance between neighbours (decreasing order)
    i1 <- 1
    s1 <- 1
    i2 <- 1
    s2 <- 2
    n <- nrow(D)

    ## logical handling the expansion of a boundary
    keepExpanding <- ((current.bd.length < bd.length)
                      && ((currentDir1$val>Dlim)|(currentDir2$val>Dlim))
                      && (s1 < nrow(matSegVal) | s2 < nrow(matSegVal)) )

    #### This while loop has the following behavior:
    ## as long as the keepExpanding is true, we try to expand the boundary by
    ## 1) finding the highest next distance among neighbours
    ## 2) test if we can draw the corresponding segment
    ## 3) - if we can, store the result, increment i1 or i2, reset s1 and s2, erase the edge
    ## 3) - if we can't, take the next distance among neighbours (incrementing s1 and s2)
    ## 4) get back to 1)
    while(keepExpanding){
        hasExpanded <- FALSE # used to test if it is relevant to check for looping
        ##  if(DEBUG){
        ##                             points(currentDir1$M[1],currentDir1$M[2],col="white",pch=20)
        ##                             points(currentDir2$M[1],currentDir2$M[2],col="white",pch=20)
        ##                         }
        if(s1 <= nrow(matSegVal)) { currentDir1 <- getNext(s1) }
        if(s2 <= nrow(matSegVal)) { currentDir2 <- getNext(s2) }
        ##  if(DEBUG){
        ##                             cat("\n\n ## dir1: trying edge",currentDir1$A[1],"-",currentDir1$B[1])
        ##                             points(currentDir1$M[1],currentDir1$M[2],col="blue",pch=20)
        ##                             readline("\npress enter")
        ##                         }

        ## first direction (dir1)
        if( currentDir1$val > Dlim && s1 <= nrow(matSegVal)) {
            if(checkNext(result[[run]]$dir1[[length(result[[run]]$dir1)]]$M,
                         currentDir1$M,
                         result[[run]]$dir1[[length(result[[run]]$dir1)]]$A[1],
                         result[[run]]$dir1[[length(result[[run]]$dir1)]]$B[1],
                         currentDir1$A[1],
                         currentDir1$B[1], curDir=as.integer(1))) {
                i1 <- i1+1
                result[[run]]$dir1[[i1]] <- currentDir1
                ## update the matrix of diff. between neighbours
                ## D[result[[run]]$dir1[[i1]]$A[1],result[[run]]$dir1[[i1]]$B[1]] <- -1
                ## D[result[[run]]$dir1[[i1]]$B[1],result[[run]]$dir1[[i1]]$A[1]] <- -1
                rmFromMatSegVal(result[[run]]$dir1[[i1]]$A[1],result[[run]]$dir1[[i1]]$B[1] ) # update matSegVal

                s1 <- 1
                ## update existing segments
                allSeg <- rbind(allSeg,c(result[[run]]$dir1[[i1-1]]$M,result[[run]]$dir1[[i1]]$M) )
                rownames(allSeg) <- c(rownames(allSeg)[-nrow(allSeg)] , paste("dir1",i1-1,i1,sep="-"))
                ## add 1 to the boundary length
                current.bd.length <- current.bd.length + 1
                hasExpanded <- TRUE
               ##  if(DEBUG) {
                ##                                                     arrows(result[[run]]$dir1[[i1-1]]$M[1], result[[run]]$dir1[[i1-1]]$M[2],
                ##                                                            result[[run]]$dir1[[i1]]$M[1], result[[run]]$dir1[[i1]]$M[2],col="blue")
                ##                                                 }

            } else{ s1 <- s1+1 }
        } # end "if( currentDir1$val>Dlim)"

          ## if(DEBUG){
        ##                             cat("\n\n ## dir2: trying edge",currentDir2$A[1],"-",currentDir2$B[1])
        ##                             points(currentDir2$M[1],currentDir2$M[2],col="red",pch=20)
        ##                             readline("\npress enter")
        ##                         }

        ## second direction (dir2)
        if( currentDir2$val > Dlim  && s2 <= nrow(matSegVal)) {
            if(checkNext(result[[run]]$dir2[[length(result[[run]]$dir2)]]$M,
                         currentDir2$M,
                         result[[run]]$dir2[[length(result[[run]]$dir2)]]$A[1],
                         result[[run]]$dir2[[length(result[[run]]$dir2)]]$B[1],
                         currentDir2$A[1],
                         currentDir2$B[1], curDir=as.integer(2))) {
                i2 <- i2+1
                result[[run]]$dir2[[i2]] <- currentDir2
                ## update the matrix of diff. between neighbours
                ## D[result[[run]]$dir2[[i2]]$A[1],result[[run]]$dir2[[i2]]$B[1]] <- -1
                ## D[result[[run]]$dir2[[i2]]$B[1],result[[run]]$dir2[[i2]]$A[1]] <- -1
                rmFromMatSegVal(result[[run]]$dir2[[i2]]$A[1],result[[run]]$dir2[[i2]]$B[1])
                s2 <- 1
                ## update existing segments
                allSeg <- rbind(allSeg,c(result[[run]]$dir2[[i2-1]]$M,result[[run]]$dir2[[i2]]$M) )
                rownames(allSeg) <- c(rownames(allSeg)[-nrow(allSeg)] , paste("dir2",i2-1,i2,sep="-"))
                ## add 1 to the boundary length
                current.bd.length <- current.bd.length + 1
                hasExpanded <- TRUE
                 ## if(DEBUG){
                ##                                                     arrows(result[[run]]$dir2[[i2-1]]$M[1], result[[run]]$dir2[[i2-1]]$M[2],
                ##                                                            result[[run]]$dir2[[i2]]$M[1], result[[run]]$dir2[[i2]]$M[2],col="red",cex=2)
                ##                                                 }

            } else{ s2 <- s2+1 }
        } # end "if( currentDir2$val>Dlim)"

        ## update the logical for the while loop
        keepExpanding <- ((current.bd.length < bd.length)
                          && ((currentDir1$val>Dlim)|(currentDir2$val>Dlim))
                          && (s1 <= nrow(matSegVal) | s2 <= nrow(matSegVal)) )

        ## handle the looping of a boundary
        if(hasExpanded && (current.bd.length>3) && allowLoop){
            ## check if the two ends of the boundary can be joined
            ## remove segments ending each direction (to avoid messy code 2 in checkNext)
            terminalEdges <- c(paste("dir1",i1-1,i1,sep="-"),
                               paste("dir2",i2-1,i2,sep="-"))

            canLoop <- checkNext(result[[run]]$dir1[[length(result[[run]]$dir1)]]$M,
                                 result[[run]]$dir2[[length(result[[run]]$dir2)]]$M,
                                 result[[run]]$dir1[[length(result[[run]]$dir1)]]$A[1],
                                 result[[run]]$dir1[[length(result[[run]]$dir1)]]$B[1],
                                 result[[run]]$dir2[[length(result[[run]]$dir2)]]$A[1],
                                 result[[run]]$dir2[[length(result[[run]]$dir2)]]$B[1],
                                 toRemove=terminalEdges
                                 )

            if(canLoop) {
                ## add the last, closing segment
                result[[run]]$dir1[[length(result[[run]]$dir1)+1]] <- result[[run]]$dir2[[length(result[[run]]$dir2)]]
                ## stop expanding
                keepExpanding <- FALSE
                ## update existing segments
                allSeg <- rbind(allSeg,c(result[[run]]$dir1[[length(result[[run]]$dir1)]]$M,
                                         result[[run]]$dir2[[length(result[[run]]$dir2)]]$M) )
            } # end looping of the boundary

        }

        ## output for debugging
        ##  if(DEBUG) {
        ##                     cat("\n","s1:",s1,"s2:",s2,"i1:",i1,"i2:",i2,"D1:",
        ##                         currentDir1$val,"D2:",currentDir2$val,"Dlim:",Dlim,
        ##                         "nrow(matSegVal)",nrow(matSegVal),"\n")
        ##                     cat("\n","D1:",currentDir1$val,"D2:",currentDir2$val,"Dlim:",Dlim,
        ##                         "cur.bd.le:", current.bd.length,"max length:", bd.length,"\n",
        ##                         "s1:",s1,"s2:",s2,"maxS:",nrow(matSegVal))
        ##                 }

    } # end of one given run

} # end for all run

# build the final output
# this is a list of class monmonier
# each element correspond to a run, i.e. to a potential boundary
# the output also contains the number of runs ($nrun) and the matched call ($call).
output=list()
for(run in 1:nrun){
	runname <- paste('run',run,sep='')
	output[[runname]] <- list(dir1=list(),dir2=list())
	# dir 1 #
	output[[runname]]$dir1$path <- matrix(-1, ncol=2,nrow=length(result[[run]]$dir1))
	colnames(output[[runname]]$dir1$path) <- c('x','y')
	rownames(output[[runname]]$dir1$path) <- paste('Point',1:nrow(output[[run]]$dir1$path),sep='_')

	for(i in 1:length(result[[run]]$dir1)) {
		output[[runname]]$dir1$path[i,] <- result[[run]]$dir1[[i]]$M
		output[[runname]]$dir1$values[i] <- result[[run]]$dir1[[i]]$val
	}

        # dir 2 #
	output[[runname]]$dir2$path <- matrix(-1, ncol=2,nrow=length(result[[run]]$dir2))
	colnames(output[[runname]]$dir2$path) <- c('x','y')
	rownames(output[[runname]]$dir2$path) <- paste('Point',1:nrow(output[[run]]$dir2$path),sep='_')
        for(i in 1:length(result[[run]]$dir2)) {
		output[[runname]]$dir2$path[i,] <- result[[run]]$dir2[[i]]$M
		output[[runname]]$dir2$values[i] <- result[[run]]$dir2[[i]]$val
	}

}

output$nrun <- nrun
output$threshold <- Dlim
output$xy <- xy
output$cn <- cn.nb
output$call <- match.call()
class(output) <- 'monmonier'
return(output)
}




##########################
# function plot.monmonier
##########################
plot.monmonier <- function(x, variable=NULL,displayed.runs=1:x$nrun,
                           add.arrows=TRUE, col='blue',lty=1,bwd=4, clegend=1,csize=0.7,
                           method = c('squaresize','greylevel'),sub='',csub=1,possub='topleft',
                           cneig=1,pixmap=NULL,contour=NULL,area=NULL,add.plot=FALSE,...){

    if (!inherits(x, "monmonier")) stop("Use only with 'monmonier' objects")
    if(!is.null(variable) & !is.numeric(variable)) stop('If provided, variable must be numeric.\n')


    xy <- x$xy
    cpoint <- 0

    if(cneig>0) {neig <- nb2neig(x$cn)} else {neig <- NULL}
    if(is.null(variable)){
	variable <- rep(1,nrow(xy))
	csize <- 0
	cpoint <- 1
	clegend <- 0
    }
    s.value(xy,variable,grid=FALSE,include.origin=FALSE,addaxes=FALSE,neig=neig,
            cneig=cneig,clegend=clegend,csize=csize,cpoint=cpoint,pch=20,pixmap=pixmap,
            method=method,sub=sub,csub=csub,possub=possub,add.plot=add.plot)
    opar <- par(no.readonly=TRUE)
    on.exit(par(mar=opar$mar))
    par(mar=c(0,0,0,0))

    for(run in displayed.runs){
        obj <- x[[run]]
        if(length(col)!=x$nrun) col <- rep(col,x$nrun)
        if(length(lty)!=x$nrun) lty <- rep(lty,x$nrun)
        if(length(obj$dir1$values) == 0) stop(paste('Monmonier object of run', run, 'is empty (no point in the path)\n'))

        if(length(obj$dir1$values) == 1 && length(obj$dir2$values) == 1) {
            points(obj$dir1$path[1],obj$dir1$path[2],pch=20,col=col[run],...)
        } else{
            ## handle boundary width
            ## the largest part corresponds to the highest distance among neighbours
            val.1 <- obj$dir1$values
            val.2 <- obj$dir2$values
            n1 <- length(val.1)
            n2 <- length(val.2)
            cex.bwd.1 <- ( val.1[1:(n1-1)] + val.1[2:n1] )/2
            cex.bwd.2 <- ( val.2[1:(n2-1)] + val.2[2:n2] )/2
            cex.bwd.max <- max(c(cex.bwd.1,cex.bwd.2), na.rm=TRUE)
            cex.bwd.1 <- cex.bwd.1/max(cex.bwd.max)
            cex.bwd.2 <- cex.bwd.2/max(cex.bwd.max)
            ## amplify the differences
            ## cex.bwd.1 <- cex.bwd.1^1.5
            ## cex.bwd.2 <- cex.bwd.2^1.5


            if(add.arrows) {
                if(n1>1) arrows(obj$dir1$path[1:(nrow(obj$dir1$path)-1),1],
                                obj$dir1$path[1:(nrow(obj$dir1$path)-1),2],
                                obj$dir1$path[2:nrow(obj$dir1$path),1],
                                obj$dir1$path[2:nrow(obj$dir1$path),2],
                                lwd=bwd*cex.bwd.1,angle=20,length=0.2,col=col[run],lty=lty[run],...)

                if(n2>1) arrows(obj$dir2$path[1:(nrow(obj$dir2$path)-1),1],
                                obj$dir2$path[1:(nrow(obj$dir2$path)-1),2],
                                obj$dir2$path[2:nrow(obj$dir2$path),1],
                                obj$dir2$path[2:nrow(obj$dir2$path),2],
                                lwd=bwd*cex.bwd.2,angle=20,length=0.2,col=col[run],lty=lty[run],...)
            } else {
                if(n1>1) segments(obj$dir1$path[1:(nrow(obj$dir1$path)-1),1],
                                  obj$dir1$path[1:(nrow(obj$dir1$path)-1),2],
                                  obj$dir1$path[2:nrow(obj$dir1$path),1],
                                  obj$dir1$path[2:nrow(obj$dir1$path),2],
                                  lwd=bwd*cex.bwd.1,col=col[run],lty=lty[run],...)

                if(n2>1)segments(obj$dir2$path[1:(nrow(obj$dir2$path)-1),1],
                                 obj$dir2$path[1:(nrow(obj$dir2$path)-1),2],
                                 obj$dir2$path[2:nrow(obj$dir2$path),1],
                                 obj$dir2$path[2:nrow(obj$dir2$path),2],
                                 lwd=bwd*cex.bwd.2,col=col[run],lty=lty[run],...)
            }
        } # end else
    } # end for

} # end function



#####################
# print function
#####################
print.monmonier <- function(x, ...){
cat("\t\n###########################################################")
cat("\t\n# List of paths of maximum differences between neighbours #")
cat("\t\n#           Using a Monmonier based algorithm             #")
cat("\t\n###########################################################\n")
cat('\n$call:')
print(x$call)

cat('\n      # Object content #')
cat("\nClass: ", class(x))
cat('\n$nrun (number of successive runs): ', x$nrun)
if(x$nrun==1)
cat('\n$run1: run of the algorithm')
else if(x$nrun==2)
cat('\n$run1, $run2: runs of the algorithm')
else cat('\n$run1 ... $run',x$nrun, ': runs of the algorithm',sep='')
cat('\n$threshold (minimum difference between neighbours): ', x$threshold)
cat("\n$xy: spatial coordinates")
cat("\n$cn: connection network")

cat('\n\n      # Runs content #')
for(i in 1:x$nrun){
	cat('\n# Run',i)
        # dir 1 #
        cat('\n# First direction')
        cat('\nClass: ', class(x$run1$dir1))
	cat('\n$path:\n')
	print(head(x[[i]]$dir1$path,n=3))
	if(nrow(x[[i]]$dir1$path) >3) cat('...\n')
	cat('\n$values:\n',head(x[[i]]$dir1$values,n=3))
	if(length(x[[i]]$dir1$values)>3) cat(' ...')
        # dir 2 #
        cat('\n# Second direction')
        cat('\nClass: ', class(x$run1$dir2))
	cat('\n$path:\n')
	print(head(x[[i]]$dir2$path,n=3))
	if(nrow(x[[i]]$dir2$path) >3) cat('...\n')
	cat('\n$values:\n',head(x[[i]]$dir2$values,n=3))
	if(length(x[[i]]$dir2$values)>3) cat(' ...')

	cat('\n')
	lenTheo <- x$nrun + 5
	if(length(names(x))> lenTheo) {
		cat('Other elements: \n')
		cat(names(x)[(lenTheo+1) : length(x)])
		}
	cat('\n')
	}
}



##############################
# function optimize.monmonier
##############################
optimize.monmonier <- function(xy,dist,cn,ntry=10, bd.length=NULL, return.best=TRUE,
                               display.graph=TRUE,threshold=NULL,scanthres=is.null(threshold),allowLoop=TRUE){

## move X to the arguments if we want to optimize an already created object
X <- NULL
#if( any(is.null(xy), is.null(dist),  is.null(cn)) & is.null(X) ) stop("Please provide either xy, dist and cn or a monmonier object (X)")

## if X is a monmonier object...
if(inherits(X,what="monmonier")){
  obj <- as.list(X$call)
  xy <- obj$xy
  dist <- obj$dist
  cn <- obj$cn
}

cn.nb <- cn
cn <- nb2neig(cn)
M <- neig2mat(cn)
D <- as.matrix(dist)
D <- M*D
if(is.null(threshold) || (threshold<0)) {Dlim <- summary(unique(D[D>0]))[5]} else {Dlim <- threshold}

if(scanthres){
  plot(sort(unique(D)[unique(D) > 0],decreasing=TRUE),main="Local distances plot", type="l", xlab="rank",ylab="Sorted local distances")
  abline(h=Dlim,lty=2)
  mtext("Dashed line indicates present threshold")
  cat("Indicate the threshold (\'d\' for default): ")
  temp <- as.character(readLines(con = getOption('adegenet.testcon'), n = 1))
  if(toupper(temp)!="D") { Dlim <- as.numeric(temp) }
}

## start the series if computations
cat(paste("Boundaries computed (required: ",ntry,")\n",sep=""))

## for loop
bdr.values <- -1 # used so that the first boundary is automatically the best

for(i in 0:(ntry-1)){
  temp <- monmonier(xy, dist, cn.nb,skip.local.diff=i,scanthres=FALSE,
                            threshold=Dlim, bd.length=bd.length, allowLoop=allowLoop)
  current.bdr.value <- sum(c(temp$run1$dir1$values, temp$run1$dir2$values))

  if(all(current.bdr.value > bdr.values)) {
      bdr.best <- temp
      bdr.skip <- i
  }

  bdr.values <- c(bdr.values , current.bdr.value)
  cat(paste(1+i," ")) # print progression in live
}

## remove the first value (-1) in bdr.value
bdr.values <- bdr.values[-1]

## graphical display
if(display.graph) barplot(bdr.values,xlab="Local differences skipped",ylab="Sum of all local differences",names.arg=0:(ntry-1))

## return the best value of skip.local.diff, or the corresponding object
if(!return.best) {
  return(bdr.skip)
  } else {
    cat(paste("\nOptimal number of skipped local differences: ",bdr.skip,"\n"))
    prevcall <- as.list(match.call())
    newcall <- bquote( monmonier(xy=.(prevcall$xy), dist=.(prevcall$dist), cn=.(prevcall$cn),
                                 threshold=.(bdr.best$threshold), bd.length=.(bd.length),
                                 nrun=1, skip.local.diff=.(as.numeric(bdr.skip)), scanthres=FALSE, allowLoop=.(allowLoop)) )

    ## assign the appropriate call to the result
    bdr.best$call <- newcall

    ## exp <- bquote( monmonier(xy=.(prevcall$xy),dist=.(prevcall$dist),cn=.(prevcall$cn),skip=.(bdr.skip),
    ##                          ,scan=FALSE,thres=.(Dlim),bd.length=.(bd.length),allowLoop=.(allowLoop)) )
    return(bdr.best)
    }
}

# Function to subset raw vectors
.subsetbin <- function(x, i){
    # Take a raw vector, subset the bits and then convert to integers.
    xint   <- as.integer(rawToBits(x)[i])
    # Figure out how many zeroes are needed to pad the end.
    zeroes <- 8 - (length(xint) %% 8)
    # Convert the integer vector with zeroes on the end back into a raw vector.
    return(packBits(c(xint, rep(0L, zeroes))))
}

# old method for [] for SNPbin
.oldSNPbinset <- function(x, i){
    if (missing(i)) i <- TRUE
    temp <- .SNPbin2int(x) # data as integers with NAs
    x <- new("SNPbin", snp=temp[i], label=x@label, ploidy=x@ploidy)
    return(x)
}

# Zhian N. Kamvar
# Mon Aug 17 09:39:12 2015 ------------------------------
# 
# This function takes two steps:
#   1. Subset the missing positions
#   2. Subset the vectors of raw SNPs
# 
# Both steps are not exactly straighforward. Because the missing vector only
# represents the positions of missing data, it must be subset by value as
# opposed to position.
.SNPbinset <- function(x, i){
  if (missing(i)) i <- TRUE
  
  # Create a logical value indicating whether or not subsetting is necessary.
  we_take_all <- length(i) == 1 && is.logical(i) && i
  n.loc <- x@n.loc
  if (length(x@NA.posi) > 0){
    if (is.logical(i)){
      if (we_take_all){
        # Keep all of the data
        return(x)
      } else {
        # If the positons are logical, perhaps the best way to address this is
        # to match the TRUE positions to the NA.posi vector. Adding nomatch = 0 
        # avoids introducing NAs.
        namatches <- match(which(i), x@NA.posi, nomatch = 0)
        nas.kept  <- x@NA.posi[namatches]
      }
    } else if (is.character(i)){
      stop("Cannot subset a SNPbin object with a character vector", call. = FALSE)
    } else if (all(i < 0)){
      # For negative subscripts, find which ones they match and then
      # negate those. Luckily -0 is allowed.
      namatches <- match(abs(i), x@NA.posi, nomatch = 0)
      # Unfortunately, if nothing matches, then the default are zeroes. When you
      # subset a vector in R with only zero, you will get an empty vector. This
      # conditional makes sure that NA positions are retained.
      if (all(namatches == 0)){
        nas.kept  <- x@NA.posi
      } else {
        nas.kept  <- x@NA.posi[-namatches]
      }
      
    } else if (all(i > 0)){
      # Positive subscripts are much easier. First you find where the subscripts
      # match and then your subset with those positions. 
      namatches <- match(i, x@NA.posi, nomatch = 0)
      nas.kept  <- x@NA.posi[namatches]
    } else {
      stop("Cannot subset a SNPbin with mixed subscripts.", call. = FALSE)
    }
    # After we find out which missing positions we need to keep, we reset the 
    # missing positions to the subset data.
    if (length(nas.kept) > 0){
      old.posi  <- 1:n.loc
      x@NA.posi <- match(nas.kept, old.posi[i])
    } else {
      x@NA.posi <- nas.kept
    }
  }
  # Here we calculate the number of loci we will have left in the data.
  if (we_take_all){
    return(x)
  } else if (all(is.logical(i))){
    n.loc <- sum(i)
  } else if (any(i < 0)){
    n.loc <- n.loc - length(i)
  } else {
    n.loc <- length(i)
  }
  # Now we loop over all chromosomes and subset.
  x@snp   <- lapply(x@snp, .subsetbin, i)
  # Set the new value of the number of loci and return.
  x@n.loc <- n.loc
  return(x)
}

###############
## '[' operators
###############
## SNPbin

setMethod("[", signature(x="SNPbin", i="ANY"), function(x, i) {
    .SNPbinset(x, i)
}) # end [] for SNPbin




## genlight
setMethod("[", signature(x = "genlight", i = "ANY", j = "ANY", drop = "ANY"),
          function(x, i, j, ..., treatOther = TRUE, quiet = TRUE, drop = FALSE) {
    if (missing(i)) i <- TRUE
    if (missing(j)) j <- TRUE

    ori.n <- nInd(x)
    ori.p <- nLoc(x)

    ## recycle logicals if needed
    if(!is.null(i) && is.logical(i)) i <- rep(i, length=ori.n)
    if(!is.null(j) && is.logical(j)) j <- rep(j, length=ori.p)


    ## SUBSET INDIVIDUALS ##
    ## genotypes
    x@gen <- x@gen[i]

    ## ind names
    x@ind.names <- x@ind.names[i]

    ## ploidy
    if(!is.null(x@ploidy)) {
        ori.ploidy <- ploidy(x) <- ploidy(x)[i]
    } else {
        ori.ploidy <- NULL
    }

    ## pop
    if(!is.null(pop(x))) {
        ori.pop <- pop(x) <- factor(pop(x)[i])
    } else {
        ori.pop <- NULL
    }
    ## strata
    if(!is.null(x@strata)) {
        ori.strata <- x@strata <- x@strata[i, , drop = FALSE]
    } else {
        ori.strata <- NULL
    }


    ## HANDLE 'OTHER' SLOT ##
    nOther <- length(other(x))
    namesOther <- names(other(x))
    counter <- 0
    if(treatOther & !(is.logical(i) && all(i))){
        f1 <- function(obj,n=ori.n){
            counter <<- counter+1
            if(!is.null(dim(obj)) && nrow(obj)==ori.n) { # if the element is a matrix-like obj
                obj <- obj[i,,drop=FALSE]
            } else if(length(obj) == ori.n) { # if the element is not a matrix but has a length == n
                obj <- obj[i]
                if(is.factor(obj)) {obj <- factor(obj)}
            } else {if(!quiet) warning(paste("cannot treat the object",namesOther[counter]))}

            return(obj)
        } # end f1

        other(x) <- lapply(x@other, f1) # treat all elements

    } # end treatOther


    ## SUBSET LOCI ##

    ## handle ind.names, loc.names, chromosome, position, and alleles
    if (is.character(j)){
      j <- match(j, x@loc.names, nomatch = 0)
    }
    x@loc.names   <- x@loc.names[j]
    x@chromosome  <- chr(x)[j]
    x@position    <- position(x)[j]
    x@loc.all     <- alleles(x)[j]
    x@gen         <- lapply(x@gen, function(e) e[j])
    x@n.loc       <- x@gen[[1]]@n.loc

    return(x)
}) # end [] for genlight







######################
##
## c, cbind, rbind...
##
######################

################
## cbind SNPbin
################
##setMethod("cbind", signature("SNPbin"), function(..., deparse.level = 1) {
cbind.SNPbin <- function(..., checkPloidy=TRUE){
    myList <- list(...)
    if(!all(sapply(myList, class)=="SNPbin")) stop("some objects are not SNPbin objects")
    ## remove empty objects
    myList <- myList[sapply(myList,nLoc)>0]
    if(length(myList)==0) {
        warning("All objects are empty")
        return(NULL)
    }


    if(checkPloidy && length(unique(sapply(myList, ploidy))) !=1 ) stop("objects have different ploidy levels")
    if(checkPloidy) {
        ori.ploidy <- ploidy(myList[[1]])
    } else {
        ori.ploidy <- NULL
    }
    x <- new("SNPbin", unlist(lapply(myList, as.integer)), ploidy=ori.ploidy)
    return(x)
} # end cbind.SNPbin
##})



c.SNPbin <- function(...){
    return(cbind(...))
}




##################
## cbind genlight
##################
##setMethod("cbind", signature(x="genlight"), function(..., deparse.level = 1) {
cbind.genlight <- function(...){
      ## store arguments
    dots <- list(...)

    ## extract arguments which are genlight objects
    myList <- dots[sapply(dots, inherits, "genlight")]

    ## keep the rest in 'dots'
    dots <- dots[!sapply(dots, inherits, "genlight")]

    if(length(myList)==1 && is.list(myList[[1]])) myList <- myList[[1]]
    if(!all(sapply(myList, class)=="genlight")) stop("some objects are not genlight objects")
    ## remove empty objects
    myList <- myList[sapply(myList,nLoc)>0 & sapply(myList,nInd)>0]
    if(length(myList)==0) {
        warning("All objects are empty")
        return(NULL)
    }

    ## different checks
    if(length(unique(sapply(myList, nInd))) > 1 ) stop("objects have different numbers of individuals")
    n.obj <- length(myList)
    n.ind <- nInd(myList[[1]])
    if(n.ind==0){
        warning("All objects are empty")
        return(NULL)
    }
    temp <- as.matrix(as.data.frame(lapply(myList, ploidy)))
    if(any(apply(temp,1,function(r) length(unique(r)))>1)) stop("non-consistent ploidy across datasets")
    ori.ploidy <- ploidy(myList[[1]])


    ## merge one individual at a time ##
    res <- list()
    for(i in 1:n.ind){
        res[[i]] <- Reduce(function(a,b) {cbind(a,b,checkPloidy=FALSE)}, lapply(myList, function(e) e@gen[[i]]) )
    }

    dots$gen <- res
    dots$Class <- "genlight"
    res <- do.call(new, dots)

    ## handle loc.names, alleles, etc. ##
    indNames(res) <- indNames(myList[[1]])
    locNames(res) <- unlist(lapply(myList, locNames))
    alleles(res) <- unlist(lapply(myList, alleles))
    pop(res) <- pop(myList[[1]])
    res@strata <- myList[[1]]@strata
    ploidy(res) <- ori.ploidy

    ## return object ##
    return(res)
} # end cbind.genlight
##})






##################
## rbind genlight
##################
##setMethod("cbind", signature(x="genlight"), function(..., deparse.level = 1) {
#' @importFrom dplyr bind_rows
rbind.genlight <- function(...){
    ## store arguments
    dots <- list(...)

    ## extract arguments which are genlight objects
    myList <- dots[sapply(dots, inherits, "genlight")]

    ## keep the rest in 'dots'
    dots <- dots[!sapply(dots, inherits, "genlight")]

    if(!all(sapply(myList, class)=="genlight")) stop("some objects are not genlight objects")

    ## remove empty objects
    myList <- myList[sapply(myList,nLoc)>0 & sapply(myList,nInd)>0]
    if(length(myList)==0) {
        warning("All objects are empty")
        return(NULL)
    }

    if(length(unique(sapply(myList, nLoc))) !=1 ) stop("objects have different numbers of SNPs")

    ## build output
    dots$Class <- "genlight"
    dots$gen <- Reduce(c, lapply(myList, function(e) e@gen))
    res <- do.call(new, dots)
    locNames(res) <- locNames(myList[[1]])
    alleles(res)  <- alleles(myList[[1]])
    indNames(res) <- unlist(lapply(myList, indNames))
    pop(res)      <- factor(unlist(lapply(myList, pop)))

    # Hierarchies are tricky. Using dplyr's bind_rows.
    res <- .rbind_strata(myList, res)

    ## return object ##
    return(res)

} # end rbind.genlight






##########
## seppop
##########
setMethod("seppop", signature(x="genlight"), function(x, pop=NULL, treatOther=TRUE, quiet=TRUE, ...){
    ## HANDLE POP ARGUMENT ##
    if(!is.null(pop)) {
      if (is.language(pop)){
        setPop(x) <- pop
      } else {
        pop(x) <- pop
      }
    }

    if(is.null(pop(x))) stop("pop not provided and pop(x) is NULL")

    ## PERFORM SUBSETTING ##
    kObj <- lapply(levels(pop(x)), function(lev) x[pop(x)==lev, , treatOther=treatOther, quiet=quiet, ...])
    names(kObj) <- levels(pop(x))

    return(kObj)
}) # end seppop






##########
## seploc
##########
setMethod("seploc", signature(x="genlight"), function(x, n.block=NULL, block.size=NULL, random=FALSE,
                               parallel=FALSE, n.cores=NULL){
    ## CHECKS ##
    if(is.null(n.block) & is.null(block.size)) stop("n.block and block.size are both missing.")
    if(!is.null(n.block) & !is.null(block.size)) stop("n.block and block.size are both provided.")
    if(parallel && !require(parallel)) stop("parallel package requested but not installed")
    if(parallel && is.null(n.cores)){
        n.cores <- parallel::detectCores()
    }


    ## GET BLOCK SIZE VECTOR ##
    P <- nLoc(x)

    ## n.block is given
    if(!is.null(n.block)){
        vec.blocksize <- rep(P %/% n.block, n.block)
        if(P %% n.block >0){
            vec.blocksize[1:(P %% n.block)] <- vec.blocksize[1:(P %% n.block)] + 1
        }

    }

    ## block.size is given
    if(!is.null(block.size)){
        vec.blocksize <- rep(block.size, P %/% block.size)
        if(P %% block.size >0){
             vec.blocksize <- c( vec.blocksize, P %% block.size)
        }
    }


    ## split data by blocks ##
    fac.block <- factor(rep(1:length(vec.blocksize), vec.blocksize))
    if(random){
        fac.block <- sample(fac.block)
    }

    if(parallel){
        if(random){
            res <- parallel::mclapply(levels(fac.block), function(lev) x[, sample(which(fac.block==lev))],
                        mc.cores=n.cores, mc.silent=TRUE, mc.cleanup=TRUE, mc.preschedule=FALSE)
        } else {
            res <- parallel::mclapply(levels(fac.block), function(lev) x[, which(fac.block==lev)],
                        mc.cores=n.cores, mc.silent=TRUE, mc.cleanup=TRUE, mc.preschedule=FALSE)
        }
    } else {
         if(random){
             res <- lapply(levels(fac.block), function(lev) x[, sample(which(fac.block==lev))])
         } else {
             res <- lapply(levels(fac.block), function(lev) x[, which(fac.block==lev)])
         }
    }

    ## return result ##
    names(res) <- paste("block", 1:length(res),sep=".")

    return(res)
}) # end seploc






###################
### TESTING
###################


## c, cbind, rbind ##
## a <- new("genlight", list(c(1,0,1), c(0,0,1,0)) )
## b <- new("genlight", list(c(1,0,1,1,1,1), c(1,0)) )
## locNames(a) <- letters[1:4]
## locNames(b) <- 1:6
## c <- cbind(a,b)
## identical(as.matrix(c),cbind(as.matrix(a), as.matrix(b))) # MUST BE TRUE
## identical(as.matrix(rbind(a,a)),rbind(as.matrix(a),as.matrix(a)))




## test subsetting with/without @other ##
## x <- new("genlight", list(a=1,b=0,c=1), other=list(1:3, letters, data.frame(2:4)))
## pop(x) <- c("pop1","pop1", "pop2")

###############################################
##
## spatial Principal Components Analysis
##
## require ade4, spdep and eventually tripack
##
## generic functions were derived from
## those of multispati class (ade4)
##
## T. Jombart (t.jombart@imperial.ac.uk)
## 31 may 2007
##############################################

spca <- function (...) UseMethod("spca")



################
## spca.default
################
spca.default <- function(x, ...) {
  stop(sprintf("No spca method for object of class %s",
               paste(class(x), collapse = " ")))
}





###############
## spca.matrix
###############
spca.matrix <- function(x, xy = NULL, cn = NULL, matWeight = NULL,
                        center = TRUE, scale = FALSE, scannf = TRUE,
                        nfposi = 1, nfnega = 1,
                        type = NULL, ask = TRUE,
                        plot.nb = TRUE, edit.nb = FALSE,
                        truenames = TRUE,
                        d1 = NULL, d2 = NULL, k = NULL,
                        a = NULL, dmin = NULL, ...) {

  ## check type of x: only numeric values are acceptable

  if (!is.numeric(x)) {
    stop("Only matrices of numeric values are accepted.")
  }


  ## check and handle xy coordinates

  if(is.null(xy) & (inherits(cn,"nb") & !inherits(cn,"listw")) ){
    xy <- attr(cn,"xy")  # xy can be retrieved from a nb object (not from listw)
  }
  if(is.null(xy)) {
    stop("xy coordinates are not provided")
  }
  if(is.data.frame(xy)) {
    xy <- as.matrix(xy)
  }
  if(!is.matrix(xy)) {
    stop("provided 'xy' cannot be converted to matrix")
  }
  if(ncol(xy) != 2) {
    stop("xy does not have two columns.")
  }
  if(nrow(xy) != nrow(x)) {
    stop("x and xy must have the same row numbers.")
  }


  ## find the spatial weights

  resCN <- NULL

  ## connection network from matWeight

  if (!is.null(matWeight)) {
    if (!is.matrix(matWeight)) {
      stop("matWeight is not a matrix")
    }
    if (!is.numeric(matWeight)) {
      stop("matWeight is not numeric")
    }
    if (nrow(matWeight) != ncol(matWeight)) {
      stop("matWeight is not square")
    }
    if (nrow(matWeight) != nrow(x)) {
      stop("dimension of datWeight does not match data")
    }
    diag(matWeight) <- 0
    matWeight <- prop.table(matWeight, 1)
    resCN <- mat2listw(matWeight)
    resCN$style <- "W"

  }

  ## connection network from cn argument

  if(is.null(resCN) & !is.null(cn)) {
    if(inherits(cn,"nb")) {
      if(!inherits(cn,"listw")){ # cn is a 'pure' nb object (i.e., nb but not listw)
        cn <- nb2listw(cn, style="W", zero.policy=TRUE)
      }
      resCN <- cn
    } else {
      stop("cn does not have a recognized class")
    }
  }


  ## connection network from xy coordinates
  if(is.null(resCN)) {
    resCN <- chooseCN(xy=xy, ask=ask, type=type, plot.nb=plot.nb, edit.nb=edit.nb,
                      result.type="listw", d1=d1, d2=d2, k=k, a=a, dmin=dmin)
  }


  ## perform the analyses: basic PCA followed by multispati

  x_pca <- ade4::dudi.pca(x, center = center, scale = scale, scannf = FALSE)

  out <- ade4::multispati(dudi = x_pca, listw = resCN, scannf = scannf,
                          nfposi = nfposi, nfnega = nfnega)

  nfposi <- out$nfposi
  nfnega <- out$nfnega

  out$tab <- x_pca$tab
  out$xy <- xy
  rownames(out$xy) <- rownames(out$li)
  colnames(out$xy) <- c("x","y")

  out$lw <- resCN

  dots <- list(...)
  if (!is.null(dots$call)) {
    out$call <- dots$call
  } else {
    out$call <- match.call()
  }

  posaxes <- if (nfposi > 0) {1:nfposi} else NULL
  negaxes <- if (nfnega > 0) {(length(out$eig)-nfnega+1):length(out$eig)} else NULL
  keptaxes <- c(posaxes, negaxes)

  ## set names of different components
  colnames(out$c1) <- paste("Axis",keptaxes)
  colnames(out$li) <- paste("Axis",keptaxes)
  colnames(out$ls) <- paste("Axis",keptaxes)
  row.names(out$c1) <- colnames(x)
  colnames(out$as) <- colnames(out$c1)
  temp <- row.names(out$as)
  row.names(out$as) <- paste("PCA", temp)

  class(out) <- "spca"

  return(out)

}






###################
## spca.data.frame
###################

spca.data.frame <- function(x, xy = NULL, cn = NULL, matWeight = NULL,
                            center = TRUE, scale = FALSE, scannf = TRUE,
                            nfposi = 1, nfnega = 1,
                            type = NULL, ask = TRUE,
                            plot.nb = TRUE, edit.nb = FALSE,
                            truenames = TRUE,
                            d1 = NULL, d2 = NULL, k = NULL,
                            a = NULL, dmin = NULL, ...) {

  call <- match.call()

  spca(as.matrix(x), xy = xy, cn = cn, matWeight = matWeight, center = center,
       cale = scale, scannf = scannf, nfposi = nfposi, nfnega = nfnega,
       type = type, ask = ask, plot.nb = plot.nb, edit.nb = edit.nb,
       truenames = truenames, d1 = d1, d2 = d2, k = k, a = a, dmin = dmin,
       call = call, ...)
}






################
## spca genind
################

spca.genind <- function(obj, xy = NULL, cn = NULL, matWeight = NULL,
                        scale = FALSE, scannf = TRUE,
                        nfposi = 1, nfnega = 1,
                        type = NULL, ask = TRUE,
                        plot.nb = TRUE, edit.nb = FALSE,
                        truenames = TRUE,
                        d1 = NULL, d2 = NULL, k = NULL,
                        a = NULL, dmin = NULL, ...){

  ## first checks

  invisible(validObject(obj))


  ## handle xy coordinates
  if(is.null(xy) & !is.null(obj$other$xy)) {
    xy <- obj$other$xy # xy from @other$xy if it exists
  }



  ## == spatial weights are done ==


  ## handle NAs warning
  if(any(is.na(obj@tab))){
    warning("NAs in data are automatically replaced (to mean allele frequency)")
  }

  ## handle NAs, centring and scaling
  X <- tab(obj, freq = TRUE, NA.method = "mean")

  call <- match.call()

  spca(X, xy = xy, cn = cn, matWeight = matWeight,
       center = TRUE, scale = scale, scannf = scannf,
       nfposi = nfposi, nfnega = nfnega,
       type = type, ask = ask,
       plot.nb = plot.nb, edit.nb = edit.nb,
       truenames = truenames,
       d1 = d1, d2 = d2, k = k,
       a = a, dmin = dmin,
       call = call, ...)

} # end spca.genind






################
## spca genpop
################

spca.genpop <- function(obj, xy = NULL, cn = NULL, matWeight = NULL,
                        scale = FALSE, scannf = TRUE,
                        nfposi = 1, nfnega = 1,
                        type = NULL, ask = TRUE,
                        plot.nb = TRUE, edit.nb = FALSE,
                        truenames = TRUE,
                        d1 = NULL, d2 = NULL, k = NULL,
                        a = NULL, dmin = NULL, ...){

  ## first checks

  invisible(validObject(obj))


  ## handle xy coordinates
  if(is.null(xy) & !is.null(obj$other$xy)) {
    xy <- obj$other$xy # xy from @other$xy if it exists
  }


  ## handle NAs warning
  if(any(is.na(obj@tab))){
    warning("NAs in data are automatically replaced (to mean allele frequency)")
  }

  ## handle NAs, centring and scaling
  X <- tab(obj, freq = TRUE, NA.method = "mean")

  call <- match.call()

  spca(X, xy = xy, cn = cn, matWeight = matWeight,
       center = TRUE, scale = scale, scannf = scannf,
       nfposi = nfposi, nfnega = nfnega,
       type = type, ask = ask,
       plot.nb = plot.nb, edit.nb = edit.nb,
       truenames = truenames,
       d1 = d1, d2 = d2, k = k,
       a = a, dmin = dmin,
       call = call, ...)

} # end spca.genpop






######################
## Function print.spca
######################

print.spca <- function(x, ...){
  cat("\t########################################\n")
  cat("\t# spatial Principal Component Analysis #\n")
  cat("\t########################################\n")
  cat("class: ")
  cat(class(x))
  cat("\n$call: ")
  print(x$call)
  cat("\n$nfposi:", x$nfposi, "axis-components saved")
  cat("\n$nfnega:", x$nfnega, "axis-components saved")

  cat("\nPositive eigenvalues: ")
  l0 <- sum(x$eig >= 0)
  cat(signif(x$eig, 4)[1:(min(5, l0))])
  if (l0 > 5)
    cat(" ...\n")
  else cat("\n")
  cat("Negative eigenvalues: ")
  l0 <- sum(x$eig <= 0)
  cat(sort(signif(x$eig, 4))[1:(min(5, l0))])
  if (l0 > 5)
    cat(" ...\n")
  else cat("\n")
  cat('\n')
  sumry <- array("", c(1, 4), list(1, c("vector", "length",
                                        "mode", "content")))
  sumry[1, ] <- c('$eig', length(x$eig), mode(x$eig), 'eigenvalues')
  class(sumry) <- "table"
  print(sumry)
  cat("\n")
  sumry <- array("", c(5, 4),
                 list(1:5, c("data.frame", "nrow", "ncol", "content")))
  sumry[1, ] <- c("$tab", nrow(x$tab), ncol(x$tab),
                  "transformed data: optionally centred / scaled")
  sumry[2, ] <- c("$c1", nrow(x$c1), ncol(x$c1),
                  "principal axes: scaled vectors of alleles loadings")
  sumry[3, ] <- c("$li", nrow(x$li), ncol(x$li),
                  "principal components: coordinates of entities ('scores')")
  sumry[4, ] <- c("$ls", nrow(x$ls), ncol(x$ls),
                  "lag vector of principal components")
  sumry[5, ] <- c("$as", nrow(x$as), ncol(x$as),
                  "pca axes onto spca axes")

  class(sumry) <- "table"
  print(sumry)

  cat("\n$xy: matrix of spatial coordinates")
  cat("\n$lw: a list of spatial weights (class 'listw')")

  cat("\n\nother elements: ")
  if (length(names(x)) > 10)
    cat(names(x)[11:(length(names(x)))], "\n")
  else cat("NULL\n")
}





########################
## Function summary.spca
########################
summary.spca <- function (object, ..., printres=TRUE) {
  if (!inherits(object, "spca"))stop("to be used with 'spca' object")

  #util <- function(n) { ## no longer used
  #  x <- "1"
  #  for (i in 2:n) x[i] <- paste(x[i - 1], i, sep = "+")
  #  return(x)
  #}
  norm.w <- function(X, w) {
    f2 <- function(v) sum(v * v * w)/sum(w)
    norm <- apply(X, 2, f2)
    return(norm)
  }

  resfin <- list()

  if(printres) {
    cat("\nSpatial principal component analysis\n")
    cat("\nCall: ")
    print(object$call)
  }

  appel <- as.list(object$call)
  ## compute original pca
  # prepare data
  obj <- eval(appel$obj)
  if(is.null(appel$truenames)) appel$truenames <- FALSE

  f1 <- function(vec){
    m <- mean(vec,na.rm=TRUE)
    vec[is.na(vec)] <- m
    return(vec)
  }

  if(is.genind(obj)) { X <- obj@tab }
  if(is.genpop(obj)) { X <- makefreq(obj, quiet=TRUE) }

  X <- apply(X,2,f1)

  nfposi <- object$nfposi
  nfnega <- object$nfnega

  dudi <- dudi.pca(X, center=TRUE, scale=FALSE, scannf=FALSE, nf=nfposi+nfnega)
  ## end of pca

  lw <- object$lw

  # I0, Imin, Imax
  n <- nrow(X)
  I0 <- -1/(n-1)
  L <- listw2mat(lw)
  ## use 'as.numeric' to avoid possible bug with large matrices,
  ## returning complex numbers with zero imaginary parts
  eigL <- suppressWarnings(as.numeric(eigen(0.5*(L+t(L)))$values))
  Imin <- min(eigL)
  Imax <- max(eigL)
  Ival <- data.frame(I0=I0,Imin=Imin,Imax=Imax)
  row.names(Ival) <- ""
  if(printres) {
    cat("\nConnection network statistics:\n")
    print(Ival)
  }

  Istat <- c(I0,Imin,Imax)
  names(Istat) <- c("I0","Imin","Imax")
  resfin$Istat <- Istat


  # les scores de l'analyse de base
  nf <- dudi$nf
  eig <- dudi$eig[1:nf]
  cum <- cumsum(dudi$eig)[1:nf]
  ratio <- cum/sum(dudi$eig)
  w <- apply(dudi$l1,2,lag.listw,x=lw)
  moran <- apply(w*as.matrix(dudi$l1)*dudi$lw,2,sum)
  res <- data.frame(var=eig,cum=cum,ratio=ratio, moran=moran)
  row.names(res) <- paste("Axis",1:nf)
  if(printres) {
    cat("\nScores from the centred PCA\n")
    print(res)
  }

  resfin$pca <- res


  # les scores de l'analyse spatiale
  # on recalcule l'objet en gardant tous les axes
  eig <- object$eig
  nfposimax <- sum(eig > 0)
  nfnegamax <- sum(eig < 0)

  ms <- multispati(dudi=dudi, listw=lw, scannf=FALSE,
                   nfposi=nfposimax, nfnega=nfnegamax)

  ndim <- dudi$rank
  nf <- nfposi + nfnega
  agarder <- c(1:nfposi,if (nfnega>0) (ndim-nfnega+1):ndim)
  varspa <- norm.w(ms$li,dudi$lw)
  moran <- apply(as.matrix(ms$li)*as.matrix(ms$ls)*dudi$lw,2,sum)
  res <- data.frame(eig=eig,var=varspa,moran=moran/varspa)
  row.names(res) <- paste("Axis",1:length(eig))

  if(printres) {
    cat("\nsPCA eigenvalues decomposition:\n")
    print(res[agarder,])
  }

  resfin$spca <- res

  return(invisible(resfin))
}



#####################
## Function plot.spca
#####################
plot.spca <- function (x, axis = 1, useLag=FALSE, ...){
  if (!inherits(x, "spca")) stop("Use only with 'spca' objects.")

  if(axis>ncol(x$li)) stop("wrong axis required.")

  opar <- par(no.readonly = TRUE)
  on.exit(par(opar))
  par(mar = rep(.1,4), mfrow=c(3,2))

  n <- nrow(x$li)
  xy <- x$xy

  ## handle useLag argument
  if(useLag){
    z <- x$ls[,axis]
  } else {
    z <- x$li[,axis]
  } # end if useLag
  nfposi <- x$nfposi
  nfnega <- x$nfnega
  ## handle neig parameter - hide cn if nore than 100 links
  nLinks <- sum(card(x$lw$neighbours))
  if(nLinks < 500) {
    neig <- nb2neig(x$lw$neighbours)
  } else {
    neig <- NULL
  }

  sub <- paste("Score",axis)
  csub <- 2

  # 1
  if(n<30) clab <- 1 else clab <- 0
  s.label(xy, clabel=clab, include.origin=FALSE, addaxes=FALSE, neig=neig,
          cneig=1, sub="Connection network", csub=2)

  # 2
  s.image(xy,z, include.origin=FALSE, grid=TRUE, kgrid=10, cgrid=1,
          sub=sub, csub=csub, possub="bottomleft")
  box()

  # 3
  if(n<30) {neig <- nb2neig(x$lw$neighbours)} else {neig <- NULL}
  s.value(xy,z, include.origin=FALSE, addaxes=FALSE, clegend=0, csize=.6,
          neig=neig, sub=sub, csub=csub, possub="bottomleft")

  # 4
  s.value(xy,z, include.origin=FALSE, addaxes=FALSE, clegend=0, csize=.6,
          method="greylevel", neig=neig, sub=sub, csub=csub, possub="bottomleft")

  # 5
  omar <- par("mar")
  par(mar = c(0.8, 2.8, 0.8, 0.8))
  m <- length(x$eig)
  col.w <- rep("white", m) # elles sont toutes blanches
  col.w[1:nfposi] <- "grey"
  if (nfnega>0) {col.w[m:(m-nfnega+1)] <- "grey"}
  j <- axis
  if (j>nfposi) {j <- j-nfposi +m -nfnega}
  col.w[j] <- "black"
  barplot(x$eig, col = col.w)
  scatterutil.sub(cha ="Eigenvalues", csub = 2.5, possub = "topright")
  par(mar=rep(.1,4))
  box()
  par(mar=omar)

  # 6
  par(mar=c(4,4,2,1))
  screeplot(x,main="Eigenvalues decomposition")
  par(mar=rep(.1,4))
  box()
  return(invisible(match.call()))
}



##########################
## Function screeplot.spca
##########################
screeplot.spca <- function(x,...,main=NULL){

  opar <- par("las")
  on.exit(par(las=opar))

  sumry <- summary(x,printres=FALSE)

  labels <- lapply(1:length(x$eig),function(i) bquote(lambda[.(i)]))

  par(las=1)

  xmax <- sumry$pca[1,1]*1.1
  I0 <- sumry$Istat[1]
  Imin <- sumry$Istat[2]
  Imax <- sumry$Istat[3]

  plot(x=sumry$spca[,2],y=sumry$spca[,3],type='n',xlab='Variance',ylab="Spatial autocorrelation (I)",xlim=c(0,xmax),ylim=c(Imin*1.1,Imax*1.1),yaxt='n',...)
  text(x=sumry$spca[,2],y=sumry$spca[,3],do.call(expression,labels))

  ytick <- c(I0,round(seq(Imin,Imax,le=5),1))
  ytlab <- as.character(round(seq(Imin,Imax,le=5),1))
  ytlab <- c(as.character(round(I0,1)),as.character(round(Imin,1)),ytlab[2:4],as.character(round(Imax,1)))
  axis(side=2,at=ytick,labels=ytlab)

  rect(0,Imin,xmax,Imax,lty=2)
  segments(0,I0,xmax,I0,lty=2)
  abline(v=0)

  if(is.null(main)) main <- ("Spatial and variance components of the eigenvalues")
  title(main)

  return(invisible(match.call()))
}





###################
## colorplot method
###################
colorplot.spca <- function(x, axes=1:ncol(x$li), useLag=FALSE, ...){
  ## some checks
  if(!any(inherits(x,"spca"))) stop("x in not a spca object.")

  ## get args to be passed to colorplot
  xy <- x$xy

  if(useLag) {
    X <- as.matrix(x$ls)
  } else {
    X <- as.matrix(x$li)
  }

  ## call to colorplot
  colorplot(xy, X, axes, ...)

} # end colorplot.spca

## THIS CODE HAS BEEN TAKEN FROM ADE4
## IT HAS BEEN MOVED TO ADESPATIAL
## THIS IS MERELY A TEMPORARY FIX.

## only orthobasis.listw is used.
## renamed into .orthobasis.listw to avoid having to document it.

## - Thibaut Jombart, April 2015

#######################################################

.orthobasis.listw <- function( listw) {
    appel = match.call()
    if(!inherits(listw,"listw")) stop ("object of class 'listw' expected")
    if(listw$style!="W") stop ("object of class 'listw' with style 'W' expected")
    n = length(listw$weights)
    fun <- function (x) {
        num = listw$neighbours[[x]]
        wei = listw$weights[[x]]
        res = rep(0,n)
        res[num] = wei
        return (res)
    }
    b0 <- matrix(unlist(lapply(1:n,fun)),n,n)
    b0=(t(b0)+b0)/2
    b0=bicenter.wt(b0)
    a0 <- eigen(b0, symmetric = TRUE)
    #barplot(a0$values)
    a0 <- a0$vectors
    a0 <- cbind(rep(1,n),a0)
    a0 <- qr.Q(qr(a0))
    a0 <- as.data.frame(a0[,-1])*sqrt(n)
    row.names(a0) <- attr(listw,"region.id")
    names(a0) <- paste("VP", 1:(n-1), sep = "")
    z <- apply(a0,2,function(x) sum((t(b0*x)*x))/n)
    attr(a0,"values") <- z
    attr(a0,"weights") <- rep(1/n,n)
    attr(a0,"call") <- appel
    attr(a0,"class") <- c("orthobasis","data.frame")
    return(a0)
}




## print.orthobasis <- function(x,...) {
##     if (!inherits(x,"orthobasis")) stop ("for 'orthobasis' object")
##     cat("Orthonormal basis: ")
##     n <- nrow(x)
##     p <- ncol(x)
##     if (n!=(p+1)) stop ("Non convenient dimension: author's error")
##     cat("data.frame with",n,"rows and",ncol(x),"columns\n")
##     cat("--------------------------------------\n")
##     cat("Columns are an orthonormal basis of 1n-orthogonal for\n")
##     cat("the inner product defined by the weights attribute\n")
##     cat("---------------------------------------\n")
##     w <- attributes(x)
##     if (!is.null(w$"names")) cat("names =", w$names[1],"...",w$names[p],"\n")
##     if (!is.null(w$"row.names")) cat("row.names =", w$row.names[1],"...",w$row.names[n],"\n")
##     if (!is.null(w$"weights")) cat("weights =", w$weights[1],"...",w$weights[n],"\n")
##     if (!is.null(w$"values")) cat("values =", w$values[1],"...",w$values[p],"\n")
##     if (!is.null(w$"class")) cat("class =", w$class,"\n")
##     if (!is.null(w$"call")) {
##         cat("call =")
##         print(w$"call")
##     }
##   }


## orthobasis.mat <- function(mat, cnw=TRUE) {
##     if (!is.matrix(mat)) stop ("matrix expected")
##     if (any(mat<0)) stop ("negative value in 'mat'")
##     if (nrow(mat)!=ncol(mat)) stop ("squared matrix expected")
##     mat <- (mat+t(mat))/2
##     nlig <- nrow(mat)
##     if (is.null(dimnames(mat))) {
##         w <- paste("P",1:nrow(mat),sep="")
##         dimnames(mat) <- list(w,w)
##     }
##     labels <- dimnames(mat)[[1]]
##     if (cnw) {
##         margi <- apply(mat,1,sum)
##         margi <- max(margi)-margi
##         mat <- mat+diag(margi)
##     }
##     mat <- mat/sum(mat)
##     wt <- rep ((1/nlig),nlig)
##     # calculs extensibles a une ponderation quelconque
##     wt <- wt/sum(wt)
##     # si mat wt est la ponderation marginale associee a mat
##     # tot = sum(mat)
##     # mat = mat-matrix(wt,nlig,nlig,byrow=TRUE)*wt*tot
##     # encore plus particulier mat = mat-1/nlig/nlig
##     # en general les precedents sont des cas particuliers
##     U <- matrix(1,nlig,nlig)
##     U  <- diag(1,nlig)-U*wt
##     mat <- U%*%mat%*%t(U)
##     wt <- sqrt(wt)
##     mat <- t(t(mat)/wt)
##     mat <- mat/wt
##     eig <- eigen(mat,sym=TRUE)
##     w0 <- abs(eig$values)/max(abs(eig$values))
##     tol <- 1e-07
##     w0 <- which(w0<tol)
##     if (length(w0)==0) stop ("abnormal output : no null eigenvalue")
##     else if (length(w0)==1) w0 <- (1:nlig)[-w0]
##     else if (length(w0)>1) {
##         # on ajoute le vecteur derive de 1n
##         w <- cbind(wt,eig$vectors[,w0])
##         # on orthonormalise l'ensemble
##         w <- qr.Q(qr(w))
##         # on met les valeurs propres a 0
##         eig$values[w0] <- 0
##         # on remplace les vecteurs du noyau par une base orthonormee contenant
##         # en premiere position le parasite
##         eig$vectors[,w0] <- w[,-ncol(w)]
##         # on enleve la position du parasite
##         w0 <- (1:nlig)[-w0[1]]
##     }
##     mat <- eig$vectors[,w0]/wt
##     mat <- data.frame(mat)
##     row.names(mat) <- labels
##     names(mat) <- paste("S",1:(nlig-1),sep="")
##     attr(mat,"values") <- eig$values[w0]
##     attr(mat,"weights") <- rep(1/nlig,nlig)
##     attr(mat,"call") <- match.call()
##     attr(mat,"class") <- c("orthobasis","data.frame")
##     return(mat)
## }

## "orthobasis.haar" <- function(n) {
## # on definit deux fonctions :
##     appel = match.call()
##     a <- log(n)/log(2)
##     b <- floor(a)
##     if ((a-b)^2>1e-10) stop ("Haar is not a power of 2")
## # la premiere est ecrite par Daniel et elle donne la demonstration (par analogie avec la fonction qui construit la base Bscores)
## # que la base Bscores est exactement la base de Haar quand on prend une phylogenie reguliere resolue.
## "haar.basis.1" <- function (n) {
##     pari <- matrix(c(1,n),1)
##     "div2" <- function (mat) {
##         res <- NULL
##         for (k in 1 : nrow(mat)) {
##             n1 <- mat[k,1]
##             n2 <- mat[k,2]
##             diff <- n2-n1
##             if (diff <=0) break
##             n3 <- floor((n1+n2)/2)
##             res <- rbind(res,c(n1,n3),c(n3+1,n2))
##         }
##         if (!is.null(res)) pari <<- rbind(pari,res)
##         return(res)
##     }
##     mat <- div2(pari)
##     while (!is.null(mat)) mat <- div2(mat)
##     res <- NULL
##     for (k in 1:nrow(pari)) {
##         x<-rep(0,n)
##         x[(pari[k,1]):(pari[k,2])] <- 1
##         res <-c(res,x)
##     }
##     res = matrix(res,n)
##     res <- qr.Q(qr(res))
##     res <- res[, -1] * sqrt(n)
##     res <- data.frame(res)
##     row.names(res) <- paste("u",1:n,sep="")
##     names(res) <- paste("B",1:(n-1),sep="")
## return(res)
## }

## # la seconde exploite les potentialites de la librairie waveslim, en remarquant qu'il existe un lien etroit entre la definition des filtres et la definition
## # des bases. Cette strategie permettra a l'avenir de definir les bases associees a d'autres famille de fonctions.
## "haar.basis.2" <-  function (n) {
##     if (!require(waveslim)) stop ("Please install waveslim")
##     J <- a    #nombre de niveau
##     res <- matrix(0, nrow = n,ncol = n-1)
##     filter.seq <- "H" #filtre correspondant au niveau 1
##     h <- waveslim::wavelet.filter(wf.name = "haar", filter.seq = filter.seq)   #parametre du filtre au niveau 1
##     k <- 0
##         for(i in 1:J){
##         z <- rep(h,2**(J-i))
##         x <- 1:n
##         y <- rep((n-1-k):(n-2**(J-i)-k),rep(2**i,2**(J-i)))
##         for(j in 1:n)   res[x[j],y[j]] <- z[j]
##         k <- k+2**(J-i)
##         filter.seq <- paste(filter.seq, "L", sep = "")
##         h <- waveslim::wavelet.filter(wf.name = "haar", filter.seq = filter.seq)
##         }
##         res <- res*sqrt(n)
##         res <- data.frame(res)
##         row.names(res) <- paste("u", 1:n, sep = "")
##         names(res) <- paste("B", 1:(n-1), sep = "")
## return(res)
## }

## # suivant que n est grand (n > 257) ou non, on choisit l'une des deux strategies :
##     if (n < 257)
##         res <- haar.basis.1(n)
##         else
##             res <- haar.basis.2(n)

##     attr(res,"values") <- NULL
##     attr(res,"weights") <- rep(1/n,n)
##     attr(res,"call") <- appel
##     attr(res,"class") <- c("orthobasis","data.frame")
##     return(res)
## }

## "orthobasis.line" <- function (n) {
##     appel <- match.call()
##     # solution de Cornillon p. 12
##     res <- NULL
##     for (k in 1:(n-1)) {
##         x <- cos(k*pi*(2*(1:n)-1)/2/n)
##         x <- sqrt(n)*x/sqrt(sum(x*x))
##         res <-c(res,x)
##     }
##     res=matrix(res,n)
##     res <- data.frame(res)
##     row.names(res) <- paste("u",1:n,sep="")
##     names(res) <- paste("B",1:(n-1),sep="")
##     w <- (1:(n-1))*pi/2/n
##     valpro <- 4*(sin(w)^2)/n
##     poivoisi <- c(1,rep(2,n-2),1)
##     poivoisi <- poivoisi/sum(poivoisi)
##     norm <- unlist(apply(res, 2, function(a) sum(a*a*poivoisi)))
##     y <- valpro*n*n/2/(n-1)
##     val <- norm - y
##     attr(res,"values") <- val
##     attr(res,"weights") <- rep(1/n,n)
##     attr(res,"call") <- appel
##     attr(res,"class") <- c("orthobasis","data.frame")

##     # verification locale. Ce paragraphe verifie que les vecteurs et les valeurs
##     # proposee par Cornillon p. 12 sont bien les vecteurs propres de l'operateur de voisinage
##     # rangee dans la solution analytique par variance locale croissante
##     # l'article de Meot est errone et a donne le graphe circulaire pour le graphe lineaire
##     # d0=neig2mat(neig(n.lin=n))
##     # d0 = d0/n
##     # d1=apply(d0,1,sum)
##     # d0=diag(d1)-d0
##     # fun2 <- function(x) {
##     #     z <- sum(t(d0*x)*x)/n
##     #     z <- z/sum(x*x)
##     #     return(z)
##     # }
##     # lambda <- unlist(apply(res,2,fun2))
##     # print(lambda)
##     # print(attr(res,"values"))
##     # plot(lambda,attr(res,"values"))
##     # abline(lm(attr(res,"values")~lambda))
##     # print(coefficients(lm(attr(res,"values")~lambda)))

##     # verification que les valeurs derivees des valeurs propres sont exactement des indices de Moran
##     # d = neig2mat(neig(n.lin=n))
##     # d = d/sum(d) # Moran type W
##     # moran <- unlist(lapply(res,function(x) sum(t(d*x)*x)))
##     # print(moran)
##     # plot(moran,attr(res,"values"))
##     # abline(lm(attr(res,"values")~moran))
##     # print(summary(lm(attr(res,"values")~moran)))
##     return(res)
## }

## "orthobasis.circ" <- function (n) {
##     appel = match.call()
##     if (n<3) stop ("'n' too small")
##     "vecprosin" <- function(k) {
##         x <- sin(2*k*pi*(1:n)/n)
##         x <- x/sqrt(sum(x*x))
##     }
##     "vecprocos" <- function(k) {
##         x <- cos(2*k*pi*(1:n)/n)
##         x <- x/sqrt(sum(x*x))
##     }
##     "valpro" <- function(k,bis=TRUE) {
##         x <- (4/n)*((sin(k*pi/n))^2)
##         if (bis) x <- c(x,x)
##         return(x)
##     }

##     k <- floor(n/2)
##     if (k==n/2) {
##         #n est pair
##         w1 <- matrix(unlist(lapply(1:k,vecprocos)),n,k)
##         w2 <- matrix(unlist(lapply(1:(k-1),vecprosin)),n,k-1)
##         res <- cbind(w1,w2)
##         res[,seq(1,2*k-1,by=2)]<-w1
##         res[,seq(2,2*k-2,by=2)]<-w2
##         vp <- unlist(lapply(1:(k-1),valpro))
##         vp <- c(vp, valpro(k,FALSE))
##     } else {
##         # n est impair
##         w1 <- matrix(unlist(lapply(1:k,vecprocos)),n,k)
##         w2 <- matrix(unlist(lapply(1:k,vecprosin)),n,k)
##         res <- cbind(w1,w2)
##         res[,seq(1,2*k-1,by=2)]<-w1
##         res[,seq(2,2*k,by=2)]<-w2
##         vp <- unlist(lapply(1:k,valpro))
##     }
##     res=sqrt(n)*res
##     res <- as.data.frame(res)
##     row.names(res) <- paste("u",1:n,sep="")
##     names(res) <- paste("B",1:(n-1),sep="")
##     attr(res,"values") <- 1 - n*vp/2
##     attr(res,"weights") <- rep(1/n,n)
##     attr(res,"call") <- appel
##     attr(res,"class") <- c("orthobasis","data.frame")
##     # verification qu'on a exactement des indices de Moran a partie des valeurs propres
##     # d = neig2mat(neig(n.cir=n))
##     # d = d/sum(d) # Moran type W
##     # moran <- unlist(lapply(res,function(x) sum(t(d*x)*x)))
##     # print(moran)
##     # plot(moran,attr(res,"values"))
##     # abline(lm(attr(res,"values")~moran))
##     # print(summary(lm(attr(res,"values")~moran)))
##     return(res)
## }

## "orthobasis.neig" <- function( neig) {
##     appel = match.call()
##     if(!inherits(neig,"neig")) stop ("object of class 'neig' expected")
##     n <- length(attr(neig,"degree"))
##     m <- sum(attr(neig,"degree"))
##     poivoisi <- attr(neig,"degree")/m
##     if (is.null(names(poivoisi))) names(poivoisi) <- as.character(1:n)
##     d0 = neig2mat(neig)
##     d0 = diag(poivoisi)-d0/m
##     eig <- eigen(d0, sym = TRUE)
##     ########
##     tol <- 1e-07
##     w0 <- abs(eig$values)/max(abs(eig$values))
##     w0 <- which(w0<tol)
##     if (length(w0)==0) stop ("abnormal output : no null eigenvalue")
##     else if (length(w0)==1) w0 <- (1:n)[-w0]
##     else if (length(w0)>1) {
##         # on ajoute le vecteur derive de 1n
##         wt <- rep(1,n)
##         w <- cbind(wt,eig$vectors[,w0])
##         # on orthonormalise l'ensemble
##         w <- qr.Q(qr(w))
##         # on met les valeurs propres a 0
##         eig$values[w0] <- 0
##         # on remplace les vecteurs du noyau par une base orthonormee contenant
##         # en premiere position le parasite
##         eig$vectors[,w0] <- w[,-ncol(w)]
##         # on enleve la position du parasite
##         w0 <- (1:n)[-w0[1]]
##     }
##     w0 <- rev(w0)
##     valpro <- eig$values[w0]
##     eig <- eig$vectors[,w0]
##     eig <- as.data.frame(eig)*sqrt(n)
##     z <- apply(eig,2,function(x) sum(x*x*poivoisi))
##     z <- z - valpro*n
##     w <- rev(order(z))
##     z <- z[w]
##     eig <- eig[,w]
##     row.names(eig) <- names(poivoisi)
##     names(eig) <- paste("VP", 1:(n-1), sep = "")
##     attr(eig,"values") <- z
##     attr(eig,"weights") <- rep(1/n,n)
##     attr(eig,"call") <- appel
##     attr(eig,"class") <- c("orthobasis","data.frame")
##     return(eig)
## }

##
## COLOR PLOT
##
## used to plot up to 3 variables in space using RGB system
##
## all coded in S3 method (arguments vary largely)
##


##########
# generic
##########
colorplot <- function(...){
    UseMethod("colorplot")
}



#################
# default method
#################
colorplot.default <- function(xy, X, axes=NULL, add.plot=FALSE, defaultLevel=0, transp=FALSE, alpha=.5, ...){

    ## some checks
    if(any(is.na(xy))) stop("NAs exist in xy")
    xy <- as.matrix(xy)
    if(!is.numeric(xy)) stop("xy is not numeric")
    if(nrow(xy) != nrow(X)) stop("xy and X have different row numbers")
    if(is.null(axes)) {
        axes <- 1:min(ncol(X),3)
    }
    X <- as.matrix(X[,axes,drop=FALSE])
    if(any(is.na(X))) stop("NAs exist in X")
    if(!is.numeric(X)) stop("X is not numeric")
    if(defaultLevel < 0 | defaultLevel>1) stop("defaultLevel must be between 0 and 1")

    ## function mapping x to [0,+inf[
    f1 <- function(x){
        if(any(x<0)) {
            x <- x + abs(min(x))
        }
        return(x)
    }

    ## apply f1 to X
    X <- apply(X, 2, f1)

    v1 <- X[,1]
    if(ncol(X)>=2) {v2 <- X[,2]} else {v2 <- defaultLevel}
    if(ncol(X)>=3) {v3 <- X[,3]} else {v3 <- defaultLevel}

    ## make colors
      if(transp){
        col <- rgb(v1/max(X), v2/max(X), v3/max(X), alpha)
    } else {
        col <- rgb(v1, v2, v3, maxColorValue=max(X))
    }

    ## handle ...
    listArgs <- list(...)
    if(is.null(listArgs$pch)) {listArgs$pch <- 20}

    ## build list of arguments
    listArgs$x <- xy
    listArgs$col <- col

    ## plot data
    if(!add.plot) {
        do.call(plot,listArgs)
    } else {
        do.call(points,listArgs)
    }

    ##return(invisible(match.call()))
    return(invisible(col))
} # end colorplot.default

###############
##
##   CLASSES
##
###############

###############
## SNPbin class
###############
setClass("SNPbin", representation(snp = "list",
                                  n.loc = "integer",
                                  NA.posi = "integer",
                                  label = "charOrNULL",
                                  ploidy = "integer"),
         prototype(snp = list(), n.loc = 0L, label = NULL, ploidy = 1L))




###############
## genlight class
###############
setClass("genlight", representation(gen = "list",
                                    n.loc = "integer",
                                    ind.names = "charOrNULL",
                                    loc.names = "charOrNULL",
                                    loc.all = "charOrNULL",
                                    chromosome = "factorOrNULL",
                                    position = "intOrNULL",
                                    ploidy = "intOrNULL",
                                    pop = "factorOrNULL",
                                    strata = "dfOrNULL",
                                    hierarchy = "formOrNULL",
                                    other = "list"),
         prototype(gen = list(), n.loc = 0L, ind.names = NULL, loc.names = NULL, loc.all = NULL,
                   chromosome = NULL, position = NULL, strata = NULL, hierarchy = NULL, ploidy=NULL, pop=NULL, other=list()))








#####################
##
##   CONSTRUCTORS
##
#####################

####################
## SNPbin constructor
####################
setMethod("initialize", "SNPbin", function(.Object, ...) {
    x <- .Object
    input <- list(...)
    if(length(input)==1) names(input) <- "snp"
    if(length(input)>1 && ! "snp" %in% names(input)) names(input)[1] <- "snp"

    ## handle snp data ##
    if(!is.null(input$snp) && length(input$snp)>0 && !all(is.na(input$snp))){
        ## a vector of raw is provided
        if(is.raw(input$snp)){
            x@snp <-list(input$snp)
        }

        ## a list of raw vectors is provided
        if(is.list(input$snp)){
            if(all(sapply(input$snp, class)=="raw")){
                x@snp <- input$snp
            }
        }

        ## a numeric/integer vector is provided
        ## conversion from a vector of 0/1 (integers)
        if(is.numeric(input$snp) | is.integer(input$snp)){
            input$snp <- as.integer(input$snp)
            ## determine ploidy
            if(is.null(input$ploidy)){
                input$ploidy <- max(input$snp, na.rm=TRUE)
                if(input$ploidy==0) input$ploidy <- 1
            }
            input$ploidy <- as.integer(input$ploidy)
            if(input$ploidy<1) stop("Ploidy is less than 1")

            ## check values in the vector
            if(any(input$snp<0 | input$snp>input$ploidy, na.rm=TRUE)){
                stop("Values of SNPs < 0 or > ploidy")
            }


            ## handle ploidy (may have to split info into binary vectors)
            x@snp <- list()
            i <- max(input$snp, na.rm=TRUE) # determine max nb of alleles
            if(i > 1){ # haplotype can be 0/1/2/...
                j <- 0 # index for the length of the list @snp
                while(i > 0){
                    j <- j+1 # update length of the result
                    temp <- as.integer(input$snp==i)
                    x@snp[[j]] <- .bin2raw(temp)$snp # make a vector of 1
                    input$snp <- input$snp-temp # deflate data (subtract the recoded alleles)
                    i <- max(input$snp, na.rm=TRUE) # update the max nb of alleles
                }
            } else { # haplotype is only 0/1/NA
                x@snp[[1]] <- .bin2raw(input$snp)$snp
            }
            x@n.loc <- length(input$snp)
            x@NA.posi <- which(is.na(input$snp))
            x@ploidy <- as.integer(input$ploidy)
            return(x)
        }
    }

    ## handle full-NA data
    if(!is.null(input$snp) && all(is.na(input$snp))){
        x@snp <- list()
        x@n.loc <- length(input$snp)
        x@snp[[1]] <- .bin2raw(rep(0L, length(input$snp)))$snp
        x@NA.posi <- 1:length(input$snp)
        if(!is.null(input$ploidy)){
            x@ploidy <- as.integer(input$ploidy)
        } else {
            x@ploidy <- as.integer(NA)
        }
        return(x)
    }


    ## handle n.loc ##
    if(!is.null(input$n.loc)){
        x@n.loc <- as.integer(input$n.loc)
    } else {
        if(!is.null(input$snp)){
            warning("number of SNPs (n.loc) not provided to the genlight constructor - using the maximum number given data coding.")
            x@n.loc <- as.integer(length(x@snp)*8)
        } else {
            x@n.loc <- 0L
        }
    }


    ## handle NA.posi ##
    if(!is.null(input$NA.posi)){
        x@NA.posi <- as.integer(input$NA.posi)
    }


    ## handle ploidy ##
    if(!is.null(input$ploidy)){
        x@ploidy <- as.integer(input$ploidy)
    }


    return(x)
}) # end SNPbin constructor







########################
## genlight constructor
########################
setMethod("initialize", "genlight", function(.Object, ..., parallel=FALSE, n.cores=NULL) {
    if(parallel && !require(parallel)) stop("parallel package requested but not installed")
    if(parallel && is.null(n.cores)){
        n.cores <- parallel::detectCores()
    }
    if( .Platform$OS.type == "windows" ){
        n.cores <- 1
    }

    x <- .Object
    input <- list(...)
    if(length(input)==1 && is.null(names(input))) names(input) <- "gen"
    if(length(input)>1 && ! "gen" %in% names(input)) names(input)[1] <- "gen"


    ## HANDLE INPUT$GEN ##
    if(!is.null(input$gen)){
        ## input$gen is a list of SNPbin ##
        if(is.list(input$gen) && all(sapply(input$gen, class)=="SNPbin")){
            ## check nb of loci in each SNPbin
            if(length(unique(sapply(input$gen, nLoc)))>1) {
                warning("SNPbin objects have different numbers of loci")
                input$gen <- lapply(input$gen, as.integer)
            } else { # all seems fine
                x@gen <- input$gen
                if(is.null(input$ind.names)){
                    input$ind.names <- names(input$gen)
                }
            }
        }


        ## input$gen is a matrix or a data.frame
        if((is.matrix(input$gen) & !inherits(input$gen,"snp.matrix")) | is.data.frame(input$gen)){
            if(is.null(input$ind.names)){
                input$ind.names <- rownames(input$gen)
            }
            if(is.null(input$loc.names)){
                input$loc.names <- colnames(input$gen)
                if(is.data.frame(input$gen)){ # do not use names if these are the default names of a data.frame
                    if(identical(colnames(input$gen), paste("V", 1:ncol(input$gen), sep=""))){
                        input$loc.names <- NULL
                    }
                }
            }
            ##input$gen <- lapply(1:nrow(input$gen), function(i) as.integer(input$gen[i,]))
            if(parallel){
                x@gen <- parallel::mclapply(1:nrow(input$gen), function(i) new("SNPbin", as.integer(input$gen[i,])),
                                  mc.cores=n.cores, mc.silent=TRUE, mc.cleanup=TRUE, mc.preschedule=FALSE)
            } else {
                x@gen <- lapply(1:nrow(input$gen), function(i) new("SNPbin", as.integer(input$gen[i,])) )
            }
        }


        ## input$gen is a list of integers/numeric ##
        if(is.list(input$gen) && !is.data.frame(input$gen) && all(sapply(input$gen, class) %in% c("integer","numeric"))){
            ## check length consistency
            lengthvec <- sapply(input$gen, length)

            ## complete with NA is necessary
            if(length(unique(lengthvec))>1) {
                warning("Genotypes have variable length; completing shorter ones with NAs.")
                for(i in 1:length(input$gen)){
                    input$gen[[i]] <- c(input$gen[[i]], rep(NA, max(lengthvec)-length(input$gen[[i]])))
                }
            }

            ## name individuals if needed
            if(is.null(input$ind.names)){
                input$ind.names <- names(input$gen)
            }

            ## create SNPbin list
            if(parallel){
                x@gen <- parallel::mclapply(input$gen, function(e) new("SNPbin",e), mc.cores=n.cores, mc.silent=TRUE, mc.cleanup=TRUE, mc.preschedule=FALSE)
            } else {
                x@gen <- lapply(input$gen, function(e) new("SNPbin",e))
            }
        }


        ## input$gen is a snp.matrix object ##
        if(inherits(input$gen,"snp.matrix")){
            if(!require(snpMatrix)){
                cat("\nThe package snp.matrix is needed for this conversion.")
                cat("\nTo install it, type:")
                cat("\n  source(\"http://bioconductor.org/biocLite.R\")")
                cat("\n  biocLite(\"snpMatrix\")\n")
                x@gen <- NULL
            } else {

                ## function to convert one indiv
                f1 <- function(x){
                    res <- as.integer(x)
                    res[res==0] <- NA
                    res <- res-1
                    return(new("SNPbin", as.integer(res), ploidy=2L))
                }

                ## create SNPbin list
                if(parallel){
                    x@gen <- parallel::mclapply(1:nrow(input$gen), function(i) f1(input$gen[i,]), mc.cores=n.cores, mc.silent=TRUE, mc.cleanup=TRUE, mc.preschedule=FALSE)
                } else {
                    x@gen <- lapply(1:nrow(input$gen), function(i) f1(input$gen[i,]))
                }

                ## handle names
                if(is.null(input$ind.names)) {input$ind.names <- rownames(input$gen)}
                if(is.null(input$loc.names)) {input$loc.names <- colnames(input$gen)}
            }
        }
    }


    if(length(x@gen) > 0) { # if non-emtpy object
        ## HANDLE INPUT$IND.NAMES ##
        if(!is.null(input$ind.names)){
            input$ind.names <- as.character(input$ind.names)

            ## check length consistency
            if(length(input$ind.names) != nInd(x)){
                warning("Inconsistent length for ind.names - storing this argument in @other.")
                if(is.null(input$other)) {
                    input$other <- list(ind.names.wrong.length=input$ind.names)
                } else {
                    input$other$ind.names.wrong.length <- input$ind.names
                }

            } else {
                ## assign value to the output object
                x@ind.names <- input$ind.names
                ## ## name list and each SNPbin ## THIS DUPLICATES THE INFORMATION
                ## names(x@gen) <- input$ind.names
                ## for(i in 1:length(x@gen)){
                ##     x@gen[[i]]@label <- input$ind.names[i]
                ## }

            }


        }


        ## HANDLE INPUT$N.LOC ##
        if(!is.null(input$n.loc)){ # n.loc is provided
            input$n.loc <- as.integer(input$n.loc)

            ## check length consistency
            if(input$n.loc != nLoc(x@gen[[1]])) {
                warning("Inconsistent number of loci (n.loc) - ignoring this argument.")
            } else {
                x@n.loc <- input$n.loc
            }
        } else { # n.loc is not provided
            x@n.loc <- nLoc(x@gen[[1]])
        }


        ## HANDLE INPUT$PLOIDY ##
        ## note: if not provided, @ploidy is NULL (saves some space)
        if(!is.null(input$ploidy)){ # ploidy is provided
            input$ploidy <- as.integer(input$ploidy)
            input$ploidy <- rep(input$ploidy, length=length(x@gen))
            x@ploidy <- input$ploidy
        }


        ## HANDLE INPUT$LOC.NAMES ##
        if(!is.null(input$loc.names) && length(input$loc.names)>0){ # ploidy is provided
            input$loc.names <- as.character(input$loc.names)

            ## check length consistency
            if(length(input$loc.names) != x@n.loc){ # if problem, store in @other
                warning("Inconsistent length for loc.names - storing this argument in @other.")
                if(is.null(input$other)) {
                    input$other <- list(loc.names.wrong.length=input$loc.names)
                } else {
                    input$other$loc.names.wrong.length <- input$loc.names
                }
            } else {
                x@loc.names <- input$loc.names
            }
        }


        ## HANDLE INPUT$LOC.ALL ##
        if(!is.null(input$loc.all) && length(input$loc.all)>0){ # ploidy is provided
            input$loc.all <- as.character(input$loc.all)

            ## check length consistency
            if(length(input$loc.all) != x@n.loc){
                warning("Inconsistent length for loc.all - storing this argument in @other.")
                if(is.null(input$other)) {
                    input$other <- list(loc.all.wrong.length=input$loc.all)
                } else {
                    input$other$loc.all.wrong.length <- input$loc.all
                }
            } else {
                ## check string consistency (format is e.g. "a/t")
                temp <- grep("^[[:alpha:]]{1}/[[:alpha:]]{1}$", input$loc.all)
                if(any(! 1:nLoc(x@gen[[1]]) %in% temp)){ # if problem, store in @other
                    ## input$loc.all <- gsub("[[:space:]]","", input$loc.all)
                    warning("Miss-formed strings in loc.all (must be e.g. 'c/g') - storing this argument in @other.")
                    if(is.null(input$other)) {
                        input$other <- list(loc.all.misformed=input$loc.all)
                    } else {
                        input$other$loc.all.misformed <- input$loc.all
                    }
                } else {
                    x@loc.all <- input$loc.all
                }
            }
        }


        ## HANDLE CHROMOSOME ##
        if(!is.null(input$chromosome)){
            if(length(input$chromosome) != x@n.loc) { # if wrong length, store in @other
                warning("chromosome argument has inconsistent length - storing this argument in @other")
                if(is.null(input$other)) {
                    input$other <- list(chromosome.wrong.length=input$chromosome)
                } else {
                    input$other$chromosome.wrong.length <- input$chromosome
                }
            } else {
                x@chromosome <- factor(input$chromosome)
            }
        }


        ## HANDLE POSITION ##
        if(!is.null(input$position)){
            if(length(input$position) != x@n.loc) { # if wrong length, store in @other
                warning("position argument has inconsistent length - storing this argument in @other")
                if(is.null(input$other)) {
                    input$other <- list(position.wrong.length=input$position)
                } else {
                    input$other$position.wrong.length <- input$position
                }
            } else {
                x@position <- as.integer(input$position)
            }
        }


        ## HANDLE INPUT$POP ##
        if(!is.null(input$pop)){
            ## check length consistency
            if(length(input$pop) != nInd(x)){
                warning("Inconsistent length for pop - ignoring this argument.")
                if(is.null(input$other)) {
                    input$other <- list(pop.wrong.length=input$pop)
                } else {
                    input$other$pop.wrong.length <- input$pop
                }
            } else {
                x@pop <- factor(input$pop)
            }
        }

        ## HANDLE INPUT$STRATA ##
        if(!is.null(input$strata)){
            ## check length consistency
            if(nrow(input$strata) != nInd(x)){
                warning("Inconsistent length for strata - ignoring this argument.")
                if(is.null(input$other)) {
                    input$other <- list(strata.wrong.length=input$strata)
                } else {
                    input$other$strata.wrong.length <- input$strata
                }
            } else {
              # Make sure that the hierarchies are factors.
              x@strata <- data.frame(lapply(input$strata, function(f) factor(f, unique(f))))
              if(!is.null(x@ind.names)){
                rownames(x@strata) <- x@ind.names
              }
            }
        }
        ## HANDLE INPUT$HIERARCHY ##
        if (!is.null(x@strata) && !is.null(input$hierarchy)){

            if (is.language(input$hierarchy)){
                the_names <- all.vars(input$hierarchy)
                if (all(the_names %in% names(x@strata))){
                    ## TODO: CHECK HIERARCHY HERE
                    x@hierarchy <- input$hierarchy
                } else {
                    warning("hierarchy names do not match names of strata. Setting slot to NULL")
                    x@hierarchy <- NULL
                }
            } else {
                warning("hierarchy must be a formula. Setting slot to NULL.")
                x@hierarchy <- NULL
            }
        }


    } # end if non-empty @gen


    ## HANDLE INPUT$OTHER ##
    if(!is.null(input$other)){
        x@other <- input$other
    }


    ## RETURN OBJECT ##
    names(x@gen) <- NULL # do not store ind.names twice
    return(x)
}) # end genlight constructor










################################
##
##   METHODS AND ACCESSORS
##
################################

###############
## show SNPbin
###############
setMethod ("show", "SNPbin", function(object){
    cat("/// SNPBIN OBJECT /////////")
    if(!is.null(object@label)) {
        cat("\n", object@label)
    }
    cat("\n", format(nLoc(object), big.mark=","), "SNPs coded as bits, size:", format(object.size(object), units="auto"))
    cat("\n Ploidy:", object@ploidy)
    temp <- round(length(object@NA.posi)/nLoc(object) *100,2)
    cat("\n ", length(object@NA.posi), " (", temp," %) missing data\n", sep="")
}) # end show method





#################
## show genlight
#################
setMethod ("show", "genlight", function(object){
    ## HEADER
    cat(" /// GENLIGHT OBJECT /////////")
    cat("\n\n //", format(nInd(object), big.mark=","), "genotypes, ",
        format(nLoc(object), big.mark=","), "binary SNPs, size:", format(object.size(object), units="auto"))

    temp <- sapply(object@gen, function(e) length(e@NA.posi))
    if(length(temp>1)){
        cat("\n ", sum(temp), " (", round((sum(temp)/(nInd(object)*nLoc(object))) *100,2)," %) missing data", sep="")
    }

    ## BASIC CONTENT
    cat("\n\n // Basic content")
    cat("\n   @gen: list of", length(object@gen), "SNPbin")

    if(!is.null(object@ploidy)){
        ploidytxt <- paste("(range: ", paste(range(object@ploidy), collapse="-"), ")", sep="")
        cat("\n   @ploidy: ploidy of each individual ", ploidytxt)
    }

    ## OPTIONAL CONTENT
    cat("\n\n // Optional content")
    optional <- FALSE

    if(!is.null(object@ind.names)){
        optional <- TRUE
        cat("\n   @ind.names: ", length(object@ind.names), "individual labels")
    }

    if(!is.null(object@loc.names)){
        optional <- TRUE
        cat("\n   @loc.names: ", length(object@loc.names), "locus labels")
    }

    if(!is.null(object@loc.all)){
        optional <- TRUE
        cat("\n   @loc.all: ", length(object@loc.all), "alleles")
    }

    if(!is.null(object@chromosome)){
        optional <- TRUE
        cat("\n   @chromosome: factor storing chromosomes of the SNPs")
    }

    if(!is.null(object@position)){
        optional <- TRUE
        cat("\n   @position: integer storing positions of the SNPs")
    }

    if(!is.null(object@pop)){
        optional <- TRUE
        poptxt <- paste("(group size range: ", paste(range(table(object@pop)), collapse="-"), ")", sep="")
        cat("\n   @pop:", paste("population of each individual", poptxt))
    }

    if (!is.null(object@strata)){
        optional <- TRUE
        cat("\n   @strata: ")
        levs <- names(object@strata)
        if (length(levs) > 6){
            levs <- paste(paste(head(levs), collapse = ", "), "...", sep = ", ")
        } else {
            levs <- paste(levs, collapse = ", ")
        }
        cat("a data frame with", length(object@strata), "columns (", levs, ")")
    }

    if (!is.null(object@hierarchy)){
        optional <- TRUE
        cat("\n   @hierarchy:", paste(object@hierarchy, collapse = ""))
    }

    if(!is.null(object@other)){
        optional <- TRUE
        cat("\n   @other: ")
        cat("a list containing: ")
        cat(ifelse(is.null(names(object@other)), "elements without names", paste(names(object@other), collapse= "  ")), "\n")
    }

    if(!optional) cat("\n   - empty -")

    cat("\n")
}) # end show method






############
## accessors
############

## nLoc
setMethod("nLoc","SNPbin", function(x,...){
    return(x@n.loc)
})

setMethod("nLoc","genlight", function(x,...){
    return(x@n.loc)
})


## nInd
setMethod("nInd","genlight", function(x,...){
    return(length(x@gen))
})

## nPop
setMethod("nPop","genlight", function(x,...){
    return(length(levels(pop(x))))
})

setMethod("dim", "genlight", function(x){
    return(c(nInd(x), nLoc(x)))
})

## $
setMethod("$","SNPbin",function(x,name) {
    return(slot(x,name))
})

setMethod("$","genlight",function(x,name) {
    return(slot(x,name))
})


setMethod("$<-","SNPbin",function(x,name,value) {
  slot(x,name,check=TRUE) <- value
  return(x)
})

setMethod("$<-","genlight",function(x,name,value) {
  slot(x,name,check=TRUE) <- value
  return(x)
})


## names
setMethod("names", signature(x = "SNPbin"), function(x){
    return(slotNames(x))
})

setMethod("names", signature(x = "genlight"), function(x){
    return(slotNames(x))
})


## ploidy
setMethod("ploidy","SNPbin", function(x,...){
    return(x@ploidy)
})

setMethod("ploidy","genlight", function(x,...){
    if(nInd(x)>0){
        if(!is.null(x@ploidy)){
            res <- x@ploidy
        } else {
            res <- sapply(x@gen, function(e) e@ploidy)
        }
        names(res) <- x@ind.names
        return(res)
    } else {
        return(NULL)
    }
})


setReplaceMethod("ploidy","SNPbin",function(x,value) {
    if(is.null(value)){
        slot(x, "ploidy", check=TRUE) <- value
        return(x)
    }
    value <- as.integer(value)
    if(any(value)<1) stop("Negative or null values provided")
    if(any(is.na(value))) stop("NA values provided")
    if(length(value)>1) warning("Several ploidy numbers provided; using only the first integer")
    slot(x,"ploidy",check=TRUE) <- value[1]
    return(x)
})

setReplaceMethod("ploidy","genlight",function(x,value) {
    if(is.null(value)){
        slot(x, "ploidy", check=TRUE) <- value
        return(x)
    }
    value <- as.integer(value)
    if(any(value)<1) stop("Negative or null values provided")
    if(any(is.na(value))) stop("NA values provided")
    if(length(value) == 1) value <- rep(value, length=nInd(x))
    if(length(value) != nInd(x)) stop("Length of the provided vector does not match nInd(x)")
    slot(x,"ploidy",check=TRUE) <- value
    return(x)
})


## locNames
setMethod("locNames","genlight", function(x,...){
    ## if loc names provided, return them
    if(!is.null(x@loc.names)) return(x@loc.names)

    ## otherwise, look for position / alleles
    if(!is.null(res <- position(x))){
        if(!is.null(alleles(x))){
            res <- paste(res, alleles(x), sep=".")
        } else { # force position to be character
            res <- as.character(res)
        }
        return(res)
    }
})


setReplaceMethod("locNames","genlight",function(x,value) {
    if(is.null(value)){
        slot(x, "loc.names", check=TRUE) <- value
        return(x)
    }
    if(length(value) != nLoc(x)) stop("Vector length does no match number of loci")
    slot(x,"loc.names",check=TRUE) <- as.character(value)
    return(x)
})


## indNames
setMethod("indNames","genlight", function(x,...){
    if(length(x@ind.names)==0) return(NULL)
    return(x@ind.names)
})


setReplaceMethod("indNames","genlight",function(x,value) {
    if(is.null(value)){
        slot(x, "ind.names", check=TRUE) <- value
        return(x)
    }
    value <- as.character(value)
    if(length(value) != nInd(x)) stop("Vector length does no match number of individuals")
    slot(x,"ind.names",check=TRUE) <- value
    return(x)
})

## popNames
setMethod("popNames","genlight", function(x,...){
    if(length(levels(pop(x)))==0) return(NULL)
    return(levels(pop(x)))
})


setReplaceMethod("popNames","genlight",function(x,value) {
    if (is.null(value) || any(is.na(value))){
        stop("Can't set population names to NULL or NA")
        return(x)
    }
    value <- as.character(value)
    if(length(value) != length(levels(pop(x)))){
      stop("Vector length does no match number of populations")
    }
    levels(pop(x)) <- value
    return(x)
})

## alleles
setMethod("alleles","genlight", function(x,...){
    if(length(x@loc.all)==0) return(NULL)
    return(x@loc.all)
})

setReplaceMethod("alleles","genlight", function(x, value){
    if(is.null(value)){
        slot(x, "loc.all", check=TRUE) <- value
        return(x)
    }
    value <- as.character(value)
    if(length(value)!=nLoc(x)) stop("replacement vector must be of length nLoc(x)")
    temp <- grep("^[[:alpha:]]{1}/[[:alpha:]]{1}$", value)
    if(any(! 1:nLoc(x) %in% temp)) stop("Miss-formed strings in replacement (must be e.g. 'c/g')")
    slot(x, "loc.all", check=TRUE) <- value
    return(x)
})


## chromosome
setGeneric("chromosome", function(x, ...) standardGeneric("chromosome"))
setGeneric("chromosome<-", function(x, value) standardGeneric("chromosome<-"))
setGeneric("chr", function(x,...) standardGeneric("chr"))
setGeneric("chr<-", function(x, value) standardGeneric("chr<-"))

setMethod("chromosome","genlight", function(x,...){
    if(length(x@chromosome)==0) return(NULL)
    return(x@chromosome)
})

setMethod("chr","genlight", function(x,...){
    return(chromosome(x))
})


setReplaceMethod("chromosome","genlight",function(x,value) {
    if(is.null(value)){
        slot(x, "chromosome", check=TRUE) <- value
        return(x)
    }
    if(length(value) != nLoc(x)) stop("Vector length does no match number of loci")
    slot(x,"chromosome",check=TRUE) <- factor(value)
    return(x)
})

setReplaceMethod("chr","genlight",function(x,value) {
    chromosome(x) <- value
    return(x)
})



## position
setGeneric("position", function(x, ...) standardGeneric("position"))
setGeneric("position<-", function(x, value) standardGeneric("position<-"))

setMethod("position","genlight", function(x,...){
    if(length(x@position)==0) return(NULL)
    return(x@position)
})


setReplaceMethod("position","genlight",function(x,value) {
    if(is.null(value)){
        slot(x, "position", check=TRUE) <- value
        return(x)
    }
    if(length(value) != nLoc(x)) stop("Vector length does no match number of loci")
    slot(x,"position",check=TRUE) <- as.integer(value)
    return(x)
})



## NA.posi
setGeneric("NA.posi", function(x, ...) standardGeneric("NA.posi"))

setMethod("NA.posi","SNPbin", function(x,...){
    return(x@NA.posi)
})

setMethod("NA.posi","genlight", function(x,...){
    res <- lapply(x@gen, function(e) e@NA.posi)
    names(res) <- indNames(x)
    return(res)
})


## pop
setMethod("pop","genlight", function(x){
    return(x@pop)
})


setReplaceMethod("pop","genlight",function(x,value) {
    if(is.null(value) | length(value)==0){
        slot(x, "pop", check=TRUE) <- NULL
        return(x)
    }
    if(length(value) != nInd(x)) stop("Vector length does no match number of individuals")
    slot(x,"pop", check=TRUE) <- factor(value)
    return(x)
})



## other
setMethod("other","genlight", function(x,...){
    if(length(x@other)==0) return(NULL)
    return(x@other)
})


setReplaceMethod("other","genlight",function(x,value) {
    if( !is.null(value) && (!is.list(value) | is.data.frame(value)) ) {
        value <- list(value)
    }
    slot(x,"other",check=TRUE) <- value
    return(x)
})








###################
##
##   CONVERSIONS
##
###################

############
## .bin2raw
###########
## each byte takes a value on [0,255]

## function to code multiple SNPs on a byte
## 8 combinations of SNPs can be coded onto a single byte (0->255)
.bin2raw <- function(vecSnp){
    ## handle missing data
    NAposi <- which(is.na(vecSnp))
    if(length(NAposi)>0){
        vecSnp[is.na(vecSnp)] <- 0L
    }


    nbBytes <- length(vecSnp) %/% 8
    if(length(vecSnp) %% 8 > 0) {nbBytes <- nbBytes +1}
    ori.length <- length(vecSnp)
    new.length <- 8*nbBytes
    vecSnp <- c(vecSnp, rep(0, new.length-ori.length)) # fill the end with 0 of necessary


    ## map info to bytes (0:255)
    vecSnp <- as.integer(vecSnp)
    ##vecRaw <- integer(nbBytes) # no longer needed - sending raw type directly
    vecRaw <- raw(nbBytes)
    # Wed Apr 12 08:26:49 2017 ------------------------------
    # I changed this from the C function to the base R function `packBits()`
    # because I was getting an error on solaris sparc. It suffers an increase
    # of a fraction of a millisecond in processing time, but I think that's
    # managable.
    # 
    # ZNK
    # 
    # library(microbenchmark)
    # set.seed(5000)
    # dat <- sample(c(0,1,NA), 1e5, prob=c(.495, .495, .01), replace=TRUE)
    # y <- microbenchmark(C = .bin2raw(dat), base = .bin2raw_original(dat), times = 1000)
    # print(y, "relative")
    ## Unit: relative
    ##  expr      min       lq     mean   median       uq      max neval cld
    ##     C 1.000000 1.000000 1.000000 1.000000 1.000000 1.000000  1000   a
    ##  base 1.074409 1.113972 1.103931 1.103837 1.083847 1.183597  1000   a
    vecRaw <- packBits(vecSnp)
    # vecRaw <- .C("binIntToBytes", vecSnp, length(vecSnp), vecRaw, nbBytes, PACKAGE="adegenet")[[3]]
    ## vecraw <- sapply(seq(1, by=8, length=nbBytes), function(i) which(apply(SNPCOMB,1, function(e) all(temp[i:(i+7)]==e))) ) # old R version

    ## return result
    res <- list(snp=vecRaw, n.loc=as.integer(ori.length), NA.posi=as.integer(NAposi))
    return(res)
} # end .bin2raw



###########
## .raw2bin
###########
## convert vector of raw to 0/1 integers
.raw2bin <- function(x){
    if(!is.raw(x)) stop("x is not of class raw")
    ## SNPCOMB <- as.matrix(expand.grid(rep(list(c(0,1)), 8)))
    ## colnames(SNPCOMB) <- NULL
    ## res <- unlist(lapply(as.integer(x), function(i) SNPCOMB[i+1,]))
    res <- .C("bytesToBinInt", x, length(x), integer(length(x)*8), PACKAGE="adegenet")[[3]]
    # Below is an equivalent function in base, but it's slowed down 8x
    # res <- as.integer(rawToBits(x))
    return(res)
} # end .raw2bin


#############
## .SNPbin2int
#############
## convert SNPbin to integers (0/1/2...)
.SNPbin2int <- function(x){
    ##res <- lapply(x@snp, .raw2bin)
    resSize <- length(x@snp[[1]])*8
    # Wed Apr 12 08:49:02 2017 ------------------------------
    # I am leaving this function along as it does not necessarily break solaris,
    # but I am leaving the code and timings just in case. 
    # 
    # ZNK
    res <- .C("bytesToInt", unlist(x@snp), length(x@snp[[1]]), length(x@snp), integer(resSize), as.integer(resSize), PACKAGE="adegenet")[[4]][1:nLoc(x)]
    # library(microbenchmark)
    # set.seed(5000)
    # dat <- sample(c(0:2,NA), 1e5, prob=c(rep(.995/5,3), 0.005), replace=TRUE)
    # x <- new("SNPbin", dat)
    # y <- microbenchmark(C = .SNPbin2int(x), base = .SNPbin2int1(x), times = 1000)
    # print(y, "relative")
    ## Unit: relative
    ##  expr      min       lq     mean   median      uq       max neval cld
    ##     C 1.000000 1.000000 1.000000 1.000000 1.00000 1.0000000  1000   a
    ##  base 2.206831 1.200831 1.019783 1.168149 1.02654 0.1668163  1000   a
    # res     <- vapply(x@snp, function(x) as.integer(rawToBits(x)), integer(resSize))
    # res     <- apply(res[1:nLoc(x), ], 1, sum)
    if (length(x@NA.posi) > 0){
        res[x@NA.posi] <- NA_integer_
    }
    return(res)
} # end .SNPbin2int






#############
## as methods
#############
## KLUDGE - needed for as.matrix.genlight to be dispatched correctly (R-2.12.1)
setGeneric("as.matrix")


## SNPbin/genlight -> other
setAs("SNPbin", "integer", def=function(from){
    res <- .SNPbin2int(from)
    return(res)
})


as.integer.SNPbin <- function(x, ...){
    return(as(x, "integer"))
}


setAs("genlight", "matrix", def=function(from){
    res <- unlist(lapply(from@gen, as.integer))
    res <- matrix(res, ncol=nLoc(from), nrow=nInd(from), byrow=TRUE)
    colnames(res) <- locNames(from)
    ## if(!is.null(alleles(from))){
    ##     colnames(res) <- paste(locNames(from), alleles(from), sep=ifelse(is.null(locNames(from)), "", "."))
    ## }
    rownames(res) <- indNames(from)
    return(res)
})


as.matrix.genlight <- function(x, ...){
    return(as(x, "matrix"))
}


setAs("genlight", "data.frame", def=function(from){
    return(as.data.frame(as.matrix(from)))
})


as.data.frame.genlight <- function(x, ...){
    return(as(x, "data.frame"))
}


setAs("genlight", "list", def=function(from){
    res <- lapply(from@gen, as.integer)
    names(res) <- indNames(from)
    return(res)
})


as.list.genlight <- function(x, ...){
    return(as(x, "list"))
}




## other -> SNPbin/genlight
setGeneric("as.SNPbin", function(x, ...) standardGeneric("as.SNPbin"))
setGeneric("as.genlight", function(x, ...) standardGeneric("as.genlight"))

setAs("integer", "SNPbin", def = function(from){
    res <- new("SNPbin", from)
    return(res)
})

setAs("numeric", "SNPbin", def = function(from){
    res <- new("SNPbin", from)
    return(res)
})


setMethod("as.SNPbin", "integer", function(x, ...) as(x, "SNPbin"))
setMethod("as.SNPbin", "numeric", function(x, ...) as(x, "SNPbin"))


setAs("matrix", "genlight", def=function(from){
    return(new("genlight", from))
})


setAs("data.frame", "genlight", def=function(from){
    return(new("genlight", from))
})


setAs("list", "genlight", def=function(from){
    return(new("genlight", from))
})


## setAs("snp.matrix", "genlight", def=function(from){
##     return(new("genlight", from))
## })




setMethod("as.genlight", "matrix", function(x, ...) as(x, "genlight"))
setMethod("as.genlight", "data.frame", function(x, ...) as(x, "genlight"))
setMethod("as.genlight", "list", function(x, ...) as(x, "genlight"))
## setMethod("as.genlight", "snp.matrix", function(x, ...) as(x, "genlight"))



setMethod("tab", "genlight",
          function(x, freq = FALSE,
                   NA.method = c("mean", "asis", "zero")) {
              
              NA.method <- match.arg(NA.method)
              out <- as.matrix(x)

              if (freq) {
                  out <- out / ploidy(x)
              }

              if (NA.method == "mean"){
                  f1 <- function(vec){
                      m <- mean(vec, na.rm = TRUE)
                      vec[is.na(vec)] <- m
                      return(vec)
                  }

                  out <- apply(out, 2, f1)

              }
              if (NA.method == "zero"){
                  out[is.na(out)] <- ifelse(freq, 0, 0L)
              }

              return(out)
          })



################################
## testing SNPbin
##
##
## library(adegenet)
## dat <- c(1,0,0,1,0,NA,1,0,0,0,0,1)
## x <- new("SNPbin",dat)
## as.integer(x)


## HAPLOID DATA - NO NA
## dat <- sample(c(0L,1L), 1e6, replace=TRUE)
## x <- new("SNPbin", dat)
## identical(as(x, "integer"),dat) # SHOULD NORMALLY BE TRUE
## all(as(x, "integer") == dat, na.rm=TRUE) # MUST BE TRUE
## object.size(dat)/object.size(x) # EFFICIENCY OF CONVERSION


## HAPLOID DATA - WITH NAs
## dat <- sample(c(0,1,NA), 1e6, prob=c(.5, .49, .01), replace=TRUE)
## x <- new("SNPbin", dat)
## identical(as(x, "integer"),dat) # SHOULD NORMALLY BE TRUE
## all(as(x, "integer") == dat, na.rm=TRUE) # MUST BE TRUE
## object.size(dat)/object.size(x) # EFFICIENCY OF CONVERSION



 ## DIPLOID DATA
## dat <- sample(c(0:2,NA), 1e6, prob=c(.4, .4, .195 ,.005), replace=TRUE)
## x <- new("SNPbin", dat)

## identical(as(x, "integer"),dat) # MUST BE TRUE

## object.size(dat)/object.size(x) # EFFICIENCY OF CONVERSION



 ## POLYLOID DATA
## dat <- sample(c(0:5,NA), 1e6, prob=c(rep(.995/6,6), 0.005), replace=TRUE)
##  x <- new("SNPbin", dat)

## identical(as(x, "integer"),dat) # MUST BE TRUE

## object.size(dat)/object.size(x) # EFFICIENCY OF CONVERSION




################################
## testing genlight
##
##


## SIMPLE TESTS
## dat <- c(1,0,0,1,0,0,1,1,1,0,1)
## x <- new("SNPbin",dat)$snp[[1]]
## as.integer(x)==dat


## library(adegenet)
## dat <- list(toto=c(1,1,0,0), titi=c(NA,1,1,0), tata=c(NA,0,3, NA))
## x <- new("genlight", dat)
## x
## as.list(x)
## as.matrix(x)

## identical(x, new("genlight", as.list(x))) # round trip - list - MUST BE TRUE
## identical(x, new("genlight", as.matrix(x))) # round trip - matrix - MUST BE TRUE
## identical(x, new("genlight", as.data.frame(x))) # round trip - data.frame - MUST BE TRUE

## ## test subsetting
## identical(as.list(x[c(1,3)]), as.list(x)[c(1,3)]) # MUST BE TRUE
## identical(x, x[]) # MUST BE TRUE
## all.equal(t(as.matrix(as.data.frame(dat)))[,1:3], as.matrix(x[,1:3])) # MUST BE TRUE




## ## ## BIG SCALE TEST - HAPLOID DATA WITH NA
## library(adegenet)
## dat <- lapply(1:50, function(i) sample(c(0,1,NA), 1e6, prob=c(.5, .49, .01), replace=TRUE))
## names(dat) <- paste("indiv", 1:length(dat))
## print(object.size(dat), unit="aut")

## system.time(x <- new("genlight", dat)) # conversion + time taken
## print(object.size(x), unit="au")
## object.size(dat)/object.size(x) # conversion efficiency


## ## time taken by subsetting (quite long, +- 35sec)
## system.time(y <- x[1:10, 1:5e5])



## c, cbind, rbind ##
## a <- new("genlight", list(c(1,0,1), c(0,0,1,0)) )
## b <- new("genlight", list(c(1,0,1,1,1,1), c(1,0)) )
## locNames(a) <- letters[1:4]
## locNames(b) <- 1:6
## c <- cbind(a,b)
## identical(as.matrix(c),cbind(as.matrix(a), as.matrix(b))) # MUST BE TRUE
## identical(as.matrix(rbind(a,a)),rbind(as.matrix(a),as.matrix(a)))




## test subsetting with/without @other ##
## x <- new("genlight", list(a=1,b=0,c=1), other=list(1:3, letters, data.frame(2:4)))
## pop(x) <- c("pop1","pop1", "pop2")

#' Maximum-likelihood genetic clustering using EM algorithm
#'
#' The function \code{snapclust} implements fast maximum-likelihood (ML) genetic
#' clustering using a 2-step approach: find a 'good' initial starting point
#' using a fast geometric (distance-based) approach, followed by an efficient
#' likelihood optimization using the expectation-maximization (EM)
#' algorithm. The likelihood for a given locus is defined as the probability of
#' a given genotype given the allele frequencies of its originating population,
#' as defined by Hardy-Weinberg equilibrium. Different approaches are available
#' for the initial distance-based clustering (see argument \code{pop.ini}). The
#' current implementation is designed for haploid or diploid data, and assumes
#' constant ploidy within the dataset.
#'
#' This function can be used in two modes:
#'
#' \itemize{
#'
#'  \item genetic clustering mode: assign individuals to a known number 'k'
#'  of panmictic clusters; see \code{\link{snapclust.choose.k}} for identifying
#'  optimal values of 'k'.
#'
#'  \item hybrid mode: as previous mode, but with the additional constraint that
#'  clusters are either one of the 2 parental populations, or any hybrid classes
#'  in between, as defined by the parameter \code{hybrid.coef}.
#'
#' }
#'
#' This method is documented in an online tutorial, which can be opened by typing:
#' \code{adegenetTutorial("snapclust")}.
#' 
#' @seealso
#' \itemize{
#'  \item \code{\link{snapclust.choose.k}} to choose 'k' using various
#'     goodness-of-fit statistics
#'
#'  \item \code{\link{AIC.snapclust}}: AIC computation
#'
#'  \item \code{\link{AICc.snapclust}}: AICc computation
#'
#'  \item \code{\link{BIC.snapclust}}: BIC computation
#'
#'  \item \code{\link{KIC.snapclust}}: KIC computation
#'
#'  }
#'
#' @author Thibaut Jombart \email{thibautjombart@@gmail.com} and Marie-Pauline
#' Beugin
#'
#' @export
#'
#' @rdname snapclust
#'
#' @param x a \linkS4class{genind} object
#'
#' @param k the number of clusters to look for
#'
#' @param pop.ini parameter indicating how the initial group membership should
#'     be found. If \code{NULL}, groups are chosen at random, and the algorithm
#'     will be run \code{n.start} times. If "kmeans", then the function
#'     \code{find.clusters} is used to define initial groups using the K-means
#'     algorithm. If "ward", then the function \code{find.clusters} is used to
#'     define initial groups using the Ward algorithm. Alternatively, a factor
#'     defining the initial cluster configuration can be provided.
#'
#' @param max.iter the maximum number of iteration of the EM algorithm
#'
#' @param n.start the number of times the EM algorithm is run, each time with
#' different random starting conditions
#'
#' @param n.start.kmeans the number of times the K-means algorithm is run to
#'     define the starting point of the ML-EM algorithm, each time with
#'     different random starting conditions
#'
#' @param hybrids a logical indicating if hybrids should be modelled
#'     explicitely; this is currently implemented for 2 groups only.
#'
#' @param dim.ini the number of PCA axes to retain in the dimension reduction
#'     step for \code{\link{find.clusters}}, if this method is used to define
#'     initial group memberships (see argument \code{pop.ini}).
#'
#' @param hybrid.coef a vector of hybridization coefficients, defining the
#'     proportion of hybrid gene pool coming from the first parental population;
#'     this is symmetrized around 0.5, so that e.g. c(0.25, 0.5) will be
#'     converted to c(0.25, 0.5, 0.75)
#'
#' @param parent.lab a vector of 2 character strings used to label the two
#'     parental populations; only used if hybrids are detected (see argument
#'     \code{hybrids})
#'
#' @param ... further arguments passed on to \code{\link{find.clusters}}
#'
#' @return
#'
#' The function \code{snapclust} returns a list with the following
#' components:
#' \itemize{
#'
#' \item \code{$group} a factor indicating the maximum-likelihood assignment of
#' individuals to groups; if identified, hybrids are labelled after
#' hybridization coefficients, e.g. 0.5_A - 0.5_B for F1, 0.75_A - 0.25_B for
#' backcross F1 / A, etc.
#'
#' \item \code{$ll}: the log-likelihood of the model
#'
#' \item \code{$proba}: a matrix of group membership probabilities, with
#' individuals in rows and groups in columns; each value correspond to the
#' probability that a given individual genotype was generated under a given
#' group, under Hardy-Weinberg hypotheses.
#'
#' \item \code{$converged} a logical indicating if the algorithm converged; if
#' FALSE, it is doubtful that the result is an actual Maximum Likelihood
#' estimate.
#'
#' \item \code{$n.iter} an integer indicating the number of iterations the EM
#' algorithm was run for.
#'
#' }
#'
#' @references Beugin M-P, Gayet T, Pontier D, Devillard S, Jombart T. A fast
#'     likelihood solution to the genetic clustering problem. Methods Ecol
#'     Evol. 2018;00:1–11. \url{https://doi.org/10.1111/2041-210X.12968}
#' 
#' @examples
#' \dontrun{
#' data(microbov)
#'
#' ## try function using k-means initialization
#' grp.ini <- find.clusters(microbov, n.clust=15, n.pca=150)
#'
#' ## run EM algo
#' res <- snapclust(microbov, 15, pop.ini = grp.ini$grp)
#' names(res)
#' res$converged
#' res$n.iter
#'
#' ## plot result
#' compoplot(res)
#'
#' ## flag potential hybrids
#' to.flag <- apply(res$proba,1,max)<.9
#' compoplot(res, subset=to.flag, show.lab=TRUE,
#'                  posi="bottomleft", bg="white")
#'
#'
#' ## Simulate hybrids F1
#' zebu <- microbov[pop="Zebu"]
#' salers <- microbov[pop="Salers"]
#' hyb <- hybridize(zebu, salers, n=30)
#' x <- repool(zebu, salers, hyb)
#'
#' ## method without hybrids
#' res.no.hyb <- snapclust(x, k=2, hybrids=FALSE)
#' compoplot(res.no.hyb, col.pal=spectral, n.col=2)
#'
#' ## method with hybrids
#' res.hyb <- snapclust(x, k=2, hybrids=TRUE)
#' compoplot(res.hyb, col.pal =
#'           hybridpal(col.pal = spectral), n.col = 2)
#'
#'
#' ## Simulate hybrids backcross (F1 / parental)
#' f1.zebu <- hybridize(hyb, zebu, 20, pop = "f1.zebu")
#' f1.salers <- hybridize(hyb, salers, 25, pop = "f1.salers")
#' y <- repool(x, f1.zebu, f1.salers)
#'
#' ## method without hybrids
#' res2.no.hyb <- snapclust(y, k = 2, hybrids = FALSE)
#' compoplot(res2.no.hyb, col.pal = hybridpal(), n.col = 2)
#'
#' ## method with hybrids F1 only
#' res2.hyb <- snapclust(y, k = 2, hybrids = TRUE)
#' compoplot(res2.hyb, col.pal = hybridpal(), n.col = 2)
#'
#' ## method with back-cross
#' res2.back <- snapclust(y, k = 2, hybrids = TRUE, hybrid.coef = c(.25,.5))
#'  compoplot(res2.back, col.pal = hybridpal(), n.col = 2)
#'
#' }

snapclust <- function(x, k, pop.ini = "ward", max.iter = 100, n.start = 10,
                      n.start.kmeans = 50, hybrids = FALSE, dim.ini = 100,
                      hybrid.coef = NULL, parent.lab = c('A', 'B'), ...) {

  if (!is.genind(x)) {
    stop("x is not a valid genind object")
  }

  if (any(ploidy(x) > 2)) {
    stop("snapclust not currently implemented for ploidy > 2")
  }

  if (all(ploidy(x) == 1)) {
    .ll.genotype <- .ll.genotype.haploid
  } else if (all(ploidy(x) == 2)) {
    .ll.genotype <- .ll.genotype.diploid
  } else {
    stop("snapclust not currently implemented for varying ploidy")
  }

  ## This function uses the EM algorithm to find ML group assignment of a set
  ## of genotypes stored in a genind object into 'k' clusters. We need an
  ## initial cluster definition to start with. The rest of the algorithm
  ## consists of:

  ## i) compute the matrix of allele frequencies

  ## ii) compute the likelihood of each genotype for each group

  ## iii) assign genotypes to the group for which they have the highest
  ## likelihood

  ## iv) go back to i) until convergence


  ## Disable multiple starts if the initial condition is not random
  use.random.start <- is.null(pop.ini)
  if (!use.random.start) {
    n.start <- 1L
  }

  if (n.start < 1L) {
    stop(sprintf(
      "n.start is less than 1 (%d); using n.start=1", n.start))
  }

  if (hybrids && k > 2) {
    warning(sprintf(
      "forcing k=2 for hybrid mode (requested k is %d)", k))
    k <- 2
  }


  ## Handle hybrid coefficients; these values reflect the contribution of the
  ## first parental population to the allele frequencies of the hybrid
  ## group. For instance, a value of 0.75 indicates that 'a' contributes to
  ## 75%, and 'b' 25% of the allele frequencies of the hybrid - a typical
  ## backcross F1 / a.

  if (hybrids) {
    if (is.null(hybrid.coef)) {
      hybrid.coef <- 0.5
    }
    hybrid.coef <- .tidy.hybrid.coef(hybrid.coef)
  }


  ## Initialisation using 'find.clusters'
  if (!is.null(pop.ini)) {
    if (tolower(pop.ini)[1] %in% c("kmeans", "k-means")) {
      pop.ini <- find.clusters(x, n.clust = k, n.pca = dim.ini,
                               n.start = n.start.kmeans,
                               method = "kmeans", ...)$grp
    } else if (tolower(pop.ini)[1] %in% c("ward")) {
      pop.ini <- find.clusters(x, n.clust = k, n.pca = dim.ini,
                               method = "ward", ...)$grp
    }
  }

  ## There is one run of the EM algo for each of the n.start random initial
  ## conditions.
  ll <- -Inf # this will be the total loglike

  for (i in seq_len(n.start)) {

    ## Set initial conditions: if initial pop is NULL, we create a random
    ## group definition (each clusters have same probability)
    if (use.random.start) {
      pop.ini <- sample(seq_len(k), nInd(x), replace=TRUE)
    }

    ## process initial population, store levels
    pop.ini <- factor(pop.ini)
    lev.ini <- levels(pop.ini)[1:k] # k+1 would be hybrids

    ## ensure 'pop.ini' matches 'k'
    if (! (length(levels(pop.ini)) %in% c(k, k + length(hybrid.coef))) ) {
      stop("pop.ini does not have k clusters")
    }

    ## initialisation
    group <- factor(as.integer(pop.ini)) # set levels to 1:k (or k+1)
    genotypes <- tab(x)
    n.loc <- nLoc(x)
    counter <- 0L
    converged <- FALSE


    ## This is the actual EM algorithm

    while(!converged && counter<=max.iter) {

      ## get table of allele frequencies (columns) by population (rows);
      ## these are stored as 'pop.freq'; note that it will include extra
      ## rows for different types of hybrids too.

      if (hybrids) {
        pop(x) <- group
        id.parents <- .find.parents(x)
        x.parents <- x[id.parents]
        pop.freq <- tab(genind2genpop(x.parents, quiet=TRUE),
                        freq=TRUE)
        pop.freq <- rbind(pop.freq, # parents
                          .find.freq.hyb(pop.freq, hybrid.coef)) # hybrids
      } else {
        pop.freq <- tab(genind2genpop(x, pop=group, quiet=TRUE),
                        freq=TRUE)
      }

      ## ensures no allele frequency is exactly zero
      pop.freq <- .tidy.pop.freq(pop.freq, locFac(x))

      ## get likelihoods of genotypes in every pop
      ll.mat <- apply(genotypes, 1, .ll.genotype, pop.freq, n.loc)

      ## assign individuals to most likely cluster
      previous.group <- group
      group <- apply(ll.mat, 2, which.max)

      ## check convergence
      ## converged <- all(group == previous.group)
      old.ll <- .global.ll(previous.group, ll.mat)
      new.ll <- .global.ll(group, ll.mat)
      if (!is.finite(new.ll)) {
        ## stop(sprintf("log-likelihood at iteration %d is not finite (%f)",
        ##              counter, new.ll))
      }
      converged <- abs(old.ll - new.ll) < 1e-14
      counter <- counter + 1L

    }

    ## ## store the best run so far
    ## new.ll <- .global.ll(group, ll.mat)

    if (new.ll > ll || i == 1L) {
      ## store results
      ll <- new.ll
      out <- list(group = group, ll = ll)

      ## group membership probability
      rescaled.ll.mat <- .rescale.ll.mat(ll.mat)
      out$proba <- prop.table(t(exp(rescaled.ll.mat)), 1)

      out$converged <- converged
      out$n.iter <- counter
    }
  } # end of the for loop

  ## restore labels of groups
  out$group <- factor(out$group)
  if (hybrids) {
    if (!is.null(parent.lab)) {
      lev.ini <- parent.lab
    }
    hybrid.labels <- paste0(hybrid.coef, "_", lev.ini[1], "-",
                            1 - hybrid.coef, "_", lev.ini[2])
    lev.ini <- c(lev.ini, hybrid.labels)
  }
  levels(out$group) <- lev.ini
  colnames(out$proba) <- lev.ini


  ## compute the number of parameters; it is defined as the number of 'free'
  ## allele frequencies, multiplied by the number of groups

  out$n.param <- (ncol(genotypes) - n.loc) * length(lev.ini)

  class(out) <- c("snapclust", "list")
  return(out)
}











## Non-exported function which computes the log-likelihood of a genotype in
## every population. For now only works for diploid individuals. 'x' is a vector
## of allele counts; 'pop.freq' is a matrix of group allele frequencies, with
## groups in rows and alleles in columns.

## TODO: extend this to various ploidy levels, possibly optimizing procedures
## for haploids.

.ll.genotype.diploid <- function(x, pop.freq, n.loc){
  ## homozygote (diploid)
  ## p(AA) = f(A)^2 for each locus
  ## so that log(p(AA)) = 2 * log(f(A))
  ll.homoz.one.indiv <- function(f) {
    sum(log(f[x == 2L]), na.rm = TRUE) * 2
  }

  ll.homoz <- apply(pop.freq, 1, ll.homoz.one.indiv)

  ## heterozygote (diploid, expl with 2 loci)
  ## p(AB) = 2 * f(A) f(B)
  ## so that log(p(AB)) = log(f(A)) + log(f(B)) + log(2)
  ## if an individual is heterozygote for n.heter loci, the term
  ## log(2) will be added n.heter times

  ll.hetero.one.indiv <- function(f) {
    n.heter <- sum(x == 1L, na.rm = TRUE) / 2
    sum(log(f[x == 1L]), na.rm = TRUE) + n.heter * log(2)
  }

  ll.heteroz <- apply(pop.freq, 1, ll.hetero.one.indiv)

  return(ll.homoz + ll.heteroz)
}



.ll.genotype.haploid <- function(x, pop.freq, n.loc){

  ## p(A) = f(A) for each locus
  ll.one.indiv <- function(f) {
    sum(log(f[x == 1L]), na.rm = TRUE)
  }

  ll <- apply(pop.freq, 1, ll.one.indiv)

  return(ll)
}






## Non-exported function computing the total log-likelihood of the model given a vector of group
## assignments and a table of ll of genotypes in each group

.global.ll <- function(group, ll){
  sum(t(ll)[cbind(seq_along(group), as.integer(group))], na.rm=TRUE)
}





## Non-exported function making a tidy vector of weights for allele frequencies
## of parental populations. It ensures that given any input vector of weights
## 'w' defining the types of hybrids, the output has the following properties:

## - strictly on ]0,1[

## - symmetric around 0.5, e.g. c(.25, .5) gives c(.25, .5, .75)

## - sorted by decreasing values (i.e. hybrid types are sorted by decreasing
## proximity to the first parental population.

.tidy.hybrid.coef <- function(w) {
  w <- w[w > 0 & w < 1]
  w <- sort(unique(round(c(w, 1-w), 4)), decreasing = TRUE)
  w
}





## Non-exported function determining vectors of allele frequencies in hybrids
## from 2 parental populations. Different types of hybrids are determined by
## weights given to the allele frequencies of the parental populations. Only one
## such value is provided and taken to be the weight of the 1st parental
## population; the complementary frequency is derived for the second parental
## population.

## Parameters are:

## - x: matrix of allele frequencies for population 'a' (first row) and 'b'
## (second row), where allele are in columns.


## - w: a vector of weights for 'a' and 'b', each value determining a type of
## hybrid. For instance, 0.5 is for F1, 0.25 for backcrosses F1/parental, 0.125
## for 2nd backcross F1/parental, etc.

## The output is a matrix of allele frequencies with hybrid types in rows and
## alleles in columns.

.find.freq.hyb <- function(x, w) {
  out <- cbind(w, 1-w) %*% x
  rownames(out) <- w
  out
}




## Non-exported function trying to find the two parental populations in a genind
## object containing 'k' clusters. The parental populations are defined as the
## two most distant clusters. The other clusters are deemed to be various types
## of hybrids. The output is a vector of indices identifying the individuals
## from the parental populations.

.find.parents <- function(x) {
  ## matrix of pairwise distances between clusters, using Nei's distance
  D <- as.matrix(dist.genpop(genind2genpop(x, quiet = TRUE), method = 1))
  parents <- which(abs(max(D)-D) < 1e-14, TRUE)[1,]
  out <- which(as.integer(pop(x)) %in% parents)
  out
}





## Non-exported function enforcing a minimum allele frequency in a table of
## allele frequency. As we are not accounting for the uncertainty in allele
## frequencies, we need to allow for genotypes to be generated from a population
## which does not have the genotype's allele represented, even if this is at a
## low probability. The transformation is ad-hoc, and has the form:
##
## g(f_i) = (a + f_i /  \sum(a + f_i))

## where f_i is the i-th frequency in a given locus. However, this ensures that
## the output has two important properties:

## - it sums to 1
## - it contains no zero

## By default, we set 'a' to 0.01.

## Function inputs are:

## - 'pop.freq': matrix of allele frequencies, with groups in rows and alleles in
## columns

## - 'loc.fac': a factor indicating which alleles belong to which locus, as
## returned by 'locFac([a genind])'

.tidy.pop.freq <- function(pop.freq, loc.fac) {
  g <- function(f, a = .01) {
    (a + f) / sum(a + f)
  }

  out <- matrix(unlist(apply(pop.freq, 1, tapply, loc.fac, g),
                       use.names = FALSE),
                byrow=TRUE, nrow=nrow(pop.freq))
  dimnames(out) <- dimnames(pop.freq)
  return(out)
}



## This function rescales log-likelihood values prior to the computation of
## group membership probabilities.

## issue reported: prop.table(t(exp(ll.mat)), 1) can cause some numerical
## approximation problems; if numbers are large, exp(...) will return Inf
## and the group membership probabilities cannot be computed
##
## Solution: rather than use p_a = exp(ll_a) / (exp(ll_a) + exp(ll_b))
## we can use p_a = exp(ll_a - C) /  (exp(ll_a - C) + exp(ll_b - C))

## where 'C' is a sufficiently large constant so that exp(ll_i + C) is
## computable; naively we could use C = max(ll.mat), but the problem is this
## scaling can cause -Inf likelihoods too. In practice, we need to allow
## different scaling for each individual.

##out$proba <-
## prop.table(t(exp(ll.mat)), 1)


.rescale.ll.mat <- function(ll.mat) {
  ## we first compute ad-hoc minimum and maximum values of log-likelihood; these
  ## will be computer dependent; this is a quick fix, but better alternatives
  ## can be found.

  ## smallest ll such that exp(ll) is strictly > 0
  new_min <- (0:-1000)[max(which(exp(0:-1000) > 0))]

  ## largest ll such that exp(ll) is strictly less than +Inf
  new_max <- (1:1000)[max(which(exp(1:1000) < Inf))]

  counter <- 0


  ## find rescaling for a single individual;
  ## x: vector of ll values
  rescale.ll.indiv <- function(x) {
    ## set minimum to new_min
    x  <- x - min(x) + new_min

    ## set sum to the maximum
    if (sum(x) > new_max) {
      counter <<- counter + 1
      x <- x - min(x) # reset min to 0
      x <- new_min + (x / sum(x)) * new_max # range: new_min to new_max
    }
    return(x)
  }

  out <- apply(ll.mat, 2, rescale.ll.indiv)

  if (counter > 0) {
    msg <- paste("Large dataset syndrome:\n",
                 "for", counter, "individuals,",
                 "differences in log-likelihoods exceed computer precision;\n",
                 "group membership probabilities are approximated\n",
                 "(only trust clear-cut values)")
    message(msg)
  }

  return(out)
}

########################################################################
# hierarchy methods definitions. 
#
# Zhian Kamvar, March 2015
# kamvarz@science.oregonstate.edu
########################################################################
.getHier <- function(x, formula = NULL, combine = TRUE, call = match.call()){
  if (is.null(x@strata)){
	  warning("Strata must be specified before defining a hierarchy. See ?strata for details.", call. = FALSE)
	  return(NULL)
	}
  if (is.null(x@hierarchy)){
    warning("A hierarchy must be defined before retrieving it.", call. = FALSE)
    return(NULL)
  }
  if (is.null(formula)){
    if (!is.null(x@hierarchy)){
      # Returning the base hierarchy
      return(.getHier(x, formula = x@hierarchy, combine = combine, call = call))      
    } else {
      return(NULL)
    }
  }
  vars <- all.vars(formula)
  if (any(!vars %in% names(x@strata))){
    stop(.strata_incompatible_warning(vars, x@strata), call. = FALSE)
  }
  if (!.test_existing_hier(vars, all.vars(x@hierarchy))){
  	callform <- as.character(call["formula"])
    lines <- paste(rep("-", 78), collapse = "")
  	stop(paste0("\n", lines, "\n", callform, "\nis not a subset of\n", 
         paste(x@hierarchy, collapse = "")), call. = FALSE)
  }
  if (combine){
    strata <- .make_strata(formula, x@strata)
  } else {
    strata <- x@strata[all.vars(formula)]
  }
  invisible(return(strata))
}

.setHier <- function(x, value, call = match.call()){
  if (is.null(value)){
    x@hierarchy <- value
    return(x)
  }
  if (is.null(x@strata)){
  	warning("Strata must be specified before defining a hierarchy. See ?strata for details.", call. = FALSE)
  	return(NULL)
  }
  if (!is.language(value)){
    callval <- as.character(call["value"])
    stop(paste(callval, "is not a formula"), call. = FALSE)
  }
  vars <- all.vars(value)
  if (any(!vars %in% names(x@strata))){
    stop(.strata_incompatible_warning(vars, x@strata), call. = FALSE)
  }

  ## TODO: Insert Jerome's method of checking whether or not the hierarchy is 
  ##       truly hierarchical.
  
  x@hierarchy <- value
  return(x)
}

.test_existing_hier <- function(query, hier){
	matches <- match(hier, query)
	matches <- matches[!is.na(matches)]
	if (all(matches == sort(matches))){
		return(TRUE)
	} else {
		return(FALSE)
	}
}

#==============================================================================#
#' Access and manipulate the population hierarchy for genind or genlight objects.
#' 
#' The following methods allow the user to quickly change the hierarchy or
#' population of a genind or genlight object. 
#' 
#' @export
#' @rdname hierarchy-methods
#' @aliases hier,genind-method hier,genlight-method
#' @param x a genind or genlight object
#' @param formula a nested formula indicating the order of the population 
#'   hierarchy to be returned.
#' @param combine if \code{TRUE} (default), the levels will be combined
#'   according to the formula argument. If it is \code{FALSE}, the levels will
#'   not be combined.
#' @param value a formula specifying the full hierarchy of columns in the strata
#'   slot. \strong{(See Details below)}
#' @docType methods
#'   
#' @details You must first specify your strata before you can specify your 
#'   hierarchies. Hierarchies are special cases of strata in that the levels 
#'   must be nested within each other. An error will occur if you specify a 
#'   hierarchy that is not truly hierarchical.
#'   
#'   \subsection{Details on Formulas}{
#'   
#'   The preferred use of these functions is with a \code{\link{formula}} 
#'   object. Specifically, a hierarchical formula argument is used to name which
#'   strata are hierarchical. An example of a hierarchical formula would
#'   be:\tabular{r}{ \code{~Country/City/Neighborhood}} This convention was
#'   chosen as it becomes easier to type and makes intuitive sense when defining
#'   a hierarchy. Note: it is important to use hierarchical formulas when
#'   specifying hierarchies as other types of formulas (eg. 
#'   \code{~Country*City*Neighborhood}) will give incorrect results.}
#'   
#' @seealso \code{\link{strata}} \code{\link{genind}}
#'   \code{\link{as.genind}}
#'   
#' @author Zhian N. Kamvar
#' @examples
#' # let's look at the microbov data set:
#' data(microbov)
#' microbov
#' 
#' # We see that we have three vectors of different names in the 'other' slot. 
#' ?microbov
#' # These are Country, Breed, and Species
#' names(other(microbov))
#' 
#' # Let's set the hierarchy
#' strata(microbov) <- data.frame(other(microbov))
#' microbov
#' 
#' # And change the names so we know what they are
#' nameStrata(microbov) <- ~Country/Breed/Species
#' 
#' # let's see what the hierarchy looks like by Species and Breed:
#' hier(microbov) <- ~Species/Breed
#' head(hier(microbov, ~Species/Breed))
#' 
#==============================================================================#
hier <- function(x, formula = NULL, combine = TRUE, value){
  standardGeneric("hier")
} 

#' @export
setGeneric("hier")

setMethod(
  f = "hier",
  signature(x = "genind"),
  definition = function(x, formula = NULL, combine = TRUE, value){
    theCall <- match.call()
    if (missing(value)){
      .getHier(x, formula = formula, combine = combine, theCall)  
    } else {
      .setHier(x, value, theCall)
    }
  })

setMethod(
  f = "hier",
  signature(x = "genlight"),
  definition = function(x, formula = NULL, combine = TRUE, value){
    theCall <- match.call()
    if (missing(value)){
      .getHier(x, formula = formula, combine = combine, theCall)  
    } else {
      .setHier(x, value, theCall)
    }
    
  })


#==============================================================================#
#' @export 
#' @rdname hierarchy-methods
#' @aliases hier<-,genind-method hier<-,genlight-method
#' @docType methods
#==============================================================================#
"hier<-" <- function(x, value){
  standardGeneric("hier<-")
}  

#' @export
setGeneric("hier<-")

setMethod(
  f = "hier<-",
  signature(x = "genind"),
  definition = function(x, value){
    theCall <- match.call()
    return(.setHier(x, value, theCall))
  })

setMethod(
  f = "hier<-",
  signature(x = "genlight"),
  definition = function(x, value){
    theCall <- match.call()
    return(.setHier(x, value, theCall))
  })

############
## haploGen
############
##
## N: number of sequences to simulate
## mu: mutation rate per nucleotid per generation
## Tmax: periode of time to simulate
## mean.gen.time, sd.gen.time: average time for transmission and its standard deviation (normal dist)
## mean.repro, sd.repro: average number of transmissions and its standard deviation (normal dist)
##
haploGen <- function(seq.length=1e4, mu.transi=1e-4, mu.transv=mu.transi/2, t.max=20,
                     gen.time=function(){1+rpois(1,0.5)},
                     repro=function(){rpois(1,1.5)}, max.nb.haplo=200,
                     geo.sim=FALSE, grid.size=10, lambda.xy=0.5,
                     mat.connect=NULL,
                     ini.n=1, ini.xy=NULL){

    ## CHECKS ##

    ## HANDLE ARGUMENTS ##
    ## if numeric value, make it a function
    if(is.numeric(gen.time)){
        gen.time.val <- gen.time[1]
        gen.time <- function(){return(gen.time.val)}
    }

    ## if numeric value, make it a function
    if(is.numeric(repro)){
        repro.val <- repro[1]
        repro <- function(){return(repro.val)}
    }



    ## GENERAL VARIABLES ##
    NUCL <- as.DNAbin(c("a","t","c","g"))
    TRANSISET <- list('a'=as.DNAbin('g'), 'g'=as.DNAbin('a'), 'c'=as.DNAbin('t'), 't'=as.DNAbin('c'))
    TRANSVSET <- list('a'=as.DNAbin(c('c','t')), 'g'=as.DNAbin(c('c','t')), 'c'=as.DNAbin(c('a','g')), 't'=as.DNAbin(c('a','g')))
    res <- list(seq = as.matrix(as.DNAbin(matrix(character(0), nrow = 0, ncol = 0))), 
                dates = integer(), 
                ances = character())
    toExpand <- logical()
    myGrid <- matrix(1:grid.size^2, ncol=grid.size, nrow=grid.size)


    ## AUXILIARY FUNCTIONS ##
    ## generate sequence from scratch
    seq.gen <- function(){
        ##res <- list(sample(NUCL, size=seq.length, replace=TRUE)) # DNAbin are no longer lists by default
        res <- sample(NUCL, size=seq.length, replace=TRUE)
        class(res) <- "DNAbin"
        return(res)
    }

    ## create substitutions for defined SNPs - no longer used
    substi <- function(snp){
        res <- sapply(1:length(snp), function(i) sample(setdiff(NUCL,snp[i]),1)) # ! sapply does not work on DNAbin vectors directly
        class(res) <- "DNAbin"
        return(res)
    }

    ## create transitions for defined SNPs
    transi <- function(snp){
        res <- unlist(TRANSISET[as.character(snp)])
        class(res) <- "DNAbin"
        return(res)
    }

    ## create transversions for defined SNPs
    transv <- function(snp){
        res <- sapply(TRANSVSET[as.character(snp)],sample,1)
        class(res) <- "DNAbin"
        return(res)
    }

    ## duplicate a sequence (including possible mutations)
    seq.dupli <- function(seq, T){ # T is the number of time units between ancestor and decendent
        ## transitions ##
        n.transi <- rbinom(n=1, size=seq.length*T, prob=mu.transi) # total number of transitions
        if(n.transi>0) {
            idx <- sample(1:seq.length, size=n.transi, replace=FALSE)
            seq[idx] <- transi(seq[idx])
        }

        ## transversions ##
        n.transv <- rbinom(n=1, size=seq.length*T, prob=mu.transv) # total number of transitions
        if(n.transv>0) {
            idx <- sample(1:seq.length, size=n.transv, replace=FALSE)
            seq[idx] <- transv(seq[idx])
        }
        return(seq)
    }

    ## what is the name of the new sequences?
    seqname.gen <- function(nb.new.seq){
        res <- max(as.integer(rownames(res$seq)), 0) + 1:nb.new.seq
        return(as.character(res))
    }

    ## how many days before duplication occurs ?
    time.dupli <- function(){
        ##res <- round(rnorm(1, mean=mean.gen.time, sd=sd.gen.time))
        res <- round(gen.time()) # force integers
        res[res<0] <- 0
        return(res)
    }

    ## when duplication occurs?
    date.dupli <- function(curDate){
        res <- curDate + time.dupli()
        return(res)
    }

    ## how many duplication/transmission occur?
    nb.desc <- function(){
        ##res <- round(rnorm(1, mean=mean.repro, sd=sd.repro))
        res <- repro()
        res[res<0] <- 0
        return(res)
    }

    ## where does an haplotype emerges in the first place?
    xy.gen <- function(){
        return(sample(1:grid.size, size=2, replace=TRUE))
    }

    ## where does a transmission occur (destination)?
    if(is.null(mat.connect)){ # use universal lambda param
        xy.dupli <- function(cur.xy, nbLoc){
            mvt <- rpois(2*nbLoc, lambda.xy) * sample(c(-1,1), size=2*nbLoc, replace=TRUE)
            res <- t(matrix(mvt, nrow=2) + as.vector(cur.xy))
            res[res < 1] <- 1
            res[res > grid.size] <- grid.size
            return(res)
        }
    } else { # use location-dependent proba of dispersal between locations
        if(any(mat.connect < 0)) stop("Negative values in mat.connect (probabilities expected!)")
        mat.connect <- prop.table(mat.connect,1)
        xy.dupli <- function(cur.xy, nbLoc){
            idxAncesLoc <- myGrid[cur.xy[1], cur.xy[2]]
            newLoc <- sample(1:grid.size^2, size=nbLoc, prob=mat.connect[idxAncesLoc,], replace=TRUE) # get new locations
            res <- cbind(row(myGrid)[newLoc] , col(myGrid)[newLoc]) # get coords of new locations
            return(res)
        }
    }


    ## check result size and resize it if needed
    resize.result <- function(){
        curSize <- length(res$dates)
        if(curSize <= max.nb.haplo) return(NULL)
        toKeep <- rep(FALSE, curSize)
        toKeep[sample(1:curSize, size=max.nb.haplo, replace=FALSE)] <- TRUE
        removed.strains <- rownames(res$seq)[!toKeep]
        res$seq <<- res$seq[toKeep,]
        res$dates <<- res$dates[toKeep]
        res$ances <<- res$ances[toKeep]
        toExpand <<- toExpand[toKeep]
        temp <- as.character(res$ances) %in% removed.strains
        if(any(temp)) {
            res$ances[temp] <<- NA
        }

        return(NULL)
    }

    ## check result size and resize it if needed - spatial version
    resize.result.xy <- function(){
        curSize <- length(res$dates)
        if(curSize <= max.nb.haplo) return(NULL)
        toKeep <- rep(FALSE, curSize)
        toKeep[sample(1:curSize, size=max.nb.haplo, replace=FALSE)] <- TRUE
        removed.strains <- rownames(res$seq)[!toKeep]
        res$seq <<- res$seq[toKeep,]
        res$dates <<- res$dates[toKeep]
        res$ances <<- res$ances[toKeep]
        res$xy <<- res$xy[toKeep,,drop=FALSE]
        toExpand <<- toExpand[toKeep]
        temp <- as.character(res$ances) %in% removed.strains
        if(any(temp)) {
            res$ances[temp] <<- NA
        }

        return(NULL)
    }



    ## MAIN SUB-FUNCTION: EXPANDING FROM ONE SEQUENCE - NON SPATIAL ##
    expand.one.strain <- function(seq, date, idx){
        toExpand[idx] <<- FALSE # this one is no longer to expand
        nbDes <- nb.desc()
        if(nbDes==0) return(NULL) # stop if no descendant
        newDates <- sapply(1:nbDes, function(i) date.dupli(date)) # find dates for descendants
        newDates <- newDates[newDates <= t.max] # don't store future sequences
        nbDes <- length(newDates)
        if(nbDes==0) return(NULL) # stop if no suitable date
        newSeq <- lapply(1:nbDes, function(i) seq.dupli(seq, newDates[i]-date)) # generate new sequences
        class(newSeq) <- "DNAbin" # lists of DNAbin vectors must also have class "DNAbin"
        newSeq <- as.matrix(newSeq) # list DNAbin -> matrix DNAbin with nbDes rows
        rownames(newSeq) <- seqname.gen(nbDes) # find new labels for these new sequences
        res$seq <<- rbind(res$seq, newSeq) # append to general output
        res$dates <<- c(res$dates, newDates) # append to general output
        res$ances <<- c(res$ances, rep(rownames(res$seq)[idx], nbDes)) # append to general output
        toExpand <<- c(toExpand, rep(TRUE, nbDes))
        return(NULL)
    }


    ## 2nd MAIN SUB-FUNCTION: EXPANDING FROM ONE SEQUENCE - SPATIAL ##
    expand.one.strain.xy <- function(seq, date, idx, cur.xy){
        toExpand[idx] <<- FALSE # this one is no longer to expand
        nbDes <- nb.desc()
        if(nbDes==0) return(NULL) # stop if no descendant
        newDates <- sapply(1:nbDes, function(i) date.dupli(date)) # find dates for descendants
        newDates <- newDates[newDates <= t.max] # don't store future sequences
        nbDes <- length(newDates)
        if(nbDes==0) return(NULL) # stop if no suitable date
        newSeq <- lapply(1:nbDes, function(i) seq.dupli(seq, newDates[i]-date)) # generate new sequences
        class(newSeq) <- "DNAbin" # lists of DNAbin vectors must also have class "DNAbin"
        newSeq <- as.matrix(newSeq) # list DNAbin -> matrix DNAbin with nbDes rows
        rownames(newSeq) <- seqname.gen(nbDes) # find new labels for these new sequences
        res$seq <<- rbind(res$seq, newSeq) # append to general output
        res$dates <<- c(res$dates, newDates) # append to general output
        res$ances <<- c(res$ances, rep(rownames(res$seq)[idx], nbDes)) # append to general output
        res$xy <<- rbind(res$xy, xy.dupli(cur.xy, nbDes))
        toExpand <<- c(toExpand, rep(TRUE, nbDes))
        return(NULL)
    }



    ## PERFORM SIMULATIONS - NON SPATIAL CASE ##
    if(!geo.sim){
        ## initialization
        res$seq <- matrix(rep(seq.gen(), ini.n), byrow=TRUE, nrow=ini.n)
        class(res$seq) <- "DNAbin"
        rownames(res$seq) <- 1:ini.n
        res$dates[1:ini.n] <- rep(0,ini.n)
        res$ances[1:ini.n] <- rep(NA,ini.n)
        toExpand <- rep(TRUE,ini.n)

        ## simulations: isn't simplicity beautiful?
        while(any(toExpand)){
            idx <- min(which(toExpand))
            expand.one.strain(res$seq[idx,], res$dates[idx], idx)
            resize.result()
        }


        ## SHAPE AND RETURN OUTPUT ##
        res$id <- as.character(1:length(res$ances))
        res$ances <- as.character(res$ances)
        names(res$dates) <- rownames(res$seq)
        res$call <- match.call()
        class(res) <- "haploGen"
        return(res)

    } # END NON-SPATIAL SIMULATIONS



    ## PERFORM SIMULATIONS - SPATIAL CASE ##
    if(geo.sim){
        ## some checks
        if(!is.null(mat.connect)) {
            if(nrow(mat.connect) != ncol(mat.connect)) stop("mat.connect is not a square matrix")
            if(nrow(mat.connect) != grid.size^2) stop("dimension of mat.connect does not match grid size")
        }

        ## initialization
        res$seq <- matrix(rep(seq.gen(), ini.n), byrow=TRUE, nrow=ini.n)
        class(res$seq) <- "DNAbin"
        rownames(res$seq) <- 1:ini.n
        res$dates[1:ini.n] <- rep(0,ini.n)
        res$ances[1:ini.n] <- rep(NA,ini.n)
        toExpand <- rep(TRUE,ini.n)

        if(is.null(ini.xy)){
            locStart <- xy.gen()
        } else{
            locStart <- as.vector(ini.xy)[1:2]
        }
        res$xy <- matrix(rep(locStart, ini.n), byrow=TRUE, nrow=ini.n)
        colnames(res$xy) <- c("x","y")

        ##cat("nb.strains","iteration.time",file="haploGenTime.out") # for debugging


        ## simulations: isn't simplicity beautiful?
        while(any(toExpand)){
            ##time.previous <- Sys.time() # FOR DEBUGGING
            idx <- min(which(toExpand))
            expand.one.strain.xy(res$seq[idx,], res$dates[idx], idx, res$xy[idx,])
            resize.result.xy()
            ## VERBOSE OUTPUT FOR DEBUGGING ##
            ## cat("\nNb strains:",length(res$ances),"/",max.nb.haplo)
            ##             cat("\nLatest date:", max(res$dates),"/",t.max)
            ##             cat("\nRemaining strains to duplicate", sum(toExpand))
            ##             cat("\n",append=TRUE,file="haploGenTime.out")
            ##             iter.time <- as.numeric(difftime(Sys.time(),time.previous,unit="sec"))
            ##             time.previous <- Sys.time()
            ##             cat(c(length(res$ances), iter.time),append=
            ##TRUE,file="haploGenTime.out")
        ## END DEBUGGING VERBOSE ##
        }

        ## VERBOSE OUTPUT FOR DEBUGGING ##
        ##cat("\nSimulation time stored in haploGenTime.out\n")

        ## SHAPE AND RETURN OUTPUT ##
        res$id <- as.character(1:length(res$ances))
        res$ances <- as.character(res$ances)
        names(res$dates) <- rownames(res$seq)

        class(res) <- "haploGen"
        res$call <- match.call()
        return(res)

    } # end SPATIAL SIMULATIONS


} # end haploGen








##################
## print.haploGen
##################
print.haploGen <- function(x, ...){

    cat("\t\n========================")
    cat("\t\n= simulated haplotypes =")
    cat("\t\n=  (haploGen object)   =")
    cat("\t\n========================\n")

    cat("\nSize :", length(x$ances),"haplotypes")
    cat("\nHaplotype length :", ncol(x$seq),"nucleotids")
    cat("\nProportion of NA ancestors :", signif(mean(is.na(x$ances)),5))
    cat("\nNumber of known ancestors :", sum(!is.na(x$ances)))
    nbAncInSamp <- sum(x$ances %in% labels(x))
    cat("\nNumber of ancestors within the sample :", nbAncInSamp)
    cat("\nDate range :", min(x$dates,na.rm=TRUE),"-",max(x$dates,na.rm=TRUE))
    ##nUniqSeq <- length(unique(apply(as.character(x$seq),1,paste,collapse="")))
    ##cat("\nNumber of unique haplotypes :", nUniqSeq)

    cat("\n\n= Content =")
    for(i in 1:length(x)){
        cat("\n")

        cat(paste("$", names(x)[i], sep=""),"\n")
        if(names(x)[i] %in% c("seq","call")) {
            print(x[[i]])
        } else if(names(x)[i]=="xy"){
            print(head(x[[i]]))
            if(nrow(x[[i]]>6)) cat("    ...\n")
        } else cat(head(x[[i]],6), ifelse(length(x[[i]])>6,"...",""),"\n")
    }


    return(NULL)
} # end print.haploGen






##############
## [.haploGen
##############
"[.haploGen" <- function(x,i,j,drop=FALSE){
    res <- x
    res$seq <- res$seq[i,,drop=FALSE]
    res$id <- res$id[i]
    res$ances <- res$ances[i]
    res$ances[!res$ances %in% res$id] <- NA
    res$dates <- res$dates[i]
    if(!is.null(res$xy)) res$xy <- res$xy[i,,drop=FALSE]

    return(res)
}






##################
## labels.haploGen
##################
labels.haploGen <- function(object, ...){
    return(object$id)
}





#######################
## as.POSIXct.haploGen
#######################
as.POSIXct.haploGen <- function(x, tz="", origin=as.POSIXct("2000/01/01"), ...){
    res <- as.POSIXct(x$dates*24*3600, origin=origin)
    return(res)
}






#####################
## seqTrack.haploGen
#####################
seqTrack.haploGen <- function(x, best=c("min","max"), prox.mat=NULL, ...){
    myX <- dist.dna(x$seq, model="raw")
    x.names <- labels(x)
    x.dates <- as.POSIXct(x)
    seq.length <- ncol(x$seq)
    myX <- myX * seq.length
    myX <- as.matrix(myX)
    prevCall <- as.list(x$call)
    if(is.null(prevCall$mu)){
        mu0 <- 0.0001
    } else {
        mu0 <- eval(prevCall$mu)
    }
    res <- seqTrack(myX, x.names=x.names, x.dates=x.dates, best=best, prox.mat=prox.mat,...)
    return(res)
}






########################
## as.seqTrack.haploGen
########################
as.seqTrack.haploGen <- function(x){
    ## x.ori <- x
    ## x <- na.omit(x)
    toSetToNA <- x$dates==min(x$dates)
    res <- list()
    res$id <- labels(x)
    res <- as.data.frame(res)
    res$ances <- x$ances
    res$ances[toSetToNA] <- NA
    res$weight <- 1 # ??? have to recompute that...
    res$weight[toSetToNA] <- NA
    res$date <- as.POSIXct(x)[labels(x)]
    res$ances.date <- as.POSIXct(x)[x$ances]

    ## set results as indices rather than labels
    res$ances <- match(res$ances, res$id)
    res$id <- 1:length(res$id)

    ## SET CLASS
    class(res) <- c("seqTrack", "data.frame")

    return(res)
}






################
## plotHaploGen
################
plotHaploGen <- function(x, annot=FALSE, date.range=NULL, col=NULL, bg="grey", add=FALSE, ...){

    ## SOME CHECKS ##
    if(class(x)!="haploGen") stop("x is not a haploGen object")
    if(is.null(x$xy)) stop("x does not contain xy coordinates - try converting to graphNEL for plotting")


    ## ## CONVERSION TO A SEQTRACK-LIKE OBJECT ##
    xy <- x$xy
    res <- as.seqTrack.haploGen(x)

    ##     res <- list()
    ##     res$id <- labels(x)
    ##     res <- as.data.frame(res)
    ##     res$ances <- x$ances
    ##     res$ances[toSetToNA] <- NA
    ##     res$weight <- 1 # ??? have to recompute that...
    ##     res$weight[toSetToNA] <- NA
    ##     res$date <- as.POSIXct(x.ori)[labels(x)]
    ##     res$ances.date <- as.POSIXct(x.ori)[x$ances]
    ##     ## set results as indices rather than labels
    ##     res$ances <- match(res$ances, res$id)
    ##     res$id <- 1:length(res$id)


    ## CALL TO PLOTSEQTRACK ##
    plotSeqTrack(x=res, xy=xy, annot=annot, date.range=date.range,
                        col=col, bg=bg, add=add, ...)

    return(invisible(res))

} # end plotHaploGen






###################
## sample.haploGen
###################
sample.haploGen <- function(x, n){
##sample.haploGen <- function(x, n, rDate=.rTimeSeq, arg.rDate=NULL){
    ## EXTRACT THE SAMPLE ##
    res <- x[sample(1:nrow(x$seq), n, replace=FALSE)]


    ## RETRIEVE SOME PARAMETERS FROM HAPLOSIM CALL
    prevCall <- as.list(x$call)
    if(!is.null(prevCall$mu)){
        mu0 <- eval(prevCall$mu)
    } else {
        mu0 <- 1e-04
    }

    if(!is.null(prevCall$seq.length)){
        L <- eval(prevCall$seq.length)
    } else {
        L <- 1000
    }

    ## truedates <- res$dates
    ## daterange <- diff(range(res$dates,na.rm=TRUE))

    ## if(identical(rDate,.rTimeSeq)){
    ##     sampdates <- .rTimeSeq(n=length(truedates), mu=mu0, L=L, maxNbDays=daterange/2)
    ## } else{
    ##     arg.rDate$n <- n
    ##     sampdates <- do.call(rDate, arg.rDate)
    ## }
    ## sampdates <- truedates + abs(sampdates)

    return(res)
} # end sample.haploGen






######################
## as.igraph.haploGen
######################
as.igraph.haploGen <- function(x, col.pal=redpal, ...){

    ## GET DAG ##
    from <- x$ances
    to <- x$id
    isNotNA <- !is.na(from) & !is.na(to)
    dat <- data.frame(from,to,stringsAsFactors=FALSE)[isNotNA,,drop=FALSE]
    vnames <- as.character(unique(unlist(dat)))
    out <- graph.data.frame(dat, directed=TRUE, vertices=data.frame(names=vnames, dates=x$dates[vnames]))

    ## SET WEIGHTS ##
    D <- as.matrix(dist.dna(x$seq,model="raw")*ncol(x$seq))
    temp <- mapply(function(i,j) return(D[i,j]), as.integer(from), as.integer(to))
    E(out)$weight <- temp[isNotNA]


    ## DATES FOR VERTICES
    V(out)$dates <- x$date

    ## SET EDGE LABELS ##
    E(out)$label <- E(out)$weight

    ## SET EDGE COLORS
    E(out)$color <- num2col(E(out)$weight, col.pal=col.pal, reverse=TRUE)

    ## SET LAYOUT ##
    ypos <- V(out)$dates
    ypos <- abs(ypos-max(ypos))
    attr(out, "layout") <- layout.fruchterman.reingold(out, params=list(miny=ypos, maxy=ypos))

    return(out)
} # end as.igraph.haploGen






#################
## plot.haploGen
#################
plot.haploGen <- function(x, y=NULL, col.pal=redpal, ...){

    ## get graph ##
    g <- as.igraph(x, col.pal=col.pal)

    ## make plot ##
    plot(g, layout=attr(g,"layout"), ...)

    ## return graph invisibly ##
    return(invisible(g))

} # end plot.haploGen
















##########################
## as("haploGen", "graphNEL")
##########################
## if(require(graph)){
##     setOldClass("haploGen")

##     setAs("haploGen", "graphNEL", def=function(from){
##         if(!require(ape)) stop("package ape is required")
##         if(!require(graph)) stop("package graph is required")

##         N <- length(from$ances)
##         areNA <- is.na(from$ances)

##         ## EXTRACT WEIGHTS (nb of mutations)
##         M <- as.matrix(dist.dna(from$seq, model="raw")*ncol(from$seq))
##         rownames(M) <- colnames(M) <- from$id
##         w <- mapply(function(i,j) {M[i, j]}, i=from$ances[!areNA], j=from$id[!areNA])


##         ## CONVERT TO GRAPH
##         res <- ftM2graphNEL(ft=cbind(from$ances[!areNA], from$id[!areNA]), W=w, edgemode = "directed", V=from$id)
##         return(res)
##     })

## }






















## #####################
## ## seqTrackG.haploGen
## #####################
## seqTrackG.haploGen <- function(x, optim=c("min","max"), ...){
##     myX <- dist.dna(x$seq, model="raw")
##     x.names <- labels(x)
##     x.dates <- as.POSIXct(x)
##     seq.length <- ncol(x$seq)
##     myX <- myX * seq.length
##     prevCall <- as.list(x$call)
##     if(is.null(prevCall$mu)){
##         mu0 <- 0.0001
##     } else {
##         mu0 <- eval(prevCall$mu)
##     }
##     res <- seqTrackG(myX, x.names=x.names, x.dates=x.dates, best=optim,...)
##     return(res)
## }






##############################
## optimize.seqTrack.haploGen
##############################
## optimize.seqTrack.haploGen <- function(x, thres=0.2, optim=c("min","max"),
##                                        typed.chr=NULL, mu0=NULL, chr.length=NULL,
##                                        prox.mat=NULL, nstep=10, step.size=1e3,
##                                        rDate=.rTimeSeq, arg.rDate=NULL, rMissDate=.rUnifTimeSeq, ...){

##     x.names <- labels(x)
##     x.dates <- as.POSIXct(x)
##     seq.length <- ncol(x$seq)
##     myX <- dist.dna(x$seq, model="raw") * seq.length
##     prevCall <- as.list(x$call)
##     if(is.null(prevCall$mu)){
##         mu0 <- 0.0001
##     } else {
##         mu0 <- eval(prevCall$mu)
##     }

##     res <- optimize.seqTrack.default(x=myX, x.names=x.names, x.dates=x.dates,
##                                      typed.chr=typed.chr, mu0=mu0, chr.length=chr.length,
##                                      thres=thres, optim=optim, prox.mat=prox.mat,
##                                      nstep=nstep, step.size=step.size,
##                                      rDate=rDate, arg.rDate=arg.rDate, rMissDate=rMissDate, ...)
## } # end optimize.seqTrack.haploGen

#'
#' Function hybridize takes two genind in inputs
#' and generates hybrids individuals having one parent
#' in both objects.
#'
#'The function \code{hybridize} performs hybridization between two set
#' of genotypes stored in \linkS4class{genind} objects (referred as the "2
#' populations"). Allelic frequencies are derived for each population,
#' and then gametes are sampled following a multinomial distribution. \cr
#'
#' The result consists in a set of 'n' genotypes, with different possible
#' outputs (see 'res.type' argument).
#'
#'
#' @export
#'
#' @param x1 a \linkS4class{genind} object
#' @param x2 a \linkS4class{genind} object
#' @param n an integer giving the number of hybrids requested
#' @param pop a character string giving naming the population of the
#'  created hybrids.
#' @param res.type a character giving the type of output requested. Must
#'  be "genind" (default), "df" (i.e. data.frame like in
#'  \code{\link{genind2df}}), or "STRUCTURE" to generate a .str file
#'  readable by STRUCTURE (in which case the 'file' must be supplied). See
#'  'details' for STRUCTURE output.
#' @param file a character giving the name of the file to be written
#'  when 'res.type' is "STRUCTURE"; if NULL, a the created file is of the
#'  form "hybrids\_[the current date].str".
#' @param quiet a logical specifying whether the writing to a file (when
#'    'res.type' is "STRUCTURE") should be announced (FALSE, default) or
#'    not (TRUE).
#' @param sep a character used to separate two alleles
#' @param hyb.label a character string used to construct the hybrids
#'  labels; by default, "h", which gives labels: "h01", "h02", "h03",...
#'
#' @return
#' A \linkS4class{genind} object (by default), or a data.frame of alleles
#' (res.type="df"). No R output if res.type="STRUCTURE" (results written
#' to the specified file).
#'
#' @details
#'   If the output is a STRUCTURE file, this file will have the following
#'  caracteristics:\cr
#'  - file contains the genotypes of the parents, and then the genotypes
#'  of hybrids\cr
#'  - the first column identifies genotypes\cr
#'  - the second column identifies the population (1 and 2 for parents x1 and x2;
#'  3 for hybrids)\cr
#'  - the first line contains the names of the markers\cr
#'  - one row = one genotype (onerowperind will be true)\cr
#'  - missing values coded by "-9" (the software's default)\cr
#'
#' @examples
#' \dontrun{
#' ## Let's make some cattle hybrids
#' data(microbov)
#'
#' ## first, isolate each breed
#' temp <- seppop(microbov)
#' names(temp)
#' salers <- temp$Salers
#' zebu <- temp$Zebu
#'
#' ## let's make some... Zeblers
#' zebler <- hybridize(salers, zebu, n=40,
#'                     pop="Zebler")
#'
#'
#' ## now let's merge all data into a single genind
#' newDat <- repool(microbov, zebler)
#'
#' ## make a correspondance analysis
#' ## and see where hybrids are placed
#' X <- genind2genpop(newDat, quiet=TRUE)
#' coa1 <- dudi.coa(tab(X),scannf=FALSE,nf=3)
#' s.label(coa1$li)
#' add.scatter.eig(coa1$eig,2,1,2)
#'
#' }
#'
#'
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}
#'
#' @seealso \code{\link{seploc}}, \code{\link{seppop}}, \code{\link{repool}}
#'
#'
#'
hybridize <- function(x1, x2, n, pop="hybrid",
                      res.type=c("genind","df","STRUCTURE"),
                      file=NULL, quiet=FALSE, sep="/", hyb.label="h"){
    ## checks
    if(!is.genind(x1)) stop("x1 is not a valid genind object")
    if(!is.genind(x2)) stop("x2 is not a valid genind object")
    if(!all(ploidy(x1)==ploidy(x1)[1])) stop("varying ploidy (in x1) is not supported for this function")
    if(!all(ploidy(x2)==ploidy(x2)[1])) stop("varying ploidy (in x2) is not supported for this function")
    if(ploidy(x1)[1] %% 2 != 0) stop("not implemented for odd levels of ploidy")
    if(ploidy(x1)[1] != ploidy(x2)[1]) stop("x1 and x2 have different ploidy")
    checkType(x1)
    checkType(x2)

    ## store a few variables
    n <- as.integer(n)
    ploidy <- ploidy(x1)[1]
    res.type <- match.arg(res.type)

    ## ensure different names for pop
    popNames(x1) <- "pop1"
    popNames(x2) <- "pop2"

    ## repool data
    x1x2 <- repool(x1, x2)
    x1 <- x1x2[pop=1]
    x2 <- x1x2[pop=2]

    ## used variables
    n1 <- nInd(x1)
    n2 <- nInd(x2)
    k <- nLoc(x1)

    #### get frequencies for each locus
    y1 <- genind2genpop(x1,pop=factor(rep(1,n1)),quiet=TRUE)
    freq1 <- tab(y1, freq=TRUE) # get frequencies
    freq1 <- split(freq1, y1@loc.fac) # split by locus
    freq1 <- freq1[locNames(x1)] # ensure right order

    y2 <- genind2genpop(x2,pop=factor(rep(1,n2)),quiet=TRUE)
    freq2 <- tab(y2, freq=TRUE) # get frequencies
    freq2 <- split(freq2, y2@loc.fac) # split by locus
    freq2 <- freq2[locNames(x2)] # ensure right order
    
    #### sampling of gametes
    ## kX1 / kX2 are lists of tables of sampled gametes
    kX1 <- lapply(freq1, function(v) t(rmultinom(n,ploidy/2,v)))
    names(kX1) <- locNames(x1)
    vec.paste1<-NULL
    Vec.all1<-NULL
    for(i in 1:k) { 
      colnames(kX1[[i]]) <- alleles(x1)[[i]]
      ## Paste the alleles locus after locus
      vec.paste1<-c(vec.paste1, alleles(x1)[[i]])
      ## Paste the number of alleles, locus after locus
      Vec.all1<-c(Vec.all1, length(alleles(x1)[[i]]))
    }
    kX2 <- lapply(freq2, function(v) t(rmultinom(n,ploidy/2,v)))
    names(kX2) <- locNames(x2)
    vec.paste2<-NULL
    Vec.all2<-NULL
    for(i in 1:k) { 
      colnames(kX2[[i]]) <- alleles(x2)[[i]]
      vec.paste2<-c(vec.paste2, alleles(x2)[[i]])
      Vec.all2<-c(Vec.all2, length(alleles(x2)[[i]]))
    }
    
    
    ## construction of zygotes ##
    ## individual gamete tables
    tab1 <- as.matrix(cbind.data.frame(kX1))
    ## Force the names of the columns for tab1 and tab2 with the pattern "locNames.allele"
    colnames(tab1)<-paste(rep(locNames(x1), Vec.all1), ".",vec.paste1, sep = "")
    tab2 <- as.matrix(cbind.data.frame(kX2))
    colnames(tab2)<-paste(rep(locNames(x2), Vec.all2), ".",vec.paste2, sep = "")
    

    ## make empty matrix with all alleles in tab1 and tab2
    zyg.rownames <- .genlab(hyb.label,n)
    zyg.colnames <- sort(unique(c(colnames(tab1),colnames(tab2))))
    zyg <- matrix(0, nrow=n, ncol=length(zyg.colnames),
                  dimnames=list(zyg.rownames, zyg.colnames))

    ## add in the alleles
    zyg[, colnames(tab1)] <- zyg[, colnames(tab1)] + tab1
    zyg[, colnames(tab2)] <- zyg[, colnames(tab2)] + tab2
    zyg <- zyg
    zyg <- genind(zyg, type="codom", ploidy=ploidy)

    ## res.type=="STRUCTURE"
    if(res.type=="STRUCTURE"){
        ## res <- paste(gam1,gam2,sep=" ") # make df for the hybrids
        ## res <- as.data.frame(matrix(res,ncol=k))
        temp <- genind2df(repool(x1,x2,zyg), usepop=FALSE, sep=" ")
        res <- unlist(apply(temp,1,strsplit," "))
        res <- as.data.frame(matrix(res, nrow=nrow(temp), byrow=TRUE))
        colnames(res) <- rep(colnames(temp),each=ploidy)
        ## df1 <- genind2df(x1,sep=" ",usepop=FALSE) # make df with parents and hybrids
        ## df2 <- genind2df(x2,sep=" ",usepop=FALSE)
        ## res <- rbind.data.frame(df1,df2,res) # rbind the three df
        res[is.na(res)] <- "-9" # this is two missing alleles for STRUCTURE
        pop <- rep(1:3,c(nrow(x1@tab), nrow(x2@tab), n)) # make a pop identifier
        res <- cbind.data.frame(pop,res, stringsAsFactors = FALSE)
        names(res)[1] <- ""

        if(is.null(file)) {
            file <- gsub("[[:space:]]|:","-",date())
            file <- paste("hybrid",file,sep="_")
            file <- paste(file,"str",sep=".")
        }
        write.table(res, file=file,row.names = TRUE, col.names = TRUE, quote=FALSE)
        if(!quiet) cat("\nWrote results to file", file, "\n")

        return(invisible())
    }


    ## res.type=="df"
    if(res.type=="df"){
        ## res <- paste(gam1,gam2,sep=sep)
        ## res <- as.data.frame(matrix(res,ncol=k), stringsAsFactors=FALSE)
        ## names(res) <- x1@loc.names
        ## row.names(res) <- .genlab(hyb.label,n)

        res <- genind2df(zyg, sep=sep)
        return(res)
    }


    ## res.type=="genind"
    if(res.type=="genind"){
        ## res <- paste(gam1,gam2,sep="")
        ##         res <- as.data.frame(matrix(res,ncol=k), stringsAsFactors=FALSE)
        ##         names(res) <- x1@loc.names
        ##         row.names(res) <- .genlab(hyb.label,n)
        ## if(is.null(pop)){ # if pop is not provided, merge the two parent populations
        ##     pop <- paste(deparse(substitute(x1)) , deparse(substitute(x2)), sep="-")
        ## }
        pop <- factor(rep(pop,n))

        res <- zyg
        pop(res) <- pop
        res@call <- match.call()

        return(res)
    }

} # end hybridize

#' Choose the number of clusters for snapclust
#'
#' The function \code{snapclust.choose.k} can be used to identify optimal values
#' of 'k' (number of panmictic clusters) using \code{snapclust}. It runs the
#' method for increasing values of 'k' and for each computes goodness-of-fit
#' statistics. These statistics are all different versions of deviance penalised
#' for the number of parameters, so that lower values should correspond to more
#' optimal clustering solutions. Currently available statistics include AIC,
#' AICc, BIC, and KIC.\cr
#'
#' There is no theoretical basis for favouring a given statistic over another
#' in the case of genetic clustering. In practice, one should look for an
#' 'elbow' in the curve of the statistics as a function of 'k'.
#'
#' @author Thibaut Jombart \email{thibautjombart@@gmail.com}
#'
#' @export
#'
#' @seealso
#' \itemize{
#'  \item \code{\link{snapclust}}: to identify clusters
#'
#'  \item \code{\link{AIC.snapclust}}: AIC computation
#'
#'  \item \code{\link{AICc.snapclust}}: AICc computation
#'
#'  \item \code{\link{BIC.snapclust}}: BIC computation
#'
#'  \item \code{\link{KIC.snapclust}}: KIC computation
#' }
#'
#' @param max An integer indicating the maximum number of clusters to seek;
#'     \code{\link{snapclust}} will be run for all k from 2 to max.
#'
#' @param IC A function computing the information criterion for
#'     \code{\link{snapclust}} objects. Available statistics are
#'     \code{AIC} (default), \code{AICc}, and \code{BIC}.
#'
#' @param IC.only A logical (TRUE by default) indicating if IC values only
#'     should be returned; if \code{FALSE}, full \code{snapclust} objects are
#'     returned.
#'
#' @param ... Arguments passed to \code{\link{snapclust}}.
#' @examples
#' \dontrun{
#'  ## 'a' is a simulated dataset with 6 populations, island model
#'  data(dapcIllus)
#'  a <- dapcIllus$a
#'  a
#'
#'  ## try and choose 'k' using AIC (real value = 6)
#'  a.aic <- snapclust.choose.k(max = 10, a)
#'  plot(1:10, a.aic, xlab = "Number of clusters (k)",
#'       ylab = "AIC", type = "b", pch = 20, cex = 3)
#'
#'  ## try and choose 'k' using AIC (real value = 6)
#'  a.kic <- snapclust.choose.k(max = 10, a, IC = KIC)
#'  plot(1:10, a.kic, xlab = "Number of clusters (k)",
#'       ylab = "KIC", type = "b", pch = 20, cex = 3)
#'
#'
#' }
snapclust.choose.k <- function(max, ..., IC = AIC, IC.only = TRUE) {

    ## This function is a glorified for loop which runs snapclust for several
    ## values of 'k', from 2 to 'max'. It returns information criterion (AIC or
    ## BIC), and can also return the full snapclust objects if needed. For
    ## k=1, AIC and BIC are computed via an internal (i.e. non-exported)
    ## procedure.

    max <- as.integer(max)
    if (any(!is.finite(max))) {
        stop("Values of k need to be finite.")
    }
    if (max < 2) {
        stop("maximum number of clusters should be at least 2")
    }
    k.values <- 2:max

    call.args <- list(...)
    genind.posi <- which(vapply(call.args, inherits, logical(1), "genind"))
    if (length(genind.posi) == 0) {
        stop("No genind provided in '...'.")
    } else if (length(genind.posi) > 1){
        warning("Too many genind objects provided in '...'. I am taking the first one.")
        genind.posi <- genind.posi[1]
    }
    names(call.args)[genind.posi] <- "x"


    out.IC <- double(length(k.values))
    out.objects <- list(length(k.values))

    for (i in seq_along(k.values)) {
        ## get clustering solution for 'k'
        call.args$k <- k.values[i]
        out.objects[[i]] <- do.call(snapclust, call.args)
    }

    names(out.objects) <- k.values
    out.IC <- .compute.null.IC(call.args$x)
    out.IC <- c(out.IC, vapply(out.objects, IC, double(1)))
    names(out.IC) <- 1:max

    if (IC.only) {
        out <- out.IC
    } else {
        out <- list(AIC = out.IC, objects = out.objects)

        ## names stat as appropriate
        names(out)[1] <- deparse(substitute(IC))
    }

    return(out)
}





## Non-exported procedure to compute the BIC for k = 1

## - 'x' is a genind object.
## - 'IC' is either AIC or BIC

.compute.null.IC <- function(x, IC = AIC) {
    group <- rep(1L, nInd(x))
    n.loc <- nLoc(x)
    genotypes <- tab(x)
    pop.freq <- tab(genind2genpop(x, pop = group, quiet = TRUE),
                    freq = TRUE)

    if (!is.genind(x)) {
      stop("x is not a valid genind object")
    }

    if (any(ploidy(x) > 2)) {
      stop("snapclust not currently implemented for ploidy > 2")
    }

    if (all(ploidy(x) == 1)) {
      .ll.genotype <- .ll.genotype.haploid
    } else if (all(ploidy(x) == 2)) {
      .ll.genotype <- .ll.genotype.diploid
    } else {
      stop("snapclust not currently implemented for varying ploidy")
    }

    ## get likelihoods of genotypes
    ll.mat <- apply(genotypes, 1, .ll.genotype, pop.freq, n.loc)
    ll.mat <- matrix(ll.mat, nrow = 1)

    ll <- .global.ll(group, ll.mat)

    ## make a fake snapclust object to get IC
    fake <- list(ll = ll,
                 group = group,
                 n.param = ncol(tab(x)) - nLoc(x))
    class(fake) <- "snapclust"
    IC(fake)
}

##########
## glSum
##########
## compute col sums
## removing NAs
##
glSum <- function(x, alleleAsUnit=TRUE, useC=FALSE){
    if(!inherits(x, "genlight")) stop("x is not a genlight object")

    if(useC){
        ## use ploidy (sum absolute frequencies)
        if(alleleAsUnit){
            vecbyte <- unlist(lapply(x@gen, function(e) e$snp))
            nbVec <- sapply(x@gen, function(e) length(e$snp))
            nbNa <- sapply(NA.posi(x), length)
            naPosi <- unlist(NA.posi(x))
            res <- .C("GLsumInt", vecbyte, nbVec, length(x@gen[[1]]@snp[[1]]), nbNa, naPosi, nInd(x), nLoc(x), ploidy(x),
                      integer(nLoc(x)), PACKAGE="adegenet")[[9]]
        } else {
            ## sum relative frequencies
            vecbyte <- unlist(lapply(x@gen, function(e) e$snp))
            nbVec <- sapply(x@gen, function(e) length(e$snp))
            nbNa <- sapply(NA.posi(x), length)
            naPosi <- unlist(NA.posi(x))
            res <- .C("GLsumFreq", vecbyte, nbVec, length(x@gen[[1]]@snp[[1]]), nbNa, naPosi, nInd(x), nLoc(x), ploidy(x),
                      double(nLoc(x)), PACKAGE="adegenet")[[9]]
        }
    } else {
        ## use ploidy (sum absolute frequencies)
        if(alleleAsUnit){
            res <- integer(nLoc(x))
            for(e in x@gen){
                temp <- as.integer(e)
                temp[is.na(temp)] <- 0L
                res <- res + temp
            }
        } else {
            ## sum relative frequencies
            res <- numeric(nLoc(x))
            myPloidy <- ploidy(x)
            for(i in 1:nInd(x)){
                temp <- as.integer(x@gen[[i]]) / myPloidy[i]
                temp[is.na(temp)] <- 0
                res <- res + temp
            }
        }

    }
    names(res) <- locNames(x)
    return(res)

} # glSum





##########
## glNA
##########
## counts NB of NAs per column
##
## if alleleAsUnit, then effective is the number of alleles sampled (sum(ploidy(x)))
## otherwise, effective is simply the number of individuals (
glNA <- function(x, alleleAsUnit=TRUE){
    if(!inherits(x, "genlight")) stop("x is not a genlight object")

    ## DEFAULT, VECTOR-WISE PROCEDURE ##
    res <- integer(nLoc(x))
    temp <- NA.posi(x)

    ## NAs in allele sampling
    if(alleleAsUnit){
        for(i in 1:length(temp)){
            if(length(temp[[i]])>0){
                res[temp[[i]]] <- res[temp[[i]]] + ploidy(x)[i]
            }
        }
    } else { ## NAs amongst individuals
        for(e in temp){
            if(length(e)>0){
                res[e] <- res[e] + 1
            }
        }
    }

    names(res) <- locNames(x)
    return(res)

} # glNA





##########
## glMean
##########
## computes SNPs means
## takes NAs into account
##
glMean <- function(x, alleleAsUnit=TRUE){
    if(!inherits(x, "genlight")) stop("x is not a genlight object")

    ## DEFAULT, VECTOR-WISE PROCEDURE ##
    if(alleleAsUnit){ # use alleles
        N <- sum(ploidy(x)) - glNA(x, alleleAsUnit=TRUE)
        res <- glSum(x, alleleAsUnit=TRUE)/N
    } else { # use relative frequencies of individuals
        N <- nInd(x) - glNA(x, alleleAsUnit=FALSE)
        res <- glSum(x, alleleAsUnit=FALSE)/N
    }

    names(res) <- locNames(x)
    return(res)

} # glMean





########
## glVar
########
## computes SNPs variances
## takes NAs into account
##
glVar <- function(x, alleleAsUnit=TRUE){
    if(!inherits(x, "genlight")) stop("x is not a genlight object")

    ## DEFAULT, VECTOR-WISE PROCEDURE ##
    res <- numeric(nLoc(x))
    myPloidy <- ploidy(x)

    if(alleleAsUnit){ # use alleles
        N <- sum(ploidy(x)) - glNA(x, alleleAsUnit=TRUE)
        xbar <- glMean(x, alleleAsUnit=TRUE)
        for(i in 1:nInd(x)){
            temp <- (as.integer(x@gen[[i]])/myPloidy[i] - xbar)^2
            temp[is.na(temp)] <- 0
            res <- res + temp*myPloidy[i]
        }
        res <- res/N
    } else { # use relative frequencies of individuals
        N <- nInd(x) - glNA(x, alleleAsUnit=FALSE)
        xbar <- glMean(x, alleleAsUnit=FALSE)

        for(i in 1:nInd(x)){
            temp <- (as.integer(x@gen[[i]])/myPloidy[i] - xbar)^2
            temp[is.na(temp)] <- 0L
            res <- res + temp
        }
        res <- res/N
    }

    names(res) <- locNames(x)
    return(res)

} # glVar






#############
## glDotProd
############
## computes all pairs of dot products
## between centred/scaled vectors
## of SNPs
glDotProd <- function(x, center=FALSE, scale=FALSE, alleleAsUnit=FALSE,
                      parallel=FALSE, n.cores=NULL){
    if(!inherits(x, "genlight")) stop("x is not a genlight object")

    ## SOME CHECKS ##
    ## if(parallel && !require(parallel)) stop("parallel package requested but not installed")
    if(parallel && is.null(n.cores)){
        n.cores <- parallel::detectCores()
    }


    ## STORE USEFUL INFO ##
    N <- nInd(x)
    ind.names <- indNames(x)


    if(!parallel){ # DO NOT USE MULTIPLE CORES
        ## GET INPUTS TO C PROCEDURE ##
        if(center){
            mu <- glMean(x,alleleAsUnit=alleleAsUnit)
        } else {
            mu <- rep(0, nLoc(x))
        }

        if(scale){
            s <- sqrt(glVar(x,alleleAsUnit=alleleAsUnit))
            if(any(s<1e-10)) {
                warning("Null variances have been detected; corresponding alleles won't be standardized.")
            }
        } else {
            s <- rep(1, nLoc(x))
        }

        vecbyte <- unlist(lapply(x@gen, function(e) e$snp))
        nbVec <- sapply(x@gen, function(e) length(e$snp))
        nbNa <- sapply(NA.posi(x), length)
        naPosi <- unlist(NA.posi(x))
        lowerTriSize <- (nInd(x)*(nInd(x)-1))/2
        resSize <- lowerTriSize + nInd(x)

        ## CALL C FUNCTION ##
        temp <- .C("GLdotProd", vecbyte, nbVec, length(x@gen[[1]]@snp[[1]]), nbNa, naPosi, nInd(x), nLoc(x), ploidy(x),
                   as.double(mu), as.double(s), as.integer(!alleleAsUnit), double(resSize), PACKAGE="adegenet")[[12]]
    } else { # USE MULTIPLE CORES
        x <- seploc(x, n.block = n.cores) # one block per core (x is now a list of genlight)
        temp <- list()
        i <- 0
        for(block in x){
            i <- i+1
            ## GET INPUTS TO C PROCEDURE ##
            if(center){
                mu <- glMean(block,alleleAsUnit=alleleAsUnit)
            } else {
                mu <- rep(0, nLoc(block))
            }

            if(scale){
                s <- sqrt(glVar(block,alleleAsUnit=alleleAsUnit))
                if(any(s<1e-10)) {
                    warning("Null variances have been detected; corresponding alleles won't be standardized.")
                }
            } else {
                s <- rep(1, nLoc(block))
            }

            vecbyte <- unlist(lapply(block@gen, function(e) e$snp))
            nbVec <- sapply(block@gen, function(e) length(e$snp))
            nbNa <- sapply(NA.posi(block), length)
            naPosi <- unlist(NA.posi(block))
            lowerTriSize <- (nInd(block)*(nInd(block)-1))/2
            resSize <- lowerTriSize + nInd(block)

            ## CALL C FUNCTION ##
            temp[[i]] <- .C("GLdotProd", vecbyte, nbVec, length(block@gen[[1]]@snp[[1]]), nbNa, naPosi, nInd(block), nLoc(block), ploidy(block),
                       as.double(mu), as.double(s), as.integer(!alleleAsUnit), double(resSize), PACKAGE="adegenet")[[12]]
        }


        ## POOL BLOCK RESULTS TOGETHER ##
        temp <- Reduce("+", temp)
    }

    res <- temp[1:lowerTriSize]
    attr(res,"Size") <- N
    attr(res,"Diag") <- FALSE
    attr(res,"Upper") <- FALSE
    class(res) <- "dist"
    res <- as.matrix(res)
    diag(res) <- temp[(lowerTriSize+1):length(temp)]

    colnames(res) <- rownames(res) <- ind.names
    return(res)
} # end glDotProd






########
## glPca
########
##
## PCA for genlight objects
##
glPca <- function(x, center=TRUE, scale=FALSE, nf=NULL, loadings=TRUE, alleleAsUnit=FALSE,
                  useC=TRUE, parallel=FALSE, n.cores=NULL,
                  returnDotProd=FALSE, matDotProd=NULL){
    if(!inherits(x, "genlight")) stop("x is not a genlight object")

    ## COMPUTE MEANS AND VARIANCES ##
    if(center) {
        vecMeans <- glMean(x, alleleAsUnit=alleleAsUnit)
        if(any(is.na(vecMeans))) stop("NAs detected in the vector of means")
    }

    if(scale){
        vecVar <- glVar(x, alleleAsUnit=alleleAsUnit)
        if(any(is.na(vecVar))) stop("NAs detected in the vector of variances")
    }


    myPloidy <- ploidy(x)

    ## NEED TO COMPUTE DOT PRODUCTS ##
    if(is.null(matDotProd)){

        ## == if non-C code is used ==
        if(!useC){
            ## if(parallel && !require(parallel)) stop("parallel package requested but not installed")
            if(parallel && is.null(n.cores)){
                n.cores <- parallel::detectCores()
            }


            ## COMPUTE DOT PRODUCTS BETWEEN GENOTYPES ##
            ## to be fast, a particular function is defined for each case of centring/scaling

            ## NO CENTRING / NO SCALING
            if(!center & !scale){
                dotProd <- function(a,b, ploid.a, ploid.b){ # a and b are two SNPbin objects
                    a <- as.integer(a) / ploid.a
                    a[is.na(a)] <- 0
                    b <- as.integer(b) / ploid.b
                    b[is.na(b)] <- 0
                    return(sum( a*b, na.rm=TRUE))
                }
            }

            ## CENTRING / NO SCALING
            if(center & !scale){
                dotProd <- function(a,b, ploid.a, ploid.b){ # a and b are two SNPbin objects
                    a <- as.integer(a) / ploid.a
                    a[is.na(a)] <- vecMeans[is.na(a)]
                    b <- as.integer(b) / ploid.b
                    b[is.na(b)] <- vecMeans[is.na(b)]
                    return(sum( (a-vecMeans) * (b-vecMeans), na.rm=TRUE) )
                }
            }


            ## NO CENTRING / SCALING (odd option...)
            if(!center & scale){
                dotProd <- function(a,b, ploid.a, ploid.b){ # a and b are two SNPbin objects
                    a <- as.integer(a) / ploid.a
                    a[is.na(a)] <- 0
                    b <- as.integer(b) / ploid.b
                    b[is.na(b)] <- 0
                    return(sum( (a*b)/vecVar, na.rm=TRUE))
                }
            }


            ## CENTRING / SCALING
            if(center & scale){
                dotProd <- function(a,b, ploid.a, ploid.b){ # a and b are two SNPbin objects
                    a <- as.integer(a) / ploid.a
                    a[is.na(a)] <- vecMeans[is.na(a)]
                    b <- as.integer(b) / ploid.b
                    b[is.na(b)] <- vecMeans[is.na(b)]
                    return( sum( ((a-vecMeans)*(b-vecMeans))/vecVar, na.rm=TRUE ) )
                }
            }


            ## COMPUTE ALL POSSIBLE DOT PRODUCTS (XX^T / n) ##
            allComb <- combn(1:nInd(x), 2)
            if(parallel){
                allProd <- unlist(parallel::mclapply(1:ncol(allComb), function(i) dotProd(x@gen[[allComb[1,i]]], x@gen[[allComb[2,i]]], myPloidy[allComb[1,i]], myPloidy[allComb[2,i]]),
                                           mc.cores=n.cores, mc.silent=TRUE, mc.cleanup=TRUE, mc.preschedule=FALSE))
            } else {
                allProd <- unlist(lapply(1:ncol(allComb), function(i) dotProd(x@gen[[allComb[1,i]]], x@gen[[allComb[2,i]]], myPloidy[allComb[1,i]], myPloidy[allComb[2,i]]) ))
            }
            allProd <- allProd / nInd(x) # assume uniform weights

            ## shape result as a matrix
            attr(allProd,"Size") <- nInd(x)
            attr(allProd,"Diag") <- FALSE
            attr(allProd,"Upper") <- FALSE
            class(allProd) <- "dist"
            allProd <- as.matrix(allProd)

            ## compute the diagonal
            if(parallel){
                temp <- unlist(parallel::mclapply(1:nInd(x), function(i) dotProd(x@gen[[i]], x@gen[[i]], myPloidy[i], myPloidy[i]),
                                        mc.cores=n.cores, mc.silent=TRUE, mc.cleanup=TRUE, mc.preschedule=FALSE))/nInd(x)
            } else {
                temp <- unlist(lapply(1:nInd(x), function(i) dotProd(x@gen[[i]], x@gen[[i]], myPloidy[i], myPloidy[i]) ))/nInd(x)
            }
            diag(allProd) <- temp
        } else { # === use C computations ====
            allProd <- glDotProd(x, center=center, scale=scale, alleleAsUnit=alleleAsUnit, parallel=parallel, n.cores=n.cores)/nInd(x)
        }
    } else { # END NEED TO COMPUTE DOTPROD
        if(!all(dim(matDotProd)==nInd(x))) stop("matDotProd has wrong dimensions.")
        allProd <- matDotProd
    }

    ## PERFORM THE ANALYSIS ##
    ## eigenanalysis
    eigRes <- eigen(allProd, symmetric=TRUE, only.values=FALSE)
    rank <- sum(eigRes$values > 1e-12)
    eigRes$values <- eigRes$values[1:rank]
    eigRes$vectors <- eigRes$vectors[, 1:rank, drop=FALSE]

    ## scan nb of axes retained
    if(is.null(nf)){
        barplot(eigRes$values, main="Eigenvalues", col=heat.colors(rank))
        cat("Select the number of axes: ")
        nf <- as.integer(readLines(con = getOption('adegenet.testcon'), n = 1))
    }

    ## rescale PCs
    res <- list()
    res$eig <- eigRes$values
    nf <- min(nf, sum(res$eig>1e-10))
    ##res$matprod <- allProd # for debugging

    ## use: li = XQU = V\Lambda^(1/2)
    eigRes$vectors <- eigRes$vectors * sqrt(nInd(x)) # D-normalize vectors
    res$scores <- sweep(eigRes$vectors[, 1:nf, drop=FALSE],2, sqrt(eigRes$values[1:nf]), FUN="*")


    ## GET LOADINGS ##
    ## need to decompose X^TDV into a sum of n matrices of dim p*r
    ## but only two such matrices are represented at a time
    if(loadings){
        if(scale) {
            vecSd <- sqrt(vecVar)
        }
        res$loadings <- matrix(0, nrow=nLoc(x), ncol=nf) # create empty matrix
        ## use: c1 = X^TDV
        ## and X^TV = A_1 + ... + A_n
        ## with A_k = X_[k-]^T v[k-]
        for(k in 1:nInd(x)){
            temp <- as.integer(x@gen[[k]]) / myPloidy[k]
            if(center) {
                temp[is.na(temp)] <- vecMeans[is.na(temp)]
                temp <- temp - vecMeans
            } else {
                temp[is.na(temp)] <- 0
            }
            if(scale){
                temp <- temp/vecSd
            }

            res$loadings <- res$loadings + matrix(temp) %*% eigRes$vectors[k, 1:nf, drop=FALSE]
        }

        res$loadings <- res$loadings / nInd(x) # don't forget the /n of X_tDV
        res$loadings <- sweep(res$loadings, 2, sqrt(eigRes$values[1:nf]), FUN="/")
    }


    ## FORMAT OUTPUT ##
    colnames(res$scores) <- paste("PC", 1:nf, sep="")
    if(!is.null(indNames(x))){
        rownames(res$scores) <- indNames(x)
    } else {
         rownames(res$scores) <- 1:nInd(x)
    }

    if(!is.null(res$loadings)){
        colnames(res$loadings) <- paste("Axis", 1:nf, sep="")
        if(!is.null(locNames(x)) & !is.null(alleles(x))){
            rownames(res$loadings) <- paste(locNames(x),alleles(x), sep=".")
        } else {
            rownames(res$loadings) <- 1:nLoc(x)
        }
    }

    if(returnDotProd){
        res$dotProd <- allProd
        rownames(res$dotProd) <- colnames(res$dotProd) <- indNames(x)
    }

    res$call <- match.call()

    class(res) <- "glPca"

    return(res)

} # glPca






###############
## print.glPca
###############
print.glPca <- function(x, ...){
    cat(" === PCA of genlight object ===")
    cat("\nClass: list of type glPca")
    cat("\nCall ($call):")
    print(x$call)
    cat("\nEigenvalues ($eig):\n", round(head(x$eig,6),3), ifelse(length(x$eig)>6, "...\n", "\n") )
    cat("\nPrincipal components ($scores):\n matrix with", nrow(x$scores), "rows (individuals) and", ncol(x$scores), "columns (axes)", "\n")
    if(!is.null(x$loadings)){
        cat("\nPrincipal axes ($loadings):\n matrix with", nrow(x$loadings), "rows (SNPs) and", ncol(x$loadings), "columns (axes)", "\n")
    }
    if(!is.null(x$dotProd)){
        cat("\nDot products between individuals ($dotProd):\n matrix with", nrow(x$dotProd), "rows and", ncol(x$dotProd), "columns", "\n")
    }
  cat("\n")
}





#################
## scatter.glPca
#################
scatter.glPca <- function(x, xax=1, yax=2, posi="bottomleft", bg="white", ratio=0.3,
                          label = rownames(x$scores), clabel = 1, xlim = NULL, ylim = NULL,
                          grid = TRUE, addaxes = TRUE, origin = c(0,0), include.origin = TRUE,
                          sub = "", csub = 1, possub = "bottomleft", cgrid = 1,
                          pixmap = NULL, contour = NULL, area = NULL, ...){

    ## set par
    opar <- par(mar = par("mar"))
    par(mar = c(0.1, 0.1, 0.1, 0.1), bg=bg)
    on.exit(par(opar))
    axes <- c(xax,yax)
    ## basic empty plot
    ## s.label(x$ind.coord[,axes], clab=0, cpoint=0, grid=FALSE, addaxes = FALSE, cgrid = 1, include.origin = FALSE, ...)
    s.label(x$scores[,axes], label = label, clabel = clabel, xlim = xlim, ylim = ylim, grid = grid, addaxes = addaxes,
            origin = origin, include.origin = include.origin, sub = sub, csub = csub, possub = possub, cgrid = cgrid,
            pixmap = pixmap, contour = contour, area = area)


    if(ratio>0.001) {
        add.scatter.eig(x$eig, ncol(x$scores), axes[1], axes[2], posi=posi, ratio=ratio, csub=csub)
    }

    return(invisible(match.call()))
} # end scatter.glPca






#####################
## loadingplot.glPca
#####################
loadingplot.glPca <- function(x, at=NULL, threshold=NULL, axis=1, fac=NULL, byfac=FALSE,
                        lab=rownames(x$loadings), cex.lab=0.7, cex.fac=1, lab.jitter=0,
                        main="Loading plot", xlab="SNP positions", ylab="Contributions", srt=90, adj=c(0,0.5), ... ){

    if(is.null(x$loadings)){
        warning("This object does not contain loadings. Re-run the analysis, specifying 'loadings=TRUE'.")
        return(invisible())
    }

    if(is.null(at)){
        at <- as.integer(gsub("[.]+.+$", "", rownames(x$loadings)))
    }
    if(is.null(threshold)){
        threshold <- quantile(x$loadings[,axis]^2,0.75)
    }

    res <- loadingplot.default(x$loadings^2, at=at, threshold=threshold, axis=axis, fac=fac, byfac=byfac,
                               lab=lab, cex.lab=cex.lab, cex.fac=cex.fac, lab.jitter=lab.jitter,
                               main=main, xlab=xlab, ylab=ylab, srt=srt, adj=adj, ...)

    axis(1)

    return(invisible(res))
} # end loadingplot.glPca





###############
## .glPca2dudi
###############
.glPca2dudi <- function(x){
    if(!inherits(x,"glPca")) stop("x is not a glPca object")
    old.names <- names(x)
    new.names <- sub("scores","li", old.names)
    new.names <- sub("loadings","c1", new.names)
    names(x) <- new.names
    class(x) <- c("pca","dudi")
    return(x)
} # end glPca2dudi




## TESTING ##
## x <- new("genlight", list(c(0,0,1,1,0), c(1,1,1,0,0,1), c(2,1,1,1,1,NA)))
## as.matrix(x)
## glNA(x)
## glSum(x)
## glMean(x)

## same ploidy everywhere
## x <- new("genlight", list(c(0,0,1,1,0), c(1,1,1,0,0,1), c(0,0,0,1,1,1)))
## f1 <- function(e) {return(mean((e-mean(e, na.rm=TRUE))^2, na.rm=TRUE))}
## all.equal(glVar(x), apply(as.matrix(x), 2, f1 )) # MUST BE TRUE
## all.equal(glVar(x,FALSE), apply(as.matrix(x), 2, f1 )) # MUST BE TRUE

## ## differences in ploidy
## x <- new("genlight", list(c(0,0,1,1,0), c(1,1,1,0,0,1), c(2,1,1,1,1,NA)))
## temp <- sweep(as.matrix(x), 1, c(1,1,2), "/")
## f2 <- function(e,w) {
##     mu <- weighted.mean(e, w, na.rm=TRUE)
##     res <- weighted.mean((e-mu)^2, w, na.rm=TRUE)
##     return(res)
## }

## all.equal(glMean(x), apply(temp,2,weighted.mean, w=c(1,1,2), na.rm=TRUE)) # MUST BE TRUE
## all.equal(glVar(x), apply(temp, 2, f2,w=c(1,1,2) )) # MUST BE TRUE

## all.equal(glMean(x,FALSE), apply(temp,2,mean,na.rm=TRUE)) # MUST BE TRUE
## all.equal(glVar(x,FALSE), apply(temp,2,f1)) # MUST BE TRUE



#### TESTING DOT PRODUCTS ####
## M <- matrix(sample(c(0,1), 100*1e3, replace=TRUE), nrow=100)
## rownames(M) <- paste("ind", 1:100)
## x <- new("genlight",M)

## ## not centred, not scaled
## res1 <- glDotProd(x,alleleAsUnit=FALSE, center=FALSE, scale=FALSE)
## res2 <- M %*% t(M)
## all.equal(res1,res2) # must be TRUE

## ##  centred, not scaled
## res1 <- glDotProd(x,alleleAsUnit=FALSE, center=TRUE, scale=FALSE)
## M <- scalewt(M,center=TRUE,scale=FALSE)
## res2 <- M %*% t(M)
## all.equal(res1,res2) # must be TRUE

## ##  centred, scaled
## res1 <- glDotProd(x,alleleAsUnit=FALSE, center=TRUE, scale=TRUE)
## M <- scalewt(M,center=TRUE,scale=TRUE)
## res2 <- M %*% t(M)
## all.equal(res1,res2) # must be TRUE


#### TESTING PCA ####
## M <- matrix(sample(c(0,1), 20*1000, replace=TRUE), nrow=20)
## rownames(M) <- paste("ind", 1:20)

## x <- new("genlight",M)
## res1 <- glPca(x, nf=4)
## res2 <- glPca(x, useC=FALSE, nf=4)
## res3 <- dudi.pca(M, center=TRUE,scale=FALSE, scannf=FALSE,nf=4)

## ## all must be TRUE
## all.equal(res1$eig,res3$eig)
## all.equal(res2$eig,res3$eig)
## all.equal(res1$eig,res2$eig)

## all(abs(res1$scores)-abs(res3$li)<1e-8)
## all(abs(res2$scores)-abs(res3$li)<1e-8)
## all(abs(res1$scores)-abs(res2$scores)<1e-8)

## all(abs(res1$loadings)-abs(res3$c1)<1e-8)
## all(abs(res2$loadings)-abs(res3$c1)<1e-8)
## all(abs(res1$loadings)-abs(res2$loadings)<1e-8)


## ## perform ordinary PCA
## titi <- dudi.pca(M, center=TRUE, scale=FALSE, scannf=FALSE, nf=4)


## ## check results
## all(round(abs(toto$scores), 10) == round(abs(titi$li), 10)) # MUST BE TRUE
## all.equal(toto$eig, titi$eig) # MUST BE TRUE
## all(round(abs(toto$loadings), 10)==round(abs(titi$c1), 10)) # MUST BE TRUE


## TEST WITH NAS ##
## M <- matrix(sample(c(0,1, NA), 1e5, replace=TRUE, prob=c(.495,.495,.01)), nrow=100)
## rownames(M) <- paste("ind", 1:100)

## ## perform glPca
## x <- new("genlight",M)
## toto <- glPca(x, nf=4)

## round(cor(toto$scores),10) # must be diag(1,4)
## round(t(toto$loadings) %*% toto$loadings,10) # must be diag(1,4)





## ## SPEED TESTS ##
## ## perform glPca
## M <- matrix(sample(c(0,1), 100*1e5, replace=TRUE), nrow=100)
## x <- new("genlight",M)
## system.time(titi <- dudi.pca(M,center=TRUE,scale=FALSE, scannf=FALSE, nf=4)) # 92 sec
## system.time(toto <- glPca(x, ,center=TRUE,scale=FALSE, useC=TRUE, nf=4)) # 102 sec

## M <- matrix(sample(c(0,1), 200*1e5, replace=TRUE), nrow=200)
## x <- new("genlight",M)
## system.time(titi <- dudi.pca(M,center=TRUE,scale=FALSE, scannf=FALSE, nf=4)) #  109 sec
## system.time(toto <- glPca(x, ,center=TRUE,scale=FALSE, useC=TRUE, nf=4)) #  360 sec

## M <- matrix(sample(c(0,1), 100*5e5, replace=TRUE), nrow=500)
## x <- new("genlight",M)
## system.time(titi <- dudi.pca(M,center=TRUE,scale=FALSE, scannf=FALSE, nf=4)) #  MEM LIMIT ISSUE
## system.time(toto <- glPca(x, ,center=TRUE,scale=FALSE, useC=TRUE, nf=4)) #  sec

## USE R PROFILING ##
## for glPca
## M <- matrix(sample(c(0,1), 100*1e5, replace=TRUE), nrow=100)
## x <- new("genlight",M)
## Rprof("glPca-prof.log")
## toto <- glPca(x, ,center=TRUE,scale=FALSE, useC=TRUE, nf=4) # 102 sec
## Rprof(NULL)
## res <- summaryRprof("glPca-prof.log")
## t <- res$by.total$total.time
## names(t) <- rownames(res$by.total)
## par(mar=c(7,4,4,2))
## barplot(t,las=3, cex.names=.7)


## ## for dudi.pca
## M <- matrix(sample(c(0,1), 100*1e5, replace=TRUE), nrow=100)
## Rprof("dudipca-prof.log")
## toto <- dudi.pca(M ,center=TRUE,scale=FALSE, scannf=FALSE, nf=4) # 102 sec
## Rprof(NULL)
## res <- summaryRprof("dudipca-prof.log")
## t <- res$by.total$total.time
## names(t) <- rownames(res$by.total)
## par(mar=c(7,4,4,2))
## barplot(t,las=3, cex.names=.7)


## test GLsum:
## library(adegenet)
## x <- new("genlight", lapply(1:50, function(i) sample(c(0,1,NA), 1e5, prob=c(.5, .49, .01), replace=TRUE)))
## res1 <- glSum(x, useC=FALSE)
## res2 <- glSum(x, useC=TRUE)
## res3 <- apply(as.matrix(x),2,sum,na.rm=TRUE)
## all(res1==res3) # must be TRUE
## all(res2==res3) # must be TRUE

## library(adegenet)
## x <- new("genlight", lapply(1:50, function(i) sample(c(0,1,2,NA), 1e5, prob=c(.5, .40, .09, .01), replace=TRUE)))
## res1 <- glSum(x, alleleAsUnit=FALSE, useC=FALSE)
## res2 <- glSum(x, alleleAsUnit=FALSE, useC=TRUE)
## res3 <- apply(as.matrix(x)/ploidy(x),2,sum,na.rm=TRUE)
## all.equal(res1,res3)
## all.equal(res2,res3)


## TEST PARALLELE C COMPUTATIONS IN GLDOTPROD
## system.time(toto <- glDotProd(x,multi=TRUE)) # 58 sec: cool!
## system.time(titi <- glDotProd(x,multi=FALSE)) # 245 sec
## all.equal(toto,titi)


## TEST PARALLELE C COMPUTATIONS IN GLPCA ##
## first dataset
## x <- new("genlight", lapply(1:50, function(i) sample(c(0,1,2,NA), 1e5, prob=c(.5, .40, .09, .01), replace=TRUE)))
## system.time(pca1 <- glPca(x, multi=FALSE, useC=FALSE, nf=1)) # no C, no parallel: 43 sec
## system.time(pca2 <- glPca(x, multi=FALSE, useC=TRUE, nf=1)) # just C: 248 sec
## system.time(pca3 <- glPca(x, multi=TRUE, useC=FALSE, nf=1, n.core=7)) # just parallel: 16 sec
## system.time(pca4 <- glPca(x, multi=TRUE, useC=TRUE, nf=1, n.core=7)) # C and parallel: 65 sec
## all.equal(pca1$scores^2, pca2$scores^2) # must be TRUE
## all.equal(pca1$scores^2, pca3$scores^2) # must be TRUE
## all.equal(pca1$scores^2, pca4$scores^2) # must be TRUE

## second dataset
## x <- new("genlight", lapply(1:500, function(i) sample(c(0,1,2,NA), 1e4, prob=c(.5, .40, .09, .01), replace=TRUE)))
## system.time(pca1 <- glPca(x, multi=FALSE, useC=FALSE, nf=1)) # no C, no parallel: 418 sec
## system.time(pca2 <- glPca(x, multi=FALSE, useC=TRUE, nf=1)) # just C:  496 sec
## system.time(pca3 <- glPca(x, multi=TRUE, useC=FALSE, nf=1, n.core=7)) # just parallel: 589 sec
## system.time(pca4 <- glPca(x, multi=TRUE, useC=TRUE, nf=1, n.core=7)) # C and parallel: 315 sec
## all.equal(pca1$scores^2, pca2$scores^2) # must be TRUE
## all.equal(pca1$scores^2, pca3$scores^2) # must be TRUE
## all.equal(pca1$scores^2, pca4$scores^2) # must be TRUE

########################################
#
# Tests for global and local structures
#
# Thibaut Jombart 2007
# t.jombart@imperial.ac.uk
#
########################################



###############
# global.rtest
###############
global.rtest <- function(X, listw, k=1, nperm=499){
  if (!inherits(listw, "listw")) stop("object of class 'listw' expected")
  if (listw$style != "W") stop("object of class 'listw' with style 'W' expected")
  if(any(is.na(X))) stop("NA entries in X")

  n <- nrow(X)
  X <- scalewt(X)

  # computation of U+
  temp <- .orthobasis.listw(listw)
  val <- attr(temp,"values")
  U <- as.matrix(temp)
  Upos <-  U[,val > -1/(n-1)]

  # test statistic
  calcstat <- function(X,k){
    R <- ( t(X) %*% Upos ) / n
    R2 <- R*R
    temp <- sort(apply(R2,2,mean),decreasing=TRUE)
    stat <- sum(temp[1:k])
    return(stat)
  }

  ini <- calcstat(X,k)

  sim <- sapply(1:nperm, function(i) calcstat( X[sample(1:n),], k ) )

  res <- as.randtest(sim=sim, obs=ini, alter="greater")
  res$call <- match.call()

  return(res)
} #end global.rtest





###############
# local.rtest
###############
local.rtest <- function(X, listw, k=1, nperm=499){
  if (!inherits(listw, "listw")) stop("object of class 'listw' expected")
  if (listw$style != "W") stop("object of class 'listw' with style 'W' expected")
  if(any(is.na(X))) stop("NA entries in X")

  n <- nrow(X)
  X <- scalewt(X)

  # computation of U-
  temp <- .orthobasis.listw(listw)
  val <- attr(temp,"values")
  U <- as.matrix(temp)
  Uneg <-  U[,val < -1/(n-1)]

  X <- scalewt(X)

  # test statistic
  calcstat <- function(X,k){
    R <- ( t(X) %*% Uneg ) / n
    R2 <- R*R
    temp <- sort(apply(R2,2,mean),decreasing=TRUE)
    stat <- sum(temp[1:k])
    return(stat)
  }

  ini <- calcstat(X,k)

  sim <- sapply(1:nperm, function(i) calcstat( X[sample(1:n),], k ) )

  res <- as.randtest(sim=sim, obs=ini, alter="greater")
  res$call <- match.call()

  return(res)
} #end local.rtest

##################
# HWE.test.genind
##################

HWE.test.genind <- function(x,pop=NULL,permut=FALSE,nsim=1999,hide.NA=TRUE,res.type=c("full","matrix")){

    message("As of adegenet_1.5-0, this function has been removed and is replaced by 'hw.test' in the package 'pegas'")

  ## if(!is.genind(x)) stop("x is not a valid genind object")
  ## if(x@ploidy != as.integer(2)) stop("not implemented for non-diploid genotypes")
  ## checkType(x)

  ## if(is.null(pop)) pop <- x@pop
  ## if(is.null(pop)) pop <- as.factor(rep("P1",nrow(x@tab)))
  ## res.type <- tolower(res.type[1])
  ## if(res.type != "full" && res.type != "matrix") stop("unknown res.type specified.")

  ## kGen <- genind2genotype(x,pop=pop,res.type="list")

  ## # ftest tests HWE for a locus and a population
  ## ftest <- function(vec,permut=permut,nperm=nsim){
  ##   temp <- unique(vec)
  ##   temp <- temp[!is.na(temp)]
  ##   if(length(temp) < 2) return(NA)
  ##   if(res.type=="full") {
  ##     res <- HWE.chisq(vec, simulate.p.value=permut, B=nperm)
  ##   } else {
  ##     res <- HWE.chisq(genotype(vec), simulate.p.value=permut, B=nperm)$p.value
  ##   }
  ##   return(res)
  ## }

  ## res <- lapply(kGen,function(e) lapply(e,ftest,permut,nsim))

  ## # clean non-tested elements in the results list
  ## if(hide.NA && res.type=="full"){
  ##   newres=list()
  ##   tokeep <- which(unlist(lapply(res,function(e) !all(is.na(e)))))
  ##   if(length(tokeep) > 0) for(i in 1:length(tokeep)) {newres[[i]] <- res[[tokeep[i]]]}
  ##   newres <- lapply(newres,function(e) {e[!is.na(e)] })
  ##   names(newres) <- names(res)[tokeep]
  ##   res <- newres
  ## }

  ## if(res.type=="matrix"){
  ##   res <- as.data.frame(lapply(res,unlist))
  ##   rnam <- rownames(res)
  ##   rownames(res) <- gsub(".X-squared","",rnam)
  ##   res <- as.matrix(res)
  ## }

  ## return(res)
}

##################
# Methods old2new
##################
.get_slot <- function(object, theslot = "strata"){
  if (.hasSlot(object, theslot)) slot(object, theslot)
  else NULL
}
#' Convert objects with obsolete classes into new objects
#'
#' The genind and genlight objects have changed in Adegenet version 2.0. They
#' have each gained strata and hierarchy slots. What's more is that the genind
#' objects have been optimized for storage and now store the tab slot as
#' integers instead of numerics. This function will convert old genind or
#' genlight objects to new ones seamlessly.
#'
#' @rdname old2new
#' @param object a genind or genlight object from version 1.4 or earlier.
#' @param donor a new object to place all the data into.
#' @aliases old2new
#' @author Thibaut Jombart \email{t.jombart@@imperial.ac.uk}\cr
#' Zhian N. Kamvar \email{kamvarz@@science.oregonstate.edu}
#' @keywords manip
#' @export
old2new_genind <- function(object, donor = new("genind")){
  if (missing(donor) || !is(donor, "genind")){
    donor <- new("genind")
  } 
  donor_tab       <- slot(object, "tab")
  donor_loc.fac   <- slot(object, "loc.fac")
  donor_all.names <- slot(object, "all.names")
  donor_ind.names <- slot(object, "ind.names")
  donor_loc.names <- slot(object, "loc.names")
  donor_loc.n.all <- slot(object, "loc.nall")
  donor_all.names <- slot(object, "all.names")
  donor_ploidy    <- slot(object, "ploidy")
  donor_type      <- slot(object, "type")
  donor_other     <- slot(object, "other")
  donor_call      <- slot(object, "call")
  donor_pop       <- slot(object, "pop")
  donor_pop.names <- slot(object, "pop.names")
  
  if (donor_type == "codom"){
    
    if (!is.integer(donor_tab) && length(donor_ploidy == 1)){
      xtab   <- donor_tab
      newtab <- as.integer(donor_tab * donor_ploidy)
      donor_tab <- matrix(newtab, nrow = nrow(xtab), ncol = ncol(xtab),
                          dimnames = dimnames(xtab))
      donor_ploidy <- rep(donor_ploidy, nrow(donor_tab))
    }
    
    names(donor_loc.names) <- NULL
    rownames(donor_tab)    <- donor_ind.names
    names(donor_all.names) <- donor_loc.names
    donor_all.names     <- lapply(donor_all.names, setNames, NULL)
    colnames(donor_tab) <- unlist(lapply(donor_loc.names,
                                         function(i) paste(i, donor_all.names[[i]],
                                                           sep = ".")),
                                  use.names = FALSE)
    levels(donor_loc.fac) <- donor_loc.names
    names(donor_loc.n.all) <- donor_loc.names
  }
  
  if (!is.null(donor_pop)){
    levels(donor_pop) <- donor_pop.names
  }
  donor_call <- match.call()
  donor@tab       <- donor_tab       
  donor@loc.fac   <- donor_loc.fac   
  donor@all.names <- donor_all.names 
  donor@loc.n.all <- donor_loc.n.all 
  donor@all.names <- donor_all.names 
  donor@ploidy    <- donor_ploidy    
  donor@type      <- donor_type      
  donor@other     <- donor_other     
  donor@call      <- donor_call      
  donor@pop       <- donor_pop       
  donor@strata    <- .get_slot(object, "strata")
  donor@hierarchy <- .get_slot(object, "hierarchy")
  return(donor)
}



#' @rdname old2new
#' @export
old2new_genlight <- function(object, donor = new("genlight")){
  if (missing(donor) || !is(donor, "genlight")){
    donor <- new("genlight")
  } 
  slot(donor, "gen") <- slot(object, "gen") 
  slot(donor, "n.loc") <- slot(object, "n.loc") 
  slot(donor, "ind.names") <- slot(object, "ind.names") 
  slot(donor, "loc.names") <- slot(object, "loc.names") 
  slot(donor, "loc.all") <- slot(object, "loc.all") 
  slot(donor, "chromosome") <- slot(object, "chromosome") 
  slot(donor, "position") <- slot(object, "position") 
  slot(donor, "ploidy") <- slot(object, "ploidy") 
  slot(donor, "pop") <- slot(object, "pop")
  slot(donor, "other") <- slot(object, "other") 
  donor@strata    <- .get_slot(object, "strata")
  donor@hierarchy <- .get_slot(object, "hierarchy")
  return(donor)
}

#' @rdname old2new
#' @export
old2new_genpop <- function(object, donor = new("genpop")){

  ## names(donor_pop.names) <- NULL
  ## names(donor_loc.names) <- NULL
  donor_tab       <- object@tab
  donor_loc.fac   <- object@loc.fac
  donor_all.names <- object@all.names
  donor_loc.n.all <- object@loc.nall
  donor_all.names <- object@all.names
  donor_ploidy    <- object@ploidy
  donor_type      <- object@type
  donor_other     <- object@other
  donor_call      <- object@call
  
  donor_pop.names <- object@pop.names
  donor_loc.names <- object@loc.names
  
  rownames(donor_tab) <- donor_pop.names
  colnames(donor_tab) <- unlist(lapply(donor_loc.names,
                                        function(i) paste(i, donor_all.names[[i]],
                                                          sep = ".")),
                                 use.names = FALSE)
  names(donor_all.names) <- donor_loc.names
  donor_all.names     <- lapply(donor_all.names, setNames, NULL)
  donor_call <- match.call()
  
  donor@tab       <- donor_tab
  donor@loc.fac   <- donor_loc.fac
  donor@all.names <- donor_all.names
  donor@loc.n.all <- donor_loc.n.all
  donor@all.names <- donor_all.names
  donor@ploidy    <- donor_ploidy
  donor@type      <- donor_type
  donor@other     <- donor_other
  donor@call      <- donor_call
  return(object)
  #   x <- object
#   res <- new("genpop")
#
#   res@tab <- as.matrix(x$tab)
#   res@pop.names <- as.character(x$pop.names)
#   res@loc.names <- as.character(x$loc.names)
#   res@loc.n.all <- as.integer(x$loc.n.all)
#   res@loc.fac <- as.factor(x$loc.fac)
#   res@all.names <- as.list(x$all.names)
#   res@ploidy <- as.integer(2)
#   res@type <- "codom"
#
#   res@call <- match.call()
#
#   if(length(object)>7) warning("optional content was not converted")
#
#   return(res)
}

###################################
#
# Distances functions
# based on old ade4 'dist.genet'
# by Daniel Chessel
#
# Thibaut Jombart
# t.jombart@imperial.ac.uk
###################################



############################
# S3 method dist for genpop
############################
dist.genpop <- function(x, method = 1, diag = FALSE, upper = FALSE) {

  if(!is.genpop(x)) stop("x is not a valid genpop object")

  ## haploidy kludge (have to get rid of that later)
  if(x@ploidy==as.integer(1)){
  x@tab <- x@tab * 2
  x@ploidy <- as.integer(2)
  }


  ## check marker type
  checkType(x)


  METHODS = c("Nei","Edwards","Reynolds","Rodgers","Provesti")
  if (all((1:5)!=method)) {
    cat("1 = Nei 1972\n")
    cat("2 = Edwards 1971\n")
    cat("3 = Reynolds, Weir and Coockerman 1983\n")
    cat("4 = Rodgers 1972\n")
    cat("5 = Provesti 1975\n")
    cat("Select an integer (1-5): ")
    method <- as.integer(readLines(con = getOption('adegenet.testcon'), n = 1))
  }
  if (all((1:5)!=method)) (stop ("Non convenient method number"))

  nloc <- length(levels(x@loc.fac))
  loc.fac <- x@loc.fac
  X <- makefreq(x,missing="mean",quiet=TRUE)
  # X is a matrix of allelic frequencies
  nlig <- nrow(X)

  if (method == 1) { # Nei
    d <- X%*%t(X)
    vec <- sqrt(diag(d))
    d <- d/vec[col(d)]
    d <- d/vec[row(d)]
    d <- -log(d)
    d <- as.dist(d)
  } else if (method == 2) { # Edward's (angular)
        X <- sqrt(X)
        d <- X%*%t(X)
        d <- 1-d/nloc
        diag(d) <- 0
        d <- sqrt(d)
        d <- as.dist(d)
    } else if (method == 3) { # Coancestrality coef (Reynold's)
       denomi <- X%*%t(X)
       vec <- apply(X,1,function(x) sum(x*x))
       d <- -2*denomi + vec[col(denomi)] + vec[row(denomi)]
       diag(d) <- 0
       denomi <- 2*nloc - 2*denomi
       diag(denomi) <- 1
       d <- d/denomi
       d <- sqrt(d)
       d <- as.dist(d)
    } else if (method == 4) { # Rogers' distance
      # kX is a list of K=nloc matrices
      kX <- lapply(split(X,loc.fac[col(X)]),matrix,nrow=nlig)
      dcano <- function(mat) {
        daux <- mat%*%t(mat)
        vec <- diag(daux)
        daux <- -2*daux + vec[col(daux)] + vec[row(daux)]
        diag(daux) <- 0
        daux <- sqrt(.5*daux)
        return(daux)
      }

      d <- matrix(0,nlig,nlig)
      for(i in 1:length(kX)) {
        d <- d + dcano(kX[[i]])
      }
      d <- d/length(kX)
      d <- as.dist(d)
    } else if (method ==5) { # Provesti (absolute genetic distance)
      w0 <- 1:(nlig-1)
      loca <- function(k) {
        w1 <- (k+1):nlig
        resloc <- unlist(lapply(w1, function(x) sum(abs(X[k,]-X[x,]))))
        return(resloc/(2*nloc))
      }
      d <- unlist(lapply(w0,loca))
    }
    attr(d, "Size") <- nlig
    attr(d, "Labels") <- popNames(x)
    attr(d, "Diag") <- diag
    attr(d, "Upper") <- upper
    attr(d, "method") <- METHODS[method]
    attr(d, "call") <- match.call()
    class(d) <- "dist"
    return(d)

} # end method dist for genpop

########################################################################
## adegenet classes definitions. All classes are S4.
##
## Initial development: Thibaut Jombart, November 2007
##
## Major reform for adegenet 2.0.0 (March-August 2015)
##
## t.jombart@imperial.ac.uk
########################################################################

###############################
# Two classes of R object are
# defined :
# gen - common part to genind and genpop
# genind - allele counts for individuals
# genpop - allele counts for populations
###############################


###############################################################
###############################################################
# AUXILIARY FUNCTIONS
###############################################################
###############################################################




###############################################################
###############################################################
# CLASSES DEFINITION
###############################################################
###############################################################

##.initAdegenetClasses <- function(){


####################
# Unions of classes
####################
setClassUnion("listOrNULL", c("list","NULL"))
setClassUnion("factorOrNULL", c("factor","NULL"))
setClassUnion("charOrNULL", c("character","NULL"))
setClassUnion("callOrNULL", c("call","NULL"))
setClassUnion("intOrNum", c("integer","numeric","NULL"))
setClassUnion("intOrNULL", c("integer","NULL"))
setClassUnion("dfOrNULL", c("data.frame", "NULL"))
setClassUnion("formOrNULL", c("formula", "NULL"))


####################
# virtual class gen
####################
.gen.valid <- function(object){
  # this function tests only the consistency
  # of the length of each component
  p <- ncol(object@tab)
  k <- length(levels(object@loc.fac))


  if(!is.null(object@loc.fac)){
      if(length(object@loc.fac) != p) {
          cat("\ninvalid length for loc.fac\n")
          return(FALSE)
      }

      if(length(levels(object@loc.fac)) != k) {
          cat("\ninvalid number of levels in loc.fac\n")
          return(FALSE)
      }
  }

  if(!is.null(object@loc.n.all)){
      if(length(object@loc.n.all) != k) {
          cat("\ninvalid length in loc.n.all\n")
          return(FALSE)
      }
  }

  if(!is.null(object@all.names)){
      if(length(unlist(object@all.names)) != p) {
          cat("\ninvalid length in all.names\n")
          return(FALSE)
      }
  }

  return(TRUE)

}# end .gen.valid


setClass("gen", representation(tab = "matrix",
                               loc.fac = "factorOrNULL",
                               loc.n.all = "intOrNum",
                               all.names = "listOrNULL",
                               ploidy = "integer",
                               type = "character",
                               other = "listOrNULL",
                               call = "callOrNULL",
                               "VIRTUAL"),
         prototype(tab=matrix(0L, ncol=0,nrow=0),
                   loc.fac=NULL,
                   loc.n.all=integer(0),
                   all.names=NULL,
                   ploidy=integer(0),
                   type=character(0),
                   other=NULL,
                   call=NULL))

setValidity("gen", .gen.valid)




########################
# virtual class indInfo
########################
setClass("indInfo", representation(pop = "factorOrNULL",
                                   strata = "dfOrNULL",
                                   hierarchy = "formOrNULL",
                                   "VIRTUAL"),
         prototype(pop=NULL, stata=NULL, hierarchy=NULL))





###############
# Class genind
###############

setClass("genind", contains=c("gen", "indInfo"))

.genind.valid <- function(object){

    validation <- TRUE
    if(!.gen.valid(object)) return(FALSE)

    if(typeof(object@tab) != "integer"){
        warning("@tab does not contain integers; as of adegenet_2.0-0, numeric values are no longer used")
        ## message("\ntab does not contain integers; as of adegenet_1.5-0, numeric values are no longer used")
        ## validation <- FALSE
    }


    if(!is.null(object@pop)){ # check pop

        if(length(object@pop) != nrow(object@tab)) {
            message("\npop is given but has invalid length\n")
            validation <- FALSE
        }

    } # end check pop

    # Check population strata
    if (!is.null(object@strata)){
      if (nrow(object@strata) != nrow(object@tab)){
        message("\na strata is defined has invalid length\n")
        validation <- FALSE
      }

      dups <- duplicated(colnames(object@strata))
      if (any(dups)){
        message("\nduplicated names found in @strata slot:\n")
        dups <- colnames(object@strata)[dups]
        message(paste0(dups, collapse = ", "))
        validation <- FALSE
      }
    }

    # TODO: CHECK HIERARCHY FORMULA

    ## check ploidy
    if(any(object@ploidy < 1L)){
        message("\nploidy inferior to 1\n")
        validation <- FALSE
    }
    if(length(object@ploidy)!= nrow(object@tab)){
        warning("as of adegenet_2.0-0, @ploidy should contain one value per individual")
    }

    ## check type of marker
    if(!object@type %in% c("codom","PA") ){
        message("\nunknown type of marker\n")
        validation <- FALSE
    }


    return(validation)
} #end .genind.valid

setValidity("genind", .genind.valid)



########################
# virtual class popInfo
########################
setClass("genpop", contains=c("gen"))



###############
# Class genpop
###############
.genpop.valid <- function(object){

    validation <- TRUE

    if(!.gen.valid(object)) return(FALSE)

     ## check ploidy
    if(length(object@ploidy) > 1 && object@ploidy < 1L){
        message("\nploidy inferior to 1\n")
        validation <- FALSE
    }

    ## check type of marker
    if(!object@type %in% c("codom","PA") ){
        message("\nunknown type of marker\n")
        validation <- FALSE
    }

    return(validation)
} #end .genpop.valid

setValidity("genpop", .genpop.valid)







###############################################################
###############################################################
## MISCELLANEOUS METHODS
###############################################################
###############################################################



#################
# Function names
#################
setMethod("names", signature(x = "genind"), function(x){
    return(slotNames(x))
})

setMethod("names", signature(x = "genpop"), function(x){
    return(slotNames(x))
})

context("Summary methods")

test_that("Diploid summaries work", {
  skip_on_cran()
  data("nancycats", package = "adegenet")
  nansum <- summary(nancycats, verbose = FALSE)
  expect_that(nansum, is_a("genindSummary"))
  expect_that(nansum$n, equals(nInd(nancycats)))
  expect_that(length(nansum$n.by.pop), equals(length(popNames(nancycats))))
  expect_that(length(nansum$pop.n.all), equals(length(popNames(nancycats))))
  expect_that(sample(nansum$Hobs, 1), is_more_than(0))
  expect_that(sample(nansum$Hexp, 1), is_more_than(0))
})

\encoding{UTF-8}
\name{a-score}
\alias{a.score}
\alias{optim.a.score}
\title{Compute and optimize a-score for Discriminant Analysis of Principal Components (DAPC)}
\description{
  These functions are under development. Please email the author before
  using them for published results.
}
\usage{
a.score(x, n.sim=10, \ldots)

optim.a.score(x, n.pca=1:ncol(x$tab), smart=TRUE, n=10, plot=TRUE,
              n.sim=10, n.da=length(levels(x$grp)), \ldots)
}
\arguments{
\item{x}{a \code{dapc} object.}
\item{n.pca}{a vector of \code{integers} indicating the number of axes retained in the
  Principal Component Analysis (PCA) steps of DAPC. \code{nsim} DAPC
  will be run for each value in \code{n.pca}, unless the smart approach
  is used (see details).}
\item{smart}{a \code{logical} indicating whether a smart, less
  computer-intensive approach should be used (TRUE, default) or not
  (FALSE). See details section.}
\item{n}{an \code{integer} indicating the numbers of values spanning the
  range of \code{n.pca} to be used in the smart approach.}
\item{plot}{a \code{logical} indicating whether the results should be
  displayed graphically (TRUE, default) or not (FALSE).}
\item{n.sim}{an \code{integer} indicating the number of simulations to
  be performed for each number of retained PC.}
\item{n.da}{an \code{integer} indicating the number of axes retained in the
  Discriminant Analysis step.}
\item{\ldots}{further arguments passed to other methods; currently unused..}
}
\details{
  The Discriminant Analysis of Principal Components seeks a reduced
  space inside which observations are best discriminated into
  pre-defined groups. One way to assess the quality of the
  discrimination is looking at re-assignment of individuals to their
  prior group, successful re-assignment being a sign of strong discrimination.

  However, when the original space is very large, ad hoc solutions can
  be found, which discriminate very well the sampled individuals but
  would perform poorly on new samples. In such a case, DAPC
  re-assignment would be high even for randomly chosen clusters.
  The a-score measures this bias. It is computed as (Pt-Pr), where Pt is
  the reassignment probability using the true cluster, and Pr is the
  reassignment probability for randomly permuted clusters. A a-score
  close to one is a sign that the DAPC solution is both strongly
  discriminating and stable, while low values (toward 0 or lower)
  indicate either weak discrimination or instability of the results.

  The a-score can serve as a criterion for choosing the optimal number of
  PCs in the PCA step of DAPC, i.e. the number of PC maximizing the
  a-score. Two procedures are implemented in \code{optim.a.score}. The
  smart procedure selects evenly distributed number of PCs in a
  pre-defined range, compute the a-score for each, and then interpolate
  the results using splines, predicting an approximate optimal number of
  PCs. The other procedure (when \code{smart} is FALSE) performs the
  computations for all number of PCs request by the user. The 'optimal'
  number is then the one giving the highest mean a-score (computed over
  the groups).
}
\value{
  === a.score ===\cr
 \code{a.score} returns a list with the following components:\cr
  \item{tab}{a matrix of a-scores with groups in columns and simulations in row.}
  \item{pop.score}{a vector giving the mean a-score for each population.}
  \item{mean}{the overall mean a-score.}\cr
  
  === optim.a.score ===\cr
  \code{optima.score} returns a list with the following components:\cr
   \item{pop.score}{a list giving the mean a-score of the populations
   for each number of retained PC (each element of the list corresponds
   to a number of retained PCs).}
  \item{mean}{a vector giving the overall mean a-score for each number
    of retained PCs.}
  \item{pred}{(only when \code{smart} is TRUE) the predictions of the
    spline, given in x and y coordinates.}
 \item{best}{the optimal number of PCs to be retained.}
}
\references{
  Jombart T, Devillard S and Balloux F  (2010) Discriminant analysis of
  principal components: a new method for the analysis of genetically
  structured populations. BMC Genetics11:94. doi:10.1186/1471-2156-11-94
}
\seealso{
  - \code{\link{find.clusters}}: to identify clusters without prior.

  - \code{\link{dapc}}: the Discriminant Analysis of Principal
    Components (DAPC)
}
\author{ Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hybridize.R
\name{hybridize}
\alias{hybridize}
\title{Function hybridize takes two genind in inputs
and generates hybrids individuals having one parent
in both objects.}
\usage{
hybridize(x1, x2, n, pop = "hybrid", res.type = c("genind", "df",
  "STRUCTURE"), file = NULL, quiet = FALSE, sep = "/", hyb.label = "h")
}
\arguments{
\item{x1}{a \linkS4class{genind} object}

\item{x2}{a \linkS4class{genind} object}

\item{n}{an integer giving the number of hybrids requested}

\item{pop}{a character string giving naming the population of the
created hybrids.}

\item{res.type}{a character giving the type of output requested. Must
be "genind" (default), "df" (i.e. data.frame like in
\code{\link{genind2df}}), or "STRUCTURE" to generate a .str file
readable by STRUCTURE (in which case the 'file' must be supplied). See
'details' for STRUCTURE output.}

\item{file}{a character giving the name of the file to be written
when 'res.type' is "STRUCTURE"; if NULL, a the created file is of the
form "hybrids\_[the current date].str".}

\item{quiet}{a logical specifying whether the writing to a file (when
'res.type' is "STRUCTURE") should be announced (FALSE, default) or
not (TRUE).}

\item{sep}{a character used to separate two alleles}

\item{hyb.label}{a character string used to construct the hybrids
labels; by default, "h", which gives labels: "h01", "h02", "h03",...}
}
\value{
A \linkS4class{genind} object (by default), or a data.frame of alleles
(res.type="df"). No R output if res.type="STRUCTURE" (results written
to the specified file).
}
\description{
The function \code{hybridize} performs hybridization between two set
of genotypes stored in \linkS4class{genind} objects (referred as the "2
populations"). Allelic frequencies are derived for each population,
and then gametes are sampled following a multinomial distribution. \cr
}
\details{
The result consists in a set of 'n' genotypes, with different possible
outputs (see 'res.type' argument).


  If the output is a STRUCTURE file, this file will have the following
 caracteristics:\cr
 - file contains the genotypes of the parents, and then the genotypes
 of hybrids\cr
 - the first column identifies genotypes\cr
 - the second column identifies the population (1 and 2 for parents x1 and x2;
 3 for hybrids)\cr
 - the first line contains the names of the markers\cr
 - one row = one genotype (onerowperind will be true)\cr
 - missing values coded by "-9" (the software's default)\cr
}
\examples{
\dontrun{
## Let's make some cattle hybrids
data(microbov)

## first, isolate each breed
temp <- seppop(microbov)
names(temp)
salers <- temp$Salers
zebu <- temp$Zebu

## let's make some... Zeblers
zebler <- hybridize(salers, zebu, n=40,
                    pop="Zebler")


## now let's merge all data into a single genind
newDat <- repool(microbov, zebler)

## make a correspondance analysis
## and see where hybrids are placed
X <- genind2genpop(newDat, quiet=TRUE)
coa1 <- dudi.coa(tab(X),scannf=FALSE,nf=3)
s.label(coa1$li)
add.scatter.eig(coa1$eig,2,1,2)

}


}
\seealso{
\code{\link{seploc}}, \code{\link{seppop}}, \code{\link{repool}}
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compoplot.R
\name{compoplot}
\alias{compoplot}
\alias{compoplot.matrix}
\alias{compoplot.dapc}
\alias{compoplot.snapclust}
\title{Genotype composition plot}
\usage{
compoplot(x, ...)

\method{compoplot}{matrix}(x, col.pal = funky, border = NA, subset = NULL,
  show.lab = FALSE, lab = rownames(x), legend = TRUE,
  txt.leg = colnames(x), n.col = 4, posi = NULL, cleg = 0.8,
  bg = transp("white"), ...)

\method{compoplot}{dapc}(x, only.grp = NULL, border = NA, ...)

\method{compoplot}{snapclust}(x, border = NA, ...)
}
\arguments{
\item{x}{an object to be used for plotting (see description)}

\item{...}{further arguments to be passed to \code{barplot}}

\item{col.pal}{a color palette to be used for the groups; defaults to \code{funky}}

\item{border}{a color for the border of the barplot; use \code{NA} to
indicate no border.}

\item{subset}{a subset of individuals to retain}

\item{show.lab}{a logical indicating if individual labels should be displayed}

\item{lab}{a vector of individual labels; if NULL, row.names of the matrix are used}

\item{legend}{a logical indicating whether a legend should be provided for the colors}

\item{txt.leg}{a character vector to be used for the legend}

\item{n.col}{the number of columns to be used for the legend}

\item{posi}{the position of the legend}

\item{cleg}{a size factor for the legend}

\item{bg}{the background to be used for the legend}

\item{only.grp}{a subset of groups to retain}
}
\description{
The compoplot uses a barplot to represent the group assignment probability of
individuals to several groups. It is a generic with methods for the following
objects:
}
\details{
\itemize{

\item \code{matrix}: a matrix with individuals in row and genetic clusters in
column, each entry being an assignment probability of the corresponding
individual to the corresponding group

\item \code{dapc}: the output of the \code{dapc} function; in this case,
group assignments are based upon geometric criteria in the discriminant space

\item \code{snapclust}: the output of the \code{snapclust} function; in
this case, group assignments are based upon the likelihood of genotypes
belonging to their groups

}
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{df2genind}
\alias{df2genind}
\title{Convert a data.frame of allele data to a genind object.}
\usage{
df2genind(X, sep = NULL, ncode = NULL, ind.names = NULL,
  loc.names = NULL, pop = NULL, NA.char = "", ploidy = 2,
  type = c("codom", "PA"), strata = NULL, hierarchy = NULL)
}
\arguments{
\item{X}{a matrix or a data.frame containing allelle data only (see
decription)}

\item{sep}{a character string separating alleles. See details.}

\item{ncode}{an optional integer giving the number of characters used for
coding one genotype at one locus. If not provided, this is determined from
data.}

\item{ind.names}{optinal, a vector giving the individuals names; if NULL,
taken from rownames of X. If factor or numeric, vector is converted to
character.}

\item{loc.names}{an optional character vector giving the markers names; if
NULL, taken from colnames of X.}

\item{pop}{an optional factor giving the population of each individual.}

\item{NA.char}{a character string corresponding to missing allele (to be
treated as NA)}

\item{ploidy}{an integer indicating the degree of ploidy of the genotypes.}

\item{type}{a character string indicating the type of marker: 'codom' stands
for 'codominant' (e.g. microstallites, allozymes); 'PA' stands for
'presence/absence' markers (e.g. AFLP, RAPD).}

\item{strata}{an optional data frame that defines population stratifications
for your samples. This is especially useful if you have a hierarchical or
factorial sampling design.}

\item{hierarchy}{a hierarchical formula that explicitely defines hierarchical
levels in your strata. see \code{\link{hierarchy}} for details.}
}
\value{
an object of the class \linkS4class{genind} for \code{df2genind}; a
  matrix of biallelic genotypes for \code{genind2df}
}
\description{
The function \code{df2genind} converts a data.frame (or a matrix) into a
\linkS4class{genind} object. The data.frame must meet the following
requirements:
\itemize{
\item genotypes are in row (one row per genotype)
\item markers/loci are in columns
\item each element is a string of characters coding alleles, ideally
separated by a character string (argument \code{sep}); if no separator is
used, the number of characters coding alleles must be indicated (argument
\code{ncode}).}
}
\details{
See \code{\link{genind2df}} to convert \linkS4class{genind} objects back to
such a data.frame.

=== Details for the \code{sep} argument ===\cr this character is directly
used in reguar expressions like \code{gsub}, and thus require some characters
to be preceeded by double backslashes. For instance, "/" works but "|" must
be coded as "\\|".
}
\examples{

## simple example
df <- data.frame(locusA=c("11","11","12","32"),
locusB=c(NA,"34","55","15"),locusC=c("22","22","21","22"))
row.names(df) <- .genlab("genotype",4)
df

obj <- df2genind(df, ploidy=2, ncode=1)
obj
tab(obj)


## converting a genind as data.frame
genind2df(obj)
genind2df(obj, sep="/")

}
\seealso{
\code{\link{genind2df}}, \code{\link{import2genind}},
  \code{\link{read.genetix}}, \code{\link{read.fstat}},
  \code{\link{read.structure}}
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}, Zhian N. Kamvar
  \email{kamvarz@science.oregonstate.edu}
}
\keyword{manip}

\encoding{UTF-8}
\docType{methods}
\name{Auxiliary functions}
\alias{checkType}
\alias{.rmspaces}
\alias{.genlab}
\alias{.readExt}
\alias{corner}
\alias{num2col}
\alias{fac2col}
\alias{any2col}
\alias{transp}
\alias{bluepal}
\alias{redpal}
\alias{greenpal}
\alias{greypal}
\alias{flame}
\alias{azur}
\alias{seasun}
\alias{lightseasun}
\alias{deepseasun}
\alias{spectral}
\alias{wasp}
\alias{funky}
\alias{virid}
\alias{hybridpal}

\title{ Auxiliary functions for adegenet}
\description{
  adegenet implements a number of auxiliary procedures that might be of
  interest for users. These include graphical tools to translate
  variables (numeric or factors) onto a color scale, adding transparency
  to existing colors, pre-defined color palettes, extra functions to
  access documentation, and low-level treatment of character vectors.

  These functions are mostly auxiliary procedures used internally in
  adegenet.\cr

  These items include:
  \itemize{
    \item \code{num2col}: translates a numeric vector into colors. 
    \item \code{fac2col}: translates a factor into colors. 
    \item \code{any2col}: translates a vector of type numeric, character
    or factor into colors. 
    \item \code{transp}: adds transparency to a vector of colors. Note that
    transparent colors are not supported on some graphical devices.
    \item \code{corner}: adds text to a corner of a figure. 
    \item \code{checkType}: checks the type of markers being used in a
    function and issues an error if appropriate.
    \item \code{.rmspaces}: remove peripheric spaces in a character string. 
    \item \code{.genlab}: generate labels in a correct alphanumeric ordering. 
    \item \code{.readExt}: read the extension of a given file. 
  }

  Color palettes include:
  \itemize{
    \item \code{bluepal}: white -> dark blue
    \item \code{redpal}: white -> dark red
    \item \code{greenpal}: white -> dark green
    \item \code{greypal}: white -> dark grey
    \item \code{flame}: gold -> red
    \item \code{azur}: gold -> blue
    \item \code{seasun}: blue -> gold -> red
    \item \code{lightseasun}: blue -> gold -> red (light variant)
    \item \code{deepseasun}: blue -> gold -> red (deep variant)
    \item \code{spectral}: red -> yellow -> blue (RColorBrewer variant)
    \item \code{wasp}: gold -> brown -> black
    \item \code{funky}: many colors
    \item \code{virid}: adaptation of the \code{viridis} palette, from
    the \code{viridis} package.
    \item \code{hybridpal}: reorder a color palette (\code{virid} by
  default) to display sharp contrast between the first two colors, and
  interpolated colors after; ideal for datasets where two parental
  populations are provided first, followed by various degrees of
  hybrids.

  }
}
\seealso{
  The R package RColorBrewer, proposing a nice selection of color
  palettes. The \code{viridis} package, with many excellent palettes.
}
\usage{
.genlab(base, n)
corner(text, posi="topleft",  inset=0.1, \dots)
num2col(x, col.pal=heat.colors, reverse=FALSE,
        x.min=min(x,na.rm=TRUE), x.max=max(x,na.rm=TRUE),
        na.col="transparent")
fac2col(x, col.pal=funky, na.col="transparent", seed=NULL)
any2col(x, col.pal=seasun, na.col="transparent")
transp(col, alpha=.5)
hybridpal(col.pal = virid)
}
\arguments{
  \item{base}{a character string forming the base of the labels}
  \item{n}{the number of labels to generate}
  \item{text}{a character string to be added to the plot}
  \item{posi}{a character matching any combinations of "top/bottom" and
    "left/right".}
  \item{inset}{a vector of two numeric values (recycled if needed)
    indicating the inset, as a fraction of the plotting region.}
  \item{\dots}{further arguments to be passed to \code{\link{text}}}
  \item{x}{a numeric vector (for \code{num2col}) or a vector converted
    to a factor (for \code{fac2col}).}
  \item{col.pal}{a function generating colors according to a given palette.}
  \item{reverse}{a logical stating whether the palette should be
    inverted (TRUE), or not (FALSE, default).}
  \item{x.min}{the minimal value from which to start the color scale}
  \item{x.max}{the maximal value from which to start the color scale}
  \item{na.col}{the color to be used for missing values (NAs)}
  \item{seed}{a seed for R's random number generated, used to fix the
    random permutation of colors in the palette used; if NULL, no
    randomization is used and the colors are taken from the palette
    according to the ordering of the levels.}
  \item{col}{a vector of colors}
  \item{alpha}{a numeric value between 0 and 1 representing the alpha
    coefficient; 0: total transparency; 1: no transparency.}
}
\value{
  For \code{.genlab}, a character vector of size "n".
  \code{num2col} and \code{fac2col} return a vector of
  colors. \code{any2col} returns a list with the following components:
  \code{$col} (a vector of colors), \code{$leg.col} (colors for the
  legend), and \code{$leg.txt} (text for the legend).
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{

.genlab("Locus-",11)

## transparent colors using "transp"
plot(rnorm(1000), rnorm(1000), col=transp("blue",.3), pch=20, cex=4)


## numeric values to color using num2col
plot(1:100, col=num2col(1:100), pch=20, cex=4)
plot(1:100, col=num2col(1:100, col.pal=bluepal), pch=20, cex=4)
plot(1:100, col=num2col(1:100, col.pal=flame), pch=20, cex=4)
plot(1:100, col=num2col(1:100, col.pal=wasp), pch=20, cex=4)
plot(1:100, col=num2col(1:100, col.pal=azur,rev=TRUE), pch=20, cex=4)
plot(1:100, col=num2col(1:100, col.pal=spectral), pch=20, cex=4)
plot(1:100, col=num2col(1:100, col.pal=virid), pch=20, cex=4)

## factor as colors using fac2col
dat <- cbind(c(rnorm(50,8), rnorm(100), rnorm(150,3),
rnorm(50,10)),c(rnorm(50,1),rnorm(100),rnorm(150,3), rnorm(50,5)))
fac <- rep(letters[1:4], c(50,100,150,50))
plot(dat, col=fac2col(fac), pch=19, cex=4)
plot(dat, col=transp(fac2col(fac)), pch=19, cex=4)
plot(dat, col=transp(fac2col(fac,seed=2)), pch=19, cex=4)

## use of any2col
x <- factor(1:10)
col.info <- any2col(x, col.pal=funky)
plot(x, col=col.info$col, main="Use of any2col on a factor")
legend("bottomleft", fill=col.info$leg.col, legend=col.info$leg.txt, bg="white")

x <- 100:1
col.info <- any2col(x, col.pal=wasp)
barplot(x, col=col.info$col, main="Use of any2col on a numeric")
legend("bottomleft", fill=col.info$leg.col, legend=col.info$leg.txt, bg="white")

}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{read.snp}
\alias{read.snp}
\title{Reading Single Nucleotide Polymorphism data}
\usage{
read.snp(file, quiet = FALSE, chunkSize = 1000, parallel = FALSE,
  n.cores = NULL, ...)
}
\arguments{
\item{file}{a character string giving the path to the file to convert, with
the extension ".snp".}

\item{quiet}{logical stating whether a conversion messages should be printed
(TRUE,default) or not (FALSE).}

\item{chunkSize}{an integer indicating the number of genomes to be read at a
time; larger values require more RAM but decrease the time needed to read
the data.}

\item{parallel}{a logical indicating whether multiple cores -if available-
should be used for the computations (TRUE, default), or not (FALSE);
requires the package \code{parallel} to be installed (see details).}

\item{n.cores}{if \code{parallel} is TRUE, the number of cores to be used in
the computations; if NULL, then the maximum number of cores available on the
computer is used.}

\item{\dots}{other arguments to be passed to other functions - currently not
used.}
}
\value{
an object of the class \code{"\linkS4class{genlight}"}
}
\description{
The function \code{read.snp} reads a SNP data file with extension '.snp' and
converts it into a \linkS4class{genlight} object. This format is devoted to
handle biallelic SNP only, but can accommodate massive datasets such as
complete genomes with considerably less memory than other formats.
}
\details{
The function reads data by chunks of a few genomes (minimum 1, no maximum)
at a time, which allows one to read massive datasets with negligible RAM
requirements (albeit at a cost of computational time). The argument
\code{chunkSize} indicates the number of genomes read at a time. Increasing
this value decreases the computational time required to read data in, while
increasing memory requirements.

A description of the .snp format is provided in an example file distributed
with adegenet (see example below).

=== The .snp format ===

Details of the .snp format can be found in the example file distributed with
adegenet (see below), or on the adegenet website (type \code{adegenetWeb()}
in R).
}
\examples{

\dontrun{
## show the example file ##
## this is the path to the file:
system.file("files/exampleSnpDat.snp",package="adegenet")

## show its content:
file.show(system.file("files/exampleSnpDat.snp",package="adegenet"))


## read the file
obj <-
read.snp(system.file("files/exampleSnpDat.snp",package="adegenet"), chunk=2)
obj
as.matrix(obj)
ploidy(obj)
alleles(obj)
locNames(obj)
}

}
\seealso{
- \code{?genlight} for a description of the class
\code{"\linkS4class{genlight}"}.

- \code{\link{read.PLINK}}: read SNPs in PLINK's '.raw' format.

- \code{\link{fasta2genlight}}: extract SNPs from alignments with fasta
format.

- \code{\link{df2genind}}: convert any multiallelic markers into adegenet
\code{"\linkS4class{genlight}"}.

- \code{\link{import2genind}}: read multiallelic markers from various
software into adegenet.\cr
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export.R
\name{genind2df}
\alias{genind2df}
\title{Convert a genind object to a data.frame.}
\usage{
genind2df(x, pop = NULL, sep = "", usepop = TRUE, oneColPerAll = FALSE)
}
\arguments{
\item{x}{a \linkS4class{genind} object}

\item{pop}{an optional factor giving the population of each individual.}

\item{sep}{a character string separating alleles. See details.}

\item{usepop}{a logical stating whether the population (argument \code{pop}
or \code{x@pop} should be used (TRUE, default) or not (FALSE)).}

\item{oneColPerAll}{a logical stating whether or not alleles should be split
into columns (defaults to \code{FALSE}). This will only work with data with
consistent ploidies.}
}
\value{
a data.frame of raw allelic data, with individuals in rows and loci in column
}
\description{
The function \code{genind2df} converts a \linkS4class{genind} back to a
data.frame of raw allelic data.
}
\examples{

## simple example
df <- data.frame(locusA=c("11","11","12","32"),
locusB=c(NA,"34","55","15"),locusC=c("22","22","21","22"))
row.names(df) <- .genlab("genotype",4)
df

obj <- df2genind(df, ploidy=2, ncode=1)
obj
obj@tab


## converting a genind as data.frame
genind2df(obj)
genind2df(obj, sep="/")

}
\seealso{
\code{\link{df2genind}}, \code{\link{import2genind}}, \code{\link{read.genetix}},
\code{\link{read.fstat}}, \code{\link{read.structure}}
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constructors.R
\docType{methods}
\name{initialize,genpop-method}
\alias{initialize,genpop-method}
\alias{initialize,genpop-methods}
\alias{genpop}
\alias{as.genpop}
\title{genpop constructor}
\usage{
\S4method{initialize}{genpop}(.Object, tab, prevcall = NULL, ploidy = 2L,
  type = c("codom", "PA"), ...)

genpop(...)

as.genpop(...)
}
\arguments{
\item{.Object}{prototyped object (generated automatically when calling 'new')}

\item{tab}{A matrix of integers corresponding to the @tab slot of a genpop object, with individuals in rows and alleles in columns, and containing either allele counts}

\item{prevcall}{an optional call to be stored in the object}

\item{ploidy}{an integer vector indicating the ploidy of the individual; each individual can have a different value; if only one value is provided, it is recycled to generate a vector of the right length.}

\item{type}{a character string indicating the type of marker: codominant ("codom") or presence/absence ("PA")}

\item{...}{further arguments passed to other methods (currently not used)}
}
\value{
a \linkS4class{genpop} object
}
\description{
The function \code{new} has a method for building \linkS4class{genpop} objects.
See the class description of \linkS4class{genpop} for more information on this data structure.
The functions \code{genpop} and \code{as.genpop} are aliases for \code{new("genpop", ...)}.
}
\details{
Most users do not need using the constructor, but merely to convert raw allele data using \code{\link{genind2genpop}}.
}
\seealso{
the description of the \linkS4class{genpop} class; \code{\link{df2genind}} and related functions for reading raw allele data
}

\name{seqTrack}
\alias{seqTrack}
\alias{seqTrack.default}
\alias{seqTrack.matrix}
\alias{plotSeqTrack}
\alias{get.likelihood}
\alias{get.likelihood.seqTrack}
\alias{seqTrack-class}
\alias{as.igraph.seqTrack}
\alias{plot.seqTrack}
%\alias{as,seqTrack,graphNEL-method}
%\alias{coerce,seqTrack,graphNEL-method}
\title{SeqTrack algorithm for reconstructing genealogies}
\description{
  The SeqTrack algorithm [1] aims at reconstructing genealogies of sampled
  haplotypes or genotypes for which a collection date is
  available. Contrary to phylogenetic methods which aims at reconstructing
  hypothetical ancestors for observed sequences, SeqTrack
  considers that ancestors and descendents are sampled together, and
  therefore infers ancestry relationships among the sampled sequences.

  This approach proved more efficient than phylogenetic approaches for
  reconstructing transmission trees in densely sampled disease outbreaks
  [1]. This implementation defines a generic function \code{seqTrack}
  with methods for specific object classes.

}
\usage{
seqTrack(\ldots)

\method{seqTrack}{matrix}(x, x.names, x.dates, best = c("min", "max"),
    prox.mat = NULL, mu = NULL, haplo.length = NULL, \dots)

\method{as.igraph}{seqTrack}(x, col.pal=redpal, \dots)

\method{plot}{seqTrack}(x, y=NULL, col.pal=redpal, \dots)

plotSeqTrack(x, xy, use.arrows=TRUE, annot=TRUE, labels=NULL, col=NULL,
                         bg="grey", add=FALSE, quiet=FALSE,
                         date.range=NULL, jitter.arrows=0, plot=TRUE, \dots)

get.likelihood(\ldots)

\method{get.likelihood}{seqTrack}(x, mu, haplo.length, \ldots)
}
\arguments{
  \item{x}{for seqTrack, a matrix giving weights to pairs of ancestries
    such that x[i,j] is the weight of 'i ancestor of j'. For
    plotSeqTrack and get.likelihood. seqTrack, a \code{seqTrack}
    object.}
  \item{x.names}{a character vector giving the labels of the haplotypes/genotypes}
  \item{x.dates}{a vector of collection dates for the sampled
    haplotypes/genotypes. Dates must have the POSIXct format. See
    \code{details} or \code{?as.POSIXct} for more information.}
  \item{best}{a character string matching 'min' or 'max', indicating
    whether genealogies should minimize or maximize the sum of weights
    of ancestries.}
  \item{prox.mat}{an optional matrix of proximities between
    haplotypes/genotypes used to resolve ties in the choice of
    ancestors, by picking up the 'closest' ancestor amongst possible
    ancestors, in the sense of \code{prox.mat}. \code{prox.mat[i,j]}
    must indicate a proximity for the relationship 'i ancestor to
    j'. For instance, if \code{prox.mat} contains spatial proximities,
    then \code{prox.mat[i,j]} gives a measure of how easy it is to
    migrate from location 'i' to 'j'.
  }
  \item{mu}{(optional) a mutation rate, per site and per day. When 'x'
    contains numbers of mutations, used to resolve ties using a maximum
    likelihood approach (requires \code{haplo.length} to be provided).}
 \item{haplo.length}{(optional) the length of analysed sequences in
    number of nucleotides. When 'x' contains numbers of mutations, used
    to resolve ties using a maximum likelihood approach (requires
    \code{mu} to be provided).}
  \item{y}{unused argument, for compatibility with 'plot'.}
  \item{col.pal}{a color palette to be used to represent weights using
   colors on the edges of the graph. See \code{?num2col}. Note that the
   palette is inversed by default.}
  \item{xy}{spatial coordinates of the sampled haplotypes/genotypes.}
  \item{use.arrows}{a logical indicating whether arrows should be used to
    represented ancestries (pointing from ancestor to descendent, TRUE),
    or whether segments shall be used (FALSE).}
  \item{annot}{a logical indicating whether arrows or segments
    representing ancestries should be annotated (TRUE) or not (FALSE).}
  \item{labels}{a character vector containing annotations of the
    ancestries. If left empty, ancestries are annotated by the descendent.}
  \item{col}{a vector of colors to be used for plotting ancestries.}
  \item{bg}{a color to be used as background.}
  \item{add}{a logical stating whether the plot should be added to
    current figure (TRUE), or drawn as a new plot (FALSE, default).}
  \item{quiet}{a logical stating whether messages other than errors
    should be displayed (FALSE, default), or hidden (TRUE).}
   \item{date.range}{a vector of length two with POSIXct format indicating
     the time window for which ancestries should be displayed.}
   \item{jitter.arrows}{a positive number indicating the amount of noise
   to be added to coordinates of arrows; useful when several arrows
   overlap. See \code{\link{jitter}}.}
  \item{plot}{a logical stating whether a plot should be drawn (TRUE,
    default), or not (FALSE). In all cases, the
    function invisibly returns plotting information.}
  \item{\ldots}{further arguments to be passed to other methods}
}
\author{
  Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\references{
  Jombart T, Eggo R, Dodd P, Balloux F (2010) Reconstructing disease
  outbreaks from genetic data: a graph approach. Heredity. doi:
  10.1038/hdy.2010.78.
}
\value{
  === output of seqTrack ===\cr
  seqTrack function returns data.frame with the class \code{seqTrack},
  in which each row is an inferred ancestry described by the following columns:
  - id: indices identifying haplotypes/genotypes\cr
  - ances: index of the inferred ancestor\cr
  - weight: weight of the inferred ancestries\cr
  - date: date of the haplotype/genotype\cr
  - ances.date: date of the ancestor

  === output of plotSeqTrack ===\cr
  This graphical function invisibly returns the coordinates of the
  arrows/segments drawn and their colors, as a data.frame.
}
\details{
  === Maximum parsimony genealogies ===\cr
  Maximum parsimony genealogies can be obtained easily using this
  implementation of seqTrack. One has to provide in \code{x} a matrix of
  genetic distances. The most straightforward distance is the number of
  differing nucleotides. See \code{\link[ape]{dist.dna}} in the ape
  package for a wide range of genetic distances between aligned
  sequences. The argument \code{best} should be set to "min" (its
  default value), so that the identified genealogy minimizes the total
  number of mutations. If \code{x} contains number of mutations, then
  \code{mu} and \code{haplo.length} should also be provided for
  resolving ties in equally parsimonious ancestors using maximum
  likelihood.

  === Likelihood of observed genetic differentiation ===\cr
  The probability of oberving a given number of mutations between a
  sequence and its ancestor can be computed using
  \code{get.likelihood.seqTrack}. Note that this is only possible
  if \code{x} contained number of mutations.


  === Plotting/converting seqTrack objects to graphs ===\cr
  seqTrack objects are best plotted as graphs. From adegenet_1.3-5
  onwards, seqTrack objects can be converted to \code{igraph} objects (from the
  package \code{igraph}), which can in turn be plotted and manipulated
  using classical graph tools. The plot method does this operation
  automatically, using colors to represent edge weights, and using
  time-ordering of the data from top (ancient) to bottom (recent).
}
\seealso{
  \code{\link[ape]{dist.dna}} in the ape package to compute pairwise
  genetic distances in aligned sequences.
}
\examples{
\dontrun{
if(require(ape && require(igraph))){
## ANALYSIS OF SIMULATED DATA ##
## SIMULATE A GENEALOGY
dat <- haploGen(seq.l=1e4, repro=function(){sample(1:4,1)}, gen.time=1, t.max=3)
plot(dat, main="Simulated data")

## SEQTRACK ANALYSIS
res <- seqTrack(dat, mu=0.0001, haplo.length=1e4) 
plot(res, main="seqTrack reconstruction")

## PROPORTION OF CORRECT RECONSTRUCTION
mean(dat$ances==res$ances,na.rm=TRUE)


## ANALYSIS OF PANDEMIC A/H1N1 INFLUENZA DATA ##
## note:
## this is for reproduction purpose only
## seqTrack is best kept for the analysis
## of densely sampled outbreaks, which
## is not the case of this dataset.
## 
dat <- read.csv(system.file("files/pdH1N1-data.csv",package="adegenet"))
ha <-  read.dna(system.file("files/pdH1N1-HA.fasta",package="adegenet"), format="fa")
na <- read.dna(system.file("files/pdH1N1-NA.fasta",package="adegenet"), format="fa")


## COMPUTE NUCLEOTIDIC DISTANCES
nbNucl <- ncol(as.matrix(ha)) + ncol(as.matrix(na))
D <- dist.dna(ha,model="raw")*ncol(as.matrix(ha)) +
dist.dna(na,model="raw")*ncol(as.matrix(na))
D <- round(as.matrix(D))


## MATRIX OF SPATIAL CONNECTIVITY
## (to promote local transmissions)
xy <- cbind(dat$lon, dat$lat)
temp <- as.matrix(dist(xy))
M <- 1* (temp < 1e-10)


## SEQTRACK ANALYSIS
dat$date <- as.POSIXct(dat$date)
res <- seqTrack(D, rownames(dat), dat$date, prox.mat=M, mu=.00502/365, haplo.le=nbNucl)


## COMPUTE GENETIC LIKELIHOOD
p <- get.likelihood(res, mu=.00502/365, haplo.length=nbNucl)
# (these could be shown as colors when plotting results)
# (but mutations will be used instead)


## EXAMINE RESULTS
head(res)
tail(res)
range(res$weight, na.rm=TRUE)
barplot(table(res$weight)/sum(!is.na(res$weight)), ylab="Frequency",
xlab="Mutations between inferred ancestor and descendent", col="orange")


## DISPLAY SPATIO-TEMPORAL DYNAMICS 
if(require(maps)){
myDates <- as.integer(difftime(dat$date, as.POSIXct("2009-01-21"), unit="day"))
myMonth <- as.POSIXct(
c("2009-02-01", "2009-03-01","2009-04-01","2009-05-01","2009-06-01","2009-07-01"))
x.month <-  as.integer(difftime(myMonth, as.POSIXct("2009-01-21"), unit="day"))


## FIRST STAGE:
## SPREAD TO THE USA AND CANADA
curRange <- as.POSIXct(c("2009-03-29","2009-04-25"))
par(bg="deepskyblue")
map("world", fill=TRUE, col="grey")
opal <- palette()
palette(rev(heat.colors(10)))
plotSeqTrack(res, round(xy), add=TRUE,annot=FALSE,lwd=2,date.range=curRange,
col=res$weight+1)
title(paste(curRange, collapse=" to "))
legend("bottom", lty=1, leg=0:8, title="number of mutations", col=1:9,
lwd=2, horiz=TRUE)


## SECOND STAGE:
## SPREAD WITHIN AMERICA, FIRST SEEDING OUTSIDE AMERICA
curRange <- as.POSIXct(c("2009-04-30","2009-05-07"))
par(bg="deepskyblue")
map("world", fill=TRUE, col="grey")
opal <- palette()
palette(rev(heat.colors(10)))
plotSeqTrack(res, round(xy), add=TRUE,annot=FALSE,lwd=2,
date.range=curRange, col=res$weight+1)
title(paste(curRange, collapse=" to "))
legend("bottom", lty=1, leg=0:8, title="number of mutations",
col=1:9,lwd=2, horiz=TRUE)


## THIRD STAGE:
## PANDEMIC
curRange <- as.POSIXct(c("2009-05-15","2009-05-25"))
par(bg="deepskyblue")
map("world", fill=TRUE, col="grey")
opal <- palette()
palette(rev(heat.colors(10)))
plotSeqTrack(res, round(xy), add=TRUE,annot=FALSE,lwd=2, date.range=curRange,
col=res$weight+1)
title(paste(curRange, collapse=" to "))
legend("bottom", lty=1, leg=0:8, title="number of mutations",
col=1:9,lwd=2, horiz=TRUE)

}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/showmekittens.R
\name{showmekittens}
\alias{showmekittens}
\title{When you need a break...}
\usage{
showmekittens(x = NULL, list = FALSE)
}
\arguments{
\item{x}{the name or index of the video to display; if NULL, a random video is chosen}

\item{list}{a logical indicating if the list of available videos should be displayed}
}
\description{
Genetic data analysis can be a harsh, tiring, daunting task.
Sometimes, a mere break will not cut it.
Sometimes, you need a kitten.
}
\details{
Please send us more! Either pull request or submit an issue with a URL (use
\code{adegenetIssues()}).
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AIC.snapclust.R
\name{AIC.snapclust}
\alias{AIC.snapclust}
\title{snapclust's Akaike Information Criterion (AIC)}
\usage{
\method{AIC}{snapclust}(object, ...)
}
\arguments{
\item{object}{An object returned by the function \code{\link{snapclust}}.}

\item{...}{Further arguments for compatibility with the \code{AIC} generic
(currently not used).}
}
\description{
This function computes Akaike Information Criterion (AIC) for
\code{snapclust} results.
}
\references{
Beugin M-P, Gayet T, Pontier D, Devillard S, Jombart T. A fast
    likelihood solution to the genetic clustering problem. Methods Ecol
    Evol. 2018;00:1–11. \url{https://doi.org/10.1111/2041-210X.12968}
}
\seealso{
\itemize{
 \item \code{\link{snapclust}}: to identify clusters

 \item \code{\link{snapclust.choose.k}}: to find the number of clusters

 \item \code{\link{AICc.snapclust}}: AICc computation

 \item \code{\link{BIC.snapclust}}: BIC computation

 \item \code{\link{KIC.snapclust}}: KIC computation
}
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{spcaIllus}
\alias{spcaIllus}
\title{Simulated data illustrating the sPCA}
\format{\code{spcaIllus} is list of 5 components being either genind or
genpop objects.}
\source{
Jombart, T., Devillard, S., Dufour, A.-B. and Pontier, D. Revealing
cryptic spatial patterns in genetic variability by a new multivariate
method. \emph{Heredity}, \bold{101}, 92--103.
}
\description{
Datasets illustrating the spatial Principal Component Analysis (Jombart et
al. 2009).  These data were simulated using various models using Easypop
(2.0.1).  Spatial coordinates were defined so that different spatial
patterns existed in the data. The \code{spca-illus} is a list containing the
following \linkS4class{genind} or \linkS4class{genpop} objects:\cr - dat2A:
2 patches \cr - dat2B: cline between two pop \cr - dat2C: repulsion among
individuals from the same gene pool \cr - dat3: cline and repulsion \cr -
dat4: patches and local alternance \cr
}
\details{
See "source" for a reference providing simulation details.
}
\examples{

data(spcaIllus)
attach(spcaIllus)
opar <- par(no.readonly=TRUE)
## comparison PCA vs sPCA

# PCA
pca2A <- dudi.pca(dat2A$tab,center=TRUE,scale=FALSE,scannf=FALSE)
pca2B <- dudi.pca(dat2B$tab,center=TRUE,scale=FALSE,scannf=FALSE)
pca2C <- dudi.pca(dat2C$tab,center=TRUE,scale=FALSE,scannf=FALSE)
pca3 <- dudi.pca(dat3$tab,center=TRUE,scale=FALSE,scannf=FALSE,nf=2)
pca4 <- dudi.pca(dat4$tab,center=TRUE,scale=FALSE,scannf=FALSE,nf=2)

# sPCA
spca2A <-spca(dat2A,xy=dat2A$other$xy,ask=FALSE,type=1,
plot=FALSE,scannf=FALSE,nfposi=1,nfnega=0)

spca2B <- spca(dat2B,xy=dat2B$other$xy,ask=FALSE,type=1,
plot=FALSE,scannf=FALSE,nfposi=1,nfnega=0)

spca2C <- spca(dat2C,xy=dat2C$other$xy,ask=FALSE,
type=1,plot=FALSE,scannf=FALSE,nfposi=0,nfnega=1)

spca3 <- spca(dat3,xy=dat3$other$xy,ask=FALSE,
type=1,plot=FALSE,scannf=FALSE,nfposi=1,nfnega=1)

spca4 <- spca(dat4,xy=dat4$other$xy,ask=FALSE,
type=1,plot=FALSE,scannf=FALSE,nfposi=1,nfnega=1)

# an auxiliary function for graphics
plotaux <- function(x,analysis,axis=1,lab=NULL,...){
neig <- NULL
if(inherits(analysis,"spca")) neig <- nb2neig(analysis$lw$neighbours)
xrange <- range(x$other$xy[,1])
xlim <- xrange + c(-diff(xrange)*.1 , diff(xrange)*.45)
yrange <- range(x$other$xy[,2])
ylim <- yrange + c(-diff(yrange)*.45 , diff(yrange)*.1)

s.value(x$other$xy,analysis$li[,axis],include.ori=FALSE,addaxes=FALSE,
cgrid=0,grid=FALSE,neig=neig,cleg=0,xlim=xlim,ylim=ylim,...)

par(mar=rep(.1,4))
if(is.null(lab)) lab = gsub("[P]","",x$pop)
text(x$other$xy, lab=lab, col="blue", cex=1.2, font=2)
add.scatter({barplot(analysis$eig,col="grey");box();
title("Eigenvalues",line=-1)},posi="bottomright",ratio=.3)
}

# plots
plotaux(dat2A,pca2A,sub="dat2A - PCA",pos="bottomleft",csub=2)
plotaux(dat2A,spca2A,sub="dat2A - sPCA glob1",pos="bottomleft",csub=2)

plotaux(dat2B,pca2B,sub="dat2B - PCA",pos="bottomleft",csub=2)
plotaux(dat2B,spca2B,sub="dat2B - sPCA glob1",pos="bottomleft",csub=2)

plotaux(dat2C,pca2C,sub="dat2C - PCA",pos="bottomleft",csub=2)
plotaux(dat2C,spca2C,sub="dat2C - sPCA loc1",pos="bottomleft",csub=2,axis=2)

par(mfrow=c(2,2))
plotaux(dat3,pca3,sub="dat3 - PCA axis1",pos="bottomleft",csub=2)
plotaux(dat3,spca3,sub="dat3 - sPCA glob1",pos="bottomleft",csub=2)
plotaux(dat3,pca3,sub="dat3 - PCA axis2",pos="bottomleft",csub=2,axis=2)
plotaux(dat3,spca3,sub="dat3 - sPCA loc1",pos="bottomleft",csub=2,axis=2)

plotaux(dat4,pca4,lab=dat4$other$sup.pop,sub="dat4 - PCA axis1",
pos="bottomleft",csub=2)
plotaux(dat4,spca4,lab=dat4$other$sup.pop,sub="dat4 - sPCA glob1",
pos="bottomleft",csub=2)
plotaux(dat4,pca4,lab=dat4$other$sup.pop,sub="dat4 - PCA axis2",
pos="bottomleft",csub=2,axis=2)
plotaux(dat4,spca4,lab=dat4$other$sup.pop,sub="dat4 - sPCA loc1",
pos="bottomleft",csub=2,axis=2)

# color plot
par(opar)
colorplot(spca3, cex=4, main="colorplot sPCA dat3")
text(spca3$xy[,1], spca3$xy[,2], dat3$pop)

colorplot(spca4, cex=4, main="colorplot sPCA dat4")
text(spca4$xy[,1], spca4$xy[,2], dat4$other$sup.pop)

# detach data
detach(spcaIllus)


}
\references{
Jombart, T., Devillard, S., Dufour, A.-B. and Pontier, D.
Revealing cryptic spatial patterns in genetic variability by a new
multivariate method. \emph{Heredity}, \bold{101}, 92--103.

Balloux F (2001) Easypop (version 1.7): a computer program for oppulation
genetics simulations \emph{Journal of Heredity}, \bold{92}: 301-302
}
\seealso{
\code{\link{spca}}
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{datasets}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/snapclust.choose.k.R
\name{snapclust.choose.k}
\alias{snapclust.choose.k}
\title{Choose the number of clusters for snapclust}
\usage{
snapclust.choose.k(max, ..., IC = AIC, IC.only = TRUE)
}
\arguments{
\item{max}{An integer indicating the maximum number of clusters to seek;
\code{\link{snapclust}} will be run for all k from 2 to max.}

\item{...}{Arguments passed to \code{\link{snapclust}}.}

\item{IC}{A function computing the information criterion for
\code{\link{snapclust}} objects. Available statistics are
\code{AIC} (default), \code{AICc}, and \code{BIC}.}

\item{IC.only}{A logical (TRUE by default) indicating if IC values only
should be returned; if \code{FALSE}, full \code{snapclust} objects are
returned.}
}
\description{
The function \code{snapclust.choose.k} can be used to identify optimal values
of 'k' (number of panmictic clusters) using \code{snapclust}. It runs the
method for increasing values of 'k' and for each computes goodness-of-fit
statistics. These statistics are all different versions of deviance penalised
for the number of parameters, so that lower values should correspond to more
optimal clustering solutions. Currently available statistics include AIC,
AICc, BIC, and KIC.\cr
}
\details{
There is no theoretical basis for favouring a given statistic over another
in the case of genetic clustering. In practice, one should look for an
'elbow' in the curve of the statistics as a function of 'k'.
}
\examples{
\dontrun{
 ## 'a' is a simulated dataset with 6 populations, island model
 data(dapcIllus)
 a <- dapcIllus$a
 a

 ## try and choose 'k' using AIC (real value = 6)
 a.aic <- snapclust.choose.k(max = 10, a)
 plot(1:10, a.aic, xlab = "Number of clusters (k)",
      ylab = "AIC", type = "b", pch = 20, cex = 3)

 ## try and choose 'k' using AIC (real value = 6)
 a.kic <- snapclust.choose.k(max = 10, a, IC = KIC)
 plot(1:10, a.kic, xlab = "Number of clusters (k)",
      ylab = "KIC", type = "b", pch = 20, cex = 3)


}
}
\seealso{
\itemize{
 \item \code{\link{snapclust}}: to identify clusters

 \item \code{\link{AIC.snapclust}}: AIC computation

 \item \code{\link{AICc.snapclust}}: AICc computation

 \item \code{\link{BIC.snapclust}}: BIC computation

 \item \code{\link{KIC.snapclust}}: KIC computation
}
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/handling.R
\docType{methods}
\name{tab}
\alias{tab}
\alias{tab,genind-method}
\alias{tab,genind-methods}
\alias{tab,genpop-method}
\alias{tab,genpop-methods}
\title{Access allele counts or frequencies}
\usage{
tab(x, ...)

\S4method{tab}{genind}(x, freq = FALSE, NA.method = c("asis", "mean",
  "zero"), ...)

\S4method{tab}{genpop}(x, freq = FALSE, NA.method = c("asis", "mean",
  "zero"), ...)
}
\arguments{
\item{x}{a \linkS4class{genind} or \linkS4class{genpop} object.}

\item{...}{further arguments passed to other methods.}

\item{freq}{a logical indicating if data should be transformed into relative frequencies (TRUE); defaults to FALSE.}

\item{NA.method}{a method to replace NA; asis: leave NAs as is; mean: replace by the mean allele frequencies; zero: replace by zero}
}
\value{
a matrix of integers or numeric
}
\description{
This accessor is used to retrieve a matrix of allele data.
By default, a matrix of integers representing allele counts is returned.
If \code{freq} is TRUE, then data are standardised as frequencies, so that for any individual and any locus the data sum to 1.
The argument \code{NA.method} allows to replace missing data (NAs).
This accessor replaces the previous function \code{truenames} as well as the function \code{makefreq}.
}
\examples{

data(microbov)
head(tab(microbov))
head(tab(microbov,freq=TRUE))


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{nancycats}
\alias{nancycats}
\title{Microsatellites genotypes of 237 cats from 17 colonies of Nancy (France)}
\format{\code{nancycats} is a genind object with spatial coordinates of the
colonies as a supplementary components (@xy).}
\source{
Dominique Pontier (UMR CNRS 5558, University Lyon1, France)
}
\description{
This data set gives the genotypes of 237 cats (\emph{Felis catus} L.) for 9
microsatellites markers. The individuals are divided into 17 colonies whose
spatial coordinates are also provided.
}
\examples{

\dontrun{
data(nancycats)
nancycats

# summary's results are stored in x
x <- summary(nancycats)

# some useful graphics
barplot(x$loc.n.all,ylab="Alleles numbers",main="Alleles numbers
per locus")

plot(x$pop.eff,x$pop.nall,type="n",xlab="Sample size",ylab="Number of alleles")
text(x$pop.eff,y=x$pop.nall,lab=names(x$pop.nall))

par(las=3)
barplot(table(nancycats@pop),ylab="Number of genotypes",main="Number of genotypes per colony")

# are cats structured among colonies ?
if(require(hierfstat)){

gtest <- gstat.randtest(nancycats,nsim=99)
gtest
plot(gtest)


dat <- genind2hierfstat(nancycats)

Fstat <- varcomp.glob(dat$pop,dat[,-1])
Fstat
}
}

}
\references{
Devillard, S.; Jombart, T. & Pontier, D. Disentangling spatial
and genetic structure of stray cat (\emph{Felis catus} L.) colonies in urban
habitat using: not all colonies are equal. submitted to \emph{Molecular
Ecology}
}
\keyword{datasets}

\encoding{UTF-8}
\docType{methods}
\name{selPopSize}
\alias{selPopSize}
\alias{selPopSize-methods}
\alias{selPopSize,ANY-method}
\alias{selPopSize,genind-method}
\title{ Select genotypes of well-represented populations}
\description{
  The function \code{selPopSize} checks the sample size of each population in
  a \linkS4class{genind} object and keeps only genotypes of populations
  having a given minimum size.
}
\usage{
\S4method{selPopSize}{genind}(x,pop=NULL,nMin=10)
}
\arguments{
  \item{x}{a \linkS4class{genind} object}
  \item{pop}{a vector of characters or a factor giving the population of
    each genotype in 'x'. If not provided, seeked from x\$pop.}
  \item{nMin}{the minimum sample size for a population to be
    retained. Samples sizes strictly less than \code{nMin} will be
    discarded, those equal to or greater than \code{nMin} are kept.}
}
\value{
  A  \linkS4class{genind} object.
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\seealso{\code{\link{seploc}}, \code{\link{repool}}}
\examples{
\dontrun{
data(microbov)

table(pop(microbov))
obj <- selPopSize(microbov, n=50)

obj
table(pop(obj))
}
}
\keyword{manip}

\name{propTyped-methods}
\docType{methods}
\alias{propTyped}
\alias{propTyped-methods}
\alias{propTyped,genind-method}
\alias{propTyped,genpop-method}
\title{ Compute the proportion of typed elements }
\description{
  The generic function \code{propTyped} is devoted to investigating the
  structure of missing data in adegenet objects.\cr

  Methods are defined for \linkS4class{genind} and \linkS4class{genpop}
  objects. They can return the proportion of available
  (i.e. non-missing) data per individual/population, locus, or the
  combination of both in with case the matrix indicates which entity
  (individual or population) was typed on which locus.
}
\usage{
\S4method{propTyped}{genind}(x,  by=c("ind","loc","both"))
\S4method{propTyped}{genpop}(x,  by=c("pop","loc","both"))
}
\arguments{
  \item{x}{a \linkS4class{genind} and \linkS4class{genpop} object}
  \item{by}{a character being "ind","loc", or "both" for
  \linkS4class{genind} object and "pop","loc", or "both" for
  \linkS4class{genpop} object. It specifies whether proportion of typed
  data are provided by entity ("ind"/"pop"), by locus ("loc") or both
  ("both"). See details.}
 }
 \value{
   A vector of proportion (when \code{by} equals "ind", "pop", or
  "loc"), or a matrix of binary data (when \code{by} equals "both")
}
\details{
  When \code{by} is set to "both", the result is a matrix of binary data
  with entities in rows (individuals or populations) and markers in
  columns. The values of the matrix are 1 for typed data, and 0 for NA.
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
\dontrun{
data(nancycats)
propTyped(nancycats,by="loc")
propTyped(genind2genpop(nancycats),by="both")
}
}
\keyword{methods}
\keyword{manip}

\encoding{UTF-8}
\name{Adegenet servers}
\alias{adegenetServer}
\title{ Web servers for adegenet }
\description{
  The function \code{adegenetServer} opens up a web page providing a
  simple user interface for some of the functionalities implemented in
  adegenet. These servers have been developed using the package
  \code{shiny}.\cr

  Currently available servers include:
  \itemize{
    \item \code{DAPC}: a server for the Discriminant Analysis of
    Principal Components (see ?dapc)
  }
}
\seealso{
  \link{dapc}
}
\usage{
adegenetServer(what=c("DAPC"))
}
\arguments{
  \item{what}{a character string indicating which server to start;
  currently accepted values are: "DAPC"}
}
\value{
 The function invisibly returns NULL.
}
\author{
  Thibaut Jombart \email{t.jombart@imperial.ac.uk}
  Caitlin Collins
}
\examples{
\dontrun{
## this opens a web page for DAPC
adegenetServer()
}
}

\encoding{UTF-8}
\name{glPca}
\alias{glPca}
\alias{print.glPca}
\alias{scatter.glPca}
\alias{loadingplot.glPca}
\title{Principal Component Analysis for genlight objects}
\description{
  These functions implement Principal Component Analysis (PCA) for
  massive SNP datasets stored as \linkS4class{genlight} object. This
  implementation has the advantage of never representing to complete data
  matrix, therefore making huge economies in terms of rapid access
  memory (RAM). When the \code{parallel} package is available,
  \code{glPca} uses multiple-core ressources for more efficient
  computations. \code{glPca} returns lists with the class \code{glPca}
  (see 'value'). 
  
  Other functions are defined for objects of this class:
  
  - \code{print}: prints the content of a \code{glPca} object.
  
  - \code{scatter}: produces scatterplots of principal components,
  with a screeplot of eigenvalues as inset.
    
  - \code{loadingplot}: plots the loadings of the analysis for one given
  axis, using an adapted version of the generic function \code{loadingplot}.
}
\usage{
glPca(x, center = TRUE, scale = FALSE, nf = NULL, loadings = TRUE, 
    alleleAsUnit = FALSE, useC = TRUE, parallel = FALSE,
  n.cores = NULL, returnDotProd=FALSE, matDotProd=NULL)

\method{print}{glPca}(x, \dots)

\method{scatter}{glPca}(x, xax = 1, yax = 2, posi = "bottomleft", bg = "white", 
    ratio = 0.3, label = rownames(x$scores), clabel = 1, xlim = NULL, 
    ylim = NULL, grid = TRUE, addaxes = TRUE, origin = c(0, 0), 
    include.origin = TRUE, sub = "", csub = 1, possub = "bottomleft", 
    cgrid = 1, pixmap = NULL, contour = NULL, area = NULL, \dots)

\method{loadingplot}{glPca}(x, at=NULL, threshold=NULL, axis=1,
    fac=NULL, byfac=FALSE, lab=rownames(x$loadings), cex.lab=0.7, cex.fac=1,
    lab.jitter=0, main="Loading plot", xlab="SNP positions",
    ylab="Contributions", srt = 90, adj = c(0, 0.5), \dots)

}
\arguments{
  \item{x}{for \code{glPca}, a \linkS4class{genlight} object; for
    \code{print}, \code{scatter}, and \code{loadingplot}, a \code{glPca}
  object.}
  \item{center}{a logical indicating whether the numbers of alleles
    should be centered; defaults to TRUE}
  \item{scale}{a logical indicating whether the numbers of alleles
    should be scaled; defaults to FALSE}
  \item{nf}{an integer indicating the number of principal components to
    be retained; if NULL, a screeplot of eigenvalues will be displayed
    and the user will be asked for a number of retained axes.}
  \item{loadings}{a logical indicating whether loadings of the alleles
    should be computed (TRUE, default), or not (FALSE). Vectors of
    loadings are not always useful, and can take a large amount of
    RAM when millions of SNPs are considered.}
  \item{alleleAsUnit}{a logical indicating whether alleles are
    considered as units (i.e., a diploid genotype equals two samples, a
    triploid, three, etc.) or whether individuals are considered as
    units of information.}
  \item{useC}{a logical indicating whether compiled C code should be
    used for faster computations; this option cannot be used alongside
    parallel option.}
  \item{parallel}{a logical indicating whether multiple cores -if
    available- should be used for the computations (TRUE, default), or
    not (FALSE); requires the package \code{parallel} to be installed
    (see details); this option cannot be used alongside useCoption.}
  \item{n.cores}{if \code{parallel} is TRUE, the number of cores to be
    used in the computations; if NULL, then the maximum number of cores
    available on the computer is used.}
  \item{returnDotProd}{a logical indicating whether the matrix of dot
    products between individuals should be returned (TRUE) or not
    (FALSE, default).}
  \item{matDotProd}{an optional matrix of dot products between
    individuals, NULL by default. This option is used internally to
    speed up computation time when re-running the same PCA several
    times. Leave this argument as NULL unless you really know what you
    are doing.}
  \item{\dots}{further arguments to be passed to other functions.}

  \item{xax,yax}{\code{integers} specifying which principal components
    should be shown in x and y axes. }
  \item{posi,bg,ratio}{arguments used to customize the inset in scatterplots
    of \code{glPca} results. See \code{\link[ade4]{add.scatter}} documentation in the
    ade4 package for more details.}
  \item{label,clabel,xlim,ylim,grid,addaxes,origin,include.origin,sub,csub,possub,cgrid,pixmap,contour,area}{
    arguments passed to \code{\link[ade4]{s.class}}; see \code{?s.label} for more information}
   \item{at}{an optional numeric vector giving the abscissa at which
    loadings are plotted. Useful when variates are SNPs with a known
    position in an alignement.}
  \item{threshold}{a threshold value above which values of x are
    identified. By default, this is the third quartile of x.}
  \item{axis}{an integer indicating the column of x to be plotted; used
    only if x is a matrix-like object.}
  \item{fac}{a factor defining groups of SNPs.}
  \item{byfac}{a logical stating whether loadings should be averaged by
    groups of SNPs, as defined by \code{fac}.}
  \item{lab}{a character vector giving the labels used to annotate
    values above the threshold.}
  \item{cex.lab}{a numeric value indicating the size of annotations.}
  \item{cex.fac}{a numeric value indicating the size of annotations for
    groups of observations.}
  \item{lab.jitter}{a numeric value indicating the factor of
    randomisation for the position of annotations. Set to 0 (by default)
  implies no randomisation.}
  \item{main}{the main title of the figure.}
  \item{xlab}{the title of the x axis.}
  \item{ylab}{the title of the y axis.}
  \item{srt}{rotation of the labels; see ?text.}
  \item{adj}{adjustment of the labels; see ?text.}
}
\details{
  === Using multiple cores ===
  
  Most recent machines have one or several processors with multiple
  cores. R processes usually use one single core. The package
  \code{parallel} allows for parallelizing some computations on
  multiple cores, which can decrease drastically computational time.

  Lastly, note that using compiled C code (\code{useC=TRUE})is an
  alternative for speeding up computations, but cannot be used together
  with the parallel option.
}
\value{
  === glPca objects ===
  
  The class \code{glPca} is a list with the following
  components:\cr
  \item{call}{the matched call.}
  \item{eig}{a numeric vector of eigenvalues.}
  \item{scores}{a matrix of principal components, containing the
    coordinates of each individual (in row) on each principal axis (in column).}
  \item{loadings}{(optional) a matrix of loadings, containing the loadings of each
    SNP (in row) for each principal axis (in column).}

  -
  
  === other outputs ===
  
  Other functions have different outputs:\cr
  - \code{scatter} return the matched call.\cr
  - \code{loadingplot} returns information about the most contributing
  SNPs (see \code{\link{loadingplot.default}})
}

\seealso{
  - \code{\linkS4class{genlight}}: class of object for storing massive binary
  SNP data.

  - \code{\link{glSim}}: a simple simulator for \linkS4class{genlight} objects.

  - \code{\link{glPlot}}: plotting \linkS4class{genlight} objects.

  - \code{\link{dapc}}: Discriminant Analysis of Principal Components.

}
\author{ Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
\dontrun{
## simulate a toy dataset
x <- glSim(50,4e3, 50, ploidy=2)
x
plot(x)

## perform PCA
pca1 <- glPca(x, nf=2)

## plot eigenvalues
barplot(pca1$eig, main="eigenvalues", col=heat.colors(length(pca1$eig)))

## basic plot
scatter(pca1, ratio=.2)

## plot showing groups
s.class(pca1$scores, pop(x), col=colors()[c(131,134)])
add.scatter.eig(pca1$eig,2,1,2)
}
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{sim2pop}
\alias{sim2pop}
\title{Simulated genotypes of two georeferenced populations}
\format{\code{sim2pop} is a genind object with a matrix of xy coordinates as
supplementary component.}
\source{
Easypop version 2.0.1 was run with the following parameters:\cr -
two diploid populations, one sex, random mating\cr - 1000 individuals per
population\cr - proportion of migration: 0.002\cr - 20 loci\cr - mutation
rate: 0.0001 (KAM model)\cr - maximum of 50 allelic states\cr - 1000
generations (last one taken)\cr
}
\description{
This simple data set was obtained by sampling two populations evolving in a
island model, simulated using Easypop (2.0.1). See \code{source} for
simulation details. Sample sizes were respectively 100 and 30 genotypes. The
genotypes were given spatial coordinates so that both populations were
spatially differentiated.
}
\examples{

\dontrun{
data(sim2pop)

if(require(hierfstat)){
## try and find the Fst
temp <- genind2hierfstat(sim2pop)
varcomp.glob(temp[,1],temp[,-1])
# Fst = 0.038
}

## run monmonier algorithm

# build connection network
gab <- chooseCN(sim2pop@other$xy,ask=FALSE,type=2)

# filter random noise
pca1 <- dudi.pca(sim2pop@tab,scale=FALSE, scannf=FALSE, nf=1)

# run the algorithm
mon1 <- monmonier(sim2pop@other$xy,dist(pca1$l1[,1]),gab, scanthres=FALSE)

# graphical display
temp <- sim2pop@pop
levels(temp) <- c(17,19)
temp <- as.numeric(as.character(temp))
plot(mon1)
points(sim2pop@other$xy,pch=temp,cex=2)
legend("topright",leg=c("Pop A", "Pop B"),pch=c(17,19))
}

}
\references{
Balloux F (2001) Easypop (version 1.7): a computer program for
oppulation genetics simulations \emph{Journal of Heredity}, \bold{92}:
301-302
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{datasets}

\name{as.SNPbin}
\alias{as,SNPbin,integer-method}
\alias{as,SNPbin,numeric-method}
\alias{as.SNPbin}
\alias{as.SNPbin,integer-method}
\alias{as.SNPbin,numeric-method}
\alias{coerce,integer,SNPbin-method}
\alias{coerce,numeric,SNPbin-method}

% \alias{,SNPbin-method}
% \alias{,SNPbin-method}
% \alias{,SNPbin-method}
% \alias{,SNPbin-method}
%%%%
\title{Conversion to class "SNPbin"}
\description{
  The class \linkS4class{SNPbin} is a formal (S4) class for storing a genotype
  of binary SNPs in a compact way, using a bit-level coding scheme. New
  instances of this class are best created using \code{new}; see the
  manpage of \linkS4class{SNPbin} for more information on this point.

  As a shortcut, conversion methods can be used to convert various
  objects into a \linkS4class{SNPbin} object. Conversions can be
  achieved using S3-style (\code{as.SNPbin(x)}) or S4-style
  (\code{as(x,"SNPbin"}) procedures. All of them call upon the
  constructor (\code{new}) of \linkS4class{SNPbin} objects.

  Conversion is currently available from the following objects:
  - integer vectors
  - numeric vectors
}
\author{Thibaut Jombart (\email{t.jombart@imperial.ac.uk})}
\seealso{
  Related class:\cr
  -  \code{\linkS4class{SNPbin}}
  -  \code{\linkS4class{genlight}}, for storing multiple binary SNP
  genotypes. \cr
}
\examples{
\dontrun{
## data to be converted
dat <- c(1,0,0,2,1,1,1,2,2,1,1,0,0,1)

## using the constructor
x1 <- new("SNPbin", dat)
x1

## using 'as' methods
x2 <- as.SNPbin(dat)
x3 <- as(dat, "SNPbin")

identical(x1,x2)
identical(x1,x3)
}
}
\keyword{classes}

\name{as.genlight}
\alias{as,genlight,matrix-method}
\alias{as,genlight,data.frame-method}
\alias{as,genlight,list-method}
% \alias{as,genlight,snp.matrix-method}
\alias{as.genlight}
\alias{as.genlight,matrix-method}
\alias{as.genlight,data.frame-method}
\alias{as.genlight,list-method}
% \alias{as.genlight,snp.matrix-method}
\alias{coerce,genlight,matrix-method}
\alias{coerce,genlight,data.frame-method}
\alias{coerce,genlight,list-method}
% \alias{coerce,genlight,snp.matrix-method}

% \alias{,genlight-method}
% \alias{,genlight-method}
% \alias{,genlight-method}
% \alias{,genlight-method}
%%%%
\title{Conversion to class "genlight"}
\description{
  The class \code{genlight} is a formal (S4) class for storing a genotypes
  of binary SNPs in a compact way, using a bit-level coding scheme. New
  instances of this class are best created using \code{new}; see the
  manpage of \linkS4class{genlight} for more information on this point.

  As a shortcut, conversion methods can be used to convert various
  objects into a \linkS4class{genlight} object. Conversions can be
  achieved using S3-style (\code{as.genlight(x)}) or S4-style
  (\code{as(x,"genlight"}) procedures. All of them call upon the
  constructor (\code{new}) of \linkS4class{genlight} objects.

  Conversion is currently available from the following objects:
  - matrix of type integer/numeric
  - data.frame with integer/numeric data
  - list of vectors of integer/numeric type
}
\author{Thibaut Jombart (\email{t.jombart@imperial.ac.uk})}
\seealso{
 Related class:\cr
  -  \code{\linkS4class{SNPbin}}, for storing individual genotypes of
  binary SNPs\cr
  
  -  \code{\linkS4class{genind}}
}
\examples{
\dontrun{
## data to be converted
dat <- list(toto=c(1,1,0,0,2,2,1,2,NA), titi=c(NA,1,1,0,1,1,1,0,0), tata=c(NA,0,3, NA,1,1,1,0,0))

## using the constructor
x1 <- new("genlight", dat)
x1

## using 'as' methods
x2 <- as.genlight(dat)
x3 <- as(dat, "genlight")

identical(x1,x2)
identical(x1,x3)
}

}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{hybridtoy}
\alias{hybridtoy}
\title{Toy hybrid dataset}
\format{a \linkS4class{genind} object}
\description{
Toy hybrid dataset
}
\examples{
data(hybridtoy)
x <- hybridtoy
pca1 <- dudi.pca(tab(x), scannf=FALSE, scale=FALSE)
s.class(pca1$li, pop(x))

if(require(ggplot2)) {
p <- ggplot(pca1$li, aes(x=Axis1)) +
    geom_density(aes(fill=pop(x)), alpha=.4, adjust=1) +
    geom_point(aes(y=0, color=pop(x)), pch="|", size=10, alpha=.5)
p
}

## kmeans
km <- find.clusters(x, n.pca=10, n.clust=2)
table(pop(x), km$grp)

## dapc
dapc1 <- dapc(x, pop=km$grp, n.pca=10, n.da=1)
scatter(dapc1)
scatter(dapc1, grp=pop(x))
compoplot(dapc1, col.pal=spectral, n.col=2)

## ML-EM with hybrids
res <- snapclust(x, k=2, hybrids=TRUE, detailed=TRUE)
compoplot(res, n.col=3)
table(res$group, pop(x))

}
\author{
Data simulated by  Marie-Pauline Beugin. Example by Thibaut Jombart.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{dapcIllus}
\alias{dapcIllus}
\title{Simulated data illustrating the DAPC}
\format{\code{dapcIllus} is list of 4 components being all genind objects.}
\source{
Jombart, T., Devillard, S. and Balloux, F.  Discriminant analysis of
principal components: a new method for the analysis of genetically
structured populations. Submitted to \emph{BMC genetics}.
}
\description{
Datasets illustrating the Discriminant Analysis of Principal Components
(DAPC, Jombart et al. submitted).\cr
}
\details{
These data were simulated using various models using Easypop (2.0.1).  The
\code{dapcIllus} is a list containing the following \linkS4class{genind}
objects:\cr - "a": island model with 6 populations \cr - "b": hierarchical
island model with 6 populations (3,2,1) \cr - "c": one-dimensional stepping
stone with 2x6 populations, and a boundary between the two sets of 6
populations\cr - "d": one-dimensional stepping stone with 24 populations\cr

See "source" for a reference providing simulation details.
}
\examples{

\dontrun{

data(dapcIllus)
attach(dapcIllus)
a # this is a genind object, like b, c, and d.


## FINS CLUSTERS EX NIHILO
clust.a <- find.clusters(a, n.pca=100, n.clust=6)
clust.b <- find.clusters(b, n.pca=100, n.clust=6)
clust.c <- find.clusters(c, n.pca=100, n.clust=12)
clust.d <- find.clusters(d, n.pca=100, n.clust=24)

## examin outputs
names(clust.a)
lapply(clust.a, head)


## PERFORM DAPCs
dapc.a <- dapc(a, pop=clust.a$grp, n.pca=100, n.da=5)
dapc.b <- dapc(b, pop=clust.b$grp, n.pca=100, n.da=5)
dapc.c <- dapc(c, pop=clust.c$grp, n.pca=100, n.da=11)
dapc.d <- dapc(d, pop=clust.d$grp, n.pca=100, n.da=23)


## LOOK AT ONE RESULT
dapc.a
summary(dapc.a)

## FORM A LIST OF RESULTS FOR THE 4 DATASETS
lres <- list(dapc.a, dapc.b, dapc.c, dapc.d)


## DRAW 4 SCATTERPLOTS
par(mfrow=c(2,2))
lapply(lres, scatter)


# detach data
detach(dapcIllus)
}

}
\references{
Jombart, T., Devillard, S. and Balloux, F.  Discriminant
analysis of principal components: a new method for the analysis of
genetically structured populations. Submitted to \emph{Genetics}.
}
\seealso{
- \code{\link{dapc}}: implements the DAPC.

- \code{\link{eHGDP}}: dataset illustrating the DAPC and
\code{find.clusters}.

- \code{\link{H3N2}}: dataset illustrating the DAPC.

- \code{\link{find.clusters}}: to identify clusters without prior.
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hs.R
\name{Hs}
\alias{Hs}
\title{Expected heterozygosity (Hs)}
\usage{
Hs(x, pop = NULL)
}
\arguments{
\item{x}{a \linkS4class{genind} or \linkS4class{genpop} object.}

\item{pop}{only used if x is a \linkS4class{genind}; an optional factor to be used as population; if not provided, pop(x) is used.}
}
\value{
a vector of Hs values (one value per population)
}
\description{
This function computes the expected heterozygosity (Hs) within
populations of a \linkS4class{genpop} object. This function is
available for codominant markers (\code{@type="codom"}) only. Hs is
commonly used for measuring within population genetic diversity (and
as such, it still has sense when computed from haploid data).
}
\details{
Let \emph{m(k)} be the number of alleles of locus \emph{k}, with a
total of \emph{K} loci. We note \eqn{f_i} the allele frequency of
allele \emph{i} in a given population. Then, \eqn{Hs} is given for a
given population by:\cr

\eqn{\frac{1}{K} \sum_{k=1}^K (1 - \sum_{i=1}^{m(k)} f_i^2)}
}
\examples{
\dontrun{
data(nancycats)
Hs(genind2genpop(nancycats))
}

}
\seealso{
\code{\link{Hs.test}} to test differences in Hs between two groups
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makefreq.R
\docType{methods}
\name{makefreq}
\alias{makefreq}
\alias{makefreq,genind-method}
\alias{makefreq,genind-methods}
\alias{makefreq,genpop-method}
\alias{makefreq,genpop-methods}
\title{Compute allelic frequencies}
\usage{
makefreq(x, ...)

\S4method{makefreq}{genind}(x, quiet = FALSE, missing = NA,
  truenames = TRUE, ...)

\S4method{makefreq}{genpop}(x, quiet = FALSE, missing = NA,
  truenames = TRUE, ...)
}
\arguments{
\item{x}{a \linkS4class{genind} or \linkS4class{genpop} object.}

\item{...}{further arguments (curently unused)}

\item{quiet}{logical stating whether a conversion message must be printed
(TRUE,default) or not (FALSE).}

\item{missing}{treatment for missing values. Can be NA, 0 or "mean" (see
details)}

\item{truenames}{deprecated; there for backward compatibility}
}
\value{
Returns a list with the following components: \item{tab}{matrix of
allelic frequencies (rows: populations; columns: alleles).}
\item{nobs}{number of observations (i.e. alleles) for each population x
locus combinaison.} \item{call}{the matched call}
}
\description{
The function \code{makefreq} is a generic to compute allele frequencies.
These can be derived for \linkS4class{genind} or \linkS4class{genpop} objects.
In the case of \linkS4class{genind} objects, data are kept at the individual level, but standardised so that allele frequencies sum up to 1.
}
\details{
There are 3 treatments for missing values: \cr - NA: kept as NA.\cr - 0:
missing values are considered as zero. Recommended for a PCA on
compositionnal data.\cr - "mean": missing values are given the mean
frequency of the corresponding allele. Recommended for a centred PCA.

Note that this function is now a simple wrapper for the accessor \code{\link{tab}}.
}
\examples{

\dontrun{
data(microbov)
obj1 <- microbov
obj2 <- genind2genpop(obj1)

# perform a correspondance analysis on counts data
Xcount <- tab(obj2, NA.method="zero")
ca1 <- dudi.coa(Xcount,scannf=FALSE)
s.label(ca1$li,sub="Correspondance Analysis",csub=1.2)
add.scatter.eig(ca1$eig,nf=2,xax=1,yax=2,posi="topleft")

# perform a principal component analysis on frequency data
Xfreq <- makefreq(obj2, missing="mean")
Xfreq <- tab(obj2, NA.method="mean") # equivalent to line above
pca1 <- dudi.pca(Xfreq,scale=FALSE,scannf=FALSE)
s.label(pca1$li,sub="Principal Component Analysis",csub=1.2)
add.scatter.eig(pca1$eig,nf=2,xax=1,yax=2,posi="top")
}

}
\seealso{
\code{\link{genpop}}
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{manip}
\keyword{multivariate}

\encoding{UTF-8}
\name{pairDistPlot}
\alias{pairDist}
\alias{pairDist.default}
\alias{pairDistPlot}
\alias{pairDistPlot.default}
\alias{pairDistPlot.dist}
\alias{pairDistPlot.matrix}
\alias{pairDistPlot.genind}
\alias{pairDistPlot.DNAbin}
\title{Pairwise distance plots}
\description{
  The function \code{pairDistPlot} extracts and plots pairwise distances
  between different groups (graphs use ggplot2). The function
  \code{pairDistPlot} does the same, without the graphs.
  
  \code{pairDistPlot} is a generic function with methods for the
 following types of objects:\cr
 - \code{dist} \cr
 - \code{matrix} (only numeric data)\cr
 - \code{\linkS4class{genind}} objects (genetic markers, individuals)\cr
 -  \code{\link[ape]{DNAbin}} objects (DNA sequences)
}
\usage{
pairDist(x, \dots)

pairDistPlot(x, \dots)

\method{pairDistPlot}{dist}(x, grp, within=FALSE, sep="-", data=TRUE,
             violin=TRUE, boxplot=TRUE, jitter=TRUE, \dots)

\method{pairDistPlot}{matrix}(x, grp, within=FALSE, sep="-", data=TRUE,
             violin=TRUE, boxplot=TRUE, jitter=TRUE, \dots)

\method{pairDistPlot}{genind}(x, grp, within=FALSE, sep="-", data=TRUE,
             violin=TRUE, boxplot=TRUE, jitter=TRUE, \dots)

\method{pairDistPlot}{DNAbin}(x, grp, within=FALSE, sep="-", data=TRUE,
             violin=TRUE, boxplot=TRUE, jitter=TRUE, \dots)

}
\arguments{
  \item{x}{pairwise distances provided as a \code{dist} or a symmetric
    \code{matrix}, or \code{\linkS4class{genind}} or \code{DNAbin}
    object. For \code{\linkS4class{genind}} objects, pairwise squared
    Euclidean distances are computed from the allele data. For
    \code{DNAbin} objects, distances are computed uing
    \code{\link{dist.dna}}, and '...' is used to pass arguments to the
    function.  }

  \item{grp}{a factor defining a grouping of individuals.}

  \item{within}{a logical indicating whether to keep within-group
    comparisons.}
  
  \item{sep}{a character used as separator between group names}

  \item{data}{a logical indicating whether data of the plot should be returned.}

  \item{violin}{a logical indicating whether a violinplot should be generated.}

  \item{boxplot}{a logical indicating whether a boxplot should be generated.}

  \item{jitter}{a logical indicating whether a jitter-plot should be generated.}

  \item{\dots}{further arguments to be used by other functions; used for
    \code{DNAbin} object to pass argumetns to \code{dist.dna}.}
}
\value{
  A list with different components, depending on the values of the
    arguments. Plots are returned as \code{ggplot2} objects.
}
\seealso{
  \code{\link{gengraph}} to identify connectivity based on distances.
}
\author{
  Thibaut Jombart \email{t.jombart@imperial.ac.uk}.
 }
\examples{
\dontrun{

## use a subset of influenza data
data(H3N2)
set.seed(1)
dat <- H3N2[sample(1:nInd(H3N2), 100)]

## get pairwise distances
temp <- pairDistPlot(dat, other(dat)$epid)

## see raw data
head(temp$data)

## see plots
temp$boxplot
temp$violin
temp$jitter

}
}

\name{virtualClasses}
\docType{class}
\alias{indInfo-class}
\alias{popInfo-class}
\alias{gen-class}
\alias{callOrNULL-class}
\alias{charOrNULL-class}
\alias{factorOrNULL-class}
\alias{intOrNum-class}
\alias{listOrNULL-class}
\alias{intOrNULL-class}
\alias{dfOrNULL-class}
\alias{formOrNULL-class}
\title{Virtual classes for adegenet}
\description{These virtual classes are only for internal use in adegenet}
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{H3N2}
\alias{H3N2}
\alias{usflu}
\alias{usflu.fasta}
\alias{USflu}
\alias{USflu.fasta}
\title{Seasonal influenza (H3N2) HA segment data}
\format{\code{H3N2} is a genind object with several data frame as
supplementary components (\code{H3N2@other) slort}, which contains the
following items: \describe{ \item{x}{a \code{data.frame} containing
miscellaneous annotations of the sequences.} \item{xy}{a matrix with two
columns indicating the geographic coordinates of the strains, as longitudes
and latitudes.} \item{epid}{a character vector indicating the epidemic of
the strains.} }}
\source{
This dataset was prepared by Thibaut Jombart
(t.jombart@imperia.ac.uk), from annotated sequences available on Genbank
(http://www.ncbi.nlm.nih.gov/Genbank/).
}
\description{
The dataset \code{H3N2} consists of 1903 strains of seasonal influenza
(H3N2) distributed worldwide, and typed at 125 SNPs located in the
hemagglutinin (HA) segment. It is stored as an R object with class
\linkS4class{genind} and can be accessed as usual using \code{data(H3N2)}
(see example). These data were gathered from DNA sequences available from
Genbank (http://www.ncbi.nlm.nih.gov/Genbank/).
}
\details{
The data file \code{usflu.fasta} is a toy dataset also gathered from
Genbank, consisting of the aligned sequences of 80 seasonal influenza
isolates (HA segment) sampled in the US, in \code{fasta} format. This file
is installed alongside the package; the path to this file is automatically
determined by R using \code{system.file} (see example in this manpage and in
?fasta2genlight) as well.
}
\examples{

\dontrun{
#### H3N2 ####
## LOAD DATA
data(H3N2)
H3N2

## set population to yearly epidemics
pop(H3N2) <- factor(H3N2$other$epid)



## PERFORM DAPC - USE POPULATIONS AS CLUSTERS
## to reproduce exactly analyses from the paper, use "n.pca=1000"
dapc1 <- dapc(H3N2, all.contrib=TRUE, scale=FALSE, n.pca=150, n.da=5)
dapc1

## (see ?dapc for details about the output)


## SCREEPLOT OF EIGENVALUES
barplot(dapc1$eig, main="H3N2 - DAPC eigenvalues")


## SCATTERPLOT (axes 1-2)
scatter(dapc1, posi.da="topleft", cstar=FALSE, cex=2, pch=17:22,
solid=.5, bg="white")




#### usflu.fasta ####
myPath <- system.file("files/usflu.fasta",package="adegenet")
myPath

## extract SNPs from alignments using fasta2genlight
## see ?fasta2genlight for more details
obj <- fasta2genlight(myPath, chunk=10) # process 10 sequences at a time
obj
}

}
\references{
Jombart, T., Devillard, S. and Balloux, F. Discriminant analysis
of principal components: a new method for the analysis of genetically
structured populations. Submitted to \emph{BMC genetics}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{extract.PLINKmap}
\alias{extract.PLINKmap}
\alias{read.PLINK}
\alias{read.plink}
\title{Reading PLINK Single Nucleotide Polymorphism data}
\usage{
extract.PLINKmap(file, x = NULL)

read.PLINK(file, map.file = NULL, quiet = FALSE, chunkSize = 1000,
  parallel = require("parallel"), n.cores = NULL, ...)
}
\arguments{
\item{file}{for \code{read.PLINK} a character string giving the path to the
file to convert, with the extension ".raw"; for \code{extract.PLINKmap}, a
character string giving the path to a file with extension ".map".}

\item{x}{an optional object of the class \code{"\linkS4class{genlight}"}, in which
the information read is stored; if provided, information is matched against
the names of the loci in \code{x}, as returned by \code{locNames(x)}; if not
provided, a list of two components is returned, containing chromosome and
position information.}

\item{map.file}{an optional character string indicating the path to a ".map"
file, which contains information about the SNPs (chromosome, position). If
provided, this information is processed by \code{extract.PLINKmap} and
stored in the \code{@other} slot.}

\item{quiet}{logical stating whether a conversion messages should be printed
(TRUE,default) or not (FALSE).}

\item{chunkSize}{an integer indicating the number of genomes to be read at a
time; larger values require more RAM but decrease the time needed to read
the data.}

\item{parallel}{a logical indicating whether multiple cores -if available-
should be used for the computations (TRUE, default), or not (FALSE);
requires the package \code{parallel} to be installed (see details).}

\item{n.cores}{if \code{parallel} is TRUE, the number of cores to be used in
the computations; if NULL, then the maximum number of cores available on the
computer is used.}

\item{\dots}{other arguments to be passed to other functions - currently not
used.}
}
\value{
- read.PLINK: an object of the class \code{"\linkS4class{genlight}"}

- extract.PLINKmap: if a \code{"\linkS4class{genlight}"} is provided as argument
\code{x}, this object incorporating the new information about SNPs in the
\code{@other} slot (with new components 'chromosome' and 'position');
otherwise, a list with two components containing chromosome and position
information.
}
\description{
The function \code{read.PLINK} reads a data file exported by the PLINK
software with extension '.raw' and converts it into a \code{"\linkS4class{genlight}"}
object. Optionally, information about SNPs can be read from a ".map" file,
either by specifying the argument \code{map.file} in \code{read.PLINK}, or
using \code{extract.PLINKmap} to add information to an existing
\code{"\linkS4class{genlight}"} object.
}
\details{
The function reads data by chunks of several genomes (minimum 1, no maximum)
at a time, which allows one to read massive datasets with negligible RAM
requirements (albeit at a cost of computational time). The argument
\code{chunkSize} indicates the number of genomes read at a time. Increasing
this value decreases the computational time required to read data in, while
increasing memory requirements.

See details for the documentation about how to export data using PLINK to
the '.raw' format.

=== Exporting data from PLINK ===

Data need to be exported from PLINK using the option "--recodeA" (and NOT
"--recodeAD"). The PLINK command should therefore look like: \code{plink
--file data --recodeA}. For more information on this topic, please look at
this webpage: \url{http://zzz.bwh.harvard.edu/plink/}
}
\seealso{
- \code{?genlight} for a description of the class
\code{"\linkS4class{genlight}"}.

- \code{\link{read.snp}}: read SNPs in adegenet's '.snp' format.

- \code{\link{fasta2genlight}}: extract SNPs from alignments with fasta
format.

- other import function in adegenet: \code{\link{import2genind}},
\code{\link{df2genind}}, \code{\link{read.genetix}}
\code{\link{read.fstat}}, \code{\link{read.structure}},
\code{\link{read.genepop}}.

- another function \code{read.plink} is available in the package
\code{snpMatrix}.
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{manip}

\encoding{UTF-8}
\name{find.clusters}
\alias{find.clusters}
\alias{find.clusters.data.frame}
\alias{find.clusters.matrix}
\alias{find.clusters.genind}
\alias{find.clusters.genlight}
\alias{.find.sub.clusters}
\title{find.cluster: cluster identification using successive K-means}

\description{

These functions implement the clustering procedure used in Discriminant Analysis
of Principal Components (DAPC, Jombart et al. 2010). This procedure consists in
running successive K-means with an increasing number of clusters (\code{k}),
after transforming data using a principal component analysis (PCA). For each
model, a statistical measure of goodness of fit (by default, BIC) is computed,
which allows to choose the optimal \code{k}. See \code{details} for a
description of how to select the optimal \code{k} and
\code{vignette("adegenet-dapc")} for a tutorial.

Optionally, hierarchical clustering can be sought by providing a prior
clustering of individuals (argument \code{clust}). In such case, clusters will
be sought within each prior group.

The K-means procedure used in \code{find.clusters} is
\code{\link[stats]{kmeans}} function from the \code{stats} package. The PCA
function is \code{\link[ade4]{dudi.pca}} from the \code{ade4} package, except
for \linkS4class{genlight} objects which use the \code{\link{glPca}} procedure
from adegenet.

\code{find.clusters} is a generic function with methods for the
 following types of objects:\cr
 \itemize{
 \item \code{data.frame} (only numeric data)\cr
 \item \code{matrix} (only numeric data)\cr
 \item \code{\linkS4class{genind}} objects (genetic markers)\cr
 \item \code{\linkS4class{genlight}} objects (genome-wide SNPs)
 }
}
\usage{

\method{find.clusters}{data.frame}(x, clust = NULL, n.pca = NULL, n.clust =
              NULL, method = c("kmeans", "ward"), stat = c("BIC","AIC", "WSS"),
              choose.n.clust = TRUE, criterion = c("diffNgroup", "min","goesup",
              "smoothNgoesup", "goodfit"), max.n.clust = round(nrow(x)/10),
              n.iter = 1e5, n.start = 10, center = TRUE, scale = TRUE,
              pca.select = c("nbEig","percVar"), perc.pca = NULL, \ldots, dudi =
              NULL)

\method{find.clusters}{matrix}(x, \ldots)

\method{find.clusters}{genind}(x, clust = NULL, n.pca = NULL, n.clust = NULL,
              method = c("kmeans", "ward"), stat = c("BIC","AIC", "WSS"),
              choose.n.clust = TRUE, criterion = c("diffNgroup", "min","goesup",
              "smoothNgoesup", "goodfit"), max.n.clust = round(nrow(x@tab)/10),
              n.iter = 1e5, n.start = 10, scale = FALSE, truenames = TRUE,
              \ldots)

\method{find.clusters}{genlight}(x, clust = NULL, n.pca = NULL, n.clust = NULL,
              method = c("kmeans", "ward"), stat = c("BIC", "AIC", "WSS"),
              choose.n.clust = TRUE, criterion = c("diffNgroup",
              "min","goesup","smoothNgoesup", "goodfit"), max.n.clust =
              round(nInd(x)/10), n.iter = 1e5,n.start = 10, scale = FALSE,
              pca.select = c("nbEig","percVar"), perc.pca = NULL,glPca=NULL,
              \ldots) }

\arguments{

  \item{x}{\code{a data.frame}, \code{matrix}, or \code{\linkS4class{genind}}
    object. For the \code{data.frame} and \code{matrix} arguments, only
    quantitative variables should be provided.}

\item{clust}{an optional \code{factor} indicating a prior group membership of
    individuals. If provided, sub-clusters will be sought within each prior
    group.}

\item{n.pca}{an \code{integer} indicating the number of axes retained in the
    Principal Component Analysis (PCA) step. If \code{NULL}, interactive
    selection is triggered.}

\item{n.clust}{ an optinal \code{integer} indicating the number of clusters to
    be sought. If provided, the function will only run K-means once, for this
    number of clusters. If left as \code{NULL}, several K-means are run for a
    range of k (number of clusters) values.}
    
  \item{method}{a \code{character} string indicating the type of clustering
  method to be used; "kmeans" (default) uses K-means clustering, and is the
  original implementation of \code{find.clusters}; "ward" is an alternative
  which uses Ward's hierarchical clustering; this latter method seems to be more
  reliable on some simulated datasets, but will be less computer-efficient for
  large numbers (thousands) of individuals.}
  
  \item{stat}{ a \code{character} string matching 'BIC', 'AIC', or 'WSS', which
    indicates the statistic to be computed for each model (i.e., for each value
    of \code{k}). BIC: Bayesian Information Criterion. AIC: Aikaike's
    Information Criterion. WSS: within-groups sum of squares, that is, residual
    variance.}

\item{choose.n.clust}{ a \code{logical} indicating whether the number of
    clusters should be chosen by the user (TRUE, default), or automatically,
    based on a given criterion (argument \code{criterion}). It is HIGHLY
    RECOMMENDED to choose the number of clusters INTERACTIVELY, since i) the
    decrease of the summary statistics (BIC by default) is informative, and ii)
    no criteria for automatic selection is appropriate to all cases (see
    details).}

\item{criterion}{ a \code{character} string matching "diffNgroup",
    "min","goesup", "smoothNgoesup", or "goodfit", indicating the criterion for
    automatic selection of the optimal number of clusters. See \code{details}
    for an explanation of these procedures.}

\item{max.n.clust}{ an \code{integer} indicating the maximum number of clusters
    to be tried. Values of 'k' will be picked up between 1 and
    \code{max.n.clust}}

\item{n.iter}{ an \code{integer} indicating the number of iterations to be used
    in each run of K-means algorithm. Corresponds to \code{iter.max} of
    \code{kmeans} function.}

\item{n.start}{ an \code{integer} indicating the number of randomly chosen
    starting centroids to be used in each run of the K-means algorithm. Using
    more starting points ensures convergence of the algorithm. Corresponds to
    \code{nstart} of \code{kmeans} function.}

\item{center}{a \code{logical} indicating whether variables should be centred to
    mean 0 (TRUE, default) or not (FALSE). Always TRUE for \linkS4class{genind}
    objects.}

\item{scale}{a \code{logical} indicating whether variables should be scaled
    (TRUE) or not (FALSE, default). Scaling consists in dividing variables by
    their (estimated) standard deviation to account for trivial differences in
    variances. In allele frequencies, it comes with the risk of giving
    uninformative alleles more importance while downweighting informative
    alleles. Further scaling options are available for \linkS4class{genind}
    objects (see argument \code{scale.method}).}

\item{pca.select}{a \code{character} indicating the mode of selection of PCA
    axes, matching either "nbEig" or "percVar". For "nbEig", the user has to
    specify the number of axes retained (interactively, or via
    \code{n.pca}). For "percVar", the user has to specify the minimum amount of
    the total variance to be preserved by the retained axes, expressed as a
    percentage (interactively, or via \code{perc.pca}).  }

\item{perc.pca}{a \code{numeric} value between 0 and 100 indicating the minimal
    percentage of the total variance of the data to be expressed by the retained
    axes of PCA.}

\item{truenames}{a \code{logical} indicating whether true (i.e., user-specified)
    labels should be used in object outputs (TRUE, default) or not (FALSE), in
    which case generic labels are used.}

\item{\ldots}{further arguments to be passed to other functions. For
    \code{find.clusters.matrix}, arguments are to match those of the
    \code{data.frame} method.}

\item{dudi}{optionally, a multivariate analysis with the class \code{dudi} (from
    the ade4 package). If provided, prior PCA will be ignored, and this object
    will be used as a prior step for variable orthogonalisation.}

\item{glPca}{an optional \code{\link{glPca}} object; if provided, dimension
    reduction is not performed (saving computational time) but taken directly
    from this object.}
    
}
\details{
  === ON THE SELECTION OF K ===\cr
  (where K is the 'optimal' number of clusters)

  So far, the analysis of data simulated under various population
  genetics models (see reference) suggested an ad hoc rule for the
  selection of the optimal number of clusters. First important result is
  that BIC seems for efficient than AIC and WSS to select the
  appropriate number of clusters (see example). The rule of thumb
  consists in increasing K until it no longer leads to an appreciable
  improvement of fit (i.e., to a decrease of BIC).  In the most simple
  models (island models), BIC decreases until it reaches the optimal K,
  and then increases. In these cases, our rule amounts to choosing the
  lowest K. In other models such as stepping stones, the decrease of BIC
  often continues after the optimal K, but is much less steep.


  An alternative approach is the automatic selection based on a fixed
  criterion. Note that, in any case, it is highly recommended to look at
  the graph of the BIC for different numbers of clusters as displayed
  during the interactive cluster selection.
  To use automated selection, set \code{choose.n.clust} to FALSE and specify
  the \code{criterion} you want to use, from the following values:

  - "diffNgroup": differences between successive values of the summary
  statistics (by default, BIC) are splitted into two groups using a
  Ward's clustering method (see \code{?hclust}), to differentiate sharp
  decrease from mild decreases or increases. The retained K is the one
  before the first group switch. Appears to work well for
  island/hierarchical models, and decently for isolation by distance
  models, albeit with some unstability. Can be impacted by an initial,
  very sharp decrease of the test statistics. IF UNSURE ABOUT THE
  CRITERION TO USE, USE THIS ONE.

  - "min": the model with the minimum summary statistics (as specified
  by \code{stat} argument, BIC by default) is retained. Is likely to
  work for simple island model, using BIC. It is likely to fail in
  models relating to stepping stones, where the BIC always decreases
  (albeit by a small amount) as K increases. In general, this approach
  tends to over-estimate the number of clusters.

  - "goesup": the selected model is the K after which increasing the
  number of clusters leads to increasing the summary statistics. Suffers
  from inaccuracy, since i) a steep decrease might follow a small 'bump'
  of increase of the statistics, and ii) increase might never happen, or
  happen after negligible decreases. Is likely to work only for
  clear-cut island models.

  - "smoothNgoesup": a variant of "goesup", in which the summary
  statistics is first smoothed using a lowess approach. Is meant to be
  more accurate than "goesup" as it is less prone to stopping to small
  'bumps' in the decrease of the statistics.

  - "goodfit": another criterion seeking a good fit with a minimum
  number of clusters. This approach does not rely on differences between
  successive statistics, but on absolute fit. It selects the model with
  the smallest K so that the overall fit is above a given threshold.
}
\value{
  The class \code{find.clusters} is a list with the following
  components:\cr
  \item{Kstat}{a \code{numeric} vector giving the values of the summary
  statistics for the different values of K. Is NULL if \code{n.clust} was specified.}
  \item{stat}{a \code{numeric} value giving the value of the summary statistics
  for the retained model}
  \item{grp}{a \code{factor} giving group membership for each individual.}
  \item{size}{an \code{integer} vector giving the size of the different clusters.}
}
\references{
  Jombart T, Devillard S and Balloux F (2010) Discriminant analysis of
  principal components: a new method for the analysis of genetically
  structured populations. BMC Genetics 11:94. doi:10.1186/1471-2156-11-94
}
\seealso{
  - \code{\link{dapc}}: implements the DAPC.

  - \code{\link{scatter.dapc}}: graphics for DAPC.

  - \code{\link{dapcIllus}}: dataset illustrating the DAPC and \code{find.clusters}.

  - \code{\link{eHGDP}}: dataset illustrating the DAPC and \code{find.clusters}.

  - \code{\link[stats]{kmeans}}: implementation of K-means in the stat package.

  - \code{\link[ade4]{dudi.pca}}: implementation of PCA in the ade4 package.

}
\author{ Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
\dontrun{
## THIS ONE TAKES A FEW MINUTES TO RUN ## 
data(eHGDP)

## here, n.clust is specified, so that only on K value is used
grp <- find.clusters(eHGDP, max.n=30, n.pca=200, scale=FALSE,
n.clust=4) # takes about 2 minutes
names(grp)
grp$Kstat
grp$stat


## to try different values of k (interactive)
grp <- find.clusters(eHGDP, max.n=50, n.pca=200, scale=FALSE)

## and then, to plot BIC values:
plot(grp$Kstat, type="b", col="blue")



## ANOTHER SIMPLE EXAMPLE ## 
data(sim2pop) # this actually contains 2 pop

## DETECTION WITH BIC (clear result)
foo.BIC <- find.clusters(sim2pop, n.pca=100, choose=FALSE)
plot(foo.BIC$Kstat, type="o", xlab="number of clusters (K)", ylab="BIC",
col="blue", main="Detection based on BIC")
points(2, foo.BIC$Kstat[2], pch="x", cex=3)
mtext(3, tex="'X' indicates the actual number of clusters")


## DETECTION WITH AIC (less clear-cut)
foo.AIC <- find.clusters(sim2pop, n.pca=100, choose=FALSE, stat="AIC")
plot(foo.AIC$Kstat, type="o", xlab="number of clusters (K)",
ylab="AIC", col="purple", main="Detection based on AIC")
points(2, foo.AIC$Kstat[2], pch="x", cex=3)
mtext(3, tex="'X' indicates the actual number of clusters")


## DETECTION WITH WSS (less clear-cut)
foo.WSS <- find.clusters(sim2pop, n.pca=100, choose=FALSE, stat="WSS")
plot(foo.WSS$Kstat, type="o", xlab="number of clusters (K)", ylab="WSS
(residual variance)", col="red", main="Detection based on WSS")
points(2, foo.WSS$Kstat[2], pch="x", cex=3)
mtext(3, tex="'X' indicates the actual number of clusters")


## TOY EXAMPLE FOR GENLIGHT OBJECTS ##
x <- glSim(100,500,500)
x
plot(x)
grp <- find.clusters(x, n.pca = 100, choose = FALSE, stat = "BIC")
plot(grp$Kstat, type = "o", xlab = "number of clusters (K)",
     ylab = "BIC",
     main = "find.clusters on a genlight object\n(two groups)")
}
}
\keyword{multivariate}

\name{colorplot}
\alias{colorplot}
\alias{colorplot.default}
\title{Represents a cloud of points with colors}
\description{
  The \code{colorplot} function represents a cloud of points with colors
  corresponding to a combination of 1,2 or 3 quantitative variables,
  assigned to RGB (Red, Green, Blue) channels. For instance, this can be useful to
  represent up to 3 principal components in space. Note that the
  property of such representation to convey multidimensional information
  has not been investigated.\cr

  \code{colorplot} is a S3 generic function. Methods are defined for
  particular objects, like \code{\link{spca}} objects.

}
\usage{
colorplot(\dots)

\method{colorplot}{default}(xy, X, axes=NULL, add.plot=FALSE, defaultLevel=0, transp=FALSE, alpha=.5, \dots)
}
\arguments{
  \item{xy}{a numeric matrix with two columns (e.g. a matrix of spatial coordinates.}
  \item{X}{a matrix-like containing numeric values that are translated
    into the RGB system. Variables are considered to be in columns.}
  \item{axes}{the index of the columns of X to be represented. Up to
    three axes can be chosen. If null, up to the first three columns of
    X are used.}
  \item{add.plot}{a logical stating whether the colorplot should be
    added to the existing plot (defaults to FALSE).}
  \item{defaultLevel}{a numeric value between 0 and 1, giving the
    default level in a color for which values are not specified. Used
    whenever less than three axes are specified.}
  \item{transp}{a logical stating whether the produced colors should be
    transparent (TRUE) or not (FALSE, default).}
  \item{alpha}{the alpha level for transparency, between 0 (fully
    transparent) and 1 (not transparent); see \code{?rgb} for more details.}
  \item{\dots}{further arguments to be passed to other methods. In
    \code{colorplot.default}, these arguments are passed to plot/points
    functions. See \code{?plot.default} and \code{?points}.}
}
\value{
  Invisibly returns a vector of colours used in the plot.
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk}}
\examples{
# a toy example
xy <- expand.grid(1:10,1:10)
df <- data.frame(x=1:100, y=100:1, z=runif(100,0,100))
colorplot(xy,df,cex=10,main="colorplot: toy example")

\dontrun{
# a genetic example using a sPCA
if(require(spdep)){
data(spcaIllus)
dat3 <- spcaIllus$dat3
spca3 <- spca(dat3,xy=dat3$other$xy,ask=FALSE,type=1,plot=FALSE,scannf=FALSE,nfposi=1,nfnega=1)
colorplot(spca3, cex=4, main="colorplot: a sPCA example")
text(spca3$xy[,1], spca3$xy[,2], dat3$pop)
mtext("P1-P2 in cline\tP3 random \tP4 local repulsion")
}
}
}
\keyword{multivariate}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{import2genind}
\alias{import2genind}
\title{Importing data from several softwares to a genind object}
\usage{
import2genind(file, quiet = FALSE, ...)
}
\arguments{
\item{file}{a character string giving the path to the file to convert, with
the appropriate extension.}

\item{quiet}{logical stating whether a conversion message must be printed
(TRUE,default) or not (FALSE).}

\item{\dots}{other arguments passed to the appropriate 'read' function
(currently passed to \code{read.structure})}
}
\value{
an object of the class \code{genind}
}
\description{
Their are several ways to import genotype data to a \linkS4class{genind}
object: i) from a data.frame with a given format (see
\code{\link{df2genind}}), ii) from a file with a recognized extension, or
iii) from an alignement of sequences (see \code{\link{DNAbin2genind}}).\cr
}
\details{
The function \code{import2genind} detects the extension of the file given in
argument and seeks for an appropriate import function to create a
\code{genind} object.\cr Current recognized formats are :\cr - GENETIX files
(.gtx) \cr - Genepop files (.gen) \cr - Fstat files (.dat) \cr - STRUCTURE
files (.str or .stru) \cr

Beware: same data in different formats are not expected to produce exactly
the same \code{genind} objects.\cr For instance, conversions made by GENETIX
to Fstat may change the the sorting of the genotypes; GENETIX stores
individual names whereas Fstat does not; Genepop chooses a sample's name
from the name of its last genotype; etc.
}
\examples{

import2genind(system.file("files/nancycats.gtx",
package="adegenet"))

import2genind(system.file("files/nancycats.dat",
package="adegenet"))

import2genind(system.file("files/nancycats.gen",
package="adegenet"))

import2genind(system.file("files/nancycats.str",
package="adegenet"), onerowperind=FALSE, n.ind=237, n.loc=9, col.lab=1, col.pop=2, ask=FALSE)

}
\references{
Belkhir K., Borsa P., Chikhi L., Raufaste N. & Bonhomme F.
(1996-2004) GENETIX 4.05, logiciel sous Windows TM pour la genetique des
populations. Laboratoire Genome, Populations, Interactions, CNRS UMR 5000,
Universite de Montpellier II, Montpellier (France). \cr

Pritchard, J.; Stephens, M. & Donnelly, P. (2000) Inference of population
structure using multilocus genotype data. \emph{Genetics}, \bold{155}:
945-959

Raymond M. & Rousset F, (1995). GENEPOP (version 1.2): population genetics
software for exact tests and ecumenicism. \emph{J. Heredity},
\bold{86}:248-249 \cr

Fstat (version 2.9.3). Software by Jerome Goudet.
http://www2.unil.ch/popgen/softwares/fstat.htm\cr

Excoffier L. & Heckel G.(2006) Computer programs for population genetics
data analysis: a survival guide \emph{Nature}, \bold{7}: 745-758
}
\seealso{
\code{\link{import2genind}}, \code{\link{read.genetix}},
\code{\link{read.fstat}}, \code{\link{read.structure}},
\code{\link{read.genepop}}
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{manip}

\encoding{UTF-8}
\name{snpposi}

\alias{snpposi.plot}
\alias{snpposi.plot.integer}
\alias{snpposi.plot.numeric}
\alias{snpposi.plot.DNAbin}

\alias{snpposi.test}
\alias{snpposi.test.integer}
\alias{snpposi.test.numeric}
\alias{snpposi.test.DNAbin}

\title{Analyse the position of polymorphic sites}
\description{
  These functions are used to describe the distribution of polymorphic
  sites (SNPs) in an alignment.

  The function \code{snpposi.plot} plots the positions and density of
  SNPs in the alignment.

  The function \code{snpposi.test} tests whether SNPs are randomly
  distributed in the genome, the alternative hypothesis being that they
  are clustered. This test is based on the distances of each SNP to the
  closest SNP. This provides one measure of clustering for each SNP.
  Different statistics can be used to summarise these values (argument
  \code{stat}), but by default the statistics used is the median.

  \code{snpposi.plot} and \code{snpposi.test} are generic functions with
  methods for vectors of integers or numeric (indicating SNP position),
  and for \code{\link[ape]{DNAbin}} objects.
}
\usage{
snpposi.plot(\dots)

\method{snpposi.plot}{integer}(x, genome.size, smooth=0.1,
              col="royalblue", alpha=.2, codon=TRUE, start.at=1, \dots)

\method{snpposi.plot}{numeric}(x, \dots)

\method{snpposi.plot}{DNAbin}(x, \dots)


snpposi.test(\dots)

\method{snpposi.test}{integer}(x, genome.size, n.sim=999, stat=median, \dots)

\method{snpposi.test}{numeric}(x, \dots)

\method{snpposi.test}{DNAbin}(x, \dots)



}
\arguments{
  \item{x}{a vector of integers or numerics containing SNP positions, or
    a set of aligned sequences in a \code{DNAbin} object.}

  \item{genome.size}{an integer indicating the length of genomes.}

  \item{smooth}{a smoothing parameter for the density estimation;
    smaller values will give more local peaks; values have to be
    positive but can be less than 1.}

  \item{col}{the color to be used for the plot; ignored if codon
    positions are represented.}

  \item{alpha}{the alpha level to be used for transparency (density curve).}

  \item{codon}{a logical indicating if codon position should be
    indicated (TRUE, default) or not.}

  \item{start.at}{an integer indicating at which base of a codon the
    alignment starts (defaults to 1); values other than 1, 2 and 3 will
    be ignored.}

  \item{n.sim}{an integer indicating the number of randomizations to be
    used in the Monte Carlo test.}

  \item{stat}{a function used to summarize the measure of physical
    proximity between SNPs; by default, the median is used.}
    \item{\dots}{further arguments to be passed to the \code{integer}
    method.}
}
\value{
  A Monte Carlo test of the class \code{randtest}.
}
\seealso{
  The \code{\link{fasta2DNAbin}} to read fasta alignments with minimum
  RAM use.
}
\author{
  Thibaut Jombart \email{t.jombart@imperial.ac.uk}.
}
\examples{
if(require(ape)){
data(woodmouse)
snpposi.plot(woodmouse, codon=FALSE)
snpposi.plot(woodmouse)

\dontrun{
snpposi.test(c(1,3,4,5), 100)
snpposi.test(woodmouse)
}
}
}

\encoding{UTF-8}
\name{monmonier}
\alias{monmonier}
\alias{optimize.monmonier}
\alias{plot.monmonier}
\alias{print.monmonier}

\title{Boundary detection using Monmonier algorithm}
\description{
  The Monmonier's algorithm detects boundaries among vertices of a
  valuated graph. This is achieved by finding the path
  exhibiting the largest distances between connected vertices.\cr

  The highest distance between two connected vertices (i.e. neighbours) is
  found, giving the starting point of the path. Then, the algorithm
  seeks the highest distance between immediate neighbours, and so on
  until a threshold value is attained.
  This threshold can be chosen from the plot of sorted distances between
  connected vertices: a boundary will likely result in an abrupt decrease
  of these values.\cr
 
  When several paths are looked for, the previous paths are taken into
  account, and cannot be either crossed or redrawn. Monmonier's
  algorithm can be used to assess the boundaries between patches of
  homogeneous observations.\cr
    
  Although Monmonier algorithm was initially designed for Voronoi
  tesselation, this implementation generalizes this algorithm to different
  connection networks. The \code{optimize.monmonier} function produces a
  \code{monmonier} object by trying several starting points, and
  returning the best boundary (i.e. largest sum of local distances). This
  is designed to avoid the algorithm to be trapped by a single strong
  local difference inside an homogeneous patch.
}

\usage{
monmonier(xy, dist, cn, threshold=NULL, bd.length=NULL, nrun=1,
skip.local.diff=rep(0,nrun),scanthres=is.null(threshold), allowLoop=TRUE)

optimize.monmonier(xy, dist, cn, ntry=10, bd.length=NULL, return.best=TRUE,
display.graph=TRUE, threshold=NULL, scanthres=is.null(threshold), allowLoop=TRUE)

\method{plot}{monmonier}(x, variable=NULL,
displayed.runs=1:x$nrun, add.arrows=TRUE,
col='blue', lty=1, bwd=4, clegend=1, csize=0.7,
method=c('squaresize','greylevel'), sub='', csub=1, possub='topleft',
cneig=1, pixmap=NULL, contour=NULL, area=NULL, add.plot=FALSE, \dots)

\method{print}{monmonier}(x, \dots)
}
\arguments{
  \item{xy}{a matrix yielding the spatial coordinates of the objects, with two columns respectively giving X and Y}
  \item{dist}{an object of class \code{dist}, giving the distances between the objects}
  \item{cn}{a connection network of class \code{nb} (package \code{spdep})}
  \item{threshold}{a number giving the minimal distance between two
    neighbours crossed by the path; by default, this is the third quartile of all the distances between neighbours}
  \item{bd.length}{an optional integer giving the requested length of
    the boundaries (in number of local differences)}
  \item{nrun}{is a integer giving the number of runs of the algorithm, that is, the number of paths to search, being one by default}
  \item{skip.local.diff}{is a vector of integers, whose length is the number of paths (\code{nrun}); each integer gives the number of starting point to skip, to avoid being stuck in a local difference between two neighbours into an homogeneous patch; none are skipped by default}
  \item{scanthres}{a logical stating whether the threshold sould be
    chosen from the barplot of sorted distances between neighbours}
  \item{allowLoop}{a logical specifying whether the boundary can loop
    (TRUE, default) or not (FALSE)}
  \item{ntry}{an integer giving the number of different starting points
    tried.}
  \item{return.best}{a logical stating whether the best monmonier object
    should be returned (TRUE, default) or not (FALSE)}
   \item{display.graph}{a logical whether the scores of each try should
    be plotted (TRUE, default) or not}
  \item{x}{a monmonier object}
  \item{variable}{a variable to be plotted using \code{s.value} (package
  \code{ade4})}
  \item{displayed.runs}{an integer vector giving the rank of the paths to represent}
  \item{add.arrows}{a logical, stating whether arrows should indicate the direction of the path (TRUE) or not (FALSE, used by default)}
  \item{col}{a characters vector giving the colors to be used for each boundary; recycled is needed; 'blue' is used by default}
  \item{lty}{a characters vector giving the type of line to be used for each boundary; 1 is used by default}
  \item{bwd}{a number giving the boundary width factor, applying to every segments of the paths; 4 is used by default}
  \item{clegend}{like in \code{s.value}, the size factor of the legend if a variable is represented}
  \item{csize}{like in \code{s.value}, the size factor of the squares used to represent a variable}
  \item{method}{like in \code{s.value}, a character giving the method to be used to represent the variable, either 'squaresize' (by default) or 'greylevel'}
  \item{sub}{a string of characters giving the subtitle of the plot}
  \item{csub}{the size factor of the subtitle}
  \item{possub}{the position of the subtitle; available choices are 'topleft' (by default), 'topright', 'bottomleft', and 'bottomright'}
  \item{cneig}{the size factor of the connection network}
  \item{pixmap}{an object of the class \code{pixmap} displayed in the map background}
  \item{contour}{ a data frame with 4 columns to plot the contour of the map: each row gives a segment (x1,y1,x2,y2)}
  \item{area}{a data frame of class 'area' to plot a set of surface units in contour}
  \item{add.plot}{a logical stating whether the plot should be added to the current one (TRUE), or displayed in a new window (FALSE, by default)}
  \item{\dots}{further arguments passed to other methods}
}
\details{
The function \code{monmonier} returns a list of the class \code{monmonier}, which contains the general informations about the algorithm, and about each run.
When displayed, the width of the boundaries reflects their 'strength'.
Let a segment MN be part of the path, M being the middle of AB, N of CD.
Then the boundary width for MN is proportionnal to (d(AB)+d(CD))/2.\cr

As there is no perfect method to display graphically a quantitative
variable (see for instance the differences between the two methods of
\code{s.value}), the boundaries provided by this algorithm seem
sometimes more reliable than the boundaries our eyes perceive (or miss).\cr

}

\value{
Returns an object of class \code{monmonier}, which contains the following elements :
	\item{run1 (run2, \dots)}{for each run, a list containing a dataframe giving the path coordinates, and a vector of the distances between neighbours of the path}
	\item{nrun}{the number of runs performed, i.e. the number of boundaries in the monmonier object}
	\item{threshold}{the threshold value, minimal distance between
	  neighbours accounted for by the algorithm}
	\item{xy}{the matrix of spatial coordinates}
	\item{cn}{the connection network of class \code{nb}}
	\item{call}{the call of the function}
}
\references{
Monmonier, M. (1973) Maximum-difference barriers: an alternative numerical regionalization method. \emph{Geographic Analysis}, \bold{3}, 245--261.

Manni, F., Guerard, E. and Heyer, E. (2004) Geographic patterns of (genetic, morphologic, linguistic) variation: how barriers can be detected by "Monmonier's algorithm". \emph{Human Biology}, \bold{76}, 173--190
}

\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk}}

\seealso{\code{\link{spca}},\code{\link[spdep]{edit.nb}}}

\examples{
if(require(spdep)){

### non-interactive example

# est-west separation
load(system.file("files/mondata1.rda",package="adegenet"))
cn1 <- chooseCN(mondata1$xy,type=2,ask=FALSE)
mon1 <- monmonier(mondata1$xy,dist(mondata1$x1),cn1,threshold=2)
plot(mon1,mondata1$x1)
plot(mon1,mondata1$x1,met="greylevel",add.arr=FALSE,col="red",bwd=6,lty=2)

# square in the middle
load(system.file("files/mondata2.rda",package="adegenet"))
cn2 <- chooseCN(mondata2$xy,type=1,ask=FALSE)
mon2 <- monmonier(mondata2$xy,dist(mondata2$x2),cn2,threshold=2)
plot(mon2,mondata2$x2,method="greylevel",add.arr=FALSE,bwd=6,col="red",csize=.5)

### genetic data example
\dontrun{
data(sim2pop)

if(require(hierfstat)){
## try and find the Fst
fstat(sim2pop,fst=TRUE)
# Fst = 0.038
}

## run monmonier algorithm

# build connection network
gab <- chooseCN(sim2pop@other$xy,ask=FALSE,type=2)

# filter random noise 
pca1 <- dudi.pca(sim2pop@tab,scale=FALSE, scannf=FALSE, nf=1)

# run the algorithm
mon1 <- monmonier(sim2pop@other$xy,dist(pca1$l1[,1]),gab,scanthres=FALSE)

# graphical display 
plot(mon1,var=pca1$l1[,1])
temp <- sim2pop@pop
levels(temp) <- c(17,19)
temp <- as.numeric(as.character(temp))
plot(mon1)
points(sim2pop@other$xy,pch=temp,cex=2)
legend("topright",leg=c("Pop A", "Pop B"),pch=c(17,19))


### interactive example

# north-south separation
xy <- matrix(runif(120,0,10), ncol=2)
x1 <- rnorm(60)
x1[xy[,2] > 5] <- x1[xy[,2] > 5]+3
cn1 <- chooseCN(xy,type=1,ask=FALSE)
mon1 <- optimize.monmonier(xy,dist(x1)^2,cn1,ntry=10)

# graphics
plot(mon1,x1,met="greylevel",csize=.6)

# island in the middle
x2 <- rnorm(60)
sel <- (xy[,1]>3.5 & xy[,2]>3.5 & xy[,1]<6.5 & xy[,2]<6.5)
x2[sel] <- x2[sel]+4
cn2 <- chooseCN(xy,type=1,ask=FALSE)
mon2 <- optimize.monmonier(xy,dist(x2)^2,cn2,ntry=10)

# graphics
plot(mon2,x2,method="greylevel",add.arr=FALSE,bwd=6,col="red",csize=.5)
}
}
}
\keyword{multivariate}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hierarchyMethods.R
\docType{methods}
\name{hier}
\alias{hier}
\alias{hier,genind-method}
\alias{hier,genlight-method}
\alias{hier<-}
\alias{hier<-,genind-method}
\alias{hier<-,genlight-method}
\title{Access and manipulate the population hierarchy for genind or genlight objects.}
\usage{
hier(x, formula = NULL, combine = TRUE, value)

hier(x) <- value
}
\arguments{
\item{x}{a genind or genlight object}

\item{formula}{a nested formula indicating the order of the population 
hierarchy to be returned.}

\item{combine}{if \code{TRUE} (default), the levels will be combined
according to the formula argument. If it is \code{FALSE}, the levels will
not be combined.}

\item{value}{a formula specifying the full hierarchy of columns in the strata
slot. \strong{(See Details below)}}
}
\description{
The following methods allow the user to quickly change the hierarchy or
population of a genind or genlight object.
}
\details{
You must first specify your strata before you can specify your 
  hierarchies. Hierarchies are special cases of strata in that the levels 
  must be nested within each other. An error will occur if you specify a 
  hierarchy that is not truly hierarchical.
  
  \subsection{Details on Formulas}{
  
  The preferred use of these functions is with a \code{\link{formula}} 
  object. Specifically, a hierarchical formula argument is used to name which
  strata are hierarchical. An example of a hierarchical formula would
  be:\tabular{r}{ \code{~Country/City/Neighborhood}} This convention was
  chosen as it becomes easier to type and makes intuitive sense when defining
  a hierarchy. Note: it is important to use hierarchical formulas when
  specifying hierarchies as other types of formulas (eg. 
  \code{~Country*City*Neighborhood}) will give incorrect results.}
}
\examples{
# let's look at the microbov data set:
data(microbov)
microbov

# We see that we have three vectors of different names in the 'other' slot. 
?microbov
# These are Country, Breed, and Species
names(other(microbov))

# Let's set the hierarchy
strata(microbov) <- data.frame(other(microbov))
microbov

# And change the names so we know what they are
nameStrata(microbov) <- ~Country/Breed/Species

# let's see what the hierarchy looks like by Species and Breed:
hier(microbov) <- ~Species/Breed
head(hier(microbov, ~Species/Breed))

}
\seealso{
\code{\link{strata}} \code{\link{genind}}
  \code{\link{as.genind}}
}
\author{
Zhian N. Kamvar
}

\name{global.rtest}
\alias{global.rtest}
\alias{local.rtest}
\title{Global and local tests}
\description{
  These two Monte Carlo tests are used to assess the existence of global
  and local spatial structures. They can be used as an aid to interprete
  global and local components of spatial Principal Component Analysis
  (sPCA).\cr
  
  They rely on the decomposition of a data matrix X into global and local
  components using multiple regression on Moran's Eigenvector Maps (MEMs).
  They require a data matrix (X) and a list of weights derived from a
  connection network. X is regressed onto global MEMs (U+) in the global
  test and on local ones (U-) in the local test. One mean \eqn{R^2}{R^2}
  is obtained for each MEM, the k highest being summed to form the test
  statistic.
  
  The reference distribution of these statistics are obtained by
  randomly permuting the rows of X.
}
\usage{
global.rtest(X, listw, k = 1, nperm = 499)
local.rtest(X, listw, k = 1, nperm = 499)

}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{X}{a data matrix, with variables in columns}
  \item{listw}{a list of weights of class \code{listw}. Can be obtained
    easily using the function \code{chooseCN}.}
  \item{k}{integer: the number of highest \eqn{R^2}{R^2} summed to form the test statistics}
  \item{nperm}{integer: the number of randomisations to be performed.}
}
\details{
 This test is purely R code. A C or C++ version will be developped soon.
}
\value{
An object of class \code{randtest}.
}
\references{
  Jombart, T., Devillard, S., Dufour, A.-B. and Pontier, D.
  Revealing cryptic spatial patterns in genetic variability by a new
  multivariate method. \emph{Heredity}, \bold{101}, 92--103.
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk}}
\seealso{ \code{\link{chooseCN}}, \code{\link{spca}}, \code{\link{monmonier}}}
\examples{
\dontrun{
 data(sim2pop)
if(require(spdep)){
cn <- chooseCN(sim2pop@other$xy,ask=FALSE,type=1,plot=FALSE,res="listw")

# global test
Gtest <- global.rtest(sim2pop@tab,cn)
Gtest

# local test
Ltest <- local.rtest(sim2pop@tab,cn)
Ltest
}
}
}
\keyword{multivariate}
\keyword{spatial}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{read.structure}
\alias{read.structure}
\title{Reading data from STRUCTURE}
\usage{
read.structure(file, n.ind = NULL, n.loc = NULL, onerowperind = NULL,
  col.lab = NULL, col.pop = NULL, col.others = NULL,
  row.marknames = NULL, NA.char = "-9", pop = NULL, sep = NULL,
  ask = TRUE, quiet = FALSE)
}
\arguments{
\item{file}{a character string giving the path to the file to convert, with
the appropriate extension.}

\item{n.ind}{an integer giving the number of genotypes (or 'individuals') in
the dataset}

\item{n.loc}{an integer giving the number of markers in the dataset}

\item{onerowperind}{a STRUCTURE coding option: are genotypes coded on a
single row (TRUE), or on two rows (FALSE, default)}

\item{col.lab}{an integer giving the index of the column containing labels
of genotypes. '0' if absent.}

\item{col.pop}{an integer giving the index of the column containing
population to which genotypes belong. '0' if absent.}

\item{col.others}{an vector of integers giving the indexes of the columns
containing other informations to be read. Will be available in @other of the
created object.}

\item{row.marknames}{an integer giving the index of the row containing the
names of the markers. '0' if absent.}

\item{NA.char}{the character string coding missing data. "-9" by default.
Note that in any case, series of zero (like "000") are interpreted as NA
too.}

\item{pop}{an optional factor giving the population of each individual.}

\item{sep}{a character string used as separator between alleles.}

\item{ask}{a logical specifying if the function should ask for optional
informations about the dataset (TRUE, default), or try to be as quiet as
possible (FALSE).}

\item{quiet}{logical stating whether a conversion message must be printed
(TRUE,default) or not (FALSE).}
}
\value{
an object of the class \code{genind}
}
\description{
The function \code{read.structure} reads STRUCTURE data files (.str ou
.stru) and convert them into a \linkS4class{genind} object. By default, this
function is interactive and asks a few questions about data content. This
can be disabled (for optional questions) by turning the 'ask' argument to
FALSE. However, one has to know the number of genotypes, of markers and if
genotypes are coded on a single or on two rows before importing data.
}
\details{
Note: \code{read.structure} is meant for DIPLOID DATA ONLY. Haploid data
with the STRUCTURE format can easily be read into R using \code{read.table}
or \code{read.csv} and then converted using \code{\link{df2genind}}.
}
\examples{

obj <- read.structure(system.file("files/nancycats.str",package="adegenet"),
  onerowperind=FALSE, n.ind=237, n.loc=9, col.lab=1, col.pop=2, ask=FALSE)

obj

}
\references{
Pritchard, J.; Stephens, M. & Donnelly, P. (2000) Inference of
population structure using multilocus genotype data. \emph{Genetics},
\bold{155}: 945-959
}
\seealso{
\code{\link{import2genind}}, \code{\link{df2genind}},
\code{\link{read.fstat}}, \code{\link{read.genetix}},
\code{\link{read.genepop}}
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{read.fstat}
\alias{read.fstat}
\title{Reading data from Fstat}
\usage{
read.fstat(file, quiet = FALSE)
}
\arguments{
\item{file}{a character string giving the path to the file to convert, with
the appropriate extension.}

\item{quiet}{logical stating whether a conversion message must be printed
(TRUE,default) or not (FALSE).}
}
\value{
an object of the class \code{genind}
}
\description{
The function \code{read.fstat} reads Fstat data files (.dat) and convert
them into a \linkS4class{genind} object.
}
\details{
Note: \code{read.fstat} is meant for DIPLOID DATA ONLY. Haploid data with
the Hierfstat format can be read into R using \code{read.table} or
\code{read.csv} after removing headers and 'POP' lines, and then converted
using \code{\link{df2genind}}.
}
\examples{

obj <- read.fstat(system.file("files/nancycats.dat",package="adegenet"))
obj

}
\references{
Fstat (version 2.9.3). Software by Jerome Goudet.
http://www2.unil.ch/popgen/softwares/fstat.htm\cr
}
\seealso{
\code{\link{import2genind}}, \code{\link{df2genind}},
\code{\link{read.genetix}}, \code{\link{read.structure}},
\code{\link{read.genepop}}
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/spca_randtest.R
\name{spca_randtest}
\alias{spca_randtest}
\title{Monte Carlo test for sPCA}
\usage{
spca_randtest(x, nperm = 499)
}
\arguments{
\item{x}{A \code{\link{spca}} object.}

\item{nperm}{The number of permutations to be used for the test.}
}
\value{
A list with two objects of the class 'randtest' (see
\code{\link[ade4]{as.randtest}}), the first one for 'global' structures
(positivie autocorrelation) and the second for 'local' structures (negative
autocorrelation).
}
\description{
The function \code{spca_randtest} implements Monte-Carlo tests for the
presence of significant spatial structures in a sPCA object. Two tests are
run, for global (positive autocorrelation) and local (negative
autocorrelation) structures, respectively. The test statistics used are the
sum of the absolute values of the corresponding eigenvalues.
}
\examples{

\dontrun{
## Load data
data(sim2pop)

## Make spca
spca1 <- spca(sim2pop, type = 1, scannf = FALSE, plot.nb = FALSE)

spca1
plot(spca1)

## run tests (use more permutations in practice, e.g. 999)
tests <- spca_randtest(spca1, nperm = 49)

## check results
tests
plot(tests[[1]]) # global structures

}

}
\author{
Original code by Valeria Montano adapted by Thibaut Jombart.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/export_to_mvmapper.R
\name{export_to_mvmapper}
\alias{export_to_mvmapper}
\alias{swallowtails}
\alias{export_to_mvmapper.default}
\alias{export_to_mvmapper.dapc}
\alias{export_to_mvmapper.dudi}
\alias{export_to_mvmapper.spca}
\title{Export analysis for mvmapper visualisation}
\usage{
export_to_mvmapper(x, ...)

\method{export_to_mvmapper}{default}(x, ...)

\method{export_to_mvmapper}{dapc}(x, info, write_file = TRUE,
  out_file = NULL, ...)

\method{export_to_mvmapper}{dudi}(x, info, write_file = TRUE,
  out_file = NULL, ...)

\method{export_to_mvmapper}{spca}(x, info, write_file = TRUE,
  out_file = NULL, ...)
}
\arguments{
\item{x}{The analysis to be exported. Can be a \code{dapc}, \code{spca}, or a
\code{dudi} object.}

\item{...}{Further arguments to pass to other methods.}

\item{info}{A \code{data.frame} with additional information containing at
least the following columns: \code{key} (unique individual identifier),
\code{lat} (latitude), and \code{lon} (longitude). Other columns will be
exported as well, but are optional.}

\item{write_file}{A \code{logical} indicating if the output should be written
out to a .csv file. Defaults to \code{TRUE}.}

\item{out_file}{A character string indicating the file to which the output
should be written. If NULL, the file used will be named
\code{'mvmapper_data_[date and time].csv'}}
}
\value{
A \code{data.frame} which can serve as input to \code{mvmapper},
containing at least the following columns:

\itemize{

\item \code{key}: unique individual identifiers

\item \code{PC1}: first principal component; further principal components are
optional, but if provided will be numbered and follow \code{PC1}.

\item \code{lat}: latitude for each individual

\item \code{lon}: longitude for each individual

}

In addition, specific information is added for some analyses:

\itemize{

\item \code{spca}: \code{Lag_PC} columns contain the lag-vectors of the
principal components; the lag operator computes, for each individual, the
average score of neighbouring individuals; it is useful for clarifying
patches and clines.

\item \code{dapc}: \code{grp} is the group used in the analysis;
\code{assigned_grp} is the group assignment based on the discriminant
functions; \code{support} is the statistical support (i.e. assignment
probability) for \code{assigned_grp}.

}
}
\description{
\code{mvmapper} is an interactive tool for visualising outputs of a
multivariate analysis on a map from a web browser. The function
\code{export_to_mvmapper} is a generic with methods for several standard
classes of analyses in \code{adegenet} and \code{ade4}. Information on
individual locations, as well as any other relevant data, is passed through
the second argument \code{info}. By default, the function returns a formatted
\code{data.frame} and writes the output to a .csv file.\cr
}
\details{
\code{mvmapper} can be found at:
\url{https://popphylotools.github.io/mvMapper/}
}
\examples{

# An example using the microsatellite dataset of Dupuis et al. 2016 (781
# individuals, 10 loci, doi: 10.1111/jeb.12931)

# Reading input file from adegenet

input_data <- system.file("data/swallowtails.rda", package="adegenet")
data(swallowtails)


# conducting a DAPC (n.pca determined using xvalDapc, see ??xvalDapc)

dapc1 <- dapc(swallowtails, n.pca=40, n.da=200)


# read in swallowtails_loc.csv, which contains "key", "lat", and "lon"
# columns with column headers (this example contains additional columns
# containing species identifications, locality descriptions, and COI
# haplotype clades)

input_locs <- system.file("files/swallowtails_loc.csv", package = "adegenet")
loc <- read.csv(input_locs, header = TRUE)


# generate mvmapper input file, automatically write the output to a csv, and
# name the output csv "mvMapper_Data.csv"

out <- export_to_mvmapper(dapc1, loc, write_file = TRUE,
                          out_file = "mvMapper_Data.csv")

}
\seealso{
\code{mvmapper} is available at:
\url{https://popphylotools.github.io/mvMapper/}
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/snapclust.R
\name{snapclust}
\alias{snapclust}
\title{Maximum-likelihood genetic clustering using EM algorithm}
\usage{
snapclust(x, k, pop.ini = "ward", max.iter = 100, n.start = 10,
  n.start.kmeans = 50, hybrids = FALSE, dim.ini = 100,
  hybrid.coef = NULL, parent.lab = c("A", "B"), ...)
}
\arguments{
\item{x}{a \linkS4class{genind} object}

\item{k}{the number of clusters to look for}

\item{pop.ini}{parameter indicating how the initial group membership should
be found. If \code{NULL}, groups are chosen at random, and the algorithm
will be run \code{n.start} times. If "kmeans", then the function
\code{find.clusters} is used to define initial groups using the K-means
algorithm. If "ward", then the function \code{find.clusters} is used to
define initial groups using the Ward algorithm. Alternatively, a factor
defining the initial cluster configuration can be provided.}

\item{max.iter}{the maximum number of iteration of the EM algorithm}

\item{n.start}{the number of times the EM algorithm is run, each time with
different random starting conditions}

\item{n.start.kmeans}{the number of times the K-means algorithm is run to
define the starting point of the ML-EM algorithm, each time with
different random starting conditions}

\item{hybrids}{a logical indicating if hybrids should be modelled
explicitely; this is currently implemented for 2 groups only.}

\item{dim.ini}{the number of PCA axes to retain in the dimension reduction
step for \code{\link{find.clusters}}, if this method is used to define
initial group memberships (see argument \code{pop.ini}).}

\item{hybrid.coef}{a vector of hybridization coefficients, defining the
proportion of hybrid gene pool coming from the first parental population;
this is symmetrized around 0.5, so that e.g. c(0.25, 0.5) will be
converted to c(0.25, 0.5, 0.75)}

\item{parent.lab}{a vector of 2 character strings used to label the two
parental populations; only used if hybrids are detected (see argument
\code{hybrids})}

\item{...}{further arguments passed on to \code{\link{find.clusters}}}
}
\value{
The function \code{snapclust} returns a list with the following
components:
\itemize{

\item \code{$group} a factor indicating the maximum-likelihood assignment of
individuals to groups; if identified, hybrids are labelled after
hybridization coefficients, e.g. 0.5_A - 0.5_B for F1, 0.75_A - 0.25_B for
backcross F1 / A, etc.

\item \code{$ll}: the log-likelihood of the model

\item \code{$proba}: a matrix of group membership probabilities, with
individuals in rows and groups in columns; each value correspond to the
probability that a given individual genotype was generated under a given
group, under Hardy-Weinberg hypotheses.

\item \code{$converged} a logical indicating if the algorithm converged; if
FALSE, it is doubtful that the result is an actual Maximum Likelihood
estimate.

\item \code{$n.iter} an integer indicating the number of iterations the EM
algorithm was run for.

}
}
\description{
The function \code{snapclust} implements fast maximum-likelihood (ML) genetic
clustering using a 2-step approach: find a 'good' initial starting point
using a fast geometric (distance-based) approach, followed by an efficient
likelihood optimization using the expectation-maximization (EM)
algorithm. The likelihood for a given locus is defined as the probability of
a given genotype given the allele frequencies of its originating population,
as defined by Hardy-Weinberg equilibrium. Different approaches are available
for the initial distance-based clustering (see argument \code{pop.ini}). The
current implementation is designed for haploid or diploid data, and assumes
constant ploidy within the dataset.
}
\details{
This function can be used in two modes:

\itemize{

\item genetic clustering mode: assign individuals to a known number 'k'
 of panmictic clusters; see \code{\link{snapclust.choose.k}} for identifying
 optimal values of 'k'.

\item hybrid mode: as previous mode, but with the additional constraint that
 clusters are either one of the 2 parental populations, or any hybrid classes
 in between, as defined by the parameter \code{hybrid.coef}.

}

This method is documented in an online tutorial, which can be opened by typing:
\code{adegenetTutorial("snapclust")}.
}
\examples{
\dontrun{
data(microbov)

## try function using k-means initialization
grp.ini <- find.clusters(microbov, n.clust=15, n.pca=150)

## run EM algo
res <- snapclust(microbov, 15, pop.ini = grp.ini$grp)
names(res)
res$converged
res$n.iter

## plot result
compoplot(res)

## flag potential hybrids
to.flag <- apply(res$proba,1,max)<.9
compoplot(res, subset=to.flag, show.lab=TRUE,
                 posi="bottomleft", bg="white")


## Simulate hybrids F1
zebu <- microbov[pop="Zebu"]
salers <- microbov[pop="Salers"]
hyb <- hybridize(zebu, salers, n=30)
x <- repool(zebu, salers, hyb)

## method without hybrids
res.no.hyb <- snapclust(x, k=2, hybrids=FALSE)
compoplot(res.no.hyb, col.pal=spectral, n.col=2)

## method with hybrids
res.hyb <- snapclust(x, k=2, hybrids=TRUE)
compoplot(res.hyb, col.pal =
          hybridpal(col.pal = spectral), n.col = 2)


## Simulate hybrids backcross (F1 / parental)
f1.zebu <- hybridize(hyb, zebu, 20, pop = "f1.zebu")
f1.salers <- hybridize(hyb, salers, 25, pop = "f1.salers")
y <- repool(x, f1.zebu, f1.salers)

## method without hybrids
res2.no.hyb <- snapclust(y, k = 2, hybrids = FALSE)
compoplot(res2.no.hyb, col.pal = hybridpal(), n.col = 2)

## method with hybrids F1 only
res2.hyb <- snapclust(y, k = 2, hybrids = TRUE)
compoplot(res2.hyb, col.pal = hybridpal(), n.col = 2)

## method with back-cross
res2.back <- snapclust(y, k = 2, hybrids = TRUE, hybrid.coef = c(.25,.5))
 compoplot(res2.back, col.pal = hybridpal(), n.col = 2)

}
}
\references{
Beugin M-P, Gayet T, Pontier D, Devillard S, Jombart T. A fast
    likelihood solution to the genetic clustering problem. Methods Ecol
    Evol. 2018;00:1–11. \url{https://doi.org/10.1111/2041-210X.12968}
}
\seealso{
\itemize{
 \item \code{\link{snapclust.choose.k}} to choose 'k' using various
    goodness-of-fit statistics

 \item \code{\link{AIC.snapclust}}: AIC computation

 \item \code{\link{AICc.snapclust}}: AICc computation

 \item \code{\link{BIC.snapclust}}: BIC computation

 \item \code{\link{KIC.snapclust}}: KIC computation

 }
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com} and Marie-Pauline
Beugin
}

\encoding{UTF-8}
\name{genlight auxiliary functions}
\alias{glSum}
\alias{glNA}
\alias{glMean}
\alias{glVar}
\alias{glDotProd}
\title{Auxiliary functions for genlight objects}
\description{
  These functions provide facilities for usual computations using
  \linkS4class{genlight} objects. When ploidy varies across individuals,
   the outputs of these functions depend on whether the information
   units are individuals, or alleles within individuals (see details).
  
  These functions are:
  
  - \code{glSum}: computes the sum of the number of second allele in each SNP.

  - \code{glNA}: computes the number of missing values in each SNP.

  - \code{glMean}: computes the mean number of second allele in each SNP.

  - \code{glVar}: computes the variance of the number of second allele in each SNP.

  - \code{glDotProd}: computes dot products between (possibly
  centred/scaled) vectors of individuals - uses compiled C code - used
  by glPca.
}
\usage{
glSum(x, alleleAsUnit = TRUE, useC = FALSE)
glNA(x, alleleAsUnit = TRUE)
glMean(x, alleleAsUnit = TRUE)
glVar(x, alleleAsUnit = TRUE)
glDotProd(x, center = FALSE, scale = FALSE, alleleAsUnit = FALSE,
                parallel = FALSE, n.cores = NULL)
}
\arguments{
  \item{x}{a \linkS4class{genlight} object}
  \item{alleleAsUnit}{a logical indicating whether alleles are
    considered as units (i.e., a diploid genotype equals two samples, a
    triploid, three, etc.) or whether individuals are considered as
    units of information.}
  \item{center}{a logical indicating whether SNPs should be centred to
    mean zero.}
  \item{scale}{a logical indicating whether SNPs should be scaled to
    unit variance.}
  \item{useC}{a logical indicating whether compiled C code should be
    used (TRUE) or not (FALSE, default).}
  \item{parallel}{a logical indicating whether multiple cores -if
    available- should be used for the computations (TRUE, default), or
    not (FALSE); requires the package \code{parallel} to be installed
    (see details); this option cannot be used alongside useCoption.}
  \item{n.cores}{if \code{parallel} is TRUE, the number of cores to
    be used in the computations; if NULL, then the maximum number of
    cores available on the computer is used.}
}
\details{
  === On the unit of information ===
  
  In the cases where individuals can have different ploidy, computation
  of sums, means, etc. of allelic data depends on what we consider as a
  unit of information.

  To estimate e.g. allele frequencies, unit of information can be
  considered as the allele, so that a diploid genotype contains two
  samples, a triploid individual, three samples, etc. In such a case,
  all computations are done directly on the number of alleles. This
  corresponds to \code{alleleAsUnit = TRUE}.

  However, when the focus is put on studying differences/similarities
  between individuals, the unit of information is the individual, and
  all genotypes possess the same information no matter what their ploidy
  is. In this case, computations are made after standardizing
  individual genotypes to relative allele frequencies. This
  corresponds to \code{alleleAsUnit = FALSE}.

  Note that when all individuals have the same ploidy, this distinction
  does not hold any more.
}
\value{
 A numeric vector containing the requested information.
}

\seealso{
  - \code{\linkS4class{genlight}}: class of object for storing massive binary
  SNP data.
  
  - \code{\link{dapc}}: Discriminant Analysis of Principal Components.

  - \code{\link{glPca}}: PCA for \linkS4class{genlight} objects.

  - \code{\link{glSim}}: a simple simulator for \linkS4class{genlight} objects.

  - \code{\link{glPlot}}: plotting \linkS4class{genlight} objects.

}
\author{ Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
\dontrun{
x <- new("genlight", list(c(0,0,1,1,0), c(1,1,1,0,0,1), c(2,1,1,1,1,NA)))
x
as.matrix(x)
ploidy(x)

## compute statistics - allele as unit ##
glNA(x)
glSum(x)
glMean(x)

## compute statistics - individual as unit ##
glNA(x, FALSE)
glSum(x, FALSE)
glMean(x, FALSE)

## explanation: data are taken as relative frequencies
temp <- as.matrix(x)/ploidy(x)
apply(temp,2, function(e) sum(is.na(e))) # NAs
apply(temp,2,sum, na.rm=TRUE) # sum
apply(temp,2,mean, na.rm=TRUE) # mean
}
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{read.genetix}
\alias{read.genetix}
\title{Reading data from GENETIX}
\usage{
read.genetix(file = NULL, quiet = FALSE)
}
\arguments{
\item{file}{a character string giving the path to the file to convert, with
the appropriate extension.}

\item{quiet}{logical stating whether a conversion message must be printed
(TRUE,default) or not (FALSE).}
}
\value{
an object of the class \code{genind}
}
\description{
The function \code{read.genetix} reads GENETIX data files (.gtx) and convert
them into a \linkS4class{genind} object.
}
\details{
Note: \code{read.genetix} is meant for DIPLOID DATA ONLY. Haploid data with
the GENETIX format can be read into R using \code{read.table} or
\code{read.csv} after removing headers and 'POP' lines, and then converted
using \code{\link{df2genind}}.
}
\examples{

obj <- read.genetix(system.file("files/nancycats.gtx",package="adegenet"))
obj

}
\references{
Belkhir K., Borsa P., Chikhi L., Raufaste N. & Bonhomme F.
(1996-2004) GENETIX 4.05, logiciel sous Windows TM pour la genetique des
populations. Laboratoire Genome, Populations, Interactions, CNRS UMR 5000,
Universite de Montpellier II, Montpellier (France). \cr
}
\seealso{
\code{\link{import2genind}}, \code{\link{df2genind}},
\code{\link{read.fstat}}, \code{\link{read.structure}},
\code{\link{read.genepop}}
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/old2new.R
\name{old2new_genind}
\alias{old2new_genind}
\alias{old2new}
\alias{old2new_genlight}
\alias{old2new_genpop}
\title{Convert objects with obsolete classes into new objects}
\usage{
old2new_genind(object, donor = new("genind"))

old2new_genlight(object, donor = new("genlight"))

old2new_genpop(object, donor = new("genpop"))
}
\arguments{
\item{object}{a genind or genlight object from version 1.4 or earlier.}

\item{donor}{a new object to place all the data into.}
}
\description{
The genind and genlight objects have changed in Adegenet version 2.0. They
have each gained strata and hierarchy slots. What's more is that the genind
objects have been optimized for storage and now store the tab slot as
integers instead of numerics. This function will convert old genind or
genlight objects to new ones seamlessly.
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}\cr
Zhian N. Kamvar \email{kamvarz@science.oregonstate.edu}
}
\keyword{manip}

\name{loadingplot}
\alias{loadingplot}
\alias{loadingplot.default}
\title{Represents a cloud of points with colors}
\description{
  The \code{loadingplot} function represents positive values of a vector
  and identifies the values above a given threshold. It can also
  indicate groups of observations provided as a factor. \cr

  Such graphics can be used, for instance, to assess the weight of each
  variable (loadings) in a given analysis.
}
\usage{
loadingplot(x, \dots)

\method{loadingplot}{default}(x, at=NULL, threshold=quantile(x,0.75),
            axis=1, fac=NULL, byfac=FALSE,
            lab=NULL, cex.lab=0.7, cex.fac=1, lab.jitter=0,
            main="Loading plot", xlab="Variables", ylab="Loadings",
            srt = 0, adj = NULL, \dots)
}
\arguments{
  \item{x}{either a vector with numeric values to be plotted, or a
    matrix-like object containing numeric values. In such case, the
    \code{x[,axis]} is used as vector of values to be plotted.}
  \item{at}{an optional numeric vector giving the abscissa at which
    loadings are plotted. Useful when variates are SNPs with a known
    position in an alignement.}
  \item{threshold}{a threshold value above which values of x are
    identified. By default, this is the third quartile of x.}
  \item{axis}{an integer indicating the column of x to be plotted; used
    only if x is a matrix-like object.}
  \item{fac}{a factor defining groups of observations.}
  \item{byfac}{a logical stating whether loadings should be averaged by
    groups of observations, as defined by \code{fac}.}
  \item{lab}{a character vector giving the labels used to annotate
    values above the threshold; if NULL, names are taken from the object.}
  \item{cex.lab}{a numeric value indicating the size of annotations.}
  \item{cex.fac}{a numeric value indicating the size of annotations for
    groups of observations.}
  \item{lab.jitter}{a numeric value indicating the factor of
    randomisation for the position of annotations. Set to 0 (by default)
  implies no randomisation.}
  \item{main}{the main title of the figure.}
  \item{xlab}{the title of the x axis.}
  \item{ylab}{the title of the y axis.}
  \item{srt}{rotation of the labels; see ?text.}
  \item{adj}{adjustment of the labels; see ?text.}
  \item{\dots}{further arguments to be passed to the plot function.}
}
  \value{
    Invisibly returns a list with the following components:\cr
    - threshold: the threshold used\cr
    - var.names: the names of observations above the threshold\cr
    - var.idx: the indices of observations above the threshold\cr
    - var.values: the values above the threshold\cr
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk}}
\examples{
x <- runif(20)
names(x) <- letters[1:20]
grp <- factor(paste("group", rep(1:4,each=5)))

## basic plot
loadingplot(x)

## adding groups
loadingplot(x,fac=grp,main="My title",cex.lab=1)
}
\keyword{multivariate}
\keyword{hplot}

\name{SNPbin-class}
\docType{class}
\alias{SNPbin}
\alias{SNPbin-class}
\alias{[,SNPbin-method}
\alias{[,SNPbin,ANY,ANY-method}
\alias{[,SNPbin,ANY,ANY,ANY-method}
\alias{initialize,SNPbin-method}
\alias{show,SNPbin-method}
\alias{nLoc,SNPbin-method}
\alias{$,SNPbin-method}
\alias{$<-,SNPbin-method}
\alias{names,SNPbin-method}
\alias{ploidy,SNPbin-method}
\alias{ploidy<-,SNPbin-method}
\alias{coerce,SNPbin,integer-method}
\alias{as.integer.SNPbin}
\alias{NA.posi,SNPbin-method}
\alias{cbind.SNPbin}
\alias{c.SNPbin}
\alias{as,integer,SNPbin-method}
\alias{as,numeric,SNPbin-method}


% \alias{,SNPbin-method}
% \alias{,SNPbin-method}
% \alias{,SNPbin-method}
% \alias{,SNPbin-method}
%%%%
\title{Formal class "SNPbin"}
\description{
  The class \code{SNPbin} is a formal (S4) class for storing a genotype
  of binary SNPs in a compact way, using a bit-level coding scheme.
  This storage is most efficient with haploid data, where the memory
  taken to represent data can reduced more than 50 times. However,
  \code{SNPbin} can be used for any level of ploidy, and still remain an
  efficient storage mode.

  A \code{SNPbin} object can be constructed from
  a vector of integers giving the number of the second allele for each
  locus.

  \code{SNPbin} stores a single genotype. To store multiple genotypes,
  use the \linkS4class{genlight} class.
}
\section{Objects from the class SNPbin}{
  \code{SNPbin} objects can be created by calls to \code{new("SNPbin",
    ...)}, where '...' can be the following arguments:
  
  \describe{
    \item{\code{snp}}{a vector of integers or numeric giving numbers of
      copies of the second alleles for each locus. If only one unnamed
      argument is provided to 'new', it is considered as this one.}
    \item{\code{ploidy}}{an integer indicating the ploidy of the
      genotype; if not provided, will be guessed from the data (as the
      maximum from the 'snp' input vector).}
    \item{\code{label}}{an optional character string serving as a label
      for the genotype.}
  }
}
\section{Slots}{
  The following slots are the content of instances of the class
  \code{SNPbin}; note that in most cases, it is better to retrieve
  information via accessors (see below), rather than by accessing the
  slots manually.
  \describe{
    \item{\code{snp}:}{a list of vectors with the class \code{raw}.}
    \item{\code{n.loc}:}{an integer indicating the number of SNPs of the
      genotype.}
    \item{\code{NA.posi}:}{a vector of integer giving the position of
      missing data.}
    \item{\code{label}:}{an optional character string serving as a label
      for the genotype..}
    \item{\code{ploidy}:}{an integer indicating the ploidy of the genotype.}
  }
}
\section{Methods}{
  Here is a list of methods available for \code{SNPbin} objects. Most of
  these methods are accessors, that is, functions which are used to
  retrieve the content of the object. Specific manpages can exist for
  accessors with more than one argument. These are indicated by a '*'
  symbol next to the method's name. This list also contains methods
  for conversion from \code{SNPbin} to other classes.
  \describe{
    \item{[}{\code{signature(x = "SNPbin")}: usual method to subset
      objects in R. The argument indicates how SNPs are to be
      subsetted. It can be a vector of signed integers or of logicals.}
    \item{show}{\code{signature(x = "SNPbin")}: printing of the
      object.}
    \item{$}{\code{signature(x = "SNPbin")}: similar to the @ operator;
      used to access the content of slots of the object.}
    \item{$<-}{\code{signature(x = "SNPbin")}: similar to the @ operator;
      used to replace the content of slots of the object.}
    \item{nLoc}{\code{signature(x = "SNPbin")}: returns the number of
      SNPs in the object.}
    \item{names}{\code{signature(x = "SNPbin")}: returns the names of
      the slots of the object.}
    \item{ploidy}{\code{signature(x = "SNPbin")}: returns the ploidy of
      the genotype.}
    \item{as.integer}{\code{signature(x = "SNPbin")}: converts a
      \code{SNPbin} object to a vector of integers. The S4 method 'as' can
      be used as well (e.g. as(x, "integer")).}
    \item{cbind}{\code{signature(x = "SNPbin")}: merges genotyping of
      the same individual at different SNPs (all stored as
      \linkS4class{SNPbin} objects) into a single \linkS4class{SNPbin}.}
    \item{c}{\code{signature(x = "SNPbin")}: same as cbind.SNPbin.}
  }
}
\author{Thibaut Jombart (\email{t.jombart@imperial.ac.uk})}
\seealso{
  Related class:\cr
  -  \code{\linkS4class{genlight}}, for storing multiple binary SNP
  genotypes. \cr
  -  \code{\linkS4class{genind}}, for storing other types of genetic markers. \cr
}
\examples{
\dontrun{
#### HAPLOID EXAMPLE ####
## create a genotype of 100,000 SNPs
dat <- sample(c(0,1,NA), 1e5, prob=c(.495, .495, .01), replace=TRUE)
dat[1:10]
x <- new("SNPbin", dat)
x
x[1:10] # subsetting
as.integer(x[1:10])

## try a few accessors
ploidy(x)
nLoc(x)
head(x$snp[[1]]) # internal bit-level coding

## check that conversion is OK
identical(as(x, "integer"),as.integer(dat)) # SHOULD BE TRUE

## compare the size of the objects
print(object.size(dat), unit="auto")
print(object.size(x), unit="auto")
object.size(dat)/object.size(x) # EFFICIENCY OF CONVERSION


#### TETRAPLOID EXAMPLE ####
## create a genotype of 100,000 SNPs
dat <- sample(c(0:4,NA), 1e5, prob=c(rep(.995/5,5), 0.005), replace=TRUE)
x <- new("SNPbin", dat)
identical(as(x, "integer"),as.integer(dat)) # MUST BE TRUE

## compare the size of the objects
print(object.size(dat), unit="auto")
print(object.size(x), unit="auto")
object.size(dat)/object.size(x) # EFFICIENCY OF CONVERSION


#### c, cbind ####
a <- new("SNPbin", c(1,1,1,1,1))
b <- new("SNPbin", c(0,0,0,0,0))
a
b
ab <- c(a,b)
ab
identical(c(a,b),cbind(a,b))
as.integer(ab)
}
}
\keyword{classes}

\encoding{UTF-8}
\name{fasta2genlight}
\alias{fasta2genlight}
\title{ Extract Single Nucleotide Polymorphism (SNPs) from alignments}
\description{
  The function \code{fasta2genlight} reads alignments with the fasta
  format (extensions ".fasta", ".fas", or ".fa"), extracts the binary
  SNPs, and converts the output into a \linkS4class{genlight} object. 

  The function reads data by chunks of a few genomes (minimum 1, no
  maximum) at a time, which allows one to read massive datasets with
  negligible RAM requirements (albeit at a cost of computational
  time). The argument \code{chunkSize} indicates the number of genomes
  read at a time. Increasing this value decreases the computational time
  required to read data in, while increasing memory requirements.

  Multiple cores can be used to decrease the overall computational time
  on parallel architectures (needs the package \code{parallel}).
}
\usage{
fasta2genlight(file, quiet=FALSE, chunkSize = 1000, saveNbAlleles=FALSE,
               parallel = require("parallel"), n.cores = NULL, \dots)
}
\arguments{
  \item{file}{ a character string giving the path to the file to
    convert, with the extension ".fa", ".fas", or ".fasta".}
  \item{quiet}{ logical stating whether a conversion messages should be
    printed (FALSE,default) or not (TRUE).}
  \item{chunkSize}{an integer indicating the number of genomes to be
    read at a time; larger values require more RAM but decrease the time
    needed to read the data.}
  \item{saveNbAlleles}{a logical indicating whether the number of
    alleles for each loci in the original alignment should be saved in the
    \code{other} slot (TRUE), or not (FALSE, default). In large genomes,
    this takes some space but allows for tracking SNPs with more than 2
    alleles, lost during the conversion.}
  \item{parallel}{a logical indicating whether multiple cores -if
    available- should be used for the computations (TRUE, default), or
    not (FALSE); requires the package \code{parallel} to be installed
    (see details).}
  \item{n.cores}{if \code{parallel} is TRUE, the number of cores to be
    used in the computations; if NULL, then the maximum number of cores
    available on the computer is used.}
  \item{\dots}{other arguments to be passed to other functions -
    currently not used.}
}
\details{

  === Using multiple cores ===

  Most recent machines have one or several processors with multiple
  cores. R processes usually use one single core. The package
  \code{parallel} allows for parallelizing some computations on
  multiple cores, which decreases drastically computational time.

  To use this functionality, you need to have the last version of the
  \code{parallel} package installed.
}
\value{an object of the class \linkS4class{genlight}}
\seealso{
  - \code{?genlight} for a description of the class \linkS4class{genlight}.

  - \code{\link{read.snp}}: read SNPs in adegenet's '.snp' format.

  - \code{\link{read.PLINK}}: read SNPs in PLINK's '.raw' format.

  - \code{\link{df2genind}}: convert any multiallelic markers into
  adegenet \linkS4class{genind}.

  - \code{\link{import2genind}}: read multiallelic markers from various
  software into adegenet.
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
\dontrun{
## show the example file ##
## this is the path to the file:
myPath <- system.file("files/usflu.fasta",package="adegenet")
myPath

## read the file
obj <- fasta2genlight(myPath, chunk=10) # process 10 sequences at a time
obj

## look at extracted information
position(obj)
alleles(obj)
locNames(obj)

## plot positions of polymorphic sites
temp <- density(position(obj), bw=10)
plot(temp, xlab="Position in the alignment", lwd=2, main="Location of the SNPs")
points(position(obj), rep(0, nLoc(obj)), pch="|", col="red")
}
}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AICc.snapclust.R
\name{AICc}
\alias{AICc}
\alias{AICc.snapclust}
\title{snapclust's corrected Akaike Information Criterion (AICc)}
\usage{
AICc(object, ...)

\method{AICc}{snapclust}(object, ...)
}
\arguments{
\item{object}{An object returned by the function \code{\link{snapclust}}.}

\item{...}{Further arguments for compatibility with the \code{AIC} generic
(currently not used).}
}
\description{
This function computes Akaike Information Criterion for small samples (AICc)
for \code{snapclust} results.
}
\references{
Beugin M-P, Gayet T, Pontier D, Devillard S, Jombart T. A fast
    likelihood solution to the genetic clustering problem. Methods Ecol
    Evol. 2018;00:1–11. \url{https://doi.org/10.1111/2041-210X.12968}
}
\seealso{
\itemize{
 \item \code{\link{snapclust}}: to identify clusters

 \item \code{\link{snapclust.choose.k}}: to find the number of clusters

 \item \code{\link{AIC.snapclust}}: AIC computation

 \item \code{\link{BIC.snapclust}}: BIC computation

 \item \code{\link{KIC.snapclust}}: KIC computation
}
}
\author{
Thibaut Jombart \email{thibautjombart@gmail.com}
}

\name{haploGen}
\alias{haploGen}
\alias{print.haploGen}
\alias{[.haploGen}
\alias{labels.haploGen}
\alias{as.POSIXct.haploGen}
\alias{seqTrack.haploGen}
\alias{haploGen-class}
\alias{as.seqTrack.haploGen}
\alias{as.igraph.haploGen}
\alias{plot.haploGen}
\alias{plotHaploGen}
\alias{sample.haploGen}
\title{Simulation of genealogies of haplotypes}
\description{
  The function \code{haploGen} implements simulations of genealogies of
  haplotypes. This forward-time, individual-based simulation tool allows
  haplotypes to replicate and mutate according to specified parameters,
  and keeps track of their genealogy.

  Simulations can be spatially explicit or not (see \code{geo.sim}
  argument). In the first case, haplotypes are assigned to locations on
  a regular grip. New haplotypes disperse from their ancestor's location
  according to a random Poisson diffusion, or alternatively according to
  a pre-specified migration scheme. This tool does not allow for
  simulating selection or linkage disequilibrium.

  Produced objects are lists with the class \code{haploGen}; see 'value'
  section for more information on this class. Other functions are
  available to print, plot, subset, sample or convert \code{haploGen}
  objects. A seqTrack method is also provided for analysing
  \code{haploGen} objects.

  Note that for simulation of outbreaks, the new tool \code{simOutbreak}
  in the \code{outbreaker} package should be used.
}
\usage{
haploGen(seq.length=1e4, mu.transi=1e-4, mu.transv=mu.transi/2, t.max=20,
         gen.time=function(){1+rpois(1,0.5)},
         repro=function(){rpois(1,1.5)}, max.nb.haplo=200,
         geo.sim=FALSE, grid.size=10, lambda.xy=0.5,
         mat.connect=NULL,
         ini.n=1, ini.xy=NULL)
\method{print}{haploGen}(x, \dots)
\method{as.igraph}{haploGen}(x, col.pal=redpal, \dots)
\method{plot}{haploGen}(x, y=NULL, col.pal=redpal, \dots)
\method{[}{haploGen}(x, i, j, drop=FALSE)
\method{labels}{haploGen}(object, \dots)
\method{as.POSIXct}{haploGen}(x, tz="", origin=as.POSIXct("2000/01/01"), \dots)
\method{seqTrack}{haploGen}(x, best=c("min","max"), prox.mat=NULL, \dots)
as.seqTrack.haploGen(x)
plotHaploGen(x, annot=FALSE, date.range=NULL, col=NULL, bg="grey", add=FALSE, \dots)
sample.haploGen(x, n)
%\S4method{coerce}{haploGen,graphNEL}(from, to, strict=TRUE)
}
\arguments{
  \item{seq.length}{an integer indicating the length of the simulated
    haplotypes, in number of nucleotides.}
  \item{mu.transi}{the rate of transitions, in number of mutation per site and per
    time unit.}
  \item{mu.transv}{the rate of transversions, in number of mutation per site and per
    time unit.}
  \item{t.max}{an integer indicating the maximum number of time units to
    run the simulation for.}
  \item{gen.time}{an integer indicating the generation time, in number
    of time units. Can be a (fixed) number or a function returning a number
    (then called for each reproduction event).}
  \item{repro}{an integer indicating the number of descendents per
    haplotype. Can be a (fixed) number or a function returning a number
    (then called for each reproduction event).}
  \item{max.nb.haplo}{an integer indicating the maximum number of
    haplotypes handled at any time of the simulation, used to control the
    size of the produced object. Larger number will lead to slower
    simulations. If this number is exceeded, the genealogy is prunded to as
    to keep this number of haplotypes.}
  \item{geo.sim}{a logical stating whether simulations should be
    spatially explicit (TRUE) or not (FALSE, default). Spatially-explicit
    simulations are slightly slower than their non-spatial counterpart.}
  \item{grid.size}{the size of the square grid of possible locations for
    spatial simulations. The total number of locations will be this number
    squared.}
  \item{lambda.xy}{the parameter of the Poisson distribution used to
    determine dispersion in x and y axes.}
  \item{mat.connect}{a matrix of connectivity describing migration
    amongts all pairs of locations. \code{mat.connect[i,j]} indicates the
    probability, being in 'i', to migrate to 'j'. The rows of this matrix
    thus sum to 1. It has as many rows and columns as there are locations,
    with row 'i' / column 'j' corresponding to locations number 'i' and 'j'.
    Locations are numbered as in a matrix in which rows and columns are
    respectively x and y coordinates. For instance, in a 5x5 grid, locations
    are numbered as in \code{matrix(1:25,5,5)}.}
  \item{ini.n}{an integer specifying the number of (identical)
    haplotypes to initiate the simulation}
  \item{ini.xy}{a vector of two integers giving the x/y coordinates of the initial haplotype.}
  \item{x,object}{\code{haploGen} objects.}
  \item{y}{unused argument, for compatibility with 'plot'.}
  \item{col.pal}{a color palette to be used to represent weights using
   colors on the edges of the graph. See \code{?num2col}. Note that the
   palette is inversed by default.}
  \item{i,j, drop}{\code{i} is a vector used for subsetting the object. For
    instance, \code{i=1:3} will retain only the first three haplotypes of the
    genealogy. \code{j} and \code{drop} are only provided for compatibility,
    but not used.}
  \item{best, prox.mat}{arguments to be passed to the
    \code{\link{seqTrack}} function. See documentation of
    \code{\link{seqTrack}} for more information.}
  \item{annot,date.range,col,bg,add}{arguments to be passed to \code{\link{plotSeqTrack}}.}
  \item{n}{an integer indicating the number of haplotypes to be retained
    in the sample}
  % \item{from, to}{arguments of the conversion function, for converting a
  %   \code{haploGen} object into a \code{graphNEL-class}.}
  \item{tz, origin}{ aguments to be passed to \code{\link{as.POSIXct}}
    (see ?as.POSIXct)}
  \item{\dots}{further arguments to be passed to other methods; for
  'plot', arguments are passed to \code{plot.igraph}.}
  % \item{strict}{a logical used for compatibility with \code{as} generic
  %   function, but not used in the conversion. See \code{\link{setAs}} for
  %   more information.}
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk}}
\references{
  Jombart T, Eggo R, Dodd P, Balloux F (2010) Reconstructing disease
  outbreaks from genetic data: a graph approach. Heredity. doi:
  10.1038/hdy.2010.78.
}
\value{
  === haploGen class ===\cr
  \code{haploGen} objects are lists containing the following slots:\cr
  - seq: DNA sequences in the DNAbin matrix format\cr
  - dates: dates of appearance of the haplotypes\cr
  - ances: a vector of integers giving the index of each haplotype's
  ancestor\cr
  - id: a vector of integers giving the index of each haplotype\cr
  - xy: (optional) a matrix of spatial coordinates of haplotypes\cr
  - call: the matched call


  === misc functions ===\cr
  - as.POSIXct: returns a vector of dates with POSIXct format\cr
  - labels: returns the labels of the haplotypes\cr
  - as.seqTrack: returns a seqTrack object. Note that this object is not a
  proper seqTrack analysis, but just a format conversion convenient for
  plotting \code{haploGen} objects.
}
\details{
  === Dependencies with other packages ===\cr
  - ape package is required as it implements efficient handling of DNA
  sequences used in \code{haploGen} objects. To install this package,
  simply type:\cr
  \code{install.packages("ape")}

  - for various purposes including plotting, converting genealogies to
  graphs can be useful. From adegenet version 1.3-5 onwards, this is
  achieved using the package \code{igraph}. See below.

  === Converting haploGen objects to graphs ===\cr
  \code{haploGen} objects can be converted to \code{igraph}
  objects (package \code{igraph}), which can in turn be plotted and manipulated using classical
  graph tools. Simply use 'as.igraph(x)' where 'x' is a
  \code{haploGen} object. This functionality requires the \code{igraph}
  package. Graphs are time oriented (top=old, bottom=recent).
}
\seealso{
 \code{simOutbreak} in the package 'outbreaker' for simulating disease
  outbreaks under a realistic epidemiological model.
}
\examples{
\dontrun{
if(require(ape) && require(igraph)){
## PERFORM SIMULATIONS
x <- haploGen(geo.sim=TRUE)
x

## PLOT DATA
plot(x)

## PLOT SPATIAL SPREAD
plotHaploGen(x, bg="white")
title("Spatial dispersion")


## USE SEQTRACK RECONSTRUCTION
x.recons <- seqTrack(x)
mean(x.recons$ances==x$ances, na.rm=TRUE) # proportion of correct reconstructions

g <- as.igraph(x)
g
plot(g)
plot(g, vertex.size=0)


}
}
}

\encoding{UTF-8}
\name{DAPC cross-validation}
\alias{xvalDapc}
\alias{xvalDapc.default}
\alias{xvalDapc.data.frame}
\alias{xvalDapc.matrix}
\alias{xvalDapc.genlight}
\alias{xvalDapc.genind}
\title{Cross-validation for Discriminant Analysis of Principal Components (DAPC)}
\description{
  The function \code{xvalDapc} performs stratified cross-validation of DAPC 
  using varying numbers of PCs (and keeping the number of discriminant
  functions fixed); \code{xvalDapc} is a generic with methods for
  \code{data.frame} and \code{matrix}.\cr
}
\usage{
xvalDapc(x, \dots)

\method{xvalDapc}{default}(x, grp, n.pca.max = 300, n.da = NULL,
              training.set = 0.9, result = c("groupMean", "overall"),
              center = TRUE, scale = FALSE,
              n.pca=NULL, n.rep = 30, xval.plot = TRUE, \dots)

\method{xvalDapc}{data.frame}(x, grp, n.pca.max = 300, n.da = NULL,
              training.set = 0.9, result = c("groupMean", "overall"),
              center = TRUE, scale = FALSE,
              n.pca=NULL, n.rep = 30, xval.plot = TRUE, \dots)

\method{xvalDapc}{matrix}(x, grp, n.pca.max = 300, n.da = NULL,
              training.set = 0.9, result = c("groupMean", "overall"),
              center = TRUE, scale = FALSE,
              n.pca=NULL, n.rep = 30, xval.plot = TRUE, \dots)

\method{xvalDapc}{genlight}(x, \dots)

\method{xvalDapc}{genind}(x, \dots)
}
\arguments{
  \item{x}{\code{a data.frame} or a \code{matrix} used as input of DAPC.}
  \item{grp}{a \code{factor} indicating the group membership of
    individuals.}
  \item{n.pca.max}{maximum number of PCA components to retain.}
  \item{n.da}{an \code{integer} indicating the number of axes retained in the
    Discriminant Analysis step. If \code{NULL}, n.da defaults to 1 less than
    the number of groups.}
  \item{training.set}{the proportion of data (individuals) to be used
    for the training set; defaults to 0.9 if all groups have >= 10 members; otherwise,
    training.set scales automatically to the largest proportion that still 
    ensures all groups will be present in both training and validation sets.}
  \item{result}{a character string; "groupMean" for group-wise assignment
    sucess, or  "overall" for an overall mean assignment success; see details.}
  \item{center}{a \code{logical} indicating whether variables should be centred to
    mean 0 (TRUE, default) or not (FALSE). Always TRUE for \linkS4class{genind} objects.}
  \item{scale}{a \code{logical} indicating whether variables should be scaled
    (TRUE) or not (FALSE, default). Scaling consists in dividing variables by their
    (estimated) standard deviation to account for trivial differences in
    variances.}
  \item{n.pca}{an \code{integer} vector indicating the number of
    different number of PCA axes to be retained for the cross
    validation; if \code{NULL}, this will be dertermined automatically.}
  \item{n.rep}{the number of replicates to be carried out at each level of
    PC retention; defaults to 30.}
  \item{xval.plot}{a logical indicating whether a plot of the cross-validation
    results should be generated.}
  \item{\dots}{further arguments to be passed to \code{\link[boot]{boot}}.
    \strong{see Details.}} }
\details{ The Discriminant Analysis of Principal Components (DAPC) relies on
  dimension reduction of the data using PCA followed by a linear discriminant
  analysis. How many PCA axes to retain is often a non-trivial question. Cross
  validation provides an objective way to decide how many axes to retain:
  different numbers are tried and the quality of the corresponding DAPC is
  assessed by cross- validation: DAPC is performed on a training set, typically
  made of 90\% of the observations (comprising 90\% of the observations in each
  subpopulation) , and then used to predict the groups of the 10\% of remaining
  observations. The current method uses the average prediction success per group
  (result="groupMean"), or the overall prediction success (result="overall").
  The number of PCs associated with the lowest Mean Squared Error is then
  retained in the DAPC.
  \subsection{Parallel Computing}{
    The permutation of the data for cross-validation is performed in part by the
    function\code{\link[boot]{boot}}. If you have a modern computer, it is
    likely that you have multiple cores on your system. R by default utilizes
    only one of these cores unless you tell it otherwise. For details, please
    see the documentation of \code{\link[boot]{boot}}. Basically, if you want to
    use multiple cores, you need two arguments:
    \enumerate{
      \item \code{parallel} - what R parallel system to use (see below)
      \item \code{ncpus} - number of cores you want to use
    } 
    If you are on a unix system (Linux or OSX), you will want to specify
    \code{parallel = "multicore"}. If you are on Windows, you will want to
    specify \code{parallel = "snow"}.
  } 
}
\value{
 A \code{list} containing seven items, and a \code{plot} of the results. The
 first is a \code{data.frame} with two columns, the first giving the number of
 PCs of PCA retained in the corresponding DAPC, and the second giving the
 proportion of successful group assignment for each replicate. The second item
 gives the mean and confidence interval for random chance. The third gives the
 mean successful assignment at each level of PC retention. The fourth indicates
 which number of PCs is associated with the highest mean success. The fifth
 gives the Root Mean Squared Error at each level of PC retention. The sixth
 indicates which number of PCs is associated with the lowest MSE. The seventh
 item contains the DAPC carried out with the optimal number of PCs, determined
 with reference to MSE.
 
 If \code{xval.plot=TRUE} a scatterplot of the results of cross-validation 
 will be displayed. 
 
}
\references{
  Jombart T, Devillard S and Balloux F  (2010) Discriminant analysis of
  principal components: a new method for the analysis of genetically
  structured populations. BMC Genetics11:94. doi:10.1186/1471-2156-11-94
}
\seealso{
  \code{\link{dapc}}
}
\author{ Caitlin Collins \email{caitlin.collins12@imperial.ac.uk},
Thibaut Jombart \email{t.jombart@imperial.ac.uk},
Zhian N. Kamvar \email{kamvarz@science.oregonstate.edu} }
\examples{
\dontrun{
## CROSS-VALIDATION ##
data(sim2pop)
xval <- xvalDapc(sim2pop@tab, pop(sim2pop), n.pca.max=100, n.rep=3)
xval

## 100 replicates ##

# Serial version (SLOW!)
system.time(xval <- xvalDapc(sim2pop@tab, pop(sim2pop), n.pca.max=100, n.rep=100))

# Parallel version (faster!)
system.time(xval <- xvalDapc(sim2pop@tab, pop(sim2pop), n.pca.max=100, n.rep=100, 
                             parallel = "multicore", ncpus = 2))
}
}
\keyword{multivariate}

\encoding{UTF-8}
\name{dapc}
\alias{dapc}
\alias{dapc.data.frame}
\alias{dapc.matrix}
\alias{dapc.genind}
\alias{dapc.dudi}
\alias{dapc.genlight}
\alias{print.dapc}
\alias{summary.dapc}
\alias{predict.dapc}
\alias{as.lda}
\alias{as.lda.dapc}
\title{Discriminant Analysis of Principal Components (DAPC)}
\description{
  These functions implement the Discriminant Analysis of Principal
  Components (DAPC, Jombart et al. 2010). This method descibes the
  diversity between pre-defined groups. When groups are unknown, use
  \code{find.clusters} to infer genetic clusters. See 'details' section
  for a succint description of the method, and
  \code{vignette("adegenet-dapc")} for a tutorial. Graphical methods for
  DAPC are documented in \code{\link{scatter.dapc}} (see
  \code{?scatter.dapc}).

 \code{dapc} is a generic function performing the DAPC on the following
 types of objects:\cr
 - \code{data.frame} (only numeric data)\cr
 - \code{matrix} (only numeric data)\cr
 - \code{\linkS4class{genind}} objects (genetic markers)\cr
 - \code{\linkS4class{genlight}} objects (genome-wide SNPs)

 These methods all return an object with class \code{dapc}.

 Functions that can be applied to these objects are (the ".dapc" can be
 ommitted):

  - \code{print.dapc}: prints the content of a \code{dapc} object.\cr
  - \code{summary.dapc}: extracts useful information from a \code{dapc}
  object.\cr
  - \code{predict.dapc}: predicts group memberships based on DAPC results.\cr
  - \code{xvalDapc}: performs cross-validation of DAPC using varying
  numbers of PCs (and keeping the number of discriminant functions
  fixed); it currently has methods for \code{data.frame} and \code{matrix}.\cr



  DAPC implementation calls upon \code{\link[ade4]{dudi.pca}} from the
  \code{ade4} package (except for \linkS4class{genlight} objects)
  and \code{\link[MASS]{lda}} from the \code{MASS} package. The
  \code{predict} procedure uses \code{\link[MASS]{predict.lda}} from the
  \code{MASS} package.

  \code{as.lda} is a generic with a method for \code{dapc} object which
  converts these objects into outputs similar to that of
  \code{lda.default}.
}
\usage{
\method{dapc}{data.frame}(x, grp, n.pca=NULL, n.da=NULL, center=TRUE,
     scale=FALSE, var.contrib=TRUE, var.loadings=FALSE, pca.info=TRUE,
     pca.select=c("nbEig","percVar"), perc.pca=NULL, \ldots, dudi=NULL)

\method{dapc}{matrix}(x, \ldots)

\method{dapc}{genind}(x, pop=NULL, n.pca=NULL, n.da=NULL, scale=FALSE,
     truenames=TRUE, var.contrib=TRUE, var.loadings=FALSE, pca.info=TRUE,
     pca.select=c("nbEig","percVar"), perc.pca=NULL, \ldots)

\method{dapc}{genlight}(x, pop=NULL, n.pca=NULL, n.da=NULL,
   scale=FALSE, var.contrib=TRUE, var.loadings=FALSE, pca.info=TRUE,
   pca.select=c("nbEig", "percVar"), perc.pca=NULL, glPca=NULL, \ldots)

\method{dapc}{dudi}(x, grp, \ldots)

\method{print}{dapc}(x, \dots)

\method{summary}{dapc}(object, \dots)

\method{predict}{dapc}(object, newdata, prior = object$prior, dimen,
         method = c("plug-in", "predictive", "debiased"), ...)
}
\arguments{
  \item{x}{\code{a data.frame}, \code{matrix}, or \code{\linkS4class{genind}}
    object. For the \code{data.frame} and \code{matrix} arguments, only
    quantitative variables should be provided.}
  \item{grp,pop}{a \code{factor} indicating the group membership of
    individuals; for \code{scatter}, an optional grouping of individuals.}
  \item{n.pca}{an \code{integer} indicating the number of axes retained in the
    Principal Component Analysis (PCA) step. If \code{NULL}, interactive selection is triggered.}
  \item{n.da}{an \code{integer} indicating the number of axes retained in the
    Discriminant Analysis step. If \code{NULL}, interactive selection is triggered.}
  \item{center}{a \code{logical} indicating whether variables should be centred to
    mean 0 (TRUE, default) or not (FALSE). Always TRUE for \linkS4class{genind} objects.}
  \item{scale}{a \code{logical} indicating whether variables should be scaled
    (TRUE) or not (FALSE, default). Scaling consists in dividing variables by their
    (estimated) standard deviation to account for trivial differences in
    variances.}
  \item{var.contrib}{a \code{logical} indicating whether the
    contribution of original variables (alleles, for \linkS4class{genind} objects)
    should be provided (TRUE, default) or not (FALSE). Such output can be useful,
    but can also create huge matrices when there is a lot of variables.}
  \item{var.loadings}{a \code{logical} indicating whether the
    loadings of original variables (alleles, for \linkS4class{genind} objects)
    should be provided (TRUE) or not (FALSE, default). Such output can be useful,
    but can also create huge matrices when there is a lot of variables.}
  \item{pca.info}{a \code{logical} indicating whether information about
    the prior PCA should be stored (TRUE, default) or not (FALSE). This
    information is required to predict group membership of new
    individuals using \code{predict}, but makes the object slightly bigger.}
  \item{pca.select}{a \code{character} indicating the mode of selection of PCA
    axes, matching either "nbEig" or "percVar". For "nbEig", the user
    has to specify the number of axes retained (interactively, or via
    \code{n.pca}). For "percVar", the user has to specify the minimum amount of
    the total variance to be preserved by the retained axes, expressed as a
    percentage (interactively, or via \code{perc.pca}).  }
  \item{perc.pca}{a \code{numeric} value between 0 and 100 indicating the
    minimal percentage of the total variance of the data to be expressed by the
    retained axes of PCA.}
  \item{\ldots}{further arguments to be passed to other functions. For
    \code{dapc.matrix}, arguments are to match those of
    \code{dapc.data.frame}; for \code{dapc.genlight}, arguments passed
    to \code{\link{glPca}}}
  \item{glPca}{an optional \code{\link{glPca}} object; if provided,
    dimension reduction is not performed (saving computational time) but
    taken directly from this object.}
  \item{object}{a \code{dapc} object.}
  \item{truenames}{a \code{logical} indicating whether true (i.e., user-specified)
    labels should be used in object outputs (TRUE, default) or not (FALSE).}
  \item{dudi}{optionally, a multivariate analysis with the class
    \code{dudi} (from the ade4 package). If provided, prior PCA will be
    ignored, and this object will be used as a prior step for variable
    orthogonalisation.}
  \item{newdata}{an optional dataset of individuals whose membership is
    seeked; can be a data.frame, a matrix, a \linkS4class{genind} or a
    \linkS4class{genlight} object, but object class must match the
    original ('training') data. In particular, variables must be exactly
    the same as in the original data. For  \linkS4class{genind}
    objects, see \code{\link{repool}} to ensure matching of alleles.}
  \item{prior,dimen,method}{see \code{?predict.lda}.}
}
\details{
  The Discriminant Analysis of Principal Components (DAPC) is designed
  to investigate the genetic structure of biological populations. This
  multivariate method consists in a two-steps procedure. First, genetic
  data are transformed (centred, possibly scaled) and submitted to a
  Principal Component Analysis (PCA). Second, principal components of
  PCA are submitted to a Linear Discriminant Analysis (LDA). A trivial
  matrix operation allows to express discriminant functions as linear
  combination of alleles, therefore allowing one to compute allele
  contributions. More details about the computation of DAPC are to be
  found in the indicated reference.

  DAPC does not infer genetic clusters ex nihilo; for this, see the
  \code{\link{find.clusters}} function.
}
\value{
  === dapc objects ===\cr
  The class \code{dapc} is a list with the following
  components:\cr
  \item{call}{the matched call.}
  \item{n.pca}{number of PCA axes retained}
  \item{n.da}{number of DA axes retained}
  \item{var}{proportion of variance conserved by PCA principal components}
  \item{eig}{a numeric vector of eigenvalues.}
  \item{grp}{a factor giving prior group assignment}
  \item{prior}{a numeric vector giving prior group probabilities}
  \item{assign}{a factor giving posterior group assignment}
  \item{tab}{matrix of retained principal components of PCA}
  \item{loadings}{principal axes of DAPC, giving coefficients of the linear
    combination of retained PCA axes.}
  \item{ind.coord}{principal components of DAPC, giving the coordinates
    of individuals onto principal axes of DAPC; also called the
    discriminant functions.}
  \item{grp.coord}{coordinates of the groups onto the principal axes of DAPC.}
  \item{posterior}{a data.frame giving posterior membership probabilities for
    all individuals and all clusters.}
  \item{var.contr}{(optional) a data.frame giving the contributions of original
    variables (alleles in the case of genetic data) to the principal components
    of DAPC.}
  \item{var.load}{(optional) a data.frame giving the loadings of original
    variables (alleles in the case of genetic data) to the principal components
    of DAPC.}
  \item{match.prp}{a list, where each item is the proportion of individuals 
    correctly matched to their original population in cross-validation.}

  
  === other outputs ===\cr
  Other functions have different outputs:\cr
  - \code{summary.dapc} returns a list with 6 components: \code{n.dim} (number
  of retained DAPC axes), \code{n.pop} (number of groups/populations),
  \code{assign.prop} (proportion of overall correct assignment),
  \code{assign.per.pop} (proportion of correct assignment per group),
  \code{prior.grp.size} (prior group sizes), and \code{post.grp.size} (posterior
  group sizes),  \code{xval.dapc}, \code{xval.genind} and \code{xval}
  (all return a list of four lists, each one with as many items as
  cross-validation runs.  The first item is a list of \code{assign}
  components, the secon is a list of \code{posterior} components, the
  thirs is a list of \code{ind.score} components and the fourth is a
  list of \code{match.prp} items, i.e. the prortion of the validation
  set correctly matched to its original population)
}
\references{
  Jombart T, Devillard S and Balloux F  (2010) Discriminant analysis of
  principal components: a new method for the analysis of genetically
  structured populations. BMC Genetics11:94. doi:10.1186/1471-2156-11-94
}
\seealso{
  \itemize{
  \item \code{\link{xvalDapc}}: selection of the optimal numbers of PCA axes
  retained in DAPC using cross-validation.

  \item \code{\link{scatter.dapc}}, \code{\link{assignplot}},
  \code{\link{compoplot}}: graphics for DAPC.

  \item \code{\link{find.clusters}}: to identify clusters without prior.

  \item \code{\link{dapcIllus}}: a set of simulated data illustrating
  the DAPC

  \item \code{\link{eHGDP}}, \code{\link{H3N2}}: empirical datasets
  illustrating DAPC
  }
}
\author{ Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
## data(dapcIllus), data(eHGDP), and data(H3N2) illustrate the dapc
## see ?dapcIllus, ?eHGDP, ?H3N2
##
\dontrun{
example(dapcIllus)
example(eHGDP)
example(H3N2)
}

## H3N2 EXAMPLE ##
data(H3N2)
pop(H3N2) <- factor(H3N2$other$epid)
dapc1 <- dapc(H3N2, var.contrib=FALSE, scale=FALSE, n.pca=150, n.da=5)

## remove internal segments and ellipses, different pch, add MStree
scatter(dapc1, cell=0, pch=18:23, cstar=0, mstree=TRUE, lwd=2, lty=2)

## label individuals at the periphery
# air = 2 is a measure of how much space each label needs
# pch = NA suppresses plotting of points
scatter(dapc1, label.inds = list(air = 2, pch = NA))

## only ellipse, custom labels
scatter(dapc1, cell=2, pch="", cstar=0, posi.da="top",
        label=paste("year\n",2001:2006), axesel=FALSE, col=terrain.colors(10))


## SHOW COMPOPLOT ON MICROBOV DATA ##
data(microbov)
dapc1 <- dapc(microbov, n.pca=20, n.da=15)
compoplot(dapc1, lab="")




\dontrun{
## EXAMPLE USING GENLIGHT OBJECTS ##
## simulate data
x <- glSim(50,4e3-50, 50, ploidy=2)
x
plot(x)

## perform DAPC
dapc1 <- dapc(x, n.pca=10, n.da=1)
dapc1

## plot results
scatter(dapc1, scree.da=FALSE)

## SNP contributions
loadingplot(dapc1$var.contr)
loadingplot(tail(dapc1$var.contr, 100), main="Loading plot - last 100 SNPs")



## USE "PREDICT" TO PREDICT GROUPS OF NEW INDIVIDUALS ##
## load data
data(sim2pop)

## we make a dataset of:
## 30 individuals from pop A
## 30 individuals from pop B
## 30 hybrids

## separate populations and make F1
temp <- seppop(sim2pop)
temp <- lapply(temp, function(e) hybridize(e,e,n=30)) # force equal popsizes

## make hybrids
hyb <- hybridize(temp[[1]], temp[[2]], n=30)

## repool data - needed to ensure allele matching
newdat <- repool(temp[[1]], temp[[2]], hyb)
pop(newdat) <- rep(c("pop A", "popB", "hyb AB"), c(30,30,30))

## perform the DAPC on the first 2 pop (60 first indiv)
dapc1 <- dapc(newdat[1:60],n.pca=5,n.da=1)

## plot results
scatter(dapc1, scree.da=FALSE)

## make prediction for the 30 hybrids
hyb.pred <- predict(dapc1, newdat[61:90])
hyb.pred

## plot the inferred coordinates (circles are hybrids)
points(hyb.pred$ind.scores, rep(.1, 30))

## look at assignment using assignplot
assignplot(dapc1, new.pred=hyb.pred)
title("30 indiv popA, 30 indiv pop B, 30 hybrids")

## image using compoplot
compoplot(dapc1, new.pred=hyb.pred, ncol=2)
title("30 indiv popA, 30 indiv pop B, 30 hybrids")

## CROSS-VALIDATION ##
data(sim2pop)
xval <- xvalDapc(sim2pop@tab, pop(sim2pop), n.pca.max=100, n.rep=3)
xval
boxplot(xval$success~xval$n.pca, xlab="Number of PCA components",
ylab="Classification succes", main="DAPC - cross-validation")

}


}
\keyword{multivariate}

\encoding{UTF-8}
\name{genind class}
\alias{genind-class}
\alias{print,genind-method}
\alias{show,genind-method}
\alias{names,genind-method}
\alias{summary,genind-method}
\alias{print,genindSummary-method}
\alias{print.genindSummary}
\alias{is.genind}
\alias{.valid.genind}
\title{adegenet formal class (S4) for individual genotypes}
\description{The S4 class \code{genind} is used to store individual genotypes.\cr
  It contains several components described in the 'slots' section).\cr
  The \code{summary} of a \code{genind} object invisibly returns a list of component.
  The function \code{.valid.genind} is for internal use.
  The function \code{genind} creates a genind object from a valid table
  of alleles corresponding to the \code{@tab} slot.
  Note that as in other S4 classes, slots are accessed using @ instead
  of \$.
}
\section{Slots}{
  \describe{
    \item{\code{tab}:}{
      matrix integers containing genotypes data for
      individuals (in rows) for all alleles (in columns). The table
      differs depending on the \code{@type} slot:\cr
      - 'codom': values
      are numbers of alleles, summing up to the
      individuals' ploidies.\cr
      - 'PA': values are presence/absence of alleles.\cr
      In all cases, rows and columns are given generic names.}
    \item{\code{loc.fac}:}{locus factor for the columns of \code{tab}}
    \item{\code{loc.n.all}:}{integer vector giving the number of alleles per locus}
    \item{\code{all.names}:}{list having one component per locus, each containing a character vector of alleles names}
    \item{\code{ploidy}:}{ an integer indicating the degree of ploidy of
      the genotypes. Beware: 2 is not an integer, but as.integer(2) is.}
    \item{\code{type}:}{ a character string indicating the type of
      marker: 'codom' stands for 'codominant' (e.g. microstallites,
      allozymes); 'PA' stands for 'presence/absence' (e.g. AFLP).}
    \item{\code{call}:}{the matched call}
    \item{\code{strata}:}{(optional) data frame giving levels of population stratification for each individual}
    \item{\code{hierarchy}:}{(optional) a hierarchical \code{\link{formula}} defining the hierarchical levels in the \code{@@strata} slot.}
    \item{\code{pop}:}{(optional) factor giving the population of each individual}
    \item{\code{other}:}{(optional) a list containing other information}
  }
}

\section{Extends}{
Class \code{"\linkS4class{gen}"}, directly.
Class \code{"\linkS4class{indInfo}"}, directly.
}
\section{Methods}{
  \describe{
    \item{names}{\code{signature(x = "genind")}: give the names of the
      components of a genind object}
    \item{print}{\code{signature(x = "genind")}: prints a genind object}
    \item{show}{\code{signature(object = "genind")}: shows a genind
      object (same as print)}
    \item{summary}{\code{signature(object = "genind")}: summarizes a
      genind object, invisibly returning its content or suppress printing of auxiliary information by specifying \code{verbose = FALSE}}
  }
}
\seealso{\code{\link{as.genind}}, \code{\link{genind2genpop}},
  \code{\link{genpop}}, \code{\link{import2genind}},
  \code{\link{read.genetix}}, \code{\link{read.genepop}},
  \code{\link{read.fstat}}\cr

  Related classes:\cr
  - \linkS4class{genpop} for storing data per populations\cr

  - \linkS4class{genlight} for an efficient storage of binary SNPs genotypes\cr
}
\author{ Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
showClass("genind")

obj <- read.genetix(system.file("files/nancycats.gtx",package="adegenet"))
obj
validObject(obj)
summary(obj)

\dontrun{
# test inter-colonies structuration
if(require(hierfstat)){
gtest <- gstat.randtest(obj,nsim=99)
gtest
plot(gtest)
}

# perform a between-class PCA
pca1 <- dudi.pca(scaleGen(obj, NA.method="mean"),scannf=FALSE,scale=FALSE)
pcabet1 <- between(pca1,obj@pop,scannf=FALSE)
pcabet1

s.class(pcabet1$ls,obj@pop,sub="Inter-class PCA",possub="topleft",csub=2)
add.scatter.eig(pcabet1$eig,2,xax=1,yax=2)

}
}
\keyword{classes}
\keyword{manip}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auxil.R
\name{adegenetWeb}
\alias{adegenetWeb}
\alias{adegenetTutorial}
\alias{adegenetIssues}
\title{Functions to access online resources for adegenet}
\usage{
adegenetWeb()

adegenetTutorial(which = c("basics", "spca", "dapc", "genomics", "strata",
  "snapclust"))

adegenetIssues()
}
\arguments{
\item{which}{a character string indicating which tutorial to open (see details)}
}
\description{
These functions simply open websites or documents
available online providing resources for adegenet.
}
\details{
\itemize{
\item adegenetWeb opens adegenet's website
\item adegenetTutorial opens adegenet tutorials
\item adegenetIssues opens the issue page on github;
this is used to report a bug or post a feature request.
}



Available tutorials are:
\itemize{
\item 'basics': general introduction to adegenet; covers basic data structures, import/export, handling, and a number of population genetics methods
\item 'spca': spatial genetic structures using the spatial Principal Component Analysis
\item 'dapc': population structure using the Discriminant Analysis of Principal Components
\item 'genomics': handling large genome-wide SNP data using adegenet
\item 'strata': introduction to hierarchical population structure in adegenet
\item 'snapclust': introduction to fast maximum-likelihood genetic clustering using snapclust
}
}

\encoding{UTF-8}
\name{findMutations}
\alias{findMutations}
\alias{findMutations.DNAbin}
\alias{graphMutations}
\alias{graphMutations.DNAbin}
\title{Identify mutations between DNA sequences}
\description{
  The function \code{findMutations} identifies mutations (position and
  nature) of pairs of aligned DNA sequences. The function
  \code{graphMutations} does the same thing but plotting mutations on a
  directed graph.\cr

  Both functions are generics, but the only methods implemented in
  adegenet so far is for \code{\link[ape]{DNAbin}} objects.
}
\usage{
findMutations(\dots)

\method{findMutations}{DNAbin}(x, from=NULL, to=NULL, allcomb=TRUE, \dots)

graphMutations(\dots)

\method{graphMutations}{DNAbin}(x, from=NULL, to=NULL, allcomb=TRUE, plot=TRUE,
               curved.edges=TRUE, \dots)

}
\arguments{
  \item{x}{a \code{DNAbin} object containing aligned sequences, as a matrix.}
  \item{from}{a vector indicating the DNA sequences from which mutations
    should be found. If \code{NULL}, all sequences are considered (i.e.,
    \code{1:nrow(x)}).}
  \item{to}{a vector indicating the DNA sequences to which mutations
    should be found. If \code{NULL}, all sequences are considered (i.e.,
    \code{1:nrow(x)}).}
  \item{allcomb}{a logical indicating whether all combinations of
    sequences (from and to) should be considered (TRUE, default), or not
    (FALSE).}
  \item{plot}{a logical indicating whether the graph should be plotted.}
  \item{curved.edges}{a logical indicating whether the edges of the
    graph should be curved.}
  \item{\dots}{further arguments to be passed to other methods. Used in
    \code{graphMutations} where it is passed to the plot method for
    \code{igraph} objects.}
}
\value{
  For \code{findMutations}, a named list indicating the mutations from
  one sequence to another. For each comparison, a three-column matrix is
  provided, corresponding to the nucleotides in first and second
  sequence, and a summary of the mutation provided as:
  [position]:[nucleotide in first sequence]->[nucleotide in second
  sequence].

  For \code{graphMutations}, a graph with the class \code{igraph}.
}
\seealso{
  The \code{\link{fasta2DNAbin}} to read fasta alignments with minimum
  RAM use.
}
\author{
  Thibaut Jombart \email{t.jombart@imperial.ac.uk}.
 }
\examples{
\dontrun{
data(woodmouse)

## mutations between first 3 sequences
findMutations(woodmouse[1:3,])

## mutations from the first to sequences 2 and 3
findMutations(woodmouse[1:3,], from=1)

## same, graphical display
g <- graphMutations(woodmouse[1:3,], from=1)

## some manual checks
as.character(woodmouse)[1:3,35]
as.character(woodmouse)[1:3,36]
as.character(woodmouse)[1:3,106]

}
}

\name{isPoly-methods}
\docType{methods}
\alias{isPoly}
\alias{isPoly-methods}
\alias{isPoly,genind-method}
\alias{isPoly,genpop-method}
\title{ Assess polymorphism in genind/genpop objects }
\description{
  The simple function \code{isPoly} can be used to check which loci are
  polymorphic, or alternatively to check which alleles give rise to polymorphism.
}
\usage{
\S4method{isPoly}{genind}(x, by=c("locus","allele"), thres=1/100)
\S4method{isPoly}{genpop}(x, by=c("locus","allele"), thres=1/100)
}
\arguments{
  \item{x}{a \linkS4class{genind} and \linkS4class{genpop} object}
  \item{by}{a character being "locus" or "allele", indicating whether
    results should indicate polymorphic loci ("locus"), or alleles giving
    rise to polymorphism ("allele").}
  \item{thres}{a numeric value giving the minimum frequency of an allele
  giving rise to polymorphism (defaults to 0.01).}
 }
 \value{
   A vector of logicals.
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
\dontrun{
data(nancycats)
isPoly(nancycats,by="loc", thres=0.1)
isPoly(nancycats[1:3],by="loc", thres=0.1)
genind2df(nancycats[1:3])
}
}
\keyword{methods}
\keyword{manip}

\encoding{UTF-8}
\name{propShared}
\alias{propShared}
\title{Compute proportion of shared alleles}
\description{The function \code{propShared} computes the proportion of
  shared alleles in a set of genotypes (i.e. from a \linkS4class{genind}
  object). Current implementation works for any level of ploidy.
}
\usage{
propShared(obj)
}
\arguments{
  \item{obj}{a \linkS4class{genind} object.}
 }
 \details{
   Computations of the numbers of shared alleles are done in C.
   Proportions are computed from all available data, i.e. proportion can
   be computed as far as there is at least one typed locus in common
   between two genotypes.
}
\value{Returns a matrix of proportions}
\seealso{\code{\link{dist.genpop}}
}
\author{ Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{

\dontrun{
## make a small object
data(microbov)
obj <- microbov[1:5, loc = locNames(microbov)[1:2]]

## verify results
propShared(obj)
genind2df(obj,sep="|")

## Use this similarity measure inside a PCoA
##  ! This is for illustration only !
## the distance should be rendered Euclidean before
## (e.g. using cailliez from package ade4).
matSimil <- propShared(microbov)
matDist <- exp(-matSimil)
D <- cailliez(as.dist(matDist))
pcoa1 <- dudi.pco(D,scannf=FALSE,nf=3)
s.class(pcoa1$li,microbov$pop,lab=popNames(microbov))

}
}
\keyword{manip}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{eHGDP}
\alias{eHGDP}
\title{Extended HGDP-CEPH dataset}
\format{\code{eHGDP} is a genind object with a data frame named
\code{popInfo} as supplementary component (\code{eHGDP@other$popInfo}),
which contains the following variables: \describe{ \item{Population: }{a
character vector indicating populations.} \item{Region: }{a character vector
indicating the geographic region of each population.} \item{Label: }{a
character vector indicating the correspondence with population labels used
in the genind object (i.e., as output by \code{pop(eHGDP)}).}
\item{Latitude,Longitude: }{geographic coordinates of the populations,
indicated as north and east degrees.} }}
\source{
Original panel by Human Genome Diversity Project (HGDP) and Centre
d'Etude du Polymorphisme Humain (CEPH). See reference [4] for Native
American populations.

This copy of the dataset was prepared by Francois Balloux.
}
\description{
This dataset consists of 1350 individuals from native Human populations
distributed worldwide typed at 678 microsatellite loci. The original
HGDP-CEPH panel [1-3] has been extended by several native American
populations [4]. This dataset was used to illustrate the Discriminant
Analysis of Principal Components (DAPC, [5]).
}
\examples{

\dontrun{
## LOAD DATA
data(eHGDP)
eHGDP


## PERFORM DAPC - USE POPULATIONS AS CLUSTERS
## to reproduce exactly analyses from the paper, use "n.pca=1000"
dapc1 <- dapc(eHGDP, all.contrib=TRUE, scale=FALSE,
n.pca=200, n.da=80) # takes 2 minutes
dapc1

## (see ?dapc for details about the output)



## SCREEPLOT OF EIGENVALUES
barplot(dapc1$eig, main="eHGDP - DAPC eigenvalues",
col=c("red","green","blue", rep("grey", 1000)))



## SCATTERPLOTS
## (!) Note: colors may be inverted with respect to [5]
## as signs of principal components are arbitrary
## and change from one computer to another
##
## axes 1-2
s.label(dapc1$grp.coord[,1:2], clab=0, sub="Axes 1-2")
par(xpd=T)
colorplot(dapc1$grp.coord[,1:2], dapc1$grp.coord, cex=3, add=TRUE)
add.scatter.eig(dapc1$eig,10,1,2, posi="bottomright", ratio=.3, csub=1.25)

## axes 2-3
s.label(dapc1$grp.coord[,2:3], clab=0, sub="Axes 2-3")
par(xpd=T)
colorplot(dapc1$grp.coord[,2:3], dapc1$grp.coord, cex=3, add=TRUE)
add.scatter.eig(dapc1$eig,10,1,2, posi="bottomright", ratio=.3, csub=1.25)



## MAP DAPC1 RESULTS
if(require(maps)){

xy <- cbind(eHGDP$other$popInfo$Longitude, eHGDP$other$popInfo$Latitude)

par(mar=rep(.1,4))
map(fill=TRUE, col="lightgrey")
colorplot(xy, -dapc1$grp.coord, cex=3, add=TRUE, trans=FALSE)
}



## LOOK FOR OTHER CLUSTERS
## to reproduce results of the reference paper, use :
## grp <- find.clusters(eHGDP, max.n=50, n.pca=200, scale=FALSE)
## and then
## plot(grp$Kstat, type="b", col="blue")

grp <- find.clusters(eHGDP, max.n=30, n.pca=200,
scale=FALSE, n.clust=4) # takes about 2 minutes
names(grp)

## (see ?find.clusters for details about the output)



## PERFORM DAPC - USE POPULATIONS AS CLUSTERS
## to reproduce exactly analyses from the paper, use "n.pca=1000"
dapc2 <- dapc(eHGDP, pop=grp$grp, all.contrib=TRUE,
scale=FALSE, n.pca=200, n.da=80) # takes around a 1 minute
dapc2


## PRODUCE SCATTERPLOT
scatter(dapc2) # axes 1-2
scatter(dapc2,2,3) # axes 2-3


## MAP DAPC2 RESULTS
if(require(maps)){
xy <- cbind(eHGDP$other$popInfo$Longitude,
eHGDP$other$popInfo$Latitude)

myCoords <- apply(dapc2$ind.coord, 2, tapply, pop(eHGDP), mean)

par(mar=rep(.1,4))
map(fill=TRUE, col="lightgrey")
colorplot(xy, myCoords, cex=3, add=TRUE, trans=FALSE)
}

}

}
\references{
[1] Rosenberg NA, Pritchard JK, Weber JL, Cann HM, Kidd KK, et
al. (2002) Genetic structure of human populations. \emph{Science} 298:
2381-2385.

[2] Ramachandran S, Deshpande O, Roseman CC, Rosenberg NA, Feldman MW, et
al. (2005) Support from the relationship of genetic and geographic distance
in human populations for a serial founder effect originating in Africa.
\emph{Proc Natl Acad Sci U S A} 102: 15942-15947.

[3] Cann HM, de Toma C, Cazes L, Legrand MF, Morel V, et al. (2002) A human
genome diversity cell line panel.  \emph{Science} 296: 261-262.

[4] Wang S, Lewis CM, Jakobsson M, Ramachandran S, Ray N, et al. (2007)
Genetic Variation and Population Structure in Native Americans. \emph{PLoS
Genetics} 3: e185.

[5] Jombart, T., Devillard, S. and Balloux, F.  Discriminant analysis of
principal components: a new method for the analysis of genetically
structured populations. Submitted to \emph{BMC genetics}.
}
\keyword{datasets}

\encoding{UTF-8}
\name{SequencesToGenind}
\alias{DNAbin2genind}
\alias{alignment2genind}
\title{ Importing data from an alignement of sequences to a genind object}
\description{
  These functions take an alignement of sequences and translate SNPs
  into a \linkS4class{genind} object. Note that only polymorphic loci
  are retained.\cr

  Currently, accepted sequence formats are:\cr
  - DNAbin (ape package): function DNAbin2genind\cr
  - alignment (seqinr package): function alignment2genind\cr
}
\usage{
DNAbin2genind(x, pop=NULL, exp.char=c("a","t","g","c"), polyThres=1/100)

alignment2genind(x, pop=NULL, exp.char=c("a","t","g","c"), na.char="-",
                 polyThres=1/100)

}
\arguments{
 \item{x}{an object containing aligned sequences.}
 \item{pop}{an optional factor giving the population to which each sequence belongs.}
 \item{exp.char}{a vector of single character providing expected values;
  all other characters will be turned to NA.}
 \item{na.char}{a vector of single characters providing values that should be
   considered as NA. If not NULL, this is used instead of \code{exp.char}.}
 \item{polyThres}{the minimum frequency of a minor allele for a locus to
  be considered as polymorphic (defaults to 0.01).}
}

\value{an object of the class \linkS4class{genind}}

\seealso{\code{\link{import2genind}}, \code{\link{read.genetix}},
  \code{\link{read.fstat}}, \code{\link{read.structure}},
  \code{\link{read.genepop}}, \code{\link[ape]{DNAbin}},
  \code{\link[seqinr]{as.alignment}}.
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
\dontrun{
data(woodmouse)
x <- DNAbin2genind(woodmouse)
x
genind2df(x)
}

if(require(seqinr)){
mase.res   <- read.alignment(file=system.file("sequences/test.mase",package="seqinr"),
format = "mase")
mase.res
x <- alignment2genind(mase.res)
x
locNames(x) # list of polymorphic sites
genind2df(x)

## look at Euclidean distances
D <- dist(tab(x))
D

## summarise with a PCoA
pco1 <- dudi.pco(D, scannf=FALSE,nf=2)
scatter(pco1, posi="bottomright")
title("Principal Coordinate Analysis\n-based on proteic distances-")

}
}
\keyword{manip}

\encoding{UTF-8}
\name{fasta2DNAbin}
\alias{fasta2DNAbin}
\title{ Read large DNA alignments into R}
\description{
  The function \code{fasta2DNAbin} reads alignments with the fasta
  format (extensions ".fasta", ".fas", or ".fa"), and outputs a
  \code{\link[ape]{DNAbin}} object (the efficient DNA representation from the
  ape package). The output contains either the full alignments, or only
  SNPs. This implementation is designed for memory-efficiency,
  and can read in larger datasets than Ape's \code{\link[ape]{read.dna}}.

  The function reads data by chunks of a few genomes (minimum 1, no
  maximum) at a time, which allows one to read massive datasets with
  negligible RAM requirements (albeit at a cost of computational
  time). The argument \code{chunkSize} indicates the number of genomes
  read at a time. Increasing this value decreases the computational time
  required to read data in, while increasing memory requirements.

}
\usage{
fasta2DNAbin(file, quiet=FALSE, chunkSize=10, snpOnly=FALSE)
}
\arguments{
  \item{file}{ a character string giving the path to the file to
    convert, with the extension ".fa", ".fas", or ".fasta".

    Can also be a \link{connection} (which will be opened for reading if
    necessary, and if so \code{\link{close}}d (and hence destroyed) at the
    end of the function call).}
  \item{quiet}{a logical stating whether a conversion messages should be
    printed (FALSE, default) or not (TRUE).}
  \item{chunkSize}{an integer indicating the number of genomes to be
    read at a time; larger values require more RAM but decrease the time
    needed to read the data.}
  \item{snpOnly}{a logical indicating whether SNPs only should be returned.}
}
\value{an object of the class \code{\link[ape]{DNAbin}}}
\seealso{
  - \code{?DNAbin} for a description of the class \code{\link[ape]{DNAbin}}.

  - \code{\link{read.snp}}: read SNPs in adegenet's '.snp' format.

  - \code{\link{read.PLINK}}: read SNPs in PLINK's '.raw' format.

  - \code{\link{df2genind}}: convert any multiallelic markers into
  adegenet \linkS4class{genind}.

  - \code{\link{import2genind}}: read multiallelic markers from various
  software into adegenet.
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
\dontrun{
## show the example file ##
## this is the path to the file:
myPath <- system.file("files/usflu.fasta",package="adegenet")
myPath

## read the file
obj <- fasta2DNAbin(myPath, chunk=10) # process 10 sequences at a time
obj
}
}
\keyword{manip}

\encoding{UTF-8}
\name{HWE.test.genind}
\alias{HWE.test.genind}

\title{Hardy-Weinberg Equilibrium test for multilocus data}
\description{The function \code{HWE.test} is a generic function to
  perform Hardy-Weinberg Equilibrium tests defined by the
  \code{genetics} package. adegenet proposes a method for \code{genind}
  objects.\cr
   
  The output can be of two forms:\cr
  - a list of tests (class \code{htest}) for each locus-population
  combinaison \cr
  - a population x locus matrix containing p-values of the tests  
}
\usage{
\method{HWE.test}{genind}(x,pop=NULL,permut=FALSE,nsim=1999,hide.NA=TRUE,res.type=c("full","matrix"))
}
\arguments{
  \item{x}{an object of class \code{genind}.}
  \item{pop}{a factor giving the population of each individual. If NULL,
  pop is seeked from x\$pop.}
  \item{permut}{a logical passed to \code{HWE.test} stating whether
    Monte Carlo version (TRUE) should be used or not (FALSE, default).}
  \item{nsim}{number of simulations if Monte Carlo is used (passed to \code{HWE.test}).}
  \item{hide.NA}{a logical stating whether non-tested loci (e.g., when
    an allele is fixed) should be hidden in the results (TRUE, default)
    or not (FALSE).}
  \item{res.type}{a character or a character vector whose only first
    argument is considered giving the type of result to display. If
    "full", then a list of complete tests is returned. If "matrix",
    then a matrix of p-values is returned.}
}
\details{Monte Carlo procedure is quiet computer-intensive when large
  datasets are involved. For more precision on the performed test, read
  \code{HWE.test} documentation (\code{genetics} package).
}
\value{Returns either a list of tests or a matrix of p-values. In the
  first case, each test is designated by locus first and then by
  population. For instance if \code{res} is the "full" output of the
  function, then the test for population "PopA" at locus "Myloc" is
  given by res$Myloc$PopA. If \code{res} is a matrix of p-values,
  populations are in rows and loci in columns. P-values are given for
  the upper-tail: they correspond to the probability that an oberved
  chi-square statistic as high as or higher than the one observed
  occured under H0 (HWE).\cr  
  
  In all cases, NA values are likely to appear in fixed loci, or
  entirely non-typed loci.
}
\seealso{\code{HWE.test} in the \code{genetics} package, \code{\link[stats]{chisq.test}}
}
\author{ Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
\dontrun{
data(nancycats)
obj <- nancycats
if(require(genetics)){
obj.test <- HWE.test(obj)

# pvalues matrix to have a preview
HWE.test(obj,res.type="matrix")

#more precise view to...
obj.test$fca90$P10
}
}
}
\keyword{manip}
\keyword{multivariate}

\encoding{UTF-8}
\name{Inbreeding estimation}
\alias{inbreeding}
\title{Likelihood-based estimation of inbreeding}
\description{
  The function \code{inbreeding} estimates the inbreeding coefficient
  of an individuals (F) by computing its likelihood function. It can
  return either the density of probability of F, or a sample of F values
  from this distribution. This operation is performed for all the
  individuals of a \linkS4class{genind} object. Any ploidy greater than
  1 is acceptable.
}
\usage{
inbreeding(x, pop = NULL, truenames = TRUE, 
           res.type = c("sample", "function", "estimate"), N = 200, M = N * 10)
}
\arguments{
  \item{x}{an object of class \linkS4class{genind}.}
  \item{pop}{a factor giving the 'population' of each individual. If NULL,
    pop is seeked from \code{pop(x)}. Note that the term population refers in
    fact to any grouping of individuals'.}
  \item{truenames}{a logical indicating whether true names should be
    used (TRUE, default) instead of generic labels (FALSE); used if
    res.type is "matrix".}
 \item{res.type}{a character string matching "sample", "function", or "estimate"
   specifying whether the output should be a function giving the density
   of probability of F values ("function"), the maximum likelihood estimate
   of F from this distribution ("estimate"), or a sample of F values taken
   from this distribution ("sample", default).}
 \item{N}{an integer indicating the size of the sample to be taken from
   the distribution of F values.}
 \item{M}{an integer indicating the number of different F values to be
   used to generate the sample. Values larger than N are recommended to
   avoid poor sampling of the distribution.}
}
\value{
  A named list with one component for each individual, each of which is
  a function or a vector of sampled F values (see \code{res.type} argument).
}
\seealso{
  \code{\link{Hs}}: computation of expected heterozygosity.
}
\details{
  Let \eqn{F} denote the inbreeding coefficient, defined as the
  probability for an individual to inherit two identical alleles from a
  single ancestor.

  Let \eqn{p_i} refer to the frequency of allele \eqn{i} in the
  population. Let \eqn{h} be an variable which equates 1 if the
  individual is homozygote, and 0 otherwise. For one locus, the
  probability of being homozygote is computed as:

  \eqn{ F + (1-F) \sum_i p_i^2}

  The probability of being heterozygote is:
  \eqn{1 - (F + (1-F) \sum_i p_i^2)}

  The likelihood of a genotype is defined as the probability of being
  the observed state (homozygote or heterozygote). In the case of
  multilocus genotypes, log-likelihood are summed over the loci.
}
\author{
  Thibaut Jombart \email{t.jombart@imperial.ac.uk}\cr
  Zhian N. Kamvar\cr
}
\examples{
\dontrun{
## cattle breed microsatellite data
data(microbov)

## isolate Lagunaire breed
lagun <- seppop(microbov)$Lagunaire

## estimate inbreeding - return sample of F values
Fsamp <- inbreeding(lagun, N=30)

## plot the first 10 results
invisible(sapply(Fsamp[1:10], function(e) plot(density(e), xlab="F",
xlim=c(0,1), main="Density of the sampled F values")))

## compute means for all individuals
Fmean=sapply(Fsamp, mean)
hist(Fmean, col="orange", xlab="mean value of F",
main="Distribution of mean F across individuals")

## estimate inbreeding - return proba density functions
Fdens <- inbreeding(lagun, res.type="function")

## view function for the first individual
Fdens[[1]]

## plot the first 10 functions
invisible(sapply(Fdens[1:10], plot, ylab="Density",
main="Density of probability of F values"))

## estimate inbreeding - return maximum likelihood estimates
Fest <- inbreeding(lagun, res.type = "estimate")
mostInbred <- which.max(Fest)
plot(Fdens[[mostInbred]], ylab = "Density", xlab = "F",
     main = paste("Probability density of F values\nfor", names(mostInbred)))
abline(v = Fest[mostInbred], col = "red", lty = 2)
legend("topright", legend = "MLE", col = "red", lty = 2)

## note that estimates and average samples are likely to be different.
plot(Fest, ylab = "F", col = "blue",
     main = "comparison of MLE and average sample estimates of F")
points(Fmean, pch = 2, col = "red")
arrows(x0 = 1:length(Fest), y0 = Fest, 
       y1 = Fmean, x1 = 1:length(Fest), length = 0.125)
legend("topleft", legend = c("estimate", "sample"), col = c("blue", "red"),
       pch = c(1, 2), title = "res.type")
}
}

\name{as methods in adegenet}
\docType{methods}
\alias{as-method}
\alias{as,genind,data.frame-method}
\alias{as,genpop,data.frame-method}
\alias{as,genind,matrix-method}
\alias{as,genpop,matrix-method}
\alias{as,genind,genpop-method}
\alias{ktab-class}
\alias{as,genind,ktab-method}
\alias{as,genpop,ktab-method}
\alias{coerce,genind,data.frame-method}
\alias{coerce,genpop,data.frame-method}
\alias{coerce,genind,matrix-method}
\alias{coerce,genpop,matrix-method}
\alias{coerce,genind,genpop-method}
\alias{coerce,genind,ktab-method}
\alias{coerce,genpop,ktab-method}
\alias{as.data.frame.genind}
\alias{as.data.frame.genpop}
\alias{as.matrix.genind}
\alias{as.matrix.genpop}
\alias{as.genpop.genind}
\alias{as.ktab.genind}
\alias{as.ktab.genpop}
\title{Converting genind/genpop objects to other classes}
\description{
 These S3 and S4 methods are used to coerce \linkS4class{genind} and
 \linkS4class{genpop} objects to matrix-like objects. In most cases,
 this is equivalent to calling the \code{@tab} slot. An exception to
 this is the convertion to \code{\link[ade4]{ktab}} objects used in the
 ade4 package as inputs for K-tables methods (e.g. Multiple Coinertia
 Analysis).\cr
}
\section{Usage}{
\code{as(object, Class)}
}
\section{Arguments}{
\describe{
  \item{\code{object}}{a \linkS4class{genind} or a \linkS4class{genpop} object.}
  \item{\code{Class}}{the name of the class to which the object should
    be coerced, for instance \code{"data.frame"} or \code{"matrix"}.}
}
}
\section{Methods}{
\describe{
  \item{coerce}{from one object class to another using
    \code{as(object,"Class")}, where the \code{object} is of the old
    class and the returned object is of the new class \code{"Class"}.}
}
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk}}
\examples{
\dontrun{
data(microbov)
x <- tab(microbov,NA.method="mean")
as(x[1:3],"data.frame")

## dudi functions attempt to convert their first argument
## to a data.frame; so they can be used on genind/genpop objects.
## perform a PCA
pca1 <- dudi.pca(x, scale=FALSE, scannf=FALSE)
pca1

x <- genind2genpop(microbov,miss="chi2")
x <- as(x,"ktab")
class(x)
## perform a STATIS analysis
statis1 <- statis(x, scannf=FALSE)
statis1
plot(statis1)

}
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\encoding{utf-8}
\name{rupica}
\alias{rupica}
\title{Microsatellites genotypes of 335 chamois (Rupicapra rupicapra) from the
Bauges mountains (France)}
\format{\code{rupica} is a genind object with 3 supplementary components
inside the \code{@other} slot: \describe{ \item{xy}{a matrix containing the
spatial coordinates of the genotypes.} \item{mnt}{a raster map of elevation,
with the \code{asc} format from the \code{adehabitat} package.}
\item{showBauges}{a function to display the map of elevation with an
appropriate legend (use \code{showBauges()}).} }}
\source{
Daniel Maillard, 'Office National de la Chasse et de la Faune
Sauvage' (ONCFS), France.
}
\description{
This data set contains the genotypes of 335 chamois (\emph{Rupicapra
rupicapra}) from the Bauges mountains, in France. No prior clustering about
individuals is known. Each genotype is georeferenced. These data also
contain a raster map of elevation of the sampling area.
}
\examples{


data(rupica)
rupica


\dontrun{
if(require(adehabitat)){

## see the sampling area
showBauges <- rupica$other$showBauges
showBauges()
points(rupica$other$xy,col="red")

## perform a sPCA
spca1 <- spca(rupica,type=5,d1=0,d2=2300,plot=FALSE,scannf=FALSE,nfposi=2,nfnega=0)
barplot(spca1$eig,col=rep(c("black","grey"),c(2,100)),main="sPCA eigenvalues")
screeplot(spca1,main="sPCA eigenvalues: decomposition")

## data visualization
showBauges(,labcex=1)
s.value(spca1$xy,spca1$ls[,1],add.p=TRUE,csize=.5)
add.scatter.eig(spca1$eig,1,1,1,posi="topleft",sub="Eigenvalues")

showBauges(,labcex=1)
s.value(spca1$xy,spca1$ls[,2],add.p=TRUE,csize=.5)
add.scatter.eig(spca1$eig,2,2,2,posi="topleft",sub="Eigenvalues")

rupica$other$showBauges()
colorplot(spca1$xy,spca1$li,cex=1.5,add.plot=TRUE)

## global and local tests
Gtest <- global.rtest(rupica@tab,spca1$lw,nperm=999)
Gtest
plot(Gtest)
Ltest <- local.rtest(rupica@tab,spca1$lw,nperm=999)
Ltest
plot(Ltest)
}
}

}
\references{
Cassar S (2008) Organisation spatiale de la variabilité
génétique et phénotypique a l'échelle du paysage: le cas du chamois et du
chevreuil, en milieu de montagne. PhD Thesis. University Claude Bernard -
Lyon 1, France. \cr

Cassar S, Jombart T, Loison A, Pontier D, Dufour A-B, Jullien J-M, Chevrier
T, Maillard D. Spatial genetic structure of Alpine chamois (\emph{Rupicapra
rupicapra}): a consequence of landscape features and social factors?
submitted to \emph{Molecular Ecology}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genind2genpop.R
\name{genind2genpop}
\alias{genind2genpop}
\title{Conversion from a genind to a genpop object}
\usage{
genind2genpop(x, pop = NULL, quiet = FALSE, process.other = FALSE,
  other.action = mean)
}
\arguments{
\item{x}{an object of class \code{genind}.}

\item{pop}{a factor giving the population of each genotype in 'x' OR a
formula specifying which strata are to be used when converting to a genpop
object. If none provided, population factors are sought in x@pop, but if
given, the argument prevails on x@pop.}

\item{quiet}{logical stating whether a conversion message must be printed 
(TRUE,default) or not (FALSE).}

\item{process.other}{a logical indicating whether the \code{@other} slot 
should be processed (see details).}

\item{other.action}{a function to be used when processing the \code{@other} 
slot. By default, 'mean' is used.}
}
\value{
A genpop object. The component @other in 'x' is passed to the 
  created genpop object.
}
\description{
The function \code{genind2genpop} converts genotypes data (genind) into 
alleles counts per population (genpop).
}
\details{
=== 'missing' argument ===\cr The values of the 'missing' argument in 
\code{genind2genpop} have the following effects:\cr - "NA": if all genotypes 
of a population for a given allele are missing, count value will be NA\cr

- "0": if all genotypes of a population for a given allele are missing, count
value will be 0\cr

- "chi2": if all genotypes of a population for a given allele are missing, 
count value will be that of a theoretical count in of a Chi-squared test. 
This is obtained by the product of the margins sums divided by the total 
number of alleles.\cr

=== processing the \code{@other} slot ===\cr Essentially, 
\code{genind2genpop} is about aggregating data per population. The function 
can do the same for all numeric items in the \code{@other} slot provided 
they have the same length (for vectors) or the same number of rows 
(matrix-like objects) as the number of genotypes. When the case is 
encountered and if \code{process.other} is TRUE, then these objects are 
processed using the function defined in \code{other.action} per population. 
For instance, spatial coordinates of genotypes would be averaged to obtain 
population coordinates.
}
\examples{

## simple conversion
data(nancycats)
nancycats
catpop <- genind2genpop(nancycats)
catpop
summary(catpop)

## processing the @other slot
data(sim2pop)
sim2pop$other$foo <- letters
sim2pop
dim(sim2pop$other$xy) # matches the number of genotypes
sim2pop$other$foo # does not match the number of genotypes

obj <- genind2genpop(sim2pop, process.other=TRUE)
obj$other # the new xy is the populations' centre

pch <- as.numeric(pop(sim2pop))
col <- pop(sim2pop)
levels(col) <- c("blue","red")
col <- as.character(col)
plot(sim2pop$other$xy, pch=pch, col=col)
text(obj$other$xy, lab=row.names(obj$other$xy), col=c("blue","red"), cex=2, font=2)
\dontrun{
data(microbov)
strata(microbov) <- data.frame(other(microbov))
summary(genind2genpop(microbov)) # Conversion based on population factor
summary(genind2genpop(microbov, ~coun)) # Conversion based on country
summary(genind2genpop(microbov, ~coun/spe)) # Conversion based on country and species

}

}
\seealso{
\linkS4class{genind}, \linkS4class{genpop}
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{classes}
\keyword{manip}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/doc_C_routines.R
\docType{data}
\name{.internal_C_routines}
\alias{.internal_C_routines}
\alias{CheckAllSeg}
\alias{GLdotProd}
\alias{GLsumFreq}
\alias{GLsumInt}
\alias{binIntToBytes}
\alias{bytesToBinInt}
\alias{bytesToInt}
\alias{nb_shared_all}
\title{Internal C routines}
\format{An object of class \code{NULL} of length 0.}
\usage{
.internal_C_routines
}
\description{
These functions are internal C routines used in adegenet. Do not use them
unless you know what you are doing.
}
\author{
Thibaut Jombart
}
\keyword{datasets}

\encoding{UTF-8}
\docType{methods}
\name{Accessors}
\alias{$,genind-method}
\alias{$,genpop-method}
\alias{$<-,genind-method}
\alias{$<-,genpop-method}
\alias{[,genind-method}
\alias{[,genind,ANY,ANY,ANY-method}
\alias{[,genpop-method}
\alias{[,genpop,ANY,ANY,ANY-method}
\alias{nLoc}
\alias{nLoc,gen-method}
\alias{nLoc,genind-method}
\alias{nLoc,genpop-method}
\alias{locFac}
\alias{locFac,gen-method}
\alias{locFac,genind-method}
\alias{locFac,genpop-method}
\alias{nAll}
\alias{nAll,gen-method}
\alias{nAll,genind-method}
\alias{nAll,genpop-method}
\alias{nPop}
\alias{nPop,genind-method}
\alias{nPop,genpop-method}
\alias{nInd}
\alias{nInd,genind-method}
\alias{pop}
\alias{pop<-}
\alias{pop,genind-method}
\alias{pop<-,gen-method}
\alias{pop<-,genind-method}
\alias{locNames}
\alias{locNames,gen-method}
\alias{locNames,genind-method}
\alias{locNames,genpop-method}
\alias{locNames<-}
\alias{locNames<-,gen-method}
\alias{locNames<-,genind-method}
\alias{locNames<-,genpop-method}
\alias{popNames}
\alias{popNames,genind-method}
\alias{popNames,genpop-method}
\alias{popNames<-}
\alias{popNames<-,genind-method}
\alias{popNames<-,genpop-method}
\alias{indNames}
\alias{indNames,genind-method}
\alias{indNames<-}
\alias{indNames<-,genind-method}
\alias{ploidy}
\alias{ploidy,genind-method}
\alias{ploidy,genpop-method}
\alias{ploidy<-}
\alias{ploidy<-,genind-method}
\alias{ploidy<-,genpop-method}
\alias{alleles}
\alias{alleles,gen-method}
\alias{alleles,genind-method}
\alias{alleles,genpop-method}
\alias{alleles<-}
\alias{alleles<-,gen-method}
\alias{alleles<-,genind-method}
\alias{alleles<-,genpop-method}
\alias{other}
\alias{other,gen-method}
\alias{other,genind-method}
\alias{other,genpop-method}
\alias{other<-}
\alias{other<-,gen-method}
\alias{other<-,genind-method}
\alias{other<-,genpop-method}

\title{Accessors for adegenet objects}
\description{
  An accessor is a function that allows to interact with slots of an
  object in a convenient way. Several accessors are available for \linkS4class{genind} or
  \linkS4class{genpop} objects. The operator "\$" and "\$<-" are used to
  access the slots, being equivalent to "@" and "@<-".\cr

  The operator "[" is a flexible way to subset data by individuals,
  populations, alleles, and loci. When using a matrix-like syntax,
  subsetting will apply to the dimensios of the @tab slot. In addition,
  specific arguments \code{loc} and \code{pop} can be used to indicate
  subsets of loci and populations. The argument \code{drop} is a logical
  indicating if alleles becoming non-polymorphic in a new dataset should
  be removed (default: FALSE). Examples:\cr
  \itemize{
  \item "obj[i,j]" returns "obj" with a subset 'i' of individuals and 'j' of
  alleles.\cr
  
  \item "obj[1:10,]" returns an object with only the first 10 genotypes (if
  "obj" is a \linkS4class{genind}) or the first 10 populations (if
  "obj" is a \linkS4class{genpop}) \cr
  
  \item "obj[1:10, 5:10]" returns an object keeping the first 10 entities and
  the alleles 5 to 10.\cr
  
  \item "obj[loc=c(1,3)]" returns an object keeping only the 1st and 3rd
  loci, using \code{locNames(obj)} as reference; logicals, or named loci
  also work; this overrides other subsetting of alleles.\cr
  
 \item "obj[pop=2:4]" returns an object keeping only individuals from the
  populations 2, 3 and 4, using \code{popNames(obj)} as reference;
  logicals, or named populations also work; this overrides other
  subsetting of individuals.\cr

  \item "obj[i=1:2, drop=TRUE]" returns an object keeping only the first
  two individuals (or populations), dropping the alleles no longer
  present in the data.\cr
}
  The argument \code{treatOther} handles the treatment of objects in the
  \code{@other} slot (see details). The argument \code{drop} can be set
  to TRUE to drop alleles that are no longer represented in the subset.
}
\section{Methods}{
  \describe{
    \item{nInd}{returns the number of individuals in the \code{genind} object}
    \item{nLoc}{returns the number of loci}
    \item{nAll}{returns the number of alleles in each locus}
    \item{nPop}{returns the number of populations}
    \item{pop}{returns a factor assigning individuals to populations.}
    \item{pop<-}{replacement method for the \code{@pop} slot of an
      object.}
    \item{popNames}{returns the names of populations.}
    \item{popNames<-}{sets the names of populations using a vector of
      length \code{nPop(x)}.}
    \item{indNames}{returns the names of individuals.}
    \item{indNames<-}{sets the names of individuals using a vector of
      length \code{nInd(x)}.}
    \item{locNames}{returns the names of markers and/or alleles.}
    \item{locNames<-}{sets the names of markers using a vector of
      length \code{nLoc(x)}.}
    \item{locFac}{returns a factor that defines which locus each column of the \code{@tab} slot belongs to}
    \item{ploidy}{returns the ploidy of the data.}
    \item{ploidy<-}{sets the ploidy of the data using an integer.}
    \item{alleles}{returns the alleles of each locus.}
    \item{alleles<-}{sets the alleles of each locus using a list with
      one character vector for each locus.}
    \item{other}{returns the content of the \code{@other} slot
      (misc. information); returns \code{NULL} if the slot is empty or of
      length zero.}
    \item{other<-}{sets the content of the \code{@other} slot
      (misc. information); the provided value needs to be a list; it
      not, provided value will be stored within a list.}
  }
}
\usage{
nInd(x, \dots)
nLoc(x, \dots)
nPop(x, \dots)
pop(x)
indNames(x, \dots)
\S4method{indNames}{genind}(x, \dots)
locNames(x, \dots)
\S4method{locNames}{genind}(x, withAlleles=FALSE, \dots)
\S4method{locNames}{genpop}(x, withAlleles=FALSE, \dots)
popNames(x, \dots)
\S4method{popNames}{genind}(x, \dots)
popNames(x, \dots)
\S4method{popNames}{genpop}(x, \dots)
ploidy(x, \dots)
\S4method{ploidy}{genind}(x, \dots)
\S4method{ploidy}{genpop}(x, \dots)
\S4method{other}{genind}(x, \dots)
\S4method{other}{genpop}(x, \dots)
}
\arguments{
  \item{x}{a \linkS4class{genind} or a \linkS4class{genpop} object.}
  \item{withAlleles}{a logical indicating whether the result should be
    of the form [locus name].[allele name], instead of [locus name].}
  \item{\dots}{further arguments to be passed to other methods
  (currently not used).}
}
\value{
  A \linkS4class{genind} or \linkS4class{genpop} object.
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\details{
  The "[" operator can treat elements in the \code{@other} slot as
  well. For instance, if \code{obj@other$xy} contains spatial
  coordinates, the \code{obj[1:3, ]@other$xy} will contain the spatial
  coordinates of the genotypes (or population) 1,2 and 3. This is
  handled through the argument \code{treatOther}, a logical defaulting
  to TRUE. If set to FALSE, the \code{@other} returned unmodified.\cr
  
  Note that only matrix-like, vector-like and lists can be proceeded in
  \code{@other}. Other kind of objects will issue a warning an be
  returned as they are, unless the argument \code{quiet} is left to
  TRUE, its default value.\cr

  The \code{drop} argument can be set to TRUE to retain only alleles
  that are present in the subset. To achieve better control of
  polymorphism of the data, see \code{\link{isPoly}}.
}
\examples{
data(nancycats)
nancycats
pop(nancycats) # get the populations
indNames(nancycats) # get the labels of individuals
locNames(nancycats) # get the labels of the loci
alleles(nancycats) # get the alleles

head(tab(nancycats)) # get allele counts

# get allele frequencies, replace NAs
head(tab(nancycats, freq = TRUE, NA.method = "mean")) 

# let's isolate populations 4 and 8
popNames(nancycats)
obj <- nancycats[pop=c(4,8)]
obj
popNames(obj)
pop(obj)

# let's isolate two markers, fca23 and fca90
locNames(nancycats)
obj <- nancycats[loc=c("fca23","fca90")]
obj
locNames(obj)

# illustrate pop
obj <- nancycats[sample(1:100, 10)]
pop(obj)
pop(obj) <- rep(c('b', 'a'), each = 5)
pop(obj)

# illustrate locNames
locNames(obj)
locNames(obj, withAlleles = TRUE)
locNames(obj)[1] <- "newLocus"
locNames(obj)
locNames(obj, withAlleles=TRUE)

# illustrate how 'other' slot is handled
data(sim2pop)
nInd(sim2pop)
other(sim2pop[1:6]) # xy is subsetted automatically
other(sim2pop[1:6, treatOther=FALSE]) # xy is left as is

}
\keyword{manip}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale.R
\docType{methods}
\name{scaleGen}
\alias{scaleGen}
\alias{scaleGen-methods}
\alias{scaleGen,genind-method}
\alias{scaleGen,genpop-method}
\alias{scaleGen,genind-method}
\alias{scaleGen,genpop-method}
\title{Compute scaled allele frequencies}
\usage{
scaleGen(x, ...)

\S4method{scaleGen}{genind}(x, center = TRUE, scale = TRUE,
  NA.method = c("asis", "mean", "zero"), truenames = TRUE)

\S4method{scaleGen}{genpop}(x, center = TRUE, scale = TRUE,
  NA.method = c("asis", "mean", "zero"), truenames = TRUE)
}
\arguments{
\item{x}{a \linkS4class{genind} and \linkS4class{genpop} object}

\item{...}{further arguments passed to other methods.}

\item{center}{a logical stating whether alleles frequencies should be
centred to mean zero (default to TRUE). Alternatively, a vector of numeric
values, one per allele, can be supplied: these values will be substracted
from the allele frequencies.}

\item{scale}{a logical stating whether alleles frequencies should be scaled
(default to TRUE). Alternatively, a vector of numeric values, one per
allele, can be supplied: these values will be substracted from the allele
frequencies.}

\item{NA.method}{a method to replace NA; asis: leave NAs as is; mean: replace by the mean allele frequencies; zero: replace by zero}

\item{truenames}{no longer used; kept for backward compatibility}
}
\value{
A matrix of scaled allele frequencies with genotypes
(\linkS4class{genind}) or populations in (\linkS4class{genpop}) in rows and
alleles in columns.
}
\description{
The generic function \code{scaleGen} is an analogue to the \code{scale}
function, but is designed with further arguments giving scaling options.\cr
}
\details{
Methods are defined for \linkS4class{genind} and \linkS4class{genpop}
objects.  Both return data.frames of scaled allele frequencies.
}
\examples{

\dontrun{
## load data
data(microbov)
obj <- genind2genpop(microbov)

## compare different scaling
X1 <- scaleGen(obj)
X2 <- scaleGen(obj,met="bin")

## compute PCAs
pcaObj <- dudi.pca(obj,scale=FALSE,scannf=FALSE) # pca with no scaling
pcaX1 <- dudi.pca(X1,scale=FALSE,scannf=FALSE,nf=100) # pca with usual scaling
pcaX2 <- dudi.pca(X2,scale=FALSE,scannf=FALSE,nf=100) # pca with scaling for binomial variance

## get the loadings of alleles for the two scalings
U1 <- pcaX1$c1
U2 <- pcaX2$c1


## find an optimal plane to compare loadings
## use a procustean rotation of loadings tables
pro1 <- procuste(U1,U2,nf=2)

## graphics
par(mfrow=c(2,2))
# eigenvalues
barplot(pcaObj$eig,main="Eigenvalues\\n no scaling")
barplot(pcaX1$eig,main="Eigenvalues\\n usual scaling")
barplot(pcaX2$eig,main="Eigenvalues\\n 'binomial' scaling")
# differences between loadings of alleles
s.match(pro1$scor1,pro1$scor2,clab=0,sub="usual -> binom (procustean rotation)")

}

}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{manip}
\keyword{methods}

\encoding{UTF-8}
\name{snpzip}
\alias{snpzip}
\title{Identification of structural SNPs}
\description{
  The function \code{snpzip} identifies the set of alleles which contribute most
  significantly to phenotypic structure.
  
  This procedure uses Discriminant Analysis of Principal Components (DAPC)
  to quantify the contribution of individual alleles to between-population
  structure. Then, defining contribution to DAPC as the measure of distance
  between alleles, hierarchical clustering is used to identify two groups
  of alleles: structural SNPs and non-structural SNPs.
}
\usage{
  snpzip(snps, y, plot = TRUE, xval.plot = FALSE, loading.plot = FALSE,
         method = c("complete", "single", "average", "centroid", 
                    "mcquitty", "median", "ward"), \dots)
}
\arguments{
  \item{snps}{a snps \code{matrix} used as input of DAPC.}
  \item{y}{either a \code{factor} indicating the group membership of individuals, 
           or a dapc object.}
  \item{plot}{a \code{logical} indicating whether a graphical representation of the 
              DAPC results should be displayed.}
  \item{xval.plot}{a \code{logical} indicating whether the results of the 
                   cross-validation step should be displayed (iff \code{y} is a factor).}
  \item{loading.plot}{a \code{logical} indicating whether a loading.plot displaying 
                      the SNP selection threshold should be displayed.}
  \item{method}{the clustering method to be used. This should be 
                (an unambiguous abbreviation of) one of \code{"complete", "single", "average", 
                                                              "centroid", "mcquitty", "median",} or \code{"ward"}.} 
  \item{\dots}{further arguments.}
  
}

\details{
  \code{snpzip} provides an objective procedure to delineate between structural 
  and non-structural SNPs identified by Discriminant Analysis of Principal Components 
  (DAPC, Jombart et al. 2010). 
  \code{snpzip} precedes the multivariate analysis with a cross-validation step 
  to ensure that the subsequent DAPC is performed optimally.
  The contributions of alleles to the DAPC are then submitted to \code{hclust}, 
  where they define a distance matrix upon which hierarchical clustering is carried out.
  To complete the procedure, \code{snpzip} uses \code{cutree} to automatically 
  subdivide the set of SNPs fed into the analysis into two groups: 
    those which contribute significantly to the phenotypic structure of interest, 
  and those which do not.   
}

\value{
  A \code{list} with four items if \code{y} is a factor, or two items if
  \code{y} is a dapc object: 
    The first cites the number of principal components (PCs) of PCA retained in the DAPC. 
  
  The second item is an embedded list which
  first indicates the number of structural and non-structural SNPs identified by 
  \code{snpzip}, second provides a list of the structuring alleles, third
  gives the names of the selected alleles, and fourth details the 
  contributions of these structuring alleles to the DAPC.
  
  The optional third item provides measures of discrimination success both overall 
  and by group.
  
  The optional fourth item contains the dapc object generated if \code{y} was a factor.
  
  
  If \code{plot=TRUE}, a scatter plot will provide a visualization of the DAPC results.
  
  If \code{xval.plot=TRUE}, the results of the cross-validation step will be displayed 
  as an \code{array} of the format generated by xvalDapc, and a scatter plot of 
  the results of cross-validation will be provided.   
  
  If \code{loading.plot=TRUE}, a loading plot will be generated to show the 
  contributions of alleles to the DAPC, and the SNP selection threshold will be indicated.
  If the number of Discriminant Axes (\code{n.da}) in the DAPC is greater than 1, 
  \code{loading.plot=TRUE} will generate one loading plot for each discriminant axis.
}

\references{
  Jombart T, Devillard S and Balloux F (2010) Discriminant analysis of principal 
  components: a new method for the analysis of genetically structured populations. 
  BMC Genetics11:94. doi:10.1186/1471-2156-11-94
}

\author{ Caitlin Collins \email{caitlin.collins12@imperial.ac.uk} }
\examples{
  \dontrun{
    simpop <- glSim(100, 10000, n.snp.struc = 10, grp.size = c(0.3,0.7), 
                    LD = FALSE, alpha = 0.4, k = 4)
    snps <- as.matrix(simpop)
    phen <- simpop@pop
    
    outcome <- snpzip(snps, phen, method = "centroid")
    outcome
  }
  \dontrun{
    simpop <- glSim(100, 10000, n.snp.struc = 10, grp.size = c(0.3,0.7), 
                    LD = FALSE, alpha = 0.4, k = 4)
    snps <- as.matrix(simpop)
    phen <- simpop@pop
    
    dapc1 <- dapc(snps, phen, n.da = 1, n.pca = 30)
    
    features <- snpzip(dapc1, loading.plot = TRUE, method = "average")
    features
  }
}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/handling.R
\name{repool}
\alias{repool}
\title{Pool several genotypes into a single dataset}
\usage{
repool(..., list = FALSE)
}
\arguments{
\item{...}{a list of \linkS4class{genind} objects, or a series of \linkS4class{genind} objects separated by commas}

\item{list}{a logical indicating whether a list of objects with matched alleles shall be returned (TRUE), or a single \linkS4class{genind} object (FALSE, default).}
}
\description{
The function \code{repool} allows to merge genotypes from different
 \linkS4class{genind} objects into a single 'pool' (i.e. a new \linkS4class{genind}).
 The markers have to be the same for all objects to be merged, but
 there is no constraint on alleles.\cr
}
\details{
This function can be useful, for instance, when hybrids are created
 using \code{\link{hybridize}}, to merge hybrids with their parent
 population for further analyses. Note that \code{repool} can also
 reverse the action of \code{\link{seppop}}.
}
\examples{
\dontrun{
## use the cattle breeds dataset
data(microbov)
temp <- seppop(microbov)
names(temp)
## hybridize salers and zebu -- nasty cattle
zebler <- hybridize(temp$Salers, temp$Zebu, n=40)
zebler
## now merge zebler with other cattle breeds
nastyCattle <- repool(microbov, zebler)
nastyCattle
}

}
\seealso{
\code{\link{seploc}}, \code{\link{seppop}}
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chooseCN.R
\name{chooseCN}
\alias{chooseCN}
\title{Function to choose a connection network}
\usage{
chooseCN(xy, ask = TRUE, type = NULL, result.type = "nb", d1 = NULL,
  d2 = NULL, k = NULL, a = NULL, dmin = NULL, plot.nb = TRUE,
  edit.nb = FALSE, check.duplicates = TRUE)
}
\arguments{
\item{xy}{an matrix or data.frame with two columns for x and y coordinates.}

\item{ask}{a logical stating whether graph should be chosen interactively
(TRUE,default) or not (FALSE). Set to FALSE if \code{type} is provided.}

\item{type}{an integer giving the type of graph (see details).}

\item{result.type}{a character giving the class of the returned object.
Either "nb" (default) or "listw", both from \code{spdep} package. See
details.}

\item{d1}{the minimum distance between any two neighbours. Used if
\code{type=5.}}

\item{d2}{the maximum distance between any two neighbours. Used if
\code{type=5}. Can also be a character: "dmin" for the minimum distance so
that each site has at least one connection, or "dmax" to have all sites
connected (despite the later has no sense).}

\item{k}{the number of neighbours per point. Used if \code{type=6}.}

\item{a}{the exponent of the inverse distance matrix. Used if \code{type=7}.}

\item{dmin}{the minimum distance between any two distinct points. Used to
avoid infinite spatial proximities (defined as the inversed spatial
distances). Used if \code{type=7}.}

\item{plot.nb}{a logical stating whether the resulting graph should be
plotted (TRUE, default) or not (FALSE).}

\item{edit.nb}{a logical stating whether the resulting graph should be
edited manually for corrections (TRUE) or not (FALSE, default).}

\item{check.duplicates}{a logical indicating if duplicate coordinates should be detected; this can be an issue for some graphs; TRUE by default.}
}
\value{
Returns a connection network having the class \code{nb} or
\code{listw}. The xy coordinates are passed as attribute to the created
object.
}
\description{
The function \code{chooseCN} is a simple interface to build a connection
network (CN) from xy coordinates. The user chooses from 6 types of graph and
one additional weighting scheme.  \code{chooseCN} calls functions from
appropriate packages, handles non-unique coordinates and returns a
connection network either with classe \code{nb} or \code{listw}. For graph
types 1-4, duplicated locations are not accepted and will issue an error.
}
\details{
There are 7 kinds of graphs proposed: \cr Delaunay triangulation (type 1)\cr
Gabriel graph (type 2)\cr Relative neighbours (type 3)\cr Minimum spanning
tree (type 4)\cr Neighbourhood by distance (type 5)\cr K nearests neighbours
(type 6)\cr Inverse distances (type 7)\cr

The last option (type=7) is not a true neighbouring graph: all sites are
neighbours, but the spatial weights are directly proportional to the
inversed spatial distances.\cr Also not that in this case, the output of the
function is always a \code{listw} object, even if \code{nb} was
requested.\cr

The choice of the connection network has been discuted on the adegenet
forum. Please search the archives from adegenet website (section 'contact')
using 'graph' as keyword.
}
\examples{

\dontrun{
data(nancycats)

par(mfrow=c(2,2))
cn1 <- chooseCN(nancycats@other$xy,ask=FALSE,type=1)
cn2 <- chooseCN(nancycats@other$xy,ask=FALSE,type=2)
cn3 <- chooseCN(nancycats@other$xy,ask=FALSE,type=3)
cn4 <- chooseCN(nancycats@other$xy,ask=FALSE,type=4)
par(mfrow=c(1,1))
}

}
\seealso{
\code{\link{spca}}
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{spatial}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/strataMethods.R
\docType{methods}
\name{setPop}
\alias{setPop}
\alias{setPop,genind-method}
\alias{setPop,genlight-method}
\alias{setPop<-}
\alias{setPop<-,genind-method}
\alias{setPop<-,genlight-method}
\title{Manipulate the population factor of genind objects.}
\usage{
setPop(x, formula = NULL)

setPop(x) <- value
}
\arguments{
\item{x}{a genind or genlight object}

\item{formula}{a nested formula indicating the order of the population
strata.}

\item{value}{same as formula}
}
\description{
The following methods allow the user to quickly change the population of a 
genind object.
}
\examples{

data(microbov)

strata(microbov) <- data.frame(other(microbov))

# Currently set on just 
head(pop(microbov)) 

# setting the strata to both Pop and Subpop
setPop(microbov) <- ~coun/breed 
head(pop(microbov))

\dontrun{

# Can be used to create objects as well.
microbov.old <- setPop(microbov, ~spe) 
head(pop(microbov.old))
}
}
\author{
Zhian N. Kamvar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adegenet.package.R
\docType{package}
\encoding{utf-8}
\name{adegenet.package}
\alias{adegenet.package}
\alias{adegenet}
\alias{adegenet.package-package}
\title{The adegenet package}
\description{
This package is devoted to the multivariate analysis of genetic markers
data. These data can be codominant markers (e.g. microsatellites) or
presence/absence data (e.g. AFLP), and have any level of ploidy.  'adegenet'
defines three formal (S4) classes:\cr - \linkS4class{genind}: a class for
data of individuals ("genind" stands for genotypes-individuals).\cr -
\linkS4class{genpop}: a class for data of groups of individuals ("genpop"
stands for genotypes-populations)\cr - \linkS4class{genlight}: a class for
genome-wide SNP data\cr
}
\details{
For more information about these classes, type "class ? genind", "class ?
genpop", or "?genlight".\cr

Essential functionalities of the package are presented througout 4
tutorials, accessible using \code{adegenetTutorial(which="name-below")}:\cr
- \code{basics}: introduction to the package.\cr - \code{spca}: multivariate
analysis of spatial genetic patterns.\cr - \code{dapc}: population structure
and group assignment using DAPC.\cr - \code{genomics}: introduction to the
class \linkS4class{genlight} for the handling and analysis of genome-wide
SNP data.\cr

Note: In older versions of adegenet, these tutorials were avilable as
vignettes, accessible through the function \code{vignette("name-below",
package="adegenet")}:\cr - \code{adegenet-basics}.\cr -
\code{adegenet-spca}.\cr - \code{adegenet-dapc}.\cr -
\code{adegenet-genomics}.\cr

Important functions are also summarized below.\cr

=== IMPORTING DATA ===\cr = TO GENIND OBJECTS = \cr \code{adegenet} imports
data to \linkS4class{genind} object from the following softwares:\cr -
STRUCTURE: see \code{\link{read.structure}}\cr - GENETIX: see
\code{\link{read.genetix}}\cr - FSTAT: see \code{\link{read.fstat}}\cr -
Genepop: see \code{\link{read.genepop}}\cr To import data from any of these
formats, you can also use the general function
\code{\link{import2genind}}.\cr

In addition, it can extract polymorphic sites from nucleotide and amino-acid
alignments:\cr - DNA files: use \code{\link[ape]{read.dna}} from the ape
package, and then extract SNPs from DNA alignments using
\code{\link{DNAbin2genind}}. \cr

- protein sequences alignments: polymorphic sites can be extracted from
protein sequences alignments in \code{alignment} format (package
\code{seqinr}, see \code{\link[seqinr]{as.alignment}}) using the function
\code{\link{alignment2genind}}. \cr

The function \code{\link{fasta2DNAbin}} allows for reading fasta files into
DNAbin object with minimum RAM requirements.\cr

It is also possible to read genotypes coded by character strings from a
data.frame in which genotypes are in rows, markers in columns. For this, use
\code{\link{df2genind}}. Note that \code{\link{df2genind}} can be used for
any level of ploidy.\cr

= TO GENLIGHT OBJECTS = \cr SNP data can be read from the following
formats:\cr - PLINK: see function \code{\link{read.PLINK}}\cr - .snp
(adegenet's own format): see function \code{\link{read.snp}}\cr

SNP can also be extracted from aligned DNA sequences with the fasta format,
using \code{\link{fasta2genlight}}\cr

=== EXPORTING DATA ===\cr \code{adegenet} exports data from

Genotypes can also be recoded from a \linkS4class{genind} object into a
data.frame of character strings, using any separator between alleles. This
covers formats from many softwares like GENETIX or STRUCTURE. For this, see
\code{\link{genind2df}}.\cr

Also note that the \code{pegas} package imports \linkS4class{genind} objects
using the function \code{as.loci}.

=== MANIPULATING DATA ===\cr Several functions allow one to manipulate
\linkS4class{genind} or \linkS4class{genpop} objects\cr -
\code{\link{genind2genpop}}: convert a \linkS4class{genind} object to a
\linkS4class{genpop} \cr - \code{\link{seploc}}: creates one object per
marker; for \linkS4class{genlight} objects, creates blocks of SNPs.\cr -
\code{\link{seppop}}: creates one object per population \cr -
- \code{\link{tab}}: access the allele data (counts or frequencies) of an object
(\linkS4class{genind} and \linkS4class{genpop}) \cr -
x[i,j]: create a new object keeping only genotypes (or populations) indexed
by 'i' and the alleles indexed by 'j'.\cr - \code{\link{makefreq}}: returns
a table of allelic frequencies from a \linkS4class{genpop} object.\cr -
\code{\link{repool}} merges genoptypes from different gene pools into one
single \linkS4class{genind} object.\cr - \code{\link{propTyped}} returns the
proportion of available (typed) data, by individual, population, and/or
locus.\cr - \code{\link{selPopSize}} subsets data, retaining only genotypes
from a population whose sample size is above a given level.\cr -
\code{\link{pop}} sets the population of a set of genotypes.\cr

=== ANALYZING DATA ===\cr Several functions allow to use usual, and less
usual analyses:\cr - \code{\link{HWE.test.genind}}: performs HWE test for all
populations and loci combinations \cr - \code{\link{dist.genpop}}: computes 5
genetic distances among populations.  \cr - \code{\link{monmonier}}:
implementation of the Monmonier algorithm, used to seek genetic boundaries
among individuals or populations. Optimized boundaries can be obtained using
\code{\link{optimize.monmonier}}. Object of the class \code{monmonier} can be
plotted and printed using the corresponding methods. \cr -
\code{\link{spca}}: implements Jombart et al.  (2008) spatial Principal
Component Analysis \cr - \code{\link{global.rtest}}: implements Jombart et
al. (2008) test for global spatial structures \cr -
\code{\link{local.rtest}}: implements Jombart et al. (2008) test for local
spatial structures \cr - \code{\link{propShared}}: computes the proportion of
shared alleles in a set of genotypes (i.e. from a genind object)\cr -
\code{\link{propTyped}}: function to investigate missing data in several ways
\cr - \code{\link{scaleGen}}: generic method to scale \linkS4class{genind} or
\linkS4class{genpop} before a principal component analysis \cr -
\code{\link{Hs}}: computes the average expected heterozygosity by population
in a \linkS4class{genpop}. Classically Used as a measure of genetic
diversity.\cr - \code{\link{find.clusters}} and \code{\link{dapc}}: implement
the Discriminant Analysis of Principal Component (DAPC, Jombart et al.,
2010).\cr - \code{\link{seqTrack}}: implements the SeqTrack algorithm for
recontructing transmission trees of pathogens (Jombart et al., 2010) .\cr
\code{\link{glPca}}: implements PCA for \linkS4class{genlight} objects.\cr -
\code{\link{gengraph}}: implements some simple graph-based clustering using
genetic data.  - \code{\link{snpposi.plot}} and \code{\link{snpposi.test}}:
visualize the distribution of SNPs on a genetic sequence and test their
randomness.  - \code{\link{adegenetServer}}: opens up a web interface for
some functionalities of the package (DAPC with cross validation and feature
selection).\cr

=== GRAPHICS ===\cr - \code{\link{colorplot}}: plots points with associated
values for up to three variables represented by colors using the RGB system;
useful for spatial mapping of principal components.\cr -
\code{\link{loadingplot}}: plots loadings of variables. Useful for
representing the contribution of alleles to a given principal component in a
multivariate method. \cr - \code{\link{scatter.dapc}}: scatterplots for DAPC
results.\cr - \code{\link{compoplot}}: plots membership probabilities from a
DAPC object. \cr

=== SIMULATING DATA ===\cr - \code{\link{hybridize}}: implements
hybridization between two populations. \cr - \code{\link{haploGen}}:
simulates genealogies of haplotypes, storing full genomes. \cr
\code{\link{glSim}}: simulates simple \linkS4class{genlight} objects.\cr

=== DATASETS ===\cr - \code{\link{H3N2}}: Seasonal influenza (H3N2) HA
segment data. \cr - \code{\link{dapcIllus}}: Simulated data illustrating the
DAPC. \cr - \code{\link{eHGDP}}: Extended HGDP-CEPH dataset. \cr -
\code{\link{microbov}}: Microsatellites genotypes of 15 cattle breeds. \cr -
\code{\link{nancycats}}: Microsatellites genotypes of 237 cats from 17
colonies of Nancy (France). \cr - \code{\link{rupica}}: Microsatellites
genotypes of 335 chamois (Rupicapra rupicapra) from the Bauges mountains
(France).\cr - \code{\link{sim2pop}}: Simulated genotypes of two
georeferenced populations.\cr - \code{\link{spcaIllus}}: Simulated data
illustrating the sPCA. \cr

For more information, visit the adegenet website using the function
\code{\link{adegenetWeb}}.\cr

Tutorials are available via the command \code{adegenetTutorial}.\cr

To cite adegenet, please use the reference given by
\code{citation("adegenet")} (or see references below).
}
\references{
Jombart T. (2008) adegenet: a R package for the multivariate
analysis of genetic markers \emph{Bioinformatics} 24: 1403-1405. doi:
10.1093/bioinformatics/btn129\cr

Jombart T. and Ahmed I. (2011) adegenet 1.3-1: new tools for the analysis of
genome-wide SNP data.  \emph{Bioinformatics}. doi:
10.1093/bioinformatics/btr521

Jombart T, Devillard S and Balloux F (2010) Discriminant analysis of
principal components: a new method for the analysis of genetically
structured populations. BMC Genetics 11:94.  doi:10.1186/1471-2156-11-94\cr

Jombart T, Eggo R, Dodd P, Balloux F (2010) Reconstructing disease outbreaks
from genetic data: a graph approach. \emph{Heredity}. doi:
10.1038/hdy.2010.78.\cr

Jombart, T., Devillard, S., Dufour, A.-B. and Pontier, D. (2008) Revealing
cryptic spatial patterns in genetic variability by a new multivariate
method. \emph{Heredity}, \bold{101}, 92--103.\cr

See adegenet website: \url{http://adegenet.r-forge.r-project.org/}\cr

Please post your questions on 'the adegenet forum':
adegenet-forum@lists.r-forge.r-project.org
}
\seealso{
adegenet is related to several packages, in particular:\cr -
\code{ade4} for multivariate analysis\cr - \code{pegas} for population
genetics tools\cr - \code{ape} for phylogenetics and DNA data handling\cr -
\code{seqinr} for handling nucleic and proteic sequences\cr - \code{shiny}
for R-based web interfaces\cr
}
\author{
Thibaut Jombart <t.jombart@imperial.ac.uk>\cr
Developers: Zhian N. Kamvar <zkamvar@gmail.com>,
Caitlin Collins <caitiecollins17@gmail.com>,
Ismail Ahmed <ismail.ahmed@inserm.fr>,
Federico Calboli, Tobias Erik Reiners, Peter
Solymos, Anne Cori, \cr Contributed datasets from: Katayoun
Moazami-Goudarzi, Denis Laloë, Dominique Pontier, Daniel Maillard, Francois
Balloux.
}
\keyword{manip}
\keyword{multivariate}

\encoding{UTF-8}
\name{gengraph}
\alias{gengraph}
\alias{gengraph.default}
\alias{gengraph.matrix}
\alias{gengraph.dist}
\alias{gengraph.genind}
\alias{gengraph.genpop}
\alias{gengraph.DNAbin}
\title{Genetic transitive graphs}
\description{
  These functions are under development. Please email the author before
  using them for published work.\cr

  The function \code{gengraph} generates graphs based on genetic
  distances, so that pairs of entities (individuals or populations) are
  connected if and only if they are distant by less than a given
  threshold distance. Graph algorithms and classes from the
  \code{\link[igraph]{igraph}} package are used.\cr

  \code{gengraph} is a generic function with methods for the
 following types of objects:\cr
 - \code{matrix} (only numeric data)\cr
 - \code{dist} \cr
 - \code{\linkS4class{genind}} objects (genetic markers, individuals)\cr
 - \code{\linkS4class{genpop}} objects (genetic markers, populations)\cr
 -  \code{\link[ape]{DNAbin}} objects (DNA sequences)
}
\usage{
gengraph(x, \dots)

\method{gengraph}{matrix}(x, cutoff=NULL, ngrp=NULL, computeAll=FALSE,
           plot=TRUE, show.graph=TRUE, col.pal=funky, truenames=TRUE,
           nbreaks=10, \dots)

\method{gengraph}{dist}(x, cutoff=NULL, ngrp=NULL, computeAll=FALSE,
           plot=TRUE, show.graph=TRUE, col.pal=funky, truenames=TRUE,
           nbreaks=10, \dots)

\method{gengraph}{genind}(x, cutoff=NULL, ngrp=NULL, computeAll=FALSE,
           plot=TRUE, show.graph=TRUE, col.pal=funky, truenames=TRUE,
           nbreaks=10, \dots)

\method{gengraph}{genpop}(x, cutoff=NULL, ngrp=NULL, computeAll=FALSE,
           plot=TRUE, show.graph=TRUE, col.pal=funky, method=1,
           truenames=TRUE, nbreaks=10, \dots)

\method{gengraph}{DNAbin}(x, cutoff=NULL, ngrp=NULL, computeAll=FALSE,
           plot=TRUE, show.graph=TRUE, col.pal=funky, truenames=TRUE,
           nbreaks=10, \dots)

}
\arguments{
  \item{x}{a \code{matrix}, \code{dist}, \code{\linkS4class{genind}},
    \code{\linkS4class{genpop}}, or \code{DNAbin} object. For
    \code{matrix} and \code{dist}, the object represents pairwise
    (by default, Hamming) distances between considered individuals.}

  \item{cutoff}{a \code{numeric} value indicating the cutoff point,
    i.e. the distance at which two entities are no longer connected in
    the garph produced by the method.}

  \item{ngrp}{an \code{integer} indicating the number of groups to be
    looked for. A message is issued if this exact number could not be found.}

  \item{computeAll}{a \code{logical} stating whether to investigate
    solutions for every (integer) cutoff point; defaults to FALSE.}

  \item{plot}{a \code{logical} indicating whether plots should be drawn;
    defaults to TRUE; this operation can take time for large, highly-connected graphs.}

  \item{show.graph}{a \code{logical} indicating whether the found graph
    should be drawn, only used in the interactive mode; this operation
    can take time for large, highly-connected graphs; defaults to FALSE.}

  \item{col.pal}{a color palette used to define group colors.}

  \item{method}{an \code{integer} ranging from 1 to 6 indicating the
    type of method to be used to derive a matrix of pairwise distances
    between populations; values from 1 to 5 are passed to the function
    \code{dist.genpop}; other values are not supported.}

  \item{truenames}{a logical indicating whether original labels should
    be used for plotting (TRUE), as opposed to indices of sequences
    (FALSE).}

  \item{nbreaks}{an integer indicating the number of breaks used by the
    heuristic when seeking an exact number of groups.}

  \item{\dots}{further arguments to be used by other functions;
    currently not used.}
}
\value{
  The class \code{gengraph} is a list with the following
  components:\cr
  \item{graph}{a graph of class \code{\link[igraph]{igraph}}.}
  \item{clust}{a list containing group information: \code{$membership}:
    an integer giving group membership; \code{$csize}: the size of each
    cluster; \code{$no}: the number of clusters}
  \item{cutoff}{the value used as a cutoff point}
  \item{col}{the color used to plot each group.}
}
\seealso{
  The \code{\link[igraph]{igraph}} package.
}
\author{
  Original idea by Anne Cori and Christophe Fraser.
  Implementation by Thibaut Jombart \email{t.jombart@imperial.ac.uk}.
 }
\examples{
if(require(ape)){
    data(woodmouse)
    g <- gengraph(woodmouse, cutoff=5)
    g
    plot(g$graph)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import.R
\name{read.genepop}
\alias{read.genepop}
\title{Reading data from Genepop}
\usage{
read.genepop(file, ncode = 2L, quiet = FALSE)
}
\arguments{
\item{file}{a character string giving the path to the file to convert, with
the appropriate extension.}

\item{ncode}{an integer indicating the number of characters used to code an allele.}

\item{quiet}{logical stating whether a conversion message must be printed
(TRUE,default) or not (FALSE).}
}
\value{
an object of the class \code{genind}
}
\description{
The function \code{read.genepop} reads Genepop data files (.gen) and convert
them into a \linkS4class{genind} object.
}
\details{
Note: \code{read.genepop} is meant for DIPLOID DATA ONLY. Haploid data with
the Genepop format can be read into R using \code{read.table} or
\code{read.csv} after removing headers and 'POP' lines, and then converted
using \code{\link{df2genind}}.
}
\examples{

obj <- read.genepop(system.file("files/nancycats.gen",package="adegenet"))
obj

}
\references{
Raymond M. & Rousset F, (1995). GENEPOP (version 1.2):
population genetics software for exact tests and ecumenicism. \emph{J.
Heredity}, \bold{86}:248-249 \cr
}
\seealso{
\code{\link{import2genind}}, \code{\link{df2genind}},
\code{\link{read.fstat}}, \code{\link{read.structure}},
\code{\link{read.genetix}}
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
\keyword{manip}

\encoding{UTF-8}
\name{glPlot}
\alias{glPlot}
\alias{plot.genlight}
\alias{plot,genlight-method}
\alias{plot,genlight,ANY-method}
\title{Plotting genlight objects}
\description{
  \linkS4class{genlight} object can be plotted using the function
  \code{glPlot}, which is also used as the dedicated \code{plot}
  method. These functions relie on \code{\link{image}} to represent SNPs
  data. More specifically, colors are used to represent the number of
  second allele for each locus and individual.
}
\usage{
glPlot(x, col=NULL, legend=TRUE, posi="bottomleft", bg=rgb(1,1,1,.5),\dots)

\S4method{plot}{genlight}(x, y=NULL, col=NULL, legend=TRUE, posi="bottomleft", bg=rgb(1,1,1,.5),\dots)

}
\arguments{
  \item{x}{a \linkS4class{genlight} object.}
  \item{col}{an optional color vector; the first value corresponds to 0
    alleles, the last value corresponds to the ploidy level of the
    data. Therefore, the vector should have a length of (\code{ploidy(x)+1}).}
  \item{legend}{a logical indicating whether a legend should be added to
    the plot.}
  \item{posi}{a character string indicating where the legend should be
    positioned. Can be any concatenation of "bottom"/"top" and "left"/"right".}
  \item{bg}{a color used as a background for the legend; by default,
    transparent white is used; this may not be supported on some
    devices, and therefore background should be specified
    (e.g. \code{bg="white"}).}
  \item{\dots}{further arguments to be passed to \code{\link{image}}.}
  \item{y}{ununsed argument, present for compatibility with the
    \code{plot} generic.}
}
\seealso{
  - \code{\link{genlight}}: class of object for storing massive binary
  SNP data.
  
  - \code{\link{glSim}}: a simple simulator for \linkS4class{genlight}
  objects.

  - \code{\link{glPca}}: PCA for \linkS4class{genlight} objects.

}
\author{ Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
\dontrun{
## simulate data
x <- glSim(100, 1e3, n.snp.struc=100, ploid=2)

## default plot
glPlot(x)
plot(x) # identical plot

## disable legend
plot(x, leg=FALSE)

## use other colors
plot(x, col=heat.colors(3), bg="white")
}
}
\keyword{multivariate}

\encoding{UTF-8}
\docType{methods}
\name{seploc}
\alias{seploc}
\alias{seploc-methods}
\alias{seploc,ANY-method}
\alias{seploc,genind-method}
\alias{seploc,genpop-method}
\alias{seploc,genlight-method}
\title{ Separate data per locus}
\description{
  The function \code{seploc} splits an object (\linkS4class{genind},
  \linkS4class{genpop} or \linkS4class{genlight}) by marker. For
  \linkS4class{genind} and \linkS4class{genpop} objects, the method
  returns a list of objects whose components each correspond to a
  marker. For \linkS4class{genlight} objects, the methods returns blocks
  of SNPs.
}
\usage{
\S4method{seploc}{genind}(x,truenames=TRUE,res.type=c("genind","matrix"))
\S4method{seploc}{genpop}(x,truenames=TRUE,res.type=c("genpop","matrix"))
\S4method{seploc}{genlight}(x, n.block=NULL, block.size=NULL, random=FALSE,
       parallel=FALSE, n.cores=NULL)
}
\arguments{
    \item{x}{a \linkS4class{genind} or a \linkS4class{genpop} object.}
    \item{truenames}{a logical indicating whether true names should be
      used (TRUE, default) instead of generic labels (FALSE).}
    \item{res.type}{a character indicating the type of returned results,
      a genind or genpop object (default) or a matrix of data
      corresponding to the 'tab' slot.}
    \item{n.block}{an integer indicating the number of blocks of SNPs to
      be returned.}
    \item{block.size}{an integer indicating the size (in number of SNPs)
      of the blocks to be returned.}
    \item{random}{should blocks be formed of contiguous SNPs, or should
      they be made or randomly chosen SNPs.}
    \item{parallel}{a logical indicating whether multiple cores -if
      available- should be used for the computations (TRUE, default), or
      not (FALSE); requires the package \code{parallel} to be installed.}
    \item{n.cores}{if \code{parallel} is TRUE, the number of cores to be
      used in the computations; if NULL, then the maximum number of cores
      available on the computer is used.}
}
\value{The function \code{seploc} returns an list of objects of the
  same class as the initial object, or a list of matrices similar to
  x\$tab.\cr
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\seealso{\code{\link{seppop}}, \code{\link{repool}}}
\examples{
\dontrun{
## example on genind objects
data(microbov)

# separate all markers
obj <- seploc(microbov)
names(obj)

obj$INRA5


## example on genlight objects
x <- glSim(100, 1000, 0, ploidy=2) # simulate data
x <- x[,order(glSum(x))] # reorder loci by frequency of 2nd allele
glPlot(x, main="All data") # plot data
foo <- seploc(x, n.block=3) # form 3 blocks
foo
glPlot(foo[[1]], main="1st block") # plot 1st block
glPlot(foo[[2]], main="2nd block") # plot 2nd block
glPlot(foo[[3]], main="3rd block") # plot 3rd block

foo <- seploc(x, block.size=600, random=TRUE) # split data, randomize loci
foo # note the different block sizes
glPlot(foo[[1]])
}
}
\keyword{manip}

\encoding{UTF-8}
\name{genpop class}
\alias{genpop-class}
\alias{dist,genpop,ANY,ANY,ANY,missing-method}
\alias{names,genpop-method}
\alias{show,genpop-method}
\alias{summary,genpop-method}
\alias{print,genpopSummary-method}
\alias{print.genpopSummary}
\alias{is.genpop}
\title{adegenet formal class (S4) for allele counts in populations}
\description{An object of class \code{genpop} contain alleles counts
  for several loci.\cr
  It contains several components (see 'slots' section).\cr
  Such object is obtained using \code{genind2genpop} which converts
  individuals genotypes of known population into a \code{genpop} object.
  Note that the function \code{summary} of a \code{genpop} object
  returns a list of components.
  Note that as in other S4 classes, slots are accessed using @ instead
  of \$.
}
\section{Slots}{
         \describe{
    \item{\code{tab}:}{matrix of alleles counts for each combinaison of population
    (in rows) and alleles (in columns).}
    \item{\code{loc.fac}:}{locus factor for the columns of \code{tab}}
    \item{\code{loc.n.all}:}{integer vector giving the number of alleles per locus}
    \item{\code{all.names}:}{list having one component per locus, each containing a character vector of alleles names}
    \item{\code{call}:}{the matched call}
    \item{\code{ploidy}:}{ an integer indicating the degree of ploidy of
      the genotypes. Beware: 2 is not an integer, but as.integer(2) is.}
    \item{\code{type}:}{ a character string indicating the type of
      marker: 'codom' stands for 'codominant' (e.g. microstallites,
      allozymes); 'PA' stands for 'presence/absence' (e.g. AFLP).}
    \item{\code{other}:}{(optional) a list containing other information}
  }
}
\section{Extends}{
Class \code{"\linkS4class{gen}"}, directly.
Class \code{"\linkS4class{popInfo}"}, directly.
}
\section{Methods}{
  \describe{
    \item{names}{\code{signature(x = "genpop")}: give the names of the
      components of a genpop object}
    \item{print}{\code{signature(x = "genpop")}: prints a genpop object}
    \item{show}{\code{signature(object = "genpop")}: shows a genpop
      object (same as print)}
    \item{summary}{\code{signature(object = "genpop")}: summarizes a
      genpop object, invisibly returning its content or suppress printing of auxiliary information by specifying \code{verbose = FALSE}}
  }
}
\seealso{\code{\link{as.genpop}}, \code{\link{is.genpop}},\code{\link{makefreq}}, \code{\link{genind}}, \code{\link{import2genind}}, \code{\link{read.genetix}}, \code{\link{read.genepop}}, \code{\link{read.fstat}}
}
\author{ Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
obj1 <- import2genind(system.file("files/nancycats.gen",
package="adegenet"))
obj1


obj2 <- genind2genpop(obj1)
obj2

\dontrun{
data(microsatt)
# use as.genpop to convert convenient count tab to genpop
obj3 <- as.genpop(microsatt$tab)
obj3

all(obj3@tab==microsatt$tab)

# perform a correspondance analysis
obj4 <- genind2genpop(obj1,missing="chi2")
ca1 <- dudi.coa(as.data.frame(obj4@tab),scannf=FALSE)
s.label(ca1$li,sub="Correspondance Analysis",csub=2)
add.scatter.eig(ca1$eig,2,xax=1,yax=2,posi="top")

}
}
\keyword{classes}
\keyword{manip}
\keyword{multivariate}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{microbov}
\alias{microbov}
\title{Microsatellites genotypes of 15 cattle breeds}
\format{\code{microbov} is a genind object with 3 supplementary components:
\describe{ \item{coun}{a factor giving the country of each individual (AF:
Afric; FR: France).} \item{breed}{a factor giving the breed of each
individual.} \item{spe}{is a factor giving the species of each individual
(BT: Bos taurus; BI: Bos indicus).} }}
\source{
Data prepared by Katayoun Moazami-Goudarzi and Denis Lalo\"e (INRA,
Jouy-en-Josas, France)
}
\description{
This data set gives the genotypes of 704 cattle individuals for 30
microsatellites recommended by the FAO. The individuals are divided into two
countries (Afric, France), two species (Bos taurus, Bos indicus) and 15
breeds. Individuals were chosen in order to avoid pseudoreplication
according to their exact genealogy.
}
\examples{

\dontrun{
data(microbov)
microbov
summary(microbov)

# make Y, a genpop object
Y <- genind2genpop(microbov)

# make allelic frequency table
temp <- makefreq(Y,missing="mean")
X <- temp$tab
nsamp <- temp$nobs

# perform 1 PCA per marker

kX <- ktab.data.frame(data.frame(X),Y@loc.n.all)

kpca <- list()
for(i in 1:30) {kpca[[i]] <- dudi.pca(kX[[i]],scannf=FALSE,nf=2,center=TRUE,scale=FALSE)}


sel <- sample(1:30,4)
col = rep('red',15)
col[c(2,10)] = 'darkred'
col[c(4,12,14)] = 'deepskyblue4'
col[c(8,15)] = 'darkblue'

# display \%PCA
par(mfrow=c(2,2))
for(i in sel) {
s.multinom(kpca[[i]]$c1,kX[[i]],n.sample=nsamp[,i],coulrow=col,sub=locNames(Y)[i])
add.scatter.eig(kpca[[i]]$eig,3,xax=1,yax=2,posi="top")
}

# perform a Multiple Coinertia Analysis
kXcent <- kX
for(i in 1:30) kXcent[[i]] <- as.data.frame(scalewt(kX[[i]],center=TRUE,scale=FALSE))
mcoa1 <- mcoa(kXcent,scannf=FALSE,nf=3, option="uniform")

# coordinated \%PCA
mcoa.axes <- split(mcoa1$axis, Y@loc.fac)
mcoa.coord <- split(mcoa1$Tli,mcoa1$TL[,1])
var.coord <- lapply(mcoa.coord,function(e) apply(e,2,var))

par(mfrow=c(2,2))
for(i in sel) {
s.multinom(mcoa.axes[[i]][,1:2],kX[[i]],n.sample=nsamp[,i],coulrow=col,sub=locNames(Y)[i])
add.scatter.eig(var.coord[[i]],2,xax=1,yax=2,posi="top")
}

# reference typology
par(mfrow=c(1,1))
s.label(mcoa1$SynVar,lab=popNames(microbov),sub="Reference typology",csub=1.5)
add.scatter.eig(mcoa1$pseudoeig,nf=3,xax=1,yax=2,posi="top")

# typologial values
tv <- mcoa1$cov2
tv <- apply(tv,2,function(c) c/sum(c))*100
rownames(tv) <- locNames(Y)
tv <- tv[order(locNames(Y)),]

par(mfrow=c(3,1),mar=c(5,3,3,4),las=3)
for(i in 1:3){
barplot(round(tv[,i],3),ylim=c(0,12),yaxt="n",main=paste("Typological value -
structure",i))
axis(side=2,at=seq(0,12,by=2),labels=paste(seq(0,12,by=2),"\%"),cex=3)
abline(h=seq(0,12,by=2),col="grey",lty=2)
}
}

}
\references{
Lalo\"e D., Jombart T., Dufour A.-B. and Moazami-Goudarzi K.
(2007) Consensus genetic structuring and typological value of markers using
Multiple Co-Inertia Analysis. \emph{Genetics Selection Evolution}.
\bold{39}: 545--567.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/constructors.R
\docType{methods}
\name{initialize,genind-method}
\alias{initialize,genind-method}
\alias{initialize,genind-methods}
\alias{genind}
\alias{as.genind}
\title{genind constructor}
\usage{
\S4method{initialize}{genind}(.Object, tab, pop = NULL, prevcall = NULL,
  ploidy = 2L, type = c("codom", "PA"), strata = NULL, hierarchy = NULL,
  ...)

genind(...)

as.genind(...)
}
\arguments{
\item{.Object}{prototyped object (generated automatically when calling 'new')}

\item{tab}{A matrix of integers corresponding to the @tab slot of a genind
object, with individuals in rows and alleles in columns, and containing
either allele counts (if type="codom") or allele presence/absence (if
type="PA")}

\item{pop}{an optional factor with one value per row in \code{tab} indicating
the population of each individual}

\item{prevcall}{an optional call to be stored in the object}

\item{ploidy}{an integer vector indicating the ploidy of the individual; each
individual can have a different value; if only one value is provided, it is
recycled to generate a vector of the right length.}

\item{type}{a character string indicating the type of marker: codominant
("codom") or presence/absence ("PA")}

\item{strata}{a data frame containing population hierarchies or
stratifications in columns. This must be the same length as the number of
individuals in the data set.}

\item{hierarchy}{a hierarchical formula defining the columns of the strata
slot that are hierarchical. Defaults to NULL.}

\item{...}{further arguments passed to other methods (currently not used)}
}
\value{
a \linkS4class{genind} object
}
\description{
The function \code{new} has a method for building \linkS4class{genind} objects.
See the class description of \linkS4class{genind} for more information on this data structure.
The functions \code{genind} and \code{as.genind} are aliases for \code{new("genind", ...)}.
}
\details{
Most users do not need using the constructor, but merely to convert raw allele data using \code{\link{df2genind}} and related functions.
}
\seealso{
the description of the \linkS4class{genind} class; \code{\link{df2genind}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/strataMethods.R
\docType{methods}
\name{strata}
\alias{strata}
\alias{strata,genind-method}
\alias{strata,genlight-method}
\alias{strata<-}
\alias{strata<-,genind-method}
\alias{strata<-,genlight-method}
\alias{nameStrata}
\alias{nameStrata,genind-method}
\alias{nameStrata,genlight-method}
\alias{nameStrata<-}
\alias{nameStrata<-,genind-method}
\alias{nameStrata<-,genlight-method}
\alias{splitStrata}
\alias{splitStrata,genind-method}
\alias{splitStrata,genlight-method}
\alias{splitStrata<-}
\alias{splitStrata<-,genind-method}
\alias{splitStrata<-,genlight-method}
\alias{addStrata}
\alias{addStrata,genind-method}
\alias{addStrata,genlight-method}
\alias{addStrata<-}
\alias{addStrata<-,genind-method}
\alias{addStrata<-,genlight-method}
\title{Access and manipulate the population strata for genind or genlight objects.}
\usage{
strata(x, formula = NULL, combine = TRUE, value)

strata(x) <- value

nameStrata(x, value)

nameStrata(x) <- value

splitStrata(x, value, sep = "_")

splitStrata(x, sep = "_") <- value

addStrata(x, value, name = "NEW")

addStrata(x, name = "NEW") <- value
}
\arguments{
\item{x}{a genind or genlight object}

\item{formula}{a nested formula indicating the order of the population
strata.}

\item{combine}{if \code{TRUE} (default), the levels will be combined according to the
formula argument. If it is \code{FALSE}, the levels will not be combined.}

\item{value}{a data frame OR vector OR formula (see details).}

\item{sep}{a \code{character} indicating the character used to separate
hierarchical levels. This defaults to "_".}

\item{name}{an optional name argument for use with addStrata if supplying
a vector. Defaults to "NEW".}
}
\description{
The following methods allow the user to quickly change the strata of a genind
or genlight object.
}
\details{
\subsection{Function Specifics}{ \itemize{ \item \strong{strata()} -
  Use this function to view or define population stratification of a 
  \linkS4class{genind} or \linkS4class{genlight} object. \item 
  \strong{nameStrata()} - View or rename the different levels of strata. 
  \item \strong{splitStrata()} - Split strata that are combined with a common
  separator. This function should only be used once during a workflow. 
  \itemize{ \item \emph{Rationale:} It is often difficult to import files 
  with several levels of strata as most data formats do not allow unlimited 
  population levels. This is circumvented by collapsing all population strata
  into a single population factor with a common separator for each 
  observation. } \item \strong{addStrata()} - Add levels to your population 
  strata. This is ideal for adding groups defined by 
  \code{\link{find.clusters}}. You can input a data frame or a vector, but if
  you put in a vector, you have the option to name it. }}
  
  \subsection{Argument Specifics}{
  
  These functions allow the user to seamlessly carry all possible population 
  stratification with their \linkS4class{genind} or \linkS4class{genlight} 
  object. Note that there are two ways of performing all methods: \itemize{ 
  \item modifying: \code{strata(myData) <- myStrata} \item preserving: 
  \code{myNewData <- strata(myData, value = myStrata)} } They essentially do 
  the same thing except that the modifying assignment method (the one with 
  the "\code{<-}") will modify the object in place whereas the non-assignment
  method will preserve the original object (unless you overwrite it). Due to 
  convention, everything right of the assignment is termed \code{value}. To 
  avoid confusion, here is a guide to the argument \strong{\code{value}} for 
  each function: \itemize{ \item \strong{strata()} \code{value = }a 
  \code{\link{data.frame}} that defines the strata for each individual in the
  rows. \item \strong{nameStrata()} \code{value = }a \code{\link{vector}} or 
  a \code{\link{formula}} that will define the names. \item 
  \strong{splitStrata()} \code{value = }a \code{\link{formula}} argument with
  the same number of levels as the strata you wish to split. \item 
  \strong{addStrata()} \code{value = }a \code{\link{vector}} or 
  \code{\link{data.frame}} with the same length as the number of individuals 
  in your data. }}
  
  \subsection{Details on Formulas}{
  
  The preferred use of these functions is with a \code{\link{formula}} 
  object. Specifically, a hierarchical formula argument is used to assign the
  levels of the strata. An example of a hierarchical formula would
  be:\tabular{r}{ \code{~Country/City/Neighborhood}} This convention was
  chosen as it becomes easier to type and makes intuitive sense when defining
  a \code{\link{hierarchy}}. Note: it is important to use hiearchical
  formulas when specifying hierarchies as other types of formulas (eg. 
  \code{~Country*City*Neighborhood}) will give incorrect results.}
}
\examples{
# let's look at the microbov data set:
data(microbov)
microbov

# We see that we have three vectors of different names in the 'other' slot. 
# ?microbov
# These are Country, Breed, and Species
names(other(microbov))

# Let's set the strata
strata(microbov) <- data.frame(other(microbov))
microbov

# And change the names so we know what they are
nameStrata(microbov) <- ~Country/Breed/Species

\dontrun{
# let's see what the strata looks like by Species and Breed:
head(strata(microbov, ~Breed/Species))

# If we didn't want the last column combined with the first, we can set
# combine = FALSE
head(strata(microbov, ~Breed/Species, combine = FALSE))

#### USING splitStrata ####

# For the sake of example, we'll imagine that we have imported our data set
# with all of the stratifications combined. 
setPop(microbov) <- ~Country/Breed/Species
strata(microbov) <- NULL

# This is what our data would look like after import.
microbov

# To set our strata here, we need to use the functions strata and splitStrata
strata(microbov) <- data.frame(x = pop(microbov))
microbov # shows us that we have "one" level of stratification
head(strata(microbov)) # all strata are separated by "_"

splitStrata(microbov) <- ~Country/Breed/Species
microbov # Now we have all of our strata named and split
head(strata(microbov)) # all strata are appropriately named and split.
}
}
\seealso{
\code{\link{setPop}} \code{\link{genind}}
  \code{\link{as.genind}}
}
\author{
Zhian N. Kamvar
}

\name{coords.monmonier}
\alias{coords.monmonier}
\title{Returns original points in results paths of an object of class 'monmonier'}
\description{
  The original implementation of \code{monmonier} in package
  \bold{adegenet} returns path coordinates, \code{coords.monmonier}
  additionally displays identities of the original points of the network,
  based on original coordinates.
}
\usage{
coords.monmonier(x)
}
\arguments{
  \item{x}{an object of class \code{\link{monmonier}}.}
}
\value{
  Returns a list with elements according to the \code{x$nrun} result of the \code{\link{monmonier}} object.
  Corresponding path points are in the same order as in the original object.

  run1 (run2, ...): for each run, a list containing a matrix giving the
  original points in the network (\code{first} and \code{second},
  indicating pairs of neighbours). Path coordinates are stored in
  columns \code{x.hw} and \code{y.hw}. \code{first} and \code{second}
  are integers referring to the row numbers in the \code{x$xy} matrix of
  the original \code{\link{monmonier}} object.
}
\author{Peter Solymos, \email{Solymos.Peter@aotk.szie.hu}}
\seealso{
  \code{\link{monmonier}}
}
\examples{

\dontrun{
if(require(spdep)){

load(system.file("files/mondata1.rda",package="adegenet"))
cn1 <- chooseCN(mondata1$xy,type=2,ask=FALSE)
mon1 <- monmonier(mondata1$xy,dist(mondata1$x1),cn1,threshold=2,nrun=3)

mon1$run1
mon1$run2
mon1$run3
path.coords <- coords.monmonier(mon1)
path.coords
}
}
}
\keyword{methods}
\keyword{manip}

\encoding{UTF-8}
\name{glSim}
\alias{glSim}
\title{Simulation of simple genlight objects}
\description{
  The function \code{glSim} simulates simple SNP data with the
  possibility of contrasted structures between two groups
  as well as background ancestral population structure. 
  Returned objects are instances of the class \linkS4class{genlight}.
}
\usage{
glSim(n.ind, n.snp.nonstruc, n.snp.struc = 0, grp.size = c(0.5, 0.5), k = NULL,
                    pop.freq = NULL, ploidy = 1, alpha = 0, parallel = FALSE,
                    LD = TRUE, block.minsize = 10, block.maxsize = 1000, theta = NULL,
                    sort.pop = FALSE, \dots)
}
\arguments{
  \item{n.ind}{an integer indicating the number of individuals to be simulated.}
  \item{n.snp.nonstruc}{an integer indicating the number of
    non-structured SNPs to be simulated; for these SNPs, all individuals
    are drawn from the same binomial distribution.}
  \item{n.snp.struc}{an integer indicating the number of
    structured SNPs to be simulated; for these SNPs, different
    binomial distributions are used for the two simulated groups;
    frequencies of the derived alleles in groups A and B are built to
    differ (see details).}
  \item{grp.size}{a vector of length 2 specifying the proportions of the two 
  phenotypic groups (must sum to 1). By default, both groups have the same
    size.}
  \item{k}{an integer specifying the number of ancestral populations to be 
  generated.}
  \item{pop.freq}{a vector of length \code{k} specifying the proportions of the
  k ancestral populations (must sum to 1). If, as by default, \code{pop.freq} 
  is null, and \code{k} is non-null, \code{pop.freq} will be the result of
  random sampling into k population groups.}
  \item{ploidy}{an integer indicating the ploidy of the simulated
    genotypes.}
  \item{alpha}{asymmetry parameter: a numeric value between 0 and 0.5,
    used to enforce allelic differences between the groups. Differences
    between groups are strongest when alpha = 0.5 and weakest when 
    alpha = 0 (see details).}
  \item{parallel}{a logical indicating whether multiple cores should be used in 
    generating the simulated data (TRUE). This option can reduce the amount of 
    computational time required to simulate the data, but is not supported on 
    Windows.}  
  \item{LD}{a logical indicating whether loci should be displaying
    linkage disequilibrium (TRUE) or be generated independently
    (FALSE, default). When set to TRUE, data are generated by blocks of
    correlated SNPs (see details).}
  \item{block.minsize}{an optional integer indicating the minimum number of 
    SNPs to be handled at a time during the simulation of linked SNPs (when 
    \code{LD=TRUE}. Increasing the minimum block size will increase 
    the RAM requirement but decrease the amount of computational time 
    required to simulate the genotypes.}
  \item{block.maxsize}{an optional integer indicating the maximum number of SNPs
  to be handled at a time during the simulation of linked SNPs. Note: if LD
  blocks of equal size are desired, set block.minsize = block.maxsize.}
  \item{theta}{an optional numeric value between 0 and 0.5 specifying 
  the extent to which linkage should be diluted.
  Linkage is strongest when theta = 0 and weakest when theta = 0.5.} 
  \item{sort.pop}{a logical specifying whether individuals should be ordered by
  ancestral population (\code{sort.pop=TRUE}) or phenotypic population 
  (\code{sort.pop=FALSE}).}
  \item{\dots}{arguments to be passed to the genlight constructor.}
}
\details{
  === Allele frequencies in contrasted groups ===
  
  When \code{n.snp.struc} is greater than 0, some SNPs are simulated in
  order to differ between groups (noted 'A' and 'B'). Different patterns 
  between groups are achieved by using different
  frequencies of the second allele for A and B, denoted \eqn{p_A} and
  \eqn{p_B}. For a given SNP, \eqn{p_A} is drawn from a uniform
  distribution between 0 and (0.5 - alpha). \eqn{p_B} is then computed
  as 1 - \eqn{p_A}. Therefore, differences between groups are mild for
  alpha=0, and total for alpha = 0.5.

  
  === Linked or independent loci ===
  
  Independent loci (\code{LD=FALSE}) are simulated using the standard
  binomial distribution, with randomly generated allele
  frequencies. Linked loci (\code{LD=FALSE}) are trickier towe need to
  simulate discrete variables with pre-defined correlation structure.

  Here, we first generate deviates from multivariate normal
  distributions with randomly generated correlation structures. These
  variables are then discretized using the quantiles of the
  distribution. Further improvement of the procedure will aim at i)
  specifying the strength of the correlations between blocks of alleles
  and ii) enforce contrasted structures between groups.
}
\value{
  A \linkS4class{genlight} object.
}

\seealso{
  - \code{\linkS4class{genlight}}: class of object for storing massive binary
  SNP data.

  - \code{\link{glPlot}}: plotting \linkS4class{genlight} objects.

  - \code{\link{glPca}}: PCA for \linkS4class{genlight} objects.
}
\author{ Caitlin Collins \email{caitlin.collins12@imperial.ac.uk},
Thibaut Jombart \email{t.jombart@imperial.ac.uk}}
\examples{
\dontrun{
## no structure
x <- glSim(100, 1e3, ploid=2)
plot(x)

## 1,000 non structured SNPs, 100 structured SNPs
x <- glSim(100, 1e3, n.snp.struc=100, ploid=2)
plot(x)

## 1,000 non structured SNPs, 100 structured SNPs, ploidy=4
x <- glSim(100, 1e3, n.snp.struc=100, ploid=4)
plot(x)

## same thing, stronger differences between groups
x <- glSim(100, 1e3, n.snp.struc=100, ploid=2, alpha=0.4)
plot(x)

##  same thing, loci with LD structures
x <- glSim(100, 1e3, n.snp.struc=100, ploid=2, alpha=0.4, LD=TRUE, block.minsize=100)
plot(x)
}
}
\keyword{multivariate}

\encoding{UTF-8}
\name{dist.genpop}
\alias{dist.genpop}
\title{ Genetic distances between populations }
\description{
  This function computes measures of genetic distances
  between populations using a \code{genpop} object. \cr
  Currently, five distances are available, some of which are euclidian
  (see details).\cr
  
  A non-euclidian distance can be transformed into an Euclidean one
  using \code{\link[ade4]{cailliez}} in order to perform a
  Principal Coordinate Analysis \code{\link[ade4]{dudi.pco}} (both
  functions in \code{ade4}). \cr
  
  The function \code{dist.genpop} is based on former \code{dist.genet}
  function of \code{ade4} package.
}
\usage{
dist.genpop(x, method = 1, diag = FALSE, upper = FALSE)
}
\arguments{
  \item{x}{ a list of class \code{genpop}  }
  \item{method}{ an integer between 1 and 5. See details }
  \item{diag}{ a logical value indicating whether the diagonal of the distance matrix should be printed by \code{print.dist} }
  \item{upper}{ a logical value indicating whether the upper triangle of the distance matrix should be printed by \code{print.dist} }
}
\details{
Let \bold{A} a table containing allelic frequencies with \emph{t} populations (rows) and \emph{m} alleles (columns).\cr
Let  \eqn{\nu} the number of loci. The locus \emph{j} gets \emph{m(j)} alleles. 
\eqn{m=\sum_{j=1}^{\nu} m(j)}\cr

For the row \emph{i} and the modality \emph{k} of the variable \emph{j}, notice the value \eqn{a_{ij}^k} (\eqn{1 \leq i \leq t}, \eqn{1 \leq j \leq \nu},
\eqn{1 \leq k \leq m(j)}) the value of the initial table.\cr

\eqn{a_{ij}^+=\sum_{k=1}^{m(j)}a_{ij}^k} and \eqn{p_{ij}^k=\frac{a_{ij}^k}{a_{ij}^+}}\cr

Let \bold{P} the table of general term \eqn{p_{ij}^k}\cr
\eqn{p_{ij}^+=\sum_{k=1}^{m(j)}p_{ij}^k=1}, \eqn{p_{i+}^+=\sum_{j=1}^{\nu}p_{ij}^+=\nu}, \eqn{p_{++}^+=\sum_{j=1}^{\nu}p_{i+}^+=t\nu}\cr

The option \code{method} computes the distance matrices between populations using the frequencies \eqn{p_{ij}^k}. \cr

1. Nei's distance (not Euclidean): \cr
\eqn{D_1(a,b)=- \ln(\frac{\sum_{k=1}^{\nu} \sum_{j=1}^{m(k)}
p_{aj}^k p_{bj}^k}{\sqrt{\sum_{k=1}^{\nu} \sum_{j=1}^{m(k)}
{(p_{aj}^k) }^2}\sqrt{\sum_{k=1}^{\nu} \sum_{j=1}^{m(k)}
{(p_{bj}^k)}^2}})}\cr

2. Angular distance or Edwards' distance (Euclidean):\cr
\eqn{D_2(a,b)=\sqrt{1-\frac{1}{\nu} \sum_{k=1}^{\nu}
\sum_{j=1}^{m(k)} \sqrt{p_{aj}^k  p_{bj}^k}}}\cr

3. Coancestrality coefficient or Reynolds' distance (Eucledian):\cr
\eqn{D_3(a,b)=\sqrt{\frac{\sum_{k=1}^{\nu}
\sum_{j=1}^{m(k)}{(p_{aj}^k - p_{bj}^k)}^2}{2 \sum_{k=1}^{\nu} (1-
\sum_{j=1}^{m(k)}p_{aj}^k p_{bj}^k)}}}\cr

4. Classical Euclidean distance or Rogers' distance (Eucledian):\cr
\eqn{D_4(a,b)=\frac{1}{\nu} \sum_{k=1}^{\nu} \sqrt{\frac{1}{2}
\sum_{j=1}^{m(k)}{(p_{aj}^k - p_{bj}^k)}^2}}\cr

5. Absolute genetics distance or Provesti 's distance (not Euclidean):\cr
\eqn{D_5(a,b)=\frac{1}{2{\nu}} \sum_{k=1}^{\nu} \sum_{j=1}^{m(k)}
|p_{aj}^k - p_{bj}^k|}
}
\value{
returns a distance matrix of class \code{dist} between the rows of the data frame
}
\references{ 
To complete informations about distances:\cr

Distance 1:\cr
Nei, M. (1972) Genetic distances between populations. \emph{American Naturalist}, \bold{106}, 283--292. \cr
Nei M. (1978) Estimation of average heterozygosity and genetic distance from a small number of individuals. \emph{Genetics}, \bold{23}, 341--369. \cr
Avise, J. C. (1994) Molecular markers, natural history and evolution. Chapman & Hall, London. 

Distance 2:\cr
Edwards, A.W.F. (1971) Distance between populations on the basis of gene frequencies. \emph{Biometrics}, \bold{27}, 873--881. \cr
Cavalli-Sforza L.L. and Edwards A.W.F. (1967) Phylogenetic analysis: models and estimation procedures. \emph{Evolution}, \bold{32}, 550--570. \cr
Hartl, D.L. and Clark, A.G. (1989) Principles of population genetics. Sinauer Associates, Sunderland, Massachussetts (p. 303).

Distance 3:\cr
Reynolds, J. B., B. S. Weir, and C. C. Cockerham. (1983) Estimation of the coancestry coefficient: basis for a short-term genetic distance. \emph{Genetics}, \bold{105}, 767--779.

Distance 4:\cr
Rogers, J.S. (1972) Measures of genetic similarity and genetic distances. \emph{Studies in Genetics}, Univ. Texas Publ., \bold{7213}, 145--153.  \cr
Avise, J. C. (1994) Molecular markers, natural history and evolution. Chapman & Hall, London. 

Distance 5:\cr
Prevosti A. (1974) La distancia genetica entre poblaciones. \emph{Miscellanea Alcobe}, \bold{68}, 109--118. \cr
Prevosti A., Oca\~na J. and Alonso G. (1975) Distances between populations of Drosophila subobscura, based on chromosome arrangements frequencies. \emph{Theoretical and Applied Genetics}, \bold{45}, 231--241. \cr

For more information on dissimilarity indexes:\cr
Gower J. and Legendre P. (1986) Metric and Euclidean properties of
dissimilarity coefficients. \emph{Journal of Classification}, \bold{3},
5--48 \cr

Legendre P. and Legendre L. (1998) \emph{Numerical Ecology}, Elsevier
Science B.V. 20, pp274--288.\cr 
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk}\cr
Former dist.genet code by Daniel Chessel \email{chessel@biomserv.univ-lyon1.fr}\cr
and documentation by Anne B. Dufour \email{dufour@biomserv.univ-lyon1.fr}
}
\seealso{ \code{\link[ade4]{cailliez}},\code{\link[ade4]{dudi.pco}} 
}
\examples{
\dontrun{
data(microsatt)
obj <- as.genpop(microsatt$tab)

listDist <- lapply(1:5, function(i) cailliez(dist.genpop(obj,met=i)))
for(i in 1:5) {attr(listDist[[i]],"Labels") <- popNames(obj)}
listPco <- lapply(listDist, dudi.pco,scannf=FALSE)

par(mfrow=c(2,3))
for(i in 1:5) {scatter(listPco[[i]],sub=paste("Dist:", i))}

}
}
\keyword{multivariate}

\name{genlight-class}
\docType{class}
\alias{genlight}
\alias{genlight-class}
\alias{[,genlight-method}
\alias{[,genlight,ANY,ANY-method}
\alias{[,genlight,ANY,ANY,ANY-method}
\alias{initialize,genlight-method}
\alias{show,genlight-method}
\alias{tab,genlight-method}
\alias{nLoc,genlight-method}
\alias{nInd,genlight-method}
\alias{dim,genlight-method}
\alias{nPop,genlight-method}
\alias{$,genlight-method}
\alias{$<-,genlight-method}
\alias{names,genlight-method}
\alias{ploidy,genlight-method}
\alias{ploidy<-,genlight-method}
\alias{locNames,genlight-method}
\alias{locNames<-,genlight-method}
\alias{indNames,genlight-method}
\alias{indNames<-,genlight-method}
\alias{popNames,genlight-method}
\alias{popNames<-,genlight-method}
\alias{alleles,genlight-method}
\alias{alleles<-,genlight-method}
\alias{chromosome}
\alias{chromosome<-}
\alias{chromosome,genlight-method}
\alias{chromosome<-,genlight-method}
\alias{chr}
\alias{chr<-}
\alias{chr,genlight-method}
\alias{chr<-,genlight-method}
\alias{position}
\alias{position<-}
\alias{position,genlight-method}
\alias{position<-,genlight-method}
\alias{pop,genlight-method}
\alias{pop<-,genlight-method}
\alias{NA.posi}
\alias{NA.posi,genlight-method}
\alias{other,genlight-method}
\alias{other<-,genlight-method}
\alias{as.matrix.genlight}
\alias{as.data.frame.genlight}
\alias{as,matrix,genlight-method}
\alias{as,data.frame,genlight-method}
\alias{as,list,genlight-method}
\alias{coerce,matrix,genlight-method}
\alias{coerce,data.frame,genlight-method}
\alias{coerce,list,genlight-method}
\alias{as.list.genlight}
\alias{cbind.genlight}
\alias{rbind.genlight}
% \alias{,genlight-method}
% \alias{,genlight-method}
% \alias{,genlight-method}
% \alias{,genlight-method}
%%%%
\title{Formal class "genlight"}
\description{
  The class \code{genlight} is a formal (S4) class for storing a genotypes
  of binary SNPs in a compact way, using a bit-level coding scheme.
  This storage is most efficient with haploid data, where the memory
  taken to represent data can be reduced more than 50 times. However,
  \code{genlight} can be used for any level of ploidy, and still remain an
  efficient storage mode.

  A \code{genlight} object can be constructed from vectors of integers
  giving the number of the second allele for each locus and each
  individual (see 'Objects of the class genlight' below).

  \code{genlight} stores multiple genotypes. Each genotype is stored
  as a \linkS4class{SNPbin} object.
}
\section{Objects from the class genlight}{
  \code{genlight} objects can be created by calls to \code{new("genlight",
    ...)}, where '...' can be the following arguments:
  \describe{
    \item{\code{gen}}{input genotypes, where each genotype is coded as a
      vector of numbers of the second allele. If a list, each slot of the
      list correspond to an individual; if a matrix or a data.frame, rows
      correspond to individuals and columns to SNPs. If individuals or
      loci are named in the input, these names will we stored in the
      produced object. All individuals are expected to have the same
      number of SNPs. Shorter genotypes are completed with NAs, issuing a
      warning.}
    \item{\code{ploidy}}{an optional vector of integers indicating the ploidy of the
      genotypes. Genotypes can therefore have different ploidy. If not
      provided, ploidy will be guessed from the data (as the
      maximum number of second alleles in each individual).}
    \item{\code{ind.names}}{an optional vector of characters giving the labels
      of the genotypes.}
    \item{\code{loc.names}}{an optional vector of characters giving the labels
      of the SNPs.}
    \item{\code{loc.all}}{an optional vector of characters indicating
      the alleles of each SNP; for each SNP, alleles must be coded by two
      letters separated by '/', e.g. 'a/t' is valid, but 'a  t' or 'a |t'
      are not.}
    \item{\code{chromosome}}{an optional factor indicating the
      chromosome to which each SNP belongs.}
    \item{\code{position}}{an optional vector of integers indicating the
      position of the SNPs.}
    \item{\code{other}}{an optional list storing miscellaneous information.}
  }
}
\section{Slots}{
  The following slots are the content of instances of the class
  \code{genlight}; note that in most cases, it is better to retrieve
  information via accessors (see below), rather than by accessing the
  slots manually.
  \describe{
    \item{\code{gen}:}{a list of genotypes stored as  \linkS4class{SNPbin} objects.}
    \item{\code{n.loc}:}{an integer indicating the number of SNPs of the
      genotype.}
    \item{\code{ind.names}:}{a vector of characters indicating the names of
      genotypes.}
    \item{\code{loc.names}:}{a vector of characters indicating the names of
      SNPs.}
    \item{\code{loc.all}:}{a vector of characters indicating the alleles
      of each SNP.}
    \item{\code{chromosome}:}{an optional factor indicating the
  chromosome to which each SNP belongs.}
    \item{\code{position}:}{an optional vector of integers indicating the
      position of the SNPs.}
    \item{\code{ploidy}:}{a vector of integers indicating the ploidy of each individual.}
    \item{\code{pop}:}{a factor indicating the population of each individual.}
    \item{\code{strata}:}{a data frame containing different levels of population definition. (For methods, see \code{\link{addStrata}} and \code{\link{setPop}})}
    \item{\code{hierarchy}:}{a hierarchical \code{\link{formula}} defining the hierarchical levels in the \code{@@strata} slot.}
    \item{\code{other}:}{a list containing other miscellaneous information.}
  }
}
\section{Methods}{
  Here is a list of methods available for \code{genlight} objects. Most of
  these methods are accessors, that is, functions which are used to
  retrieve the content of the object. Specific manpages can exist for
  accessors with more than one argument. These are indicated by a '*'
  symbol next to the method's name. This list also contains methods
  for conversion from \code{genlight} to other classes.
  \describe{
    \item{[}{\code{signature(x = "genlight")}: usual method to subset
      objects in R. Is to be applied as if the object was a matrix where
      genotypes were rows and SNPs were columns. Indexing can be done via
      vectors of signed integers or of logicals. See details for extra
      supported arguments.}
    \item{show}{\code{signature(x = "genlight")}: printing of the
      object.}
    
    \item{$}{\code{signature(x = "genlight")}: similar to the @ operator;
      used to access the content of slots of the object.}
    \item{$<-}{\code{signature(x = "genlight")}: similar to the @ operator;
      used to replace the content of slots of the object.}

    \item{tab}{\code{signature(x = "genlight")}: returns a table of
    allele counts (see \code{\link{tab}}; additional arguments are
    \code{freq}, a logical stating if relative frequencies should be
    returned (use for varying ploidy), and \code{NA.method}, a character
    indicating if missing values should be replaced by the mean
    frequency("mean"), or left as is ("asis").}
    
    \item{nInd}{\code{signature(x = "genlight")}: returns the number of
      individuals in the object.}
    \item{nPop}{\code{signature(x = "genlight")}: returns the number of
      populations in the object.}
    \item{nLoc}{\code{signature(x = "genlight")}: returns the number of
      SNPs in the object.}
    \item{dim}{\code{signature(x = "genlight")}: returns the number of 
      individuals and SNPs in the object, respectively.}
    
    \item{names}{\code{signature(x = "genlight")}: returns the names of
      the slots of the object.}
    
    \item{indNames}{\code{signature(x = "genlight")}: returns the names of
      the individuals, if provided when the object was constructed.}
    \item{indNames<-}{\code{signature(x = "genlight")}: sets the names of
      the individuals using a character vector of length
      \code{nInd(x)}.}
    \item{popNames}{\code{signature(x = "genlight")}: returns the names of
      the populations, if provided when the object was constructed.}
    \item{popNames<-}{\code{signature(x = "genlight")}: sets the names of
      the populations using a character vector of length
      \code{nPop(x)}.}
    
    \item{locNames}{\code{signature(x = "genlight")}: returns the names of
      the loci, if provided when the object was constructed.}
    \item{locNames<-}{\code{signature(x = "genlight")}: sets the names of
      the SNPs using a character vector of length \code{nLoc(x)}.}
    
    \item{ploidy}{\code{signature(x = "genlight")}: returns the ploidy of
      the genotypes.}
    \item{ploidy<-}{\code{signature(x = "genlight")}: sets the ploidy of
      the individuals using a vector of integers of size \code{nInd(x)};
      if a single value is provided, the same ploidy is assumed for all
      individuals.}
    
    \item{NA.posi}{\code{signature(x = "genlight")}: returns the indices
      of missing values (NAs) as a list with one vector of integer for each individual.}

    \item{alleles}{\code{signature(x = "genlight")}: returns the names
      of the alleles of each SNPs, if provided when the object was
      constructed.}
    \item{alleles<-}{\code{signature(x = "genlight")}: sets the names
      of the alleles of each SNPs using a character vector of length
      \code{nLoc(x)}; for each SNP, two alleles must be provided,
      separated by a "/", e.g. 'a/t', 'c/a', etc.}
    
    \item{chromosome}{\code{signature(x = "genlight")}: returns a factor
      indicating the chromosome of each SNPs, or NULL if the information
      is missing.}
    \item{chromosome<-}{\code{signature(x = "genlight")}: sets the
      chromosome to which SNPs belong using a factor of length
      \code{nLoc(x)}.}
    
    \item{chr}{\code{signature(x = "genlight")}: shortcut for
      \code{chromosome}.}
    \item{chr<-}{\code{signature(x = "genlight")}: shortcut for
      \code{chromosome<-}.}
    
    \item{position}{\code{signature(x = "genlight")}: returns an integer
      vector indicating the position of each SNPs, or NULL if the
      information is missing.}
    \item{position<-}{\code{signature(x = "genlight")}: sets the
      positions of the SNPs using an integer vector of length
      \code{nLoc(x)}.}
    
    \item{pop}{\code{signature(x = "genlight")}: returns a factor
      indicating the population of each individual, if provided when the
      object was constructed.}
    \item{pop<-}{\code{signature(x = "genlight")}: sets the population
      of each individual using a factor of length \code{nInd(x)}.}
    \item{other}{\code{signature(x = "genlight")}: returns the content of
      the slot \code{@other}.}
    \item{other<-}{\code{signature(x = "genlight")}: sets the content of
      the slot \code{@other}.}
    
    \item{as.matrix}{\code{signature(x = "genlight")}: converts a
      \code{genlight} object into a matrix of integers, with individuals
      in rows and SNPs in columns. The S4 method 'as' can be used as
      well (e.g. as(x, "matrix")).}
    \item{as.data.frame}{\code{signature(x = "genlight")}: same as \code{as.matrix}.}
    \item{as.list}{\code{signature(x = "genlight")}: converts a
      \code{genlight} object into a list of genotypes coded as vector of
      integers (numbers of second allele). The S4 method 'as' can be
      used as well (e.g. as(x, "list")).}
    
    \item{cbind}{\code{signature(x = "genlight")}: merges several
      \linkS4class{genlight} objects by column, i.e. regroups data of
      identical individuals genotyped for different SNPs.}
    \item{rbind}{\code{signature(x = "genlight")}: merges several
      \linkS4class{genlight} objects by row, i.e. regroups data of
      different individuals genotyped for the same SNPs.}
  }
}
\details{
  === On the subsetting using \code{[} ===
  
  The function \code{[} accepts the following extra arguments:
  \describe{
    \item{treatOther}{a logical stating whether elements of the
      \code{@other} slot should be treated as well (TRUE), or not
      (FALSE). If treated, elements of the list are examined for a
      possible match of length (vectors, lists) or number of rows
      (matrices, data frames) with the number of individuals. Those who
      match are subsetted accordingly. Others are left as is, issuing a
      warning unless the argument \code{quiet} is set to TRUE.}
    \item{quiet}{a logical indicating whether warnings should be issued
      when trying to subset components of the \code{@other} slot which
      do not match the number of individuals (TRUE), or not (FALSE,
      default). }
    \item{\dots}{further arguments passed to the genlight constructor.}
    }
}
\author{Thibaut Jombart (\email{t.jombart@imperial.ac.uk})\cr
Zhian N. Kamvar (\email{kamvarz@science.oregonstate.edu})}
\seealso{
 Related class:\cr
  -  \code{\linkS4class{SNPbin}}, for storing individual genotypes of
  binary SNPs\cr
  
  -  \code{\linkS4class{genind}}, for storing other types of genetic markers. \cr
}
\examples{
\dontrun{
## TOY EXAMPLE ##
## create and convert data
dat <- list(toto=c(1,1,0,0), titi=c(NA,1,1,0), tata=c(NA,0,3, NA))
x <- new("genlight", dat)
x

## examine the content of the object
names(x)
x@gen
x@gen[[1]]@snp # bit-level coding for first individual

## conversions
as.list(x)
as.matrix(x)

## round trips - must return TRUE
identical(x, new("genlight", as.list(x))) # list
identical(x, new("genlight", as.matrix(x))) # matrix
identical(x, new("genlight", as.data.frame(x))) # data.frame

## test subsetting
x[c(1,3)] # keep individuals 1 and 3
as.list(x[c(1,3)])
x[c(1,3), 1:2] # keep individuals 1 and 3, loci 1 and 2
as.list(x[c(1,3), 1:2])
x[c(TRUE,FALSE), c(TRUE,TRUE,FALSE,FALSE)] # same, using logicals
as.list(x[c(TRUE,FALSE), c(TRUE,TRUE,FALSE,FALSE)])


## REAL-SIZE EXAMPLE ##
## 50 genotypes of 1,000,000 SNPs
dat <- lapply(1:50, function(i) sample(c(0,1,NA), 1e6, prob=c(.5, .49, .01), replace=TRUE))
names(dat) <- paste("indiv", 1:length(dat))
print(object.size(dat), unit="aut") # size of the original data

x <- new("genlight", dat) # conversion
x
print(object.size(x), unit="au") # size of the genlight object
object.size(dat)/object.size(x) # conversion efficiency



#### cbind, rbind ####
a <- new("genlight", list(toto=rep(1,10), tata=rep(c(0,1), each=5), titi=c(NA, rep(1,9)) ))

ara <- rbind(a,a)
ara
as.matrix(ara)

aca <- cbind(a,a)
aca
as.matrix(aca)


#### subsetting @other ####
x <- new("genlight", list(a=1,b=0,c=1), other=list(1:3, letters,data.frame(2:4)))
x
other(x)
x[2:3]
other(x[2:3])
other(x[2:3, treatOther=FALSE])


#### seppop ####
pop(x) # no population info
pop(x) <- c("pop1","pop1", "pop2") # set population memberships
pop(x)
seppop(x)
}
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Hs.R
\name{Hs.test}
\alias{Hs.test}
\title{Test differences in expected heterozygosity (Hs)}
\usage{
Hs.test(x, y, n.sim = 999, alter = c("two-sided", "greater", "less"))
}
\arguments{
\item{x}{a \linkS4class{genind} object.}

\item{y}{a \linkS4class{genind} object.}

\item{n.sim}{the number of permutations to be used to generate the reference distribution.}

\item{alter}{a character string indicating the alternative hypothesis}
}
\value{
an object of the class randtest
}
\description{
This procedure permits to test if two groups have
significant differences in expected heterozygosity (Hs).
The test statistic used is simply the difference in Hs
between the two groups 'x' and 'y':
}
\details{
\eqn{Hs(x) - Hs(y)}

Individuals are randomly permuted between groups to obtain
a reference distribution of the test statistics.
}
\examples{
\dontrun{
data(microbov)
Hs(microbov)
test <- Hs.test(microbov[pop="Borgou"],
                microbov[pop="Lagunaire"],
                n.sim=499)
test
plot(test)
}

}
\seealso{
\code{\link{Hs}} to compute Hs for different populations;
\code{\link[ade4]{as.randtest}} for the class of Monte Carlo tests.
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}

\encoding{UTF-8}
\name{dapc graphics}
\alias{scatter.dapc}
\alias{assignplot}
\title{Graphics for Discriminant Analysis of Principal Components (DAPC)}
\description{
  These functions provide graphic outputs for Discriminant Analysis of
  Principal Components (DAPC, Jombart et al. 2010). See \code{?dapc} for
  details about this method. DAPC graphics are detailed in the DAPC tutorial
  accessible using \code{vignette("adegenet-dapc")}.

  These functions all require an object of class \code{dapc}
  (the ".dapc" can be ommitted when calling the functions):\cr
  - \code{scatter.dapc}: produces scatterplots of principal components (or
    'discriminant functions'), with a screeplot of eigenvalues as inset.\cr
  - \code{assignplot}: plot showing the probabilities of assignment of
  individuals to the different clusters.\cr

}
\usage{
\method{scatter}{dapc}(x, xax=1, yax=2, grp=x$grp, col=seasun(length(levels(grp))),
      pch=20, bg="white", solid=.7, scree.da=TRUE,
      scree.pca=FALSE, posi.da="bottomright",
      posi.pca="bottomleft", bg.inset="white", ratio.da=.25,
      ratio.pca=.25, inset.da=0.02, inset.pca=0.02,
      inset.solid=.5, onedim.filled=TRUE, mstree=FALSE, lwd=1,
      lty=1, segcol="black", legend=FALSE, posi.leg="topright",
      cleg=1, txt.leg=levels(grp), cstar = 1, cellipse = 1.5,
      axesell = FALSE, label = levels(grp), clabel = 1, xlim =
      NULL, ylim = NULL, grid = FALSE, addaxes = TRUE, origin =
      c(0,0), include.origin = TRUE, sub = "", csub = 1, possub =
      "bottomleft", cgrid = 1, pixmap = NULL, contour = NULL, area
      = NULL, label.inds = NULL, \ldots)

assignplot(x, only.grp=NULL, subset=NULL, new.pred=NULL, cex.lab=.75,pch=3)
}
\arguments{
  \item{x}{a \code{dapc} object.}
  \item{xax,yax}{\code{integers} specifying which principal components of DAPC
    should be shown in x and y axes. }
  \item{grp}{a factor defining group membership for the individuals. The scatterplot
    is optimal only for the default group, i.e. the one used in the
    DAPC analysis.}
  \item{col}{a suitable color to be used for groups. The specified vector
    should match the number of groups, not the number of individuals.}
  \item{pch}{a \code{numeric} indicating the type of point to be used to indicate
    the prior group of individuals (see \code{\link{points}} documentation for
    more details); one value is expected for each group; recycled if necessary.}
  \item{bg}{the color used for the background of the scatterplot.}
  \item{solid}{a value between 0 and 1 indicating the alpha level for the colors of
    the plot; 0=full transparency, 1=solid colours.}
  \item{scree.da}{a logical indicating whether a screeplot of Discriminant Analysis
    eigenvalues should be displayed in inset (TRUE) or not (FALSE).}
  \item{scree.pca}{a logical indicating whether a screeplot of Principal Component Analysis
    eigenvalues should be displayed in inset (TRUE) or not (FALSE); retained axes are
    displayed in black.}
  \item{posi.da}{the position of the inset of DA eigenvalues; can match any
    combination of "top/bottom" and "left/right".}
  \item{posi.pca}{the position of the inset of PCA eigenvalues; can match any
    combination of "top/bottom" and "left/right".}
  \item{bg.inset}{the color to be used as background for the inset plots.}
  \item{ratio.da}{the size of the inset of DA eigenvalues as a proportion of the
    current plotting region.}
  \item{ratio.pca}{the size of the inset of PCA eigenvalues as a proportion of the
    current plotting region.}
   \item{inset.da}{a vector with two numeric values (recycled if needed) indicating
     the inset to be used for the screeplot of DA eigenvalues as a proportion of the
     current plotting region; see \code{?add.scatter} for more details.}
   \item{inset.pca}{a vector with two numeric values (recycled if needed) indicating
     the inset to be used for the screeplot of PCA eigenvalues as a proportion of the
     current plotting region; see \code{?add.scatter} for more details.}
   \item{inset.solid}{a value between 0 and 1 indicating the alpha level for the colors of
    the inset plots; 0=full transparency, 1=solid colours.}
  \item{onedim.filled}{a logical indicating whether curves should be filled when
    plotting a single discriminant function (TRUE), or not (FALSE).}
 \item{mstree}{a logical indicating whether a minimum spanning tree linking the
   groups and based on the squared distances between the groups inside the entire
   space should added to the plot (TRUE), or not (FALSE).}
  \item{lwd,lty,segcol}{the line width, line type, and segment colour to be used for
    the minimum spanning tree.}
  \item{legend}{a logical indicating whether a legend for group colours should added
    to the plot (TRUE), or not (FALSE).}
  \item{posi.leg}{the position of the legend for group colours; can match any
    combination of "top/bottom" and "left/right", or a set of x/y coordinates stored
    as a list (\code{locator} can be used).}
  \item{cleg}{a size factor used for the legend.}
  \item{cstar,cellipse,axesell,label,clabel,xlim,ylim,grid,addaxes,origin,include.origin,sub,csub,possub,cgrid,pixmap,contour,area}{arguments
    passed to \code{\link[ade4]{s.class}}; see \code{?s.class} for more informations}
  \item{only.grp}{a \code{character} vector indicating which groups should be
    displayed. Values should match values of \code{x$grp}. If \code{NULL}, all
    results are displayed}
  \item{subset}{\code{integer} or \code{logical} vector indicating which
    individuals should be displayed. If \code{NULL}, all
    results are displayed}
  \item{new.pred}{an optional list, as returned by the \code{predict} method for
    \code{dapc} objects; if provided, the individuals with unknown groups are added
    at the bottom of the plot. To visualize these individuals only, specify
    \code{only.grp="unknown"}.}
  \item{cex.lab}{a \code{numeric} indicating the size of labels.}
  \item{txt.leg}{a character vector indicating the text to be used in
    the legend; if not provided, group names stored in \code{x$grp} are
    used.}
  \item{label.inds}{Named list of arguments passed to the 
    \code{\link[vegan]{orditorp}} function. This will label individual points 
    witout overlapping. Arguments \code{x} and \code{display} are hardcoded and 
    should not be specified by user.}
  \item{\ldots}{further arguments to be passed to other functions. For
    \code{scatter}, arguments passed to \code{points}; for \code{compoplot},
    arguments passed to \code{barplot}.}
}
\details{
  See the documentation of \code{\link{dapc}} for more information about the method.
}
\value{
  All functions return the matched call.\cr
}
\references{
  Jombart T, Devillard S and Balloux F  (2010) Discriminant analysis of
  principal components: a new method for the analysis of genetically
  structured populations. BMC Genetics11:94. doi:10.1186/1471-2156-11-94
}
\seealso{
  - \code{\link{dapc}}: implements the DAPC.

  - \code{\link{find.clusters}}: to identify clusters without prior.

  - \code{\link{dapcIllus}}: a set of simulated data illustrating the DAPC

  - \code{\link{eHGDP}}, \code{\link{H3N2}}: empirical datasets illustrating
  DAPC
}
\author{ Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
\dontrun{
data(H3N2)
dapc1 <- dapc(H3N2, pop=H3N2$other$epid, n.pca=30,n.da=6)

## defautl plot ##
scatter(dapc1)

## label individuals at the periphery
# air = 2 is a measure of how much space each label needs
# pch = NA suppresses plotting of points
scatter(dapc1, label.inds = list(air = 2, pch = NA))

## showing different scatter options ##
## remove internal segments and ellipses, different pch, add MStree
scatter(dapc1, pch=18:23, cstar=0, mstree=TRUE, lwd=2, lty=2, posi.da="topleft")

## only ellipse, custom labels, use insets
scatter(dapc1, cell=2, pch="", cstar=0, posi.pca="topleft", posi.da="topleft", scree.pca=TRUE,
inset.pca=c(.01,.3), label=paste("year\n",2001:2006), axesel=FALSE, col=terrain.colors(10))

## without ellipses, use legend for groups
scatter(dapc1, cell=0, cstar=0, scree.da=FALSE, clab=0, cex=3,
solid=.4, bg="white", leg=TRUE, posi.leg="topleft")

## only one axis
scatter(dapc1,1,1,scree.da=FALSE, legend=TRUE, solid=.4,bg="white")



## example using genlight objects ##
## simulate data
x <- glSim(50,4e3-50, 50, ploidy=2)
x
plot(x)

## perform DAPC
dapc2 <- dapc(x, n.pca=10, n.da=1)
dapc2

## plot results
scatter(dapc2, scree.da=FALSE, leg=TRUE, txt.leg=paste("group",
c('A','B')), col=c("red","blue"))

## SNP contributions
loadingplot(dapc2$var.contr)
loadingplot(tail(dapc2$var.contr, 100), main="Loading plot - last 100 SNPs")



## assignplot / compoplot ##
assignplot(dapc1, only.grp=2006)

data(microbov)
dapc3 <- dapc(microbov, n.pca=20, n.da=15)
compoplot(dapc3, lab="")
}
}
\keyword{multivariate}

\encoding{UTF-8}
\name{spca}
\alias{spca}
\alias{spca.default}
\alias{spca.matrix}
\alias{spca.data.frame}
\alias{spca.genind}
\alias{spca.genpop}
\alias{print.spca}
\alias{summary.spca}
\alias{plot.spca}
\alias{screeplot.spca}
\alias{colorplot.spca}
\title{Spatial principal component analysis}
\description{

  These functions implement the spatial principal component analysis
  (sPCA). The function \code{spca} is a generic with methods for:
  \itemize{
    \item \code{matrix}: only numeric values are accepted
    \item \code{data.frame}: same as for matrices
    \item \code{genind}: any \linkS4class{genind} object is accepted
    \item \code{genpop}: any \linkS4class{genpop} object is accepted
  }
  
  The core computation use \code{multispati} from the \code{ade4} package.\cr
  
  Besides the set of \code{spca} functions, other functions include:
  \itemize{
    
    \item \code{print.spca}: prints the spca content
    
    \item \code{summary.spca}: gives variance and autocorrelation
    statistics
    
    \item \code{plot.spca}: usefull graphics (connection network, 3 different
    representations of map of scores, eigenvalues barplot and
    decomposition)
  
    \item \code{screeplot.spca}: decomposes spca eigenvalues into variance and
    autocorrelation
    
    \item \code{colorplot.spca}: represents principal components of sPCA in
    space using the RGB system.
    
  }

  A tutorial on sPCA can be opened using:\cr
  \code{adegenetTutorial(which="spca")}.
}
\usage{
spca(...)

\method{spca}{default}(x, ...)

\method{spca}{matrix}(x, xy = NULL, cn = NULL, matWeight = NULL,
            center = TRUE, scale = FALSE, scannf = TRUE,
            nfposi = 1, nfnega = 1,
            type = NULL, ask = TRUE,
            plot.nb = TRUE, edit.nb = FALSE,
            truenames = TRUE,
            d1 = NULL, d2 = NULL, k = NULL,
            a = NULL, dmin = NULL, ...)

\method{spca}{data.frame}(x, xy = NULL, cn = NULL, matWeight = NULL,
            center = TRUE, scale = FALSE, scannf = TRUE,
            nfposi = 1, nfnega = 1,
            type = NULL, ask = TRUE,
            plot.nb = TRUE, edit.nb = FALSE,
            truenames = TRUE,
            d1 = NULL, d2 = NULL, k = NULL,
            a = NULL, dmin = NULL, ...)

\method{spca}{genind}(obj, xy = NULL, cn = NULL, matWeight = NULL,
            scale = FALSE, scannf = TRUE,
            nfposi = 1, nfnega = 1,
            type = NULL, ask = TRUE,
            plot.nb = TRUE, edit.nb = FALSE,
            truenames = TRUE,
            d1 = NULL, d2 = NULL, k = NULL,
            a = NULL, dmin = NULL, ...)

\method{spca}{genpop}(obj, xy = NULL, cn = NULL, matWeight = NULL,
            scale = FALSE, scannf = TRUE,
            nfposi = 1, nfnega = 1,
            type = NULL, ask = TRUE,
            plot.nb = TRUE, edit.nb = FALSE,
            truenames = TRUE,
            d1 = NULL, d2 = NULL, k = NULL,
            a = NULL, dmin = NULL, ...)


\method{print}{spca}(x, \dots)

\method{summary}{spca}(object, \dots, printres=TRUE)

\method{plot}{spca}(x, axis = 1, useLag=FALSE, \dots)

\method{screeplot}{spca}(x, \dots, main=NULL)

\method{colorplot}{spca}(x, axes=1:ncol(x$li), useLag=FALSE, \dots)
}
\arguments{

  \item{x}{a \code{matrix} or a \code{data.frame} of numeric values,
    with individuals in rows and variables in columns; categorical
    variables with a binary coding are acceptable too; for \code{print}
    and plotting functions, a spca object.}

  \item{obj}{a \code{genind} or \code{genpop} object.}

  \item{xy}{a matrix or data.frame with two columns for x and y
    coordinates. Seeked from obj\$other\$xy if it exists when xy is not
    provided. Can be NULL if a \code{nb} object is provided in
    \code{cn}.\cr Longitude/latitude coordinates should be converted
    first by a given projection (see 'See Also' section).}

  \item{cn}{a connection network of the class 'nb' (package spdep). Can
    be NULL if xy is provided. Can be easily obtained using the function
    chooseCN (see details).}

  \item{matWeight}{a square matrix of spatial weights, indicating the
    spatial proximities between entities. If provided, this argument
    prevails over \code{cn} (see details).}

  \item{center}{a logical indicating whether data should be centred to
    a mean of zero; used implicitely for \linkS4class{genind} or
    \linkS4class{genpop} objects.}

  \item{scale}{a logical indicating whether data should be scaled to
    unit variance (TRUE) or not (FALSE, default).}

  \item{scannf}{a logical stating whether eigenvalues should be chosen
    interactively (TRUE, default) or not (FALSE).}

  \item{nfposi}{an integer giving the number of positive eigenvalues
    retained ('global structures').}

  \item{nfnega}{an integer giving the number of negative eigenvalues
    retained ('local structures').}

  \item{type}{an integer giving the type of graph (see details in
    \code{chooseCN} help page). If provided, \code{ask} is set to
    FALSE.}

  \item{ask}{a logical stating whether graph should be chosen
    interactively (TRUE,default) or not (FALSE).}

  \item{plot.nb}{a logical stating whether the resulting graph should be
    plotted (TRUE, default) or not (FALSE).}

  \item{edit.nb}{a logical stating whether the resulting graph should be
    edited manually for corrections (TRUE) or not (FALSE, default).}

  \item{truenames}{a logical stating whether true names should be used
    for 'obj' (TRUE, default) instead of generic labels (FALSE)}

  \item{d1}{the minimum distance between any two neighbours. Used if
    \code{type=5.}}

  \item{d2}{the maximum distance between any two neighbours. Used if
    \code{type=5}.}

  \item{k}{the number of neighbours per point. Used if \code{type=6}.}

  \item{a}{the exponent of the inverse distance matrix. Used if
    \code{type=7}.}

  \item{dmin}{the minimum distance between any two distinct points. Used
    to avoid infinite spatial proximities (defined as the inversed
    spatial distances). Used if \code{type=7}.}

  \item{object}{a \code{spca} object.}

  \item{printres}{a logical stating whether results should be printed on
    the screen (TRUE, default) or not (FALSE).}

  \item{axis}{an integer between 1 and (nfposi+nfnega) indicating which
    axis should be plotted.}

  \item{main}{a title for the screeplot; if NULL, a default one is
    used.}

  \item{\dots}{further arguments passed to other methods.}

  \item{axes}{the index of the columns of X to be represented. Up to
    three axes can be chosen.}

  \item{useLag}{a logical stating whether the lagged components
    (\code{x\$ls}) should be used instead of the components
    (\code{x\$li}).}

}
\details{
  The spatial principal component analysis (sPCA) is designed to
  investigate spatial patterns in the genetic variability. Given
  multilocus genotypes (individual level) or allelic frequency
  (population level) and spatial coordinates, it finds individuals (or
  population) scores maximizing the product of variance and spatial
  autocorrelation (Moran's I). Large positive and negative eigenvalues
  correspond to global and local structures.\cr

  Spatial weights can be obtained in several ways, depending how the
  arguments \code{xy}, \code{cn}, and \code{matWeight} are set.\cr
  When several acceptable ways are used at the same time, priority is as
  follows:\cr
  \code{matWeight} >  \code{cn} > \code{xy} \cr
}
\value{The class \code{spca} are given to lists with the following
  components:\cr
  \item{eig}{a numeric vector of eigenvalues.}
  \item{nfposi}{an integer giving the number of global structures
    retained.}
  \item{nfnega}{an integer giving the number of local structures retained.}
  \item{c1}{a data.frame of alleles loadings for each axis.}
  \item{li}{a data.frame of row (individuals or populations) coordinates
    onto the sPCA axes.}
  \item{ls}{a data.frame of lag vectors of the row coordinates; useful
    to clarify maps of global scores .}
  \item{as}{a data.frame giving the coordinates of the PCA axes onto the
    sPCA axes.}
  \item{call}{the matched call.}
  \item{xy}{a matrix of spatial coordinates.}
  \item{lw}{a list of spatial weights of class \code{listw}.}
  
  Other functions have different outputs:\cr
  - \code{summary.spca} returns a list with 3 components: \code{Istat}
  giving the null, minimum and maximum Moran's I values; \code{pca}
  gives variance and I statistics for the principal component analysis;
  \code{spca} gives variance and I statistics for the sPCA.\cr

  - \code{plot.spca} returns the matched call.\cr

  - \code{screeplot.spca} returns the matched call.  
}
\references{
Jombart, T., Devillard, S., Dufour, A.-B. and Pontier, D.
Revealing cryptic spatial patterns in genetic variability by a new
multivariate method. \emph{Heredity}, \bold{101}, 92--103.

Wartenberg, D. E. (1985) Multivariate spatial correlation: a method for
exploratory geographical analysis. \emph{Geographical Analysis},
\bold{17}, 263--283.

Moran, P.A.P. (1948) The interpretation of statistical
maps. \emph{Journal of the Royal Statistical Society, B}
\bold{10}, 243--251.

Moran, P.A.P. (1950) Notes on continuous stochastic
phenomena. \emph{Biometrika}, \bold{37}, 17--23.

de Jong, P. and Sprenger, C. and van Veen, F. (1984) On extreme values
of Moran's I and Geary's c. \emph{Geographical Analysis}, \bold{16}, 17--24.

}
\seealso{\code{\link{spcaIllus}} and \code{\link{rupica}} for datasets illustrating the sPCA \cr
  \code{\link{global.rtest}} and \code{\link{local.rtest}} \cr
  \code{\link{chooseCN}}, \code{\link[ade4]{multispati}},
  \code{\link[ade4]{multispati.randtest}}\cr
  \code{convUL}, from the package 'PBSmapping' to convert longitude/latitude to
  UTM coordinates.
}
\author{ Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\examples{
## data(spcaIllus) illustrates the sPCA
## see ?spcaIllus
##
\dontrun{
example(spcaIllus)
example(rupica)
}
}
\keyword{spatial}
\keyword{multivariate}

\encoding{UTF-8}
\docType{methods}
\name{seppop}
\alias{seppop}
\alias{seppop-methods}
\alias{seppop,ANY-method}
\alias{seppop,genind-method}
\alias{seppop,genlight-method}
\title{ Separate genotypes per population}
\description{
  The function \code{seppop} splits a \linkS4class{genind} or a
  \linkS4class{genlight} object by population, returning a list of
  objects whose components each correspond to a population.\cr

  For \linkS4class{genind} objects, the output can either be a list of
  \linkS4class{genind} (default), or a list of matrices corresponding to
  the \code{@tab} slot.
}
\usage{
\S4method{seppop}{genind}(x,pop=NULL,truenames=TRUE,res.type=c("genind","matrix"),
  drop=FALSE, treatOther=TRUE, quiet=TRUE)

\S4method{seppop}{genlight}(x,pop=NULL, treatOther=TRUE, quiet=TRUE, \dots)
}
\arguments{
  \item{x}{a \linkS4class{genind} object}
  \item{pop}{a factor giving the population of each genotype in 'x' OR a
    formula specifying which strata are to be used when converting to a genpop
    object. If none provided, population factors are sought in x@pop, but if
    given, the argument prevails on x@pop.}
  \item{truenames}{a logical indicating whether true names should be
    used (TRUE, default) instead of generic labels (FALSE); used if
    res.type is "matrix".}
  \item{res.type}{a character indicating the type of returned results,
    a list of \linkS4class{genind} object (default) or a matrix of data
    corresponding to the 'tab' slots.}
  \item{drop}{a logical stating whether alleles that are no longer
    present in a subset of data should be discarded (TRUE) or kept anyway
    (FALSE, default).}
  \item{treatOther}{a logical stating whether elements of the
    \code{@other} slot should be treated as well (TRUE), or not
    (FALSE). See details in accessor documentations
    (\code{\link{pop}}).}
  \item{quiet}{a logical indicating whether warnings should be issued
    when trying to subset components of the \code{@other} slot (TRUE),
    or not (FALSE, default). }
  \item{\dots}{further arguments passed to the genlight constructor.}
}
\value{
  According to 'res.type': a list of \linkS4class{genind} object
  (default) or a matrix of data corresponding to the 'tab' slots.
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\seealso{\code{\link{seploc}}, \code{\link{repool}}}
\examples{
\dontrun{
data(microbov)
strata(microbov) <- data.frame(other(microbov))

obj <- seppop(microbov)
names(obj)

obj$Salers

### example using strata
obj2 <- seppop(microbov, ~coun/spe)
names(obj2)

obj2$AF_BI

#### example for genlight objects ####
x <- new("genlight", list(a=rep(1,1e3),b=rep(0,1e3),c=rep(1, 1e3)))
x

pop(x) # no population info
pop(x) <- c("pop1","pop2", "pop1") # set population memberships
pop(x)
seppop(x)
as.matrix(seppop(x)$pop1)[,1:20]
as.matrix(seppop(x)$pop2)[,1:20,drop=FALSE]
}
}
\keyword{manip}

\encoding{UTF-8}
\docType{methods}
\name{truenames}
\alias{truenames}
\alias{truenames-methods}
\alias{truenames,ANY-method}
\alias{truenames,genind-method}
\alias{truenames,genpop-method}
\title{ Restore true labels of an object}
\description{
  The function \code{truenames} returns some elements of an object
  (\linkS4class{genind} or \linkS4class{genpop}) using true names
  (as opposed to generic labels) for individuals, markers, alleles, and
  population.\cr

  Important: as of adegenet_2.0-0, these functions are deprecated as
  true labels are used whenever possible. Please use the function
  \code{\link{tab}} instead.
}
\usage{
\S4method{truenames}{genind}(x)
\S4method{truenames}{genpop}(x)
}
\arguments{
  \item{x}{a \linkS4class{genind} or a \linkS4class{genpop} object}
}
\value{
  If x\$pop is empty (NULL), a matrix similar to the x\$tab slot but
  with true labels.
  
  If x\$pop exists, a list with this matrix (\$tab) and a population
  vector with true names (\$pop).\cr
  
}
\author{Thibaut Jombart \email{t.jombart@imperial.ac.uk} }
\seealso{\code{\link{tab}}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/minorAllele.R
\name{minorAllele}
\alias{minorAllele}
\title{Compute minor allele frequency}
\usage{
minorAllele(x)
}
\arguments{
\item{x}{a \linkS4class{genind} object}
}
\description{
This function computes the minor allele frequency for each locus in a \linkS4class{genind} object. To test if loci are polymorphic, see the function  \code{\link{isPoly}}.
}
\examples{
\dontrun{

## LOAD DATA
data(nancycats)

## COMPUTE ALLELE FREQUENCIES
x <- nancycats
apply(tab(x, freq=TRUE),2,mean, na.rm=TRUE)

## GET MINOR ALLELE FREQUENCY
m.freq <- minorAllele(x)
m.freq
}

}
\seealso{
\code{\link{isPoly}}
}
\author{
Thibaut Jombart \email{t.jombart@imperial.ac.uk}
}
